/*
// Bridge
*/

var bridge = {
	tables		: [ ],
	url 		: "DAX/",
	sessionId	: null,
	language	: null,
	admin		: null,
	queryId		: null
}

function dax_bridge_login(username, password) {
	makeCall (bridge.url + "login?username=" + username + "&password=" + password, dax_bridge_login_handler);
}

function dax_bridge_login_handler(http_result) {

	if (http_result.readyState != 4)
		return;
		
	var errorResult = dax_bridge_errorCheck(http_result);	
		
	if (errorResult.hasError) {
		dax_onafterlogin_failure();
		return false;		
	}	
		
	var result = http_result.responseXML;
	
	result = result.getElementsByTagName("loginresult").item(0);
	
	if (!result)
		return;
		
	var success = result.getAttribute("success") == "true";
	
	if (success) {
	
		bridge.language = result.getAttribute('language');
	
		// save the session id
		var sessionId = result.getElementsByTagName("sessionId").item(0);
		
		if (sessionId) {
		
			bridge.sessionId = sessionId.firstChild.nodeValue;
			bridge.admin = sessionId.getAttribute("AdminFlag") != "no";
		
		}
	
		dax_onafterlogin_success();
		
	} else {
		
		dax_onafterlogin_failure();
		
	}
}

function dax_bridge_logout() {
	makeCall (bridge.url + "logout?sessionid=" + bridge.sessionId, dax_bridge_logout_handler);
}

function dax_bridge_logout_handler(http_result) {
	
	if (http_result.readyState != 4)
		return;
		
	var errorResult = dax_bridge_errorCheck(http_result);	
		
	if (errorResult.hasError) {
		dax_onaftersignout_failure();
		return false;		
	}	
		
	var result = http_result.responseXML;
		
	logoutSuccessNode = result.getElementsByTagName('logoutSuccess');
	if (getTextValue(logoutSuccessNode.item(0)).toLowerCase() == 'true')
		dax_onaftersignout_success();
	else
		dax_onaftersignout_failure();
}

function dax_bridge_errorCheck(http_result) {
	
	var errorCode = null;
	var errorNodes = http_result.responseXML.getElementsByTagName('errorCode');
	
	if (errorNodes)
		if (errorNodes.item(0))
			var errorCode = errorNodes.item(0);
			
	if (!errorCode)
		var result = {
			hasError	: false			
		};
	else
		var result = {
			hasError	: true,
			hint		: errorCode.getAttribute('hint'),
			message		: errorCode.firstChild.nodeValue
		};
		
	return result;
	
}

/*
// Core functions
*/

function $(id){
	return document.getElementById(id);
}

function makeCall(url, handler, callRef, syncCall) {

	if (!syncCall)
		syncCall = false;

	if (!handler)
		handler = function (http_request, callRef) { };
		
	var http_request = new XMLHttpRequest();
	
	if (!http_request)
		return;
		
	if (syncCall == false) {
		http_request.onreadystatechange = function() { handler(http_request, callRef); };
		http_request.open("GET", url, true);
		http_request.send(null);
	} else {
		http_request.open("GET", url, false);
		http_request.send(null);
		handler (http_request, callRef);		
	}
}