/*
// Chart
*/

chart = {
	type		: null, // canvas, png, urlimage
	chartNode	: null,
	visible		: false,
	x			: [ ],
	y			: [ ],
	yLabel		: [ ],
	minY		: 1,
	maxY		: 100,
	canvas		: null,	// reference to canvas object
	labelX		: null,	// label X object reference
	urls		: [ ],
	currentUrl	: null,
	imageNode	: null	// for png and urlimage only
}

/*
// View
*/

view = {
	name		: null,
	current		: null,
	width		: 0,
	lock		: false, // disable view change, usually while waiting for response
	reportName	: ''
}

// changes current view, all views start with id 'dax_view_'
function dax_view_setView(viewName) {
		
	// display new view
	var newViewNode = $('dax_view_' + viewName);
	if (newViewNode) {
		newViewNode.style.display = 'block';	

		// hide current view
		if (view.current)
			$(view.current.style.display = 'none');

		view.current = newViewNode;
		view.name = viewName;
		
		// specific view code
		switch(viewName) {
			case 'reportdisplay':
				dax_navbar_showNavbar(view.reportName, STR['back'], dax_report_backToList, null, null);
			break;
			case 'reportlist':
				dax_navbar_showNavbar('Dashboard', STR['logout'], dax_login_signOut, null, null);
				dax_reportlist_getReports();
			break;
			case 'login':
				dax_navbar_hideNavbar();
			break;			
		}
		
		// hide address bar
		setTimeout(scrollTo, 100, 0, 1);
	    
	}
	
}

/*
// Login screen
*/

function dax_login_signIn(){
	
	var loginButton = $('signin_button');
	loginButton.disabled = true;
	
	var username = $('dax_login_username').value;
	var password = $('dax_login_password').value;
	
	dax_bridge_login(username, password);
	
}

function dax_login_signOut(){
	
	var loginButton = $('signin_button');
	loginButton.disabled = false;
	
	var loginError = $('dax_login_error');
	loginError.innerHTML = STR['signingoff'];
	loginError.style.display = 'block';
	
	dax_view_setView('login');
	dax_bridge_logout();
	
}

function dax_onaftersignout_success(){
	
	var loginError = $('dax_login_error');
	loginError.innerHTML = STR['signoffsuccess'];
	loginError.style.display = 'block';
	
}

function dax_onaftersignout_failure(){
	
	var loginError = $('dax_login_error');
	loginError.innerHTML = STR['signofffail'];
	loginError.style.display = 'block';
	
}


function dax_onafterlogin_success(){

	var loginButton = $('signin_button');
	loginButton.disabled = false;

	var loginError = $('dax_login_error');
	loginError.style.display = 'none';
	
	dax_view_setView('reportlist');
}

function dax_onafterlogin_failure(){
	
	var loginButton = $('signin_button');
	loginButton.disabled = false;
	
	var loginError = $('dax_login_error');
	loginError.innerHTML = STR['invalidlogin'];
	loginError.style.display = 'block';

}

/*
// Nav bar
*/

function dax_navbar_showNavbar(title, leftTitle, leftFunction, rightTitle, rightFunction)
{
	var navBar = $('dax_navbar');
	navBar.style.display = 'block';	
	getElementByClassName(navBar, 'title').innerHTML = '<span>' + title + '</span>';
	$('container').style.top = navBar.offsetHeight + 'px';

	var leftButton = getElementByClassName(navBar, 'left');
	if (leftTitle) {
		leftButton.innerHTML = leftTitle;
		if (leftFunction) {
			leftButton.onclick = leftFunction;			
		}		
		leftButton.style.display = 'block';
	} else {
		leftButton.style.display = 'none';		
	}

	var rightButton = getElementByClassName(navBar, 'right');
	if (rightTitle) {
		rightButton.innerHTML = rightTitle;
		if (rightFunction) {
			rightButton.onclick = rightFunction;			
		}		
		rightButton.style.display = 'block';
	} else {
		rightButton.style.display = 'none';		
	}
}

function dax_navbar_hideNavbar()
{
	$('dax_navbar').style.display = 'none';	
	$('container').style.top = '0px';
}

/*
// Report list
*/

function dax_reportlist_getReports()
{
	makeCall (bridge.url + "reportsList?sessionid=" + bridge.sessionId + "&tableid=0" + "&platform=iphone", dax_reportlist_getreportshandler);	
}

function dax_reportlist_getreportshandler(http_result)
{
	if (http_result.readyState != 4)
		return;
		
	var errorResult = dax_bridge_errorCheck(http_result);	
		
	if (errorResult.hasError) {
		alert(errorResult.message);
		return false;		
	}	

	var reportListNode = getElementByClassName($('dax_view_reportlist'), 'reportlist');
	reportListNode.innerHTML = '';
		
	var result = http_result.responseXML;
	
	reports = result.getElementsByTagName("report");
	
	if (reports.length == 0) {
		getElementByClassName($('dax_view_reportlist'), 'noreportsfound').style.display = 'block';		
		return false;
	} else {
		getElementByClassName($('dax_view_reportlist'), 'noreportsfound').style.display = 'none';
	}
	
	
	for (var reportCount = 0; reportCount < reports.length; reportCount++) {
		var reportNode = reports.item(reportCount);
		var reportName = reportNode.getAttribute('name');
		var reportTableId = reportNode.getAttribute('tableid');
		var reportDesc = reportNode.getAttribute('description');
		var reportItemNode = getTemplate('reportitem').cloneNode(true);
		
		getElementByClassName(reportItemNode, 'title').innerHTML = '<span>' + reportName + '</span>';
		getElementByClassName(reportItemNode, 'description').innerHTML = '<span>' + reportDesc + '</span>';
		getElementByClassName(reportItemNode, 'arrowright').style.top = (66 * reportCount) + 17 + 'px';
		getElementByClassName(reportItemNode, 'arrowrightloading').style.top = (66 * reportCount) + 17 + 'px';

		reportItemNode.reportTableId = reportTableId;
		reportItemNode.reportName = reportName;

		reportListNode.appendChild(reportItemNode);
		reportItemNode.style.display = 'block';
	}
}

function dax_reportlist_onitemclick(itemRef) {
	
	if (view.lock)
		return false;
	else
		view.lock = true;
		
	getElementByClassName(itemRef, 'arrowrightloading').style.display = 'block';
	getElementByClassName(itemRef, 'arrowright').style.display = 'none';
	dax_report_showReport(itemRef.reportName, itemRef.reportTableId);
	view.reportName = itemRef.reportName;
}

/* 
// Report view
*/

function dax_report_showReport(reportName, reportTableId){
	-
	makeCall (bridge.url + "runReport?sessionid=" + bridge.sessionId +
	 	"&tableid=" + reportTableId +
		"&reportname=" + escape(reportName) +
		"&queryid=" +
		"&start=0" +
		"&type=public" +
		"&length=0", dax_report_showReportHandler);
	
}

function dax_report_showReportHandler(http_result)
{
	if (http_result.readyState != 4)
		return;
		
	var errorResult = dax_bridge_errorCheck(http_result);	
		
	if (errorResult.hasError) {
		alert(errorResult.message);
		view.lock = false;
		return false;		
	}	

	// parse XML
	var result = http_result.responseXML;
	var reportNode = result.getElementsByTagName('report')[0];
	
	var rowNode = getTemplate('reportrow', $('templates'));
	var cellNode = getTemplate('reportcell', $('templates'));
	
	var targetNode = getElementByClassName($('dax_view_reportdisplay'),'contents', true);
	
	targetNode.innerHTML = '';
	targetNode.style.height = '0px';
	
	var columns = [ ];
	var detail = null;	// detail object properties
	var isAltRow = false;
	var totalColumnWidth = 0; // no need to exceed 460px
	
	if (reportNode.getAttribute('csstheme').toLowerCase() != 'blue')
		getElementByClassName($('dax_view_reportdisplay'),'contents', true).className = 'contents blacktheme';
	else
		getElementByClassName($('dax_view_reportdisplay'),'contents', true).className = 'contents bluetheme';
	
	// parse chart info
	if (reportNode.getAttribute('usechart') == 'true') {
		chart.visible = true;
		var chartXMLNode = reportNode.getElementsByTagName('dashchart')[0];
		chart.type = chartXMLNode.getAttribute('type');
		if (chart.type == 'canvas') {
			chart.x = (chartXMLNode.getElementsByTagName('Xlabel')[0]).firstChild.nodeValue.split(',');
			chart.y = (chartXMLNode.getElementsByTagName('value')[0]).firstChild.nodeValue.split(',');
			chart.yLabel = (chartXMLNode.getElementsByTagName('Ylabel')[0]).firstChild.nodeValue.split(',');
			chart.minY = +chartXMLNode.getElementsByTagName('value')[0].getAttribute('min');
			chart.maxY = +chartXMLNode.getElementsByTagName('value')[0].getAttribute('max');
		} else if ((chart.type == 'png') || (chart.type == 'urlimage')){
			chart.type == 'urlimage';
			var chartUrls = chartXMLNode.getElementsByTagName('link');
			chart.urls = [ ];
			for (var chartUrlCount = 0; chartUrlCount < chartUrls.length; chartUrlCount++) {
				chart.urls.push(chartUrls[chartUrlCount].getAttribute('url'));
			}
			chart.currentUrl = 0;
		} else {
			// chart not supported, probably SVG
			chart.visible = false;
		}
	} else {
		chart.visible = false;
	}
	
	// parse through report nodes
	for (var nodeCount = 0; nodeCount < reportNode.childNodes.length; nodeCount++) {
		var currentNode = reportNode.childNodes[nodeCount];
		var headerGroupNodeValue = null;
		switch (currentNode.nodeName) {
			case 'reportheader':
			case 'reportfooter':
			case 'pageheader':
			case 'pagefooter':
				/*
				var currentObject = {
					type		: currentNode.nodeName,
					style		: {
						align		: currentNode.getAttribute('align'),
						color		: currentNode.getAttribute('color'),
						font		: currentNode.getAttribute('font'),
						size		: currentNode.getAttribute('size'),
						className	: '',
						cached		: false
					},
					height		: currentNode.getAttribute('height'),
					visible		: currentNode.getAttribute('visible'),
					elements	: new Array(),
					cacheHTML	: null
				}
				for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
					var subNode = currentNode.childNodes[subNodeCount];
					switch (subNode.nodeName) {
						case 'graphic':
							var subObject = {
								style		: {
									align		: subNode.getAttribute('align'),
									inline		: subNode.getAttribute('inline'),
									className	: ''
								},
								type		: 'graphic',
								image		: subNode.getAttribute('image')
							}
							currentObject.elements.push(subObject);
						break;
						case 'chart':
							var subObject = {
								type		: 'chart',
								style		: {
									align		: subNode.getAttribute('align'),
									inline		: subNode.getAttribute('inline'),
									className	: ''
								},
								chartid		: subNode.getAttribute('chartid')
							}
							currentObject.elements.push(subObject);
						break;
						case 'line':
							var subObject = {
								type		: 'line',
								style		: {
									align		: subNode.getAttribute('align'),
									font		: subNode.getAttribute('font'),
									color		: subNode.getAttribute('color'),
									size		: subNode.getAttribute('size'),
									className	: ''
								},
								value		: dax_report_parseResponseText(getTextValue(subNode))
							}
							currentObject.elements.push(subObject);
						break;							
					}
				}
				if (currentNode.nodeName == 'reportheader')
					report.staticObjects.reportHeader = currentObject;
				else if (currentNode.nodeName == 'reportfooter')
					report.staticObjects.reportFooter = currentObject;
				else if (currentNode.nodeName == 'pageheader')
					report.staticObjects.pageHeader = currentObject;
				else if (currentNode.nodeName == 'pagefooter')
					report.staticObjects.pageFooter = currentObject;
				*/
			break;			
			case 'detailheader':
				var subNodes = currentNode.getElementsByTagName('field');
				for (var subNodeCount = 0; subNodeCount < subNodes.length; subNodeCount++) {
					var subNode = subNodes[subNodeCount];

						// get column properties and place them in the array
						var column = {
							width	: subNode.getAttribute('width'),
							altcol	: subNode.getAttribute('altcol') ? subNode.getAttribute('altcol') : null,
							detailAlign	: null,
							typeChecked	: false
						}
						
						if (!column.altcol)
							totalColumnWidth += +column.width;
						
						columns[subNodeCount] = column;
						
	
				}				
			case 'detail':
			case 'groupheader':
			case 'groupfooter':
			case 'detailfooter':
			
				// all detail objects have same properties
				if (currentNode.nodeName == 'detail') {
					
		
					if (!detail) {
						var detail = {
							heightMultiplier	: currentNode.getAttribute('height'),
							show				: (currentNode.getAttribute('visible').toLowerCase() == 'true') ? true : false,
							align				: (currentNode.getAttribute('align') != 'default') ? (currentNode.getAttribute('align')) : 'left'
						};
					}
					var showRow = detail.show;
					var currentHeightMultiplier = detail.heightMultiplier;
					var currentAlign = detail.align;
				} else {
					var showRow = (currentNode.getAttribute('visible').toLowerCase() == 'true') ? true : false;
					var currentHeightMultiplier = currentNode.getAttribute('height');		
					var currentAlign = currentNode.getAttribute('align');								
				}
						
				if (showRow) {
					
					// add detail header row	
					var currentRowNode = rowNode.cloneNode(true);

					currentRowNode.style.display = 'block';
					targetNode.appendChild(currentRowNode);
					
					// update row style
					if (isAltRow) {
						currentRowNode.className = 'reportrowalt';
						isAltRow = false;						
					 } else {
						currentRowNode.className = 'reportrow';
						isAltRow = true;
					}
								
					var subNodes = currentNode.getElementsByTagName('field');
					for (var subNodeCount = 0; subNodeCount < subNodes.length; subNodeCount++) {
						var subNode = subNodes[subNodeCount];
							if (!columns[subNodeCount].altcol) {
								if (currentNode.nodeName == 'groupheader') {
									var currentValue = (!!subNode.getAttribute('value')) ? subNode.getAttribute('value') : dax_report_parseResponseText(getTextValue(subNode));
									if ((!headerGroupNodeValue) && (currentValue)) {
										/*
										// group header, one value per row
										*/
										headerGroupNodeValue = currentValue;
										var currentCellNode = cellNode.cloneNode(true);
										currentCellNode.style.display = 'inline';
										var currentValue = (!!subNode.getAttribute('value')) ? subNode.getAttribute('value') : dax_report_parseResponseText(getTextValue(subNode));
										//alert(currentValue);
										currentCellNode.innerHTML = '<span>' + currentValue + '</span>'; // span prevents bug with special character
										//alert(currentCellNode.innerHTML);
										//return;
										currentCellNode.currentValue = currentValue;
										currentCellNode.type = currentNode.nodeName;
										//currentCellNode.style.width = '460px';
										currentCellNode.style.height = (rowHeight * currentHeightMultiplier) + 'px';
										currentRowNode.appendChild(currentCellNode);
										currentCellNode.className = 'cell ' + currentNode.nodeName;
										
									}
								
								} else {
									// create cell and append to row
									var currentCellNode = cellNode.cloneNode(true);
									currentCellNode.style.display = 'inline';
									var currentValue = (!!subNode.getAttribute('value')) ? subNode.getAttribute('value') : dax_report_parseResponseText(getTextValue(subNode));
									var iPhoneAppType = subNode.getAttribute('iphonetype');
									if ((!columns[subNodeCount].typeChecked) && (currentNode.nodeName == 'detail')) {
										if (iPhoneAppType)
											columns[subNodeCount].type = iPhoneAppType;
										else
											columns[subNodeCount].type = null;
											
										columns[subNodeCount].typeChecked = true;
									}
								
									currentCellNode.innerHTML = '<span>' + currentValue + '</span>';  // span prevents bug with special character
									currentCellNode.currentValue = currentValue;
									currentCellNode.type = currentNode.nodeName;
									if (+columns[subNodeCount].width == 0)
										currentCellNode.style.display = 'none';
									else
									currentCellNode.style.width = columns[subNodeCount].width + 'px';
									currentCellNode.style.height = (rowHeight * currentHeightMultiplier) + 'px';

									// indicators
									var currentIndicatorStyle = subNode.getAttribute('class') ? subNode.getAttribute('class') : ' ';
									currentCellNode.indicatorClass = currentIndicatorStyle;
									
									currentRowNode.appendChild(currentCellNode);
									currentCellNode.className = 'cell ' + currentNode.nodeName + ' ' + currentIndicatorStyle;
									if (currentNode.nodeName == 'detail') {
										if (!columns[subNodeCount].detailAlign)
											columns[subNodeCount].detailAlign = (subNode.getAttribute('align') != 'default') ? subNode.getAttribute('align') : 'none';
										currentCellNode.style.textAlign = (columns[subNodeCount].detailAlign != 'none') ? columns[subNodeCount].detailAlign : detail.align;
									} else {
										if (subNode.getAttribute('align'))
											currentCellNode.style.textAlign = (subNode.getAttribute('align') != 'default') ? subNode.getAttribute('align') : 'left';
										else
											currentCellNode.style.textAlign = (currentNode.getAttribute('align') != 'default') ? currentNode.getAttribute('align') : 'left';
										
									}
									if (currentNode.nodeName == 'detail') {
										if (columns[subNodeCount].type == 'link') {
											currentCellNode.currentType = columns[subNodeCount].type;
											currentCellNode.onclick = dax_report_onClickAction;	
											currentCellNode.className += ' link';
										} else if (columns[subNodeCount].type) {
											currentCellNode.currentType = columns[subNodeCount].type;
											currentCellNode.onclick = dax_report_onClickAction;	
											currentCellNode.className += ' clickable';									
										}
									}
								}
							} else {
								if (currentNode.nodeName != 'groupheader') {
									var mainNode = currentRowNode.childNodes[columns[subNodeCount].altcol - 1];
									mainNode.alternateValue = (!!subNode.getAttribute('value')) ? subNode.getAttribute('value') : dax_report_parseResponseText(getTextValue(subNode));
									mainNode.alternateClass = 'cell ' + currentNode.nodeName + ' ' + subNode.getAttribute('class');
									mainNode.onclick = dax_report_swapValues;
									mainNode.columnNumber = columns[subNodeCount].altcol - 1;
									//currentCellNode.className = 'cell ' + currentNode.nodeName + ' ' + currentCellNode.indicatorClass;
								}
							}
					}
					if ((totalColumnWidth < 460) && (currentNode.nodeName != 'groupheader')){
						// make spacer cell to fill up rest of the row
						var currentCellNode = cellNode.cloneNode(true);
						currentCellNode.style.display = 'inline';
						currentCellNode.style.width = 460 - totalColumnWidth + 'px';
						currentCellNode.style.height = (rowHeight * currentHeightMultiplier) + 'px';
						currentRowNode.appendChild(currentCellNode);
						currentCellNode.className = 'cell ' + currentNode.nodeName;
						
					}				
				}
			break;
		}
	}

	dax_view_setView('reportdisplay');
	var newHeight = targetNode.scrollHeight + ((chart.visible) ? 220 : 0) + 'px';
	targetNode.style.height = newHeight;
	
	// draw chart
	if ((chart.visible) && (chart.type == 'canvas')) {
		
		var chartNode = getTemplate('chart', $('templates')).cloneNode(true);
		targetNode.appendChild(chartNode);
		chart.chartNode = chartNode;
		
		chartNode.style.display = 'block';

		// set labels
		
		/*
		getElementByClassName(chartNode, 'chart_y_label_1', true).innerHTML = Math.round(chart.maxY);		
		getElementByClassName(chartNode, 'chart_y_label_2', true).innerHTML = Math.round(((chart.maxY + chart.minY) / 4) * 3);		
		getElementByClassName(chartNode, 'chart_y_label_3', true).innerHTML = Math.round((chart.maxY + chart.minY) / 2);
		getElementByClassName(chartNode, 'chart_y_label_4', true).innerHTML = Math.round((chart.maxY + chart.minY) / 4);		
		getElementByClassName(chartNode, 'chart_y_label_5', true).innerHTML = Math.round(chart.minY);		
		*/
		
		for (var labelCount = 0; labelCount < 5; labelCount++) {
			getElementByClassName(chartNode, 'chart_y_label_' + (5 - labelCount), true).innerHTML = chart.yLabel[labelCount];
		}
		
		var labelCount = chart.x.length;
		switch (chart.x.length) {
			case 1:
				var labelXclassName = 'chart_x_one_label';
			break;
			case 2:
				var labelXclassName = 'chart_x_two_labels';
			break;
			case 3:
				var labelXclassName = 'chart_x_three_labels';
			break;
			case 4:
				var labelXclassName = 'chart_x_four_labels';
			break;
			default:
				var labelXclassName = 'chart_x_five_labels';
				var labelCount = 5;
			break;
		}

		chart.labelX = getElementByClassName(chartNode, 'chart_x_label', true);
		chart.labelX.className = labelXclassName + ' chart_x_label';

		for (var xLabelCount = 0; xLabelCount < labelCount; xLabelCount++) {
			getElementByClassName(chartNode, 'chart_x_label_' + (+xLabelCount + 1), true).innerHTML = chart.x[xLabelCount];			
		}

				
		var canvas = getElementByClassName(chartNode, 'chart_canvas', true);
		chart.canvas = canvas;
		var ctx = canvas.getContext('2d');

		var verScale = 200 / (chart.maxY - chart.minY);

        ctx.strokeStyle = "rgb(128,128,128)";

        ctx.beginPath();
		ctx.lineWidth = 1;
		ctx.moveTo(0,100);
		ctx.lineTo(400, 100);
		ctx.moveTo(0,50);
		ctx.lineTo(400, 50);
		ctx.moveTo(0,150);
		ctx.lineTo(400, 150);
		ctx.stroke();		

		
        ctx.strokeStyle = "rgb(255,255,255)";
       // ctx.fillStyle = "rgb(128,128,128)";

		
        ctx.beginPath();
		ctx.lineWidth = 1.5;
		ctx.moveTo(0,0);
		ctx.lineTo(0, 200);
		ctx.lineTo(400, 200);
		ctx.stroke();

		// create pattern
		var ptrn = ctx.createPattern(chart.graphPatternImg,'repeat');
		ctx.fillStyle = ptrn;
		//ctx.fillRect(0,0,150,150);
		
        ctx.beginPath();
		ctx.moveTo(0, 200);
		ctx.lineTo(0, 200 - (chart.y[0] - chart.minY) * verScale);
		for (var lineCount = 1; lineCount < chart.y.length; lineCount++) {
			var dotX = (400 / chart.y.length) * (lineCount + 1);
			var dotY = (chart.y[lineCount] - chart.minY) * verScale;
			ctx.lineTo(dotX, 200 - dotY);
		}
		ctx.lineTo(400, 200);
		ctx.fill();
		
		
		ctx.lineWidth = 2.5;
		ctx.lineJoin = 'round';
		ctx.lineCap = 'round';
        ctx.beginPath();
		ctx.moveTo(0, 200 - (chart.y[0] - chart.minY) * verScale);
		for (var lineCount = 1; lineCount < chart.y.length; lineCount++) {
			var dotX = (400 / chart.y.length) * (lineCount + 1);
			var dotY = (chart.y[lineCount] - chart.minY) * verScale;
			ctx.lineTo(dotX, 200 - dotY);
		}
		ctx.stroke();

		var chartWidth = (window.orientation == '0' ? 240 : 400);
		chart.canvas.style.width = chartWidth + 'px';
		chart.labelX.style.width = chartWidth + 'px';
		
	} else if ((chart.visible) && ((chart.type == 'urlimage') || (chart.type == 'png')))  {
	
//		if (!chart.imageNode) {
			chart.imageNode = document.createElement('img');
			chart.imageNode.style.width = (window.orientation == '0' ? 300 : 460) + 'px';
			chart.imageNode.style.height = '200px';
			targetNode.appendChild(chart.imageNode);
			
			chart.indicator = {
				node			: document.createElement('div'),
				nodeContainer	: document.createElement('div'),
				urls	: [ ]
			}
			
			chart.indicator.nodeContainer.className = 'chart_bottom_indicator_container';
			targetNode.appendChild(chart.indicator.nodeContainer);

			chart.indicator.node.className = 'chart_bottom_indicator';
			chart.indicator.nodeContainer.appendChild(chart.indicator.node);
			
			for (var urlNodesCount = 0; urlNodesCount < chart.urls.length; urlNodesCount++) {

				var urlNode = document.createElement('div');
				urlNode.className = 'chart_bottom_indicator_bullet';
				chart.indicator.urls.push(urlNode);
				chart.indicator.node.appendChild(urlNode);
			}
			
			chart.indicator.node.style.width = (18 * chart.urls.length) + 'px';
			chart.indicator.urls[0].className = 'chart_bottom_indicator_bullet_active';
			
			chart.imageNode.onclick = function() {

				chart.indicator.urls[chart.currentUrl].className = 'chart_bottom_indicator_bullet';
			
				if (++chart.currentUrl == chart.urls.length)
					chart.currentUrl = 0;
					
				chart.imageNode.src = chart.urls[chart.currentUrl]
				chart.indicator.urls[chart.currentUrl].className = 'chart_bottom_indicator_bullet_active';
			
			}
			
//		}
		
		chart.imageNode.src = chart.urls[chart.currentUrl];
	
	} else {
		
		if (chart.chartNode)
			chart.chartNode.style.display = 'none';
			
		if (chart.imageNode)
			chart.imageNode.style.display = 'none';
	
	}

	
	getElementByClassName($('dax_view_reportdisplay'),'left').style.height = newHeight;
	getElementByClassName($('dax_view_reportdisplay'),'right').style.height = newHeight;
	view.lock = false;
}

function dax_report_swapValues(){
	var rows = getElementByClassName($('dax_view_reportdisplay'), 'contents', true).childNodes;
	for (var rowCount = 0; rowCount < rows.length; rowCount++) {
		if (rows[rowCount].nodeName.toLowerCase() == 'div') {
			var cellReference = rows[rowCount].childNodes[this.columnNumber];
			if (cellReference) {
				if ((cellReference.type == 'detail') || (cellReference.type == 'detailfooter') || (cellReference.type == 'detailheader') || (cellReference.type == 'groupfooter')) {
					// swap values
					var currentValue = cellReference.currentValue;
					cellReference.innerHTML = '<span>' + cellReference.alternateValue + '</span>';
					cellReference.currentValue = cellReference.alternateValue;
					cellReference.alternateValue = currentValue;
					// swap classes
					var currentClass = cellReference.className;
					cellReference.className = cellReference.alternateClass;
					cellReference.currentClass = cellReference.alternateClass;
					cellReference.alternateClass = currentClass;
				}
			}
		}	
	}
}

function dax_report_onClickAction(){

	var cellValue = this.currentValue;
	var cellType = this.currentType;
	
	if (!cellValue)
		return false;
	
	switch(cellType) {
		case 'link':
			var goToUrl = cellValue;
			window.open(goToUrl);
		break;
		case 'email':
			var goToUrl = 'mailto:' + cellValue;
			document.location.href = goToUrl;
		break;
		case 'phone':
			var goToUrl = 'tel:' + cellValue;
			document.location.href = goToUrl;
		break;
		case 'maps':
			var goToUrl = 'http://maps.google.com/maps?q=' + cellValue;
			document.location.href = goToUrl;
			break;
		default:
			return false;
		break;
	}

}

function dax_report_parseResponseText(textIn) {
	if (!textIn)
		return '';
	var textOut = ReplaceString(textIn, '##BR', '<br />')
	return textOut;
}

// Move from report view to report list
function dax_report_backToList() {
	dax_view_setView('reportlist');
}

/*
// iPhone specific functions
*/


window.onorientationchange = function() {

    var bodyClassName = 'dax_body_' + (window.orientation == '0' ? "profile" : "landscape");
	document.body.className = bodyClassName;
	if ((chart.visible) && (chart.canvas)){
		var chartWidth = (window.orientation == '0' ? 240 : 400);
		chart.canvas.style.width = chartWidth + 'px';
		chart.labelX.style.width = chartWidth + 'px';
	} else if (chart.visible) {
		var chartWidth = (window.orientation == '0' ? 300 : 460);
		chart.imageNode.style.width = chartWidth + 'px';
	}
}

/*
// Events
*/

// on load events
addEventListener("load", function(event)
{	
	dax_view_setView('login');
    setTimeout(window.onorientationchange, 0);
	
	// set default username and password
	$('dax_login_username').value = defaultUsername;
	$('dax_login_password').value = defaultPassword;
	
	
	// create new image object to use as pattern
	chart.graphPatternImg = new Image();
	chart.graphPatternImg.src = 'dax/iphone/dashboard/images/graph_gradient.png';
	
}, false);

/*
// Core functions
*/ 


function getElementByClassName(parent, className, match)
{
	if (!parent || className == "")
		return null;
	
	if (match != null && match != undefined && match) {
		var cname = parent.className != undefined ? parent.className : "";
		if (cname.indexOf(className) >= 0) {
			return parent;
		}
	} else {
		if (parent.className == className)
			return parent;
	}
	
	// get the first level of children
	var children = parent.childNodes;
	var node = null;
	var i = 0;
	
	while (!node && i < children.length) {
		node = getElementByClassName(children.item(i), className, match);
		i++;
	}
			
	return node;
}

function getTemplate(templateName)
{

	return getElementByClassName($('templates'), templateName);

}

function getTextValue(aNode)
{
//	if (!isDefined(aNode) || !aNode.hasChildNodes())
//		return null;
	
	for (var child = aNode.firstChild; child; child = child.nextSibling) {
		if (child.nodeType == 3) {
			return child.nodeValue;
		}
	}	
	
	return null;
}

function ReplaceString(sString, sReplaceThis, sWithThis) {
	if (sReplaceThis != "" && sReplaceThis != sWithThis) {
		var counter = 0;
		var start = 0;
		var before = "";
		var after = "";
		while (counter<sString.length) {
			start = sString.indexOf(sReplaceThis, counter);
			if (start == -1){
				break;
			} else {
				before = sString.substr(0, start);
				after = sString.substr(start + sReplaceThis.length, sString.length);
				sString = before + sWithThis + after;
				counter = before.length + sWithThis.length;
			}
		}
	}
	return sString;
}