// bridge.js
// Rudolf Psenicnik
//
// Generic connection object for DAX framework
// For DAX app connection, look at connect.js

// static global objects
var itemfrom = 0;
var itemto = 0;
var labeledit = false;
var editedobject = 0;
var userlanguage = "";
var dax_uniqueId = 0;

var taskbar; // reserved for taskbar
var aSidebar; // reserved for sidebar

function daxLoginSuccess() { };
function daxLoginFail() { };

/*
// do a browser check
if ((!browser.ie7) && (!browser.safari) && (!browser.gecko)) {

	alert (STR['unsupportedbrowser']);

}

*/

/**
 * Get the Key Code
 * @ author Julien Feasson
 */
function getkey(e)
{
if (window.event)
   return window.event.keyCode;
else if (e)
   return e.which;
else
   return null;
}


/**
 * Controller
 * @ author Jonathan Le
 */
var Controller = {
	// object focusing
	focusedObject : null,
	setFocused : function(aObject)
	{
		// blur objects when focus needs to be switched
		if (aObject != this.focusedObject) {
			Controller.blur(this.focusedObject);
		}
		
		this.focusedObject = aObject;
	},
	focus : function(aObject)
	{
		this.setFocused(aObject);

	},
	blur : function(aObject, aEvent)
	{
		if (this.focusedObject && this.focusedObject.blur)
			try {
				var keepFocused = this.focusedObject.blur(aObject, aEvent);
			} catch (e) { };

		if (!keepFocused)
			this.focusedObject = null;
	},
		
	// record locking
	lockedRecords : new Array(),
	recordHash : function(tableId, recordId)
	{
//		return tableId+"_"+recordId;
		return "r_" + recordId;
	},
	isLocked : function(tableId, recordId)
	{
		var hash = this.recordHash(tableId, recordId);
		return isDefined(this.lockedRecords[hash]) ? this.lockedRecords[hash] : false;
	},
	lockRecord : function(tableId, recordId)
	{
		this.lockedRecords[this.recordHash(tableId, recordId)] = true;
	},
	unlockRecord : function(tableId, recordId)
	{
		this.lockedRecords[this.recordHash(tableId, recordId)] = false;
	},
	
	// file uploading
	uploading : false, 		// simple flag
	uploads : new Array(), 	// queue of objects wanting to upload
	enqueue : function(aObject)
	{
		// enqueue it
		this.uploads.push(aObject);
		debugAlert('this.uploading = ' + this.uploading);
		//if (!this.uploading){
			debugAlert("!this.uploading");
			this.upload();
			debugAlert('uploaded');
		//}
		//else
			debugAlert('else');
	},
	dequeue : function()
	{
		return this.uploads.shift();
	},
	upload : function()
	{
		// check our resource
		if (this.uploads.length <= 0){
			debugAlert('this.uploads.length <=0');
			return;
		}
//		var debugTest = '.upload() in init.js, this.uploading=' + this.uploading + ' this.isUploadReady()= '+ this.isUploadReady();
//		debugAlert (debugTest);
			
		// if we are not already uploading & resource is ready
		if (!this.uploading && this.isUploadReady()) {
			//debugAlert('!this.uploading && this.isUploadReady())');
			this.uploading = true;
			var object = this.dequeue();
			object.upload(); // object handles its own uploading
			// this.uploading = false;
		} else { // wait
			// sleep
			//debugAlert('ready to upload');
			//debugAlert('length = ' + this.uploads.length);
			var controller = this;
			setTimeout(function() { controller.upload(); }, 15); 
		}
	},
	isUploadReady : function()
	{
		var upload = frames['upload'].document;
		debugAlert('.isUploadReady, upload= ' + upload);
		var state = upload.getElementById("state");
		debugAlert('.isUploadReady, state= ' + state);
		var uploadReply = state ? state.className == "ready" : false;
		if (((!state) & (!this.reloading) & (upload.location.pathname != "/dax/js/templates/upload.html")) | ((upload.getElementById("error")) & (upload.location.pathname != "/DAX/UploadBinary"))) {
			upload.location.href = "/dax/js/templates/upload.html?random="+Math.random();
			this.reloading = true;
		}
		else
			if (state)
				this.reloading = false;
		debugAlert ('.isUploadReady, reply=' + uploadReply);
		return uploadReply;
	}
};
document.onkeydown = function(aEvent)
{
	if (Controller.focusedObject && Controller.focusedObject.onkeydown) {
		Controller.focusedObject.onkeydown(aEvent);
	}
};
document.onkeypress = function(aEvent)
{
	if (Controller.focusedObject && Controller.focusedObject.onkeypress) {
		Controller.focusedObject.onkeypress(aEvent);
	}
};
document.onkeyup = function(aEvent)
{
	if (Controller.focusedObject && Controller.focusedObject.onkeyup) {
		Controller.focusedObject.onkeyup(aEvent);
	}
};

document.onclick = function(aEvent)
{
	if (browser.ie7) var aEvent = xpEvent(aEvent);
	// release the controller's focused object
	Controller.blur(null, aEvent);
}

// because IE makes my life a nightmare
if(!document.importNode){
	document.importNode = function(externalNode, deep){
		var newNode = null;

		if (externalNode.nodeType == 1) {
			newNode = document.createElement(externalNode.nodeName);
			for (var i = 0; i < externalNode.attributes.length; i++) {
				var attr = externalNode.attributes[i];
				if (attr.nodeValue != null && attr.nodeValue != "") {
					newNode.setAttribute(attr.name,
						attr.value);
				}
			}
	
			// explicit
			newNode.className = externalNode.className;
			newNode.style.cssText = externalNode.style.cssText;
		} else if (externalNode.nodeType == 3 ) {
			newNode = document.createTextNode(externalNode.nodeValue);
		}
		
		if (deep && externalNode.hasChildNodes()) {
			for (var childNode = externalNode.firstChild; childNode; childNode = childNode.nextSibling) {
				var cloneNode = document.importNode(childNode, true);
				if (cloneNode)
					newNode.appendChild(cloneNode);
			}
		}

		return newNode;
	}
}



// extend Object
Object.inherit =
	function(parent)
	{
		for (var property in parent) {
			this[property] = parent[property];
		}
	};

/**
 * create getters for classes where 'prv' property is used.
 * prv property must declare member variables starting with an 'm' prefix for
 * this to work correctly.
 * @author Jonathan Le
 **/
Function.prototype.decl = function(aClass) {
	if (this.prv) {
		for (var property in this.prv) {
			var name = property.charAt(0) == 'm' ? property.substring(1) : property;
			this['get'+name] =
				new Function("return this.prv['"+property+"'];");
			this['set'+name] =
				new Function("value", "this.prv['"+property+"'] = value;");
		}
	}
}


if (typeof(dax_bridge) == 'undefined') {
	dax_bridge = new bridge();
}

// opens new window with debug output
// place debugAlert (message) calls in the code to send debug output
function showDebugger ()
{
	if (dax_bridge)
		dax_bridge.showDebugger();
	
}

function debugAlert (message)
{
	if (dax_bridge.debugWindow)
		if (message)
			if (dax_bridge.debugWindow.alernateRow) {
				dax_bridge.debugWindow.debugText.innerHTML = "<div class = 'normal_row'>" + message + "</div>" + dax_bridge.debugWindow.debugText.innerHTML ;
				dax_bridge.debugWindow.alernateRow = false;
			} else {
				dax_bridge.debugWindow.debugText.innerHTML = "<div class = 'alt_row'>" + message + "</div>" + dax_bridge.debugWindow.debugText.innerHTML ;
				dax_bridge.debugWindow.alernateRow = true;
			}
}

// user defined
function onAfterInit () {

	//if (dax_bridge)
	//	dax_bridge.onAfterInit();

}


function onLogoutSuccess() { };

// Login
//
// will cache table list on successful login
// dax_bridge must be initialized (done automatically when loading bridge.js)
function Login(username, password)
{
	if ((dax_bridge) && (username))
		dax_bridge.login (username, password);

}

function bridge() {
	
	// user defined
	//this.onAfterInit = function () {  };

	this.isClient = false;
	this.serverSet = 'UTF-8';
	
	this.encode = function(value) {
		if (this.serverSet == 'UTF-8')
			return encodeURIComponent(value);
		else
			return encodeURIComponent(value);

	
	}
	
	this.updating = false;
	this.showDebugger = function ()
	{
		if (!this.debugWindow) {
			this.debugWindow = WindowManager.createNewWindow();
			if (this.debugWindow) {
				this.debugWindow.bridgeLink = this;
				this.debugWindow.titleBar.innerHTML = 'Console';
				setStyle(this.debugWindow.contentArea, 'overflow', 'scroll');
				debugText = document.createElement('div');
				this.debugWindow.contentArea.appendChild(debugText);
				debugText.className = 'dax_console';
				this.debugWindow.debugText = debugText;
				this.debugWindow.activate();
				this.debugWindow.open();
				this.debugWindow.logo.className = 'window_logo_debug';
				debugAlert ("<b>User Agent:</b>"+navigator.userAgent);
				// on window refresh, clear the contents area
				this.debugWindow.refreshBox.boundTo = this.debugWindow;
				this.debugWindow.refreshBox.onmouseup = function(event)
				{
					var event = xpEvent(event, true);
					this.className = "window_refreshbox";
					this.parent.debugText.innerHTML = '';
					debugAlert ("<b>User Agent:</b>"+navigator.userAgent);
					return false;
				};
				
				// on close, destroy debug reference
				this.debugWindow.closeBox.onmouseup = function(event)
				{
					var event = xpEvent(event, true);
					WindowManager.destroyWindow(this.parent);
					this.parent.bridgeLink.debugWindow = null;
					return false;
				};				
			}
		} else {
		// debugger exists, highlight it
			this.debugWindow.activate();
		}
	}

	// session id is blank if not connected
	this.sessionId = '';
	
	// beginning of the url
	this.url = "/DAX/";	

	// user information
	this.username = '';
	this.admin = false;
	this.language = null;
	this.shuttingDown = false;
	
	this.tables = new Array();
	this.reports = new Array();
	this.styleNode = null; // reference to style node
	
	this.login = function (username, password) {
		makeCall (dax_bridge.url + "login?username=" + dax_bridge.encode(username) + "&password=" + dax_bridge.encode(password), dax_bridge.login_handler , null);
	}
	
	this.logout = function () {
		makeCall (dax_bridge.url + "logout?sessionid=" + dax_bridge.sessionId, dax_bridge.logout_handler , null);
	}
	
	this.logout_handler = function (http_result) {

		if (http_result.readyState != 4)
			return;
			
		var result = http_result.responseXML;
		
		//if (errorCheck(result)) {
			//on_login_error();
			//return;
		//}
		
		result = result.getElementsByTagName("result").item(0);
		
		if (!result)
			return;
			
		var success = result.getElementsByTagName("logoutSuccess").item(0).firstChild.nodeValue == "true";
		
		if (success) {
		
			if (typeof(onLogoutSuccess) != 'undefined')
				if (onLogoutSuccess)
					onLogoutSuccess();
			
		} else {
		
			//onLogoutFail();
		
		}
	}
	
	// localize DAX based on set language
	this.localize = function (aLang) {
	
		// if no language is passed or set, use english as default
		if ((!aLang) && (!this.language))
			var aLang = 'en';
	
		// if no language is passed, but one is set in preferences, use that one
		if ((!aLang) && (this.language))
			var aLang = this.language;

		var locCommand = 'STR = STR_' + aLang.toUpperCase() + ';';
		
		try {
			debugAlert ('<b>Bridge</b> (.localize):Setting language to ' + aLang);
			eval(locCommand);
		} catch (e) {
			// language change failed
		}
			
	}
	
	this.login_handler = function (http_result) {

		if (http_result.readyState != 4)
			return;

		if (checkResponse(http_result)) {
			daxLoginFail();
			return;
		}
			
		var result = http_result.responseXML;
		
		result = result.getElementsByTagName("loginresult").item(0);
		
		if (!result)
			return;
			
		var success = result.getAttribute("success") == "true";
		
		if (success) {
		
			dax_bridge.language = result.getAttribute('language');
		
			// save the session id
			var sessionId = result.getElementsByTagName("sessionId").item(0);
			
			if (sessionId) {
			
				dax_bridge.sessionId = sessionId.firstChild.nodeValue;
				dax_bridge.admin = sessionId.getAttribute("AdminFlag") != "no";
				dax_bridge.updateTables();
				
				// for compatibility reasons
				if ((structure) && (connection)){
					if (!structure.cached) {
						if (!connection.sessionId)
							connection.sessionId = dax_bridge.sessionId;
						//structure.update();
					}
				}

			} else {
				// network error?
			}
			
			daxLoginSuccess();
			
		} else {
			// failed, alert the user
			daxLoginFail();
		}
	}
	
	this.updateTables = function (syncCall) {
		if (dax_bridge.sessionId) {
			this.updating= true;
			
			if (!syncCall)
				syncCall = false;
			else
				syncCall = true;
			
			url = dax_bridge.url + "getTableList?sessionId=" + dax_bridge.sessionId;
			makeCall (url, dax_bridge.updateTables_handler , false, syncCall);
		}
	}

	// gets info for only one table, this is usually done for tables that are initially invisible, also assumes that table doesn't exist in cache
	this.getSingleTable = function (tableId) {
		if (dax_bridge.sessionId) {
			url = dax_bridge.url + "getSingleTable?tableid=" + tableId + "&sessionId=" + dax_bridge.sessionId;
			makeCall (url, dax_bridge.updateTables_handler, true, true);
		}
	}
	
	// update table structure
	this.updateTables_handler = function (http_result, isGetSingleTable) {
		
		if (http_result.readyState != 4)
			return;

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('getTable request error.');
			return false;
		}
			
		var database = http_result.responseXML.getElementsByTagName("tables").item(0);
		var tableList = database ? database.getElementsByTagName("table") : null;
		
		if (!tableList) {
			tableList = [ ];
		}
		
		for (ij = 0; ij < tableList.length; ij++) {
			var table_alias = tableList.item(ij).getAttribute("alias");
			var table_calendarview = tableList.item(ij).getAttribute("calendarview");
			var table_datatreeview = tableList.item(ij).getAttribute("datatreeview");
			var table_datamatrixview = tableList.item(ij).getAttribute("datamatrixview");
			var table_imagematrixview = tableList.item(ij).getAttribute("imagematrixview");
			var table_name = tableList.item(ij).getAttribute("name");
			var table_id = tableList.item(ij).getAttribute("id");
			var table_visible = tableList.item(ij).getAttribute("visible");
			var table_ddw = tableList.item(ij).getAttribute("ddw");
			var table_ddwtype = tableList.item(ij).getAttribute("ddwtype");
			var table_selectionviewid = tableList.item(ij).getAttribute("selectionviewid");
			var table_showallrecordstab = 
				(tableList.item(ij).getAttribute("allrecords") == '') ? 'True' :
					(tableList.item(ij).getAttribute("allrecords"));
			
			// add to list of tables
			var table = {
				tableid : table_id,
				tablename : table_name,
				tablealias : table_alias,
				tablecalendarview : table_calendarview,
				tabledatatreeview : table_datatreeview,
				tabledatamatrixview : table_datamatrixview,
				tableimagematrixview : table_imagematrixview,
				tablevisible : table_visible,
				cached : false,
				ddw : table_ddw,
				ddwtype : table_ddwtype,
				selectionviewid : table_selectionviewid,
				showallrecordstab: table_showallrecordstab,
				fields			: [ ],
				fieldsList		: [ ],
				fieldsDetail	: [ ]
			}

			table.ddwOptions = new Array();
			
			table.preselectQueries = new Array();
			table.preselectedDynamicQueries = new Array();
			
			// add preselected queries
			var preselectedQueries = tableList.item(ij).getElementsByTagName('query');
			
			if (preselectedQueries.length > 0) {
				for (var qCount = 0; qCount < preselectedQueries.length; qCount++) {
					var queryName = preselectedQueries.item(qCount).getAttribute('name');
					// static query
					if (queryName.indexOf('4daf_unique_') == -1) {
						var query = {
							name : queryName					
						}
						table.preselectQueries.push(query);
					} else {
						// dynamic query
						var fieldId = queryName.substr(12);
						var query = {
							fieldId 	: fieldId,
							names		: [ ],
							updating	: false
						}
						table.preselectedDynamicQueries.push(query);
					}
				}
			}
			
			table.parent = this;
			table.cacheFields = function () {
				dax_bridge.cacheFields (this);
			}
			
			// add table to the array
			dax_bridge.tables.push(table);
		}
		
		if (!isGetSingleTable) {
		
			// update old structure for compatibility reasons
			if (structure)
				buildTableList(http_result);
			
		}
		
		dax_bridge.updating = false;
		dax_reports_getReportList();
		
	}
	
	// check if fields are cached, if not cache them
	this.cacheFields = function (table) {
		if (table.cached == false) {
			this.updateFields (table.tableid, true);
		}
	}

	this.updateFields = function (tableId, syncCall) {
		if (dax_bridge.sessionId) {
			if (!syncCall)
				syncCall = false;
			url = dax_bridge.url + "getFieldList?sessionId=" + dax_bridge.sessionId + "&tableId=" + tableId;
			makeCall (url, dax_bridge.updateFields_handler , null, syncCall);
			debugAlert('<b>Bridge (.updateFields):</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');

			// update ddw information for a selection
			ddwUrl = dax_bridge.url + "getDDWOption?sessionId=" + dax_bridge.sessionId + "&objectid=" + tableId;
			makeCall (ddwUrl, dax_bridge.updateFieldsDDW_handler , null, syncCall);
			debugAlert('<b>Bridge (.updateDDWs):</b>'+'<a href="'+ddwUrl+'" target="_blank">'+ddwUrl+'</a>');

			// update ddw information for a selection
			callbackUrl = dax_bridge.url + "GetCallBack?sessionId=" + dax_bridge.sessionId + "&tableid=" + tableId + "&fieldid=0";
			makeCall (callbackUrl, dax_bridge.updateFieldsCallBack_handler , null, false);
			debugAlert('<b>Bridge (.updateCallBacks):</b>'+'<a href="'+callbackUrl+'" target="_blank">'+callbackUrl+'</a>');
		}
	}

	// after each call is processed, call back original object
	this.updateFieldsAsync = function (tableId, originObject, fieldHandler) {
		if (dax_bridge.sessionId) {

			var callProperties = {
				originObject	: originObject,
				fieldHandler	: fieldHandler
			}
		
			url = dax_bridge.url + "getFieldList?sessionId=" + dax_bridge.sessionId + "&tableId=" + tableId;
			makeCall (url, dax_bridge.updateFields_handler, callProperties, false);
			debugAlert('<b>Bridge (.updateFields):</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');

			// update ddw information for a selection
			ddwUrl = dax_bridge.url + "getDDWOption?sessionId=" + dax_bridge.sessionId + "&objectid=" + tableId;
			//makeCall (ddwUrl, dax_bridge.updateFieldsDDW_handler , callProperties, false);
			debugAlert('<b>Bridge (.updateDDWs):</b>'+'<a href="'+ddwUrl+'" target="_blank">'+ddwUrl+'</a>');

			// update ddw information for a selection
			callbackUrl = dax_bridge.url + "GetCallBack?sessionId=" + dax_bridge.sessionId + "&tableid=" + tableId + "&fieldid=0";
			//makeCall (callbackUrl, dax_bridge.updateFieldsCallBack_handler , callProperties, false);
			debugAlert('<b>Bridge (.updateCallBacks):</b>'+'<a href="'+callbackUrl+'" target="_blank">'+callbackUrl+'</a>');
		}
	}	
	
	
	this.updateFieldsCallBack_handler = function (http_result, syncCall) {
		
		if (!syncCall) {
			if (http_result.readyState != 4) {
				return;
			}
		}

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('updateFieldsCallBack_handler request error.');
			return false;
		}		
				
		
		// get all Callbacks
		var callback = http_result.responseXML.getElementsByTagName("GetCallBack").item(0);
		var callbacks = callback ? callback.getElementsByTagName("CallBack") : null;

		// check for valid reply
		if ((!callbacks) || (callbacks.length == 0))
			return false;
			
		// assign callbacks values to the field objects
		for (h = 0; h < callbacks.length; h++) {
		
			// update table information
			var tableid = callbacks[h].getAttribute("tableid");
			var fieldid = callbacks[h].getAttribute("fieldid");
			var field = getField (tableid,fieldid);	
			
			if (field) {
				switch (callbacks[h].getAttribute("eventid")) {
					case "1": // on load
						field.fieldcallbackonload = true;
					break;
					case "20": // on load
						field.fieldcallbackonchange = true;
					break;						
				}
			}			
		}
	}

	this.updateFieldsDDW_handler = function (http_result, syncCall) {
		
		if (!syncCall) {
			if (http_result.readyState != 4) {
				return;
			}
		}

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('updateFieldsDDW_handler request error.');
			return false;
		}		
		
		// get all DDW options
		var ddwOption = http_result.responseXML.getElementsByTagName("GetDDWOption").item(0);
		var ddwOptions = ddwOption ? ddwOption.getElementsByTagName("DDW") : null;

		// check for valid reply
		if ((!ddwOptions) || (ddwOptions.length == 0))
			return false;

		// update table information
		var tableid = ddwOption.getAttribute("objectid");
		var table = getTable (tableid);	
			
		table.ddwOptions = new Array ();	
			
		// assign DDW values to the table object
		for (h = 0; h < ddwOptions.length; h++) {
		
			var title = ddwOptions[h].getAttribute("title");
			var type = ddwOptions[h].getAttribute("type");
			var url = ddwOptions[h].getAttribute("url");
		
			var ddwOption = new Object();
			
			ddwOption.title = title;
			ddwOption.type = type;
			ddwOption.url = url.replace(/#38;/g,"");
			
			table.ddwOptions.push (ddwOption);
					
		}
	}
	
	this.updateFields_handler = function (http_result, objectProperties, syncCall) {
		
		if (!syncCall) {
			if (http_result.readyState != 4) {
				return;
			}
		}

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('getField request error.');
			return false;
		}
		
		var field = http_result.responseXML.getElementsByTagName("fields").item(0);
		var fieldList = field ? field.getElementsByTagName("field") : null;
		
		var tableid = field.getAttribute("tableid");
				
		if (!fieldList) {
			debugAlert('No visible fields found for table ' + getTable(tableid).tablename + '. Check Admin preferences for field visibility.');
			return false;
		}	

		// add field to appropriate array
		table = getTable (tableid);
		
		if (table.columnWidths) {
			var columnWidths = table.columnWidths.split('_');
		}
		
		table.fields = [ ];
		
		// now we are going to update the field attributes
		for (var i = 0; i < fieldList.length; i++) {
			var field_alias = fieldList.item(i).getAttribute("alias");
			var field_id = fieldList.item(i).getAttribute("id");
			var field_indetail = fieldList.item(i).getAttribute("indetail");
			var field_inlist = fieldList.item(i).getAttribute("inlist");
			var field_name = fieldList.item(i).getAttribute("name");
			var field_type = fieldList.item(i).getAttribute("type");
			var field_indexed = fieldList.item(i).getAttribute("indexed");
			var field_invisible = fieldList.item(i).getAttribute("invisible");
			var field_mandatory = fieldList.item(i).getAttribute("mandatory");
			var field_nonEnterable = fieldList.item(i).getAttribute("nonEnterable");
			var field_nonModifiable = fieldList.item(i).getAttribute("nonModifiable");
			var field_unique = fieldList.item(i).getAttribute("unique");
			var field_breaklevel = fieldList.item(i).getAttribute("breaklevel");
			var field_format = fieldList.item(i).getAttribute("format");
			var field_choicelist = fieldList.item(i).getAttribute("choicelist");
			
			if ((field_type.toLowerCase() == 'date') && (!field_format))
				field_format = 'd1';
			
			if (columnWidths) {
				var colElement = findInArray(columnWidths, field_id, null, null, true);
				if (colElement != -1) {
					var field_columnwidth = columnWidths[colElement].split(':')[1];
				} else {
					var field_columnwidth = null;
				}
			} else {
				var field_columnwidth = null;
			}
			
			// add to list of tables
			var field = {
				fieldalias : field_alias,
				fieldid : field_id,
				fieldindetail : field_indetail,
				fieldinlist : field_inlist,
				fieldname : field_name,
				fieldtype : field_type,
				fieldindexed : field_indexed,
				fieldinvisible : field_invisible,
				fieldmandatory : field_mandatory,
				fieldnonEnterable : field_nonEnterable,
				fieldnonModifiable : field_nonModifiable,
				fieldunique : field_unique,
				fieldbreaklevel : field_breaklevel,
				fieldformat : field_format,
				fieldchoicelist : field_choicelist,
				fieldcolumnwidth : field_columnwidth,
				table			: table
			}

			if (!table.fields) {
				table.fields = new Array();
			}
			table.fields.push (field);
			if (field_inlist.toLowerCase() == 'true')
				table.fieldsList.push(field);
			if (field_indetail.toLowerCase() == 'true')
				table.fieldsDetail.push(field);
		}
		// set table as cached
		table.cached = true;
		
		// for compatibility
		drawFields (http_result, tableid);
		
		if (objectProperties)
			objectProperties.fieldHandler(objectProperties.originObject);
		
	}

	this.deleteRecords = function(selectionName, recordId, handler, objectRef) {

		this.cacheFields (selectionName);
		if ((getTable(selectionName)) && (recordId) && (this.sessionId)) {

			var url = dax_bridge.url + "deleteRecord?sessionId=" + dax_bridge.sessionId +
				"&tableId=" + getTable(selectionName).tableid + "&recordId=";
			
				if (isArray(recordId)) {
					// delete multiple
					for (b = 0; b < recordId.length; b++) {
						url += recordId[b];
						if ((b + 1) != recordId.length)
							url += ","
					}
					
				} else {
					// delete single
					url += recordId;
				}
				
			makeCall (url, handler, objectRef);
		
		}
	}
	
	this.addRecord = function(selectionName, fieldArray, valueArray, handler, objectRef) {

		this.cacheFields (selectionName);
		var url = this.url + "addRecord?sessionid="+this.sessionId+"&tableid=" + getTable(selectionName).tableid;

		// compose adding string
		for (var fieldCount = 0; fieldCount < fieldArray.length; fieldCount++) {
			url += "&field" + getField(selectionName, fieldArray[fieldCount]).fieldid + "=" + valueArray[fieldCount];
		}
	
		makeCall (url, handler, objectRef);
	
	}
	
	this.modifyRecord = function(selectionName, fieldArray, valueArray, recordId, handler, objectRef) {

		this.cacheFields (selectionName);
		var url = this.url + "modifyRecord?sessionid="+this.sessionId+"&tableid=" + getTable(selectionName).tableid;

		// compose adding string
		for (var fieldCount = 0; fieldCount < fieldArray.length; fieldCount++) {
			url += "&field" + getField(selectionName, fieldArray[fieldCount]).fieldid + "=" + valueArray[fieldCount];
		}
		
		url += "&recordid=" + recordId;
	
		makeCall (url, handler, objectRef);
	
	}
	
	this.query = function(selection) {

		this.objectRef = null;
		this.handler = function () { };
		
		this.updateQueryId = true;
		this.queryId = null;
		this.queryName = null;
		this.currentQueryId = null;
		
		this.start = 1;
		this.length = 0;
		this.size = 0;
		this.recordsInSelection = 0;
		this.lastRecord = false;
	
		this.sortOrder = null;
		this.sortField = null;
		this.refreshOnSort = false;
		this.sortHandler = function () { };

		this.customValuesFrom4D = {
			name	: [ ],
			value	: [ ]		
		}
		
		this.queryEngine = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}		

		// separate list for live searches that don't update query id
		this.queryEngineSearch = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}	
		
		// sent as additional attributes in url
		this.customValues = {
			name		: [],
			value		: []
		}
		
		// max number of characters
		this.maxChar = null;
		
		if (selection)
			this.table = getTable(selection);
		
		this.setNamedQuery = function(queryName) {
			this.queryId = null;
			this.queryName = queryName;		
		}
		
		this.sort = function(field, sortOrder, refreshWhenDone) {

			var sortField = getField(this.table.tableid, field);
			
			if (sortField)
				this.sortField = sortField;
	
			this.refreshOnSort = refreshWhenDone;
	
			this.sortOrder = ((sortOrder == 'desc') ? 'desc' : 'asc');
	
			var url = dax_bridge.url+"Sort?sessionId="+dax_bridge.sessionId+
				"&tableid=" + this.table.tableid+
				"&queryid="+this.queryId+
				"&fieldid="+this.sortField.fieldid+
				"&direction="+this.sortOrder;

			makeCall (url, this.sortHandlerInternal, this);
					
		}
		
		this.sortHandlerInternal = function(http_response, queryRef){

			if (http_response.readyState != 4)
				return;

			if (checkResponse(http_response))
				return;
				
			if (queryRef.refreshOnSort)
				queryRef.getRecords();
			
			queryRef.sortHandler();
		
		}

		this.getRecords = function(startRecord, length) {
		
			if (startRecord)
				this.start = startRecord;
				
			if (length)
				this.length = length;

			// use query engine if query engine data is present, otherwise go with regular query
			if (this.queryEngine.field.length > 0) {
				var url = dax_bridge.url+"QueryEngine?sessionId="+dax_bridge.sessionId+
					"&dax_qe_table="+this.table.tableid+"&dax_qe_start="+this.start+"&dax_qe_length="+this.length;
				for (s = 0; s < this.queryEngine.field.length; s++) {
					if ((this.queryEngine.field[s]) && (this.queryEngine.operator[s]) && (this.queryEngine.value[s]) && (this.queryEngine.andorflag[s])) {
						url += "&dax_qe_field" + (s + 1) + "=" + this.queryEngine.field[s] +
							"&dax_qe_argument" + (s + 1) + "=" + this.queryEngine.operator[s] + 
							"&dax_qe_searchvalue" + (s + 1) + "=" + dax_bridge.encode(this.queryEngine.value[s]) + 
							"&dax_qe_andor" + (s + 1) + "=" + this.queryEngine.andorflag[s];
					}
				}
				
				if (this.queryId)
					url += "&queryid="+this.queryId;					
				
			} else {
			
				if (this.queryId) {
					var url = dax_bridge.url+"Query?sessionId="+dax_bridge.sessionId+
						"&queryid=" + this.queryId+
						"&start="+this.start+
						"&length="+this.length;
				} else if (this.queryName) {
					var url = dax_bridge.url+"Query?sessionId="+dax_bridge.sessionId+
						"&tableid=" + this.table.tableid+
						"&queryname=" + this.queryName+
						"&start="+this.start+
						"&length="+this.length;
				} else {
					var url = dax_bridge.url+"AllRecords?sessionId="+dax_bridge.sessionId+
						"&tableid=" + this.table.tableid+
						"&start="+this.start+
						"&length="+this.length;
				}
			}
			
			if (this.queryEngineSearch.field.length > 0) {
				var url = dax_bridge.url+"QueryEngine?sessionId="+dax_bridge.sessionId+
					"&dax_qe_table="+this.table.tableid+"&dax_qe_start="+this.start+"&dax_qe_length="+this.length;
				for (s = 0; s < this.queryEngineSearch.field.length; s++) {
					if ((this.queryEngineSearch.field[s]) && (this.queryEngineSearch.operator[s]) && (this.queryEngineSearch.value[s]) && (this.queryEngineSearch.andorflag[s])) {
						url += "&dax_qe_field" + (s + 1) + "=" + this.queryEngineSearch.field[s] +
							"&dax_qe_argument" + (s + 1) + "=" + this.queryEngineSearch.operator[s] + 
							"&dax_qe_searchvalue" + (s + 1) + "=" + dax_bridge.encode(this.queryEngineSearch.value[s]) + 
							"&dax_qe_andor" + (s + 1) + "=" + this.queryEngineSearch.andorflag[s];
					}
				}
			}
			
			// max number of characters
			if (this.maxChar)
				url += "&maxchar=" + this.maxChar;
			
			// append custom values
			if (this.customValues.name.length > 0) {
				for (var customValueCount = 0; customValueCount < this.customValues.name.length; customValueCount++) {
					url += "&" + dax_bridge.encode(this.customValues.name[customValueCount]) + "=" + dax_bridge.encode(this.customValues.value[customValueCount]);
				}
			}
			
			makeCall (url, this.handler, this.objectRef);

		}
			
	}

	// parse query result
	// placed outside query object so it can be used by other calls such as getRecord
	this.parseQuery = function (http_result, queryRef) {
			
		var result = http_result.responseXML.getElementsByTagName("queryResult").item(0);
		if (!result)
			return;

		if (queryRef) {
			
			// update query values
			if (queryRef.updateQueryId)
				queryRef.queryId = result.getAttribute('queryid');
			queryRef.currentQueryId = result.getAttribute('queryid');
			queryRef.recordsInSelection = result.getAttribute('recordsinselection');
			queryRef.size = result.getAttribute('size');

			// clear custom values from 4d
			queryRef.customValuesFrom4D = {
				name	: [ ],
				value	: [ ]		
			}
			
			// check if 4d sent any custom values, if so add them to the array
			var customValues = result.getElementsByTagName('variable');		
			if (customValues.length > 0) {
				for (var customValuesCount = 0; customValuesCount < customValues.length; customValuesCount++) {
					queryRef.customValuesFrom4D.name.push(customValues[customValuesCount].getAttribute('name'));
					queryRef.customValuesFrom4D.value.push(customValues[customValuesCount].getAttribute('value'));
				}			
			}
			
			// check if it is last batch
			if ((parseFloat(queryRef.start) + parseFloat(queryRef.size)) <= queryRef.recordsInSelection) {
				queryRef.currentPos += parseFloat(queryRef.size);
			} else {
				queryRef.lastRecord = true;
				queryRef.currentPos = queryRef.size;
			}
		
		}
		
		// parse row results
		var queryResult = [ ];
		var resultRows = result.getElementsByTagName('row');
		var tableId = result.getAttribute('tableid');
		
		for (i = 0; i < resultRows.length; i++) {
			var fields = new Array();
			var resultFields = resultRows[i].getElementsByTagName('field');
			for (j = 0; j < resultFields.length; j++) {
			
				var value = (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : ('');
			
				// format value
				if (value) {
					var elField = getField (tableId, resultFields[j].getAttribute('id'));
					var elFieldType = elField.fieldtype;
					var elFormat = elField.fieldformat;
					if ((elFieldType == 'boolean') || (elFieldType == 'date')) {
						value = doFormat(value, '4daf_default', elFormat, elFieldType); //format 
					} else {
						if (elFormat) {
							value = format(elFormat, value); //old format
						}
					}				
				}
			
				var field = {
					id : resultFields[j].getAttribute('id'),
					value : value,
					originalValue : (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : (''),
					ddw : resultFields[j].getAttribute('ddw'),
					ddwtype : resultFields[j].getAttribute('ddwtype'),
					fieldrecordid : resultFields[j].getAttribute('fieldrecordid'),
					height : resultFields[j].getAttribute('height'),
					width : resultFields[j].getAttribute('width')
				}
			fields.push(field);
			}
			var row = {
				recordid : resultRows[i].getAttribute('recordid'),
				selectionid : resultRows[i].getAttribute('selectionid'),
				fields : fields
			}
		queryResult.push(row);
		}
		
		if (queryRef)
			queryRef.parsedData = queryResult;
		
		return queryResult;
	}
	
	this.getRecord = function (selection, recordId, handler, objectRef) {

		var aSelection = getTable(selection);
		if (!aSelection)
			return false;
		
		var getRecordProperties = {
			handler		: handler,
			objectRef	: objectRef
		}
			
		var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + aSelection.tableid + "&recordid=" + recordId;
		makeCall (url, this.getRecord_handler, getRecordProperties);	
	}
	
	this.getRecord_handler = function (http_result, getRecordProperties) {
	
		if (http_result.readyState != 4) {
			return;
		}

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('get init values request error.');
			return false;
		}

		var parsedRecord = dax_bridge.parseQuery(http_result);
		
		// call handler and pass object reference and record info
		getRecordProperties.handler(parsedRecord, getRecordProperties.objectRef);
		
	}
	
	this.getChoiceList = function(listName, handler, objectRef) {
		var url = dax_bridge.url + "GetChoiceList?sessionid=" + dax_bridge.sessionId + "&choicelist=" + listName;
		
		makeCall(url, handler, objectRef);
	}
	
	this.getInitInfo = function() {
		var url = this.url + "getInitInfo";
		
		makeCall(url, this.getInitInfo_handler);
	
	}
	
	this.getInitInfo_handler = function(http_result) {

		if (http_result.readyState != 4) {
			return;
		}

		var errorResponse = checkResponse(http_result);
		
		if (errorResponse) {
			debugAlert('get init values request error.');
			return false;
		}
		
		var encoding = http_result.responseXML.getElementsByTagName("charset").item(0);
		var version = http_result.responseXML.getElementsByTagName("version").item(0);
		
		if (encoding.firstChild.nodeValue.toLowerCase() == 'iso-8859-1') {
			dax_bridge.serverSet = 'Windows-1251';
		} else {
			dax_bridge.serverSet = 'UTF-8';
		}
		
		if (version)
			dax_bridge.version = version.firstChild.nodeValue.toLowerCase();
	
	}

	this.updateUniqueQueries = function(tableName, externalHandler, objectRef) {
	
		var currentTable = getTable(tableName);
		
		if (!currentTable)
			return false;
		
		if (currentTable.preselectedDynamicQueries.length > 0) {
			for (var qCount = 0; qCount < currentTable.preselectedDynamicQueries.length; qCount++) {
				// check for valid string
				if (currentTable.preselectedDynamicQueries[qCount].fieldId.indexOf('][') != -1) {
					// generate a call to backend to get the list of unique values
					currentTable.preselectedDynamicQueries[qCount].updating = true;
					var url = dax_bridge.url + "GetUniqueQueries?sessionId=" + dax_bridge.sessionId + "&tableid=" + currentTable.tableid +
						"&fieldid=" + currentTable.preselectedDynamicQueries[qCount].fieldId;
					
					var targetObject = {
						externalHandler		: externalHandler,
						objectRef			: objectRef
					}
						
					makeCall (url, this.updateUniqueQueries_handler, targetObject);
				}
			}
		}
	}
	
	this.updateUniqueQueries_handler = function(http_result, targetObject) {

		if (http_result.readyState != 4)
			return;

		var responseError = checkResponse (http_result);
			
		if (responseError)	
			return;

		var queryNode = http_result.responseXML.getElementsByTagName("queries").item(0);
		var queryNames = queryNode ? queryNode.getElementsByTagName("query") : null;		

		var fieldId = queryNode.getAttribute("fieldid");
		var tableId = queryNode.getAttribute("tableid");
		
		var currentTable = getTable(tableId);

		var allPresetQueriesLoaded = true;
		
		// look for preset query field and update it with values
		if (currentTable.preselectedDynamicQueries.length > 0) {
			for (var qCount = 0; qCount < currentTable.preselectedDynamicQueries.length; qCount++) {
				if (currentTable.preselectedDynamicQueries[qCount].fieldId == fieldId) {
					currentTable.preselectedDynamicQueries[qCount].names = [ ];
					for (var queryCount = 0; queryCount < queryNames.length; queryCount++) {
						var queryName = queryNames.item(queryCount).getAttribute('name');
						// disregard empty string and remove '4daf_unique_'
						if (queryName != '4daf_unique_')
							currentTable.preselectedDynamicQueries[qCount].names.push(queryName.substr(12));
						
						currentTable.preselectedDynamicQueries[qCount].updating = false;
					}
				} else {
					// if all queries are loaded, call a handler function later
					if (currentTable.preselectedDynamicQueries[qCount].updating)
						allPresetQueriesLoaded = false;				
				}
			}
		}
		
		// if all preset queries are loaded, call external handler
		if (allPresetQueriesLoaded)
			targetObject.externalHandler(targetObject.objectRef);

	}
	
	//
	this.executeCallback = function(tableId, fieldId, value, eventType, externalHandler, objectRef) {

		// onload event id = 1, onchange event id = 20
		if (eventType == 'onload')
			eventId = 1;
		else
			eventId = 20;
	
		var url = dax_bridge.url + 
			"ExecuteCallBack?sessionId=" + dax_bridge.sessionId + 
			"&eventid=" + eventId +
			"&tableid=" + tableId +
			"&fieldid=" + fieldId;
		
		if (eventType != 'onload')
			url += "&value=" + value;
		
		var targetObject = {
			externalHandler		: externalHandler,
			objectRef			: objectRef
		}
			
		makeCall (url, this.executeCallback_handler, targetObject);
	}
	
	this.executeCallback_handler = function(http_result, targetObject) {

		if (http_result.readyState != 4)
			return;

		var responseError = checkResponse (http_result);
			
		if (responseError)	
			return;

		var callbackValues = http_result.responseXML.getElementsByTagName("fieldvalue");
		
		var callbackReply = {
			values	: [ ]
		}
		
		for (var callbackValuesCount = 0; callbackValuesCount < callbackValues.length; callbackValuesCount++) {

			var callbackValue = {
				fieldId		: callbackValues[callbackValuesCount].getAttribute('fieldid'),
				value		: callbackValues[callbackValuesCount].getAttribute('value')
			}
			
			callbackReply.values.push(callbackValue);
		}
		
		targetObject.externalHandler(callbackReply, targetObject.objectRef);
	
	}
	
	// on load events
	this.getInitInfo();	

}

function dax_reports_getReportList(selection) {

		var tableId = null;

		if (selection)
			var tableId = getTable(selection);
			
		// get all tables is selection is not passed
		if (!tableId)
			tableId = "0";
		
		var url = dax_bridge.url + "ReportsList?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId;
		
		// get list of reports
		makeCall(url, dax_reports_getReportListHandler);

}

function dax_reports_getReportListHandler(http_response) {

	if (http_response.readyState != 4)
		return;
				
	var reportElements = http_response.responseXML.getElementsByTagName('report');
	
	dax_bridge.reports = [ ];
	
	// clear out table report arrays
	for (var tableCount = 0; tableCount < dax_bridge.tables.length; tableCount++) {
		dax_bridge.tables[tableCount].reports = [ ];
	}
	
	for (var reportCount = 0; reportCount < reportElements.length; reportCount++) {

		var reportTable = getTable(reportElements[reportCount].getAttribute('tableid'));
		if (reportTable) {
			var report = {
				description	: reportElements[reportCount].getAttribute('description'),
				table		: reportTable,
				id			: ReplaceString(reportElements[reportCount].getAttribute('id'), ".xml", ""), //take .xml out of file name
				name		: reportElements[reportCount].getAttribute('name'),
				type		: reportElements[reportCount].getAttribute('type')
			}
			if (reportTable.reports == undefined) 
				reportTable.reports = [];
			reportTable.reports.push(report);
		}
		
		// place report info in bridge and in specific table arrays
		dax_bridge.reports.push(report);
		
	}
	
	if (onAfterInit) {
		onAfterInit();
	}
	
	if (aSidebar)
		appendReportsToSidebar();

}

// sets preset sessionId
function initSessionId (sessionId) {

	if (dax_bridge) {
		dax_bridge.sessionId = sessionId;
		dax_bridge.updateTables();
	}
}

// gets table by name or id
function getTable (nameOrId) {

	// check if valid value was passed
	if ((!nameOrId) && (!dax_bridge)) {
		return null;
	}

	// check if it is a number
	if (isNaN(nameOrId)) {
		// not a number, search by name
		for (iTab = 0; iTab < dax_bridge.tables.length; iTab++) {
			if (dax_bridge.tables[iTab].tablename == nameOrId) {
				return dax_bridge.tables[iTab];
			}
		}
	} else {
		// number, search by id
		for (iTab = 0; iTab < dax_bridge.tables.length; iTab++) {
			if (dax_bridge.tables[iTab].tableid == nameOrId) {
				return dax_bridge.tables[iTab];
			}
		}

		// not found by number, try to get invisible table
		dax_bridge.getSingleTable (nameOrId);

		// try to find it again
		for (iTab = 0; iTab < dax_bridge.tables.length; iTab++) {
			if (dax_bridge.tables[iTab].tableid == nameOrId) {
				return dax_bridge.tables[iTab];
			}
		}
	}
	return null;
}

// gets field object by name or id
function getField (tableNameOrId, fieldNameOrId) {

	// check if valid value was passed
	if ((!tableNameOrId) && (!dax_bridge)) {
		return null;
	}

	// check if field is passed in [x][y] format, if so, assign it to table var
	//if (!(fieldNameOrId.toString().indexOf('][') == -1)) {
	//	tableNameOrId = fieldNameOrId;
	//}
	
	// check if table is passed in [x][y] format
	
	if (tableNameOrId.toString().indexOf(']') != -1) {
		fieldNameOrId = tableNameOrId;
		if (tableNameOrId.toString().indexOf('][') != -1)
			tableNameArray = tableNameOrId.split('][');
		else
			tableNameArray = tableNameOrId.split(']');
		if (tableNameArray[0]) {
			tableNameOrId = tableNameArray[0].substring (1);
			if (!tableNameOrId)
				return;
		}
	}
	
	table = getTable (tableNameOrId);
	
	if (!table)
		return;

	//check if field name is number, if so convert it into [x][y] format
	if (!(isNaN(fieldNameOrId))) {
		fieldNameOrId = '[' + table.tableid + '][' + fieldNameOrId + ']';
	}
		
	// check if table has field properties, if not get them
	dax_bridge.cacheFields (table);
	
	// check if it is a [x][y] format, if not it's text
	if (fieldNameOrId) {
		if (fieldNameOrId.indexOf('][') == -1) {
			// not a [x][y], search by name
			if (fieldNameOrId.indexOf(']') != -1) {
				newFieldNameArray = fieldNameOrId.split(']');
				fieldNameOrId = newFieldNameArray[1];
			}
			for (var iFil = 0; iFil < table.fields.length; iFil++) {
				if (table.fields[iFil].fieldname == fieldNameOrId){
					return table.fields[iFil];
				}
			}
		} else {
			// number, search by id
			for (var iFil = 0; iFil < table.fields.length; iFil++) {
				if (table.fields[iFil].fieldid == fieldNameOrId) {
					return table.fields[iFil];
				}
			}	
		}
	}
	return null;
}



// 4D query object
function Query (table) {

	// table that's queried
	if (table)
		this.table = getTable (table);
	
	// batch mode gets bunch of records at once
	// single mode (not batch) gets one at a time
	this.batchMode = true;
	this.lastRecord = false;

	// query call handler will receive this reference as second parameter
	this.callRef = null;
	
	// stop query
	this.die = false;
	
	// batch size, max size depends on backend
	this.batchSize = 10;
	this.maxBatchSize = 50;
	this.currentPos = 1;
	
	// conditional query options
	this.queryFieldId = null;
	this.queryComp = null;
	this.queryValue = null;
	this.queryEndRangeId = null;
	
	// parameters for getting break levels
	this.breakLevel = 0;
	this.breakSelected = null;
	this.breakGetLevel = null;
	this.breakQueryId = null;

	// custom parameters
	this.optionName = new Array();
	this.optionValue = new Array();
	
	// maximum number of characters
	this.maxChar = null;
	
	// formatting parent
	this.formatObject = null;
	
	// for named queries
	this.queryName = null;
	
	// which fields to display?
	this.displayFields = new Array();
	
	// declare variables returned by query reply
	this.queryId 				= null;
	this.recordsInSelection 	= 0;
	this.size					= 0;

	// update query id when reply is parsed?
	this.updateQueryId = true;

	
	// contents of this dom will be populated if it is set
	// batch uses grid, and will take care of row addition & removal automatically
	// single mode will take any div tag, and looks for html ids that equals to field name
	this.autoPopulate = null;

	this.clearQueryEngine = function () {
		// queryengine-type calls
		this.queryEngine =
		{
			field		: [],
			argument	: [],
			value		: [],
			andor		: []
		}	
	}
	
	this.clearQueryEngine();
	
	//batch mode commands
	this.nextBatch = function (handlerMethod) {
		if (!this.batchMode)
			return;
		if (this.die) {
			this.lastRecord = true;
			return;
		}
		
		// check if we have queryid, if not make initial query
		if ((!this.queryId) || (!this.currentPos))
			this.currentPos = 1;
		
		if (!this.queryId) {
			this.call (handlerMethod, this.currentPos, this.batchSize);
		} else {
			if (!this.currentPos)
				return;
			this.call (handlerMethod, this.currentPos, this.batchSize);
		}
	}
	
	// make query
	this.call = function (handlerMethod, start, length) {
		// check if we're logged in
		if (dax_bridge.sessionId) {
			// use existing query?
			if (this.queryId) {
				if (this.queryEngine.field.length > 0) {
					var url = dax_bridge.url+"QueryEngine?sessionId="+dax_bridge.sessionId+
						"&queryid="+this.queryId+"&dax_qe_table="+this.table.tableid+"&dax_qe_start="+start+"&dax_qe_length="+length;
					for (s = 0; s < this.queryEngine.field.length; s++) {
						if ((this.queryEngine.field[s]) && (this.queryEngine.argument[s]) && (this.queryEngine.value[s]) && (this.queryEngine.andor[s])) {
							url += "&dax_qe_field" + (s + 1) + "=" + this.queryEngine.field[s] +
								"&dax_qe_argument" + (s + 1) + "=" + this.queryEngine.argument[s] + 
								"&dax_qe_searchvalue" + (s + 1) + "=" + dax_bridge.encode(this.queryEngine.value[s]) + 
								"&dax_qe_andor" + (s + 1) + "=" + this.queryEngine.andor[s];
						}
					}
				// check if we use breakpoints
				} else if (this.breakLevel == 0) {
					// no breakpoints
					var url = dax_bridge.url+"query?sessionId="+dax_bridge.sessionId+"&queryid="+
						this.queryId+"&start="+start+"&length="+length+"&tableid=" + this.table.tableid;;
				}
			// don't use existing query
			} else if (this.queryName) {
				if (this.queryEngine.field.length > 0) {
					var url = dax_bridge.url+"QueryEngine?sessionId="+dax_bridge.sessionId+
						"&queryname="+dax_bridge.encode(this.queryName)+"&dax_qe_table=" + this.table.tableid+"&dax_qe_start="+start+"&dax_qe_length="+length;	
					for (s = 0; s < this.queryEngine.field.length; s++) {
						if ((this.queryEngine.field[s]) && (this.queryEngine.argument[s]) && (this.queryEngine.value[s]) && (this.queryEngine.andor[s])) {
							url += "&dax_qe_field" + (s + 1) + "=" + this.queryEngine.field[s] +
								"&dax_qe_argument" + (s + 1) + "=" + this.queryEngine.argument[s] + 
								"&dax_qe_searchvalue" + (s + 1) + "=" + dax_bridge.encode(this.queryEngine.value[s]) + 
								"&dax_qe_andor" + (s + 1) + "=" + this.queryEngine.andor[s];
						}
					}
				// check if we use breakpoints
				} else if (this.breakLevel == 0) {
					// no breakpoints
					var url = dax_bridge.url+"query?sessionId="+dax_bridge.sessionId+"&queryname="+
						dax_bridge.encode(this.queryName)+"&start="+start+"&length="+length+"&tableid=" + this.table.tableid;
				}
			// don't use existing query
			} else {
			// using query engine?
				if (this.queryEngine.field.length > 0) {
					var url = dax_bridge.url+"QueryEngine?sessionId="+dax_bridge.sessionId+
						"&dax_qe_table="+this.table.tableid+"&dax_qe_start="+start+"&dax_qe_length="+length;	
					for (s = 0; s < this.queryEngine.field.length; s++) {
						if ((this.queryEngine.field[s]) && (this.queryEngine.argument[s]) && (this.queryEngine.value[s]) && (this.queryEngine.andor[s])) {
							url += "&dax_qe_field" + (s + 1) + "=" + this.queryEngine.field[s] +
								"&dax_qe_argument" + (s + 1) + "=" + this.queryEngine.argument[s] + 
								"&dax_qe_searchvalue" + (s + 1) + "=" + dax_bridge.encode(this.queryEngine.value[s]) + 
								"&dax_qe_andor" + (s + 1) + "=" + this.queryEngine.andor[s];
						}
					}
				// check if we use breakpoints
				} else if ((this.breakLevel == 0) && (!this.breakGetLevel)) {
					// new query
					var url = dax_bridge.url+"query?sessionId="+dax_bridge.sessionId+
						"&start="+start+"&length="+length+"&tableid="+this.table.tableid;
					// if search conditions are set, use them
					if ((this.queryFieldId) && (this.queryComp) && (this.queryValue)) {
						// update url to include search conditions
						url = url + 
							"&fieldid=" + this.queryFieldId +
							"&comp=" + this.queryComp +
							"&value=" + dax_bridge.encode(this.queryValue);
						// if end range is passed, use it
						if (this.queryEndRangeId) {
							url = url +
								"&endrangeid=" + this.queryEndRangeId;
						}
					} 
				} else {
					// is it last level breakpoint query?
					if (this.breakGetLevel) {
						var url = dax_bridge.url+
							"query?sessionId="+dax_bridge.sessionId+
							"&start="+start+
							"&length="+length+
							"&tableid="+this.table.tableid+
							"&getbreak="+this.breakGetLevel+
							"&queryid="+this.breakQueryId+
							"&selected="+this.breakSelected;
					// use breakpoints, new query
					} else if ((this.breakSelected) && (this.breakQueryId)) {
						var url = dax_bridge.url+
							"query?sessionId="+dax_bridge.sessionId+
							"&start="+start+
							"&length="+length+
							"&tableid="+this.table.tableid+
							"&breaklevel="+this.breakLevel+
							"&queryid="+this.breakQueryId+
							"&selected="+this.breakSelected;
					} else {
						var url = dax_bridge.url+
							"query?sessionId="+dax_bridge.sessionId+
							"&start="+start+
							"&length="+length+
							"&tableid="+this.table.tableid+
							"&breaklevel="+this.breakLevel;
					}
				}
			}
			
			// visible fields
			if (this.displayFields.length > 0) {
				url += "&display=";
				for (s = 0; s < this.displayFields.length; s++) {
					url += this.displayFields[s] + ",";
				}
			}
			
			// additional options
			if (this.optionName.length > 0) {
				for (s = 0; s < this.optionName.length; s++) {
					url += "&" + this.optionName[s] + "=" + this.optionValue[s];
				}
			}
			
			// max number of characters
			if (this.maxChar)
				url += "&maxchar=" + this.maxChar;
			
			debugAlert('<b>Query:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
			makeCall (url, handlerMethod, this.callRef, false);
		} else {
			// no session id
		}
	}
	
	// parse query result
	this.parseResult = function (http_result) {
		var result = http_result.responseXML.getElementsByTagName("queryResult").item(0);
		if (!result)
			return;

		// update query values
		if (this.updateQueryId)
			this.queryId = result.getAttribute('queryid');
		this.recordsInSelection = result.getAttribute('recordsinselection');
		this.size = result.getAttribute('size');
		
		// check if it is last batch
		if ((parseFloat(this.currentPos) + parseFloat(this.size)) <= this.recordsInSelection) {
			this.currentPos += parseFloat(this.size);
		} else {
			this.lastRecord = true;
			this.currentPos = this.size;
		}
		
		// parse row results
		this.result = new Array();
		var resultRows = result.getElementsByTagName('row');

		for (i = 0; i < resultRows.length; i++) {
			var fields = new Array();
			var resultFields = resultRows[i].getElementsByTagName('field');
			for (j = 0; j < resultFields.length; j++) {
			
				var value = (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : ('');
			
				// format value
				if (value) {
					var elField = getField (this.table.tableid, resultFields[j].getAttribute('id'));
					var elFieldType = elField.fieldtype;
					var elFormat = elField.fieldformat;
					if ((elFieldType == 'boolean') || (elFieldType == 'date')) {
						value = doFormat(value, '4daf_default', elFormat, elFieldType, this.formatObject); //format 
					} else {
						if (elFormat) {
							value = format(elFormat, value); //old format
						}
					}				
				}
			
				var field = {
					id : resultFields[j].getAttribute('id'),
					value : value,
					originalValue : (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : (''),
					ddw : resultFields[j].getAttribute('ddw'),
					ddwtype : resultFields[j].getAttribute('ddwtype'),
					fieldrecordid : resultFields[j].getAttribute('fieldrecordid'),
					height : resultFields[j].getAttribute('height'),
					width : resultFields[j].getAttribute('width')
				}
			fields.push(field);
			}
			var row = {
				recordid : resultRows[i].getAttribute('recordid'),
				selectionid : resultRows[i].getAttribute('selectionid'),
				fields : fields
			}
		this.result.push(row);
		}
	}
	
	// parse breaklevel result
	this.parseBreak = function (http_result) {
		var result = http_result.responseXML.getElementsByTagName("queryResult").item(0);
		if (!result)
			return;

		// update query values
		this.queryId = result.getAttribute('queryid');
		this.recordsInSelection = result.getAttribute('recordsinselection');
		this.size = result.getAttribute('size');
		
		// check if it is last batch
		if ((parseFloat(this.currentPos) + parseFloat(this.size)) <= this.recordsInSelection) {
			this.currentPos += parseFloat(this.size);
		} else {
			this.lastRecord = true;
			this.currentPos = this.size;
		}

		this.fieldid = result.getAttribute('fieldid');
		// parse row results
		this.result = new Array();
		var resultRows = result.getElementsByTagName('row');

		for (i = 0; i < resultRows.length; i++) {
			var fields = new Array();
			var resultFields = resultRows[i].getElementsByTagName('field');
			var row = {
				value: (resultRows[i].firstChild) ? resultRows[i].firstChild.nodeValue : '',
				ddw: resultRows[i].getAttribute('ddw'),
				ddwtype: resultRows[i].getAttribute('ddwtype'),
				ddwtitle: resultRows[i].getAttribute('ddwtitle')
			}

		this.result.push(row);
		}
	}	
}

// clear queries, takes an array as value
function clearQuery (queryIds, isArray) {

	if (!isArray) {

		if (queryIds) {
			// build clear query url
			var url = dax_bridge.url + "clearQuery?sessionid=" + dax_bridge.sessionId + "&queryid=" + queryIds;		
			// make clear query call
			makeCall (url, null, null, null);
		}


	} else {

		if ((queryIds) && (queryIds.length > 0)) {
			// build clear query url
			var url = dax_bridge.url + "clearQuery?sessionid=" + dax_bridge.sessionId + "&queryid=";		
			for (c = 0; c < queryIds.length; c++) {
				url += queryIds[c];
				// add separator
				if (c != (queryIds.length - 1))
					url += ',';
			}
			// make clear query call
			makeCall (url, null, null, null);
		}
	
	}		
}

function makeCall(url, handler, callRef, syncCall) {

	if (!syncCall)
		syncCall = false;

	if (!handler)
		handler = function (http_request, callRef) { };
		
	var http_request = getXMLHttpRequest();
	
	if (!http_request)
		return;

	//debugAlert('<b>Bridge:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a></div>');
		
	if (syncCall == false) {
		http_request.onreadystatechange = function() { handler(http_request, callRef); };
		http_request.open("GET", url, true);
		http_request.send(null);
	} else {
		http_request.open("GET", url, false);
		http_request.send(null);
		handler (http_request, callRef);		
	}
}

// deletes one or multiple records
// pass record id as a string for single or array for multiple 
function deleteRecords (tableId, recordId) {

	if ((tableId) && (recordId) && (dax_bridge.sessionId)) {

	var url = dax_bridge.url + "deleteRecord?sessionId=" + dax_bridge.sessionId +
		"&tableId=" + tableId + "&recordId=";
	
		if (isArray(recordId)) {
			// delete multiple
			for (b = 0; b < recordId.length; b++) {
				url += recordId[b];
				if ((b + 1) != recordId.length)
					url += ","
			}
			
		} else {
			// delete single
			url += recordId;
		}
		
	makeCall (url, null, null, null);
	
	}
}



// fOriginalFormat or fNewFormat are null for defaults
function doFormat(fValue, fOriginalFormat, fNewFormat, fFieldType, fDestination)
{

	// check for format type
	if (fFieldType)
		var formatType = fFieldType.substring(0,1);
	else if (fOriginalFormat)
		var formatType = fOriginalFormat.substring(0,1);
	else
		var formatType = fNewFormat.substring(0,1);
	
	if (!formatType)
		return null;
	
	// set target array to use from resources file
	switch (formatType) {
		case 'b':
			var admin_values = STR['admin_values_boolean'];
		break;
		case 'd':
			var admin_values = STR['admin_values_date'];
		break;	
	}
	
	// set input and output formats
	if (fOriginalFormat) {
		// remove format type from the beginning of the string
		fOriginalFormat = fOriginalFormat.substring(1);

		// if format is default, use first one available from resources
		if (fOriginalFormat == '1')
			fOriginalFormat = admin_values[1].substring(1);
		
	} else {
		fOriginalFormat = admin_values[1].substring(1);
	}
	
	if (fNewFormat) {
		// remove format type from the beginning of the string
		fNewFormat = fNewFormat.substring(1);
		
		// if format is default, use first one available from resources
		if (fNewFormat == '1')
			fNewFormat = admin_values[1].substring(1);
		
	} else {
		fNewFormat = admin_values[1].substring(1);
	}
	
	// branch based on format type
	switch(formatType){
					
		// formatting for date
		case 'd':

			// separate date into three elements
			if ((!fOriginalFormat) || (fOriginalFormat == 'YYYY-MM-DD') || (fOriginalFormat == 'MM-DD-YYYY') || (fOriginalFormat == 'DD-MM-YYYY') || (fOriginalFormat == 'DD-MMM-YYYY') || (fOriginalFormat == 'daf_default'))
				var fValueSplit = fValue.split('-');
			else
				return null;
			
			// check if there are three elements total
			if (fValueSplit.length == 3) {
			
				// extract date information
				switch(fOriginalFormat) {
				
					case 'MM-DD-YYYY':
						var fDay = +fValueSplit[1];
						var fMonth = +fValueSplit[0]; 
						var fYear = +fValueSplit[2];
					break;

					case 'YYYY-MM-DD':
						var fDay = +fValueSplit[2];
						var fMonth = +fValueSplit[1]; 
						var fYear = +fValueSplit[0];
					break;
					
					case 'DD-MM-YYYY':
						var fDay = +fValueSplit[0];
						var fMonth = +fValueSplit[1]; 
						var fYear = +fValueSplit[2];
					break;

					case 'DD-MMM-YYYY':
						var fDay = +fValueSplit[0]; 
						// check if month is passed as a number or string
						if (("" + parseInt(+fValueSplit[1])) == +fValueSplit[1])
							var fMonth = +fValueSplit[1];						
						else
							var fMonth = +(findInArray (STR['calendar_month_abr'], fValueSplit[1], null, true)) + 1;
						var fYear = +fValueSplit[2];
					break;
					
					// backend default
					case '4daf_default':
					default:
						var fDay = +fValueSplit[2];
						var fMonth = +fValueSplit[1]; 
						var fYear = +fValueSplit[0];
					break;					
				}
			
			} else {
			
				// original date value doesn't split into three elements, return null
				return null;
			
			}
			
			// check for obviously invalid values
			if (fMonth > 12)
				fMonth = 12;
				
			if (fMonth < 1)
				fMonth = 1;
			
			// if year is too small, add extra digits
			if (fYear < 999)
				fYear += 2000; 
			
			// validate the date
			fValidateDate = new Date(fYear, (fMonth - 1), fDay);
			
			// if any of the values are not matching, set the default value
			if (fValidateDate.getFullYear() != fYear) {
				var fToday = new Date();
				fYear = fToday.getFullYear();
			}
			
			if (fValidateDate.getMonth() != (fMonth - 1))
				fMonth = 0;
			
			if (fValidateDate.getDate() != fDay)
				fDay = 1;
				


			// add leading zeroes
			if (fMonth < 10)
				fMonth = '0' + fMonth;

			if (fDay < 10)
				fDay = '0' + fDay;
			
			// compose new date
			switch(fNewFormat) {
			
				case 'MM-DD-YYYY':
					var fReturnValue = '' + fMonth  + '-' + fDay + '-' + fYear;
				break;
				
				case 'DD-MM-YYYY':
					var fReturnValue = '' + fDay + '-' + fMonth + '-' + fYear;
				break;

				case 'DD-MMM-YYYY':
					var fReturnValue = '' + fDay + '-' +STR['calendar_month_abr'][(fMonth - 1)] + '-'  + fYear;
				break;
				
				case 'YYYY-MM-DD':
					// backend default
					var fReturnValue = '' + fYear + '-' + fMonth  + '-' + fDay;
				break;

				case 'daf_default':
				default:
					// backend default
					var fReturnValue = '' + fYear + '-' + fMonth  + '-' + fDay;
				break;
			}

			return fReturnValue;				
			
			
		// end date format
		break;
		
		// start boolean format
		case 'b':

			// branch based on original format
			// find out what's the input value
			switch (fOriginalFormat) {
				
				case 'True':
				
					if (STR['ttrue'].toLowerCase() == fValue.toLowerCase())
						var originalValue = true;
					else
						var originalValue = false;
					
				break;

				case 'Yes':

					if (STR['yes'].toLowerCase() == fValue.toLowerCase())
						var originalValue = true;
					else
						var originalValue = false;

				break;

				case 'Male':

					if (STR['male'].toLowerCase() == fValue.toLowerCase())
						var originalValue = true;
					else
						var originalValue = false;

				break;

				case 'daf_default':				
				default:
					if ('true' == fValue.toLowerCase())
						var originalValue = true;
					else
						var originalValue = false;
				break;
			
			}

			// now that we have original value in boolean format, apply destination formatting
			
			if (originalValue) {

				switch (fNewFormat) {
			
					case 'True':
						var fReturnValue = STR['ttrue'];
					break;

					case 'Yes':
						var fReturnValue = STR['yes'];
					break;

					case 'Male':
						var fReturnValue = STR['male'];
					break;

					case 'Checkbox':
						if (fDestination == 'calendar')
							var fReturnValue = '<div style="text-align:center; float: left"><div class="fourdaf_checkbox_check"></div></div>';
						else
							var fReturnValue = '<div style="text-align:center;"><div class="fourdaf_checkbox_check"></div></div>';
					break;

					case 'daf_default':					
					default:
						var fReturnValue = 'True';
					break;
					
				}
				
			} else {

				switch (fNewFormat) {
			
					case 'True':
						var fReturnValue = STR['ffalse'];
					break;

					case 'Yes':
						var fReturnValue = STR['no'];
					break;

					case 'Male':
						var fReturnValue = STR['female'];
					break;

					case 'Checkbox':
						if (fDestination == 'calendar')
							var fReturnValue = '<div style="text-align:center; float: left"><div class="fourdaf_checkbox_uncheck"></div></div>';
						else
							var fReturnValue = '<div style="text-align:center;"><div class="fourdaf_checkbox_uncheck"></div></div>';
					break;					
					
					case 'daf_default':
					default:
						var fReturnValue = 'False';
					break;		
				
				}
			}			
					
			
	return fReturnValue;				
	
	}
}

// generates unique id for the object
function generateUniqueId (obj) {

	var idC = dax_uniqueId++;
	var idN= obj.id;

	if (!idN)
		idN = 'gen_id_';
		
	obj.id = idN + idC;
	
}

// populate all input fields in the following area with data
function placeRecordData (targetRef, objectRef, tableId, recordId, onPopulateFunction) {

	// check if target and object references are passed
	if (!targetRef) {
		debugAlert ('placeRecord data needs target node reference where record data will be placed');
		return false;	
	}

	// do we use object reference, or table & record pair?	
	if (objectRef) {

		// branch based on object type
		switch (objectRef.toString()) {
		
			// image browser cell
			case '[object ImageBrowserCell]':
				
				// get record and table numbers
				if (objectRef.recordId) {
					var recordId = objectRef.recordId;
					var tableId = objectRef.boundTo.table.tableid;
				} else {
					debugAlert ('placeRecordData could not find table or record id for image browser/data matrix.');
					return false;			
				}
				
				var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId + "&recordid=" + recordId;
				
				var infoObj = {
					targetRef	: targetRef,
					objectRef	: objectRef,
					tableId		: tableId,
					recordId	: recordId			
				}

				infoObj.onPopulateFunction = onPopulateFunction;
				makeCall(url, onPlaceRecordData, infoObj);
			
			break;

			// data tree row
			case '[object DataTreeGridRow]':
				
				// get record and table numbers
				if (objectRef.recordId) {
					var recordId = objectRef.recordId;
					var tableId = objectRef.prv.mParent.owner.table.tableid;
				} else {
					debugAlert ('placeRecordData could not find table or record id for data tree row.');
					return false;			
				}
				
				var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId + "&recordid=" + recordId;
				
				var infoObj = {
					targetRef	: targetRef,
					objectRef	: objectRef,
					tableId		: tableId,
					recordId	: recordId			
				}

				infoObj.onPopulateFunction = onPopulateFunction;
				makeCall(url, onPlaceRecordData, infoObj);
			
			break;
			
			// calendar cell
			case '[object CalendarCell]':
				
				// get record and table numbers
				if (objectRef.recordId) {
					var recordId = objectRef.recordId;
					var tableId = objectRef.owner.table.tableid;
				} else {
					debugAlert ('placeRecordData could not find table or record id for calendar cell.');
					return false;			
				}
				
				var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId + "&recordid=" + recordId;
				
				var infoObj = {
					targetRef	: targetRef,
					objectRef	: objectRef,
					tableId		: tableId,
					recordId	: recordId			
				}

				infoObj.onPopulateFunction = onPopulateFunction;
				makeCall(url, onPlaceRecordData, infoObj);
			
			break;
			
			// data window
			case '[object GridRow]':
				
				// get record and table numbers
				if (objectRef.recordId) {
					var recordId = objectRef.recordId;
					var tableId = objectRef.prv.mParent.owner.owner.table.tableid;
				} else {
					debugAlert ('placeRecordData could not find table or record id for calendar cell.');
					return false;			
				}
				
				var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId + "&recordid=" + recordId;
				
				var infoObj = {
					targetRef	: targetRef,
					objectRef	: objectRef,
					tableId		: tableId,
					recordId	: recordId			
				}
				
				infoObj.onPopulateFunction = onPopulateFunction;
				makeCall(url, onPlaceRecordData, infoObj);
			
			break;			
			
		}
	} else {
		// table and record pair
		if ((!tableId) || (!recordId)) {
			debugAlert ("placeRecord needs table and record id since object reference isn't passed.");
			return false;	
		}	
		
		tableId = getTable (tableId);
		
		if (!tableId) {
			debugAlert ("placeRecord could not find the passed table.");
			return false;	
		}
		
		var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + tableId.tableid + "&recordid=" + recordId;
		
		var infoObj = {
			targetRef	: targetRef,
			objectRef	: null,
			tableId		: tableId,
			recordId	: recordId			
		}

		infoObj.onPopulateFunction = onPopulateFunction;
		makeCall(url, onPlaceRecordData, infoObj);
		
	}
}

function onPlaceRecordData (http_response, infoObj) {

	if (http_response.readyState != 4)
		return;
		
	// create new query
	placeQuery = new Query (infoObj.tableId);
	placeQuery.parseResult (http_response);
	
	// get all relevetant elements
	var pageObjects = getElementsByClassName (infoObj.targetRef, '4daf_', true);
	
	if (!(pageObjects.length > 0)) {
		debugAlert ('No valid page objects found for placeRecordData call.');
		return false;
	}
		
	// build the list of references
	for (var objCount = 0; objCount < pageObjects.length; objCount++) {
		// get object class
		var objectClass = pageObjects[objCount].className;
		// split class into elements to extract class name, this is for multi element classes
		var objectClassAr = objectClass.split(' ');
		var objectClassIndex = findInArray (objectClassAr, '4daf_', 0, false, true);
		var objectClassName = ReplaceString(objectClassAr[objectClassIndex], '.', ' ');
		var objectField = getField(objectClassName.slice(5));
		if (!objectField) {
			debugAlert ('placeRecordData could not find the field ' + objectClassAr[objectClassIndex].slice(5));
			return false;
		}
		// we have the id, now find the data and place the object into the html object
		var queryResult = placeQuery.result;
		if (queryResult) {
			// loop through results
			if (queryResult.length > 0) {
				for (var fieldCount = 0; fieldCount < queryResult[0].fields.length; fieldCount++) {
					if (queryResult[0].fields[fieldCount].id == objectField.fieldid) {
						// put the data into the cell
						if ((objectField.fieldtype == 'picture') && (pageObjects[objCount].nodeName.toLowerCase() == 'img')) {
							// picture type into img object
							pageObjects[objCount].src = queryResult[0].fields[fieldCount].value;
						} else if (pageObjects[objCount].nodeName.toLowerCase() == 'input') {
							// text type into input object
							pageObjects[objCount].value = queryResult[0].fields[fieldCount].value;
						} else if (pageObjects[objCount].nodeName.toLowerCase() == 'div') {
							// text type into input object
							pageObjects[objCount].innerHTML = queryResult[0].fields[fieldCount].value;
						} else if (pageObjects[objCount].nodeName.toLowerCase() == 'span') {
							// text type into input object
							pageObjects[objCount].innerHTML = queryResult[0].fields[fieldCount].value;
						} else if (pageObjects[objCount].nodeName.toLowerCase() == 'textarea') {
							// text type into textarea object
							pageObjects[objCount].innerHTML = queryResult[0].fields[fieldCount].value;
						} else if (pageObjects[objCount].nodeName.toLowerCase() == 'td') {
							// text type into td object
							pageObjects[objCount].innerHTML = queryResult[0].fields[fieldCount].value;
						}
					}
				}
			}
		}
	}
	if (infoObj.onPopulateFunction)
		infoObj.onPopulateFunction(infoObj.tableId, infoObj.recordId);
}

// saves data from specific html area
function saveRecordData(targetRef, selection, onSaveFunction) {

	// get all relevetant elements
	var pageObjects = getElementsByClassName (targetRef, '4daf_', true);
	
	if (!(pageObjects.length > 0)) {
		debugAlert ('No valid page objects found for placeRecordData call.');
		return false;
	}

	var url = dax_bridge.url + "addRecord?sessionid="+connection.sessionId+"&tableid=" + getTable(selection).tableid;
	
	// build the list of references
	for (var objCount = 0; objCount < pageObjects.length; objCount++) {
		// get object class
		var objectClass = pageObjects[objCount].className;
		// split class into elements to extract class name, this is for multi element classes
		var objectClassAr = objectClass.split(' ');
		var objectClassIndex = findInArray (objectClassAr, '4daf_', 0, false, true);
		var objectField = getField(objectClassAr[objectClassIndex].slice(5));
		if (!objectField) {
			debugAlert ('placeRecordData could not find the field ' + objectClassAr[objectClassIndex].slice(5));
			break;
		}

		var objectValue = '';
		
		// check for supported object type and build save record string
		if ((pageObjects[objCount].nodeName.toLowerCase() == 'input') && (pageObjects[objCount].type == 'text')){
			objectValue = pageObjects[objCount].value;
		} else if ((pageObjects[objCount].nodeName.toLowerCase() == 'input') && (pageObjects[objCount].type == 'hidden')){
			objectValue = pageObjects[objCount].value;
		} else if (pageObjects[objCount].nodeName.toLowerCase() == 'textarea') {
			objectValue = pageObjects[objCount].value;
		} else if ((pageObjects[objCount].nodeName.toLowerCase() == 'input') && (pageObjects[objCount].type == 'checkbox')){
			objectValue = pageObjects[objCount].checked;
		}
		
		// compose adding string
		url += "&field" + objectField.fieldid + "=" + objectValue;
	}
	
	makeCall(url, onSaveRecordData, onSaveFunction);

}


function onSaveRecordData(http_response, onSaveFunction) {

	if (http_response.readyState != 4)
		return;
		
	if (onSaveFunction)
		onSaveFunction(http_response);		
}


// Check responses, returns true if error is present
function checkResponse(http_request) {

	if (!http_request)
		return true;
	if (!http_request.responseXML)
		return true;
	
	var result = http_request.responseXML.getElementsByTagName("error").item(0);
	if (!result)
		return false;
	var errorCode = result.getElementsByTagName("errorCode");
	if ((errorCode) && (errorCode[0])) {
		
		// get error properties
		var errorElement = errorCode[0];
		var errorHint = errorElement.getAttribute ('hint');
		var errorMessage = errorElement.getAttribute ('message');

		// branch based on error code
		switch (errorHint) {
			// critical errors, reload
			case "IllegalAccessPoint":
			case "InvalidSessionId":
				if (dax_bridge.isClient) {
					if (dax_bridge.shuttingDown)
						return;
					else
						dax_bridge.shuttingDown = true;
					alert (errorMessage);
					window.onbeforeunload = null;
					window.location.reload(false);
					break;
				} else {
					debugAlert ('<b>Error</b> (code: ' + errorHint + '): ' + errorMessage);
					// output to safari and firebug
					if (typeof(console) != 'undefined')
						if (typeof(console.log) != 'undefined')
							console.log ('4DAF error (code: ' + errorHint + '): ' + errorMessage);
					fourdaf_dev_errorTrap(errorHint, errorMessage);
					return true;
				}
			// silent error by default
			default:
				debugAlert ('<b>Error</b> (code: ' + errorHint + '): ' + errorMessage);
				// output to safari and firebug
				if (typeof(console) != 'undefined')
					if (typeof(console.log) != 'undefined')
						console.log ('4DAF error (code: ' + errorHint + '): ' + errorMessage);
				fourdaf_dev_errorTrap(errorHint, errorMessage);
				return true;
		}
	}
}

// Populate list with choice list stored on 4D side
function DAX_Bridge_GetChoiceList(listName, listToPopulate) {

	var url = dax_bridge.url + "GetChoiceList?sessionid=" + dax_bridge.sessionId + "&choicelist=" + listName;
	
	makeCall(url, DAX_Bridge_GetChoiceList_handler, listToPopulate);
}

// Populate list handler
function DAX_Bridge_GetChoiceList_handler(http_result, listToPopulate) {

	if (http_result.readyState != 4) {
		return;
	}

	var errorResponse = checkResponse(http_result);
	
	if (errorResponse) {
		debugAlert('getChoiceList request error.');
		listToPopulate.choicelistloaded = true;
		return false;
	}
	
	// keep current list value
	if (listToPopulate.selectedIndex > -1)
		var currentValue = (listToPopulate.options[listToPopulate.selectedIndex].value) ? (listToPopulate.options[listToPopulate.selectedIndex].value) : null;
	else
		var currentValue = null;
	var currentValueAdded = false;
	
	var list = http_result.responseXML.getElementsByTagName("list").item(0);
	var lists = list ? list.getElementsByTagName("listitem") : null;
	
	if (currentValue) {
		if (listToPopulate.format.charAt(0) == 'd')
			currentValue = doFormat(currentValue, '4daf_default', listToPopulate.format, 'date');	
	}
	
	if (!lists) {
		if (currentValue)
			listToPopulate.options[0] = new Option(currentValue, currentValue);
		else
			listToPopulate.options[0] = new Option(STR['no_data_found'], null);
		listToPopulate.selectedIndex = 0;
		listToPopulate.choicelistloaded = true;
		return false;	
	}
		
	if (lists.length == 0) {
		if (currentValue)
			listToPopulate.options[0] = new Option(currentValue, currentValue);
		else
			listToPopulate.options[0] = new Option(STR['no_data_found'], null);
		listToPopulate.selectedIndex = 0;	
		listToPopulate.choicelistloaded = true;
		return false;	
	}
	
	listToPopulate.options.length = 0;	
	for (var listCount = 0; listCount < lists.length; listCount++) {
		var listValue = (lists.item(listCount).firstChild) ? (lists.item(listCount).firstChild.nodeValue) : null;
		if (listValue) {
			// must do formatting for dates
			if (listToPopulate.format.charAt(0) == 'd')
				listValue = doFormat(listValue, 'YYYY-MM-DD', listToPopulate.format, 'date');
			listToPopulate.options[listCount] = new Option(listValue, listValue);
			if (currentValue)
				if (listValue == currentValue) {
					listToPopulate.selectedIndex = listCount;
					currentValueAdded = true;
				}
		}
	}

	/*
	// add current value if it was not in the retreived list
	if ((!currentValueAdded) && (currentValue)) {
		listToPopulate.options[listToPopulate.length] = new Option(currentValue, currentValue);
		listToPopulate.selectedIndex = listToPopulate.length - 1;		
	} else if (!currentValue) {
		listToPopulate.selectedIndex = 0;
	}
	*/
	
	listToPopulate.onchange(true);
	if (!listToPopulate.disableChoiceList)
		listToPopulate.disabled = false;

	listToPopulate.choicelistloaded = true;
}



/*
// PHASE OUT STUFF BELOW
*/

//connect.js by Jonathan Le

function Connection()
{
	this.sessionId = "";
	this.language = "";
	
	this.address = new Object();
	this.address.url = "/DAX/";
	this.address.login = this.address.url + "Login";
	this.address.gettablelist = this.address.url + "GetTableList";
	this.address.getfieldlist = this.address.url + "GetFieldList";
	this.address.allrecords = this.address.url + "AllRecords";
	this.address.query = this.address.url + "Query";
	this.address.clearquery = this.address.url + "ClearQuery";
	this.address.getrecord = this.address.url + "GetRecord";
	this.address.deleterecord = this.address.url + "DeleteRecord";
	this.address.modifyrecord = this.address.url + "ModifyRecord";
	this.address.addrecord = this.address.url + "AddRecord";
	this.address.getrelatedlist = this.address.url + "GetRelatedList";
	this.address.executecallback = this.address.url + "ExecuteCallBack";
	this.address.getcallback = this.address.url + "GetCallBack";
	this.address.getDDWInfo = this.address.url + "GetDDWInfo";
	this.address.getgeneralprefs = this.address.url + "GetPreferences";
	this.address.getgrouplist = this.address.url + "AdminGetGroupList";
	this.address.admingettablepreferences = this.address.url + "AdminGetTablePreferences";
	this.address.admingetfieldpreferences = this.address.url + "AdminGetFieldPreferences";
	this.address.adminsettablepreferences = this.address.url + "AdminSetTablePreferences";
	this.address.adminsetfieldpreferences = this.address.url + "AdminSetFieldPreferences";
	this.address.adminsetselectionview = this.address.url + "AdminSetSelectionView";
	this.address.adminsetfieldbreakpoint = this.address.url + "AdminSetBreakPoint";
	this.address.admincreatenewview = this.address.url + "AdminCreateVirtualTable";
	this.address.admindeleteview = this.address.url + "AdminDeleteVirtualTable";
	this.address.admingetrelatedtables = this.address.url + "AdminGetRelatedTable";
	this.address.admingetgeneralprefs = this.address.url + "AdminGetPreferences";
	this.address.adminsetgeneralprefs = this.address.url + "AdminSetPreferences";
	this.address.admingetcallback = this.address.url + "AdminGetCallBack";
	this.address.adminsetcallback = this.address.url + "AdminSetCallBack";
	this.address.adminsettableposition = this.address.url + "AdminSetTablePosition";
	this.address.adminsetfieldposition = this.address.url + "AdminSetFieldPosition";
	this.address.admincreateDDWinfo = this.address.url + "AdminCreateDDWInfo";
	this.address.admindeleteDDWinfo = this.address.url + "AdminDeleteDDW";
	this.address.adminsetDDWtoobject = this.address.url + "AdminSetDDWToObject";
	this.address.admingetQueryList = this.address.url + "AdminGetQueriesList";
	this.address.adminsetQueryInfo = this.address.url + "AdminSetQueryInfo";
	this.address.admindeleteQuery = this.address.url + "AdminDeleteQuery";
	this.address.admingetquerytemplates = this.address.url + "AdminGetQueryTemplates";
	this.address.adminSetQueryTemplate = this.address.url + "AdminSetQueryTemplate";
	this.address.adminSetQueryPosition = this.address.url + "AdminSetQueryPosition";
	this.address.AdminSetAllRecords = this.address.url + "AdminSetAllRecords";
	this.address.AdminGetAllRecords = this.address.url + "AdminGetAllRecords";
	this.address.AdminGetFormats = this.address.url + "getformats";
	this.address.AdminDeleteReport = this.address.url + "DeleteReport";
	this.address.admingetReportList = this.address.url + "ReportsList";
	this.address.adminLoadReport = this.address.url + "LoadReport";
	
	
	this.command = function(aSelector) {
		return this.address.url + aSelector + "?sessionid="+this.sessionId;
	};
		
	this.setAddress =
		function(url)
		{
			this.address.url = url;
		};
	this.admin = false;
}

var connection = new Connection();

function getXMLHttpRequest()
{
    var http_request = false;

    if (window.XMLHttpRequest) { // Mozilla, Safari,...
        http_request = new XMLHttpRequest();
        if (http_request.overrideMimeType) {
            http_request.overrideMimeType('text/xml');
         }
    } else if (window.ActiveXObject) { // IE
    /*@cc_on
    	@if (@_jscript_version >= 5)
	        try {
	            http_request = new ActiveXObject("Msxml2.XMLHTTP");
	        } catch (e) {
	            try {
	                http_request = new ActiveXObject("Microsoft.XMLHTTP");
	            } catch (e) {}
	        }
        @end
   	@*/
    }

	return http_request;
}

// url - string (url to request)
// handler - function (how to handle the request)
// postdata - string of postdata
function makeRequest(url, handler, postdata) {
	var http_request = getXMLHttpRequest();
	
	if (!http_request)
		return;
      
    http_request.onreadystatechange = function() { handler(http_request); };
    
    var command = "GET";
    
    if (postdata)
    	command = "POST"
       
    try {
		http_request.open(command, url, true);
		
		if (postdata)
		   	http_request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		
  	  	http_request.send(postdata);
    } catch(e) {
    	alert(e);
    
    }
}

///////////////////////////////
// Database Functions
///////////////////////////////

function buildFieldList(table_id)
{
	// make another request
	var http_request = getXMLHttpRequest();
	
	if (http_request) {
		// http_request.onreadystatechange = function() {  };

		try {
			var url = connection.address.getfieldlist + "?tableid=" + table_id + "&sessionid=" + connection.sessionId;

			http_request.open("GET", url, false);
  	  		http_request.send(null);
			drawFields(http_request, table_id);
   		} catch(e) {
	    }
	}
}

function drawFields(http_request, table_id) {
	if (http_request.readyState != 4 || http_request.status != 200)
   		return;
   	
	// now build the fields
	var table = http_request.responseXML.getElementsByTagName("fields").item(0);
	var fieldList = table ? table.getElementsByTagName("field") : null;
		
	var tp = structure;
	var table = null;
	
	for (var i in tp.tables) {
		if (tp.tables[i].tableid == table_id) {
			table = tp.tables[i];
			table.fields = new Array();
		}
	}
	
	if (!fieldList) {
		table.updating = false;
		if (browser.ie6)
			http_request.onreadystatechange = function(){};
		else
			http_request.onreadystatechange = null;
		delete http_request;
		return;
	}
	
	// now we update the structure
	for (var i = 0; i < fieldList.length; i++) {
		field_name = fieldList.item(i).getAttribute("alias");
		field_id = fieldList.item(i).getAttribute("id");
		field_type = fieldList.item(i).getAttribute("type");
		field_unique = fieldList.item(i).getAttribute("unique");
		field_mandatory = fieldList.item(i).getAttribute("mandatory");
		field_nonmodifiable = fieldList.item(i).getAttribute("nonModifiable");
		field_nonenterable = fieldList.item(i).getAttribute("nonEnterable");
		field_breaklevel = fieldList.item(i).getAttribute("breaklevel");
		field_format = fieldList.item(i).getAttribute("format");
		field_ddw = fieldList.item(i).getAttribute("ddw");
		field_fieldrecordid = fieldList.item(i).getAttribute("fieldrecordid");
		
		if (table) {
			var field = {
				fieldname : field_name,
				fieldid : field_id,
				fieldtype : field_type,
				fieldunique : field_unique,
				fieldmandatory : field_mandatory,
				fieldnonModifiable : field_nonmodifiable,
				fieldnonEnterable : field_nonenterable,
				fieldbreaklevel : field_breaklevel,
				fieldformat : field_format,
				fieldddw : field_ddw,
				fieldrecordid : field_fieldrecordid
			}
			table.fields.push(field);
		}
	}
	
	// free
	if (table) {
		table.cached = true;
		table.updating = false;
	}
	
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

/**
 * Builds a table in the table pane object.  the table pane object gets the following
 * properties:
 *
 * 
 **/
function buildTableList(http_request)
{
    if (http_request.readyState != 4)
		return;
	
	var database = http_request.responseXML.getElementsByTagName("tables").item(0);
	var tableList = database ? database.getElementsByTagName("table") : null;
	
	if (!tableList)
		return;
	
	structure.tables.length = 0;
	
	// now we are going to update the structure
	for (i = 0; i < tableList.length; i++) {
		var table_id = tableList.item(i).getAttribute("id");
		var table_name = tableList.item(i).getAttribute("alias");

		// add to list of tables
		var table = {
			tableid : table_id,
			tablename : table_name,
			updating : true,
			cached : false
		}

		table.updating = false;	
		
		// update the struct
		structure.tables.push(table);
				
		// build field list.
		// buildFieldList(table_id);
	}
	structure.updating = false;
	structure.cached = true;
	
	// free
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
	
}

function errorCheck(response)
{
	try {
		var errors = response.getElementsByTagName("error");
		if (errors)
			errors = errors.item(0);
		else
			return false;
			
		var errorCode = errors.getElementsByTagName("errorCode");
		if (errorCode)
			errorCode = errorCode.item(0);
		else
			return false;
			
		var text = getTextNode(errorCode);
		err = text.nodeValue;
		
		on_error(err);
		
		return true;
	} catch (e) {
	}
	
	return false;
}

function login(http_request)
{
	// state 3 LOADING
	// state 4 is DONE
	if (http_request.readyState != 4)
		return;

	$('loginspinner').style.display = 'none';

	var result = http_request.responseXML;

	if (checkResponse(http_request)) {
		on_login_error();
		return;
	}
	
	result = result.getElementsByTagName("loginresult").item(0);
	
	if (!result)
		return;
		
	var success = result.getAttribute("success") == "true";
	
	if (success) {
		// save the session id
		var sessionId = result.getElementsByTagName("sessionId").item(0);
		connection.language = result.getAttribute('language');

		if (sessionId) {
			connection.sessionId = sessionId.firstChild.nodeValue;
			connection.admin = sessionId.getAttribute("AdminFlag") != "no";
			if (AdminShowAdmin)
				if (!connection.admin)
					on_login_error();
				else
					on_login_success();
			else
				on_login_success();	
			
			
		} else {
			// network error?
			on_login_error();
		}
	} else {
		// failed, alert the user
		on_login_error();
	}
	
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

/*
function showRecords(event)
{
	var tablerow = this.parentNode;
	do_allRecords(tablerow.table_id, 0, 25);
}
*/

function query(http_request, element)
{
	if (http_request.readyState != 4) {
		return;
	}
	
//	alert(http_request.responseText);

	var result = http_request.responseXML.getElementsByTagName("queryResult").item(0);
	checkResponse(http_request);
	
	if (!result) {
		if (element.onafterload) {
			element.onafterload(false);
		}
		
		return;
	}
	
	var tableid = result.getAttribute("tableid");
	var queryid = result.getAttribute("queryid");
	var tablename = result.getAttribute("tablename");
	var total = result.getAttribute("recordsinselection");
	
	var rows = result.getElementsByTagName("row");
	var content = null;

	var grid = null;
	
	if (element.grid != null && element.grid != undefined) {
		grid = element.grid; // existing grid used
	} else {
		grid = new Grid(); // new grid created
		element.appendChild(grid.dom);
		element.grid = grid;
		
		var columnNames = new Array();
		grid.total = total;
	}
	
	for (var i = 0; i < rows.length; i++) {
		var row = new Array();
		
		var columns = rows.item(i).getElementsByTagName("field");
		for (var k = 0; k < columns.length; k++) {
			var value = getTextNode(columns.item(k));
			if (value != null || value != undefined) {
				row[k] = value.nodeValue;
			} else {
				row[k] = "";
			}
			
			if (i == 0 && isDefined(columnNames)) {
				columnNames[k] = columns.item(k).getAttribute("name");
			}
		}

		grid.insertRow(row);
	}
	
	if (isDefined(columnNames)) {
		for (var i = 0; i < columnNames.length; i++) {
			grid.column(i).setTitle(columnNames[i]);
		}
	
		var selectCol = new GridColumn();
		selectCol.type = 0;
		var rowCount = grid.length;
	} else {
		var selectCol = grid.column(0);
	}

	grid.refresh();
	
	// call event handler
	if (element.onafterload) {
		element.onafterload(true);
	}
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

function refresh_table(obj)
{
	// find the toolbar
	var toolbar = null;
	if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
		return;
	
	var contentArea = toolbar.parent.contentArea;
	var grid = contentArea.grid;	
	grid.reload();
}

function expandRecord(http_request, row)
{
	if (http_request.readyState != 4)
		return;
	
	var result = http_request.responseXML.getElementsByTagName("queryResult").item(0);
	var errorResponse = checkResponse(http_request);
	var tableid = result.getAttribute("tableid");

	var tablebody = row.parentNode;
	var colcount = row.getElementsByTagName("td").length;

	var record = result.getElementsByTagName("row").item(0);
	var recordid = record.getAttribute("recordid");
	
	var fields = record.getElementsByTagName("field");
	
	// create the record input form
	tablebody = drawRecordInput(fields, tablebody, tableid);

	var inputs = tablebody.recordInput.getElementsByTagName("input");
	
	// ok now insert the record values
	for (var i = 0; i < fields.length; i++) {
		var fieldName = fields.item(i).getAttribute("name");
		var fieldid = fields.item(i).getAttribute("id");
		var fieldtype = structure.getField(tableid, fieldid).fieldtype;
		
		for (var k = 0; k < inputs.length; k++) {
			var input = inputs.item(i);
			if (input.name == fieldName) {
				var text = getTextNode(fields.item(i));
				
				switch (fieldtype) {
					case 'picture':
						input.img.src = text.nodeValue;
						input.href.href = input.img.src;
						break;
					default:
						if (text)
							input.value = text.nodeValue;
				}
				
				k = inputs.length;
			}
		}
	}
	
	tablebody.expander.parent = row;
	if (browser.ie)
		tablebody.expander.style.display = "block";
	else
		tablebody.expander.style.display = "table-row";
	
	tablebody.recordInput.recordid = recordid;
	tablebody.recordInput.tableid = tableid;
	tablebody.recordInput.style.visibility = "visible";
	tablebody.recordInput.style.display = "block";
	tablebody.expanded = true;
	
	tablebody.insertBefore(tablebody.expander, row.nextSibling);
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

function selectRecord(event)
{
	// parent of the checkbox
	var row = this.parentNode.parentNode.parentNode;
	
	// highlighting
	if (!row.selected) {
		row.className = "recordHighlight";
		row.selected = true;
	} else {
		if (row.selectionid % 2) {
			row.className = "recordLine";
		} else {
			row.className = "recordLineAlt";
		}
		row.selected = false;
	}
	
	event.cancelBubble = true;
	event.returnValue = false;
}

function delete_records(obj)
{
	var toolbar = null;
	if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
		return;

	var win = toolbar.parent;
	var content = win.contentArea;
	var grid = content.grid;
	
	if (!isDefined(grid.selection) || !isDefined(grid.selection.selected))
		return;

	// ask the user	
	var rv = confirm(STR['confirm_delete_records']);
	
	if (!rv)
		return;
	
	var url = connection.command(DB.DELETE_RECORD);
	var postdata ="sessionid="+connection.sessionId+"&tableid=" + grid.selection.tableId + "&recordid=";
	
	for (recordId in grid.selection.selected) {
		if (grid.selection.selected[recordId])
			postdata += recordId+",";
	}
	
	var http_request = getXMLHttpRequest();
		
	if (!http_request)
		return;
		
	try {

		debugAlert('<b>delete_record url:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
		debugAlert('<b>delete_record post:</b>'+postdata);
		
		http_request.onreadystatechange = function() { deleteRecord(http_request, obj); };
		http_request.open("POST", url, true);
		http_request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	  	http_request.send(postdata);
	} catch (e) {
		alert(e);
		if (browser.ie6)
			http_request.onreadystatechange = function(){};
		else
			http_request.onreadystatechange = null;
		delete http_request;
	}
}

function deleteRecord(http_request, obj)
{
	if (http_request.readyState != 4)
		return;

	var result = http_request.responseXML.getElementsByTagName("result").item(0);
	var errorResponse = checkResponse(http_request);
	
	if (!isDefined(result)) {
		//alert('check response ' + http_request.responseText);
		var errorCode = http_request.responseXML.getElementsByTagName("errorCode");
		if ((errorCode) && (errorCode[0])) {
			// get error properties
			var errorElement = errorCode[0];
			alert (errorElement.getAttribute ('message'));
		}
		if (browser.ie6)
			http_request.onreadystatechange = function(){};
		else
			http_request.onreadystatechange = null;
		delete http_request;
		return;
	}
	
	var toolbar = null;
	if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
		return;

	var win = toolbar.parent;
	var content = win.contentArea;
	var grid = content.grid;	
	
	// clear the selection
	if (grid.selection.selected)
		grid.selection.selected = new Array();
	
	// refresh the table
	inline_query(obj, null, "*");

	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

function save_new_record(recordInput)
{
	//var recordInput = getParentByClassName(obj, "recordInput");
	
	if(!recordInput)
		return;
	
	var url = connection.address.addrecord + "?sessionid="+connection.sessionId+"&tableid=" + recordInput.tableid;
	var fields = getElementsByClassName(recordInput, "fieldValue");
	
	for (var i = 0; i < fields.length; i++) {
		url += "&field" + fields[i].fieldid + "=" + fields[i].value;
	}
	
	makeRequest(url, saveRecord, null);
}

/*

function clearQuery (queryid)
{
	if (!queryid)
		return;
	
	var url = connection.address.clearquery + "?sessionid="+connection.sessionId+"&queryid="+queryid;
	makeRequest(url, clearQuery_handler, null);		

}

function clearQuery_handler (http_reply)
{
}
*/




function save_record(obj)
{
	var recordInput = getParentByClassName(obj, "recordInput");
	
	if(!recordInput)
		return;
	
	var url = connection.address.modifyrecord + "?sessionid="+connection.sessionId+"&tableid=" + recordInput.tableid + "&recordid=" + recordInput.recordid;
	
	// TODO: switch to getElementsByClassName
	var fields = getElementsByClassName(recordInput, "fieldValue");
	
	for (var i = 0; i < fields.length; i++) {
		url += "&field" + fields[i].fieldid + "=" + fields[i].value;
	}
	
	makeRequest(url, saveRecord, null);
}

function saveRecord(http_request)
{
	if (http_request.readyState != 4)
		return;

	//alert(http_request.responseText);	
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

function close_record(obj)
{
	var recordInput = getParentByClassName(obj, "recordInput");
	
	var expander = recordInput.parentNode;
	var tablebody = expander.parerentNode;
	
	expander.style.display = "none";
	tablebody.expanded = false;
}

function create_record(obj)
{
	var toolbar = null;
	if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
		return;

	// cover the content area
	var win = toolbar.parent;
	
	if (win.table.fieldsDetail.length == 0)
		return false;
	
	// find the editor if we have one, otherwise create a new one
	var editor = win.getEditor();
	// clear the editor, set its record to -1 to signify that it is a new record
	//if (isDefined(editor.record)) {
	//	editor.record.recordId = -1;
	//	editor.getDOM().deleteBtn.disabled = false; // make sure an empty record can't be deleted
	//	
	//} else {
	var grid = win.contentArea.grid;
	editor.record = { recordId : -1, tableId: grid.selection.tableId };
	editor.buildEditorByTable(grid.selection.tableId);
	//}
	
	// now we are ready to edit
	//editor.clear();
	editor.edit();
	
/*	
	var body = document.getElementsByTagName("body").item(0);
	var recordPopout = $('recordPopout');
	
	recordPopout = recordPopout.cloneNode(true);
	recordPopout.style.display = "block";
	
	recordPopout.className = "createRecord";
	recordPopout.id = "cr_" + getElementsByClassName(body, "createRecord").length + 1;
	
	var popoutInput = getElementByClassName(recordPopout, "popoutInput");
	
	// the record input
	var recordInput = getElementByClassName(contentArea, "recordInput");
	var tableid = recordInput.tableid;
	var fields = getElementsByClassName(recordInput, "fieldValue");
	
	if (recordInput) {
		recordInput = recordInput.cloneNode(true);
		recordInput.onclick = function(event) { event.cancelBubble = true; };
		recordInput.id = "ri_"+recordPopout.id;
		recordInput.tableid = tableid;
		
		popoutInput.appendChild(recordInput);
		
		// styles
		recordInput.style.visibility = "visible";
		recordInput.style.display = "block";
		
		recordInput.className = "";
		
		// clear the values
		var inputs = getElementsByClassName(recordInput, "fieldValue");
		for (var i = 0; i < inputs.length; i++) {
			inputs[i].value = "";
			inputs[i].onmousedown =
				function(event) {
					event.cancelBubble = true;
				};
			
			// TODO: change this to properly reflect field id
			inputs[i].fieldid = fields[i].fieldid;
			
			var fieldtype = structure.getField(tableid, inputs[i].fieldid).fieldtype;
	
			if (fieldtype == "picture") {
				inputs[i].previousSibling.style.display = "none"; 
				
				inputs[i].style.display = "block";
				inputs[i].type = "file";
			}
		}

		// add the window taskbar item
		var tbitem = do_addTaskbarItem($('window_tbitem').cloneNode(true));
	
		if (tbitem) {
			tbitem.style.display = 'block';
			tbitem.onclick = function() { $(recordPopout.id).style.zIndex = ++WindowManager.maxZIndex; };		
			tbitem.id = "tb_"+recordPopout.id;
		
			var tbitem_name = getElementByClassName(tbitem, "tbitem_name");
			if (tbitem_name)
				tbitem_name.innerHTML = "Create Record";
		}

		// set events for buttons		
		var cancelBtn = getElementByClassName(recordInput, "recordCancel");
		var saveBtn = getElementByClassName(recordInput, "recordSave");
		
		cancelBtn.onclick = 
			function() {
				var body = document.getElementsByTagName('body').item(0);
				body.removeChild(recordPopout);
				
				do_removeTaskbarItem($(tbitem.id));
				
			};
			
		saveBtn.onclick = 
			function() {
				save_new_record($(recordInput.id));
				cancelBtn.onclick();
			}
	}
	
	body.appendChild(recordPopout);
	recordPopout.style.zIndex = ++WindowManager.maxZIndex;
	
	do_enableDragging(recordPopout);
	*/
}

function save_query(obj, tableid, fieldid, queryname, queryId)
{

	// the obj expecting to save the query
	var toolbar = null;
//	var grid = null;
	if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
		return;
		
	var win = toolbar.parent;
//	var grid = getElementByClassName(win.dom, "newgrid");	

	var tabsFrame = win.toolbars[1];
	if (tabsFrame != null && tabsFrame != undefined) {
		var tabs = tabsFrame.parent;
		var previousTooltip = tabs.current.tooltip;
		var tab = tabs.insert(queryname);
		
		tab.m.innerHTML = '<div style="display:inline; float: left">' + queryname + '</div><div class="dax_taskbar_close"></div>';

		if (previousTooltip)
			tab.tooltip = previousTooltip  + ' ' + STR['and'] + ' <br />';
		else
			tab.tooltip = '';

		var deleteButton = getElementByClassName(tab.m, 'dax_taskbar_close');
		
		if (deleteButton) {
			deleteButton.boundTo = tab;
			deleteButton.onclick = function() {
				var tabs = this.boundTo.parent;
				if (!tabs.lock) {
					tabs.lock = true;
					if (this.boundTo.tooltip)
						hideTooltip();
					tabs.remove(this.boundTo.index);
//					tabs.activate(0);
//					tabs.tab(0).dom.onclick();
					tabs.refresh();
					tabs.lock = false;
				}
			}		
		}
			
		tab.tooltip += '<b>' + getField(tableid, fieldid).fieldalias + '</b> ' + STR['starts_with'] + ' <b>' + queryname + '</b>';
			
		// update query id
		obj.owner.currentQueryId = obj.owner.grid.receivedQueryId;			
		obj.owner.grid.currentQueryId = obj.owner.grid.receivedQueryId;			
		obj.owner.search.inputBox.queryId = obj.owner.grid.receivedQueryId;
		obj.owner.grid.selection.queryId = obj.owner.grid.receivedQueryId;
		obj.inputBox.queryId = obj.owner.grid.receivedQueryId;
		
		tab.owner = obj.owner;
		tab.queryId = queryId;
		obj.owner.currentTab = tab;
		tab.onswitch = function()
		{
			if (this.owner.grid.selection)		
				this.owner.grid.selection.selected = new Array();
			// do an inline query
			this.owner.currentTab = this;
			inline_query(obj, fieldid, '*', this.queryId);
			this.owner.search.inputBox.value = '';
			getElementByClassName (this.owner.dom, 'queryname').queryId = this.queryId;
		}
		
		
		tab.userMouseOver = function (aEvent) {
			//this.className = 'dax_sidebar_button ' + this.defaultClass + '_o';
			if (this.tooltip) {
				objectPos = getObjectPosition (this.r);
				showTooltip(this.tooltip, objectPos.x, objectPos.y + this.r.offsetHeight);
			}
		}
		
		
		tab.userMouseOut = function (aEvent) {
			//this.className = 'dax_sidebar_button ' + this.defaultClass;
			if (this.tooltip)
				hideTooltip();
		}
		
		tabs.refresh();
	
	}
}

function activateTab(event, tab)
{
	// the actor is set to active
	tab.actor.className = "tabItemActive";
	
	var parent = tab.parent;
	parent.current = tab;
	
	// every other tab becomes inactive
	for (var i in parent.tabs) {
		if (parent.tabs[i] != tab)
			parent.tabs[i].actor.className = "tabItemInactive";
	}
	
	on_tab_switched(tab);
}


function drawRecordInput(fields, tablebody, tableid)
{
	if (!fields || !tablebody)
		return;
	
	// create another row, set the colspan
	// there should only be one per table
	if (!tablebody.expander) {
		tablebody.expander = document.createElement("tr");
		var area = document.createElement("td");
		area.colSpan = fields.length + 1;
	
		// TODO: change from framework to dax
		var doc = frames['framework'].document;
		tablebody.recordInput = getElementByClassName(doc, "recordInput").cloneNode(true);

		// fill out the labels
		for (var i = 0; i < fields.length; i++) {
			var fieldid = fields.item(i).getAttribute("id");
			var label = getElementByClassName(tablebody.recordInput, "fieldLabel");
			var fieldArea = getElementByClassName(tablebody.recordInput, "fieldArea")
			var fieldtype = structure.getField(tableid, fieldid).fieldtype;
			
			if (i > 0) {
				label = label.cloneNode(true);
				fieldArea = fieldArea.cloneNode(true);
			} else {
				var containerRow = fieldArea.parentNode;
				var parentTable = containerRow.parentNode;
				var saveRow = containerRow.nextSibling;
			}
			
			// now get the label
			var fieldName = fields.item(i).getAttribute("name");
			label.innerHTML = fieldName;
			
			// find the input so we can manipulate it
			var input = fieldArea.getElementsByTagName("input").item(0);
			
			switch (fieldtype) {
				case 'picture':
					var img = document.createElement("img");
					img.style.width = "125px";
					img.style.height = "125px";
					img.style.borderWidth = "0px";

					var link = document.createElement("a");
					link.target = "_blank";
					link.appendChild(img);
										
					fieldArea.insertBefore(link, input);
					
					input.img = img;
					input.href = link;
					input.style.display = "none";
				default:			
					input.name = fieldName;
					input.fieldid = fieldid;
			}
			
			// switch row if necessary
			if (i > 0 && (i % 2 == 0)) {
				containerRow = containerRow.cloneNode(false);
				parentTable.insertBefore(containerRow, saveRow);
			}
			
			// add the new field area and label
			containerRow.appendChild(label);
			containerRow.appendChild(fieldArea);
		}
		
		area.appendChild(tablebody.recordInput);
		tablebody.expander.appendChild(area);
	}
	
	return tablebody;
}

function populateRelated(http_request, ids, values, propertyNames)
{
	// do a query
	if (http_request.readyState != 4)
		return;
	
	var result = http_request.responseXML;
	if (!result)
		return;
	
	var related = result.getElementsByTagName("related").item(0);
	var fields = related.getElementsByTagName("relatedfield");
	
	for (var i = 0; i < fields.length; i++) {
		var fieldname = fields[i].getAttribute("fieldname");
		var tablename = fields[i].getAttribute("tablename");
		var qualField = "[" + tablename + "]" + fieldname;
	
		var field = fields.item(i).getElementsByTagName("field").item(0);
		var table = fields.item(i).getElementsByTagName("table").item(0);
		
		var fieldid = field.getAttribute("fieldid");
		var tableid = table.getAttribute("tableid");
		
		for (var k = 0; k < ids.length; k++) {
			// check if we need to do a query and populate
			if (qualField.toLowerCase() == ids[k].toLowerCase()) {
				// we have a matching field
				var hr = getXMLHttpRequest();
				var sessionId = connection.sessionId;
					
				if (hr) {
					var url = connection.address.query + "?sessionId=" + sessionId + "&tableid=" + tableid + "&fieldid=" + fieldid + "&comp=equal&value=" + values[k] + "&start=1&length=1";
					
					hr.onreadystatechange = 
						function() {
							populateElementsByIds(hr, ids, propertyNames);
						};
						
					try {
						hr.open("GET", url, true);
				  	  	hr.send(null);
				   	} catch(e) {
				   		delete hr;
					}
				}	
			}
		}
	}
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
}

/**
 * generic record class.
 **/
function Record(tableid, recordid)
{
	this.tableid = tableid;
	this.recordid = recordid;
}

// Rodrigo
// takes % and $ out of the string, the % sign is interepreted as a value in 4D
function cleanUp( values, format ){
	if(format.substr(0,1) != 'b' && format.substr(0,1) != 'r' && format.substr(0,1) != 'd'){
		var regEx = /[%EUR\s+]/g
		values = values.toString();
/*		
		if(isNaN(values.substr(0,1)) && !isNaN(values.substr(1,1)) && values.substr(0,1) != '-' && format.substr(0,1) != 'i' && format.substr(0,1) != 'l' ){
			var newValue = values.substr(1,values.length);
			return newValue;
		}
		else if(values.substr(0,1) == ',' && format.substr(0,1) == 'i' && format.substr(0,1) == 'l' )
			values = values.substr(0,values.indexOf(','));
		else if(values.indexOf(',') != -1 && (format.substr(0,1) == 'i' || format.substr(0,1) == 'l') )
			values = values.substr(0,values.lastIndexOf(','));						
		else
			values = values;
	*/		
			while (regEx.test(values)){ 
				values = values.replace(regEx,'');
			}
			if(!isNaN(values))
				values = Math.round(values);
			
		return values;
	}
	else 
		return values;
}

// check and fix the sizes of the time array "00:00:00"
function checkSize(valueSize){
	if(valueSize.length < 2)
		valueSize = '0' + valueSize;
	else if(valueSize.length > 2)
		valueSize = valueSize.substr(0,2);
	else
		valueSize = valueSize;
	return valueSize;
}

function checkDateFormat(data, formatting) {
	var newDate = "";

	var newDateArray = data.split('-');
	if(formatting == 'MM-DD-YYYY'){
		if(newDateArray[0].length == 4)
			newDate = newDateArray[0] + '-' + newDateArray[1] + '-' + newDateArray[2];
		else
			newDate = newDateArray[2] + '-' + newDateArray[0] + '-' + newDateArray[1];
	}
	else if(formatting == 'DD-MM-YYYY' || formatting == 'DD-MMM-YYYY'){
		if(newDateArray[0].length == 4)
			newDate = newDateArray[0] + '-' + newDateArray[1] + '-' + newDateArray[2];
		else
			newDate = newDateArray[2] + '-' + newDateArray[1] + '-' + newDateArray[0];
	}
	else
		newDate = data;
	return newDate;
}

function fixValueSize( stringValue ){
	if(stringValue){
		var newStringValue = stringValue.split('-');
	
		if(newStringValue[0].length < 2)
			newStringValue[0] = '0' + newStringValue[0];
		if(newStringValue[1].length < 2)
			newStringValue[1] = '0' + newStringValue[1];
		 return newStringValue[0] + '-' + newStringValue[1] + '-' + newStringValue[2];
	}
}


function keepDetailDate( value ){
	var newValue = value.split('-');
	if(newValue[0].length < 2)
		newValue[0] = '0' + newValue[0];
	switch(newValue[1]){
		case 'Jan':
			newValue[1] = '01';
		break;
		case 'Feb':
			newValue[1] = '02';
		break;		
		case 'Mar':
			newValue[1] = '03';
		break;		
		case 'Apr':
			newValue[1] = '04';
		break;	
		case 'May':
			newValue[1] = '05';
		break;
		case 'Jun':
			newValue[1] = '06';
		break;	
			case 'Jul':
			newValue[1] = '07';
		break;
		case 'Aug':
			newValue[1] = '08';
		break;		
		case 'Sep':
			newValue[1] = '09';
		break;		
		case 'Oct':
			newValue[1] = '10';
		break;	
		case 'Nov':
			newValue[1] = '11';
		break;
		case 'Dec':
			newValue[1] = '12';
		break;		
	}
	if(newValue[0].length == 4)
		return newValue[0] + '-' + newValue[1] + '-' + newValue[2];
	else
		return newValue[2] + '-' + newValue[1] + '-' + newValue[0];
}

function dateValidation( data, formatting ){
	var newDateValue = data.split('-')
	var dateFlag = "True";
	testDate = new Date(newDateValue[0], newDateValue[1]-1, newDateValue[2]);
		
	if(data != "0000-00-00" || data != "00-00-00"){
		if (newDateValue[0] > 1900 && newDateValue[0] < 2100 && newDateValue[1] > 0 && newDateValue[1] < 13 && newDateValue[2] > 0 && newDateValue[2] < 32){
			if(newDateValue[1] == 1 || newDateValue[1] == 3|| newDateValue[1] == 5 || newDateValue[1] == 7||newDateValue[1] ==8 || newDateValue[1] == 10 || newDateValue[1] == 12){
				if(newDateValue[2] > 31)	
					dateFlag = 'False';
				else
					dateFlag = 'True';
			}
			else if(newDateValue[1] == 4 || newDateValue[1] == 6 || newDateValue[1] == 9 || newDateValue[1] == 11 ){
				if(newDateValue[2] > 30)	
					dateFlag = 'False';
				else
					dateFlag = 'True';
			}
			else if(newDateValue[1] == 2){
				if(newDateValue[0] % 4 == 0 && newDateValue[2] <= 29)
					dateFlag = 'True';
				else if(newDateValue[0] % 4 != 0 && newDateValue[2] > 28)
					dateFlag = 'False';
				else
					dateFlag = 'True';
			}
		}
	}
	else
		dateFlag = 'False';
	
	return dateFlag;
}

function valueOk(strValue){
	var regEx = /[,.]/g;
	strValue = strValue.replace(regEx,"");
	return strValue;
}

function clearSpaces(string) {
	var tstring = "";
	string = '' + string;
	splitstring = string.split(" ");
	for(i = 0; i < splitstring.length; i++)
	tstring += splitstring[i];
	return tstring;
}

function RealIntLongInt(formatType, types, valuee){
	
	var spliter = "";
	var decSpliter = "";
	var newFormat = types.split(',');				
	var newType = types;
	var lookupType = types;
	var types = "";
	var negativeSign = "";
	var decimalSring = "";
	var NoDecimalArray = ["l","i"];
	var errorFlag = 0;
	var nStr = "";		
	var valuee = valuee.toString();
	
	//make sure to not take the field type as a prefix
	if(newFormat[0].length < 1)
		newFormat[0] = "";
	
	// assign the separators and decimal places	
	if(lookupType !=  1){
		if(newFormat[1].length){
			if(newFormat[1] == "cm")
				spliter = ",";
			else if(newFormat[1] == "per")
				spliter = ".";
			else
				spliter = newFormat[1];
		}	
			
		if(newFormat[2] == "cm")
			decSpliter = ",";
		else if(newFormat[2] == "per")
			decSpliter = ".";
		else
			decSpliter = newFormat[2];						
				
		var decimalPoint = valuee.indexOf(decSpliter);
		if(isNaN(valuee.substr(1,1)) && valuee.substr(1,1) != '.' && valuee.substr(1,1) != ',')
			valuee = valuee.replace(valuee.substr(1,1),"");
	
		// replace and prefix or postfix with nothig		
		valuee = valuee.replace(newFormat[0],"");
		valuee = valuee.replace(newFormat[4],"");
	
		if(spliter != '-'){
			if(valuee.indexOf('-') != -1){
				negativeSign = "-";
				var rgx = /-/g;
				valuee = valuee.replace(rgx,'');
			}
		}
		
		for(im=0; im<= NoDecimalArray.length; im++){	
			if(formatType == NoDecimalArray[im])
				errorFlag = 1
		}
		if(!errorFlag){ // clean string without commas or decimal points to send to 4D
			if(decSpliter == "," && formatType != 'r'){
				while (valuee.indexOf(spliter) != -1) {
					valuee = valuee.replace(spliter, "");
				}
			}
			else if(decSpliter == "." && spliter.length){
				while (valuee.indexOf(spliter) != -1) {
					valuee = valuee.replace(spliter, "");
				}
			}
		}
		
		if(formatType == 'i' || formatType	== 'l'){
			if(valuee.substr(0,1) == '.'){
				valuee = '0.' + valuee.substr(valuee.indexOf('.')+1,valuee.length);
			}
			else if(valuee.indexOf(',') != -1){
				var regEx = /,/g;
				valuee = valuee.replace(regEx,'');
			}
			else
				valuee = valuee;
		}
				
		nStr = valuee;
		nStr += '';		
		nStr = nStr.replace(spliter,"");							
		x = nStr.split(decSpliter);
	
		x1 = x[0];
		x2 = x.length > 1 ? '.' + x[1] : '';
		var rgx = /(\d+)(\d{3})/;
		
		if(spliter){
			while (rgx.test(x1)) { // break string with thousand separators
				x1 = x1.replace(rgx, '$1' + spliter + '$2');
			}
		}
		if(x2.length)
			x2 = x2.replace('.',decSpliter);
			
		types = types + x1 + x2;
		
		//define the decimal places in the string
		
		if(newFormat[3].length && newFormat[3] != 0 && formatType != 'i' && formatType	!= 'l'){
			if (decimalPoint == -1)
				decimalString = '';
			else
				decimalString = valuee.substr(valuee.indexOf(decSpliter), valuee.length);
			
			var cleanDecimal = decimalString;//clean string
			myRegularExp =/(\%)|(\.)|(,)|(EUR)/;
			
			if(cleanDecimal.length){
				myDecimalPoint = myRegularExp.exec(cleanDecimal);
				if(myDecimalPoint[0])
					cleanDecimal = cleanDecimal.replace(myDecimalPoint[0],"");
			}
			
			decimalString = cleanDecimal;	
			// if there are too many decimal places, remove extras	
			if( decimalString.length > newFormat[3] && errorFlag != 1){
				decimalString = decimalString.substr(0,newFormat[3]);
				if( spliter == decSpliter ){
					var newSpliter = types.split(spliter);
					types = types.substr(0, types.lastIndexOf(spliter) )+ decSpliter + decimalString;
				}
				else{
					types = types.substr(0, types.lastIndexOf(decSpliter) );
					if(!decimalString){
						if(types.lastIndexOf(decSpliter)+1 == ''){
							var decimalString = valuee.substr(valuee.lastIndexOf(decSpliter)+1, valuee.length);
							types = types + decSpliter + decimalString;
						}	
					}
					else
						types = types + decSpliter + decimalString;
					//debugAlert('Decimal String = ' + decimalString );
					//debugAlert('Types = ' + types);
				}
			}
				
			// if there are too litle, add zeroes	
			else if (decimalString.length < newFormat[3] && errorFlag != 1) { 
				if(decimalString.length != 0 || decimalString != "")
					var decimalShort = newFormat[3] - decimalString.length;
				else
					var decimalShort = newFormat[3];
				
				// add the missing 0 to match the decimal places defined on the formatting
				for(z = 1; z <= decimalShort; z++){
					decimalString += '0';
				}
				//if(formatType != 'i' && formatType != 'l')
					types = x1 + decSpliter + decimalString;
			//	else
				//	types = x1;
			} 
			else if(decimalString.length == newFormat[3] && errorFlag != 1)
				if(newFormat[3] != 0)
					types = x1 + decSpliter + decimalString;
				else
					types = x1 + decimalString;
			else
				types = types;
		}	
		
		// puts the "%,EUR or whatever the last value of the array was defined
		if(newFormat[4].length){
			if(newFormat[4] == "~"){
				newFor = "%";		
				var objRegExp = /,/g;
				if(types.indexOf(',') != types.lastIndexOf(','))
					types = types.replace(objRegExp,'');
					
				types = clearSpaces(types);	
				types = types + ' ' + newFor;
			}
			else if(newFormat[4] == 'EUR'){
				types = types + " " + newFormat[4];
			}
		}	
		else{
			types = types + " " + newFormat[4];
		}	
		
		if(newFormat[0].length){
			types = types.replace(newFormat[0],"");
			types = negativeSign + newFormat[0] + types;
		}
	else
		types = negativeSign + types;
	}
	
//	if(!types && types != 0)

	if(!types)
		types = valuee;
	
	return types;	
}
			
// define a default value	
function format(type, valuee) {
	if(!valuee && valuee != 0){
		valuee = "00-00-00";
		return valuee;
	}
	if(!type)
		return valuee;
	
	valuee = valuee.toString();
	var formatType = type.substring(0,1);
	
	//gets the code for each field property ( long integer, real, date, etc, tha comes from admin
	// identify the formating and strips off the format indicator
	var type = type.slice(1);
	
	var newType = "";
	switch(formatType){
					
		// formatting for date
		case 'd':
			if (type.indexOf('1') != -1){ // if formatting is NONE, gets the first formatting option after None, if there is no formatting option, forces the 4D formatting.
				if(STR['admin_values_date'][1])
					type = STR['admin_values_date'][1].slice(1);
				else
					type = "MM-DD--YYYY";
			}
			else
				type = type;
			
			var arrD = valuee.replace("/","-");
			//var newData = checkDateFormat(valuee, type);
			goodData = dateValidation(valuee);
				
			arrD = valuee.split('-');
			dates = new Date(arrD[0],(arrD[1] - 1),arrD[2]);		
			if(valuee == '0000-00-00'){
				valuee = '00-00-00';
				return valuee; // returns the default value if it is empty or null
			}
			
			// replace year
			if (type.indexOf('YYYY') != -1) {
				type = type.replace('YYYY',arrD[0]);
				
			} else if (valuee.indexOf('YY') != -1) {
				// replace YY with year
				type = type.replace('YY',arrD[0]);
			}
			
			// replace month
			if(type.indexOf('MMM') != -1) {	
				if( arrD[1] != "00" ){
					var newMonth = STR['calendar_month_abr'][parseInt(dates.getMonth())]; // gets the month name from resources.js file
					type = type.replace('MMM', newMonth);
				}
				else{
					valuee = arrD[1] + "-" + arrD[0] + "-" + arrD[2]; // returns value in 4D formatting
					return valuee;
				}
			} 
			else if (type.indexOf('MM') != -1) {
			
				type = type.replace('MM',arrD[1]);
			}
			// replace day
			if(type.indexOf('DD') != -1) {
				type = type.replace('DD', arrD[2]);
			}
			return type;
		
		break;	
		
		// formatting for time
		case 't':
			if (type.indexOf('1') != -1){ // if formatting is NONE, gets the first formatting option after None, if there is no formatting option, forces the 4D formatting.
				if(STR['admin_values_time'][1])
					type = STR['admin_values_time'][1].slice(1);
				else
					type = "HH:MM:SS";
			}
			else
				type = type;
			if(valuee.indexOf(':') == -1)
				valuee = valuee + ':00:00';
				
			var arrT = valuee.split(':');
				
			if(arrT){	
				//check to make sure the values passed are valid in size, if not format with the right amount of numbers
				if(arrT[0]){
					if(arrT[0].length < 2)
						arrT[0] = checkSize(arrT[0]);
				}
				if(arrT[1]){
					if(arrT[1].length < 2 || arrT[1].length > 2)
						arrT[1] = checkSize(arrT[1]);
				}
				if(arrT[2]){
					if(arrT[2].length < 2 || arrT[2].length > 2)
						arrT[2] = checkSize(arrT[2]);
				}
			}
			var newType = type;
			//debugAlert('Type = : ' + type + 'Valuee = ' + valuee);
			if(type == "HH:MM:SS am/pm" && arrT[0] < 25){
				if(valuee == "00:00:00" && type.indexOf('am/pm') != -1)
					valuee = "12:00:00 am";
				else if(valuee == "24:00:00"  && type.indexOf('am/pm') != -1)
					valuee = "12:00:00 am";										
				else if(valuee == "12:00:00"  && type.indexOf('am/pm') != -1)
					valuee = "12:00:00 pm";
					
			}
			else if(type == "HH:MM am/pm" && arrT[0] < 25){
				if(valuee == "00:00" && type.indexOf('am/pm') != -1)
					valuee = "12:00 am";
				else if(valuee == "24:00" && type.indexOf('am/pm') != -1)
					valuee = "12:00 am";
				else if(valuee == "12:00" && type.indexOf('am/pm') != -1)
					valuee = "12:00 pm";
					
			}
			
			// make sure the array has 3 elements HH:MM:SS
			if(arrT.length < 3){
				if(type == "MM:SS")
					arrN = "00:" + arrT[0] + ":" + arrT[1];
				else
					if(arrT[1])
						arrN = arrT[0] + ":" + arrT[1] + ":00";
					else
						arrN = arrT[0] + ":00:00";
				arrT = arrN.split(':');
			}
				
			// replaces the HH for hours
			if (type.indexOf('HH') != -1) {
				if(arrT[0]){
					if(arrT[0] > 24 && type.indexOf('am/pm') != -1){
						var newHour =  (arrT[0] % 24); 
						arrT[0] = newHour;
					}
					else
						var newHour = arrT[0];
					
					if (type.indexOf('am/pm') != -1) {
					
						//if hour is greatr than 12 puts the am/pm

						if (arrT[0] > 12){
							arrT[0] = arrT[0] - 12;
							type = type.replace('HH',arrT[0]);
						}
						else if(arrT[0] == 0)
							type = type.replace('HH', '12');
						else
							type = type.replace('HH',arrT[0]);
					}
					else{
						if (arrT[0] > 12){
							if (type.indexOf('am/pm') != -1) {
								arrT[0] = arrT[0] - 12;
							} else {
								arrT[0] = arrT[0];
								type = type.replace("HH",arrT[0]);
							}
						}
						else{
							if (type.indexOf('am/pm') != -1) {
								var values = arrT[0];
							} else {
								arrT[0] = arrT[0];
								type = type.replace("HH",arrT[0]);
							}
						}
					}
				}
				else 
					type = type.replace('HH',"00");			
			} 
				
			if (type.indexOf('MM') != -1) {
				// replace MM with minutes
				if(arrT[1]){
					if(arrT[1].length < 2)
						arrT[1] = arrT[1] + "0";
					type = type.replace('MM',arrT[1]);
				}
				else{
					if(arrT[0])
						type = type.replace('MM',arrT[0]);
					else
						type = type.replace('MM',"00");
				}
			}
			if (type.indexOf('SS') != -1) {
				// replace SS with seconds
				if(arrT[2]){
					if(arrT[2].length < 2)
						arrT[2] =  arrT[2] + "0"; 
				type = type.replace('SS',arrT[2].substr(0,2));
				}
				else{
					if(arrT[1])
						type = type.replace('SS',arrT[1]);
					else
						type = type.replace('SS',"00");
				}
			}	
			
			// clear the am/pm if any and appends am or pm when defined in admin
			myRegularExpression =/( am\/pm)|( am )| ( pm)/;
			if(type.length){
				myAMPM = myRegularExpression.exec(type);
				if ((myAMPM) && (myAMPM[0]))
					type = type.replace(myAMPM[0],"");
			}
			// add the am/pm if specified on the admin
			//if(arrT[0] !="00"){
				if (newType.indexOf('am/pm') != -1){
					if(newHour >= 12 && newHour < 24)
						type = type + " pm";
					else
						type = type + " am";
				}
			//}
			//else
				//type = type;
			
			typeSize = type.substr(0,type.indexOf(':'));
			if( typeSize.length < 2 )
				type = "0" + type;
					
			return type;
		
		break;
		
		// formatting for long integer, integer, real
		case 'l' :
		case 'i' :
		case 'r' :
			type = RealIntLongInt(formatType, type, valuee);
			return type;
		break;
		
		// apha numeric formatting
		case 'a':
		
			if (type.indexOf('1') != -1)
				return valuee;
			
			// converts string to lowercase
			else if(type.indexOf("uuuu") != -1){
				valuee = valuee.toLowerCase();
				return valuee;
			}
			
			// convert string to UPPERCASE
			else if(type.indexOf("UUUU") != -1){
				valuee = valuee.toUpperCase();
				return valuee;
			}
			else
				return valuee;
			break;
		
		// text formatting
		case 'x':
			if (type.indexOf('1') != -1)
				return valuee;		
			
			// converts string to lowercase			
			else if(type.indexOf("uuuu") != -1){
				valuee = valuee.toLowerCase();
				return valuee;
			}
			
			// convert string to UPPERCASE
			else if(type.indexOf("UUUU") != -1){
				valuee = valuee.toUpperCase();
				return valuee;
			}
			else
				return valuee;
			break;
		
		// boolean formatting
		case 'b':
		
			if (type.indexOf('1') != -1)
				type = "T";

			if(type.indexOf('T') != -1){	
				if(valuee.toLowerCase() == 't' || valuee.toLowerCase() == 'true')
					valuee = STR['ttrue'];
				else
					valuee = STR['ffalse'];
			}
			else if(type.indexOf('Y') != -1){
				if(valuee.substr(0,1).toLowerCase() == 'y' || valuee.toLowerCase() == 'yes' || valuee.toLowerCase() == 'true')
					valuee = STR['yes'];
				else
					valuee = STR['no'];
			}
			else if(type.indexOf('F') != -1){	
				if(valuee.substr(0,3).toLowerCase() == 'm' || valuee.toLowerCase() == 'female' || valuee.toLowerCase() == 'false')
					valuee = STR['female'];
				else
					valuee = STR['male'];
			}
			else
				return valuee;
			
			return valuee;
			break;

		default: 
			return valuee;
	}	
}

function Structure() {
	this.tables = new Array();
	this.updating = false;
	this.cached = false;
	
	// retrieve a table
	this.getTable =
		function(id)
		{
			var tp = this;
			for (var i in tp.tables) {
				if ((tp.tables[i].tableid == id) || (tp.tables[i].tablename == id))
					return tp.tables[i];
			}
			return null;
		};
	
	this.getField =
		function(tableid, fieldid)
		{
			var table = this.getTable(tableid);
			for (var i in table.fields) {
				if (table.fields[i].fieldid == fieldid)
					return table.fields[i];
			}
			return null;
		};
	
	this.update =
		function()
		{
			if (connection.sessionId == "" || this.updating)
				return;
			
			if (this.cached) {
				return;
			}
			
			this.updating = true;
			
			var url = connection.address.gettablelist+"?sessionId="+connection.sessionId;
			makeRequest(url, buildTableList, null);
		};
	
	this.isUpdated =
		function()
		{
			if (this.updating)
				return false;
				
			// check each field table to see if any are still updating
			for (var i = 0; i < this.tables.length; i++) {
				if (this.tables[i].updating) {
					return false;
				}
			}
			
			return true;
		};
	
	return this;
}

var structure = new Structure();

// Database Backend commands
var DB = {
	SORT 			: "Sort",
	GET_RECORD 		: "GetRecord",
	MODIFY_RECORD	: "ModifyRecord",
	ADD_RECORD		: "AddRecord",
	DELETE_RECORD	: "DeleteRecord"
};