//
// CalendarObject 
// author: Rudolf Psenicnik
//
// feel free to pass null for all parameters
// kind, actionTarget, and parent parameters are used with picker object
// columnWidth is a temporary solution for the grid that should size itself automatically

function CalendarObject (columnWidth, kind, actionTarget, parent) {

	var node = getTemplate("dax_calendar");
	if (!node)
		return null;
	
	this.dom 					= node.cloneNode(true);
	this.header 		 		= getElementByClassName(this.dom, "dax_calendar_header");
	this.header_text			= getElementByClassName(this.dom, "dax_calendar_header_text");
	this.HeaderLeft		 		= getElementByClassName(this.dom, "dax_calendar_header_left");
	this.HeaderRight			= getElementByClassName(this.dom, "dax_calendar_header_right");
	this.HeaderLeftYear	 		= getElementByClassName(this.dom, "dax_calendar_header_left_year");
	this.HeaderRightYear		= getElementByClassName(this.dom, "dax_calendar_header_right_year");
	this.parentWindow			= null; // parent window, used for resizing
	
	setStyle (this.dom, 'display', 'block');
	
	// use this until framework starts supporting auto-resizing grid

	this.toString = function()
	{
		return "calendar";
	};
	
	if (columnWidth != null) {
		if (columnWidth > 0) {
			this.columnWidth = columnWidth;
		} else {
			this.columnWidth = 50;
		}
	}

	// week starts on Mon or Sun
	this.weekStart = 'Sun';
	
	// set week start day
	this.setWeekStart = function (weekStart) {
		if (weekStart == 'Mon')
			this.weekStart = 'Mon';
		else
			this.weekStart = 'Sun';
	}
	
	if (kind == 'picker') {	
		this.kind = kind;
		this.actionTarget = actionTarget;
	} else {
		this.kind = '';
		this.actionTarget = '';	
	}

	if (parent != null) {
		this.parent = parent;
	}
		
	// check if date already exists, if not create one
	// default date, month, and year are current date
	var today = new Date();
	
	if (this.date == null) {
		var today = new Date();
		this.date = today.getDate();
		this.month = (today.getMonth() + 1);
		this.year = today.getFullYear();	
	}

	// gets today's date
	this.getToday = function () {
		var today = new Date();
		var date = today.getDate();
		var month = (today.getMonth() + 1);
		var year = today.getFullYear();	
		return this.getDate (null, year, month, date);
	}
	
	// calendar id
	// used when assigning id to dom objects
	// such as selected date
	
	i = 1;
	while ($('dax_calendar' + i) != null) {
		i++;
	}
	
	this.idNumber = i;
	this.dom.id = 'dax_calendar' + this.idNumber;

	var myref = this.dom;
	this.dom.onclick = function (event) {
		var event = xpEvent (event, true);
		Controller.setFocused (myref);
		return false;
	}

	this.onkeyup = function (event) {
		var event = xpEvent (event, true);
			if (event.keyCode == 39) {
					// arrow right
					this.nextMonth();
					this.drawCalendar();
			} else if (event.keyCode == 37) {
				// arrow left
					this.prevMonth();
					this.drawCalendar();
			}		
	}

	// PHASE OUT by localization
	// sets the language
	// lang = {en, fr, de, es}
	this.setLanguage = function (lang) {
		if ((lang == 'en') || (lang='fr') ||
			(lang == 'es') || (lang='de')) {
			this.language = lang;
		}
	}

	// returns the name of the language
	this.getLanguage = function () {
		return this.language;
	}

	// sets the date
	this.setDate = function(date) {
		brokenDate = date.split("-");
		if (brokenDate.length == 3) {
			year = brokenDate[0];
			month = brokenDate[1];
			date = brokenDate[2];
			// check date validity
			testDate = new Date(year, month-1, date);
			if ((testDate.getFullYear() == year) && 
				(testDate.getMonth() == month-1)) {
//				(testDate.getDate() == date)) {
//				it's ok to have date 00
//				no date is selected in that case
				this.year = year - 0;
				this.month = month - 0;
				this.date = date - 0;
				this.drawCalendar();
			}
		}
	}
	
	// gets the date
	this.getDate = function(format, year, month, date) {
		
		if (!year)
			var year = this.year;
		if (!month)
			var month = this.month;
		if (!date)
			var date = this.date;
			
		if (date < 10) {
			date = '0' + '' + parseFloat(date);
		}

		if (month < 10) {
			month = '0' + '' + parseFloat(month);
		}
		
		if (!format) {
			var date = year + '-' +month + '-' + date;
		} else if (format == 'mm/dd/yyyy') {
			var date = month + '/' +date + '/' + year;
		}
		return date;
	
	}

	this.value = this.getDate();

	// goes to the previous year
	this.prevYear = function () {
		this.year = parseFloat(this.year) - 1;
		this.date = 0;
		this.drawCalendar();
	}

	// goes to the next year
	this.nextYear = function () {
		this.year = parseFloat(this.year) + 1;
		this.date = 0;
		this.drawCalendar();
	}
	
	// goes to the previous month
	this.prevMonth = function () {
		if (this.month == 1) {
			this.month = 12;
			this.year = parseFloat(this.year) - 1;
		} else {
			this.month = parseFloat(this.month) - 1;
		}
		this.date = 0;
		this.drawCalendar();
	}

	// goes to the next month
	this.nextMonth = function () {
		if (this.month == 12) {
			this.month = 1;
			this.year = parseFloat(this.year) + 1;
		} else {
			this.month = parseFloat(this.month) + 1;
		}
		this.date = 0;
		this.drawCalendar();
	}

	// default language
	this.language = 'en';
		
	// resize grid based on parent window size
	this.refresh = function () {
		// check if parent window is valid
		if (!this.parentWindow)
			return false;
		// get parent window size
		var parentWidth		= this.parentWindow.style.width;
		var parentHeight	= this.parentWindow.style.height;
		// calculate cell size
		var cellWidth		= (parseFloat(parentWidth) / 7); 
		// height is parent window minus header height minus grid header height 
		var parentHeight	= (parseFloat(parentHeight) - parseFloat(this.header.offsetHeight) - parseFloat(this.calGridObj.grid.column(0).prv.mHeader.prv.mDOM.offsetHeight));
		var cellHeight		= (parseFloat(parentHeight) / 6);
		// set grid cell width
		this.columnWidth	= (cellWidth - 1);
		this.cellHeight		= (cellHeight - 1);
		//this.drawCalendar();
		// resize cells
		for (i=0; i < this.calGridObj.grid.prv.mRows.length; i++) {
			// width
			this.calGridObj.grid.setColumnWidth(parseInt(this.columnWidth));
			for (j=0; j < this.calGridObj.grid.prv.mRows[i].prv.mElements.length; j++) {
				cell = this.calGridObj.grid.prv.mRows[i].prv.mElements[j].prv.mDOM;
				// height
				setStyle (cell, 'height', parseInt(this.cellHeight) + 'px');
			}
		}
		this.calGridObj.grid.refresh();
	}
	
	// add custom post-refresh method here
	this.afterRefresh = function () {
	
	}
	
	// draw calendar function
	this.drawCalendar = function() {
		
		if (this.calGridObj == null) {
			this.calGridObj = new GridObject (this.dom);
		}
		
		// initialize month and day names
		//var monthNames = initMonths (this.language);
		//var dayNames = initDaysOfWeek (this.language);
	
		// create header
	
		var HeaderText = document.createTextNode(STR['calendar_month'][this.month-1] + ' ' + this.year);
		
		while (this.header_text.hasChildNodes()) {
			this.header_text.removeChild(this.header_text.firstChild);
		}	
		
		this.header_text.appendChild (HeaderText);
		
		// sets the parent for change month/year buttons
		this.HeaderLeft.parent = this;
		this.HeaderRight.parent = this;
		this.HeaderLeftYear.parent = this;
		this.HeaderRightYear.parent = this;

		// set classes
		this.HeaderLeft.className = 'dax_calendar_header_left dax_calendar_header_left_normal';
		this.HeaderRight.className = 'dax_calendar_header_right dax_calendar_header_right_normal';
		this.HeaderLeftYear.className = 'dax_calendar_header_left dax_calendar_header_left_year_normal';
		this.HeaderRightYear.className = 'dax_calendar_header_right dax_calendar_header_right_year_normal';
		
		
		// change month button action
		this.HeaderLeft.onclick = function(aEvent){
			xpEvent(aEvent, true);
			this.parent.prevMonth();
			return false;
		};

		// mouse events for header left
		this.HeaderLeft.onmouseover = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_over';
		}
		this.HeaderLeft.onmouseout = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_normal';
		}
		this.HeaderLeft.onmousedown = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_down';
		}
		this.HeaderLeft.onmouseup = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_over';
		}

		
		this.HeaderRight.onclick = function(aEvent){
			xpEvent(aEvent, true);
			this.parent.nextMonth();
			return false;
		};

		// mouse events for header right
		this.HeaderRight.onmouseover = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_over';
		}
		this.HeaderRight.onmouseout = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_normal';
		}
		this.HeaderRight.onmousedown = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_down';
		}
		this.HeaderRight.onmouseup = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_over';
		}
		
		
		// change year button action
		this.HeaderLeftYear.onclick = function(aEvent){
			xpEvent(aEvent, true);
			this.parent.prevYear();
			return false;
		};

		// mouse events for header left
		this.HeaderLeftYear.onmouseover = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_year_over';
		}
		this.HeaderLeftYear.onmouseout = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_year_normal';
		}
		this.HeaderLeftYear.onmousedown = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_year_down';
		}
		this.HeaderLeftYear.onmouseup = function () {
			this.className = 'dax_calendar_header_left dax_calendar_header_left_year_over';
		}	
		
		this.HeaderRightYear.onclick = function(aEvent){
			xpEvent(aEvent, true);
			this.parent.nextYear();
			return false;
		};

		// mouse events for header right
		this.HeaderRightYear.onmouseover = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_year_over';
		}
		this.HeaderRightYear.onmouseout = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_year_normal';
		}
		this.HeaderRightYear.onmousedown = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_year_down';
		}
		this.HeaderRightYear.onmouseup = function () {
			this.className = 'dax_calendar_header_right dax_calendar_header_right_year_over';
		}
		
		var calGrid = this.calGridObj.grid;

		// update minimum width, as it is probably above what calendar needs
		for (i = 0; i < 7; i++) {
			calGrid.column(i).prv.mMinimumWidth = 25;
		}
		// set column width
		calGrid.setColumnWidth(this.columnWidth);
		
		var calViewport = this.calGridObj.viewport;	
		this.viewport = calViewport;
		// viewport that holds calendar should auto-resize
		setStyle (this.viewport.dom, 'width', '100%');		
		
		// customize viewport
		calViewport.vscrollbar.hide();
		calViewport.hscrollbar.hide();
		calViewport.setOverflow(0,0);
	
		// get date array for current month
		var dateArray = new Array;
		dateArray = buildCalendarArray (this.month, this.year);

		if (this.weekStart == 'Mon') {
			var buffer = dateArray.shift();
			if (parseInt(buffer) == 1) {
				var dummy = dateArray.unshift (' ', ' ', ' ', ' ', ' ', ' ', '01'); 
				dateArray.length = 6 * 7;
			}			
		}
		
		// populate the calendar
		var addRows=new Array(6)
		for (i=0; i <6; i++) {
			addRows[i]=new Array(7);
		}
		
		var currentRow = 0;
		for (i=0; i < dateArray.length; i++) {
			addRows[currentRow][(i % 7)] = dateArray[i];
			if (((i % 7) == 6) && (i < dateArray.length)) {
				addRows[currentRow][(i % 7)] = dateArray[i];
				currentRow++;
			}
		}

		/*
		// shift elements for weeks that start on Monday
		if (this.weekStart = 'Mon') {
			for (i=0; i < addRows.length; i++) {
				for (j = 0; j < addRows[i].length; j++) {
					var buffer = addRows[i][j];
					if (buffer) {
						var dummy = addRows[i].shift();
						addRows[i].push(buffer);
						break;
					}
				}				
			}
		}
		*/
		
		// draw the grid
		this.calGridObj.load(addRows);
		calGrid.size(7,6);	
		calGrid.setColumnWidth(this.columnWidth);
		calGrid.refresh();

		
		// show days of the week
		for (i=0; i < 7 ; i++) {
			// get header element reference
			var header = calGrid.column(i).prv.mHeader;
			// set the day of the week
			if (this.weekStart == 'Sun') {
				header.setContent (STR['calendar_weekday'][i]);
			} else if (this.weekStart == 'Mon') {
				if (i == 6) {
					header.setContent (STR['calendar_weekday'][0]);
				} else {
					header.setContent (STR['calendar_weekday'][i+1]);
				}
			}
			// remove on click functions
			header.prv.mDOM.onmouseover = '';
			header.prv.mDOM.onmouseout = '';
			header.prv.mDOM.onclick = '';
			header.prv.mDOM.onmouseup = '';
			header.prv.mDOM.onmousedown = '';			
		}
	
		// assign events to rows
		
		for (i=0; i < calGrid.prv.mRows.length; i++) {
	
			row = calGrid.prv.mRows[i].prv.mDOM;
			row.onmouseover = '';
			row.onmouseout = '';
			row.onclick = '';

			for (j=0; j < calGrid.prv.mRows[i].prv.mElements.length; j++) {
		
				cell = calGrid.prv.mRows[i].prv.mElements[j].prv.mDOM;
				//calGrid.prv.mRows[i].prv.mElements[j].setHeight (this.cellHeight + 'px');
				setStyle (cell, 'height', this.cellHeight + 'px');
				cell.style.backgroundColor = "#FFFFFF";
				
				// check for white spaces, those are cells without date
				if ((cell.innerHTML != '') && (cell.innerHTML != ' ') ){
				
					cell.parent = this;
					cell.date = cell.innerHTML;
					
					// check if it is selected date, if so, color it
					if (cell.innerHTML == this.date) {
						cell.style.backgroundColor =  "#FFCC66";
						cell.id = 'dax_calendar'+this.idNumber+'_selected';						
					} else {
						cell.style.backgroundColor =  "#FFFFFF";
						cell.id = '';						
					}
				
					cell.onmouseout = function()
					{
					if (this.id == 'dax_calendar'+this.parent.idNumber+'_selected')
						this.style.backgroundColor = "#FFCC66";
					else
						this.style.backgroundColor = "#FFFFFF";
					};
					
					cell.onmouseover = function()
					{
						this.style.backgroundColor = "#DDDDDD";
					};	
			
					//check if action was set, if so add it into the function
					if (this.kind == 'picker') {

						cell.onclick = function()
						{
							selected = $('dax_calendar'+this.parent.idNumber+'_selected');
							if (selected != null) {
								selected.style.backgroundColor = '#FFFFFF';
								selected.id = '';
							}
							this.id = 'dax_calendar'+this.parent.idNumber+'_selected';
							this.style.backgroundColor = "#FFCC66";
							this.parent.date = this.innerHTML;
							this.parent.actionTarget.value = this.parent.getDate();
							this.parent.actionTarget.realDate = this.parent.getDate();
							this.parent.parent.style.display = 'none';
							this.parent.parent.style.top = '-1000px';
							this.parent.parent.style.left = '-1000px';
							// execute input node on change method
							// this is done for compatibility with dax application
							var inputNode = this.parent.parent.owner.value;
							if ((inputNode) && (inputNode.onchange))
								inputNode.onchange(true);							
							Controller.setFocused(this.parent);
							
							this.parent.parent.owner.onUserClick();
							
						};
					
					} else {
			
						cell.onclick = function()
						{
							selected = $('dax_calendar'+this.parent.idNumber+'_selected');
							if (selected != null) {
								selected.style.backgroundColor = '#FFFFFF';
								selected.id = '';
								}
							this.id = 'dax_calendar'+this.parent.idNumber+'_selected';
							this.style.backgroundColor = "#FFCC66";
							this.parent.date = this.innerHTML;
							Controller.setFocused(this.parent);
						};
					}
	
				} else {

					cell.date = '';
				
					cell.onmouseout = '';
					cell.onmouseover = '';
					cell.onclick = '';
					
				}
	
				cell.style.textAlign = 'center';
				cell.style.fontSize = '12px';
			}
		}

		// call post-refresh method
		// this is defined by external objects that use calendar
		this.afterRefresh();

		// TO BE PHASED OUT WITH RESOURCE LOCALIZATION
		function initDaysOfWeek (lang) {
		// sets the months for the selected language
		// options are en, fr, de, es
		
			if (lang == 'fr') {
				var days = new Array ('Dim','Lun','Mar','Mer','Jeu','Ven','Sam');
			} else if (lang == 'de') {
				var days = new Array ('So','Mo','Di','Mi','Do','Fr','Sa');
			} else if (lang == 'es') {
				var days = new Array ('Dom','Lun','Mar','Mi?r','Jue','Vi?r','S?b');
			} else {
				var days = new Array ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
			}
			return days;
		}

		function initMonths (lang) {
		// sets the months for the selected language
		// options are en, fr, gr, es
		
			if (lang == 'fr') {
				var months = new Array ('Janvier','F?vrier','Mars','Avril','Mai','Juin','Juillet','Ao?t','Septembre','Octobre','Novembre','D?cembre');
			} else if (lang == 'de') {
				var months = new Array ('Januar','Februar','M?rz','April','Mai','Juni','Juli','August','September','Oktober','November','Dezember');
			} else if (lang == 'es') {
				var months = new Array ('Enero','Febrero','Marzo','Abril','Mayo','Junio','Julio','Agosto','Septiembre','Octubre','Noviembre','Diciembre');
			} else {
				var months = new Array ('January','February','March','April','May','June','July','August','September','October','November','December');
			}
			return months;	
		}

		function buildCalendarArray (m, y) {
		
			var dateArray = new Array;
			
			var days=[31,0,31,30,31,30,31,31,30,31,30,31];
			
			var oD = new Date(y, m-1, 1); 
			oD.od=oD.getDay()+1; 
			
			var todaydate=new Date() 
			var scanfortoday=(y==todaydate.getFullYear() && m==todaydate.getMonth()+1)? todaydate.getDate() : 0 
			days[1]=(((oD.getFullYear()%100!=0)&&(oD.getFullYear()%4==0))||(oD.getFullYear()%400==0))?29:28;
			
			for(i=1;i<=42;i++){
				var x=((i-oD.od>=0)&&(i-oD.od<days[m-1]))? i-oD.od+1 : ' ';
				dateArray.push (x);
			}
			
			return dateArray;
			
		}
	}


// selectDate (this.date);

	function selectDate (date) {
	
		for (i=0; i < calGrid.prv.mRows.length; i++) {
			for (j=0; j < calGrid.prv.mRows[i].prv.mElements.length; j++) {
				cell = calGrid.prv.mRows[i].prv.mElements[j].prv.mDOM;
				if (cell.innerHTML == date) {
					selected = $('calendar_selected');
					if (selected != null) {
						setStyle (selected, 'background-color', '#FFFFFF');
						selected.id = '';
						}
					cell.id = 'calendar_selected';
					cell.style.backgroundColor = "#FFCC66";			
				}
			}
		}
	}
}

function PickerObject(columnWidth) {

	var node = getTemplate("dax_picker");
	var nodePicker = getTemplate("dax_picker_popup");

	if (!node)
		return null;
	
	this.dom 				= node.cloneNode(true);
	this.value	 		 	= getElementByClassName(this.dom, "dax_picker_value");
	this.button			 	= getElementByClassName(this.dom, "dax_picker_button");
	this.pickerimage	 	= getElementByClassName(this.dom, "dax_picker_button_image");
	
	if (!columnWidth)
		var columnWidth = 30;
	
	// to override framework events
	this.value.onmousedown = function (event) {
		xpEvent (event, true);
	}

	this.value.onmouseup = function (event) {
		xpEvent (event, true);
	}

	// custom function for user click
	this.onUserClick = function () { };
	
	if (!($('dax_picker_popup0'))) {
	
		this.calendarDiv	 	= nodePicker.cloneNode(true);
		document.body.appendChild (this.calendarDiv);
		this.calendarDiv.id = 'dax_picker_popup0';
		this.calendarDiv.owner = this;

		// create a calendar
		this.calendar = new CalendarObject (columnWidth, 'picker',this.value,this.calendarDiv);
		this.calendarDiv.appendChild (this.calendar.dom);
		this.calendar.parentWindow = this.calendarDiv;
		this.calendar.drawCalendar();
		this.calendarDiv.calendar = this.calendar;
		
		this.calendarDiv.style.display = 'none';
		
		this.calendar.value = this.calendar.getDate();

	} else {
	
		this.calendarDiv = $('dax_picker_popup0');
		this.calendarDiv.owner = this;
		this.calendar = this.calendarDiv.calendar;
	
	}
	
	setStyle (this.dom, 'display', 'block');

	// sets the unique id number

	i = 1;
	while ($('dax_picker' + i) != null) {
		i++;
	}
		
	this.idNumber = i;	
	this.dom.id = 'dax_picker' + this.idNumber;
	this.value.id = 'dax_picker_value' + this.idNumber;
	this.isDisabled = false;
	this.button.parent = this;
	
	this.button.onclick = function (aEvent) {
		
		if (this.parent.isDisabled)
			return false;
		
		this.parent.calendarDiv.owner = this.parent;
		this.parent.calendar.actionTarget = this.parent.value;
		if (this.parent.value.realDate != undefined)
			this.parent.setValue(this.parent.value.realDate);
		else
			this.parent.setValue(this.parent.calendar.getToday());
		
		var event = xpEvent(aEvent, true);
		var mousecoord = getMousePosition(event);

//		this.parent.calendarDiv.style.top = mousecoord.y + 'px';
//		this.parent.calendarDiv.style.left = mousecoord.x + 'px';
		
		this.parent.calendarDiv.style.position = 'absolute';
		
		setStyle (this.parent.calendarDiv, 'left', mousecoord.x + 'px');
		setStyle (this.parent.calendarDiv, 'top', mousecoord.y + 'px');
		
		this.parent.calendarDiv.style.display = 'block';
//		Controller.setFocused (this.parent.calendar);
		if (browser.ie) {
			this.parent.calendar.calGridObj.grid.setColumnWidth(this.parent.calendar.columnWidth);
			this.parent.calendar.calGridObj.grid.refresh();
			this.parent.calendar.calGridObj.viewport.dom.style.width = '100%';
			this.parent.calendarDiv.style.width = '246px';
		}
		if (browser.gecko) {
			this.parent.calendar.calGridObj.viewport.dom.style.height = '200px';
			this.parent.calendar.calGridObj.viewport.dom.style.width = '246px';
			this.parent.calendar.calGridObj.grid.size(7,6);
			this.parent.calendar.calGridObj.grid.header.style.backgroundColor = '#FFFFFF';
			this.parent.calendar.header.style.width = '231px';
		}
		if (browser.safari) {
			setStyle (this.parent.calendarDiv, 'width', 250 + 'px');
			this.parent.calendar.calGridObj.grid.setColumnWidth(this.parent.calendar.columnWidth);
			this.parent.calendar.calGridObj.grid.refresh();
			this.parent.calendar.calGridObj.viewport.dom.style.width = '100%';
			this.parent.calendar.calGridObj.viewport.dom.style.height = '200px';
			this.parent.calendar.calGridObj.viewport.dom.style.width = '246px';
			this.parent.calendar.calGridObj.grid.size(7,6);
			this.parent.calendar.calGridObj.grid.header.style.backgroundColor = '#FFFFFF';
			this.parent.calendar.header.style.width = '238px';
		}		
	
		this.parent.calendarDiv.blur = function () {
			this.style.top = '-1000px';
			this.style.left = '-1000px';
			this.style.position = 'absolute';
			this.style.display = 'none';
		}
		Controller.setFocused (this.parent.calendarDiv);
		return false;
	};

	//this.calendarDiv.id = 'dax_picker_popup' + this.idNumber;

	
	this.getValue = function (format) {
		return this.value.realDate;
	}
	
	this.setValue = function (date) {
		this.calendar.setDate(date);
	}
}

/**
 * Calendar View
 * takes a 4D table with date field and displays it as a calendar
 * user must be sucessfully logged in (see 
 *
  * @author Rudolf Psenicnik
 *
 * @param pParent		parent HTML object such as DIV where calendar view will be inserted
 *					cell sizing is based on parent
 *					if null, will create calendar as a new window
 * @param pTable		table name or id which will be displayed as a calendar
 * @param pDateField		date field name or id that places records in specific date (optional)
 * @param pDateEndField	date field used as a end date range (for events that last for several days, optional)
 * @param pDisplayField	field that is displayed in calendar view (optional)
  * */

function calendarView (pParent, pTable, pDateField, pDateEndField, pDisplayField1, pDisplayField2, pQueryField, pQueryValue)
{

	this.loaded = false;

	// load templates for calendar cell contents
	this.headerNode 		= getTemplate('calendar_view_cell_header');
	this.bodyNode			= getTemplate('calendar_view_cell_body');
	this.cellNode 			= getTemplate('calendar_view_cell_record');
	this.searchNode 		= getTemplate('calendar_view_search');
	this.extendNode 		= getTemplate('calendar_view_extend');
	this.calendarViewNode	= null; // only if window is not used
	// abort if template loading failed
	if ((!this.headerNode) || (!this.cellNode) || (!this.bodyNode) || (!this.searchNode) || (!this.extendNode))
		return false;

	this.preloaded	= false;	
		
		// check if valid table is passed
	this.table			= null;
	this.table			= getTable (pTable);
	if (!this.table)
		return false;
	dax_bridge.cacheFields (this.table); // cache field information

	this.searchTimeout	= null;
	
	// if date field is not passed, find first available date within the table and use it
	if (!pDateField) {
		for (i = 0; i < this.table.fields.length; i++) {
			if (this.table.fields[i].fieldtype == 'date') {
				var pDateField = this.table.fields[i].fieldid;
				break;
			}
		}
	}
	
	// check if valid date field is passed
	this.dateField	= null;
	this.dateField	= getField (pTable,pDateField);
	// check for existence
	if (!this.dateField)
		return false;
	// check for type
	if (this.dateField.fieldtype != 'date')
		return false;

	// check if valid end date field is passed
	this.dateEndField	= null;
	if (pDateEndField)
		this.dateEndField	= getField (pTable,pDateEndField);
	// if date end exists
	if (this.dateEndField) {
		// check for type
		if (this.dateEndField.fieldtype != 'date')
			return false;		
		// check if end field is same as start, if so abort
		if (this.dateField.fieldid == this.dateEndField.fieldid)
			this.dateEndField = null;
	}	
		
	// if display field is not passed, find first available field within the table and use it
	if (!pDisplayField1) {
		for (var i = 0; i < this.table.fields.length; i++) {
			if ((this.table.fields[i].fieldtype != 'picture') && (this.table.fields[i].fieldinlist.toLowerCase() == 'true')) {
				var pDisplayField1 = this.table.fields[i].fieldid;
				break;
			}
		}
	}		
	
	// is calendar currently downloading data?
	this.updating = false;
	
	// check if display field is passed
	this.displayField	= null;
	this.displayField	= getField (pTable,pDisplayField1);
	// check for existence
	if (!this.displayField)
		return false;
		
	this.searchField	= getField (pTable,pDisplayField1);
	this.updatingTabsFieldId = null;		
		
	// check if aux display field is passed
	this.displayAux1Field	= null;
	this.displayAux1Field	= getField (pTable,pDisplayField2);
	
	// custom query
	if ((pQueryField) && (pQueryValue)) {
		this.customQueryField = getField (pTable, pQueryField);
		if (this.customQueryField) {
			this.customQueryValue = pQueryValue;
		}	
	}
	
	// if preselected query tabs are used
	this.queryName = null;
	this.initializing = true;	
		
	// developer defined functions
	this.onRecordClick = 		function() { };
	this.onRecordDblClick = 	function() { };

	// show this if record value is blank
	this.blankValue = '---';
	this.setBlankValue = function (newValue)
	{
		if (newValue)
			this.blankValue = newValue;
	}
	
	// window title bar update
	this.setTitle = function (newTitle)
	{
		if (newTitle)
			if (this.container.titleBar)
				this.container.titleBar.innerHTML = newTitle;
	}
	
	// change the week start date
	this.setWeekStart = function (weekStart) {
		this.calendar.setWeekStart(weekStart);
		this.calendar.drawCalendar();
		this.calendar.refresh();
		this.refresh();
	}

	this.displayBullets = true;
	// show or hide bullets
	this.showBullets = function (showBullets) {
		if ((showBullets) && (showBullets == false)) {
			this.displayBullets = true;			
		} else {
			this.displayBullets = false;		
		}	
	}
	
	// starts new custom query
	this.fetchData = function (fieldId, queryValue, eraseOnQueryDone, updateSearchBox) {

		if (eraseOnQueryDone == true)
			this.eraseOnQueryDone = true;
		else
			this.eraseOnQueryDone = false;
	
		queryField = getField (this.table.tableid, fieldId);
	
		if (!((queryField) && (queryValue)))
			return false;

		this.customQueryField = queryField;
		this.customQueryValue = queryValue;
		
		if (updateSearchBox)
			if (updateSearchBox == true) {
				var searchSelect = getElementByClassName(this.search, 'fieldSearchSelect');
				var selectedIndex = null;
				var fieldId = getField (this.table.tableid, fieldId).fieldid;
				for (var ind = 0; ind < searchSelect.options.length; ind++) {
					if (searchSelect.options[ind].fieldid == fieldId) {
						selectedIndex = ind;
						break;
					}
				}
				
				if ((selectedIndex) || (selectedIndex == 0)) {
				
					searchSelect.selectedIndex = selectedIndex;
					searchSelect.onchange(); // Apply change in the search popup in calendar
				
				}
 
				this.searchBox.value = queryValue; // Assign the value from the selected header				
			}
		
		this.refresh();

	}
	
	this.eraseOnQueryDone = false;
	
	// pointer to current query object
	this.currentQuery = null;
	
	// allow selection of multiple records in calendar view
	this.selectMultipleRecords = false;
	// function to allow user to change settings
	this.selectMultiple = function (sel) {
		if ((sel) && (sel == true))
			this.selectMultipleRecords = true;
		else
			this.selectMultipleRecords = false;
	}
	
	// create calendar
	// 50 is column width, but we'll resize that automatically
	this.calendar = new CalendarObject (50, null, null, null);
	// append class name used in css styles
	this.calendar.dom.className = this.calendar.dom.className + ' calendar_view';
	//reference parent and append calendar
	//if parent is null, create a window and assign window as a parent

	if (pParent) {
		// get calendar view template
		var calendarViewTemplate = getTemplate('calendar_view_cont');
		if (!calendarViewTemplate)
			return false;
		this.calendarViewNode = calendarViewTemplate.cloneNode(true);
		// assign unique calendar view id
		i = 1;
		while ($('calendar_view_cont' + i) != null) {
			i++;
		}
		this.calendarViewNode.id = 'calendar_view_cont' + i;		

		// add container to the passed pParent element
		this.container = pParent;
		this.container.appendChild (this.calendarViewNode);
		// add calendar into the contents section of the container
		this.parent	= getElementByClassName(this.calendarViewNode, 'calendar_view_contents');
		this.parent.appendChild (this.calendar.dom);
		// match container size with the parent
		if ((!(browser.ie)) || (!(this.container.style.width)))
			setStyle (this.parent, 'width', this.container.offsetWidth + 'px');
		else
			setStyle (this.parent, 'width', this.container.style.width);
		this.displayType = 'container';
	} else {
		var aWindow = WindowManager.createNewWindow();
		// set window title to "table name - display field (date field)"

		aWindow.titleBar.innerHTML = this.table.tablealias; 
			
		// the first Tab is all records
		var tabs = new Tabs();

		if ((this.table.showallrecordstab.toLowerCase() == 'true') || ((this.table.preselectQueries.length == 0) && (this.table.preselectedDynamicQueries.length == 0))) {
		
		
			var tab = tabs.insert(STR['all']+ " " +aWindow.titleBar.innerHTML);
			tab.owner = this;
			tabs.dom.className = "tabs_tb";
			tab.firstTab = true;
			tab.queryId = null;
			tab.queryName = null;

			tab.onswitch = function()
			{
				this.owner.queryName = null;
				this.owner.refresh();
			
			}
			
			tab.tooltip = '';

			// query properties
			// tab keeps 'permanent' query id 
			this.currentTab = tab;
			
		}
			
		
		// add tabs to the window
		/*
		if (browser.safari) {
			//tabs.dom.style.width = '100%';
			tabs.dom.parentNode.style.width = '100%';
		}
		*/

		// add preselect query tabs
		for (var tCount = 0; tCount < this.table.preselectQueries.length; tCount++) {
			var tab = tabs.insert(this.table.preselectQueries[tCount].name);
			tab.owner = this;
			tabs.dom.className = "tabs_tb";
			tab.firstTab = false;
			tab.queryId = null;
			tab.queryName = this.table.preselectQueries[tCount].name;

			tab.onswitch = function()
			{
				this.owner.queryName = this.queryName;
				this.owner.refresh();
			}
			
			tab.tooltip = '';	

			if (!this.currentTab) {
				this.currentTab = tab;
			}
		}


		
		// set first tab active
		tabs.activateById(0);
		//winObj.resize();
		
		this.tabsbar = tabs;			
		
		aWindow.contentArea.appendChild(this.calendar.dom);							
		
		aWindow.activate();
		aWindow.open();	

		// update window on close box method
		aWindow.closeBox.onmouseup = function (aEvent) {
			WindowManager.hideWindow(this.parent);
			window.clearTimeout(this.parent.boundTo.refreshTimeout);
			window.clearTimeout(this.parent.boundTo.dynamicTabId);
			this.parent.tbitem.dom.style.display = 'none';
			return false;
		}

		aWindow.tableId = this.table.tableid;
		
		aWindow.status.style.height = '19px';
		
		// set minimum sizes
		aWindow.size (726,500);
		aWindow.minimumWidth = 726;
		aWindow.minimumHeight = 500;
		
		// set icon
		aWindow.logo.className = 'window_logo_calendar';
		this.displayType = 'window';
		// set window as parent
		this.parent	= aWindow.contentArea;
		this.container = aWindow;
		// set calendar redraw on window resize
		aWindow.boundTo = this;
		aWindow.onsize = function () {
			this.boundTo.calendar.refresh();
			this.boundTo.resizing = true;
		}
		aWindow.onsizestop = function () {
			this.boundTo.calendar.afterRefresh();
			this.boundTo.resizing = false;
			this.boundTo.tabsbar.refresh();
		}
		// if taskbar exists, add taskbar icon
		if (taskbar) {
			var ti = new taskbarItem(taskbar);
			ti.setLabel(aWindow.titleBar.innerHTML);
			ti.setIcon("calendar");
			setStyle (ti.dom, 'display', 'block');
			// create a handler for clicking
			var winRef = aWindow;
			ti.dom.onclick = function() { 
				this.parent.setSelected (true, true);
				winRef.activate(); 
			};

			ti.setSelected (true, true);
			ti.parent.refresh();

			aWindow.tbitem = ti;
		}

		// update window title bar refresh button action
		aWindow.refreshBox.boundTo = this;
		aWindow.refreshBox.onmouseup = function(event)
		{
			var event = xpEvent(event, true);
			this.className = "window_refreshbox";
			this.boundTo.refresh();
			return false;
		};
		// window on close method
		aWindow.onclose = function() {
			// erase taskbar item
			if (this.tbitem)
				this.tbitem.remove();
		
			// get rid of the editor
			if (this.editor)
				this.editor.destroy();
		}
	}

	// handling dynamic tabs
	// update tabs with dynamic values
	this.updateDynamicQueryTabs = function()
	{
		if (this.table) {
			if (this.table.preselectedDynamicQueries) {
				if (this.table.preselectedDynamicQueries.length > 0) {
					for (var qCount = 0; qCount < this.table.preselectedDynamicQueries.length; qCount++) {
						// check for valid string
						if (this.table.preselectedDynamicQueries[qCount].fieldId.indexOf('][') != -1) {
							// generate a call to backend to get the list of unique values
							var url = dax_bridge.url + "GetUniqueQueries?sessionId=" + dax_bridge.sessionId + "&tableid=" + this.table.tableid +
								"&fieldid=" + this.table.preselectedDynamicQueries[qCount].fieldId;
							makeCall (url, this.updateDynamicQueryTabs_handler, this);
						}
					}
				}
			}
		}
	}
	
	this.updateDynamicQueryTabs_handler = function(http_result, aRef)
	{
	
		if (http_result.readyState != 4)
			return;

		var responseError = checkResponse (http_result);
			
		if (responseError)	
			return;
		
		var queryNode = http_result.responseXML.getElementsByTagName("queries").item(0);
		var queryNames = queryNode ? queryNode.getElementsByTagName("query") : null;		
		var fieldId = queryNode.getAttribute("fieldid");
		
		if (!queryNames)
			return;
		
		if (queryNames.length == 0)
			return;

		if (aRef.updatingTabsFieldId) {
			if (aRef.updatingTabsFieldId == fieldId) {
				return;
			} else {
				setTimeout(function() {DataWindowPartThree_updateDynamicQueryTabs_handler(http_result, aRef) }, 1000);
				return;
			}
		} else {
			aRef.updatingTabsFieldId = fieldId;
		}			
			
		for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
			// mark all tabs as removable until proven otherwise later
			aRef.tabsbar.tabs[tabCount].keepTab = false;
		}
			
		for (qnCount = 0; qnCount < queryNames.length; qnCount++) {	
			var queryName = queryNames.item(qnCount).getAttribute('name');
			if (queryName) {
				if (queryName != '4daf_unique_') {
					var tabExists = false;
					// check if the tab exists before adding it
					for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
						if (aRef.tabsbar.tabs[tabCount].queryName == queryName) {
							var tabExists = true;
							// so we won't erase it later when purging the non-existent tabs
							aRef.tabsbar.tabs[tabCount].keepTab = true;
						}						
					}

					if (!tabExists) {
						// add tab
						var tabs = aRef.tabsbar;
						var tab = tabs.insert(queryName.substr(12));
						tab.owner = aRef;
						tabs.dom.className = "tabs_tb";
						tab.firstTab = false;
						tab.queryId = null;
						tab.queryName = queryName;
						tab.keepTab = true;

						tab.onswitch = function()
						{
							this.owner.queryName = this.queryName;
							this.owner.refresh();
						}
						
						tab.tooltip = '';
					}
				}
			}
		}
		// erase tabs
		var eraseTabsArray = new Array();
		for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
			if (!aRef.tabsbar.tabs[tabCount].firstTab) {
				if ((!aRef.tabsbar.tabs[tabCount].keepTab) && (aRef.tabsbar.tabs[tabCount].queryName)) {
					if ((aRef.tabsbar.tabs[tabCount].queryName.indexOf('4daf_unique_') != -1) && (aRef.tabsbar.tabs[tabCount].fieldId == fieldId))
						eraseTabsArray.push(aRef.tabsbar.tabs[tabCount]);
				}
			}
		}
		if (eraseTabsArray) {
			if (eraseTabsArray.length > 0) {
				for (var tabCount = 0; tabCount < eraseTabsArray.length; tabCount++) {
					var clearCurrentTab = false;
					// remove current tab manually from data window
					if (aRef.currentTab = eraseTabsArray[tabCount])
						clearCurrentTab = true;
					// remove tab
					aRef.tabsbar.remove(eraseTabsArray[tabCount].index);
					if (clearCurrentTab)
						aRef.currentTab = null;
				}
			}			
		}
		
		// current tab still exists, reselect it
		if (aRef.currentTab) {
			aRef.tabsbar.activate(aRef.currentTab);
		} else {
			aRef.currentTab = aRef.tabsbar.tabs[0];
			aRef.currentTab.dom.onclick();
		}

		// if object is in init state, open first dynamic tab
		if (aRef.initialize == true) {
			aRef.initialize = false;
			if (aRef.tabsbar.tab[0])
				aRef.tabsbar.tab[0].dom.onclick();
		}		
		
		// release lock
		aRef.updatingTabsFieldId = null;
		
	}
	
	this.refreshDynamicTabs = function()
	{
		if (this.dynamicTabId) {
			clearTimeout(this.dynamicTabId);
		}

		this.updateDynamicQueryTabs();
		
		var thisWindow = this;
					
		this.dynamicTabId = setTimeout(function() { thisWindow.refreshDynamicTabs(); }, 60 * 3 * 1000);

	}


	// editor function for window
	this.getEditor = function()
	{
		if (this.editor)
			return this.editor;
	
		// create editor object
		this.editor = new Editor();
		var dom = this.editor.getDOM();
		dom.style.zIndex = ++WindowManager.maxZIndex;
		if (this.displayType == 'window') {
			this.container.dom.m.insertBefore(dom, this.container.toolbar);
			var w_off =  (this.container.dom.offsetWidth / 2) -
				(this.container.dom.offsetWidth / 2);
		} else {
			this.container.insertBefore(dom, this.calendarViewNode);
			var w_off =  (this.container.offsetWidth / 2) -
				(this.container.offsetWidth / 2);
		}
	

		setStyle(dom, "position", "absolute");
		setStyle(dom, "left", w_off + "px");
		
		this.editor.owner = this;
		
		var w_ref = this; // use this for when editor is detatched
		this.editor.onaftersave = function()
		{
			this.owner.refresh();
		}

		this.editor.onafterdelete = function()
		{
			this.owner.refresh();
			this.getTearAway().contract(1, 50);
			this.destroy();
		}
		
		// customize
		dom.ondragstop = function(aEvent)
		{
			this.className = "tearaway_torn";
			var editor = this.owner;
			
			if (browser.ie) {
				this.top.t.style.fontSize = '1pt';
				this.top.tl.style.fontSize = '1pt';
				this.top.tr.style.fontSize = '1pt';
				this.top.style.fontSize = '1pt';

			}
			
			// create new window when dragging is done
			if (editor.owner) {
				var winObj = editor.owner.container;
				
				if (!winObj.editor)
					return false;
					
				var dom = editor.getDOM();
				var pos = getObjectPosition(this); // global coordinates

				if (taskbar) {
				
					// add taskbar item for new window
					var ti = new taskbarItem(taskbar);

					// make unique label based on table name
					i = 1;
					while ($('dax_taskbar_button' + winObj.titleBar.innerHTML + i) != null) {
						i++;
					}
					// set taskbar button title
					ti.dom.id = 'dax_taskbar_button' + winObj.titleBar.innerHTML + i;

					ti.setLabel("Edit " + winObj.titleBar.innerHTML + " " + i);
					ti.setIcon("editor");
				
					// create a handler for clicking
					var ref = this;
					ti.dom.onclick = function() { 
						this.parent.setSelected (true, true);
						ref.onclick();
					};
					setStyle (ti.dom, 'display', 'block');
					ti.setSelected (true, true);
					// refresh taskbar
					ti.parent.refresh();
				}
				
				// override cancel button
				dom.cancelBtn.onclick = function(aEvent) {
					if (ti)
						ti.remove(); // remove taskbar item
					document.body.removeChild(this.parent.prv.mDOM);
					this.parent.destroy();
					delete this.parent;
				};
				
				// override the focusing
				this.onclick = function(aEvent) {
					if (WindowManager.active)
						WindowManager.active.deactivate();
						setStyle(this, "zIndex", ++WindowManager.maxZIndex);
						if (ti) {
							ti.setSelected(true,true);
						}
				};

				// release
				if (this.owner.owner.displayType == 'window')
					winObj.deactivate();
				this.owner.owner.editor = null;
				//this.parent.owner = null;
				
				// reposition
				document.body.insertBefore(this, null);
				setStyle(this, "left", pos.x + "px");
				setStyle(this, "top", pos.y + "px");
				
			}
			
		};
		return this.editor;
	};		
	

	// add toolbar
	this.toolbar = new Toolbar();
	
	if (this.toolbar) {
	
		this.toolbar.prv.mDOM.className = 'toolbar calendar_view_toolbar';

		// create the buttons we need
		var createBtn = this.toolbar.addButton(STR['button_create']);
		createBtn.boundTo = this;
		this.createButton = createBtn;
		createBtn.onclick = function() { 
		
			if (this.boundTo.table.fieldsDetail.length == 0)
				return false;
		
			// // find the editor if we have one, otherwise create a new one
			if (this.boundTo.editor)
				this.boundTo.editor.destroy();
			var editor = this.boundTo.getEditor();
			// clear the editor, set its record to -1 to signify that it is a new record
//			if (isDefined(editor.record)) {
//				// editor exists, use it
//				editor.record.recordId = -1;
//				editor.getDOM().deleteBtn.disabled = false; // make sure an empty record can't be deleted
//			} else {
				// editor doesn't exists, create new one and use it
				//var grid = this.boundTo.contentArea.grid;
				editor.record = { recordId : -1, tableId: this.boundTo.table.tableid };
				editor.buildEditorByTable(this.boundTo.table.tableid);
//			}
			// now we are ready to edit
			editor.clear();
			// set date value into the editor
			var eValue = this.boundTo.calendar.getToday();
			// editor.prv.mInputs[editor.record.map[this.boundTo.dateField.fieldid]].prv.mValue = eValue;
			// editor.prv.mInputs[editor.record.map[this.boundTo.dateField.fieldid]].picker.value.value = eValue;
			editor.edit();
		};
		
		// delete record(s) button
		var deleteBtn = this.toolbar.addButton(STR['button_delete']);
		deleteBtn.boundTo = this;
		this.deleteButton = deleteBtn;
		deleteBtn.onclick = function() { 
			// string that lists all records to be deleted
			var deleteRecords = '';
			for (var k in this.boundTo.recordsMap) {
				if (this.boundTo.recordsMap[k].selected == true)
					if (deleteRecords)
						deleteRecords += ',' + k;
					else
						deleteRecords = k;
			}
			
			// make deletion call
			if (deleteRecords) {
				// confirm record deletion
				var userConfirm = confirm(STR['confirm_delete_records']);
				if (!userConfirm)
					return;
				// send delete call
				url = dax_bridge.url + "deleteRecord?sessionId=" + dax_bridge.sessionId	+ 
				"&recordId=" + deleteRecords +
				"&tableId=" + this.boundTo.table.tableid;
				makeCall (url, this.boundTo.onDelete, this.boundTo);
			}
		};
		
		// on completed delete call, refresh the calendar
		this.onDelete = function (http_result, aCalendar) {
			if (http_result.readyState != 4)
				return;
			var responseError = checkResponse (http_result);
			aCalendar.refresh();
		}
		
		// check for errors
		
		
		// add the toolbar to the window
		if (aWindow) {
			aWindow.addToolbar(this.toolbar.prv.mDOM);
			aWindow.resizeToolbar(false);
			aWindow.addToolbar(tabs.dom);
	
		} else {
			// simulate toolbar
			var toolbarNode = getElementByClassName (this.container, 'calendar_view_toolbar');
			if (toolbarNode) {
				toolbarNode.appendChild (this.toolbar.prv.mDOM);
				// adjust sizes for fixed container
				setStyle (this.parent, 'height', this.container.offsetHeight - this.toolbar.prv.mDOM.offsetHeight + 'px');
				setStyle (toolbarNode, 'width', this.container.offsetWidth + 'px');
			} else {
				// toolbar failed to load, use different resizing
				setStyle (this.parent, 'height', this.container.offsetHeight + 'px');
			}
		}
		
		// add the spinner
		var spinner = getTemplate("spinner").cloneNode(true);
		setStyle(spinner, "display", "block");
		this.toolbar.addContent(spinner);
		this.toolbar.spinner = spinner;
		// assign unique id to the spinner
		i = 1;
		while ($('spinner' + i) != null) {
			i++;
		}
		spinner.id = 'spinner' + i;				
		
		//var search = getTemplate("searchwidget").cloneNode(true);
		//search.id = "searchwidget"+getElementsByClassName(document.body, "searchwidget");
		//search.inputBox = getElementByClassName (search, 'queryname');

		// setup and display search
		this.search 	= this.searchNode.cloneNode(true);
		this.calendar.header.insertBefore (this.search, getElementByClassName (this.calendar.header, 'dax_calendar_header_text'));
		this.searchBox	= getElementByClassName (this.search, 'calendar_view_search_box');
		setStyle (this.search, 'display', 'block');

		// assign unique id to the object
		i = 1;
		while ($('calendar_view_search' + i) != null) {
			i++;
		}
		this.search.id = 'calendar_view_search' + i;		
		
		// set the search method
		// search will hide all cell elements that don't meet search criteria
		this.searchBox.owner = this;
		this.searchBox.onkeyup = function (aEvent) {
		
			var cRef = this.owner;
			if (cRef.searchTimeout) {
				window.clearTimeout (cRef.searchTimeout);
			}
				
			cRef.searchTimeout = window.setTimeout (function() {cRef.refresh(cRef); }, 1000);
		
		//alert ('searching ' + this.owner.searchField.fieldname + ' for ' + this.value);		
		
			/*
		
			// deselect all records
			for (var k in this.owner.recordsMap) {
				if (this.owner.recordsMap[k].selected == true)
					this.owner.recordsMap[k].selected = false;
				for (l = 0; l < this.owner.recordsMap[k].length; l++) {
					// apply styles based on type
					this.owner.recordsMap[k][l].l.className = 'l calendar_view_l_close';
					this.owner.recordsMap[k][l].r.className = 'r calendar_view_r_close';
					this.owner.recordsMap[k][l].m.className = 'm';
					switch (this.owner.recordsMap[k][l].rangeType) {
						case "start":
							this.owner.recordsMap[k][l].r.className = 'r calendar_view_r_normal';
							break;
						case "end":
							this.owner.recordsMap[k][l].l.className = 'l calendar_view_l_normal';
							break;
						case "middle":
							this.owner.recordsMap[k][l].r.className = 'r calendar_view_r_normal';
							this.owner.recordsMap[k][l].l.className = 'l calendar_view_l_normal';
							break;
					}
				}
			}
		
			// get search value, set it to lower case to avoid case sensitive search
			var searchValue = this.value.toLowerCase();
			// loop through cells and hide the cells that don't meet search criteria
			for (i = 0; i < this.owner.records.length; i++) {
				// if search value is not matched, hide the record
				var cellValue = this.owner.records[i].value.toLowerCase();
				if (cellValue.indexOf(searchValue) == 0) {
					// search entry found
					setStyle(this.owner.records[i], 'display', 'block');
				} else {
					// not found
					setStyle(this.owner.records[i], 'display', 'none');
				}
			}
			
			*/
		}
			
			
		// add display and range fields to toolbar
		this.displayFieldNode = getTemplate("calendar_view_display_field").cloneNode(true);
		if (this.displayFieldNode) {
			setStyle (this.displayFieldNode, 'display', 'block');
			this.toolbar.addContent(this.displayFieldNode);

			// assign unique id to the object
			i = 1;
			while ($('calendar_view_display_field' + i) != null) {
				i++;
			}
			this.displayFieldNode.id = 'calendar_view_display_field' + i;	

			// localize range
			getElementByClassName (this.displayFieldNode, 'label_range', true).innerHTML = STR['calendar_toolbar_range'];
			getElementByClassName (this.displayFieldNode, 'label_to', true).innerHTML = STR['calendar_toolbar_to'];
			getElementByClassName (this.displayFieldNode, 'label_display', true).innerHTML = STR['calendar_toolbar_display'];
			var fieldsOptGroup = getElementsByClassName (this.displayFieldNode, 'searchOptGroup');
			for (k = 0; k < fieldsOptGroup.length; k++) {
				fieldsOptGroup[k].label = STR['fields'];
			}
			
			// updates displayFieldNode
			this.displayFieldNodeRefresh = function ()
			{
				// link to all necessary fields
				// for display field
				var displayOption = getElementByClassName(this.displayFieldNode, "fieldDisplayOption");
				var displayGroup = displayOption.parentNode;
				var displaySelect = getElementByClassName(this.displayFieldNode, "fieldDisplaySelect");
				var displayCount = 0;

				var displayAux1Option = getElementByClassName(this.displayFieldNode, "fieldAux1DisplayOption");
				var displayAux1Group = displayAux1Option.parentNode;
				var displayAux1Select = getElementByClassName(this.displayFieldNode, "fieldAux1DisplaySelect");
				
				// for range start field
				var rangeStartOption = getElementByClassName(this.displayFieldNode, "fieldRangeStartOption");
				var rangeStartGroup = rangeStartOption.parentNode;
				var rangeStartSelect = getElementByClassName(this.displayFieldNode, "fieldRangeStartSelect");
				var rangeStartCount = 0;
				
				// for range end field
				var rangeEndOption = getElementByClassName(this.displayFieldNode, "fieldRangeEndOption");
				var rangeEndGroup = rangeEndOption.parentNode;
				var rangeEndSelect = getElementByClassName(this.displayFieldNode, "fieldRangeEndSelect");
				var rangeEndCount = 0;

				var searchOption	= getElementByClassName(this.search, "fieldSearchOption");
				var searchGroup		= searchOption.parentNode;
				var searchSelect	= getElementByClassName(this.search, "fieldSearchSelect");
				
				// build the select lists in the toolbar
				for (var k in this.table.fields) {

					// filter out date values for range fields
					if ((this.table.fields[k].fieldtype == STR['type_date']) && (this.table.fields[k].fieldinlist.toLowerCase() == 'true')){
						// add new select node
						// first field exists, but others must be cloned
						if (rangeStartCount > 0) {
							rangeStartOption = rangeStartOption.cloneNode(true);
						} else {
							rangeStartSelect.fieldid = this.table.fields[k].fieldid;
						}
						// field is a date, add it to the start range				
						rangeStartOption.parent = rangeStartSelect;
						rangeStartOption.innerHTML = this.table.fields[k].fieldalias;
						//option.value = table.fields[k].fieldid;
						rangeStartOption.fieldid = this.table.fields[k].fieldid;
						rangeStartSelect.boundTo = this;
						rangeStartSelect.endSelect = rangeEndSelect;
						rangeStartSelect.onchange =
							function() { 
								this.boundTo.dateField = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
								// if start date is same as end date, change end date to none
								if (this.boundTo.dateEndField) {
									if (this.boundTo.dateField.fieldid == this.boundTo.dateEndField.fieldid) {
										this.endSelect.selectedIndex = 0;
										this.boundTo.dateEndField = null;
									}
								}
								// hide selected start date in end date range
								var startDateId = this.options[this.selectedIndex].fieldid;
								for (i = 0; i < this.endSelect.options.length; i++) {
									if (this.endSelect.options[i].fieldid == startDateId) {
										setStyle (this.endSelect.options[i], 'display', 'none')
									} else {
										setStyle (this.endSelect.options[i], 'display', 'block')
									}
								}
								// redraw calendar
								this.boundTo.refresh();
							};						
						rangeStartGroup.appendChild(rangeStartOption);
						// if this start date is selected, set it
						if (this.table.fields[k].fieldid == this.dateField.fieldid)
							rangeStartOption.selected = true;					
						rangeStartCount++;
						// do the same for end range, as long as it's not the same as select start range
						if (true) {
							if (rangeEndCount > 0) {
								rangeEndOption = rangeEndOption.cloneNode(true);
							} else {
								rangeEndSelect.fieldid = this.table.fields[k].fieldid;
								// add None option since end range is optional
								rangeEndOption.parent = rangeEndSelect;
								rangeEndOption.innerHTML = STR['calendar_toolbar_none'];
								//option.value = table.fields[k].fieldid;
								rangeEndOption.fieldid = null;
								rangeEndSelect.boundTo = this;
								rangeEndSelect.onchange =
									function() { 
										this.boundTo.dateEndField = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
										this.boundTo.refresh();
									};	
								rangeEndGroup.appendChild(rangeEndOption);
								rangeEndCount++;				
								rangeEndOption = rangeEndOption.cloneNode(true);
							}
							
							// add date field to the end range				
							rangeEndOption.parent = rangeEndSelect;
							rangeEndOption.innerHTML = this.table.fields[k].fieldalias;
							rangeEndOption.fieldid = this.table.fields[k].fieldid;
							rangeEndOption.onclick =
								function() { rangeEndSelect.fieldid = this.fieldid;};
							rangeEndGroup.appendChild(rangeEndOption);
							// if it is end date field, select it
							if (this.dateEndField) {
								if (this.table.fields[k].fieldid == this.dateEndField.fieldid)
									rangeEndOption.selected = true;		
							}
							// if it is set in start date field, hide it
							if (this.table.fields[k].fieldid == this.dateField.fieldid) {
								setStyle (rangeEndOption, 'display', 'none')
							} else {
								setStyle (rangeEndOption, 'display', 'block')
							};
						}
					}

					// add display fields
					if ((
							(this.table.fields[k].fieldtype == STR['type_alpha']) ||
							(this.table.fields[k].fieldtype == STR['type_text']) ||
							(this.table.fields[k].fieldtype == STR['type_real']) ||
							(this.table.fields[k].fieldtype == STR['type_longint']) ||
							(this.table.fields[k].fieldtype == STR['type_integer']) ||
							(this.table.fields[k].fieldtype == STR['type_boolean']) ||
							(this.table.fields[k].fieldtype == STR['type_date']) ||
							(this.table.fields[k].fieldtype == STR['type_time'])
						) && (this.table.fields[k].fieldinlist.toLowerCase() == 'true')) {
					
						// first field exists, but others must be cloned
						if (displayCount > 0) {
							displayOption = displayOption.cloneNode(true);
							displayAux1Option = displayOption.cloneNode(true);
							searchOption = searchOption.cloneNode(true);
						} else {
							// first aux1 field is set to none
							displaySelect.fieldid = this.table.fields[k].fieldid;
							displayAux1Select.fieldid = null;
							displayAux1Option.parent = displayAux1Select;
							displayAux1Option.innerHTML = STR['calendar_toolbar_none'];

							searchSelect.fieldid = this.table.fields[k].fieldid;

							//option.value = table.fields[k].fieldid;
							displayAux1Option.fieldid = null;
							displayAux1Select.boundTo = this;
							displayAux1Select.onchange =
								function() { 
									this.boundTo.displayAux1Field = null;
									this.boundTo.refresh();
								};
							displayAux1Group.appendChild(displayAux1Option);
							displayAux1Option = displayOption.cloneNode(true);
							
						}				

						displayOption.innerHTML = this.table.fields[k].fieldalias;
						displayOption.parent = displaySelect;
						//option.value = table.fields[k].fieldid;
						displayOption.fieldid = this.table.fields[k].fieldid;
						displaySelect.boundTo = this;
						displaySelect.onchange =
							function() { 
								this.boundTo.displayField = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
								this.boundTo.refresh();
							};
						displayGroup.appendChild(displayOption);
						// if this display is selected, set it
						if (this.table.fields[k].fieldid == this.displayField.fieldid)
							displayOption.selected = true;

						// aux 1
							
						displayAux1Option.parent = displayAux1Select;
						displayAux1Option.innerHTML = this.table.fields[k].fieldalias;
						//option.value = table.fields[k].fieldid;
						displayAux1Option.fieldid = this.table.fields[k].fieldid;
						displayAux1Select.boundTo = this;
						displayAux1Select.onchange =
							function() { 
								this.boundTo.displayAux1Field = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
								this.boundTo.refresh();
							};
						displayAux1Group.appendChild(displayAux1Option);
			
						// search						
						searchOption.innerHTML = this.table.fields[k].fieldalias;
						searchOption.parent = searchSelect;
						//option.value = table.fields[k].fieldid;
						searchOption.fieldid = this.table.fields[k].fieldid;
						searchSelect.boundTo = this;
						searchSelect.onchange =
							function() { 
								this.boundTo.searchField = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
								//alert ('searching ' + this.boundTo.searchField.fieldname);
							};
						searchGroup.appendChild(searchOption);

						displayCount++;
					}
				}
			}
		}
		this.displayFieldNodeRefresh();			

		// add ddw button to the toolbar
		var eTable = this.table;
		if ((eTable) && (eTable.ddwOptions) && (eTable.ddwOptions.length > 0)) {
			// add a ddw button
			var ddwOption = eTable.ddwOptions[0];
			
			var ddwButton = this.toolbar.addButton (ddwOption.title);
			ddwButton.url = ddwOption.url;
			ddwButton.title = ddwOption.title;
			ddwButton.owner = this;
			if (ddwOption.type.indexOf("Static") != -1) {
				ddwButton.onclick = function () {
					var allRecords = new Array();
					var selectedRecords = new Array();
					if (this.owner.recordsMap) {
						for (z in this.owner.recordsMap) {
							if (this.owner.recordsMap[z][0].style.display != 'none') {
								allRecords.push(z);
								if (this.owner.recordsMap[z].selected) {
									selectedRecords.push(z);
								}
							}
						}
					}
					var tableId = this.owner.table.tableid;
					if (selectedRecords.length > 0)
						new DDW(this.url, 'static_url', this.window, "&tableid=" + tableId, selectedRecords);
					else
						new DDW(this.url, 'static_url', this.window, "&tableid=" + tableId, allRecords);
					// clear arrays
					delete selectedRecords;
					delete allRecords;
				};
			} else if (ddwOption.type.indexOf("Dynamic") != -1) {
				ddwButton.onclick = function () {
					var allRecords = new Array();
					var selectedRecords = new Array();
					if (this.owner.recordsMap) {
						for (z in this.owner.recordsMap) {
							if (this.owner.recordsMap[z][0].style.display != 'none') {
								allRecords.push(z);
								if (this.owner.recordsMap[z].selected) {
									selectedRecords.push(z);
								}
							}
						}
					}
					var tableId = this.owner.table.tableid;
					if (selectedRecords.length > 0)
						new DDW(this.url, 'dynamic_url', this.window, "&tableid=" + tableId, selectedRecords);	
					else	
						new DDW(this.url, 'dynamic_url', this.window, "&tableid=" + tableId, allRecords);	
					// clear arrays
					delete selectedRecords;
					delete allRecords;
				};
			} else if (ddwOption.type.indexOf("Message") != -1) {
				ddwButton.onclick = function () {
					var allRecords = new Array();
					var selectedRecords = new Array();
					if (this.owner.recordsMap) {
						for (z in this.owner.recordsMap) {
							if (this.owner.recordsMap[z][0].style.display != 'none') {
								allRecords.push(z);
								if (this.owner.recordsMap[z].selected) {
									selectedRecords.push(z);
								}
							}
						}
					}
					var tableId = this.owner.table.tableid;
					if (selectedRecords.length > 0)
						new DDW(this.url, 'message_window', this.owner.container, "&tableid=" + tableId, selectedRecords);	
					else
						new DDW(this.url, 'message_window', this.owner.container, "&tableid=" + tableId, allRecords);	
					// clear arrays
					delete selectedRecords;
					delete allRecords;
				};
			}
		}
	}
	
	
	
	// set toolbar size
	if (this.toolbar) {
		if (this.parent.offsetWidth < 800) {
			setStyle (this.toolbar.prv.mDOM, 'width', '800px');
		//	setStyle (this.tabsbar.dom, 'width', '800px');
		} else {
			setStyle (this.toolbar.prv.mDOM, 'width', this.parent.offsetWidth + 'px');
		//	setStyle (this.tabsbar.dom, 'width', this.parent.offsetWidth + 'px');
		}
	}
		
	// flag to check if calendar is being resized
	// if it is, there is no need to reload data
	this.resizing = false;
	
	//set calendar parent window object
	// this is used for resizing purposes
	this.calendar.parentWindow = this.parent;
	
	// draw and resize the calendar
	this.calendar.date = 0;
	this.calendar.drawCalendar();
	this.calendar.refresh();


	// refresh calendar
	this.refresh = function(aCalRef) {
		if (!aCalRef)
			aCalRef = this;
		// Fix
		if (!aCalRef.initializeCells)
			aCalRef = this.WindowManager.active.boundTo;
		// check if update is already in progress
		// if so, try again later
		if (aCalRef.updating == true) {
			if (aCalRef.delayInt)
				clearInterval (aCalRef.delayInt);		
			aCalRef.delayInt = setInterval (aCalRef.refresh, 1000, aCalRef);
		} else {
			if (aCalRef.delayInt)
				clearInterval (aCalRef.delayInt);
			aCalRef.updating = true;
			aCalRef.initializeCells();

			// if this is initial loding, check if allrecords, static, or dynamic tab should be opened
			if (aCalRef.initializing) {
				// initializing done
				aCalRef.initializing = false;
				aCalRef.updating = false;
		
				if (aCalRef.table.showallrecordstab.toLowerCase() == 'true') {
					// show all records
					// ie, do nothing at this point
				} else if (aCalRef.table.preselectQueries.length > 0) {
					// static preselected queries, no all records tab
					if (aCalRef.tabsbar.tabs[0])
						aCalRef.tabsbar.tabs[0].dom.onclick();
					return true;
				} else if (aCalRef.table.preselectedDynamicQueries.length > 0) {
					// call for tabs right now, we need them for data
					aCalRef.initializing = false;
					aCalRef.refreshDynamicTabs();
					return true;
				} else {
				
				}
			}

			aCalRef.populate(aCalRef);
		}
	}
	
	// reference grid object within calendar
	this.grid		= this.calendar.calGridObj.grid;

	// this node is used to extend dates that don't fit into the main cell
	this.extendDiv 	= this.extendNode.cloneNode(true);
	this.grid.body.appendChild (this.extendDiv);
	
	// assign unique id to the object
	i = 1;
	while ($('calendar_view_extend' + i) != null) {
		i++;
	}
	this.extendDiv.id = 'calendar_view_extend' + i;		
	
	this.extendDiv.cellOrigin = null;
	this.extendDiv.cellChild = null;
	
	this.extendDiv.hide = function () {
		if (!this.extended)
			return;
		setStyle (this, 'display', 'none');
		this.cellOrigin.appendChild(this.cellChild);
		this.cellOrigin.extended = false;
		for (i = 0; i < this.cellOrigin.records.length; i++) {
			if (this.cellOrigin.records[i].showValue)
				this.cellOrigin.records[i].m.value.innerHTML = this.cellOrigin.records[i].value;
			else
				this.cellOrigin.records[i].m.value.innerHTML = '';
		}
		this.cellOrigin = null;
		this.cellChild = null;
		this.extended = false;
	}
	
	// prefix is YYYY-MM-
	// we'll also use this prefix for quering
	this.datePrefix	= this.calendar.getDate().substring(0,8);
	
	// keeps list of all records in the calendar for quick search
	this.records = new Array();
	// map those records by record id
	// this.recordsMap[id] = array of cell records with the [id]
	this.recordsMap = new Array();
	
	// customize calendar
	this.customize = function (aCreateDelete, aDisplayBar, aSearchBar) {
	
		// set default values
		if (!aCreateDelete)
			aCreateDelete = false;
		if (!aDisplayBar)
			aDisplayBar = false;
		if (!aSearchBar)
			aSearchBar = false;
	
		// if both buttons and display bar are disabled, hide entire toolbar
		if ((aCreateDelete == false) && (aDisplayBar == false)) {

			this.toolbar.prv.mDOM.style.display = 'none';
					
		} else {

			this.toolbar.prv.mDOM.style.display = 'block';
		
			// hide or show display bar
			if (aDisplayBar == true) {
				this.displayFieldNode.style.display = 'block';
			} else {
				this.displayFieldNode.style.display = 'none';
			}

			// hide or show display bar
			if (aCreateDelete == true) {
				this.createButton.prv.mDOM.style.display = 'block';
				this.deleteButton.prv.mDOM.style.display = 'block';
			} else {
				this.createButton.prv.mDOM.style.display = 'none';
				this.deleteButton.prv.mDOM.style.display = 'none';
			}
		
		}
		
		// hide or show search bar
		if (aSearchBar == true) {
			this.search.style.display = 'block';
		} else {
			this.search.style.display = 'none';
		}
	}
	
	// set calendar refresh method to initialize and repopulate cells
	this.calendar.owner				= this;
	this.calendar.afterRefresh		= function () {
		// set toolbar size
		if (this.owner.toolbar)
			if (this.owner.parent.offsetWidth < 800) {
				setStyle (this.owner.toolbar.prv.mDOM, 'width', '800px');
				//setStyle (this.owner.tabsbar.dom, 'width', '800px');
			 } else {
				setStyle (this.owner.toolbar.prv.mDOM, 'width', this.owner.parent.offsetWidth + 'px');
				//setStyle (this.owner.tabsbar.dom, 'width', this.owner.parent.offsetWidth + 'px');
			}

		// resize window to compensate for toolbar addition
//		this.owner.container.size(this.owner.container.dom.offsetWidth, this.owner.container.dom.offsetHeight);
//		this.owner.container.resize(1, 1);

		
		//if window is resizing, there is no need to reload data
		if (!this.owner.resizing) {
			if (this.owner.currentQuery) {
				this.owner.currentQuery.die = true;
				this.owner.currentQuery = null;
			}		
			// this array binds specific date to point to a specific cell
			// array elements are named as YYYY-MM-DD format to make it 4D friendly
			// example: dateCell[1984-11-23] = cellX;
			this.owner.dateCell	= new Array();
			// keep track of all records as well
			this.owner.records = new Array();
			this.owner.recordsMap = new Array();
			// prefix is YYYY-MM-
			// we'll also use this prefix for quering
			this.owner.datePrefix	= this.owner.calendar.getDate().substring(0,8);
			this.owner.initializeCells();
			// clear search field
			//this.owner.searchBox.value = '';
			// make call to populate calendar with data
			this.owner.populate(this.owner);
		} 
		// recalculate cell size to set overflow for appropriate cells
		for (i=0; i < this.owner.grid.prv.mRows.length; i++) {
			for (j=0; j < this.owner.grid.prv.mRows[i].prv.mElements.length; j++) {			
				// get cell reference
				var cell = this.owner.grid.prv.mRows[i].prv.mElements[j].prv.mDOM;
				// set cell body height
				if (cell.header) {
					var bodyHeight = cell.offsetHeight - cell.header.offsetHeight;
					setStyle (cell.body, 'height', bodyHeight + 'px');						
					setStyle (cell.header.extend, 'display', 'none');
					// resize elements
					if ((cell.records) && (cell.records.length > 0)) {
						for (k=0; k < cell.records.length; k++) {
							var cellWidth = cell.records[k].offsetWidth - cell.records[k].l.offsetWidth - cell.records[k].r.offsetWidth;
							setStyle (cell.records[k].m, 'width', cellWidth +'px');
							if (browser.ie) {
								cell.records[k].m.value.style.width = cellWidth +'px';
							}
						}
						if ((cell.records[0].offsetHeight * (cell.countElements + 1)) > cell.offsetHeight) {
							setStyle (cell.header.extend, 'display', 'block');
						} 	
					}
				}
			}
		}
	}
	
	// should be executed after every redraw
	// prepares calendar cell for population
	this.initializeCells = function () {

		if (this.toolbar) {
			setStyle(spinner, "visibility", "visible");
			//setStyle(this.toolbar.spinner, "display", "block");
		}
		
		// maps dates to cells: dateCell[date] = cell
		this.dateCell = new Array();
		// keep track of all records as well
		this.records = new Array();
		this.recordsMap = new Array();
		
		//loop through the calendar grid and populate it
		//first loop through rows
		for (i=0; i < this.grid.prv.mRows.length; i++) {

			// reference to a row i
			// add row specific behaviour here
			var row = this.grid.prv.mRows[i].prv.mDOM;
		
			// loop through cells in row i
			for (j=0; j < this.grid.prv.mRows[i].prv.mElements.length; j++) {

				// reference to a cell i,j
				// add row specific behaviour here
				var cell = this.grid.prv.mRows[i].prv.mElements[j].prv.mDOM;
				
				// bind this cell to date array
				if (cell.date) {
					// append 0 if date is one digit
					if (cell.date.length == 1)
						cell.date = '0' + cell.date;
					var date = this.datePrefix + cell.date;
					this.dateCell[date] = cell;			
				}
				// erase cell contents
				while (cell.firstChild)
					cell.removeChild(cell.firstChild);

				if (cell.date) {
					// get cell element templates
					var header = this.headerNode.cloneNode (true);
					var body = this.bodyNode.cloneNode (true);
					// assign unique id's
					// assign unique id to the object
					p = 1;
					while ($('calendar_view_cell_header' + p) != null) {
						p++;
					}
					header.id = 'calendar_view_cell_header' + p;							
					body.id = 'calendar_view_cell_body' + p;							
					//place header and body
					cell.appendChild (header);
					cell.header = header;
					cell.appendChild (body);
					cell.body = body;
					// show cell and body
					setStyle (header, 'display', 'block');
					setStyle (body, 'display', 'block');
					// set calendar view object as cell parent
					cell.owner = this;
					cell.header.owner = this;
					cell.header.parent = cell;
					probe (header);
					// put date in header
					header.value.innerHTML = cell.date;
					// override mouse events
					// change color on hover
					cell.onmouseout = function()
					{
						this.header.className = "calendar_view_cell_header";
						this.body.className = "calendar_view_cell_body";
						if ((this.owner.extendDiv.extended) && (this != this.owner.extendDiv.cellOrigin))
							this.owner.extendDiv.hide();						
					};
					cell.onmouseover = function()
					{
						this.header.className = "calendar_view_cell_header calendar_view_cell_header_selected";
						// if cell if full, show scrollbars
						/*
						if (this.showOverflow) {
							this.body.className = "calendar_view_cell_body_scroll";
						} else {
							this.body.className = "calendar_view_cell_body";
						}
						*/
					};
					// on header double-click, create new record
					cell.header.ondblclick = function () {
						if (this.mouseHover)
							return;
						if (this.owner.table.fieldsDetail.length == 0)
							return false;							
						// find the editor if we have one, otherwise create a new one
						var editor = this.owner.getEditor();
						// clear the editor, set its record to -1 to signify that it is a new record
						if (isDefined(editor.record)) {
							// editor exists, use it
							editor.record.recordId = -1;
							//if (editor.getDOM().deleteBtn)
							//	editor.prv.mDOM.deleteBtn.disabled = true; // make sure an empty record can't be deleted
						} else {
							// editor doesn't exists, create new one and use it
							editor.record = { recordId : -1, tableId: this.owner.table.tableid };
							editor.buildEditorByTable(this.owner.table.tableid);
						}
						// now we are ready to edit
						editor.clear();
						// set date value into the editor
						var eValue = this.owner.datePrefix + this.parent.date;
						var aFormat = (this.owner.dateField.fieldformat ? this.owner.dateField.fieldformat : 'd1');
						editor.prv.mInputs[editor.record.map[this.owner.dateField.fieldid]].prv.mValue = eValue;
						editor.prv.mInputs[editor.record.map[this.owner.dateField.fieldid]].picker.value.value = 
							doFormat (eValue, null, aFormat);
						editor.edit();
					}
					// on cell click
					cell.onclick = '';
					// array of cell elements (records)
					cell.records = new Array();
					// we assume that contents of the cell will fit
					cell.showOverflow = false;
					// we'll add height of the contents as added to check when overflow happens
					cell.contentsHeight = 0;
					cell.countElements = 0;
					cell.rowPosition = j;
					// calculate body height and set it
					var bodyHeight = cell.offsetHeight - cell.header.offsetHeight;
					setStyle (cell.body, 'height', bodyHeight + 'px');
					// hide extender
					setStyle (cell.header.extend, 'display', 'none');
					// mouse over and out methods for extender
					cell.header.extend.parent = cell.header;
					cell.header.extend.onmouseout = function () {
						this.className = 'extend';
						this.parent.mouseHover = false;
					}
					cell.header.extend.onmouseover = function () {
						this.className = 'extend_over';
						this.parent.mouseHover = true;
					}
					cell.header.extend.onclick = function () {
						aCalendar = this.parent.owner;
						aExtend = aCalendar.extendDiv;
						aCell = this.parent.parent;
						if (aCell.extended)
							return;
						//show all values
						for (i = 0; i < aCell.records.length; i++) {
							if (aCell.records[i].value)
								aCell.records[i].m.value.innerHTML = aCell.records[i].value;
						}
						// position extend node
						setStyle (aExtend, 'top',  aCell.offsetTop + aCell.header.offsetHeight - 2 + 'px');
						setStyle (aExtend, 'left', aCell.offsetLeft - 2 +'px');
						setStyle (aExtend, 'display', 'block');
						// add contents
						setStyle (aExtend, 'height', (aCell.records[0].offsetHeight * aCell.countElements) + 'px');
						setStyle (aExtend, 'width', aCell.offsetWidth + 'px');
						aExtend.appendChild (aCell.body);
						aExtend.cellChild = aCell.body;
						aExtend.cellOrigin = aCell;
						aExtend.extended = true;
						setStyle (aCell.body, 'height', aExtend.offsetHeight + 'px');
						// set cell as extented
						aCell.extended = true;
					}					
				}
			}
		}
	}
	
	// populates the calendar based on current month
	// will make several calls until all data is here
	this.populate = function (cCalendar) {
		// if calendar already has a query in progress, set it to die
		if (cCalendar.currentQuery) {
			cCalendar.currentQuery.die = true;
			cCalendar.currentQuery = null;
		}
		// set query parameters
		aQuery = new Query(cCalendar.table.tableid);
		aQuery.formatObject = 'calendar';
		aQuery.batchSize = 50;

		// is custom query present?
		if (cCalendar.customQueryValue) {
			aQuery.queryEngine.field.push(cCalendar.customQueryField.fieldid);
			aQuery.queryEngine.argument.push('equal');
			aQuery.queryEngine.value.push(cCalendar.customQueryValue);
			aQuery.queryEngine.andor.push('and');
		}		
		
		if (cCalendar.dateEndField) {
			// multiple date range
			// range start
			aQuery.queryEngine.field.push(cCalendar.dateField.fieldid);
			aQuery.queryEngine.argument.push('less');
			aQuery.queryEngine.value.push(cCalendar.datePrefix.substring (0,7) + '-31');
			aQuery.queryEngine.andor.push('and');
			aQuery.displayFields.push(cCalendar.dateField.fieldid);		
			// range end
			aQuery.queryEngine.field.push(cCalendar.dateEndField.fieldid);
			aQuery.queryEngine.argument.push('greater');
			aQuery.queryEngine.value.push(cCalendar.datePrefix.substring (0,7) + '-01');
			aQuery.queryEngine.andor.push('and');
			aQuery.displayFields.push(cCalendar.dateEndField.fieldid);			
		} else {
			// single date range
			aQuery.queryEngine.field.push(cCalendar.dateField.fieldid);
			aQuery.queryEngine.argument.push('equal');
			aQuery.queryEngine.value.push(cCalendar.datePrefix.substring (0,7));
			aQuery.queryEngine.andor.push('and');
			aQuery.displayFields.push(cCalendar.dateField.fieldid);
		}
		
		// if search option is used
		if (cCalendar.searchBox.value) {
		
			var doSearch = true;
			var searchValue = cCalendar.searchBox.value;
			// if date is used, format it
			if (cCalendar.searchField.fieldtype == 'date') {
				var searchValue = doFormat(cCalendar.searchBox.value, cCalendar.searchField.fieldformat, 'YYYY-MM-DD');
				if (!searchValue)
					doSearch = false;
			}
		
			if (doSearch) {
				aQuery.queryEngine.field.push(cCalendar.searchField.fieldid);
				aQuery.queryEngine.argument.push('starts with');
				aQuery.queryEngine.value.push(searchValue);
				aQuery.queryEngine.andor.push('and');
			}
		}

		if (cCalendar.eraseOnQueryDone) {
			cCalendar.customQueryValue = null;
			cCalendar.eraseOnQueryDone = false;
		}		
		
		aQuery.displayFields.push(cCalendar.displayField.fieldid);

		if (cCalendar.displayAux1Field)
			aQuery.displayFields.push(cCalendar.displayAux1Field.fieldid);
		
		aQuery.queryName = cCalendar.queryName;
		
		
		/*
		aQuery.queryFieldId = cCalendar.dateField.fieldid;
		aQuery.queryComp = 'equal';
		aQuery.queryValue = cCalendar.datePrefix.substring (0,7);
		if (cCalendar.dateEndField)
			aQuery.queryEndRangeId = cCalendar.dateEndField.fieldid;
		*/
		// using variables because 'this' reference will be lost when processing reply
		aHandler = cCalendar.populate_handler;
		aCellNode = cCalendar.cellNode;
		aCalendar = cCalendar;
		aCalendar.currentQuery = aQuery;
		
		// start batch calls
		aQuery.nextBatch(aHandler);			
	}
	
	this.populate_handler = function (http_response) {
		// proceed only if http call state is 4
		if (http_response.readyState !=4)
			return;

		if (!aCalendar.currentQuery)
			return;
		
		// check for errors
		if (checkResponse (http_response))
			return;

		if (callback_onViewQuery)
			http_response = callback_onViewQuery(aQuery.table.tablename, http_response, 'calendar');
			
		// parse response
		aQuery.parseResult (http_response);
		// loop through responses
		if (aQuery.size > 0) {
			for (i = 0; i < aQuery.result.length; i++) {
				// loop for date and value
				var value		= null;
				var valueAux1	= null;
				var dateStart 	= null;
				var dateEnd		= null;
				var ddw			= null;				
				var ddwType		= null;				
				for (j = 0; j < aQuery.result[i].fields.length; j++) {
					// check if it is display field
					if (aQuery.result[i].fields[j].id == aCalendar.displayField.fieldid) {
						var value = aQuery.result[i].fields[j].value;
						var ddw = aQuery.result[i].fields[j].ddw ? aQuery.result[i].fields[j].ddw : null;
						var ddwType = aQuery.result[i].fields[j].ddwtype ? aQuery.result[i].fields[j].ddwtype : null;
					}
					if (aCalendar.displayAux1Field) {
						if (aQuery.result[i].fields[j].id == aCalendar.displayAux1Field.fieldid) {
							var valueAux1 = aQuery.result[i].fields[j].value;
						}
					}
					// check for date field
					if (aQuery.result[i].fields[j].id == aCalendar.dateField.fieldid)
						var dateStart = aQuery.result[i].fields[j].originalValue;
					// check for end date field
					if (aCalendar.dateEndField)
						if (aQuery.result[i].fields[j].id == aCalendar.dateEndField.fieldid)
							var dateEnd = aQuery.result[i].fields[j].originalValue;
					}
				// set default value
				if (!value)
					var value = aCalendar.blankValue;
				
				// if date and value exist, place them into calendar
				if ((dateStart) && (value)) {
					// build date range array
					var dateRange = new Array ();
					if (!dateEnd) {
						// no end date, draw calendar with only one date
						dateRange.push(dateStart);
					} else {
						// convert start and end date to js date format for comparison
						var dateStartArray = dateStart.split("-");
						var dateEndArray = dateEnd.split("-");
						if (dateStartArray[2].length == 1)
							dateStartArray[2] = '0' + dateStartArray[2];
						var dateStartDate = new Date (dateStartArray[0], (dateStartArray[1] - 1), dateStartArray[2]);
						if (dateEndArray[2].length == 1)
							dateEndArray[2] = '0' + dateEndArray[2];
						var dateEndDate = new Date (dateEndArray[0], (dateEndArray[1] - 1), dateEndArray[2]);
						var currentMonth = aCalendar.datePrefix.split("-")[1];
						var currentYear = aCalendar.datePrefix.split("-")[0];
						
						if (dateStartDate >= dateEndDate) {
							// start date is larger than end date, ignore end date
							dateRange.push(dateStart);
						} else {
							// range seems valid, build range array
							// first check if start or end are outside this month
							// if so, modify them so that they fit inside
							if ((dateStartDate.getMonth()) != parseInt(currentMonth - 1)) {
								dateStartDate.setMonth(parseInt(currentMonth - 1));
								dateStartDate.setDate(1);
							}
							if ((dateEndDate.getMonth()) != parseInt(currentMonth - 1)) {
								dateEndDate.setMonth(parseInt(currentMonth - 1));
								// sets date to last day of the month
								dateEndDate.setDate(new Date(currentYear, currentMonth, 0).getDate());
							}
							var datesInRange = dateEndDate.getDate() - dateStartDate.getDate();
							var firstDate = dateStartDate.getDate();
							if (datesInRange > 0) {
								// loop and build the array
								for (d = 0; d <= datesInRange; d++) {
									var currentDate = d + firstDate;
									if (currentDate.toString().length == 1)
										currentDate = '0' + currentDate.toString();
									var dateToAdd = currentYear + '-' + currentMonth + '-' + currentDate;
									dateRange.push(dateToAdd);
								}
							}
						}
					}
					// loop through date range and populate cells
					if ((dateRange) && (dateRange.length > 0)) {
						// if date range is larger than one, find the largest blank space contingent available and place it there
						var maxLevel = 0;
						if (dateRange.length > 1) {
							for (d = 0; d < dateRange.length; d++) {
								var date = dateRange[d];
								var cell = aCalendar.dateCell[date.substring(0,10)]; //substring - take out time from date/time string
								if ((cell.countElements > 0) && (maxLevel < cell.countElements)){
									maxLevel = cell.countElements;
								}
							}
						}
						for (d = 0; d < dateRange.length; d++) {
							var date = dateRange[d];
							// get cell reference
							var cell = aCalendar.dateCell[date.substring(0,10)]; //substring - take out time from date/time string
							if ((cell) && (cell.countElements < 10) && (date.substring(0,8) == aCalendar.datePrefix)) {
								// use leveling if this is range
								if ((dateRange.length > 1) && (maxLevel > 0)){
									var loopCount = (maxLevel - cell.countElements);
									for (f = 0; f < loopCount; f++) {
										dummyCell = aCellNode.cloneNode (true);
										probe (dummyCell);
										cell.body.appendChild (dummyCell);
										var h = 0;
										while ($('calendar_view_cell_record' + h) != null)
											h++;
										dummyCell.id = 'calendar_view_cell_record' + h;
											
										setStyle (dummyCell, 'display', 'block');
										cell.countElements++;
									}
								}
	
								// get cell template
								recordCell = aCellNode.cloneNode (true);
								probe (recordCell);
								cell.body.appendChild (recordCell);
								// unique id
								var h = 0;
								while ($('calendar_view_cell_record' + h) != null)
									h++;
								recordCell.id = 'calendar_view_cell_record' + h;

								setStyle (recordCell, 'display', 'block');
								recordCell.owner = aCalendar;
								// add to the list of records for both cell and entire object
								cell.records.push (recordCell);
								aCalendar.records.push (recordCell);
								cell.countElements++;
								// assign value
								var contents = recordCell.m.value; 
								// only if it's first record in the week
								if ((dateRange.length == 1) || (d == 0) || (cell.rowPosition == 0)) {
									// if there are ddws, append the icon
									if (ddw) {
										aIcon = getTemplate ("opens_window_icon").cloneNode(true);
										aIcon.style.display = 'inline';
										//assign unique id
										aId = 1;
										while ($('opens_window_icon' + aId) != null) {
											aId++;
										}
										aIcon.id = 'opens_window_icon' + aId;
										recordCell.m.value.appendChild (aIcon);
									}
									if (valueAux1) {
										contents.innerHTML += value + ' ' + valueAux1;
									} else {
										contents.innerHTML += value;
									}
									recordCell.showValue = true;
								} else {
									recordCell.showValue = false;
								}
								recordCell.boundTo = aCalendar;
								recordCell.value = value + ' ' + valueAux1;
								if (valueAux1) {
									recordCell.value = value + ' ' + valueAux1;
								} else {
									recordCell.value = value;
								}

								recordCell.parent = cell;
								recordCell.ddw = ddw;
								recordCell.ddwType = ddwType;
								recordCell.recordId = aQuery.result[i].recordid;
								
								// ie fix for overflow
								// body width must be large enough
								if (browser.ie) {
								}

								// add it to the map as well for access by record number
								if (!aCalendar.recordsMap[recordCell.recordId]) {
									aCalendar.recordsMap[recordCell.recordId] = new Array();
									aCalendar.recordsMap[recordCell.recordId].selected = false;
								}
								aCalendar.recordsMap[recordCell.recordId].push(recordCell);
								// if bullets are disabled, hide them
								if (!aCalendar.displayBullets) {
									var cellBullet = recordCell.m.bullet;
									setStyle (cellBullet, 'display', 'none');
								}
								// if there is more than one record in range, apply different style
								if (dateRange.length == 1) {
									recordCell.rangeType = 'single';
								} else {
									if (d == 0) {
										recordCell.rangeType = 'start';
									} else if ((d + 1) == dateRange.length) {
										recordCell.rangeType = 'end';
									} else {
										recordCell.rangeType = 'middle';
									}
								}
								// record cell type
								recordCell.toString = function() { return '[object CalendarCell]';};
								// apply styles based on type
								recordCell.l.className = 'l calendar_view_l_close';
								recordCell.r.className = 'r calendar_view_r_close';
								recordCell.m.className = 'm';
								switch (recordCell.rangeType) {
									case "start":
										recordCell.r.className = 'r calendar_view_r_normal';
										break;
									case "end":
										recordCell.l.className = 'l calendar_view_l_normal';
										break;
									case "middle":
										recordCell.r.className = 'r calendar_view_r_normal';
										recordCell.l.className = 'l calendar_view_l_normal';
										break;
									}
								// set record size
								var cellWidth = recordCell.offsetWidth - recordCell.l.offsetWidth - recordCell.r.offsetWidth;
								setStyle (recordCell.m, 'width', cellWidth +'px');
								// enable extender if cell is too small
								if ((recordCell.offsetHeight * cell.countElements) > cell.body.offsetHeight) {
									setStyle (cell.header.extend, 'display', 'block');
								} else {
									setStyle (cell.header.extend, 'display', 'none');
								}
								// set cell mouse hover methods
								recordCell.onmouseout = function () {
									hideTooltip();
									/*
									// get all cells with the same record id
									var recordMap = this.owner.recordsMap[this.recordId];
									// change selection state
									if (recordMap.selected == true) {
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											// apply styles based on type
											recordMap[i].l.className = 'l calendar_view_l_close_selected';
											recordMap[i].r.className = 'r calendar_view_r_close_selected';
											recordMap[i].m.className = 'calendar_view_m_selected';
											switch (recordMap[i].rangeType) {
												case "start":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													break;
												case "end":
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
												case "middle":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
											}
										}
									} else {
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											// apply styles based on type
											recordMap[i].l.className = 'l calendar_view_l_close';
											recordMap[i].r.className = 'r calendar_view_r_close';
											recordMap[i].m.className = 'm';
											switch (recordMap[i].rangeType) {
												case "start":
													recordMap[i].r.className = 'r calendar_view_r_normal';
													break;
												case "end":
													recordMap[i].l.className = 'l calendar_view_l_normal';
													break;
												case "middle":
													recordMap[i].r.className = 'r calendar_view_r_normal';
													recordMap[i].l.className = 'l calendar_view_l_normal';
													break;
											}
										}										
									}
									*/
								}
								recordCell.onmouseover = function () {
									var objectPos = getObjectPosition(this);
									showTooltip(this.value, objectPos.x, objectPos.y);
									/*
									var recordMap = this.owner.recordsMap[this.recordId];
									// change selection state
									if (recordMap.selected == true) {
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											// apply styles based on type
											recordMap[i].l.className = 'l calendar_view_l_close_selected';
											recordMap[i].r.className = 'r calendar_view_r_close_selected';
											recordMap[i].m.className = 'calendar_view_m_selected';
											switch (recordMap[i].rangeType) {
												case "start":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													break;
												case "end":
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
												case "middle":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
											}
										}
									} else {
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											// apply styles based on type
											recordMap[i].l.className = 'l calendar_view_l_close';
											recordMap[i].r.className = 'r calendar_view_r_close';
											recordMap[i].m.className = 'm';
											switch (recordMap[i].rangeType) {
												case "start":
													recordMap[i].r.className = 'r calendar_view_r_normal';
													break;
												case "end":
													recordMap[i].l.className = 'l calendar_view_l_normal';
													break;
												case "middle":
													recordMap[i].r.className = 'r calendar_view_r_normal';
													recordMap[i].l.className = 'l calendar_view_l_normal';
													break;
											}
										}										
									}
									*/
								}
								// on click, select or deselect record
								recordCell.onclick = function (event) {
									xEvent = xpEvent (event, true);
									// dev defined action
									this.owner.onRecordClick(this);
									// if multiple selection is not allowed, deselect all other records first
									if (!this.owner.selectMultipleRecords) {
										for (var k in this.boundTo.recordsMap) {
											if (this.boundTo.recordsMap[k].selected == true)
												this.boundTo.recordsMap[k].selected = false;
											for (l = 0; l < this.boundTo.recordsMap[k].length; l++) {
												// apply styles based on type
												this.boundTo.recordsMap[k][l].l.className = 'l calendar_view_l_close';
												this.boundTo.recordsMap[k][l].r.className = 'r calendar_view_r_close';
												this.boundTo.recordsMap[k][l].m.className = 'm';
												switch (this.boundTo.recordsMap[k][l].rangeType) {
													case "start":
														this.boundTo.recordsMap[k][l].r.className = 'r calendar_view_r_normal';
														break;
													case "end":
														this.boundTo.recordsMap[k][l].l.className = 'l calendar_view_l_normal';
														break;
													case "middle":
														this.boundTo.recordsMap[k][l].r.className = 'r calendar_view_r_normal';
														this.boundTo.recordsMap[k][l].l.className = 'l calendar_view_l_normal';
														break;
												}
											}
										}
									}

									// get all cells with the same record id
									var recordMap = this.owner.recordsMap[this.recordId];
									// change selection state
									if (recordMap.selected == true) {
										recordMap.selected = false;
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											//recordMap[i].className = 'calendar_view_cell_record';
										}
									} else {
										recordMap.selected = true;
										// loop and change selection style
										for (i = 0; i < recordMap.length; i++) {
											// apply styles based on type
											recordMap[i].l.className = 'l calendar_view_l_close_selected';
											recordMap[i].r.className = 'r calendar_view_r_close_selected';
											recordMap[i].m.className = 'calendar_view_m_selected';
											switch (recordMap[i].rangeType) {
												case "start":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													break;
												case "end":
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
												case "middle":
													recordMap[i].r.className = 'r calendar_view_r_normal_selected';
													recordMap[i].l.className = 'l calendar_view_l_normal_selected';
													break;
											}										
										}										
									}
									if (event) {
										if ((event.ctrlKey) || (event.metaKey)) {
											// CTRL key is pressed, open DDW if it exists
											if (this.ddwType.indexOf ("Static") != -1) {
												new DDW(this.ddw, 'static_url', this.value, '&recordId=' + this.recordId);
											} else if (this.ddwType.indexOf ("Dynamic") != -1) {
												new DDW(this.ddw, 'dynamic_url', this.value, '&recordId=' + this.recordId);
											} else if (this.ddwType.indexOf ("Message") != -1) {
												new DDW(this.ddw, 'message_window', this.boundTo.container, '&recordId=' + this.recordId);
											}										
											return;
										}
									}
								}
								// on doubleclick, open the editor
								recordCell.ondblclick = function()
								{
									// dev defined action
									this.owner.onRecordDblClick(this);
									if (this.boundTo.table.fieldsDetail.length == 0)
										return false;
									
									// check if editor exists, if not create it
									if (isDefined(this.owner.editor) && isDefined(this.owner.editor.record)) {
										// exists
										this.owner.editor.edit();
									} else {
										// doesn't exist
										this.owner.getEditor();
										setStyle(this.owner.editor.getDOM(), "display", "block");
										var editor = this.owner.editor;
										var auto = editor.prv.mAutoReload;
										if (auto > 0) {
					                      setTimeout(function() {editor.reload();}, 500);
										}
									}
									// call editor and display data
									this.owner.editor.pull(this.owner.table.tableid, this.recordId);
								};						
								// update cell contents height
								//cell.contentsHeight = cell.contentsHeight + cell.offsetHeight;
								// check if the added element overflows the cell
								// if so, set the flag
								//if (cell.contentsHeight > cell.offsetHeight) {		
								//	cell.showOverflow = true;
								//}
							}
						}
					}
				}
			}
		}
	
		// check if it is last batch, if not ask for next one
		if (!(aQuery.lastRecord)) {
			aQuery.nextBatch(aCalendar.populate_handler);
		} else {
			// last record, clear query
			clearQuery (aQuery.queryId, false);
			if (aCalendar.toolbar)
				setStyle(spinner, "visibility", "hidden");
			// setStyle(aCalendar.toolbar.spinner, "display", "none");
			
			// refresh calendar as if window is resized
			aCalendar.resizing = true;
			aCalendar.calendar.afterRefresh();
			aCalendar.resizing = false;
			if (aCalendar.refreshTimeout)
				window.clearTimeout(aCalendar.refreshTimeout);			
			aCalendar.refreshTimeout = setTimeout(function() { aCalendar.refresh(); }, 5000 * 60);	
			aCalendar.clearupdateTimeout = setTimeout(function() { aCalendar.updating = false; }, 1000);	
			//aCalendar.updating = false;			
		}
	}
	
	// on load initialization
	this.refresh();
	if (this.tabsbar)
		this.tabsbar.refresh();
	aThisObject = this;
	if (!this.initializing)
		setTimeout(function() { aThisObject.refreshDynamicTabs(); }, 5000);

	// window border bug fix
	if (aWindow)
		aWindow.resize (1,1);
		
	this.loaded = true;
}