// common functionality for data views

function dax_dataCore(selection){

	/*
	// Properties
	*/

	var selection = getTable(selection);
	
	if (!selection){
		debugAlert (selection + ' selection not found.');
		return null;
	}

	// communication with 4D
	this.data = {
		preInitialized		: false,			// preparations before field list is requested
		initialized			: false,			// check if object is setup for specific table
		firstLoad			: false,			// first batch of data is not succesfully loaded yet
		selection			: selection,		// selection object
		currentQuery		: {					// keep track of current preset query
			type			: null,				// dynamic, static, allrecords, custom
			name			: null,
			fieldId			: null,				// dynamic queries are separated by field id
			custom			: {					// when building custom queries
				field		: [],
				operator	: [],
				value		: [],
				andorflag	: []
			}
		},
		defaultQuery		: {					// query to fall back to if no preset query is used, allrecords by default
			type			: null,				// dynamic, static, allrecords
			name			: null,
			fieldId			: null				// dynamic queries are separated by field id
		},
		query				: new dax_bridge.query(selection.tableid),		// query object
		populating			: false,
		selectedRecords		: {
			count			: 0
		},
		presetQueryMapSidebar	: {				// maps to existing preset queries in bridge by field id ([x][y])
			staticQueries		: null,
			savedQueries		: null
		},
		presetQueryMapTabs		: {
			staticQueries		: null,		
			savedQueries		: null
		},
		savedQueries		: {
			queries			: [ ],
			sidebarBlock	: null				// dom element for saved queries block
		},
		refreshInterval		: 180,				// in seconds
		refreshEnabled		: true,				// use auto refresh every .refreshInterval seconds?
		searchValue			: null				// link to live search input box
	};

	// toolbars
	this.toolbars = {
		main	: null
	};

	this.sidebar = {
		initialized			: false,
		active				: null,		// active sidebar gets refreshed and redrawn when needed
		mode				: null,		// possible values: presetQuery
		viewport			: null,
		presetQueries		: null,
		header				: null,
		viewportContainer	: null,
		spinner				: null
	};
	
	// query
	this.data.query.handler = this.dataHandler;	
	
	// called after list of fields is received from the BE
	function dax_dataCore_fieldUpdateHandler(dataObject) {
		// finish initialization
		// was used in async get field
		// dataObject.go();
		
		// update toolbar search list if one is present
		if (dataObject.updateSearchFieldList)
			dataObject.updateSearchFieldList();
	}
	
	this.fieldUpdateHandler = dax_dataCore_fieldUpdateHandler;
	
	// loads the data view
	function dax_dataCore_go() {
		if (this.data.initialized)
			return false;
			
		if (!this.data.preInitialized) {
			this.data.preInitialized = true;
			this.preInitialize();	
		}
			
		// check if field list is present, if not place async call for it
		// still sync for now
		if (!this.data.selection.cached) {
			//this.activate();
			dax_bridge.cacheFields(this.data.selection);
			this.fieldUpdateHandler(this);
			//dax_bridge.updateFieldsAsync(this.data.selection.tableid, this, this.fieldUpdateHandler);
			//return true;
		}	
		
		// function in data view
		this.initialize();
		this.data.initialized = true;

		var dataObject = this;
		
		// redraw tabs, if they are added
		if (this.tabs)
			this.tabs.redraw();
		
		// determine which records to show on load
		if (this.data.selection.showallrecordstab.toLowerCase() == 'true') {

			// 'all records' is enabled, no further action required
			this.data.defaultQuery.type = 'allrecords';
			this.data.currentQuery.type = 'allrecords';
			
			// start data population
			setTimeout(function() { dataObject.populate(); }, 500 );
			
		} else if (this.data.selection.preselectQueries.length > 0) {

			// static query
			this.data.defaultQuery.type = 'static';
			this.data.defaultQuery.name = this.data.selection.preselectQueries[0].name;

			// start data population
			setTimeout(function() { dataObject.runPresetQuery(dataObject.data.defaultQuery.name); }, 500 );
			
		} else if (this.data.selection.preselectedDynamicQueries.length > 0) {

			// dynamic query
			// data will be populated when 4D returns list of all dynamic queries
			this.data.defaultQuery.type = 'dynamic';
			
		}
	}	

	this.go = dax_dataCore_go;
	
	// clear the query and start a new one
	function dax_dataCore_newQuery() {
		// clear current query
		this.data.currentQuery.custom = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}	
	}
	
	this.newQuery = dax_dataCore_newQuery;

	// add a query parameter
	function dax_dataCore_addQuery(fieldName, operator, value, andorflag) {

		// get field id, return if no field id is found
		var field = getField(this.data.selection.tableid, fieldName);
		if (!field)
			return false;
		else
			fieldId = field.fieldid;
			
		// default operator is equal
		switch (operator) {
			case '#':
				var operator = 'notequal';
			break;
			case '<':
				var operator = 'less';
			break;
			case '>':
				var operator = 'greater';
			break;
			case '<=':
				var operator = 'lesseq';
			break;
			case '>=':
				var operator = 'greatereq';
			break;
			case '=':
			default:
				var operator = 'equal';
			break;
		}

		// encode query value
		var queryValue = dax_bridge.encode(value);
		
		// andorflag can be 'and' / 'or', default value is 'and'
		if (!andorflag) {
			var andOrFlag = 'and';
		} else if (andorflag == 'or') {
			var andOrFlag = 'or';
		} else {
			var andOrFlag = 'and';
		}
		
		this.data.currentQuery.custom.field.push(fieldId);
		this.data.currentQuery.custom.operator.push(operator);
		this.data.currentQuery.custom.value.push(queryValue);
		this.data.currentQuery.custom.andorflag.push(andOrFlag);
		
	}
	
	this.addQuery = dax_dataCore_addQuery;

	// runs a query from parameters set by .runQuery
	function dax_dataCore_runQuery() {
		// clear existing query
		this.data.query.queryEngine = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}	
	
		// clear query id
		this.data.query.queryId = null;
		this.data.query.currentQueryId = null;
	
		// loop through elements and add them to the query object
		var currentQuery = this.data.currentQuery.custom;
		for (var queryElementCount = 0; queryElementCount < currentQuery.field.length; queryElementCount++) {
			this.data.query.queryEngine.field.push(currentQuery.field[queryElementCount]);
			this.data.query.queryEngine.operator.push(currentQuery.operator[queryElementCount]);
			this.data.query.queryEngine.value.push(currentQuery.value[queryElementCount]);
			this.data.query.queryEngine.andorflag.push(currentQuery.andorflag[queryElementCount]);
		}
		
		this.populate();
	}
	
	this.runQuery = dax_dataCore_runQuery;
	
	// form values sent to 4D, 4D gets them via GET WEB FORM VARIABLES 
	function dax_dataCore_addCustomValue(customName, customValue) {
		this.data.query.customValues.name.push(customName);
		this.data.query.customValues.value.push(customValue);
	}
	
	this.addCustomValue = dax_dataCore_addCustomValue;
	
	// clear custom values
	function dax_dataCore_clearCustomValues() {
		this.data.query.customValues = {
			name	: [],
			value	: []
		}	
	}
	
	this.clearCustomValues = dax_dataCore_clearCustomValues;
	
	function dax_dataCore_clearSearch() {
		this.data.query.queryEngineSearch = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}
		
		if (this.data.searchValue)
			this.data.searchValue.value = '';
			
		if (this.data.advancedsearchvalues) {
			this.data.advancedsearchvalues = null;
					
			// Deleting advanced toolbars
			for (var i=0; i < this.toolbars.advancedsearch.length; i++) {
				this.window.removeToolbar(this.toolbars.advancedsearch[i].prv.mDOM);	
			}
			this.toolbars.advancedsearch = null;
			
			if (this.data.initialized) {
				this.activate();
			}
		}	
		
		// Reformat the main toolbar
		if (this.toolbars.main) {
			this.toolbars.main.search.saveButton.prv.mDOM.className = 'toolbarbutton dax_toolbarbutton_searchset dax_toolbarbutton_join_left';
			this.toolbars.main.search.moreButton.prv.mDOM.style.display = "block";
			this.toolbars.main.search.moreButton.prv.mDOM.m.label.firstChild.nodeValue = "Advanced";
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').style.display = "none";
		}

	}
	
	this.clearSearch = dax_dataCore_clearSearch;
	
	// runs preset query by name and selects corresponding tab and  element
	function dax_dataCore_runPresetQuery(queryName, fieldName, savedQueryRef) {

		// clear search engine
		this.data.query.queryEngine = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}	
	
		this.clearSearch();
	
		if (savedQueryRef) {
		
			// saved query
			this.data.currentQuery.type = 'saved';
			this.data.currentQuery.name = null;
			this.data.currentQuery.queryId = savedQueryRef.queryId;

			this.data.query.queryId = savedQueryRef.queryId;

		} else if (!fieldName) {
			// static query
			this.data.currentQuery.type = 'static';
			this.data.currentQuery.name = queryName;

			this.data.query.setNamedQuery(dax_bridge.encode(queryName));
			
		} else {

			// dynamic query

			var fieldId = getField(this.data.selection.tableid, fieldName).fieldid;
			
			this.data.currentQuery.type = 'dynamic';
			this.data.currentQuery.name = queryName;
			this.data.currentQuery.fieldId = fieldId;
			
			this.data.query.setNamedQuery('4daf_unique_' + dax_bridge.encode(queryName));
		}
		
		// select sidebar element

		if (this.sidebar.initialized) {
			if (this.sidebar.presetQueries.currentDom) {
				this.sidebar.presetQueries.currentDom.className = 'dax_sidebar_presetquery_element';
			}
			
//			this.sidebar.presetQueries.current = queryName;
			if (savedQueryRef)
				var currentDom = savedQueryRef.sidebarRef;
			else if (!fieldId)
				var currentDom = this.data.presetQueryMapSidebar.staticQueries.elements[queryName].node;
			else
				var currentDom = this.data.presetQueryMapSidebar[fieldId].elements[queryName].node;
			
			currentDom.className = 'dax_sidebar_presetquery_element dax_sidebar_presetquery_element_selected';
			this.sidebar.presetQueries.currentDom = currentDom;
		}
		
		if (this.tabs) {
			
			if (savedQueryRef)
				this.tabs.selectTabByNode(savedQueryRef.tabRef);
			else if (!fieldId)
				this.tabs.selectTabByNode(this.data.presetQueryMapTabs.staticQueries.elements[queryName].node);
			else
				this.tabs.selectTabByNode(this.data.presetQueryMapTabs[fieldId].elements[queryName].node);
			
		
		}
		
		this.populate();
	}
	
	this.runPresetQuery = dax_dataCore_runPresetQuery;
	
	function dax_dataCore_dataHandler(http_response, refDataObject) {

		if (http_response.readyState != 4)
			return;

		if (checkResponse(http_response))
			return false;
			
		var queryResult = dax_bridge.parseQuery(http_response, refDataObject.data.query);

		refDataObject.onBeforeDataDisplay();
		
		// update status bar
		refDataObject.updateStatusMessage();
		
		refDataObject.onpopulate();	
		
		if (!refDataObject.data.firstLoad)
			refDataObject.data.firstLoad = true;
			
		// start the refresh timer
		var dataObject = refDataObject;
		if (refDataObject.dataRefreshTimeout)
			window.clearTimeout(refDataObject.dataRefreshTimeout);
		if (dataObject.data.refreshEnabled)
			dataObject.dataRefreshTimeout = window.setTimeout(function() {dataObject.populate();}, +dataObject.data.refreshInterval * 1000);

	}
	
	this.dataHandler = dax_dataCore_dataHandler;

	// triggers an instant preset query update
	function dax_dataCore_updatePresetQueries() {

		var dataObject = this;
		dax_bridge.updateUniqueQueries(dataObject.data.selection.tablename, dataObject.uniqueQueryHandler, dataObject);
	
	}
	
	this.updatePresetQueries = dax_dataCore_updatePresetQueries;
	
	// set refresh interval in seconds
	function dax_dataCore_setRefreshInterval(seconds) {
		this.data.refreshInterval = seconds;		
	}
	
	this.setRefreshInterval = dax_dataCore_setRefreshInterval;
	
	// enable auto data refresh every .data.refreshInterval seconds
	function dax_dataCore_enableAutoRefresh() {
		if (!this.data.refreshEnabled) {
			this.data.refreshEnabled = true;
			this.populate();
		}
	}
	
	this.enableAutoRefresh = dax_dataCore_enableAutoRefresh;

	// disable auto data refresh every .data.refreshInterval seconds
	function dax_dataCore_disableAutoRefresh() {
		this.data.refreshEnabled = false;
	}
	
	this.disableAutoRefresh = dax_dataCore_disableAutoRefresh;

	
	// starts refresh timer for preset queries, disables one if tabs & preset query sidebar is not visible
	function dax_dataCore_uniqueQueryRefresh() {

		if ((this.tabs) || (this.sidebar.mode == 'presetQuery')) {
			var dataObject = this;
			if (this.presetQueryTimeout)
				window.clearTimeout(this.presetQueryTimeout);
			this.presetQueryTimeout = window.setTimeout(function() {dax_bridge.updateUniqueQueries(dataObject.data.selection.tablename, dataObject.uniqueQueryHandler, dataObject);}, 1000 * 3 * 60);
		}
	}
	
	this.uniqueQueryRefresh = dax_dataCore_uniqueQueryRefresh;
	
	// called when preset queries are updated
	function dax_dataCore_uniqueQueryHandler(objectRef) {

		// do another refresh if there is visible sidebar with preset queries or tabs
		var refreshAgain = false;
	
		if (objectRef.sidebar.mode == 'presetQuery') {
			objectRef.updatePresetQueriesSidebarHandler();
			refreshAgain = true;
		}
			
		if (objectRef.tabs) {
			objectRef.updateTabs();
			refreshAgain = true;
		}

		// if this is first dynamic query call, and there are no other queries to display (including all records), then display first dynamic query
		if ((!objectRef.data.currentQuery) && (objectRef.data.defaultQuery.type == 'dynamic') && (!objectRef.data.defaultQuery.name)) {
			objectRef.runPresetQuery(objectRef.data.selection.preselectedDynamicQueries[0].names[0], getField(objectRef.data.selection.preselectedDynamicQueries[0].fieldId).fieldname);
		}
		
		if (refreshAgain)
			objectRef.uniqueQueryRefresh();
	
	}
	
	this.uniqueQueryHandler = dax_dataCore_uniqueQueryHandler;
	
	// show tabs wrapper
	function dax_dataCore_showQueryTabs() {
		this.updateTabs();
	}
	
	this.showQueryTabs = dax_dataCore_showQueryTabs;
	
	// wrapper for maximum number of characters
	function dax_dataCore_querySetMaxChar(numberOfCharacters) {
		this.data.query.maxChar = numberOfCharacters;
	}
	
	this.querySetMaxChar = dax_dataCore_querySetMaxChar;
	
	function dax_dataCore_updatePresetQueriesSidebarHandler() {
	
		var preselectQueries = this.data.selection.preselectQueries;
		var preselectedDynamicQueries = this.data.selection.preselectedDynamicQueries;
		
		if (this.sidebar.mode != 'presetQuery') {

			var titleNode = getTemplate('dax_sidebar_presetquery');
			
			var sidebarContents = getTemplate('dax_sidebar_presetquery_container').cloneNode(true);
			generateUniqueId(sidebarContents);
			sidebarContents.style.display = 'block';

			this.sidebar.viewport.setContents(sidebarContents);
			
			this.sidebar.presetQueries = {
				current		: null,
				initialized	: true
			};
			
			var sidebarRefreshButton = getTemplate('dax_datacore_sidebar_refresh');
			generateUniqueId(sidebarRefreshButton);
			sidebarRefreshButton.style.display = 'block';
			sidebarRefreshButton.boundTo = this;
			
			this.sidebar.presetQueries.refreshButton = sidebarRefreshButton;
			getElementByClassName(this.sidebar.header, 'dax_datacore_sidebar_buttons').appendChild(sidebarRefreshButton);
			getElementByClassName(this.sidebar.header, 'dax_datacore_sidebar_title').innerHTML = 'Queries';
			
			sidebarRefreshButton.onclick = function (event) {
				
				xpEvent(event, true);
				this.boundTo.updatePresetQueries();
			}
			
			this.sidebar.mode = 'presetQuery';
			
		} else {
			sidebarContents = this.sidebar.viewport.contents;
		}

		function dax_dataCore_sidebar_presetQueryHeaderOnClick(event) {

			xpEvent(event, true);
			var isCollapsed = (this.contentsNode.style.display == 'none');
			if (isCollapsed) {
				this.contentsNode.style.display = 'block';
				getElementByClassName(this,'dax_sidebar_presetquery_arrow_collapsed').className = 'dax_sidebar_presetquery_arrow_expanded';
			} else {
				this.contentsNode.style.display = 'none';
				getElementByClassName(this,'dax_sidebar_presetquery_arrow_expanded').className = 'dax_sidebar_presetquery_arrow_collapsed';
			}
			this.boundTo.sidebar.viewport.redraw();
			return false;
		
		}

		function dax_dataCore_sidebar_presetQueryElementOnClick(event) {
		
			xpEvent(event, true);
			this.boundTo.runPresetQuery(this.queryName, this.queryFieldId, this.savedQueryRef);
			return false;
		
		}
		
		// check for static queries first, this will be executed only once
		if ((!this.data.presetQueryMapSidebar.staticQueries) && (preselectQueries.length > 0)) {

			var elementNode = getTemplate('dax_sidebar_presetquery_element');
		
			this.data.presetQueryMapSidebar.staticQueries = {
				node 			: currentNode,
				elements		: new Object()
			}
		
			var currentNode = titleNode.cloneNode(true);
			var contentsNode = getElementByClassName(currentNode, 'dax_sidebar_presetquery_contents');

			generateUniqueId(currentNode);
			currentNode.style.display = 'block';
			currentNode.boundTo = this;
			currentNode.onclick = dax_dataCore_sidebar_presetQueryHeaderOnClick;
			currentNode.contentsNode = contentsNode;
			currentNode.contentsNode.style.display = 'none';
			
			getElementByClassName(currentNode, 'dax_sidebar_presetquery_label').innerHTML = 'APPLICATION';
			
			sidebarContents.appendChild(currentNode);
		
			for (var queryCount = 0; queryCount < preselectQueries.length; queryCount++) {
			
				var currentChildNode = elementNode.cloneNode(true);
				
				generateUniqueId(currentChildNode);
				currentChildNode.style.display = 'block';
				
				var labelNode = getElementByClassName(currentChildNode, 'dax_sidebar_presetquery_element_label');
				
				labelNode.innerHTML = preselectQueries[queryCount].name;
				currentChildNode.queryName = preselectQueries[queryCount].name;
				currentChildNode.onclick = dax_dataCore_sidebar_presetQueryElementOnClick;
				currentChildNode.boundTo = this;
				currentChildNode.queryType = 'static';
				currentChildNode.queryFieldId = null;
				
				contentsNode.appendChild(currentChildNode);

				this.data.presetQueryMapSidebar.staticQueries.elements[preselectQueries[queryCount].name] = {
					name	: elementName,
					erase	: false,
					node	: currentChildNode
				};							
			}
		}
		
		// saved queries area
		if(!this.data.savedQueries.sidebarBlock) {
		
			var currentNode = titleNode.cloneNode(true);
			var contentsNode = getElementByClassName(currentNode, 'dax_sidebar_presetquery_contents');

			generateUniqueId(currentNode);
			currentNode.boundTo = this;
			currentNode.onclick = dax_dataCore_sidebar_presetQueryHeaderOnClick;
			currentNode.contentsNode = contentsNode;
			currentNode.contentsNode.style.display = 'none';
			
			getElementByClassName(currentNode, 'dax_sidebar_presetquery_label').innerHTML = 'SAVED';
			
			sidebarContents.appendChild(currentNode);
			
			this.data.savedQueries.sidebarBlock = currentNode;
		
		}	
		
		// create new sidebar elements for saved queries that don't have corresponding element yet
		if (this.data.savedQueries.queries.length > 0) {
	
			// show saved queries area
			this.data.savedQueries.sidebarBlock.style.display = 'block';
			
			for (var queryCount = 0; queryCount < this.data.savedQueries.queries.length; queryCount++) {
				// check if reference is missing, this means that corresponding sidebar element was not created
				if (!this.data.savedQueries.queries[queryCount].sidebarRef) {

					// create corresponding node
					var elementNode = getTemplate('dax_sidebar_presetquery_element');
					var currentChildNode = elementNode.cloneNode(true);
					
					generateUniqueId(currentChildNode);
					currentChildNode.style.display = 'block';
					
					var labelNode = getElementByClassName(currentChildNode, 'dax_sidebar_presetquery_element_label');
					
					labelNode.innerHTML = this.data.savedQueries.queries[queryCount].name;
					currentChildNode.queryId = this.data.savedQueries.queries[queryCount].queryId;
					currentChildNode.queryName = null;
					currentChildNode.onclick = dax_dataCore_sidebar_presetQueryElementOnClick;
					currentChildNode.savedQueryRef = this.data.savedQueries.queries[queryCount];
					currentChildNode.boundTo = this;
					currentChildNode.queryType = 'saved';
					currentChildNode.queryFieldId = null;
					
					var closeButton = getElementByClassName(currentChildNode, 'dax_sidebar_presetquery_element_close');
					
					function dax_dataCore_presetQuerySidebar_removeSavedQuery(event) {
						xpEvent(event, true);
						
						this.boundTo.boundTo.deleteSearch(this.boundTo.savedQueryRef);
					}					

					closeButton.style.visibility = 'visible';
					closeButton.boundTo = currentChildNode;
					closeButton.onclick = dax_dataCore_presetQuerySidebar_removeSavedQuery;
					
					this.data.savedQueries.sidebarBlock.contentsNode.appendChild(currentChildNode);
					this.data.savedQueries.queries[queryCount].sidebarRef = currentChildNode;
				
				}	
			}
	
			
			
		} else {

			// no saved queries, hide the area
			this.data.savedQueries.sidebarBlock.style.display = 'none';
		}

		
		// check for dynamic queries
		if (preselectedDynamicQueries.length > 0) {
		
			for (var queryElementCount = 0; queryElementCount < preselectedDynamicQueries.length; queryElementCount++) {
			
				var queryElement = preselectedDynamicQueries[queryElementCount];

				// create container node if it doesn't exist
				if (!this.data.presetQueryMapSidebar[queryElement.fieldId]) {

					var currentNode = titleNode.cloneNode(true);
					currentNode.boundTo = this;
					currentNode.onclick = dax_dataCore_sidebar_presetQueryHeaderOnClick;
					var contentsNode = getElementByClassName(currentNode, 'dax_sidebar_presetquery_contents');
					
					generateUniqueId(currentNode);
					currentNode.style.display = 'block';
					currentNode.contentsNode = contentsNode;
					currentNode.contentsNode.style.display = 'none';
					
					getElementByClassName(currentNode, 'dax_sidebar_presetquery_label').innerHTML = getField(queryElement.fieldId).fieldalias.toUpperCase();
					
					sidebarContents.appendChild(currentNode);
				
					this.data.presetQueryMapSidebar[queryElement.fieldId] = {
						node		: currentNode,
						elements	: new Object()
					}
				} else {
					currentNode = this.data.presetQueryMapSidebar[queryElement.fieldId].node;
					var contentsNode = getElementByClassName(currentNode, 'dax_sidebar_presetquery_contents');
					
					// mark all elements for erasing, existing elements will be untagged so we can erase non-existing ones at the end
					for (var currentElement in this.data.presetQueryMapSidebar[queryElement.fieldId].elements) {
						this.data.presetQueryMapSidebar[queryElement.fieldId].elements[currentElement].erase = true;					
					}
				}
				
				if (queryElement.names.length != 0) {

					currentNode.style.display = 'block';
					
					for (var queryCount = 0; queryCount < queryElement.names.length; queryCount++) {

						var elementName = queryElement.names[queryCount];
					
						// create element if it doesn't exist
						if (!this.data.presetQueryMapSidebar[queryElement.fieldId].elements[elementName]) {

							if (!elementNode)
								var elementNode = getTemplate('dax_sidebar_presetquery_element');

						
							var currentChildNode = elementNode.cloneNode(true);
							
							generateUniqueId(currentChildNode);
							currentChildNode.style.display = 'block';
							
							var labelNode = getElementByClassName(currentChildNode, 'dax_sidebar_presetquery_element_label');
							
							labelNode.innerHTML = elementName;
							currentChildNode.queryName = elementName;
							currentChildNode.onclick = dax_dataCore_sidebar_presetQueryElementOnClick;
							currentChildNode.boundTo = this;
							currentChildNode.queryType = 'dynamic';
							currentChildNode.queryFieldId = queryElement.fieldId;
							
							contentsNode.appendChild(currentChildNode);
							
							this.data.presetQueryMapSidebar[queryElement.fieldId].elements[elementName] = {
								name	: elementName,
								erase	: false,
								node	: currentChildNode
							};

						} else {
							this.data.presetQueryMapSidebar[queryElement.fieldId].elements[elementName].erase = false;
						}						
					}
					
					//erase non existing elements
					for (var currentElementName in this.data.presetQueryMapSidebar[queryElement.fieldId].elements) {
						
						var currentElement =  this.data.presetQueryMapSidebar[queryElement.fieldId].elements[currentElementName];
						
						if (currentElement.erase) {
							contentsNode.removeChild(currentElement.node);
							delete this.data.presetQueryMapSidebar[queryElement.fieldId].elements[currentElementName];
						}					
					}					

					
				} else {
					currentNode.style.display = 'none';
				}
				
			}
			
			if (this.sidebar.spinner)
				this.sidebar.spinner.style.display = 'none';

			if (this.sidebar.presetQueries.refreshButton)
				this.sidebar.presetQueries.refreshButton.style.display = 'block';
				
		}
		
		this.sidebar.viewport.redraw();
		
	}
	
	this.updatePresetQueriesSidebarHandler = dax_dataCore_updatePresetQueriesSidebarHandler;
	
	function dax_dataCore_updateStatusMessage() {
		var message = this.data.query.recordsInSelection + ' record(s) total';
		
		if (this.data.selectedRecords.count > 0)
			message += ', ' + this.data.selectedRecords.count + ' selected.';
		else
			message += '.';
		
		this.showStatusMessage(message);	
	}
	
	this.updateStatusMessage = dax_dataCore_updateStatusMessage;

	function dax_dataCore_showQuerySidebar(orientation) {

		// show sidebar if one is not up yet
		this.showSidebar(orientation);
	
		// update preset queries
		var preselectQueries = this.data.selection.preselectQueries;
		var preselectedDynamicQueries = this.data.selection.preselectedDynamicQueries;
		var dataObject = this;
		
		if (preselectedDynamicQueries.length > 0) {
			window.setTimeout(function() {dataObject.updatePresetQueries();}, 3000);
			//this.timeouts.presetQueries = timeoutId;
		}
			 
		this.updatePresetQueriesSidebarHandler();

	
	}
	
	this.showQuerySidebar = dax_dataCore_showQuerySidebar;

	
	function dax_dataCore_showSidebar(orientation) {
	
		// check if sidebar exists
		if (!this.sidebar.initialized) {

			var sidebarOrientation = (orientation == 'right') ? 'right' : 'left';
		
			this.window.addSidebar(sidebarOrientation);

			var sidebarHeader = getTemplate('dax_datacore_sidebar_header').cloneNode(true);
			generateUniqueId(sidebarHeader);
			sidebarHeader.style.display = 'block';

			var sidebarViewportArea = getTemplate('dax_datacore_sidebar_viewport').cloneNode(true);
			generateUniqueId(sidebarViewportArea);
			sidebarViewportArea.style.display = 'block';

			var sidebar = this.window.sidebar.dom;
			sidebar.appendChild(sidebarHeader);
			sidebar.appendChild(sidebarViewportArea);

			if (!this.sidebar.viewport) {
				this.sidebar.viewport = new dax_viewport(sidebarViewportArea);
			}

			this.sidebar.header = sidebarHeader;
			this.sidebar.viewportContainer = sidebarViewportArea;

			this.sidebar.spinner = getElementByClassName(this.sidebar.header, 'dax_datacore_sidebar_spinner');
			
			this.sidebar.viewportContainer.style.height = this.window.sidebar.dom.clientHeight - this.sidebar.header.clientHeight + 'px';
			
			var closeButton = getElementByClassName(this.sidebar.header, 'dax_datacore_sidebar_close');
			
			closeButton.boundTo = this;
			closeButton.onclick = function(event){
				xpEvent(event, true);
				this.boundTo.hideSidebar();
				return false;
			}
			
			this.sidebar.viewport.boundTo = this;
			this.sidebar.initialized = true;
			this.sidebar.active = true;

			this.window.resize();	
		}
	}
	
	this.showSidebar = dax_dataCore_showSidebar;
	
	function dax_dataCore_hideSidebar() {
		this.window.hideSidebar();
		this.sidebar.active = false;
		this.window.resize();
	}
	
	this.hideSidebar = dax_dataCore_hideSidebar;
	
	// saves current search as tab and sidebar item
	function dax_dataCore_saveSearch(searchName) {
	
		// add this to the list of saved queries
		var currentQuery = {
			tabRef		: null,
			sidebarRef	: null,
			queryId		: this.data.query.currentQueryId,
			erase		: false,
			name		: searchName
		}

		this.data.savedQueries.queries.push(currentQuery);

		// update tabs & sidebar
		if (this.sidebar.mode == 'presetQuery') {
			this.updatePresetQueriesSidebarHandler();
		}
	
		if (this.tabs) {
			this.updateTabs();
		}
	}
	
	this.saveSearch = dax_dataCore_saveSearch;
	
	// remove custom search and take out corresponding tab & sidebar element
	function dax_dataCore_deleteSearch(searchRef) {
		
		// remove tab element
		if (searchRef.tabRef)
			this.tabs.deleteTab(null, false, searchRef.tabRef);
			
		var updateSidebar = false;	
			
		// remove sidebar element
		if (searchRef.sidebarRef) {
			this.data.savedQueries.sidebarBlock.contentsNode.removeChild(searchRef.sidebarRef);
			var updateSidebar = true;
		}
		
		// erase saved query element
		for (var savedQueryCount = 0; savedQueryCount < this.data.savedQueries.queries.length; savedQueryCount++) {
			if (this.data.savedQueries.queries[savedQueryCount] == searchRef) {
				this.data.savedQueries.queries.splice(savedQueryCount, 1);
				break;
			}		
		}
		
		// update sidebar if needed, this will make sure that saved section is hidden if there are no elements left
		if (updateSidebar)
			this.updatePresetQueriesSidebarHandler();	

	}


	
	this.deleteSearch = dax_dataCore_deleteSearch;
	
	// populate with data
	function dax_dataCore_populate(startRecord){
		if (this.data.populating)
			return false;
		
		this.data.populating = true;
		this.updateQueryId = true;

		this.data.query.queryEngineSearch = {	
			field		: [],
			operator	: [],
			value		: [],
			andorflag	: []
		}	
		
		// check if there is a search value, if so do a query without updating query id
		if (!this.toolbars.advancedsearch) {
			if (this.data.searchValue) {
				if (this.data.searchValue.value) {
					var searchValue = this.data.searchValue.value;
					var searchField = getField(this.data.selection.tableid, this.toolbars.main.search.fieldList[this.toolbars.main.search.fieldList.selectedIndex].value);
					// if date is used, format it
					if (searchField.fieldtype == 'date') {
						var searchValue = doFormat(searchValue, searchField.fieldformat, 'YYYY-MM-DD');
						if (!searchValue) 
							doSearch = false;
					}
					
					this.data.query.updateQueryId = false;
					
					this.data.query.queryEngineSearch = {
						field: [searchField.fieldid],
						operator: ['starts with'],
						value: [dax_bridge.encode(searchValue)],
						andorflag: ['and']
					}
				}
			}
		}
		else {
			if (this.data.advancedsearchvalues)	{
				this.data.query.queryEngineSearch = {
					field: [],
					operator: [],
					value: [],
					andorflag: []
				}
				if (this.data.searchValue) {
					if (this.data.searchValue.value) {
						var searchValue = this.data.searchValue.value;
						var searchField = getField(this.data.selection.tableid, this.toolbars.main.search.fieldList[this.toolbars.main.search.fieldList.selectedIndex].value);
						var searchoperator = this.toolbars.main.search.operator[this.toolbars.main.search.operator.selectedIndex].value;
						var searchandor = this.toolbars.main.search.andor[this.toolbars.main.search.andor.selectedIndex].value;
						// if date is used, format it
						if (searchField.fieldtype == 'date') {
							var searchValue = doFormat(searchValue, searchField.fieldformat, 'YYYY-MM-DD');
							if (!searchValue) 
								doSearch = false;
						}
						
						this.data.query.updateQueryId = false;
						
						this.data.query.queryEngineSearch = {
							field: [searchField.fieldid],
							operator: [searchoperator],
							value: [dax_bridge.encode(searchValue)],
							andorflag: [searchandor]
						}
					}
				}
				for (var i = 0; i < this.data.advancedsearchvalues.length; i++) {
					if (this.data.advancedsearchvalues[i].value) {
						var searchValue = this.data.advancedsearchvalues[i].value;
						var searchField = getField(this.data.selection.tableid, this.toolbars.advancedsearch[i].search.fieldList[this.toolbars.advancedsearch[i].search.fieldList.selectedIndex].value);
						var searchoperator = this.toolbars.advancedsearch[i].search.operator[this.toolbars.advancedsearch[i].search.operator.selectedIndex].value;
						var searchandor = this.toolbars.advancedsearch[i].search.andor[this.toolbars.advancedsearch[i].search.andor.selectedIndex].value;
						// if date is used, format it
						if (searchField.fieldtype == 'date') {
							var searchValue = doFormat(searchValue, searchField.fieldformat, 'YYYY-MM-DD');
							if (!searchValue) 
								doSearch = false;
						}
						
						this.data.query.updateQueryId = false;
						
						this.data.query.queryEngineSearch.field.push(searchField.fieldid);
						this.data.query.queryEngineSearch.operator.push(searchoperator);
						this.data.query.queryEngineSearch.value.push(dax_bridge.encode(searchValue));
						this.data.query.queryEngineSearch.andorflag.push(searchandor);
					}
				}
			}
			else {
				if (this.data.searchValue) {
					if (this.data.searchValue.value) {
						var searchValue = this.data.searchValue.value;
						var searchField = getField(this.data.selection.tableid, this.toolbars.main.search.fieldList[this.toolbars.main.search.fieldList.selectedIndex].value);
						var searchoperator = this.toolbars.main.search.operator[this.toolbars.main.search.operator.selectedIndex].value;
						// if date is used, format it
						if (searchField.fieldtype == 'date') {
							var searchValue = doFormat(searchValue, searchField.fieldformat, 'YYYY-MM-DD');
							if (!searchValue) 
								doSearch = false;
						}
						
						this.data.query.updateQueryId = false;
						
						this.data.query.queryEngineSearch = {
							field: [searchField.fieldid],
							operator: [searchoperator],
							value: [dax_bridge.encode(searchValue)],
							andorflag: ['and']
						}
					}
				}	
			}
		}	
		
		this.data.query.handler = this.dataHandler;
		this.data.query.objectRef = this;
		this.data.query.getRecords(startRecord, this.getVisibleRecordCount());		
	}

	this.populate = dax_dataCore_populate;
	
	function dax_dataCore_addRecordToSelection(recordId) {
	
		this.data.selectedRecords[recordId] = true;
		this.data.selectedRecords.count++;
	
		this.updateStatusMessage();
	
	}
	
	this.addRecordToSelection = dax_dataCore_addRecordToSelection;

	function dax_dataCore_removeRecordFromSelection(recordId) {
		
		if (this.data.selectedRecords[recordId]) {
			delete this.data.selectedRecords[recordId];
			this.data.selectedRecords.count--;
			this.updateStatusMessage();			
		}
	}
	
	this.removeRecordFromSelection = dax_dataCore_removeRecordFromSelection;
	
	function dax_dataCore_removeAllRecordsFromSelection() {
		this.data.selectedRecords = new Object();
		this.data.selectedRecords.count = 0;
		this.updateStatusMessage();			
	}
	
	this.removeAllRecordsFromSelection = dax_dataCore_removeAllRecordsFromSelection;
	
	this.onBeforeSort = function() { };
	this.onAfterSort = function() { };
	
	this.onBeforeSortInt = function() { };
	this.onAfterSortInt = function() { };
	
	this.onBeforeDataDisplay = function() { };
	this.onDataLoad = function() { };
	
	function dax_dataCore_showStatusMessage(message) {
		var messageNode = getElementByClassName(this.window.status, 'window_status_message');
		messageNode.innerHTML = message;
		messageNode.style.display = 'block';
	
	}
	
	this.showStatusMessage = dax_dataCore_showStatusMessage;
	
	function dax_dataCore_sort(field, order) {

		this.onBeforeSortInt();
		this.onBeforeSort();

		this.data.query.sort(field, order, true);	
	}
	
	this.sort = dax_dataCore_sort;
	
	function dax_dataCore_sortHandler() {

		this.onAfterSortInt();
		this.onAfterSort();
	
	}
	
	this.sortHandler = dax_dataCore_sortHandler;
	this.data.query.sort.sortHandler = this.sortHandler;
	
	// clears current selection and get all records, only available if 'all records' is enabled for this table
	function dax_dataCore_selectAllRecords() {
	
		// return if all records is not enabled
		if (this.data.selection.showallrecordstab.toLowerCase() == 'false')
			return false;
			
		// select first tab if tabs are enabled
		if (this.tabs)
			this.tabs.selectTabByIndex(0);

		// clear sidebar selection
		if (this.sidebar.initialized) {
			if (this.sidebar.presetQueries.currentDom) {
				this.sidebar.presetQueries.currentDom.className = 'dax_sidebar_presetquery_element';
			}
			
			this.sidebar.presetQueries.currentDom = null;
		}

		// clear query and run new one
		this.data.currentQuery.type = 'allrecords';
		this.data.query.queryId = null;
		this.data.query.queryName = null;
		this.populate();
	}
	
	this.selectAllRecords = dax_dataCore_selectAllRecords;
	
	function dax_dataCore_addTabs(){

		this.tabs = new dax_tabs();
		this.tabs.boundTo = this;
		
		function dax_dataCore_tabs_presetQueryElementOnClick(tabProperties) {

			var dataGrid = this.boundTo.boundTo;
		
			if (this.properties.queryType == 'all') {
				dataGrid.selectAllRecords();
			} else {
				dataGrid.runPresetQuery(this.properties.queryName, this.properties.queryField, this.properties.savedQueryRef);
			}
				
			return false;
		}
		
		this.tabs.tabOnClick = dax_dataCore_tabs_presetQueryElementOnClick;

		function dax_dataCore_tabs_onTabDelete(tabProperties) {
		
			this.boundTo.deleteSearch(tabProperties.savedQueryRef);
		
		}
		
		this.tabs.tabOnClose = dax_dataCore_tabs_onTabDelete;
		
		this.window.addToolbar(this.tabs.dom.container);
		this.window.resize();
		this.tabs.viewport.redraw();
		
	}

	this.addTabs = dax_dataCore_addTabs;
	
	function dax_dataCore_updateTabs() {
	
		if (!this.tabs) {
			this.addTabs();	
		}
			
		// check if tabs are initialized, if not add all records and static query tabs
		if (!this.tabs.presetQuery) {

			this.tabs.presetQuery = true;
		
			// check for all records first
			if (this.data.selection.showallrecordstab.toLowerCase() == 'true') {
				var newTab = this.tabs.addTab('All ' + this.data.selection.tablealias, true);
				newTab.queryType = 'all';
				
				// if current selection is all records, then make 'all records' tab selected
				if (this.data.currentQuery.type == 'allrecords')
					this.tabs.selectTabByNode(newTab);
			}

			// if there are static queries, add them
			var preselectQueries = this.data.selection.preselectQueries;

			if (preselectQueries.length > 0) {

				this.data.presetQueryMapTabs.staticQueries = {
					elements		: new Object()
				}

				for (var queryCount = 0; queryCount < preselectQueries.length; queryCount++){
					
					var queryName = preselectQueries[queryCount].name;
					var newTab = this.tabs.addTab(queryName, true);
					newTab.queryType = 'static';
					newTab.queryName = queryName;

					this.data.presetQueryMapTabs.staticQueries.elements[queryName] = {
						name	: queryName,
						erase	: false,
						node	: newTab
					};
				}			
			}

			var preselectedDynamicQueries = this.data.selection.preselectedDynamicQueries;

			if (preselectedDynamicQueries.length > 0) {
				var dataObject = this;
				window.setTimeout(function() {dataObject.updatePresetQueries();}, 3000);
			}
		}

		// check if saved queries are added, if not add them
		if (this.data.savedQueries.queries.length > 0) {
			for (var queryCount = 0; queryCount < this.data.savedQueries.queries.length; queryCount++) {

				// tab doesn't exist yet, create it
				if (!this.data.savedQueries.queries[queryCount].tabRef) {

					var queryName = this.data.savedQueries.queries[queryCount].name;
					var newTab = this.tabs.addTab(this.data.savedQueries.queries[queryCount].name, true, true);
					newTab.queryType = 'saved';
					newTab.queryName = this.data.savedQueries.queries[queryCount].name;
					newTab.queryId = this.data.savedQueries.queries[queryCount].queryId;
					newTab.savedQueryRef = this.data.savedQueries.queries[queryCount];
				
					this.data.savedQueries.queries[queryCount].tabRef = newTab;
				}
			}
		}
		
		// check for dynamic preset queries, and add/remove tabs if they exist		
		var preselectedDynamicQueries = this.data.selection.preselectedDynamicQueries;
		if (preselectedDynamicQueries.length > 0) {
		
			for (var queryElementCount = 0; queryElementCount < preselectedDynamicQueries.length; queryElementCount++) {
			
				var queryElement = preselectedDynamicQueries[queryElementCount];

				// create container node if it doesn't exist, this is used to quickly compare which tabs are added/removed
				if (!this.data.presetQueryMapTabs[queryElement.fieldId]) {				
					this.data.presetQueryMapTabs[queryElement.fieldId] = {
						elements	: new Object()
					}	
				} else {
					// mark all elements for erasing, existing elements will be untagged so we can erase non-existing ones at the end
					for (var currentElement in this.data.presetQueryMapTabs[queryElement.fieldId].elements) {
						this.data.presetQueryMapTabs[queryElement.fieldId].elements[currentElement].erase = true;					
					}
				}

				
				if (queryElement.names.length != 0) {

					for (var queryCount = 0; queryCount < queryElement.names.length; queryCount++) {

						var elementName = queryElement.names[queryCount];
					
						// create element if it doesn't exist
						if (!this.data.presetQueryMapTabs[queryElement.fieldId].elements[elementName]) {
					
							var elementName = queryElement.names[queryCount];
							var newTab = this.tabs.addTab(elementName, true);
							newTab.queryType = 'dynamic';
							newTab.queryName = elementName;
							newTab.queryField = queryElement.fieldId;

							this.data.presetQueryMapTabs[queryElement.fieldId].elements[elementName] = {
								name	: elementName,
								erase	: false,
								node	: newTab
							};
							
						} else {
							this.data.presetQueryMapTabs[queryElement.fieldId].elements[elementName].erase = false;
						}
					}
				}
				
				//erase non existing elements
				for (var currentElementName in this.data.presetQueryMapTabs[queryElement.fieldId].elements) {
					
					var currentElement = this.data.presetQueryMapTabs[queryElement.fieldId].elements[currentElementName];
					
					if (currentElement.erase) {
						// find tab by query name
						for (var tabCount = 0; tabCount < this.tabs.elements.length; tabCount++) {
							if ((this.tabs.elements[tabCount].queryName == currentElementName) && (this.tabs.elements[tabCount].queryField == queryElement.fieldId)) {
								this.tabs.deleteTab(tabCount);
								delete this.data.presetQueryMapTabs[queryElement.fieldId].elements[currentElementName];
							}
						}
					}					
				}					
			}

			// redraw tabs, only if data grid is initialized
			if (this.data.initialized)
				this.tabs.redraw();
			
		}
	}
	
	this.updateTabs = dax_dataCore_updateTabs;
	
	// editor
	function dax_dataCore_editorEditRecord(recordId) {
		
		if (this.data.selection.fieldsDetail.length == 0)
			return false;
	
		// check if editor exists, if not create it
		if (isDefined(this.editor) && isDefined(this.editor.record)) {
			// exists
			this.editor.edit();
		} else {
			// doesn't exist
			this.getEditor();
			setStyle(this.editor.getDOM(), "display", "block");
			var editor = this.editor;
			var auto = editor.prv.mAutoReload;
			if (auto > 0) {
			  setTimeout(function() {editor.reload();}, 500);
			}
		}
								
		// call editor and display data
		this.editor.pull(this.data.selection.tableid, recordId);
		
	}
	
	this.editorEditRecord = dax_dataCore_editorEditRecord;
	
	// editor function for window
	this.getEditor = function()	{
		if (this.editor)
			return this.editor;
	
		// create editor object
		this.editor = new Editor();
		var dom = this.editor.getDOM();
		dom.style.zIndex = ++WindowManager.maxZIndex;
		this.window.dom.m.insertBefore(dom, this.window.toolbar);
		var w_off =  (this.window.dom.offsetWidth / 2) -
			(dom.offsetWidth / 2);

		setStyle(dom, "position", "absolute");
		setStyle(dom, "left", w_off + "px");
		
		this.editor.owner = this;
		
		var w_ref = this; // use this for when editor is detatched
		this.editor.onaftersave = function()
		{
			this.owner.refresh(true);
		}

		this.editor.onafterdelete = function()
		{
			this.owner.refresh(true);
			this.getTearAway().contract(1, 50);
			this.destroy();
		}
		
		if (browser.ie7) {
			this.editor.onafterrefresh = function()
			{
				//setStyle(this.owner.editor.prv.mTearAway.dom, "display", "block");
				if ((this.owner) && (this.owner.editor)) {
					setStyle(this.prv.mTearAway.dom, "top", "29px");
					setStyle(this.prv.mTearAway.dom, "position", "absolute");
				}
			}
		}
		
		// customize
		dom.ondragstop = function(aEvent)
		{
			this.className = "tearaway_torn";
			var editor = this.owner;
			
			if (browser.ie) {
				this.top.t.style.fontSize = '1pt';
				this.top.tl.style.fontSize = '1pt';
				this.top.tr.style.fontSize = '1pt';
				this.top.style.fontSize = '1pt';

			}
		 
			// create new window when dragging is done
			if (editor.owner) {
				var winObj = editor.owner;
				
				if (!winObj.editor)
					return false;
					
				var dom = editor.getDOM();
				var pos = getObjectPosition(this); // global coordinates

				if (taskbar) {
				
					// add taskbar item for new window
					var ti = new taskbarItem(taskbar);

					// make unique label based on table name
					i = 1;
					while ($('dax_taskbar_button' + winObj.window.titleBar.innerHTML + i) != null) {
						i++;
					}
					// set taskbar button title
					ti.dom.id = 'dax_taskbar_button' + winObj.window.titleBar.innerHTML + i;

					ti.setLabel("Edit " + winObj.window.titleBar.innerHTML + " " + i);
					ti.setIcon("editor");
				
					// create a handler for clicking
					var ref = this;
					ti.dom.onclick = function() { 
						this.parent.setSelected (true, true);
						ref.onclick();
					};
					setStyle (ti.dom, 'display', 'block');
					ti.setSelected (true, true);
					// refresh taskbar
					ti.parent.refresh();
				}
				
				// override cancel button
				if (dom.cancelBtn) {
					dom.cancelBtn.onclick = function(aEvent) {
						if (ti)
							ti.remove(); // remove taskbar item
						document.body.removeChild(this.parent.prv.mDOM);
						this.parent.destroy();
						delete this.parent;
					};
				}
				
				// override the focusing
				this.onclick = function(aEvent) {
					if (WindowManager.active)
						WindowManager.active.deactivate();
						setStyle(this, "zIndex", ++WindowManager.maxZIndex);
						if (ti) {
							ti.setSelected(true,true);
						}
				};

				// release
				if (this.owner.owner.window)
					winObj.window.deactivate();
				this.owner.owner.editor = null;
				//this.parent.owner = null;
				
				// reposition
				document.body.insertBefore(this, null);
				setStyle(this, "left", pos.x + "px");
				setStyle(this, "top", pos.y + "px");
				
			}
		};
		return this.editor;
	};		

	// opens the editor and starts a new record
	function dax_dataCore_editorNewRecord()
	{ 
	
		if (this.data.selection.fieldsDetail.length == 0)
			return false;
	
		// find the editor if we have one, otherwise create a new one
		var editor = this.getEditor();

		editor.record = { recordId : -1, tableId: this.data.selection.tableid };
		editor.buildEditorByTable(this.data.selection.tableid);

		editor.clear();

		editor.edit();
	};

	this.editorNewRecord = dax_dataCore_editorNewRecord;
	
	function dax_dataCore_deleteSelectedRecords(skipConfirmation) {
	
		// check first if there are any records selected
		if (this.data.selectedRecords.count == 0)
			return false;
		
		// prompt for conformation if it's not disabled
		if ((!skipConfirmation) && (!window.confirm('Delete selected records?')))
			return false;
		
		var selectedRecords = [ ];
		
		for (var recordId in this.data.selectedRecords) {
		
			if (recordId != 'count') {
		
				selectedRecords.push(recordId);
				this.removeRecordFromSelection(recordId);
			
			}
		}
		
		dax_bridge.deleteRecords(this.data.selection.tablename, selectedRecords);
		
		this.populate();
	
	}
	
	this.deleteSelectedRecords = dax_dataCore_deleteSelectedRecords;
		
	// main toolbar
	function dax_dataCore_toolbar_showToolbar(elementsToAdd) {

		// create toolbar if one doesn't exist
		if (!this.toolbars.main) {
		
			this.toolbars.main = new Toolbar();
			this.window.addToolbar(this.toolbars.main.prv.mDOM);
			this.toolbars.main.prv.mDOM.className = 'dax_toolbar';

		}
		
		// if elements to add is present, cycle through and add/make elements visible
		if (elementsToAdd) {
			for (var elementsCount = 0; elementsCount < elementsToAdd.length; elementsCount++) {
				switch (elementsToAdd[elementsCount]) {
					case 'createrecord':
						this.showToolbarCreateRecordButton(true);
					break;
					case 'deleterecords':
						this.showToolbarDeleteRecordsButton(true);
					break;
					case 'search':
						this.showToolbarSearchWidget(this.toolbars.main, true);
					break;
				}
			}		
		}
	}

	this.showToolbar = dax_dataCore_toolbar_showToolbar;

	function dax_dataCore_toolbar_showCreateButton(skipShowToolbar) {

		// check if toolbar exists, if not create one
		if (!skipShowToolbar)
			this.showToolbar();

		// if button was not created yet, create a new one
		if (!this.toolbars.main.createButton) {

			this.toolbars.main.createButton = this.toolbars.main.addButton('+');

			this.toolbars.main.createButton.boundTo = this;
			
			function dax_dataCore_toolbar_createButton_onClick() {
				this.boundTo.toolbarCreateButton();
			}
			
			this.toolbars.main.createButton.onclick = dax_dataCore_toolbar_createButton_onClick;

			function dax_dataCore_toolbarCreate_onclick() {
				this.editorNewRecord();
			}

			this.toolbarCreateButton = dax_dataCore_toolbarCreate_onclick;

			// just assign round button style for now, will add refresh toolbar call here later that will take care of that
			this.toolbars.main.createButton.prv.mDOM.className += ' dax_toolbarbutton_createset dax_toolbarbutton_join_left';
			
		}
	}
	
	this.showToolbarCreateRecordButton = dax_dataCore_toolbar_showCreateButton;
	
	function dax_dataCore_toolbar_showDeleteButton(skipShowToolbar) {
	
		// check if toolbar exists, if not create one
		if (!skipShowToolbar)
			this.showToolbar();

		if (!this.toolbars.main.deleteButton) {
			
			this.toolbars.main.deleteButton = this.toolbars.main.addButton('-');

			this.toolbars.main.deleteButton.boundTo = this;
			
			function dax_dataCore_toolbar_deleteButton_onClick() {
				this.boundTo.toolbarDeleteButton();
			}
			
			this.toolbars.main.deleteButton.onclick = dax_dataCore_toolbar_deleteButton_onClick;

			function dax_dataCore_toolbarDelete_onclick() {
				this.deleteSelectedRecords();
			}
			
			this.toolbarDeleteButton = dax_dataCore_toolbarDelete_onclick;

			// just assign round button style for now, will add refresh toolbar call here later that will take care of that
			this.toolbars.main.deleteButton.prv.mDOM.className += ' dax_toolbarbutton_createset dax_toolbarbutton_join_right';
			
		}
	}

	this.showToolbarDeleteRecordsButton = dax_dataCore_toolbar_showDeleteButton;

	// advanced search toolbar
	function dax_dataCore_toolbar_showAdvancedSearch() {

		// create toolbar if one doesn't exist
		if (!this.toolbars.advancedsearch) {
		
			this.toolbars.advancedsearch = [];
			
			// Remove buttons here
			// Replace with Less
			// this.toolbars.main.removeContent(this.toolbars.main.search.saveButton.prv.mDOM);
			// this.toolbars.main.removeContent(this.toolbars.main.search.moreButton.prv.mDOM);
			
			this.toolbars.main.search.moreButton.prv.mDOM.m.label.firstChild.nodeValue = "+";
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').style.display = "block";
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[0].text = STR['adminQuery_Operator1'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[1].text = STR['adminQuery_Operator2'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[2].text = STR['adminQuery_Operator3'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[3].text = STR['adminQuery_Operator4'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[4].text = STR['adminQuery_Operator5'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[5].text = STR['adminQuery_Operator6'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[6].text = STR['adminQuery_Operator7'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[7].text = STR['adminQuery_Operator8'];
			
		}
		else {				

			// this.toolbars.main.search.moreButton.prv.mDOM.m.label.firstChild.nodeValue = "-";
			if (this.toolbars.advancedsearch.length > 0) {
				this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search.lessButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_solo';
				this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search.moreButton.prv.mDOM.style.display = "none";
			}
			else {
				this.toolbars.main.search.saveButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_solo';
				this.toolbars.main.search.moreButton.prv.mDOM.style.display = "none";
			}
						
			this.toolbars.advancedsearch[this.toolbars.advancedsearch.length] = new Toolbar();
			if (this.tabs)
				this.window.addToolbar(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].prv.mDOM, this.tabs.dom.container.parentNode);
			else
				this.window.addToolbar(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].prv.mDOM);
			
			this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].prv.mDOM.className = 'dax_toolbar';
			
			this.showToolbarSearchWidget(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1], true);
			getElementByClassName(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search, 'dax_search_operator').style.display = "block";
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[0].text = STR['adminQuery_Operator1'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[1].text = STR['adminQuery_Operator2'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[2].text = STR['adminQuery_Operator3'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[3].text = STR['adminQuery_Operator4'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[4].text = STR['adminQuery_Operator5'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[5].text = STR['adminQuery_Operator6'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[6].text = STR['adminQuery_Operator7'];
			getElementByClassName(this.toolbars.main.search, 'dax_search_operator').options[7].text = STR['adminQuery_Operator8'];
			getElementByClassName(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search, 'dax_search_andor').style.display = "block";
			getElementByClassName(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search, 'dax_search_andor').options[0].text = STR['adminQuery_Link1'];
			getElementByClassName(this.toolbars.advancedsearch[this.toolbars.advancedsearch.length - 1].search, 'dax_search_andor').options[1].text = STR['adminQuery_Link2'];	
		}
				
		if (this.data.initialized) {
		//	this.window.onsize();
			this.activate();
		}
	}

	this.showAdvancedSearch = dax_dataCore_toolbar_showAdvancedSearch;
	
	function dax_dataCore_toolbar_showWidget(toolbar, skipShowToolbar) {

		// check if toolbar exists, if not create one
		if (!skipShowToolbar)
			this.showToolbar();
			
		if (!toolbar.search) {
			
			if (!this.toolbars.advancedsearch) {
				toolbar.search = getTemplate('dax_search').cloneNode(true);
				
				toolbar.addContent(toolbar.search);
				toolbar.search.style.display = 'block';
	
				toolbar.search.saveButton = toolbar.addButton('Save');
				toolbar.search.moreButton = toolbar.addButton('Advanced');

				// just assign round button style for now, will add refresh toolbar call here later that will take care of that
				toolbar.search.saveButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_join_left';
				toolbar.search.moreButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_join_right';
			}
			else {
				toolbar.search = getTemplate('dax_search').cloneNode(true);
				
				toolbar.addContent(toolbar.search);
				toolbar.search.style.display = 'block';
				toolbar.search.style.paddingLeft = "7px";
	
				toolbar.search.lessButton = toolbar.addButton('-');
				toolbar.search.moreButton = toolbar.addButton('+');				

				// just assign round button style for now, will add refresh toolbar call here later that will take care of that
				toolbar.search.lessButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_join_left';
				toolbar.search.moreButton.prv.mDOM.className += ' dax_toolbarbutton_searchset dax_toolbarbutton_join_right';
			}
							
			//populate search dropdown with searchable fields
			toolbar.search.fieldList = getElementByClassName(toolbar.search, 'dax_search_fields');
			toolbar.search.operator = getElementByClassName(toolbar.search, 'dax_search_operator');
			toolbar.search.andor = getElementByClassName(toolbar.search, 'dax_search_andor');
			
			this.searchtoolbar = toolbar;
			
			function dax_dataCore_toolbar_updateSearchFieldList() {
				// don't update if fields are loaded
				if (!this.data.selection.cached)
					return false;
				
				// update field list
				for (var fieldCount = 0; fieldCount < this.data.selection.fields.length; fieldCount++) {
					this.searchtoolbar.search.fieldList.options[fieldCount] = new Option(this.data.selection.fields[fieldCount].fieldname, this.data.selection.fields[fieldCount].fieldid);
				}
			}
			
			toolbar.search.value = getElementByClassName(toolbar.search, 'dax_search_input');
			
			function dax_dataCore_toolbar_searchOnKeyUp(event) {

				var boundTo = this.boundTo;
				if (boundTo.searchTimeout) {
					window.clearTimeout (boundTo.searchTimeout);
				}
					
				boundTo.searchTimeout = window.setTimeout (function() {boundTo.populate(); }, 500);			
			}
			
			toolbar.search.value.onkeyup = dax_dataCore_toolbar_searchOnKeyUp;
			toolbar.search.value.boundTo = this;
			
			if (this.toolbars.advancedsearch) {
				if (!this.data.advancedsearchvalues)
					this.data.advancedsearchvalues = [];		
				this.data.advancedsearchvalues[this.data.advancedsearchvalues.length] = toolbar.search.value;
				toolbar.search.value.value = "";	
			}	
			else
				this.data.searchValue = toolbar.search.value;
						
			this.updateSearchFieldList = dax_dataCore_toolbar_updateSearchFieldList;
			
			this.updateSearchFieldList();

			function dax_dataCore_toolbar_saveSearch() {
				
				if (this.boundTo.data.advancedsearchvalues) {
					var queryname = "";
					for (var i = 0; i < this.boundTo.data.advancedsearchvalues.length; i++) {
						queryname += " & " + this.boundTo.data.advancedsearchvalues[i].value;
					}
					this.boundTo.saveSearch(this.boundTo.data.searchValue.value + queryname);
				}
				else 
					this.boundTo.saveSearch(this.boundTo.data.searchValue.value);
				
			}

			if (!this.toolbars.advancedsearch) {
				toolbar.search.saveButton.boundTo = this;
				toolbar.search.saveButton.onclick = dax_dataCore_toolbar_saveSearch;
			}
			
			function dax_dataCore_toolbar_advancedSearch() {
				this.boundTo.showAdvancedSearch();
			}

			toolbar.search.moreButton.boundTo = this;
			toolbar.search.moreButton.onclick = dax_dataCore_toolbar_advancedSearch;

			function dax_dataCore_toolbar_removetoolbar() {
				for (var i=0; i < this.boundTo.toolbars.advancedsearch.length; i++) {
					if (this.boundTo.toolbars.advancedsearch[i].prv.mDOM == this.toolbar.prv.mDOM) {
						if (i == this.boundTo.toolbars.advancedsearch.length - 1) {
							if (i == 0) {
								this.boundTo.toolbars.main.search.saveButton.prv.mDOM.className = 'toolbarbutton dax_toolbarbutton_searchset dax_toolbarbutton_join_left';
								this.boundTo.toolbars.main.search.moreButton.prv.mDOM.style.display = "block";
							}
							else {
								this.boundTo.toolbars.advancedsearch[i - 1].search.lessButton.prv.mDOM.className = 'toolbarbutton dax_toolbarbutton_searchset dax_toolbarbutton_join_left';
								this.boundTo.toolbars.advancedsearch[i - 1].search.moreButton.prv.mDOM.style.display = "block";
							}
						}
						this.boundTo.data.advancedsearchvalues.splice(i,1);
						this.boundTo.toolbars.advancedsearch.splice(i, 1);
					}
				}

				this.boundTo.window.removeToolbar(this.toolbar.prv.mDOM);

				if (this.boundTo.data.initialized) {
					this.boundTo.activate();
				}
				
				var boundTo = this.boundTo;
				if (boundTo.searchTimeout) {
					window.clearTimeout (boundTo.searchTimeout);
				}
					
				boundTo.searchTimeout = window.setTimeout (function() {boundTo.populate(); }, 500);

			}
			
			if (toolbar.search.lessButton) {
				toolbar.search.lessButton.boundTo = this;
				toolbar.search.lessButton.toolbar = toolbar;
				toolbar.search.lessButton.onclick = dax_dataCore_toolbar_removetoolbar;
			}
		}	
	}
	
	this.showToolbarSearchWidget = dax_dataCore_toolbar_showWidget;
	
	// manipulate parsed data values before they are placed into the grid, usually used in .onBeforeDataDisplay
	function dax_dataCore_getParsedDataValue(record, field) {
	
		if (!this.data.query.parsedData)
			return null;
			
		if (!this.data.query.parsedData[record])
			return null;
	
		if (!this.data.query.parsedData[record].fields[field])
			return null;
			
		return this.data.query.parsedData[record].fields[field].value;
	
	}
	
	this.getParsedDataValue = dax_dataCore_getParsedDataValue;

	// manipulate parsed data values before they are placed into the grid, usually used in .onBeforeDataDisplay
	function dax_dataCore_setParsedDataValue(record, field, value) {
		if (!this.data.query.parsedData)
			return null;
			
		if (!this.data.query.parsedData[record])
			return null;
	
		if (!this.data.query.parsedData[record].fields[field])
			return null;
			
		this.data.query.parsedData[record].fields[field].value = value;
	}
	
	this.setParsedDataValue = dax_dataCore_setParsedDataValue;

	// manipulate parsed data values before they are placed into the grid, usually used in .onBeforeDataDisplay
	function dax_dataCore_getParsedDataRecordCount() {
	
		if (!this.data.query.parsedData)
			return null;
			
		return this.data.query.parsedData.length;
	
	}
	
	this.getParsedDataRecordCount = dax_dataCore_getParsedDataRecordCount;
	
}

function dax_dataGrid(selection, targetNode, headerRows, lockedLeftColumns, useControlColumn){

	// set default values for parameters that were not passed
	if ((!useControlColumn) && (useControlColumn != false))
		useControlColumn = true;

	if ((headerRows !== 0) && (!headerRows))
		headerRows = 1;

	if ((lockedLeftColumns !== 0) && (!lockedLeftColumns))
		lockedLeftColumns = 0;
		
	var lockColumns = useControlColumn ? (lockedLeftColumns + 1) : lockedLeftColumns;
	
	// initialize dependencies
	dax_superGridObject.apply(this, [targetNode, headerRows, lockColumns]);

	this.window.status.style.height = '19px';
	
	this.gridProperties.headerRows = headerRows ? 1 : 0;

	dax_dataCore.apply(this, [selection]);

	this.dataGrid = {
		inlineEdit			: {
			active				: false,
			row					: null,
			recordId			: null,
			allow				: false,
			floatingTextArea	: null,		// node to expand input area
			activeFloat			: null,		// current float area
			floatingCalendarPicker	: null	// calendar picker for inline editing
		},
		startRowForData			: headerRows ? headerRows : 0,
		startColumnForData		: useControlColumn ? 1 : 0,
		useControlColumn		: useControlColumn,
		enablePicturePreview	: true,
		nodes			: {
			headerSortGraphic	: getTemplate('dax_datagrid_header_sort_graphic').cloneNode(true),
			headerLabel			: getTemplate('dax_datagrid_header_label')
		
		},
		selectedRow				: 0,
		selectionMode			: 'multi', // none, single, multi
		lastSelectedColumn		: null,
		lastSelectedRecordId	: null
	};
	
	generateUniqueId(this.dataGrid.nodes.headerSortGraphic);
	
	// setup grid
	this.setGridClass(null, 'dax_datagrid');
//	this.allowMultipleSelection(false);
//	this.allowSelection(-1, -1, false, true);
	this.allowColumnSwap(false);
	this.gridProperties.allowColumnSelection = false;
	this.setRowHeightInPx(21);

	// set window title
	this.window.titleBar.innerHTML = this.data.selection.tablealias;		
	
	// setup viewport
	this.viewport.scrollbars.ver.autoHeight = false;
	this.viewport.scrollbars.ver.autoContent = false;
	this.viewport.scrollbars.ver.customValues = true;
	
	this.viewport.scrollbars.ver.arrowStep = 1;
		
	function dax_dataGrid_blankFunction() { };
		
	// dev defined events
	this.onDataColumnClick = dax_dataGrid_blankFunction;
	this.onDataRowClick = dax_dataGrid_blankFunction;
	this.onDataCellClick = dax_dataGrid_blankFunction;
	
	function dax_dataGrid_allowInlineEditing(allow) {
		daDataGrid.dataGrid.inlineEdit.allow = allow;
	}
	
	this.allowInlineEditing = dax_dataGrid_allowInlineEditing;
	
	function dax_dataGrid_onRowFooterChange(skipInitializationCheck) {
	
		// don't do it if grid is not initialized, it means that there are no rows yet to style
		if ((!this.data.initialized) && (!skipInitializationCheck))
			return false;
	
		// if there are footers available, set styles for them
		if (this.gridProperties.lockedBottom > 0) {
			for (var footerCount = 0; footerCount < this.gridProperties.lockedBottom; footerCount++) {
				var footerRow = this.getFooterRowNumber(footerCount + 1);
				this.setInternalStyleClass(footerRow, null, 'dax_datagrid_footer');
			}
		}
	}
	
	this.onRowFooterChange = dax_dataGrid_onRowFooterChange;
	
	function dax_dataGrid_onVScroll(){
	
		// hide inline edit button if one is enabled
		if (this.boundTo.dataGrid.inlineEdit.editButtonNode)
			this.boundTo.dataGrid.inlineEdit.editButtonNode.style.display = 'none';
	
		// subtracting -1 from total record count because on BE record 0 is same as record 1
		if ((this.scrollbars.ver.position + 1) != this.boundTo.data.query.start)
			this.boundTo.populate(this.scrollbars.ver.position + 1, this.boundTo.getVisibleRecordCount());
	}
	
	this.viewport.onvscroll = dax_dataGrid_onVScroll;
	
	// setup before field info is acquired
	function dax_dataGrid_preInitialize() {
		this.activate();
	}
	
	this.preInitialize = dax_dataGrid_preInitialize;
	
	// setup grid for data
	this.initialize = function() {
		this.size(this.dimensions.rows, this.data.selection.fieldsList.length + this.dataGrid.startColumnForData);
		if (this.dataGrid.useControlColumn)
			this.setColumnWidth(0, 30);
		
		// set header titles
		for (var fieldCount = 0; fieldCount < this.data.selection.fieldsList.length; fieldCount++) {
			var colCount = fieldCount + this.dataGrid.startColumnForData;
			
			this.cols[colCount].field = getField(this.data.selection.tableid, this.data.selection.fieldsList[fieldCount].fieldid);
		
			// only add field labels if headers exist
			if (this.gridProperties.lockedTop > 0) {
				var fieldNameNode = this.dataGrid.nodes.headerLabel.cloneNode(true);
				generateUniqueId(fieldNameNode);
				fieldNameNode.style.display = 'inline';
				this.rows[0].cells[colCount].appendChild(fieldNameNode);
				fieldNameNode.innerHTML = this.data.selection.fieldsList[fieldCount].fieldalias;
				
				// set header class
				this.setInternalStyleClass(0, null, 'dax_datagrid_firstheader');

				// if there are more header zones, apply header styling
				if (this.gridProperties.lockedTop > 1) {
					for (var headerCount = 1; headerCount < this.gridProperties.lockedTop; headerCount++) {
						this.setInternalStyleClass(headerCount, null, 'dax_datagrid_header');
					}
				}
			}

			// assign class and column width based on data type
			switch(this.data.selection.fieldsList[fieldCount].fieldtype) {
				case 'real':
				case 'longint':
				case 'integer':
					this.setInternalStyleClass(null, colCount, 'datanumeric');
					var columnWidth = 75;
				break
				case 'alpha':
				case 'text':
					this.setInternalStyleClass(null, colCount, 'datatext');
					var columnWidth = 150;
				break;
				case 'boolean':
					this.setInternalStyleClass(null, colCount, 'databoolean');
					var columnWidth = 50;
				break;
				case 'date':
					this.setInternalStyleClass(null, colCount, 'datadate');
					var columnWidth = 75;
				break;
				case 'time':
					this.setInternalStyleClass(null, colCount, 'datatime');
					var columnWidth = 75;
				break;
			}
			
			// check if column width is too short - both title and sort indicator must fit
			if (this.gridProperties.lockedTop > 0) {
				if (columnWidth < fieldNameNode.clientWidth + 28) {
					columnWidth = fieldNameNode.clientWidth + 28;
				}
			}
				
			this.setColumnWidth(colCount, columnWidth);

		}
		
		// do footer styling, in case they are created before initialization
		this.onRowFooterChange(true);
		this.redraw(true, true);
	}

	// how many records are visible on the screen?
	function dax_dataGrid_getVisibleRecordCount(){
		return Math.floor(this.viewport.dom.container.offsetHeight / this.gridProperties.defaultRowHeight);
	}
	
	// gets total number of visible records
	this.getVisibleRecordCount = dax_dataGrid_getVisibleRecordCount;
	
	function dax_dataGrid_onWindowSizeExt() {
		var visibleRecords = this.getVisibleRecordCount();
		
		if ((visibleRecords > this.data.query.size) && (this.data.initialized) && (this.data.firstLoad))
			this.populate();
			
		if (this.sidebar.active) {
			this.sidebar.viewportContainer.style.height = this.window.sidebar.dom.clientHeight - this.sidebar.header.clientHeight + 'px';
			this.sidebar.viewport.redraw();
		}
		
		if (this.tabs)
			this.tabs.redraw();
	
	}
	
	this.onWindowSizeExt = dax_dataGrid_onWindowSizeExt;
	
	function dax_dataGrid_inlineEdit(cellRow, recordId) {
	
		var previousEditMode = false;

		// considered a new record if no cellRow & recordId were passed
		if ((!cellRow) && (cellRow != 0) && (!recordId))
			var newRecord = true;
		else
			var newRecord = false;
		
		// find cell based on passed field & record id, or cell position
		if (!newRecord) {
			if (recordId) {
				// find recordId row
				var editRow = this.getRowByRecordId(recordId);
				var editRecordId = recordId;
			} else if ((cellRow) || (cellRow == 0)) {
				var editRecordId = this.rows[cellRow].recordId;
				var editRow = cellRow;
			} else {
				return false;		
			}
		} else {
			var editRow = this.gridProperties.lockedTop;
			var editRecordId = -3;
		}

		// show save button
		if (this.dataGrid.inlineEdit.editButtonNode)
			this.dataGrid.inlineEdit.editButtonNode.style.display = 'none';
		if (!this.dataGrid.inlineEdit.saveButtonNode) {
			this.dataGrid.inlineEdit.saveButtonNode = getTemplate('dax_datagrid_inlineedit_save').cloneNode(true);
			generateUniqueId(this.dataGrid.inlineEdit.saveButtonNode);
		}

		this.rows[editRow].cells[0].appendChild(this.dataGrid.inlineEdit.saveButtonNode);		
		this.dataGrid.inlineEdit.saveButtonNode.style.display = 'block';
		
		// set properties
		this.dataGrid.inlineEdit.active = true;
		this.dataGrid.inlineEdit.row = editRow;
		this.dataGrid.inlineEdit.recordId = editRecordId;
		
		this.redraw(true, true);
		
		// branch based on data type
		for (var colCount = this.dataGrid.startColumnForData; colCount < this.cols.length; colCount++) {
			var editCell = this.getCell(editRow, colCount);
			var editField = this.cols[colCount].field;
			var fieldType = editField.fieldtype;
			
			if (editField.fieldchoicelist)	
				fieldType = 'choicelist';
			
			// is field editable?
			// non-modifiable applies to the existing records
			var fieldEditable = true;
			if (editField.fieldnonEnterable.toLowerCase() == 'true')
				fieldEditable = false;
			else if ((editField.fieldnonModifiable.toLowerCase() == 'true') && (!newRecord))
				fieldEditable = false;
			
			if (fieldEditable) {
			
				switch(fieldType) {

					case 'boolean':
					
						// checkbox boolean will stay as a checkbox, other booleans will have dropdown instead
						if (editField.fieldformat == 'bCheckbox') {
							this.setCellType(editCell.row, editCell.column, 'checkbox');
							editCell.contentObject.onclick = this.inlineEdit_checkboxOnClick;	
							
						} else {
						
							if (editField.fieldformat == 'bYes') {
								var booleanTrue = STR['yes'];
								var booleanFalse = STR['no'];
							} else if (editField.fieldformat == 'bMale') {
								var booleanTrue = STR['male'];
								var booleanFalse = STR['female'];
							} else {
								var booleanTrue = STR['ttrue'];
								var booleanFalse = STR['ffalse'];
							}
						
							this.setCellType(editCell.row, editCell.column, 'dropdown');
							this.setCellValue(editCell.row, editCell.column, [booleanTrue, booleanFalse]);
							
							editCell.contentObject.options[0].value = 'true';
							editCell.contentObject.options[1].value = 'false';

							editCell.contentObject.onkeydown = this.inlineEdit_inputOnKeyDown;
						}
					
					break;
				
					case 'choicelist':

						this.setCellType(editCell.row, editCell.column, 'dropdown');
						
						if (editField.fieldcallbackonchange) {
							editCell.contentObject.onchange = this.inlineEdit_fireCallback;
						}

						editCell.contentObject.onkeydown = this.inlineEdit_inputOnKeyDown;
						
						// pull choicelist for new records, for existing records they are pulled when new record call is made
						dax_bridge.getChoiceList(this.cols[editCell.column].field.fieldchoicelist, this.choiceListHandler, editCell.contentObject);	
	
					break;
				
					case 'picture':
					break;

					case 'text':
						this.setCellType(editCell.row, editCell.column, 'textarea');
						
						// on focus, textarea will float above the grid, and on blur, it will return to the cell
						editCell.contentObject.onfocus = this.inlineEdit_textAreaOnFocus;

						editCell.contentObject.onkeydown = this.inlineEdit_inputOnKeyDown;
					break;

					case 'date':
					
						this.setCellType(editCell.row, editCell.column, 'input');

						dax_attachCalendarPicker(editCell.contentObject);
						
						if (editField.fieldcallbackonchange)
							editCell.contentObject.onchange = this.inlineEdit_fireCallback;
							
						editCell.contentObject.onkeydown = this.inlineEdit_inputOnKeyDown;
						
					break;		
					
					case 'alpha':
					default:
					
						this.setCellType(editCell.row, editCell.column, 'input');

						if (editField.fieldcallbackonchange)
							editCell.contentObject.onchange = this.inlineEdit_fireCallback;
							
						editCell.contentObject.onkeydown = this.inlineEdit_inputOnKeyDown;
						
					break;		
				}
			}
		}

		if (!newRecord)
			this.inlineEditGetRecord(this.rows[editRow].recordId);
			
	}
	
	this.inlineEdit = dax_dataGrid_inlineEdit;

	// on focus, textarea will float above the grid, and on blur, value will be placed back into the grid
	function dax_dataGrid_inlineEdit_textAreaOnFocus(event) {

		xpEvent(event, true, true);

		if (!this.boundTo.dataGrid.inlineEdit.floatingTextArea) {
			this.boundTo.dataGrid.inlineEdit.floatingTextArea = getTemplate('dax_datagrid_inlineedit_textareafloat').cloneNode(true);
			//this.boundTo.inlineEdit.floatingTextArea.onmouseout = this.boundTo.inlineEdit_textAreaOnBlur;
			document.body.appendChild(this.boundTo.dataGrid.inlineEdit.floatingTextArea);
			generateUniqueId(this.boundTo.dataGrid.inlineEdit.floatingTextArea);
			this.boundTo.dataGrid.inlineEdit.floatingTextArea.boundTo = this.boundTo;
			this.boundTo.dataGrid.inlineEdit.floatingTextArea.onkeydown = this.boundTo.inlineEdit_inputOnKeyDown;
			this.boundTo.dataGrid.inlineEdit.floatingTextArea.onLeave = this.boundTo.inlineEdit_textAreaOnLeave;
			this.boundTo.dataGrid.inlineEdit.floatingTextArea.onblur = this.boundTo.inlineEdit_textAreaOnBlur;
		}
		
		var floatingTextArea = this.boundTo.dataGrid.inlineEdit.floatingTextArea;

		this.boundTo.dataGrid.inlineEdit.activeFloat = floatingTextArea;
		
		floatingTextArea.value = this.parentCell.contentObject.value;
		
		var cellPosition = getObjectPosition(this.parentCell);
		floatingTextArea.style.display = 'block';
		floatingTextArea.style.top = cellPosition.y + 'px';
		floatingTextArea.style.left = cellPosition.x + 'px';
		
		// which cell to save value to
		floatingTextArea.parentCell = this.parentCell;
		floatingTextArea.parentInput = this.parentCell.contentObject;
		floatingTextArea.focus();
							
	}
	
	this.inlineEdit_textAreaOnFocus = dax_dataGrid_inlineEdit_textAreaOnFocus;

	function dax_dataGrid_inlineEdit_textAreaOnLeave() {
		this.boundTo.dataGrid.inlineEdit.activeFloat = null;
		this.parentInput.value = this.value;
		this.style.display = 'none';
	}
	
	this.inlineEdit_textAreaOnLeave = dax_dataGrid_inlineEdit_textAreaOnLeave;
	
	function dax_dataGrid_inlineEdit_textAreaOnBlur(event) {

		xpEvent(event, true, true);
		this.onLeave();

	}
	
	this.inlineEdit_textAreaOnBlur = dax_dataGrid_inlineEdit_textAreaOnBlur;

	
	// gets the record for the inline editing
	function dax_dataGrid_inlineEditGetRecord(recordId){
	
		dax_bridge.getRecord(this.data.selection.tableid, recordId, this.inlineEditPopulateRecord, this);
	
	}
	
	this.inlineEditGetRecord = dax_dataGrid_inlineEditGetRecord;
	
	function dax_dataGrid_inlineEdit_checkboxOnClick() {
	
		if (this.parentCell.value) {
			this.parentCell.value = false;
			this.className = 'fourdaf_checkbox_uncheck';
		} else {
			this.parentCell.value = true;
			this.className = 'fourdaf_checkbox_check';
		}
	
	}
	
	this.inlineEdit_checkboxOnClick = dax_dataGrid_inlineEdit_checkboxOnClick;
	
	function dax_dataGrid_inlineEdit_inputOnKeyDown(event){

		var event = xpEvent(event);
		if ((event.keyCode == 13) && (this.nodeName == 'TEXTAREA'))
			return;
	
		switch (event.keyCode) {
		
			// tab key, go to next field
			case 9:
				// look for next field
				var rowNumber = this.parentCell.row;
				var columnNumber = this.parentCell.column;
				
				// if this field is a date, close picker
				if (this.boundTo.cols[this.parentCell.column].field.fieldtype == 'date')
					dax_bridge.calendarPickerFloat.blur();

				while (this.boundTo.rows[rowNumber].cells[++columnNumber]) {
					var cellRef = this.boundTo.rows[rowNumber].cells[columnNumber];
					if (cellRef.contentObject) {
						if ((cellRef.contentObject.nodeName == 'INPUT') || (cellRef.contentObject.nodeName == 'TEXTAREA') || (cellRef.contentObject.nodeName == 'SELECT')) {
							this.boundTo.focusCell(rowNumber, columnNumber);
							var focusObject = cellRef.contentObject;
							// timeout is required for browser to redraw the screen
							window.setTimeout(function() { focusObject.focus()}, 1);
							return false;
						}
					}
				}
				// no more fields that user can tab to, save record
				var event = xpEvent(event, true, true);
				this.boundTo.inlineEditSave();
			break;

			// ESC key, cancel editing
			case 27:

				// if this field is a date, close picker
				if (this.boundTo.cols[this.parentCell.column].field.fieldtype == 'date')
					dax_bridge.calendarPickerFloat.blur();

				var event = xpEvent(event, true, true);
				this.boundTo.inlineEditClear();
			break;

			// ENTER/RETURN key, save record
			case 13:

				// if this field is a date, close picker
				if (this.boundTo.cols[this.parentCell.column].field.fieldtype == 'date')
					dax_bridge.calendarPickerFloat.blur();

				var event = xpEvent(event, true, true);
				this.boundTo.inlineEditSave();
			break;
			
		}		
	}
	
	this.inlineEdit_inputOnKeyDown = dax_dataGrid_inlineEdit_inputOnKeyDown;
	
	// place requested record into the inline edit row
	function dax_dataGrid_inlineEditPopulateRecord(parsedRecord, objectRef){
	
		var dataGrid = objectRef;
		var editRow = dataGrid.dataGrid.inlineEdit.row;

		if (parsedRecord[0])
			var record = parsedRecord[0];
		else
			return false;

		for (var fieldCount = 0; fieldCount < record.fields.length; fieldCount++) {
		
			var fieldId = record.fields[fieldCount].id;
			var value = record.fields[fieldCount].value;
			var originalValue = record.fields[fieldCount].originalValue;	// not formatted
			var column = dataGrid.getColumnByFieldId(fieldId);
			var field = dataGrid.cols[column].field;
			
			if (column) {
				var cell = dataGrid.rows[editRow].cells[column];
				
				if ((field.fieldtype == 'boolean') && (field.fieldformat == 'bCheckbox')) {
					// checkbox boolean
					if (originalValue.toLowerCase() == 'true')
						dataGrid.setCellValue(editRow, column, true);
					else
						dataGrid.setCellValue(editRow, column, false);

				} else if (field.fieldtype == 'boolean') {
					//  dropdown boolean
					if (originalValue.toLowerCase() == 'true') {
						cell.contentObject.selectedIndex = 0;
					} else {
						cell.contentObject.selectedIndex = 1;
					}

				} else if (cell.cellType == 'dropdown') {
					if (dataGrid.cols[column].field.fieldchoicelist) {
						dataGrid.setCellValue(editRow, column, [value]);
					//	dax_bridge.getChoiceList(dataGrid.cols[column].field.fieldchoicelist, objectRef.choiceListHandler, cell.contentObject);	
					//	cell.contentObject.disabled = true;
					}
				} else {
					dataGrid.setCellValue(editRow, column, value);
				}			
			}
		}
	}
	
	this.inlineEditPopulateRecord = dax_dataGrid_inlineEditPopulateRecord;
	
	function dax_dataGrid_inlineEditSave(){

		// hide active popup if one exists
		if (this.dataGrid.inlineEdit.activeFloat)
			this.dataGrid.inlineEdit.activeFloat.onLeave();

	
		// find recordId row
		var editRow = this.dataGrid.inlineEdit.row;
		var editRecordId = this.dataGrid.inlineEdit.recordId;
	
		// show spinner
		this.dataGrid.inlineEdit.saveButtonNode.style.display = 'none';
		if (!this.dataGrid.inlineEdit.spinnerButtonNode) {
			this.dataGrid.inlineEdit.spinnerButtonNode = getTemplate('dax_datagrid_inlineedit_spinner').cloneNode(true);
			generateUniqueId(this.dataGrid.inlineEdit.spinnerButtonNode);
		}

		this.rows[editRow].cells[0].appendChild(this.dataGrid.inlineEdit.spinnerButtonNode);		
		this.dataGrid.inlineEdit.spinnerButtonNode.style.display = 'block';

		// record to be saved
		var fieldArray = [ ];
		var valueArray = [ ];
		
		// branch based on data type
		for (var colCount = this.dataGrid.startColumnForData; colCount < this.cols.length; colCount++) {
			var editCell = this.getCell(editRow, colCount);
			var editField = this.cols[colCount].field;
			switch(editField.fieldtype) {
				case 'picture':
				break;
				case 'date':
					editCell.contentObject.disabled = true;
					fieldArray.push(editField.fieldid);
					var currentValue = this.getCellValue(editRow, colCount);
					currentValue = doFormat(currentValue, editField.fieldformat, '4daf_default', 'date')
					valueArray.push(currentValue);
				break;
				case 'alpha':
				case 'text':
				default:
					if ((editCell.cellType == 'dropdown') || (editCell.cellType == 'checkbox') || (editCell.cellType == 'input')) {
						editCell.contentObject.disabled = true;
						fieldArray.push(editField.fieldid);
						valueArray.push(this.getCellValue(editRow, colCount));
					}
				break;
			}
		}
		
		if (editRecordId != -3)
			dax_bridge.modifyRecord(this.data.selection.tableid, fieldArray, valueArray, editRecordId, this.inlineEditSaveHandler, this);
		else
			dax_bridge.addRecord(this.data.selection.tableid, fieldArray, valueArray, this.inlineEditSaveHandler, this);
	}
	
	this.inlineEditSave = dax_dataGrid_inlineEditSave;

	function dax_dataGrid_inlineEditSaveHandler(http_response, dataGridRef) {
		if (http_response.readyState != 4)
			return false;

		dataGridRef.inlineEditClear();
		
	}
	
	this.inlineEditSaveHandler = dax_dataGrid_inlineEditSaveHandler;
	
	function dax_dataGrid_inlineEdit_fireCallback() {
		
		var tableId = this.boundTo.data.selection.tableid;
		var fieldId = this.boundTo.cols[this.parentCell.column].field.fieldid + '[' + this.parentCell.fieldrecordid + ']';
		var value = this.boundTo.getCellValue(this.parentCell.row, this.parentCell.column);
		
		dax_bridge.executeCallback(tableId, fieldId, value, 'onchange', this.boundTo.inlineEdit_callbackHandler, this.boundTo);
		
	}
	
	this.inlineEdit_fireCallback = dax_dataGrid_inlineEdit_fireCallback;
	
	function dax_dataGrid_inlineEdit_callbackHandler(callbackReply, dataGridRef) {
	
		var dataGrid = dataGridRef;
	
		// cycle through values and add them to the grid
		for (var callbackReplyValueCount = 0; callbackReplyValueCount < callbackReply.values.length; callbackReplyValueCount++) {
			var column = dataGrid.getColumnByFieldId(callbackReply.values[callbackReplyValueCount].fieldId);
			if (column) {
				var row = dataGrid.dataGrid.inlineEdit.row;
				var callbackValue = callbackReply.values[callbackReplyValueCount].value;
				
				if (dataGrid.rows[row].cells[column].cellType == 'dropdown') {
					// for dropdowns, find value in the list and assign it
					var dropdown = dataGrid.rows[row].cells[column].contentObject;
					for (var dropdownCount = 0; dropdownCount < dropdown.options.length; dropdownCount++) {
						if (dropdown.options[dropdownCount].value == callbackValue)
							dropdown.selectedIndex = dropdownCount;
					}
				} else {
					dataGrid.setCellValue(row, column, callbackValue);
				}
			}
		}	
	}
	
	this.inlineEdit_callbackHandler = dax_dataGrid_inlineEdit_callbackHandler;
	
	function dax_dataGrid_inlineEditClear() {
		
		if (!this.dataGrid.inlineEdit.active)
			return false;
		
		// hide active popup if one exists
		if (this.dataGrid.inlineEdit.activeFloat)
			this.dataGrid.inlineEdit.activeFloat.onLeave();
		
//		if (this.dataGrid.inlineEdit.recordId != -1) {
//			this.dataGrid.inlineEdit.editButtonNode.style.display = 'block';
//			this.setInternalStyleClass(this.dataGrid.inlineEdit.row, null, 'selected_cell', 'selected_cell');
//		} else {
//			this.setInternalStyleClass(this.dataGrid.inlineEdit.row, null, ' ', ' ');
//			this.rows[this.dataGrid.inlineEdit.row].cells[0].onmouseout();
//		}

		for (var colCount = this.dataGrid.startColumnForData; colCount < this.cols.length; colCount++) {
			this.setCellType(this.dataGrid.inlineEdit.row, colCount, 'html');
		}
		
		this.dataGrid.inlineEdit.active = false;
		if (this.dataGrid.inlineEdit.spinnerButtonNode)
			this.dataGrid.inlineEdit.spinnerButtonNode.style.display = 'none';

		if (this.dataGrid.inlineEdit.saveButtonNode)
			this.dataGrid.inlineEdit.saveButtonNode.style.display = 'none';

		//this.redraw(true, true);

		this.populate();
		
	}
	
	this.inlineEditClear = dax_dataGrid_inlineEditClear;

	function dax_dataCore_inlineEdit_choiceListHandler(http_response, objectRef) {
		
		if (http_response.readyState != 4)
			return;

		if (checkResponse(http_response))
			return false;
			
		var listItems = http_response.responseXML.getElementsByTagName('listitem');
		
		if (objectRef.options.length > 0)
			var currentValue = objectRef.options[objectRef.selectedIndex].value;
		else
			var currentValue = null;
		
		for (var listItemCount = 0; listItemCount < listItems.length; listItemCount++) {
			var value = listItems[listItemCount].firstChild.nodeValue;
			objectRef.options[listItemCount] = new Option(value, value);
			if (currentValue === value)
				objectRef.selectedIndex = listItemCount;
		}

		objectRef.disabled = false;
	}
	
	this.choiceListHandler = dax_dataCore_inlineEdit_choiceListHandler;
	
	function dax_dataGrid_selectRow(row, ctrlKeyPressed) {
		// if in inline edit mode, clear the edited row
		if ((this.dataGrid.inlineEdit.active) && (this.dataGrid.inlineEdit.row != row))
			this.inlineEditClear();
	
		var selectCurrentRow = false;
		var unselectCurrentRow = false;
		var unselectAllRows = false;

		if (this.dataGrid.selectionMode == 'none') {
		
			// no selection allowed
			return true;
		
		} else {
	
			// if ctrl is pressed:
			//      selected rows are removed from selection
			//      unselected rows are added to selection
			// otherwise 
			//      select the row
			if (ctrlKeyPressed) {
			
				if (this.rows[row].isSelected)
					unselectCurrentRow = true;
				else
					selectCurrentRow = true;
			
			} else {

				selectCurrentRow = true;
				unselectAllRows = true;
			
			}
			
			// for single-selection system, clear all other rows
			if (this.dataGrid.selectionMode == 'single')
				var unselectAllRows = true;
	
		}
			
		if ((unselectCurrentRow) || (unselectAllRows)) {	
	
			this.dataGrid.selectedRow = null;
			if (this.dataGrid.inlineEdit.editButtonNode)
				this.dataGrid.inlineEdit.editButtonNode.style.display = 'none';
			this.setSelected(row, null, false);
			this.removeRecordFromSelection(this.rows[row].recordId);

			// unselect all rows
			if (unselectAllRows) {
			
				// cycle through selected records and remove them
				for (recordId in this.data.selectedRecords) {
					if (recordId != 'count') {
						var rowNumber = this.getRowByRecordId(recordId);
						if (rowNumber !== null)
							this.setSelected(rowNumber, null, false);
					}
				}
			
				this.removeAllRecordsFromSelection();
			}	
		}
				
		if (selectCurrentRow) {
			
			if (this.dataGrid.inlineEdit.allow) {
				if (!this.dataGrid.inlineEdit.editButtonNode) {
					this.dataGrid.inlineEdit.editButtonNode = getTemplate('dax_datagrid_inlineedit_edit').cloneNode(true);
					generateUniqueId(this.dataGrid.inlineEdit.editButtonNode);
				}

				this.rows[row].cells[0].appendChild(this.dataGrid.inlineEdit.editButtonNode);
				if (this.dataGrid.inlineEdit.editButtonNode)
					this.dataGrid.inlineEdit.editButtonNode.style.display = 'block';
			}
				
			this.dataGrid.selectedRow = row;
			this.setSelected(row, null, true);
			this.addRecordToSelection(this.rows[row].recordId);
			this.dataGrid.lastSelectedRecordId = this.rows[row].recordId;
		} 	
	}
	
	this.selectRow = dax_dataGrid_selectRow;
	
	function dax_dataGrid_selectRowByRecordId(recordId){
		
		var rowNumber = this.getRowByRecordId(recordId);
		
		this.selectRow(rowNumber);
	
	}
	
	this.selectRowByRecordId = dax_dataGrid_selectRowByRecordId;
	
	function dax_dataGrid_onCellClick(row, column, event) {
		
		// call dev set function
		if ((column) && (column != 0))
			var cellClickReply = this.onDataCellClick(row, column, this.rows[row].recordId, this.cols[column].field);
		
		var rowClickReply = this.onDataRowClick(row, this.rows[row].recordId);

		if ((cellClickReply == false) || (rowClickReply == false))
			return false;

		// headers and footers are not selectable
		if ((this.rows[row].type == 'header') || (this.rows[row].type == 'footer'))
			return false;
			
		if ((column != 0) || (!this.dataGrid.useControlColumn)) {
			
			// is ctrl or command key pressed?
			if (browser.ie)
				var ctrlPressed = event.ctrlKey;
			else
				var ctrlPressed = ((event.ctrlKey) || (event.metaKey));

			// regular row click, anywhere except on control column, only allowed if recordId exists and it's not header or footer row
			if ((this.rows[row].type == 'default') && (this.rows[row].recordId))
				this.selectRow(row, ctrlPressed);
	
		} else {
			// control column click
			if (this.dataGrid.inlineEdit.allow) {
				if (!this.dataGrid.inlineEdit.active)
					this.inlineEdit(row);
				else
					this.inlineEditSave(row);
				
			}
		}

		return false;
	}
	
	this.onCellClick = dax_dataGrid_onCellClick;
	this.onRowClick = dax_dataGrid_onCellClick;
	
	function dax_dataGrid_onColumnClick(column) {
		
		// call dev set function
		var columnClickReply = this.onDataColumnClick(column);

		if (columnClickReply == false)
			return false;
		
		// don't do anything if this is control column
		if ((column == 0) && (this.dataGrid.useControlColumn))
			return false;
		
		// apply inverse sorting if it's the same field, if not start with asc sort
		if (!this.data.query.sortField) {
			var sortOrder = 'asc';
		} else if (this.cols[column].field.fieldid == this.data.query.sortField.fieldid) {
			var sortOrder = ((this.data.query.sortOrder == 'asc') ? 'desc' : 'asc');
		} else {
			var sortOrder = 'asc';
		}

		// remove last selected column style and set style for selected header
		if ((this.dataGrid.lastSelectedColumn) || (this.dataGrid.lastSelectedColumn == 0)){
			this.setInternalStyleClass(0, this.dataGrid.lastSelectedColumn, ''); 
		}
		
		this.dataGrid.lastSelectedColumn = column;
		
		this.setInternalStyleClass(0, column, 'dax_grid_cell_firstheader_selected'); 
		this.rows[0].cells[column].originalClassName = this.rows[0].cells[column].className;
		
		
		this.sort(this.cols[column].field.fieldid, sortOrder);
		
		if (this.dataGrid.inlineEdit.editButtonNode)
			this.dataGrid.inlineEdit.editButtonNode.style.display = 'none';

		// clear selection on sort
		this.removeAllRecordsFromSelection();
		
		// prevent default column actions
		//return false;
	}
	
	this.onColumnClick = dax_dataGrid_onColumnClick;

	function dax_dataGrid_onCellDblClick(row, column) {
		
		if ((this.rows[row].type == 'default') && (this.rows[row].recordId))
			this.editorEditRecord(this.rows[row].recordId);
		
	}
	
	this.onCellDblClick = dax_dataGrid_onCellDblClick;
	
	function dax_dataGrid_getRowByRecordId(recordId){
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			if (this.rows[rowCount].recordId == recordId)
				return rowCount;		
		}
		return null;
	}
	
	this.getRowByRecordId = dax_dataGrid_getRowByRecordId;
	
	function dax_dataGrid_getColumnByFieldId(fieldId){
		for (var colCount = 0; colCount < this.cols.length; colCount++) {
			if (this.cols[colCount].field)
				if (this.cols[colCount].field.fieldid == fieldId)
					return colCount;		
		}
	}
	
	this.getColumnByFieldId = dax_dataGrid_getColumnByFieldId;
	
	function dax_dataCore_getCustomValuesFrom4D() {
	
		var returnCustomValues = [ ];
		
		for (var customValuesCount = 0; customValuesCount < this.data.query.customValuesFrom4D.name.length; customValuesCount++) {
			var customValue = {
				name	: this.data.query.customValuesFrom4D.name[customValuesCount],
				value	: this.data.query.customValuesFrom4D.value[customValuesCount]
			}	
				
			returnCustomValues.push(customValue);
		}
		
		return returnCustomValues;
	
	}
	
	this.getCustomValuesFrom4D = dax_dataCore_getCustomValuesFrom4D;

	this.onBeforeHover = function(obj) { };
	this.onAfterHover = function(obj) { };
	
	this.enablePicturePreview = function(bool) {
		this.enablePicturePreview = bool;
	}
	
	// on populate handler
	this.onpopulate = function() {
	
		// update alt row colors based on data position
		if ((this.data.query.start % 2) == 1)
			this.setGridClass(null, 'dax_datagrid');
		else
			this.setGridClass(null, 'dax_datagrid dax_datagrid_inverse');

		var rowCount = 0;
		
		var queryResult = this.data.query.parsedData;
		
		// populate grid with results
		if (queryResult) {
			// loop through results
			if (queryResult.length > 0) {
				for (var rowCount = 0; rowCount < queryResult.length; rowCount++) {
					
					var currentRowNumber = this.dataGrid.startRowForData + rowCount;
					var skipRow = false;
					
					// check if row exists, if not skip this step
					if (!this.rows[currentRowNumber])
						skipRow = true;
						
					// don't show data on headers and footers
					if (this.rows[currentRowNumber].type != 'default')
						skipRow = true;
					
					if (!skipRow) {
					
						this.rows[currentRowNumber].recordId = queryResult[rowCount].recordid;
						
						if (this.data.selectedRecords[queryResult[rowCount].recordid])
							this.setSelected(currentRowNumber, null, true);
						else
							this.setSelected(currentRowNumber, null, false);
							
						for (var fieldCount = 0; fieldCount < this.data.selection.fieldsList.length; fieldCount++) {
							switch (this.data.selection.fieldsList[fieldCount].fieldtype) {
								case "picture":
									if (queryResult[rowCount].fields[fieldCount].value != "") {
										this.setCellValue(currentRowNumber, fieldCount + this.dataGrid.startColumnForData, "<a class='dax_grid_imagelink' href='#'>"+STR['view_image']+"</a>");
										
										var piclink = getElementsByClassName(this.getCell(currentRowNumber, fieldCount + this.dataGrid.startColumnForData), "dax_grid_imagelink", true);
										
										piclink[0].aContent = queryResult[rowCount].fields[fieldCount].value;
										piclink[0].parent = {
											dataGrid : this,
											picWidth : queryResult[rowCount].fields[fieldCount].width,
											picHeight : queryResult[rowCount].fields[fieldCount].height
										}
										
										piclink[0].onclick = NGridElement_link_onclick;
										
										if (this.dataGrid.enablePicturePreview) {
											piclink[0].onmouseover = function(aEvent){
											
												if (aEvent) 
													xpEvent(aEvent, false);
												
												this.parent.dataGrid.onBeforeHover(this);
												
												this.imageNode = getTemplate('image_preview_container');
												
												i = 1;
												while ($('image_preview_container' + i) != null) {
													i++;
												}
												
												this.imageNode.id = 'image_preview_container' + i;
												
												image = getElementByClassName(this.imageNode, 'image_preview');
												
												if (this.parent.height > this.parent.width) {
													image.style.height = "100px";
													image.style.width = Math.round(this.parent.width / (this.parent.height / 100)) + "px";
												}
												else {
													image.style.width = "100px";
													image.style.height = Math.round(this.parent.height / (this.parent.width / 100)) + "px";
												}
												
												var mousePosition = getMousePosition(aEvent);
												this.imageNode.style.left = mousePosition.x + 20 + "px";
												this.imageNode.style.top = mousePosition.y + 20 + "px";
												
												document.body.appendChild(this.imageNode);
												this.imageNode.style.zIndex = 9999;
												this.imageNode.style.position = 'absolute';
												
												image.src = this.aContent;
												
												this.parent.dataGrid.onAfterHover(this);
											};
											
											piclink[0].onmouseout = function(aEvent){
											
												document.body.removeChild(this.imageNode);
												
											};
										}
									}
									else 
										this.setCellValue(currentRowNumber, fieldCount + this.dataGrid.startColumnForData, queryResult[rowCount].fields[fieldCount].value);							
									break;
								default:
									this.setCellValue(currentRowNumber, fieldCount + this.dataGrid.startColumnForData, queryResult[rowCount].fields[fieldCount].value);							
							}
							this.rows[currentRowNumber].cells[fieldCount + this.dataGrid.startColumnForData].fieldrecordid = queryResult[rowCount].fields[fieldCount].fieldrecordid;
						}
					}
				}
			}
		}
		
		// erase remaining rows
		if (rowCount < this.rows.length) {
			for (var blankRowCount = rowCount + this.dataGrid.startRowForData; blankRowCount < this.rows.length; blankRowCount++) {

			this.setSelected(blankRowCount, null, false);

			for (var colCount = 0; colCount < this.rows[blankRowCount].cells.length; colCount++) {
					this.setCellValue(blankRowCount, colCount + this.dataGrid.startColumnForData, '');
				}					
			}
		}
		
		// redraw sort indicators
		if (this.data.query.sortField) {
			var columnNumber = this.getColumnByFieldId(this.data.query.sortField.fieldid);
			this.rows[0].cells[columnNumber].appendChild(this.dataGrid.nodes.headerSortGraphic);
			if (this.data.query.sortOrder == 'asc')
				this.dataGrid.nodes.headerSortGraphic.className = 'dax_datagrid_header_sort_graphic_asc';
			else
				this.dataGrid.nodes.headerSortGraphic.className = 'dax_datagrid_header_sort_graphic_desc';
			this.dataGrid.nodes.headerSortGraphic.style.display = 'inline';
		}
		
		this.dataRedraw();	
		this.onDataLoad();
		this.data.populating = false;
	}
	
	function dax_dataGrid_redraw() {
	
		// redraw viewport
		var visibleRecords = this.getVisibleRecordCount();

		this.viewport.scrollbars.ver.height = this.data.query.recordsInSelection;
		this.viewport.scrollbars.ver.visibleHeight = visibleRecords;

		this.viewport.scrollbars.ver.trackStep = visibleRecords;

		this.redraw();
		
	}
	
	this.dataRedraw = dax_dataGrid_redraw;

	function dax_dataGrid_showStatusBar(showStatusBar) {
	
		this.window.status.style.display = (showStatusBar) ? 'block' : 'none';
	
		if (this.data.preInitialized) {
			this.window.open();		
		}	
	}

	this.showStatusBar = dax_dataGrid_showStatusBar;
	
	// selection mode: multi, single, none
	function dax_dataGrid_setSelectionMode(selectionMode) {
		this.dataGrid.selectionMode = selectionMode;
	}
	
	this.setSelectionMode = dax_dataGrid_setSelectionMode;
	
	// window settings
	this.windowSettings.addRowsOnResize = true;

}

// attach calendar picker to date input field
// add inputField.dateFormat to specify date format, possible values are: "d1"  , "dMM-DD-YYYY" , "dDD-MM-YYYY",  "dDD-MMM-YYYY"
function dax_attachCalendarPicker(inputNode) {

	// initialize calendar float if this was not done yet
	if (!dax_bridge.calendarPickerFloat)
		dax_bridge.calendarPickerFloat = new dax_calendarPicker;

	// if focus is on input field, show the picker
	function dax_calendarPicker_onFocus(event) {

		var inputPosition = getObjectPosition(this);
		dax_bridge.calendarPickerFloat.style.top = inputPosition.y + this.offsetHeight + 'px';
		dax_bridge.calendarPickerFloat.style.left = inputPosition.x + 'px';
		dax_bridge.calendarPickerFloat.style.display = 'block';

		dax_bridge.calendarPickerFloat.boundTo = this;
		dax_bridge.calendarPickerFloat.setDateFromInput();

		Controller.setFocused(dax_bridge.calendarPickerFloat);
		
		this.onCalendarFocus();
		xpEvent(event, true, true);
		return false;
	}

	function dax_calendarPicker_onKeyUp(event) {
	
		dax_bridge.calendarPickerFloat.setDateFromInput();
		this.onCalendarKeyUp();
	
	}
	
	function dax_calendarPicker_blank() { }
	
	// attach events to the input node
	inputNode.onfocus = dax_calendarPicker_onFocus;
	inputNode.onkeyup = dax_calendarPicker_onKeyUp;

	inputNode.onCalendarFocus = dax_calendarPicker_blank;
	inputNode.onCalendarBlur = dax_calendarPicker_blank;
	inputNode.onCalendarKeyUp = dax_calendarPicker_blank;
	
}

function dax_calendarPicker() {

	var pickerDom = getTemplate('dax_calendarpicker').cloneNode(true);
	document.body.appendChild(pickerDom);
	generateUniqueId(pickerDom);
	
	this.currentYear = null;
	this.currentMonth = null;
	this.currentDate = null;
	
	this.currentDateNode = null;
	
	// initialize months and select current month
	var monthSelect = getElementByClassName(pickerDom, 'month_select');
	
	for (var monthCount = 0; monthCount < 12; monthCount++) {
		monthSelect.options[monthCount] = new Option(STR['calendar_month'][monthCount], monthCount);
	}
	
	// hide calendar if something else that input field is focused on
	pickerDom.blur = function(nullValue, event) {
		//if (Controller.focusedObject == this)
		//	return false;
		
		if (event) {
			var event = xpEvent(event);
			if ((event._target == this.boundTo) || (getParentByClassName(event._target, 'dax_calendarpicker'))) {
				return true;
			}
		}

		this.style.display = 'none';
		return false;
	}
	
	pickerDom.buildCalendarArray = function(m, y) {
		
		var dateArray = [ ];
		
		var days=[31,0,31,30,31,30,31,31,30,31,30,31];
		
		var oD = new Date(y, m-1, 1); 
		oD.od=oD.getDay()+1; 
		
		var todaydate=new Date() 
		var scanfortoday=(y==todaydate.getFullYear() && m==todaydate.getMonth()+1)? todaydate.getDate() : 0 
		days[1]=(((oD.getFullYear()%100!=0)&&(oD.getFullYear()%4==0))||(oD.getFullYear()%400==0))?29:28;
		
		for(i=1;i<=42;i++){
			var x=((i-oD.od>=0)&&(i-oD.od<days[m-1]))? i-oD.od+1 : ' ';
			dateArray.push (x);
		}
		
		return dateArray;
		
	}

	// place appropriate dates into the cells
	pickerDom.setDate = function(y, m, d) {
	
		if (y)
			this.currentYear = y;
		else
			var y = this.currentYear;
		
		if (m)
			this.currentMonth = m;
		else
			var m = this.currentMonth;
			
		if ((d) || (d === 0))
			this.currentDate = d;
		else
			var d = this.currentDate;
		
		getElementByClassName(this, 'year_input').value = y;
		getElementByClassName(this, 'month_select').selectedIndex = +m - 1;
		
		var dateArray = this.buildCalendarArray(m, y);
		var calendarElements = getElementsByClassName(getElementByClassName(this, 'dax_calendarpicker_calendar'), 'date', true);
		
		for (var dateCount = 0; dateCount < dateArray.length; dateCount++) {
			calendarElements[dateCount].innerHTML = dateArray[dateCount];
			calendarElements[dateCount].date = dateArray[dateCount];
			if ((dateArray[dateCount] == d) && (d !== null))
				this.setCurrentDateNode(calendarElements[dateCount]);
		}
	}
	
	pickerDom.setCurrentDateNode = function(node) {
	
		if (this.currentDateNode) {
			this.currentDateNode.className = this.currentDateNode.originalClassName;		
		}
		
		node.originalClassName = node.className;
		node.className += ' current';
		this.currentDateNode = node;
	}
	
	pickerDom.updateInputField = function() {
		var currentDate = this.currentYear + '-' + this.currentMonth + '-' + this.currentDate;
		var formattedDate = doFormat(currentDate, '4daf_default', this.dateFormat, 'date');
		this.boundTo.value = formattedDate;
	}
	
	pickerDom.setDateFromInput = function() {
		var formattedDate = doFormat(this.boundTo.value, this.boundTo.dateFormat, '4daf_default', 'date');
		if (formattedDate) {
			var dateArray = formattedDate.split('-');
			this.setDate(dateArray[0], dateArray[1], dateArray[2]);
		}
	}
	
	function dax_calendarPicker_onCellClick(event) {
	
		if (this.date == ' ')
			return false;
	
		this.boundTo.setDate(this.boundTo.currentYear, this.boundTo.currentMonth, this.date);
		this.boundTo.updateInputField();
	}
	
	function dax_calendarPicker_nextYear() {
		this.boundTo.setDate(+this.boundTo.currentYear + 1);
		this.boundTo.updateInputField();
	}
	
	function dax_calendarPicker_prevYear() {
		this.boundTo.setDate(+this.boundTo.currentYear - 1);
		this.boundTo.updateInputField();
	}

	function dax_calendarPicker_nextMonth() {
		var year = this.boundTo.currentYear;
		var month = this.boundTo.currentMonth;
		
		if (month + 1 > 12) {
			month = 1;
			year++;
		} else {
			month++;
		}
		
		this.boundTo.setDate(year, month);
		this.boundTo.updateInputField();
	}

	function dax_calendarPicker_prevMonth() {
		var year = this.boundTo.currentYear;
		var month = this.boundTo.currentMonth;

		if (month - 1 < 1) {
			month = 12;
			year--;		
		} else {
			month--;
		}

		this.boundTo.setDate(year, month);
		this.boundTo.updateInputField();
	}
	
	var nextYearButton = getElementByClassName(pickerDom, 'right_year_normal', true);
	var prevYearButton = getElementByClassName(pickerDom, 'left_year_normal', true);

	var nextMonthButton = getElementByClassName(pickerDom, 'right_month_normal', true);
	var prevMonthButton = getElementByClassName(pickerDom, 'left_month_normal', true);
		
	nextYearButton.onclick = dax_calendarPicker_nextYear;
	prevYearButton.onclick = dax_calendarPicker_prevYear;

	nextMonthButton.onclick = dax_calendarPicker_nextMonth;
	prevMonthButton.onclick = dax_calendarPicker_prevMonth;
	
	nextYearButton.boundTo = pickerDom;
	prevYearButton.boundTo = pickerDom;

	nextMonthButton.boundTo = pickerDom;
	prevMonthButton.boundTo = pickerDom;

	function dax_calendarPicker_onInputChange(){
		var year = getElementByClassName(this.boundTo, 'year_input').value;
		var month = getElementByClassName(this.boundTo, 'month_select').selectedIndex + 1;
		
		this.boundTo.setDate(year, month);
		this.boundTo.updateInputField();
	
	}
	
	var yearInput = getElementByClassName(pickerDom, 'year_input');
	
	yearInput.onchange = dax_calendarPicker_onInputChange;
	monthSelect.onchange = dax_calendarPicker_onInputChange;
	
	yearInput.boundTo = pickerDom;
	monthSelect.boundTo = pickerDom;
	
	// initialize cell click event
	var calendarElements = getElementsByClassName(getElementByClassName(pickerDom, 'dax_calendarpicker_calendar'), 'date', true);
	for (var cellCount = 0; cellCount < calendarElements.length; cellCount++) {
		calendarElements[cellCount].onclick = dax_calendarPicker_onCellClick;
		calendarElements[cellCount].boundTo = pickerDom;
	}
	
	return pickerDom;
}