// create new data filler area
function dax_dataFillerArea(selection, area) {

	this.data = {
		selection	: getTable(selection),
		recordId	: null,	
		isNew		: true,
		recordNodes	: new Object					// array with link to html fields based on field names, recordNodes[fieldName]
	}
	
	this.properties = {
		area		: area
	}
	
	// dev defined events
	function dax_dataFillerArea_blankDevFunction() { return true; }
	
	this.onBeforeSave = dax_dataFillerArea_blankDevFunction;
	this.onAfterSave = dax_dataFillerArea_blankDevFunction;
	this.onSaveHandler = dax_dataFillerArea_blankDevFunction;
	
	// look for 4DAF fields in the set area, and put the information into this.data.recordNodes
	function dax_dataFillerArea_parseArea() {
		
		// get all relevetant elements
		var pageObjects = getElementsByClassName (this.properties.area, '4daf_', true);
		
		this.data.recordNodes = new Object;
		
		// build the list of references
		for (var objCount = 0; objCount < pageObjects.length; objCount++) {
			// get object class
			var objectClass = pageObjects[objCount].className;
			// split class into elements to extract class name, this is for multi element classes
			var objectClassAr = objectClass.split(' ');
			var objectClassIndex = findInArray (objectClassAr, '4daf_', 0, false, true);
			var objectClassName = ReplaceString(objectClassAr[objectClassIndex], '.', ' ');
			var objectField = getField(objectClassName.slice(5));
				
			if (objectField) {
			
				var objectField = {
					node	:	pageObjects[objCount],
					field	:	objectField				
				}
				
				// link object to 4D field
				this.data.recordNodes[objectField.field.fieldid] = objectField;
			}
		}
	}
		
	this.parseArea = dax_dataFillerArea_parseArea;
	
	function dax_dataFillerArea_getRecordById(recordId) {

		var url = dax_bridge.url + "GetRecord?sessionid=" + dax_bridge.sessionId + "&tableid=" + this.data.selection.tableid + "&recordid=" + recordId;
		
		makeCall(url, this.getRecordHandler, this);
	
	}
	
	this.getRecordById = dax_dataFillerArea_getRecordById;
	
	function dax_dataFillerArea_getRecordHandler(http_response, dataFillerObject) {
	
		if (http_response.readyState != 4)
			return;

		var result = http_response.responseXML.getElementsByTagName("queryResult").item(0);
		if (!result)
			return;

		// parse reply		
		var resultRows = result.getElementsByTagName('row');

		for (i = 0; i < resultRows.length; i++) {
			var queryResult = [ ];
			var resultFields = resultRows[i].getElementsByTagName('field');
			for (j = 0; j < resultFields.length; j++) {
			
				var value = (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : ('');
			
				var field = {
					id : resultFields[j].getAttribute('id'),
					value : value,
					originalValue : (resultFields[j].childNodes.length > 0) ? (resultFields[j].firstChild.nodeValue) : (''),
					fieldrecordid : resultFields[j].getAttribute('fieldrecordid'),
					height : resultFields[j].getAttribute('height'),
					width : resultFields[j].getAttribute('width')
				}
				
			queryResult.push(field);
			
			}
		}
		
		dataFillerObject.data.recordId = resultRows[0].getAttribute('recordid');
		dataFillerObject.data.isNew = false;
			
		// we have the id, now find the data and place the object into the html object
		if (queryResult) {
			// loop through results
			if (queryResult.length > 0) {
				for (var fieldCount = 0; fieldCount < queryResult.length; fieldCount++) {
					if (dataFillerObject.data.recordNodes[queryResult[fieldCount].id]) {
						var pageObject = dataFillerObject.data.recordNodes[queryResult[fieldCount].id].node;
						// put the data into the cell
						if (pageObject.nodeName.toLowerCase() == 'img') {
							// picture type into img object
							pageObject.src = queryResult[fieldCount].value;
						} else if ((pageObject.nodeName.toLowerCase() == 'input') && (pageObject.type == 'checkbox')) {
							// text type into input object
							pageObject.checked = (queryResult[fieldCount].value == 'true') ? true : false;
						} else if (pageObject.nodeName.toLowerCase() == 'input') {
							// text type into input object
							pageObject.value = queryResult[fieldCount].value;
						} else if (pageObject.nodeName.toLowerCase() == 'div') {
							// text type into input object
							pageObject.innerHTML = queryResult[fieldCount].value;
						} else if (pageObject.nodeName.toLowerCase() == 'span') {
							// text type into input object
							pageObject.innerHTML = queryResult[fieldCount].value;
						} else if (pageObject.nodeName.toLowerCase() == 'textarea') {
							// text type into textarea object
							pageObject.innerHTML = queryResult[fieldCount].value;
						} else if (pageObject.nodeName.toLowerCase() == 'td') {
							// text type into td object
							pageObject.innerHTML = queryResult[fieldCount].value;
						}
					}
				}
			}
		}			
	}
	
	this.getRecordHandler = dax_dataFillerArea_getRecordHandler;
	
	// when created, automatically parse the area
	this.parseArea();
	
	function dax_dataFillerArea_saveRecord() {

		// get all relevetant elements
		if (this.data.isNew)
			var url = dax_bridge.url + "addRecord?sessionid="+connection.sessionId+"&tableid=" + this.data.selection.tableid;
		else
			var url = dax_bridge.url + "modifyRecord?sessionid="+connection.sessionId+"&tableid=" + this.data.selection.tableid + "&recordid=" + this.data.recordId;

		for (var recordId in this.data.recordNodes) {
		
			var fieldId = this.data.recordNodes[recordId].field.fieldid;
			var fieldNode = this.data.recordNodes[recordId].node;
		
			// check for supported object type and build save record string
			if ((fieldNode.nodeName.toLowerCase() == 'input') && (fieldNode.type == 'checkbox')){
				objectValue = fieldNode.checked;
			} else if (fieldNode.nodeName.toLowerCase() == 'input'){
				objectValue = fieldNode.value;
			} else if (fieldNode.nodeName.toLowerCase() == 'textarea') {
				objectValue = fieldNode.value;
			}
			
			// compose adding string
			url += "&field" + fieldId + "=" + objectValue;

		}

		alert(url);
		makeCall (url);
	}
	
	this.saveRecord = dax_dataFillerArea_saveRecord;
	
	function dax_dataFillerArea_newRecord(){
		this.data.recordId = null;
		this.data.isNew = true;
	}
	
	this.newRecord = dax_dataFillerArea_newRecord;
}