//Rudolf Psenicnik

// Image browser
function dataMatrix (targetNode, selectionName, header, imageField, contents, zoomContents, imageOrientation, aMargin, zoomLevel, aScrollMode, forceCol, forceRow, allowCellMouseInteraction, maxCharNumber, skipQueryOnStart)
{
	this.loaded = false;

	// check if valid table is passed
	this.table			= null;
	this.table			= getTable (selectionName);
	if (!this.table) {
		debugAlert ('Selection named ' + selectionName + ' not found.');
		return false;
	}

	dax_bridge.cacheFields (this.table); // cache field information

	this.sortField = null;
	this.imageField = null;
	this.headerFields = new Array();
	//this.contentFields = new Array();
	//this.contentZoomFields = new Array();
	this.autoRefresh = true;
	if (skipQueryOnStart)
		this.skipQueryOnStart = true;
	else
		this.skipQueryOnStart = false;

	this.customQueryOptionName = new Array();
	this.customQueryOptionValue = new Array();
	
	// dev defined functions
	this.onPopulateCustom = null;
	
	this.customHeader = null;
	this.customText = null;
	this.customZoomText = null;

	// flags
	this.needImageUpdate = false;
	this.imageUpdating = false;
	
	// use auto refresh
	function dataMatrix_useAutoRefresh(useAutoRefresh) {
		this.autoRefresh = useAutoRefresh;		
	}
	
	this.useAutoRefresh = dataMatrix_useAutoRefresh;
	
	// wrapper for setting relevant 4d fields and cell contents	
	function dataMatrix_setFields(header, imageField, contents, zoomContents)
	{
		var headerText = header;
		var contentsText = contents;
		var contentsZoomText = zoomContents;
		var blankString = '';

		for (var field = 0; field < this.table.fieldsList.length; field++) {

			if (header) {
		
				// auto select header?
				if (headerText != 'auto') {
			
					// parse through header and contents, extract field names
					var fieldName = '[' + this.table.tablename + ']' + this.table.fieldsList[field].fieldname;
					while (headerText.indexOf(fieldName) != -1) {
						headerText = headerText.replace (fieldName, blankString);
						this.headerFields.push(this.table.fieldsList[field]);
					}

					// if something is left inside header text or there are multiple fields, it's  a custom header
					if ((headerText) || (this.headerFields.length > 1))
						this.customHeader = header;
					else
						this.customHeader = null;
				
				} else {
				
					if ((this.headerFields.length == 0) && (this.table.fieldsList[field].fieldtype != 'picture'))
						this.headerFields.push(this.table.fieldsList[field]);
				
				}
			}
			
			if (contents) {
			
				if (contentsText) {
				
					while (contentsText.indexOf(fieldName) != -1) {
						contentsText = contentsText.replace (fieldName, blankString);
						//this.contentFields.push(this.table.fieldsList[field]);
					}

				}
				
				// if something is left inside content text or there are multiple fields, it's  a custom content string
				//if (contentsText)// || (this.contentFields.length > 1))
					this.customText = contents;		
				//else
				//	this.customText = null;
				
			}
			
			if (zoomContents) {
			
				if (contentsZoomText) {
				
					while (contentsZoomText.indexOf(fieldName) != -1) {
						contentsZoomText = contentsZoomText.replace (fieldName, blankString);
						//this.contentZoomFields.push(this.table.fieldsList[field]);
					}
				
				}
				
				// if something is left inside zoom content text or there are multiple fields, it's  a custom zoom contents string
				//if (contentsZoomText)// || (this.contentZoomFields.length > 1))
					this.customZoomText = zoomContents;	
				//else
				//	this.customZoomText = null;

			}
				
			if (imageField) {
			
				// auto set image field
				if (imageField = 'auto') {
					if ((!this.imageField) && (this.table.fieldsList[field].fieldtype == 'picture'))
						this.imageField = this.table.fieldsList[field];
				}
			}
		}
	}

	this.setFields = dataMatrix_setFields;
	
	this.setFields(header, imageField, contents, zoomContents);
			
	// get templates
	containerNode = getTemplate('imagebrowsercontainer');
	this.container = containerNode.cloneNode(true);
	generateUniqueId (this.container);
	this.container.boundTo = this;
	
	this.viewport = new ViewPort();

	this.viewport.dom.className += ' dax_matrix_viewport';
	
	this.parent = targetNode;
	this.initializing = true;
	this.hibernating = false;	// suspended state
	this.populating = false;	// in process of making call to 4D and processing response
	
	this.queryStart = 1;
	this.totalRecords = 0;
	this.updatingTabsFieldId = null;
	// when using tabs with query name
	this.queryName = null;
	if (maxCharNumber)
		this.maxCharNumber = maxCharNumber;
	else
		this.maxCharNumber = null;
		
	// set parameters
	this.colCount = 2;
	this.rowCount = 2;
	this.cellMargin = 10;
	this.defMargin = 10;
	this.allowEditor = true;
	
	// force number of rows and columns
	this.forceCol = null;
	this.forceRow = null;
	
	if (forceCol)
		if (!(isNaN(forceCol)))
			this.forceCol = forceCol;
	if (forceRow)
		if (!(isNaN(forceRow)))
			this.forceRow = forceRow;

	// sets scroll mode, vertical or horizontal
	this.scrollMode = 'ver';
	this.zoomOnClick = true;
	
	// sets cell margin
	if ((aMargin) && (!(isNaN(aMargin)))) {
		this.cellMargin = aMargin;
		this.defMargin = aMargin;
	}
	
	// margin for low level zooms
	this.lowLevelMargins = ['5', '10', '10'];
		
	if (aScrollMode)
		if (aScrollMode == 'hor')
			this.scrollMode = 'hor';
	
	this.hoverCell = null;
	this.zoomInCell = null;
	this.selectedCell = null;
	this.cells = new Array();

	this.customQueryFields = new Array();
	this.customQueryValues = new Array();
	
	this.sortingDone = true;
	
	// keep lock when cells are animated, so other actions won't be triggered
	this.animating = false;
	
	if (allowCellMouseInteraction) {
		if (allowCellMouseInteraction == true)
			this.hideCellLock = true;
		else
			this.hideCellLock = false;
	} else {
		this.hideCellLock = false;
	}
	
	// user defined actions
	this.onCellHover = function (cellRef) { };
	this.onCellLeave = function (cellRef) { };
	this.onCellClick = function (cellRef) { };
	this.onCellDblClick = function (cellRef) { };

	// set zoom level
	this.zoomLevel = 4;
	if ((zoomLevel) & (!(isNaN(zoomLevel))))
		this.zoomLevel = zoomLevel;

	function dataMatrix_setMaxCharNumber(maxChar) {
		this.maxCharNumber = maxChar;
	}
		
	function dataMatrix_changeLayout(imageOrientation)
	{
		
		if (!imageOrientation) {
			this.imageOrientation = 'top';
		} else {
			this.imageOrientation = imageOrientation;
		}

		switch (imageOrientation) {

			case 'bottom':
				this.imageOrientation = 'bottom';
			break;

			case 'left':
				this.imageOrientation = 'left';
			break;
			
			case 'right':
				this.imageOrientation = 'right';
			break;		
			
			case 'top':
			default:
				this.imageOrientation = 'top';
			break;
		}

		this.cellNode = getTemplate('dax_matrix_cell_' + this.imageOrientation);
	}	
		
	this.changeLayout = dataMatrix_changeLayout;	
		
	this.changeLayout(imageOrientation);	
	
	// reapply object main settings
	function dataMatrix_reInit(selectionName, header, imageField, contents, zoomContents, layout, aMargin, zoomLevel, aScrollMode, forceCol, forceRow, allowCellMouseInteraction)
	{
		
		// select new table
		if (selectionName) {
			var newTable = getTable (selectionName);
			if (newTable) {
				this.table = newTable;
				this.imageQuery.table = newTable;
				this.imageQuery.queryId = null;
				dax_bridge.cacheFields(newTable);				
			}
		}

		// update fields
		this.setFields(header, imageField, contents, zoomContents);
		
		// update layout
		if (layout)
			this.changeLayout (layout);
		
		// force number of rows and columns
		if (forceCol)
			if (!(isNaN(forceCol)))
				this.forceCol = forceCol;
				
		if (forceRow)
			if (!(isNaN(forceRow)))
				this.forceRow = forceRow;

		// sets cell margin
		if (aMargin) {
			if (!(isNaN(aMargin))) {
				this.cellMargin = aMargin;
				this.defMargin = aMargin;
			}
		}
		
		if (aScrollMode)
			if (aScrollMode == 'hor')
				this.scrollMode = 'hor';

		if (allowCellMouseInteraction) {
			if (allowCellMouseInteraction == true)
				this.hideCellLock = true;
			else
				this.hideCellLock = false;
		} else {
			this.hideCellLock = false;
		}
				
		this.updateToolbarDropdowns();		
		this.refresh(true);
		
	}

	this.reInit = dataMatrix_reInit;
	
	// calculate rows & columns based on zoom level
	function dataMatrix_calcZoom()
	{

		if ((this.forceCol) && (this.forceRow)) {

			// force number of rows and columns
			this.rowCount = this.forceRow;
			this.colCount = this.forceCol;
		
		} else {
	
			// min zoom level for matrix mode
			if ((this.zoomLevel < 3) && (this.mode == 'matrix'))
				this.zoomLevel = 3;
		
			this.totalWidth = (this.scrollMode == 'ver') ? (this.parent.offsetWidth - 17) : (this.parent.offsetWidth);
			this.totalHeight = (this.scrollMode == 'hor') ? (this.parent.offsetHeight - 17) : (this.parent.offsetHeight);

			// modify margins for low level cells
			if ((this.zoomLevel < this.lowLevelMargins.length) && (!this.defMargin))
				this.cellMargin = +this.lowLevelMargins[this.zoomLevel];
			else
				this.cellMargin = +this.defMargin;

			// calculate desired cell size
			if ((this.imageOrientation == 'top') || (this.imageOrientation == 'bottom'))
				var cellWidth = (+this.cellMargin * 2) + (this.zoomLevel * 40);
			else
				var cellWidth = (+this.cellMargin * 2) + (this.zoomLevel * 80);
			
			var cellHeight = (+this.cellMargin * 2) + (this.zoomLevel * 30);
			
			// check how many rows and columns we should have
			this.rowCount = (Math.ceil (this.totalWidth / cellWidth) > 0) ? (Math.ceil (this.totalWidth / cellWidth)) : 1;
			this.colCount = (Math.ceil (this.totalHeight / cellHeight) > 0) ? (Math.ceil (this.totalHeight / cellHeight)) : 1;	
			
		}		
	}
	
	this.calcZoom = dataMatrix_calcZoom;

	// highlights specific cell
	function dataMatrix_highlight(cellRef, aInterval, aIncrement, aCount)
	{

		// make sure we don't get stuck in endless loop
		if (!aCount)
			aCount = 1;
		else
			aCount++;
	
		this.hoverCell = cellRef;
		
		// calculate resizing values
		var aInterval = isDefined(aInterval) ? aInterval : 1;
		var heightIncrement = Math.abs(Math.ceil((+cellRef.highHeight - +cellRef.normalHeight) / aIncrement));
		var widthIncrement = Math.abs(Math.ceil((+cellRef.highWidth - +cellRef.normalWidth) / aIncrement));

		// set minimum values
		if (heightIncrement < 2)
			heightIncrement = 2;
		
		if (widthIncrement < 2)
			widthIncrement = 2;

		var needsUpdate = false;
		
		var heightIncrement = Math.min(heightIncrement, cellRef.highHeight - cellRef.clientHeight);
		var widthIncrement = Math.min(widthIncrement, cellRef.highHeight - cellRef.clientHeight);

		// change width & height, and move object accordingly
		if (cellRef.clientHeight < cellRef.highHeight) {
			setStyle(cellRef, "height", (cellRef.clientHeight) + heightIncrement + "px");
			setStyle(cellRef, "top", (cellRef.offsetTop) - (heightIncrement / 2) + "px");
			
			needsUpdate = true;
		}

		if (cellRef.clientWidth < cellRef.highWidth) {
			setStyle(cellRef, "width", (cellRef.clientWidth) + widthIncrement + "px");
			setStyle(cellRef, "left", (cellRef.offsetLeft) - (widthIncrement / 2) + "px");
			needsUpdate = true;
		}
		
		// current width change - total width change  * 100
		var opacityValue = (Math.floor(Math.abs((cellRef.highHeight - cellRef.clientHeight) - (cellRef.highHeight - cellRef.boundTo.cellHeight)) * 1.5)); 
		
		if (opacityValue < 10)
			opacityValue = '' + '0' + opacityValue;
		
		//console.log (opacityValue);
		
		setStyle(cellRef.overlayNode, 'opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, '-moz-opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, 'filter', 'alpha(opacity = ' + opacityValue + ')');
		
		if ((needsUpdate) && (aCount < aIncrement))
			cellRef.timeoutId = setTimeout(function() { cellRef.boundTo.highlight(cellRef, aInterval, aIncrement, aCount); }, aInterval);

	}

	this.highlight = dataMatrix_highlight;
	
	// dehighlights specific cell, returns it to normal state
	function dataMatrix_dehighlight(cellRef)
	{
	
		if (cellRef.timeoutId) {
			window.clearTimeout (cellRef.timeoutId);
			cellRef.timeoutId = null;
		}
	
		cellRef.style.top = ((+this.cellHeight + (+this.cellMargin * 2)) * (+cellRef.col)) + (+this.cellMargin) + 'px';
		cellRef.style.left = ((+this.cellWidth + (+this.cellMargin * 2)) * (+cellRef.row)) + (+this.cellMargin) + 'px';
		cellRef.style.height = +this.cellHeight + 'px';
		cellRef.style.width = +this.cellWidth + 'px';

		setStyle(cellRef.overlayNode, 'opacity', '0');
		setStyle(cellRef.overlayNode, '-moz-opacity', '0');
		setStyle(cellRef.overlayNode, 'filter', 'alpha(opacity = 0)');
	
		
	}
	
	this.dehighlight = dataMatrix_dehighlight;

	// zooms in specific cell
	function dataMatrix_cellzoomin(cellRef, aInterval, aIncrement, aCount)
	{

		// make sure we don't get stuck in endless loop
		if (!aCount)
			aCount = 1;
		else
			aCount++;
			
		this.zoomInCell = cellRef;
		cellRef.boundTo.animating = true;
		
		// calculate resizing values
		var aInterval = isDefined(aInterval) ? aInterval : 1;
		var heightIncrement = Math.abs(Math.ceil((+cellRef.boundTo.totalHeight - +cellRef.highHeight) / aIncrement));
		var widthIncrement = Math.abs(Math.ceil((+cellRef.boundTo.totalWidth - +cellRef.highWidth) / aIncrement));

		var topIncrement = Math.floor(+cellRef.highTop / aIncrement);
		var leftIncrement = Math.floor(+cellRef.highLeft / aIncrement);
		
		var needsUpdate = false;
		
		var heightIncrement = Math.min(heightIncrement, cellRef.boundTo.totalHeight - cellRef.clientHeight);
		var widthIncrement = Math.min(widthIncrement, cellRef.boundTo.totalWidth - cellRef.clientWidth);

		var topIncrement = Math.min(topIncrement, cellRef.offsetTop);
		var leftIncrement = Math.min(leftIncrement, cellRef.offsetLeft);
		
		// change width & height, and move object accordingly
		if (cellRef.clientHeight < cellRef.boundTo.totalHeight) {
			setStyle(cellRef, "height", (cellRef.clientHeight) + heightIncrement + "px");
			needsUpdate = true;
		}

		if (cellRef.clientWidth < cellRef.boundTo.totalWidth) {
			setStyle(cellRef, "width", (cellRef.clientWidth) + widthIncrement + "px");
			needsUpdate = true;
		}
	
		if (cellRef.offsetTop > 0) {
			setStyle(cellRef, "top", (cellRef.offsetTop) - (topIncrement) + "px");
		}

		if (cellRef.offsetLeft > 0) {
			setStyle(cellRef, "left", (cellRef.offsetLeft) - (leftIncrement) + "px");
		}

		
		// opacity change for main element
		var opacityValue = Math.floor(100 - ((cellRef.clientWidth / cellRef.boundTo.totalWidth) * 100));
		
		if (opacityValue < 10)
			opacityValue = '' + '0' + opacityValue;
		
		setStyle(cellRef.overlayNode, 'opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, '-moz-opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, 'filter', 'alpha(opacity = ' + opacityValue + ')');
		
		if ((needsUpdate) && (aCount < aIncrement)) {
			cellRef.timeoutId = setTimeout(function() { cellRef.boundTo.cellzoomin(cellRef, aInterval, aIncrement, aCount); }, aInterval);
		} else {
			cellRef.boundTo.animating = false;
			this.redrawToolbar();
			
			// set cell to target position and size			
			cellRef.style.top = '0px';
			cellRef.style.left = '0px';
			cellRef.style.height = +cellRef.boundTo.viewport.dom.clientHeight + 'px';
			cellRef.style.width = +cellRef.boundTo.viewport.dom.clientWidth + 'px';
		}
	}

	this.cellzoomin	= dataMatrix_cellzoomin;

	// zooms in specific cell
	function dataMatrix_cellzoomout(cellRef, aInterval, aIncrement, aCount)
	{

		// make sure we don't get stuck in endless loop
		if (!aCount)
			aCount = 1;	
		else
			aCount++;
	
		cellRef.boundTo.animating = true;

		// calculate resizing values
		var aInterval = isDefined(aInterval) ? aInterval : 1;
		var heightIncrement = Math.abs(Math.ceil((+cellRef.boundTo.totalHeight - +cellRef.highHeight) / aIncrement));
		var widthIncrement = Math.abs(Math.ceil((+cellRef.boundTo.totalWidth - +cellRef.highWidth) / aIncrement));

		var topIncrement = Math.floor(+cellRef.highTop / aIncrement);
		var leftIncrement = Math.floor(+cellRef.highLeft / aIncrement);
		
		var needsUpdate = false;
		
		var heightIncrement = Math.min(heightIncrement, cellRef.clientHeight - cellRef.highHeight);
		var widthIncrement = Math.min(widthIncrement, cellRef.clientWidth - cellRef.highWidth);

		var topIncrement = Math.min(topIncrement, cellRef.highTop);
		var leftIncrement = Math.min(leftIncrement, cellRef.highLeft);
		
		
		if (cellRef.offsetTop < cellRef.highTop) {
			setStyle(cellRef, "top", (cellRef.offsetTop) + (topIncrement) + "px");
		}

		if (cellRef.offsetLeft < cellRef.highLeft) {
			setStyle(cellRef, "left", (cellRef.offsetLeft) + (leftIncrement) + "px");
		}

		// change width & height, and move object accordingly
		if (cellRef.clientHeight > cellRef.highHeight) {
			setStyle(cellRef, "height", (cellRef.clientHeight) - heightIncrement + "px");
			needsUpdate = true;
		}

		if (cellRef.clientWidth > cellRef.highWidth) {
			setStyle(cellRef, "width", (cellRef.clientWidth) - widthIncrement + "px");
			needsUpdate = true;
		}

		
		// opacity change for main element
		
		/*
		var opacityValue = Math.floor((cellRef.offsetTop / cellRef.highTop) * 75);
//		var opacityValueOther = Math.floor((cellRef.offsetTop / cellRef.highTop) * 75);
		
		if (opacityValue < 10)
			opacityValue = '' + '0' + opacityValue;
		
		//console.log (opacityValue);
		
		setStyle(cellRef.overlayNode, 'opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, '-moz-opacity', '0.' + opacityValue);
		setStyle(cellRef.overlayNode, 'filter', 'alpha(opacity = ' + opacityValue + ')');
		
		*/
		
		if ((needsUpdate) && (aCount < aIncrement)) {
			cellRef.timeoutId = setTimeout(function() { cellRef.boundTo.cellzoomout(cellRef, aInterval, aIncrement, aCount); }, aInterval);
		} else {
			for (var cellCount = 0; cellCount < this.cells.length; cellCount++) {
				if (this.cells[cellCount].recordId)
					this.cells[cellCount].style.display = 'block';
			}
			this.zoomInCell = null;
			cellRef.style.top = ((+this.cellHeight + (+this.cellMargin * 2)) * (+cellRef.col)) + (+this.cellMargin) + 'px';
			cellRef.style.left = ((+this.cellWidth + (+this.cellMargin * 2)) * (+cellRef.row)) + (+this.cellMargin) + 'px';
			cellRef.style.height = +this.cellHeight + 'px';
			cellRef.style.width = +this.cellWidth + 'px';

			// show or hide scrollbars
			if ((cellRef.boundTo.scrollMode == 'ver') && (cellRef.boundTo.viewport.vscrollbar.visible))
				cellRef.boundTo.viewport.vscrollbar.dom.style.display = 'block';
			else if ((cellRef.boundTo.scrollMode == 'hor') && (cellRef.boundTo.viewport.hscrollbar.visible))
				cellRef.boundTo.viewport.hscrollbar.dom.style.display = 'block';
			else {
				cellRef.boundTo.viewport.vscrollbar.dom.style.display = 'none';
				cellRef.boundTo.viewport.vscrollbar.dom.style.display = 'none';
			}

			
			setStyle(cellRef.overlayNode, 'opacity', '0');
			setStyle(cellRef.overlayNode, '-moz-opacity', '0');
			setStyle(cellRef.overlayNode, 'filter', 'alpha(opacity = 0)');
			
			//resize image
			if (cellRef.boundTo.imageField) {
				cellRef.imageNode.style.width = cellRef.horPerc + '%';
				cellRef.imageNode.style.height = cellRef.verPerc + '%';
			}
			
			if (cellRef.textNode)
				cellRef.textNode.innerHTML = cellRef.textValue;

			cellRef.boundTo.animating = false;
			this.redrawToolbar();
		}
	}

	this.cellzoomout = dataMatrix_cellzoomout;
	
	// image refresh, separate from data refresh to avoid safari crash
	function dataMatrix_refreshImages(notFirstCall)
	{
		
		if (this.imageRefreshTimeout)
			window.clearTimeout(this.imageRefreshTimeout);
		
		if (!notFirstCall) {
			imgBrw = this;
			if (browser.safari) {
				imgBrw.imageRefreshTimeout = setTimeout(function() { imgBrw.refreshImages(true); }, 500);	
			} else {
				imgBrw.imageRefreshTimeout = setTimeout(function() { imgBrw.refreshImages(true); }, 100);
			}
		} else {	

			if (!this.needImageUpdate)
				return null;
			else
				this.needImageUpdate = false;

			// assign images
			for (var cellCount = 0; cellCount < this.cells.length; cellCount++) {
				if (this.cells[cellCount].imageURL)
					this.cells[cellCount].imageNode.src = this.cells[cellCount].imageURL;
				else
					this.cells[cellCount].imageNode.src = '/DAX/images/dax_watermark.png';
			}
		}
	}
	
	this.refreshImages = dataMatrix_refreshImages;
	
	// starts new custom query
	function dataMatrix_fetchData(fieldId, queryValue, optionName, optionValue)
	{

		// clear query if one already exists
		if (this.imageQuery)
			if (this.imageQuery.queryId) {
				clearQuery(this.imageQuery.queryId);
				this.imageQuery.queryId = null;			
			}		

		if (fieldId) {
			this.customQueryFields = new Array;
			this.customQueryValues = new Array;
			
			for (var arCount = 0; arCount < fieldId.length; arCount++) {
				var queryField = getField (this.table.tableid, fieldId[arCount]);
				if (queryField) {
					this.customQueryFields.push(queryField);
					this.customQueryValues.push(queryValue[arCount]);
				}
			}
		}
		
		if (optionName) {
			this.customQueryOptionName = optionName.slice();
			this.customQueryOptionValue = optionValue.slice();
		}
		
		this.queryStart = 1;
		this.refresh(true);
	
	}

	this.fetchData = dataMatrix_fetchData;
	
	function dataMatrix_createCells()
	{

		// remove existing cells
		if (this.cells.length > 0) {
			for (var ce = 0; ce < this.cells.length; ce++) {
				this.cells[ce].boundTo = null;
	
				this.cells[ce].imageNode = null;
				this.cells[ce].imageContainerNode = null;
				this.cells[ce].overlayNode = null;
				this.cells[ce].labelNode = null;
				this.cells[ce].textContainerNode = null;
				this.cells[ce].textNode = null;
				this.cells[ce].spacerNode2 = null;
				
				this.cells[ce].toString = null;
				this.cells[ce].onmouseover = null;
				this.cells[ce].onmouseout = null;
				this.cells[ce].onclick = null;
				this.cells[ce].ondblclick = null;
				
				this.container.removeChild (this.cells[ce]);
			}		
		}
	
		this.cells = new Array();
	
		// create cell rows
		for (col = 0; col < this.colCount; col++) {
			for (row = 0; row < this.rowCount; row++) {
				var cell = this.cellNode.cloneNode(true);
				this.container.appendChild (cell);
				generateUniqueId (cell);
				this.cells.push(cell);
				
				cell.row = row;
				cell.col = col;
				
				cell.boundTo = this;
				
				//cell.style.top = (+this.cellHeight * (+this.cellMargin * 2) * +col) + (+this.cellMargin) + 'px';
				//cell.style.left = (+this.cellWidth * (+this.cellMargin * 2) * +row) + (+this.cellMargin) + 'px';
				//cell.style.height = +this.cellHeight + 'px';
				//cell.style.width = +this.cellWidth + 'px';
				
				cell.style.display = 'none';
				
				//if (row == (this.rowCount - 1))
				//	cell.style.clear = 'right';
				
				cell.toString = function() {
					return '[object ImageBrowserCell]';
				}
				
				cell.onmouseover = function (aEvent) {
					
					if (!this.boundTo.hideCellLock) {
					
						// start animation if one is already not in progress
						if ((!this.boundTo.animating) && (!this.boundTo.zoomInCell) && (this.recordId)) {
							if (!browser.safari)
								this.boundTo.highlight(this, 12, 10);
							else
								this.boundTo.highlight(this, 1, 1);

						}
					}
					// call custom action
					this.boundTo.onCellHover (this, aEvent);
					
				}
				
				cell.onmouseout = function (aEvent) {
					if (!this.boundTo.hideCellLock) {
						if ((!this.boundTo.animating) && (!this.boundTo.zoomInCell))
							this.boundTo.dehighlight(this);
					}
					// call custom action
					this.boundTo.onCellLeave (this, aEvent);
				}
				
				cell.onclick = function (aEvent) {
				
					if (!this.recordId)
						return;
						
					if (!this.boundTo.zoomInCell) {
				
						// deselect this cell if already selected
						if (this.boundTo.selectedCell == this) {
							this.className = 'dax_matrix_cell_' + this.boundTo.imageOrientation + ' fourdaf_matrix_cell_' + this.boundTo.imageOrientation;
							this.boundTo.selectedCell = null;
						} else {
							// unselect previously selected cell
							if (this.boundTo.selectedCell)
								this.boundTo.selectedCell.className = 'dax_matrix_cell_' + this.boundTo.imageOrientation + ' fourdaf_matrix_cell_' + this.boundTo.imageOrientation;
							// select this cell
							this.className = 'dax_matrix_cell_' + this.boundTo.imageOrientation + '_selected fourdaf_matrix_cell_' + this.boundTo.imageOrientation + '_selected';
							this.boundTo.selectedCell = this;
						}

					this.boundTo.redrawToolbar();
					
					}
					
					// call custom action
					this.boundTo.onCellClick (this, aEvent);
				}
				
				cell.ondblclick = function (aEvent) {
				
					if (!this.recordId)
						return;	
						
					// call custom action
					this.boundTo.onCellDblClick (this, aEvent);

					this.boundTo.redrawToolbar();
					
					// if editing is allowed
					if (this.boundTo.allowEditor) {
					
						if (this.boundTo.table.fieldsDetail.length == 0)
							return false;
					
						// check if editor exists, if not create it
						if (isDefined(this.boundTo.editor) && isDefined(this.boundTo.editor.record)) {
							// exists
							this.boundTo.editor.edit();
						} else {
							// doesn't exist
							this.boundTo.getEditor();
							setStyle(this.boundTo.editor.getDOM(), "display", "block");
							var editor = this.boundTo.editor;
							var auto = editor.prv.mAutoReload;
							if (auto > 0) {
							  setTimeout(function() {editor.reload();}, 500);
							}
						}
												
						// call editor and display data
						this.boundTo.editor.pull(this.boundTo.table.tableid, this.recordId);
						
					}
				}
				
				// link cell elements
				cell.imageNode = getElementByClassName (cell, 'imagebrowsercellimage', true);
				cell.imageContainerNode = getElementByClassName (cell, 'dax_matrix_image_container', true);
				cell.overlayNode = getElementByClassName (cell, 'dax_matrix_overlay', true);
				cell.labelNode = getElementByClassName (cell, 'dax_matrix_caption', true);
				cell.textContainerNode = getElementByClassName (cell, 'imagebrowsercellfulltextcontainer', true);
				cell.textNode = getElementByClassName (cell, 'imagebrowserfulltext', true);
				cell.spacerNode2 = getElementByClassName (cell, 'dax_matrix_cell_spacer2', true);
				
				if (this.hideCellLock) {
					cell.overlayNode.style.display = 'none';
				} else {
					cell.overlayNode.style.display = 'block';
				}
				
				// show or hide elements based on mode
				if ((this.imageOrientation == 'top') || (this.imageOrientation == 'bottom')) {
					// header visible?
					if ((this.headerFields[0]) || (this.customHeader)) {
						cell.imageNode.style.display = 'block';
						cell.imageContainerNode.style.display = 'block';
						cell.labelNode.style.display = 'block';
						
						this.cellWidthPer = 90;
						this.cellHeightPer = 75;
						
//						cell.textContainerNode.style.display = 'none';
					} else {
						cell.imageNode.style.display = 'block';
						cell.imageContainerNode.style.display = 'block';
						cell.labelNode.style.display = 'none';
						
						this.cellWidthPer = 90;
						this.cellHeightPer = 100;
						
//						cell.textContainerNode.style.display = 'none';

						cell.imageContainerNode.style.height = '100%';

						
					}
				} else {
					// image visible?
					if (this.imageField) {
						cell.imageNode.style.display = 'block';
						cell.imageContainerNode.style.display = 'block';
						cell.labelNode.style.display = 'none';
						cell.textContainerNode.style.display = 'block';
						cell.spacerNode2.style.display = 'block';
						
						cell.textContainerNode.style.width = '60%';
						
						this.cellWidthPer = 25;
						this.cellHeightPer = 90;
						
					} else {
						cell.labelNode.style.display = 'none';
						cell.textContainerNode.style.display = 'block';
						
						cell.imageNode.style.display = 'none';
						cell.imageContainerNode.style.display = 'none';
						cell.textContainerNode.style.width = '90%';
						cell.spacerNode2.style.display = 'none';
					}
				}
			}		
		}

		/*
		
		if (browser.safari) {
			var dx = +this.window.dom.r.offsetWidth + +this.window.dom.l.offsetWidth;
			this.parent.style.width = +this.window.dom.offsetWidth - dx + "px";

			alert (dx);
			
			var dy = this.window.dom.b.offsetHeight + this.window.dom.t.offsetHeight;
			var tbDy = 0;
			
			for (var i in this.window.toolbars) {
				tbDy += this.window.toolbars[i].offsetHeight;
			}

			if (this.window.dom.offsetHeight - dy - tbDy - this.window.status.offsetHeight > 0)
				this.parent.style.height = this.window.dom.offsetHeight - dy - tbDy -	this.window.status.offsetHeight + "px";

			alert (this.window.dom.offsetHeight - dy - tbDy -	this.window.status.offsetHeight + "px");

				
		}
		
		*/
		
		// recalculate values
		this.totalWidth = (this.scrollMode == 'ver') ? (this.parent.offsetWidth - 17) : (this.parent.offsetWidth);
		this.totalHeight = (this.scrollMode == 'hor') ? (this.parent.offsetHeight - 17) : (this.parent.offsetHeight);

		// resize container to match parent
		this.container.style.width = this.totalWidth + (+this.cellMargin * 3)+ 'px';
		this.container.style.height = this.totalHeight + (+this.cellMargin * 3)+ 'px';

		// resize viewport to match parent
		this.viewport.dom.style.width = this.parent.offsetWidth + 'px';
		this.viewport.dom.style.height = this.parent.offsetHeight + 'px';

		this.cellWidth = Math.floor((+this.totalWidth / +this.rowCount) - (+this.cellMargin * 2));
		this.cellHeight = Math.floor((+this.totalHeight / +this.colCount) - (+this.cellMargin * 2));
		
		for (cellCount = 0; cellCount < this.cells.length ; cellCount++) {

			cell = this.cells[cellCount];
			
			cell.style.top = ((+this.cellHeight + (+this.cellMargin * 2)) * (+this.cells[cellCount].col)) + (+this.cellMargin) + 'px';
			cell.style.left = ((+this.cellWidth + (+this.cellMargin * 2)) * (+this.cells[cellCount].row)) + (+this.cellMargin) + 'px';
			cell.style.height = +this.cellHeight + 'px';
			cell.style.width = +this.cellWidth + 'px';
			
		}
			
	}

	this.createCells = dataMatrix_createCells;
	
	// zooms in or out on selected cell
	function dataMatrix_zoomCell(cellRef)
	{
	
		if ((cellRef) && (cellRef.toString() == '[object ImageBrowserCell]'))
			this.selectedCell = cellRef;
	
		this.toolbarZoomButton.onclick();
	
	}
	
	this.zoomCell = dataMatrix_zoomCell;
	
	// customize toolbar
	function dataMatrix_customize(showToolbar, showAuxToolbar, useEditor, createBtn, deleteBtn, zoomBtn, setupBtn, displayBtn, layoutBtn, searchBtn, headerDropdown, imageDropdown, sortDropdown)
	{
		var winHeightModifier = 0;
	
		// set editor options
		if (useEditor == true)
			this.allowEditor = true;
		else
			this.allowEditor = false;
	
		// hide or show toolbar
		if (showToolbar) {
			// show or hide individiual elements
			if (createBtn)
				this.toolbarCreate.prv.mDOM.style.display = 'block';
			else
				this.toolbarCreate.prv.mDOM.style.display = 'none';

			if (deleteBtn)
				this.toolbarDelete.prv.mDOM.style.display = 'block';
			else
				this.toolbarDelete.prv.mDOM.style.display = 'none';

			if (zoomBtn)
				this.toolbarZoomButton.prv.mDOM.style.display = 'block';
			else
				this.toolbarZoomButton.prv.mDOM.style.display = 'none';
				
			if (setupBtn)
				this.toolbarSetupButton.prv.mDOM.style.display = 'block';
			else
				this.toolbarSetupButton.prv.mDOM.style.display = 'none';				

			if (displayBtn) {
				this.toolbarZoom.style.display = 'block';
				this.toolbarZoomIn.prv.mDOM.style.display = 'block';
				this.toolbarZoomOut.prv.mDOM.style.display = 'block';
			} else {
				this.toolbarZoom.style.display = 'none';
				this.toolbarZoomIn.prv.mDOM.style.display = 'none';
				this.toolbarZoomOut.prv.mDOM.style.display = 'none';
			}
				
			if (searchBtn)
				this.toolbarSearch.style.display = 'block';
			else
				this.toolbarSearch.style.display = 'none';

		} else {
			// hide toolbar
			this.toolbar.prv.mDOM.style.display = 'none';
		
		}	
		
		if (showAuxToolbar) {

			// show toolbar
			this.auxToolbar.prv.mDOM.style.display = 'block';		
		
			if (layoutBtn)
				this.toolbarOrientation.style.display = 'block';
			else
				this.toolbarOrientation.style.display = 'none';		

			if (headerDropdown)
				this.toolbarHeader.style.display = 'block';
			else
				this.toolbarHeader.style.display = 'none';

			if (imageDropdown)
				this.toolbarImage.style.display = 'block';
			else
				this.toolbarImage.style.display = 'none';

			if (sortDropdown)
				this.toolbarSort.style.display = 'block';
			else
				this.toolbarSort.style.display = 'none';
				
		} else {
		
			// hide toolbar
			this.auxToolbar.prv.mDOM.style.display = 'none';				
		}
		
		this.window.resize (1,1);

		if (!this.initializing)
			this.refresh(true);
	}

	this.customize = dataMatrix_customize;
	
	// cycle through all toolbar dropdowns and select appropriate fields
	function dataMatrix_updateToolbarDropdowns()
	{
	
		// sort field
		if (this.sortField) {
			for (var opCount = 0; opCount < this.sortSelect.options.length; opCount++) {
				if (this.sortSelect.options[opCount].fieldid == this.sortField.fieldid)
					this.sortSelect.selectedIndex = opCount;
			}
		}
		
		// image field
		if (this.imageField) {
			for (var opCount = 0; opCount < this.pictureSelect.options.length; opCount++) {
				if (this.pictureSelect.options[opCount].fieldid == this.imageField.fieldid)
					this.pictureSelect.selectedIndex = opCount;
			}
		}
		
		// header field
		if (this.headerFields) {
			if (this.headerFields[0]) {
				for (var opCount = 0; opCount < this.headerSelect.options.length; opCount++) {
					if (this.headerSelect.options[opCount].fieldid == this.headerFields[0].fieldid)
						this.headerSelect.selectedIndex = opCount;
				}			
			}		
		}

		// image orientation
		for (var opCount = 0; opCount < this.imageSelect.options.length; opCount++) {
			if (this.imageSelect.options[opCount].value.toLowerCase() == this.imageOrientation)
				this.imageSelect.selectedIndex = opCount;
		}
	}

	this.updateToolbarDropdowns = dataMatrix_updateToolbarDropdowns;
	
	function dataMatrix_sort(field, order)
	{

		// get field
		var sortField = getField(this.table.tableid, field);
		
		// return if field is not found
		if (!sortField) {
			debugAlert ('Image browser could not find the ' + field + ' to sort by');
			return false;
		} else {
			this.sortField = sortField;
		}

		// set sorting order
		if (order) {
			if (order != 'asc')
				order = 'desc';
		} else {
			order = 'asc';
		}
		
		this.sortOrder = order;
		
		// return if query id doesn't exist
		if (!this.imageQuery.queryId) {
			debugAlert ('Image browser is not able to sort without proper query id stored, will try again later.');
			this.sortingDone = false;
			return false;		
		} else {
			this.sortingDone = true;
		}

		var url = dax_bridge.url + "sort?tableid="+this.table.tableid+
			"&fieldid="+sortField.fieldid+"&direction="+order+"&queryid="+
			this.imageQuery.queryId+"&sessionid="+dax_bridge.sessionId;
		
		var sortCallRefresh = this.refresh;
		
		makeCall (url, null, null, false);

		this.refresh(true);
		this.updateToolbarDropdowns();
		
	}
	
	this.sort = dataMatrix_sort;
	
	function dataMatrix_refresh()
	{
		
		// do sorting
		if ((!this.sortingDone) && (this.sortingField)) {
			this.sort(this.sortField.fieldid, this.sortOrder);
			return;
		}

		// if this is initial loding, check if allrecords, static, or dynamic tab should be opened
		if (this.initializing) {
			// initializing done
			//this.initializing = false;
	
			if (this.table.showallrecordstab.toLowerCase() == 'true') {
				// show all records
				// ie, do nothing at this point
				this.initializing = false;
			} else if (this.table.preselectQueries.length > 0) {
				// static preselected queries, no all records tab
				this.initializing = false;
				if (this.tabsbar.tabs[0])
					this.tabsbar.tabs[0].dom.onclick();
				return true;
			} else if (this.table.preselectedDynamicQueries.length > 0) {
				// call for tabs right now, we need them for data
				// this.initializing = false;
				this.refreshDynamicTabs();
				return true;
			} else {
			
			}
		}
		
		// figure out number of rows and columns needed
		this.calcZoom();		
		this.createCells();
		this.populate();
		this.redrawToolbar();
		
	}
	
	this.refresh = dataMatrix_refresh;
	
	function dataMatrix_redraw()
	{

		// update viewport

		var voverflow = 0;
		var hoverflow = 0;
		
		if (this.scrollMode == 'ver') {
		// oveflow is content size minus viewport size (hor or ver)
			var areaHeight = (Math.ceil(this.totalRecords / this.rowCount) * (this.cellHeight + (this.cellMargin * 2)));
			voverflow = (areaHeight - this.totalHeight);
			this.viewport.vscrollbar.areaHeight = areaHeight;
		} else {
			var areaWidth = (Math.ceil(this.totalRecords / this.colCount) * (this.cellWidth + (this.cellMargin * 2)));
			hoverflow = (areaHeight - this.totalWidth);	
			this.viewport.hscrollbar.areaHeight = areaWidth;
		}

		// update viewport vertical scrollbar
		this.viewport.setOverflow(voverflow, hoverflow);
		
		this.viewport.refresh();

		// update scrollbar positions
		if (this.scrollMode == 'ver') {
			this.viewport.vscrollbar.dom.style.left = (+this.viewport.dom.offsetWidth) - (+this.viewport.vscrollbar.dom.offsetWidth) + 'px'; 
			this.viewport.vscrollbar.dom.style.top = '0px'; 
			if (this.viewport.vscrollbar.slider.offsetTop+this.viewport.vscrollbar.slider.offsetHeight > this.viewport.vscrollbar.slideArea.offsetTop + this.viewport.vscrollbar.slideArea.offsetHeight) {
				setStyle(this.viewport.vscrollbar.slider, "top", this.viewport.vscrollbar.trackSpace - xpGetSize(this.viewport.vscrollbar.slider).height + "px");
				this.viewport.vscrollbar.last = this.viewport.vscrollbar.trackSpace - xpGetSize(this.viewport.vscrollbar.slider).height;
			}
			this.viewport.vscrollbar.increment = Math.ceil((this.viewport.vscrollbar.trackSpace - this.viewport.vscrollbar.sliderHeight) / (this.colCount)); 
			//this.viewport.vscrollbar.pageIncrement = Math.ceil((this.viewport.vscrollbar.trackSpace - this.viewport.vscrollbar.sliderHeight) / (this.colCount)); 
			this.viewport.vscrollbar.pageIncrement = (this.viewport.dom.offsetHeight) * this.viewport.vscrollbar.trackSpace / (this.viewport.dom.offsetHeight + this.viewport.overflow);
			if (this.viewport.vscrollbar.increment > this.viewport.vscrollbar.pageIncrement)
				this.viewport.vscrollbar.increment = this.viewport.vscrollbar.pageIncrement;
		} else {
			this.viewport.hscrollbar.dom.style.top = (+this.viewport.dom.offsetHeight) - (+this.viewport.hscrollbar.dom.offsetHeight) + 'px'; 
			this.viewport.hscrollbar.increment = Math.ceil((this.viewport.hscrollbar.trackSpace - this.viewport.hscrollbar.sliderWidth) / (this.rowCount)); 
			//this.viewport.hscrollbar.pageIncrement = Math.ceil((this.viewport.hscrollbar.trackSpace - this.viewport.vscrollbar.sliderHeight) / (this.colCount)); 
			this.viewport.hscrollbar.pageIncrement = (this.viewport.dom.offsetWidth) * this.viewport.hscrollbar.trackSpace / (this.viewport.dom.offsetWidth + this.viewport.overflow);
			if (this.viewport.hscrollbar.increment > this.viewport.hscrollbar.pageIncrement)
				this.viewport.hscrollbar.increment = this.viewport.hscrollbar.pageIncrement;
		}
				
		for (var cellIndex = 0; cellIndex < this.cells.length; cellIndex++) {
		
			// update cell targets
			// for highlighted state
			this.cells[cellIndex].highTop = ((+this.cellHeight + (+this.cellMargin * 2)) * (+this.cells[cellIndex].col)) + (+this.cellMargin * 0.5);
			this.cells[cellIndex].highLeft = ((+this.cellWidth + (+this.cellMargin * 2)) * (+this.cells[cellIndex].row)) + (+this.cellMargin * 0.5);
			
			this.cells[cellIndex].highWidth = this.cells[cellIndex].offsetWidth + (this.cellMargin * 1.5);
			this.cells[cellIndex].highHeight = this.cells[cellIndex].offsetHeight + (this.cellMargin * 1.5);

			// normal state

			this.cells[cellIndex].normalTop = ((+this.cellHeight + (+this.cellMargin * 2)) * (+this.cells[cellIndex].col)) + (+this.cellMargin);
			this.cells[cellIndex].normalLeft = ((+this.cellWidth + (+this.cellMargin * 2)) * (+this.cells[cellIndex].row)) + (+this.cellMargin);
			
			this.cells[cellIndex].normalWidth = +this.cellHeight;
			this.cells[cellIndex].normalHeight = +this.cellWidth;
	
		}
	}
	
	this.redraw = dataMatrix_redraw;

	// enable or disable appropriate buttons
	function dataMatrix_redrawToolbar()
	{
		
		// only allow zoom button when cell is zoomed in
		if (this.zoomInCell) {
			this.toolbarZoomButton.enable();
			this.toolbarCreate.disable();			
			this.toolbarDelete.disable();	
			this.toolbarZoomIn.disable();
			this.toolbarZoomOut.disable();
			this.toolbarSetupButton.disable();
			this.searchBox.disabled = true;
		} else {

			this.toolbarSetupButton.enable();
			this.searchBox.disabled = false;				
			this.toolbarCreate.enable();	
		
			// hide or display toolbar buttons based on zoom level
			if ((this.zoomLevel == 1) || ((this.forceRow) && (this.forceCol)))
				this.toolbarZoomOut.disable();
			else
				this.toolbarZoomOut.enable();

			if (((this.rowCount == 1) && (this.colCount == 1)) || ((this.forceRow) && (this.forceCol)))
				this.toolbarZoomIn.disable();
			else
				this.toolbarZoomIn.enable();		

			// enable zoom and delete only if cell is selected 
			if (this.selectedCell) {
				this.toolbarZoomButton.enable();
				this.toolbarDelete.enable();			
			} else {
				this.toolbarZoomButton.disable();
				this.toolbarDelete.disable();
			}			
		}
	}
	
	this.redrawToolbar = dataMatrix_redrawToolbar;
	
	function dataMatrix_populate()
	{
	
		this.populating = true;
	
		// clear query engine parameters
		this.imageQuery.clearQueryEngine();

		// don't update only if searching
		this.imageQuery.updateQueryId = true;

		// user passed query	
		if (this.customQueryValues.length > 0) {
			for (var arrCount = 0; arrCount < this.customQueryValues.length; arrCount++) {
				this.imageQuery.queryEngine.field.push(this.customQueryFields[arrCount].fieldid);
				this.imageQuery.queryEngine.argument.push('equal');
				this.imageQuery.queryEngine.value.push(this.customQueryValues[arrCount]);
				this.imageQuery.queryEngine.andor.push('and');
			}
		}

		if (this.customQueryOptionName.length > 0) {
			this.imageQuery.optionName = this.customQueryOptionName.slice();
			this.imageQuery.optionValue = this.customQueryOptionValue.slice();
		} else {
			this.imageQuery.optionName = new Array();
			this.imageQuery.optionValue = new Array();
		}

		// search box
		if (this.searchBox.value) {
		
			var doSearch = true;
			var searchValue = this.searchBox.value;
			var searchField = getField(this.table.tableid, this.searchSelect.fieldid);
			// if date is used, format it
			if (searchField.fieldtype == 'date') {
				var searchValue = doFormat(searchValue, searchField.fieldformat, 'YYYY-MM-DD');
				if (!searchValue)
					doSearch = false;
			}
		
			if (doSearch) {
				this.imageQuery.queryEngine.field.push(this.searchSelect.fieldid);
				this.imageQuery.queryEngine.argument.push('starts with');
				this.imageQuery.queryEngine.value.push(searchValue);
				this.imageQuery.queryEngine.andor.push('and');
				this.imageQuery.updateQueryId = false;
			}
		}		
		
		var usedRows = Math.ceil((this.totalRecords - (this.queryStart - 1)) / this.rowCount);
		var emptyRows = this.colCount - usedRows;
		
		if ( usedRows < this.colCount) {
			if (emptyRows * this.rowCount > this.queryStart) {
				this.queryStart = 1; 
			} else {
				this.queryStart -= emptyRows * this.rowCount;
			}
		}
		
		if (this.maxCharNumber)
			this.imageQuery.maxChar = this.maxCharNumber;
		
		this.imageQuery.queryName = this.queryName;
		this.imageQuery.callRef = this;
		this.imageQuery.currentPos = this.queryStart;
		this.imageQuery.batchSize = this.colCount * this.rowCount;				
		this.imageQuery.nextBatch(this.onpopulate);
	
	}
	
	this.populate = dataMatrix_populate;
	
	function dataMatrix_onpopulate(http_response, imgRef)
	{
	
		if (http_response.readyState !=4)
			return;

		cellCount = 0;
			
		imgRef.imageQuery.parseResult(http_response);
		var queryResult = imgRef.imageQuery.result;
		
		if (imgRef.onPopulateCustom)
			imgRef.onPopulateCustom(queryResult);
		
		// populate grid with results
		if (queryResult) {
			// loop through results
			if (queryResult.length > 0) {
				for (var row = 0; row < queryResult.length; row++) {
					var label = '';
					var header = '';
					var zoomlabel = '';
					if (imgRef.customHeader)
						header = imgRef.customHeader;
					if (imgRef.customText)
						label = imgRef.customText;
					if (imgRef.customZoomText)
						zoomlabel = imgRef.customZoomText;
					if (cellCount < imgRef.cells.length) {
						for (var field = 0; field < imgRef.table.fieldsList.length; field++) {
							// gather data, create labels, set images
							
							// image parsing
							if (imgRef.imageField) {
								if (queryResult[row].fields[field].id == imgRef.imageField.fieldid) {
									imgRef.cells[cellCount].imageURL = queryResult[row].fields[field].value;
									// get image dimensions
 							
									var picWidth = queryResult[row].fields[field].width;
									var picHeight = queryResult[row].fields[field].height;
									
									if ((picWidth != 0) && (picHeight != 0)) {
									
										var canvasWidth = imgRef.cellWidth * imgRef.cellWidthPer / 100;
										var canvasHeight = imgRef.cellHeight * imgRef.cellHeightPer / 100;

										var windowWidth = imgRef.totalWidth * imgRef.cellWidthPer / 100;
										var windowHeight = imgRef.totalHeight * imgRef.cellHeightPer / 100;
										
										// calculate image ratios, so that it's not displayed streched out
										var picRatio = picHeight / picWidth;				
										var canvasRatio = canvasHeight / canvasWidth;
										var windowRatio = windowHeight / windowWidth;
										
										if (picRatio > canvasRatio) {
											imgRef.cells[cellCount].verPerc = 100;
											imgRef.cells[cellCount].horPerc = Math.round(canvasRatio / picRatio * 100);									
										}
										else {
											imgRef.cells[cellCount].horPerc = 100;
											imgRef.cells[cellCount].verPerc = Math.round(picRatio / canvasRatio  * 100);
										}

										if (picRatio > windowRatio) {
											imgRef.cells[cellCount].verPercZoom = 100;
											imgRef.cells[cellCount].horPercZoom = Math.round(windowRatio / picRatio * 100);									
										}
										else {
											imgRef.cells[cellCount].horPercZoom = 100;
											imgRef.cells[cellCount].verPercZoom = Math.round(picRatio / windowRatio  * 100);
										}
									
									} else {
										
										// one of the image dimensions is 0
										imgRef.cells[cellCount].verPercZoom = 0;
										imgRef.cells[cellCount].horPercZoom = 0;
										imgRef.cells[cellCount].verPerc = 0;
										imgRef.cells[cellCount].horPerc = 0;
									
									}
									/*
									console.log(cellCount + ':' + ' picW:' + picWidth + ' picH:' + picHeight +
										' canW:' + canvasWidth + ' canH:' + canvasHeight +
										' picR:' + picRatio + ' canR:' + canvasRatio +
										' perW:' + imgRef.cells[cellCount].horPerc + ' perH:' + imgRef.cells[cellCount].verPerc);
									*/
									
									// update style
									imgRef.cells[cellCount].imageNode.style.display = 'block';
									// place image
//									imgRef.cells[cellCount].imageNode.src = queryResult[row].fields[field].value;
									imgRef.cells[cellCount].imageNode.src = '/DAX/images/dax_watermark.png';
									imgRef.cells[cellCount].imageNode.style.width = imgRef.cells[cellCount].horPerc + '%';
									imgRef.cells[cellCount].imageNode.style.height = imgRef.cells[cellCount].verPerc + '%';
								} 
								
								// these modes  use labels
								//if ((imgRef.imageOrientation == 'top') || (imgRef.imageOrientation == 'bottom')) {
								
							}
							
							// header parsing
							
							if (imgRef.customHeader) {
							
								// custom header used, generate the label
								var tableField = '[' + getField(imgRef.table.fieldsList[field].fieldid).table.tablename + ']' + imgRef.table.fieldsList[field].fieldname;
								var newValue = queryResult[row].fields[field].value;
								while (header.indexOf(tableField) != -1) {
									header = header.replace (tableField, newValue);
								}
							
							} else {
							
								if (imgRef.headerFields.length == 1) {
									if (imgRef.headerFields[0]) {
										if (queryResult[row].fields[field].id == imgRef.headerFields[0].fieldid) {
											header = queryResult[row].fields[field].value;
										}
									}
								} 
							}

							// detail parsing
							
							if ((imgRef.imageOrientation == 'right') || (imgRef.imageOrientation == 'left')) {
								// data matrix mode	
						
								// keep track if we've used the field already in header
								var headerField = false;
						
								// not a header field, add to text body
								if ((!headerField) && (getField(imgRef.table.tableid, queryResult[row].fields[field].id).fieldtype != 'picture')) {
									
									if (!imgRef.customText) {
										label += '<tr><td class="dax_matrix_fieldcolumn fourdaf_matrix_fieldcolumn">' + getField(imgRef.table.tableid, queryResult[row].fields[field].id).fieldalias +
											'</td><td class="dax_matrix_valuecolumn fourdaf_matrix_valuecolumn">' + queryResult[row].fields[field].value +
											'</td></tr>';
									} else {
										// custom label used, generate the label
										var tableField = '[' + getField(imgRef.table.fieldsList[field].fieldid).table.tablename + ']' + imgRef.table.fieldsList[field].fieldname;
										while (label.indexOf(tableField) != -1) {
											label = label.replace (tableField,queryResult[row].fields[field].value);
										}									
									}
									if (imgRef.customZoomText) {
										// custom label used, generate the label
										var tableField = '[' + getField(imgRef.table.fieldsList[field].fieldid).table.tablename + ']' + imgRef.table.fieldsList[field].fieldname;
										while (zoomlabel.indexOf(tableField) != -1) {
											zoomlabel = zoomlabel.replace (tableField,queryResult[row].fields[field].value);
										}									
									}
								}
							}
						}
						// finish up creating cells
						// image mode
						if ((imgRef.imageOrientation == 'top') || (imgRef.imageOrientation == 'bottom')) {
							// save cell properties
							imgRef.cells[cellCount].recordId = queryResult[row].recordid;
							// place header
							if (header != '') {
								imgRef.cells[cellCount].labelNode.innerHTML = header;
//							imgRef.cells[cellCount].labelNode.style.display = 'block';
								imgRef.cells[cellCount].style.display = 'block';
								imgRef.cells[cellCount].label = header;
							} else {
								imgRef.cells[cellCount].labelNode.style.display = 'block';
								imgRef.cells[cellCount].style.display = 'block';
								imgRef.cells[cellCount].label = null;							
							
							}
						} else if ((imgRef.imageOrientation == 'left') || (imgRef.imageOrientation == 'right')){	
							imgRef.cells[cellCount].recordId = queryResult[row].recordid;
//						imgRef.cells[cellCount].textNode.style.display = 'block';
							imgRef.cells[cellCount].style.display = 'block';
							if (!imgRef.customText) {
								label = '<table>' + label + '</table>';
							}
							
							if (!imgRef.customZoomText)
								zoomlabel = label;
							if (header)
								header = '<div class="dax_matrix_header fourdaf_matrix_header">' + header + '</div>';
							else
								header = '';
							
							imgRef.cells[cellCount].textNode.innerHTML = header + label;
							imgRef.cells[cellCount].textValue = header + label;
							imgRef.cells[cellCount].zoomTextValue = header + zoomlabel;
						
						}
					cellCount++;
					}
				}
			}
		}
		
		// blank out remaining images
		if ((+imgRef.colCount * +imgRef.rowCount) > imgRef.imageQuery.size) {
			for (var cellIndex = imgRef.imageQuery.size; cellIndex < (+imgRef.colCount * +imgRef.rowCount); cellIndex++) {
				/*
				imgRef.cells[cellIndex].imageNode.style.display = 'none';
				imgRef.cells[cellIndex].labelNode.style.display = 'none';
				imgRef.cells[cellIndex].textNode.style.display = 'none';
				*/
				imgRef.cells[cellIndex].style.display = 'none';
				imgRef.cells[cellIndex].recordId = null;
				imgRef.cells[cellIndex].label = null;
			}		
		}
		
		imgRef.totalRecords = +imgRef.imageQuery.recordsInSelection;
		imgRef.needImageUpdate = true;
		imgRef.refreshImages();
		
		imgRef.redraw();

		// Fix ->
		if (imgRef.queryStart == 1) {
			setStyle(imgRef.viewport.vscrollbar.slider, "top", "0px");
			imgRef.viewport.vscrollbar.last = 0;
		}
		
		// set refresh timer
		var imgBrw = imgRef;
		if (imgBrw.refreshTimeout)
			window.clearTimeout (imgBrw.refreshTimeout);
		
		if (imgBrw.autoRefresh)
			imgBrw.refreshTimeout = setTimeout(function() { imgBrw.refresh(true); }, 3000 * 60);	

		imgRef.populating = false;
		imgRef.initializing = false;
		
	}
	
	function dataMatrix_hibernate()
	{
		// remove refresh timers
		if (this.refreshTimeout)
			window.clearTimeout (this.refreshTimeout);
		if (this.releaseBusyTimeout)
			window.clearTimeout(this.releaseBusyTimeout);
		if (this.dynamicTabTimeout)
			window.clearTimeout(this.dynamicTabTimeout);

		WindowManager.hideWindow(this.window);	
		
		this.hibernating = true;
			
	}
	
	this.hibernate = dataMatrix_hibernate;
	
	function dataMatrix_wakeUp()
	{
		if (!this.hibernate)
			return false;
			
		this.hibernating = false;
	
		this.refresh();
		this.dynamicTabTimeout = setTimeout(function() { aThisObject.refreshDynamicTabs(); }, 7000);

		this.window.activate();		
		
	}
	
	this.wakeUp = dataMatrix_wakeUp;
	
	
	this.onpopulate = dataMatrix_onpopulate;
	
	function dataMatrix_destroy()
	{ 

		// remove refresh timers
		if (this.refreshTimeout)
			window.clearTimeout (this.refreshTimeout);
		if (this.releaseBusyTimeout)
			window.clearTimeout(this.releaseBusyTimeout);
		if (this.dynamicTabTimeout)
			window.clearTimeout(this.dynamicTabTimeout);
			
		// remove existing cells
		if (this.cells.length > 0) {
			for (var ce = 0; ce < this.cells.length; ce++) {
				this.cells[ce].boundTo = null;
	
				this.cells[ce].imageNode = null;
				this.cells[ce].imageContainerNode = null;
				this.cells[ce].overlayNode = null;
				this.cells[ce].labelNode = null;
				this.cells[ce].textContainerNode = null;
				this.cells[ce].textNode = null;
				this.cells[ce].spacerNode2 = null;
				
				this.cells[ce].toString = null;
				this.cells[ce].onmouseover = null;
				this.cells[ce].onmouseout = null;
				this.cells[ce].onclick = null;
				this.cells[ce].ondblclick = null;
				
				this.container.removeChild (this.cells[ce]);
			}		
		}

		// remove references to the parent and erase objects
		this.auxToolbar.boundTo = null;
		delete this.auxToolbar;
		
		// remove references to the parents
		this.container.boundTo = null;
		this.currentTab = null;
		this.parent = null;
		
		this.headerSelect.boundTo = null;
		this.imageSelect.boundTo = null;
		this.pictureSelect.boundTo = null;
		this.searchSelect.boundTo = null;
		this.sortSelect.boundTo = null;
		
		this.toolbarCreate.boundTo = null;
		this.toolbarDelete.boundTo = null;
		this.toolbarSearch.boundTo = null;
		this.toolbarSetupButton.boundTo = null;
		this.toolbarZoomButton.boundTo = null;
		this.toolbarZoomIn.boundTo = null;
		this.toolbarZoomOut.boundTo = null;
		
		this.viewport.boundTo = null;
		this.window.boundTo = null;

		this.onCellHover = null;
		this.onCellClick = null;
		this.onCellDblClick = null;
		
		// call other destroy functions and delete objects
		this.tabsbar.destroy();
		this.viewport.destroy();

		delete this.imageQuery;			
	
	};

	this.destroy = dataMatrix_destroy;
	
	// construct
	{

		this.window = WindowManager.createNewWindow();
		this.window.boundTo = this;
		
		// set window title to "table name - display field (date field)"
		this.window.titleBar.innerHTML = this.table.tablealias; 
		this.window.contentArea.appendChild(this.viewport.dom);

		this.parent = this.window.contentArea;
		
		this.window.size (800, 450);
		
		// set minimum sizes
		this.window.minimumWidth = 400;
		this.window.minimumHeight = 150;
		this.window.tableId = this.table.tableid;

		// set icon
		this.window.logo.className = 'window_logo_datamatrix';
		
		// set window actions
		function dataMatrix_window_onsizestop() {
			// recreate cells and populate
			this.boundTo.zoomInCell = null;
			this.boundTo.selectedCell = null;
			this.boundTo.refresh(true);
			this.boundTo.redrawToolbar();
			this.boundTo.tabsbar.refresh();
		}
		
		this.window.onsizestop = dataMatrix_window_onsizestop;
		
		// create toolbar
		this.toolbar = new Toolbar();

		this.auxToolbar = new Toolbar();
		
		this.toolbar.prv.mDOM.className += ' dax_matrix_toolbar';
		this.auxToolbar.prv.mDOM.className += ' dax_matrix_toolbar';

		
		// for overflow hidden ie7 quirk
		if (browser.ie7) {
			this.window.contentArea.style.position = 'relative';
			this.window.toolbar.style.position = 'relative';
		}
		
		// add create & delete record buttons
		this.toolbarCreate = this.toolbar.addButton(STR['ib_toolbar_create']);
		this.toolbarDelete = this.toolbar.addButton(STR['ib_toolbar_delete']);
		this.toolbarZoomButton = this.toolbar.addButton(STR['ib_toolbar_zoom']);
		this.toolbarSetupButton = this.toolbar.addButton(STR['ib_toolbar_setup']);
		// this.toolbarFlip = this.toolbar.addButton(STR['ib_toolbar_flip']);

		// add zoom toolbar component
		this.toolbarZoom = getTemplate('imagebrowsertoolbarzoom').cloneNode(true);
		this.toolbarHeader = getTemplate('imagebrowsertoolbarheader').cloneNode(true);
		generateUniqueId (this.toolbarHeader);
		this.toolbarSort = getTemplate('imagebrowsertoolbarsort').cloneNode(true);
		generateUniqueId (this.toolbarSort);
		this.toolbarOrientation = getTemplate('imagebrowsertoolbarorientation').cloneNode(true);
		generateUniqueId (this.toolbarOrientation);
		this.toolbarImage = getTemplate('imagebrowsertoolbarimage').cloneNode(true);
		generateUniqueId (this.toolbarImage);
		this.toolbar.addContent(this.toolbarZoom);
		generateUniqueId (this.toolbarZoom);
		this.toolbarZoom.innerHTML = STR['ib_toolbar_display'];
		this.toolbarZoom.style.display = 'block';
		this.toolbarZoomOut = this.toolbar.addButton(STR['ib_toolbar_less']);
		this.toolbarZoomIn = this.toolbar.addButton(STR['ib_toolbar_more']);
		this.toolbarSearch = getTemplate('imagebrowsersearch').cloneNode(true);
		generateUniqueId (this.toolbarSearch);
		this.auxToolbar.addContent(this.toolbarOrientation);
		this.toolbar.addContent(this.toolbarSearch);
		this.auxToolbar.addContent(this.toolbarHeader);
		this.auxToolbar.addContent(this.toolbarImage);
		this.auxToolbar.addContent(this.toolbarSort);
		this.toolbarSearch.style.display = 'block';
		this.toolbarHeader.style.display = 'block';
		this.toolbarSort.style.display = 'block';
		this.toolbarOrientation.style.display = 'block';
		this.toolbarImage.style.display = 'block';
		this.searchBox	= getElementByClassName (this.toolbarSearch, 'dax_matrix_searchbox');
		
		// localize
		getElementByClassName(this.toolbarHeader, 'formlabel').innerHTML = STR['ib_toolbar_header'];
		getElementByClassName(this.toolbarSort, 'formlabel').innerHTML = STR['ib_toolbar_sort'];
		getElementByClassName(this.toolbarOrientation, 'formlabel').innerHTML = STR['ib_toolbar_layout'];
		getElementByClassName(this.toolbarImage, 'formlabel').innerHTML = STR['ib_toolbar_image'];

		
//		this.toolbarZoom.prv.mDOM.style.paddingLeft = '15px';

//		this.toolbarFlip.prv.mDOM.style.paddingLeft = '15px';

		// bind buttons to the object
		this.toolbarZoomIn.boundTo = this;
		this.toolbarZoomOut.boundTo = this;
		this.toolbarCreate.boundTo = this;
		this.toolbarDelete.boundTo = this;
		this.toolbarSetupButton.boundTo = this;
		this.toolbarZoomButton.boundTo = this;
		this.toolbarSearch.boundTo = this;
	
		// set zoom button actions
		function dataMatrix_toolbarZoomIn_onclick()
		{
			if ((this.boundTo.colCount == 1) && (this.boundTo.rowCount == 1))
				return;

			var colCount = this.boundTo.colCount;
			var rowCount = this.boundTo.rowCount;
			
			while ((colCount == this.boundTo.colCount) && (rowCount == this.boundTo.rowCount)) {
				// decrease zoom level
				this.boundTo.zoomLevel++;
				this.boundTo.calcZoom();
			}
				
			this.boundTo.selectedCell = null;	
				
			// recreate cells and populate
			this.boundTo.createCells();
			this.boundTo.populate();
			this.boundTo.redrawToolbar();
		}

		this.toolbarZoomIn.onclick = dataMatrix_toolbarZoomIn_onclick;
		
		function dataMatrix_toolbarZoomOut_onclick()
		{
			
			// 1 is minimum zoom level
			if (this.boundTo.zoomLevel == 1)
				return; 
				
			if ((this.boundTo.zoomLevel == 3) && (this.boundTo.mode == 'matrix'))	
				return;				

			var colCount = this.boundTo.colCount;
			var rowCount = this.boundTo.rowCount;
			
			while ((colCount == this.boundTo.colCount) && (rowCount == this.boundTo.rowCount)) {
				// decrease zoom level
				this.boundTo.zoomLevel--;
				this.boundTo.calcZoom();
			}

			this.boundTo.selectedCell = null;	
			
			// recreate cells and populate
			this.boundTo.createCells();
			this.boundTo.populate();
			this.boundTo.redrawToolbar();
		}

		this.toolbarZoomOut.onclick = dataMatrix_toolbarZoomOut_onclick;
		
		function dataMatrix_toolbarSetupButton_onclick()
		{
			if (this.boundTo.auxToolbar.prv.mDOM.style.display == 'none')
				this.boundTo.auxToolbar.prv.mDOM.style.display = 'block';
			else
				this.boundTo.auxToolbar.prv.mDOM.style.display = 'none';
			this.boundTo.window.resize();
			this.boundTo.refresh(true);
		}		
		
		this.toolbarSetupButton.onclick = dataMatrix_toolbarSetupButton_onclick;
		
		function dataMatrix_toolbarZoomButton_onclick()
		{
		
			if (!this.boundTo.animating) {
		
				if (this.boundTo.zoomInCell) {

					// zoom out
					if (!browser.safari)
						this.boundTo.cellzoomout(this.boundTo.selectedCell, 7, 3);			
					else
						this.boundTo.cellzoomout(this.boundTo.selectedCell, 7, 3);			
						
				} else if (this.boundTo.selectedCell) {

					// hide viewport scrollbars
					this.boundTo.viewport.hscrollbar.dom.style.display = 'none';
					this.boundTo.viewport.vscrollbar.dom.style.display = 'none';

					// show custom text
					if (this.boundTo.selectedCell.textNode)
						this.boundTo.selectedCell.textNode.innerHTML = this.boundTo.selectedCell.zoomTextValue;
					
					//resize image
					if (this.boundTo.imageField) {
						this.boundTo.selectedCell.imageNode.style.width = this.boundTo.selectedCell.horPercZoom + '%';
						this.boundTo.selectedCell.imageNode.style.height = this.boundTo.selectedCell.verPercZoom + '%';
					}

					// update image sizes if appropriate
					// show or hide elements based on mode
					if ((this.boundTo.imageOrientation == 'top') || (this.boundTo.imageOrientation == 'bottom')) {
						// header visible?
						if ((this.boundTo.headerFields[0]) || (this.boundTo.customHeader)) {

							this.boundTo.selectedCell.imageContainerNode.style.height = '75%';
						
						} else {
							
							this.boundTo.selectedCell.imageContainerNode.style.height = '100%';
					
						}
					} else {
						// image visible?
						if (this.boundTo.imageField) {

							this.boundTo.selectedCell.textContainerNode.style.width = '60%';
						
						} else {

							this.boundTo.selectedCell.textContainerNode.style.width = '80%';
						}
					}
					
					// zoom in
					if (!browser.safari)
						this.boundTo.cellzoomin(this.boundTo.selectedCell, 10, 7);
					else
						this.boundTo.cellzoomin(this.boundTo.selectedCell, 10, 3);
					
					//hide other cells
					for (var cellCount = 0; cellCount < this.boundTo.cells.length; cellCount++) {
						if (this.boundTo.cells[cellCount] != this.boundTo.selectedCell)
							this.boundTo.cells[cellCount].style.display = 'none';
					}
				}
				this.boundTo.redrawToolbar();				
			}
		}
		
		this.toolbarZoomButton.onclick = dataMatrix_toolbarZoomButton_onclick;
		
		function dataMatrix_toolbarCreate_onclick()
		{ 
		
			if (this.boundTo.table.fieldsDetail.length == 0)
				return false;
		
			// find the editor if we have one, otherwise create a new one
			var editor = this.boundTo.getEditor();

			editor.record = { recordId : -1, tableId: this.boundTo.table.tableid };
			editor.buildEditorByTable(this.boundTo.table.tableid);

			editor.clear();

			editor.edit();
		};

		this.toolbarCreate.onclick = dataMatrix_toolbarCreate_onclick;
		
		function dataMatrix_toolbarDelete_onclick()
		{ 
			// make deletion call
			if (this.boundTo.selectedCell) {
				// confirm record deletion
				var userConfirm = confirm(STR['confirm_delete_records']);
				if (!userConfirm)
					return;
				// send delete call
				url = dax_bridge.url + "deleteRecord?sessionId=" + dax_bridge.sessionId	+ 
					"&recordId=" + this.boundTo.selectedCell.recordId +
					"&tableId=" + this.boundTo.table.tableid;
					makeCall (url, this.boundTo.ondelete, this.boundTo);
			}
		};
		
		this.toolbarDelete.onclick = dataMatrix_toolbarDelete_onclick;
		
		// on completed delete call, refresh the calendar
		function dataMatrix_ondelete(http_result, aImageBrowser) {
			if (http_result.readyState != 4)
				return;
			checkResponse (http_result);
			aImageBrowser.refresh(true);
		}
		
		this.ondelete = dataMatrix_ondelete;
		
		// setup search
		this.searchBox.owner = this;
		function dataMatrix_searchBox_onkeyup(aEvent) {
		
			var iRef = this.owner;
			if (iRef.searchTimeout) {
				window.clearTimeout (iRef.searchTimeout);
			}
				
			iRef.searchTimeout = window.setTimeout (function() {iRef.refresh(true); }, 500);
		}

		this.searchBox.onkeyup = dataMatrix_searchBox_onkeyup;
		
		var searchOption	= getElementByClassName(this.toolbarSearch, "fieldSearchOption");
		var searchGroup		= searchOption.parentNode;
		var searchSelect	= getElementByClassName(this.toolbarSearch, "fieldSearchSelect");

		var headerOption	= getElementByClassName(this.toolbarHeader, "fieldSearchOption");
		var headerGroup		= headerOption.parentNode;
		var headerSelect	= getElementByClassName(this.toolbarHeader, "fieldSearchSelect");

		var sortOption		= getElementByClassName(this.toolbarSort, "fieldSearchOption");
		var sortGroup		= sortOption.parentNode;
		var sortSelect		= getElementByClassName(this.toolbarSort, "fieldSearchSelect");		
		
		var pictureOption	= getElementByClassName(this.toolbarImage, "fieldSearchOption");
		var pictureGroup	= pictureOption.parentNode;
		var pictureSelect	= getElementByClassName(this.toolbarImage, "fieldSearchSelect");

		var imageOption		= getElementByClassName(this.toolbarOrientation, "fieldSearchOption1");
		var imageGroup		= imageOption.parentNode;
		var imageSelect		= getElementByClassName(this.toolbarOrientation, "fieldSearchSelect");

		this.searchSelect	= searchSelect;
		this.headerSelect	= headerSelect;
		this.imageSelect	= imageSelect;
		this.pictureSelect	= pictureSelect;
		this.sortSelect		= sortSelect;
		
		searchSelect.boundTo	= this;
		headerSelect.boundTo	= this;
		pictureSelect.boundTo	= this;
		imageSelect.boundTo		= this;
		sortSelect.boundTo		= this;
		
		imageSelect.onchange = 
			function() {this.boundTo.changeLayout(this.options[this.selectedIndex].orientation); this.boundTo.refresh(true); };

		sortSelect.onchange = function() { 
			this.boundTo.sort(this.options[this.selectedIndex].fieldid);
//			this.boundTo.refresh();
		};			
			
		headerSelect.onchange = function() { 
			this.boundTo.headerFields[0] = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid);
			this.boundTo.customHeader = null;
			this.boundTo.refresh(true);
		};
		
		pictureSelect.onchange = function() {
			this.boundTo.imageField = getField(this.boundTo.table.tableid, this.options[this.selectedIndex].fieldid); 
			this.boundTo.refresh(true);		
		}
		
		imageOption.parent = imageSelect;
		imageOption.innerHTML = STR['ib_toolbar_top'];
		//option.value = table.fieldsList[k].fieldid;
		imageOption.orientation = 'top';
		/*
		imageOption.onclick =
			function() {
				this.boundTo.changeLayout ('top'); 
				this.boundTo.refresh();
			};
		*/
		imageOption.boundTo = this;
		imageGroup.appendChild(imageOption);
	
		imageOption = imageOption.cloneNode(true);

		imageOption.parent = imageSelect;
		imageOption.innerHTML = STR['ib_toolbar_bottom'];
		//option.value = table.fieldsList[k].fieldid;
		imageOption.orientation = 'bottom';
		/*
		imageOption.onclick =
			function() {this.boundTo.changeLayout('bottom'); this.boundTo.refresh(); };
		*/
		imageOption.boundTo = this;
		imageGroup.appendChild(imageOption);

		var imageOption		= getElementByClassName(this.toolbarOrientation, "fieldSearchOption2");
		var imageGroup		= imageOption.parentNode;
		
		imageOption.parent = imageSelect;
		imageOption.innerHTML = STR['ib_toolbar_left'];
		//option.value = table.fieldsList[k].fieldid;
		imageOption.orientation = 'left';
		/*
		imageOption.onclick =
			function() {this.boundTo.changeLayout ('left'); this.boundTo.refresh(); };
		*/
		imageOption.boundTo = this;
		imageGroup.appendChild(imageOption);

		imageOption = imageOption.cloneNode(true);

		imageOption.parent = imageSelect;
		imageOption.innerHTML = STR['ib_toolbar_right'];
		//option.value = table.fieldsList[k].fieldid;
		imageOption.orientation = 'right';
		/*
		imageOption.onclick =
			function() {this.boundTo.changeLayout('right'); this.boundTo.refresh(); };
		*/
		imageOption.boundTo = this;
		imageGroup.appendChild(imageOption);

		
		if (searchSelect) {

			searchSelect.tableId = this.table.tableid;
			headerSelect.tableId = this.table.tableid;
			pictureSelect.tableId = this.table.tableid;
			sortSelect.tableId = this.table.tableid;
			
			headerOption.parent = headerSelect;
			headerOption.innerHTML = STR['ib_toolbar_none'];
			//option.value = table.fieldsList[k].fieldid;
			headerOption.fieldid = null;
			/*
			headerOption.onclick =
				function() { this.boundTo.headerFields = new Array(); this.boundTo.refresh(); };
			*/
			headerOption.boundTo = this;
			headerGroup.appendChild(headerOption);

			pictureOption.parent = pictureSelect;
			pictureOption.innerHTML = STR['ib_toolbar_none'];
			//option.value = table.fieldsList[k].fieldid;
			pictureOption.fieldid = null;
			/*
			pictureOption.onclick =
				function() { this.boundTo.imageField = null; this.boundTo.refresh(); };
			*/
			sortOption.boundTo = this;
			sortGroup.appendChild(sortOption);

			sortOption.parent = sortSelect;
			sortOption.innerHTML = STR['ib_toolbar_none'];
			//option.value = table.fieldsList[k].fieldid;
			sortOption.fieldid = null;
			/*
			pictureOption.onclick =
				function() { this.boundTo.imageField = null; this.boundTo.refresh(); };
			*/
			sortOption.boundTo = this;
			sortGroup.appendChild(sortOption);			
			
			// build the select list in the toolbar
			for (var k in this.table.fieldsList) {
				if (k > 0) {
					searchOption = searchOption.cloneNode(true);
					headerOption = headerOption.cloneNode(true);
					pictureOption = pictureOption.cloneNode(true);
					sortOption = sortOption.cloneNode(true);
				} else {
					searchSelect.fieldid = this.table.fieldsList[k].fieldid;
					headerOption = headerOption.cloneNode(true);
					pictureOption = pictureOption.cloneNode(true);
					sortOption = sortOption.cloneNode(true);
					//headerSelect.fieldid = this.table.fieldsList[k].fieldid;
				}
					
				searchOption.parent = searchSelect;
				searchOption.innerHTML = this.table.fieldsList[k].fieldalias;
				//option.value = table.fieldsList[k].fieldid;
				searchOption.fieldid = this.table.fieldsList[k].fieldid;
				searchSelect.onchange =
					function() { 
						this.fieldid = this.options[this.selectedIndex].fieldid;
					};
				searchGroup.appendChild(searchOption);

				if ((this.table.fieldsList[k].fieldtype != 'picture') && (this.table.fieldsList[k].fieldinlist.toLowerCase() == 'true')){				
				
					headerOption.parent = headerSelect;
					headerOption.innerHTML = this.table.fieldsList[k].fieldalias;
					//option.value = table.fieldsList[k].fieldid;
					headerOption.fieldid = this.table.fieldsList[k].fieldid;
					/*
					headerOption.onclick = function() { 
						this.boundTo.headerFields[0] = getField(this.boundTo.table.tableid, this.fieldid);
						this.boundTo.customHeader = null;
 						this.boundTo.refresh();
					};
					*/
					headerOption.boundTo = this;
					headerGroup.appendChild(headerOption);

					sortOption.parent = sortSelect;
					sortOption.innerHTML = this.table.fieldsList[k].fieldalias;
					//option.value = table.fieldsList[k].fieldid;
					sortOption.fieldid = this.table.fieldsList[k].fieldid;
					/*
					headerOption.onclick = function() { 
						this.boundTo.headerFields[0] = getField(this.boundTo.table.tableid, this.fieldid);
						this.boundTo.customHeader = null;
 						this.boundTo.refresh();
					};
					*/
					sortOption.boundTo = this;
					sortGroup.appendChild(sortOption);
					
				}
				
				if ((this.table.fieldsList[k].fieldtype == 'picture')  && (this.table.fieldsList[k].fieldinlist.toLowerCase() == 'true')){
				
					pictureOption.parent = pictureSelect;
					pictureOption.innerHTML = this.table.fieldsList[k].fieldalias;
					//option.value = table.fieldsList[k].fieldid;
					pictureOption.fieldid = this.table.fieldsList[k].fieldid;
					/*
					pictureOption.onclick =
						function() { this.boundTo.imageField = getField(this.boundTo.table.tableid, this.fieldid); this.boundTo.refresh(); };
					*/
					pictureOption.boundTo = this;
					pictureGroup.appendChild(pictureOption);
					
				}
			}
		}
		
		// add toolbar to window
		this.window.addToolbar(this.toolbar.prv.mDOM);
		this.window.resizeToolbar(false);
		this.window.addToolbar(this.auxToolbar.prv.mDOM);
		this.window.resizeToolbar(false);

		// add ddw button to the toolbar
		var eTable = getTable (this.table.tableid);
		if ((eTable) && (eTable.ddwOptions) && (eTable.ddwOptions.length > 0)) {
			
			// add more space for DDW button
			this.window.resize(50,0);
			
			// add a ddw button
			var ddwOption = eTable.ddwOptions[0];
			
			var ddwButton = this.toolbar.addButton (ddwOption.title);
			ddwButton.url = ddwOption.url;
			ddwButton.title = ddwOption.title;
			ddwButton.owner = this;
			if (ddwOption.type.indexOf("Static") != -1) {

				function dataMatrix_ddwButton_onclick() {
					var queryId = this.owner.imageQuery.queryId;
					var tableId = this.owner.table.tableid;
					var selectedRecords = new Array();
					if (this.owner.selectedCell)
						selectedRecords.push(this.owner.selectedCell.recordId);
					new DDW(this.url, 'static_url', this.title, '&queryid=' + queryId + "&tableid=" + tableId, selectedRecords);
				};
				
				// function DataWindowPartTwo_
				
				ddwButton.onclick = dataMatrix_ddwButton_onclick;
				
			} else if (ddwOption.type.indexOf("Dynamic") != -1) {
				ddwButton.onclick = function () {
					var queryId = this.owner.imageQuery.queryId;
					var tableId = this.owner.table.tableid;
					var selectedRecords = new Array();
					if (this.owner.selectedCell)
						selectedRecords.push(this.owner.selectedCell.recordId);

					new DDW(this.url, 'dynamic_url', this.title, '&queryid=' + queryId + "&tableid=" + tableId, selectedRecords);	
				};
			} else if (ddwOption.type.indexOf("Message") != -1) {
				ddwButton.onclick = function () {
					var queryId = this.owner.imageQuery.queryId;
					var tableId = this.owner.table.tableid;
					var selectedRecords = new Array();
					if (this.owner.selectedCell)
						selectedRecords.push(this.owner.selectedCell.recordId);
					new DDW(this.url, 'message_window', this.owner.window, '&queryid=' + queryId + "&tableid=" + tableId, selectedRecords);	
				};
			}
		}
		
		
		// hide aux toolbar on load
		this.auxToolbar.prv.mDOM.style.display = 'none';		

		// the first Tab is all records
		var tabs = new Tabs();

		if ((this.table.showallrecordstab.toLowerCase() == 'true') || ((this.table.preselectQueries.length == 0) && (this.table.preselectedDynamicQueries.length == 0))) {
		
		
			var tab = tabs.insert(STR['all']+ " " + this.table.tablealias);
			tab.owner = this;
			tabs.dom.className = "tabs_tb";
			tab.firstTab = true;
			tab.queryId = null;
			tab.queryName = null;

			tab.onswitch = function()
			{
				// move scrollbar to 0
				this.owner.viewport.hscrollbar.slider.style.top = '0px';
				this.owner.viewport.vscrollbar.slider.style.top = '0px';
			
				if (this.owner.imageQuery)
					this.owner.imageQuery.queryId = null;
				this.owner.queryName = null;
				this.owner.refresh(true);			
			}
			
			tab.tooltip = '';

			// query properties
			// tab keeps 'permanent' query id 
			this.currentTab = tab;
			
		}
			
		// add preselect query tabs
		for (var tCount = 0; tCount < this.table.preselectQueries.length; tCount++) {
			var tab = tabs.insert(this.table.preselectQueries[tCount].name);
			tab.owner = this;
			tabs.dom.className = "tabs_tb";
			tab.firstTab = false;
			tab.queryId = null;
			tab.queryName = this.table.preselectQueries[tCount].name;

			tab.onswitch = function()
			{
				// move scrollbar to 0
				this.owner.viewport.hscrollbar.slider.style.top = '0px';
				this.owner.viewport.vscrollbar.slider.style.top = '0px';

				if (this.owner.imageQuery)
					this.owner.imageQuery.queryId = null;			
				this.owner.queryName = this.queryName;
				this.owner.refresh(true);	
			}
			
			tab.tooltip = '';	

			if (!this.currentTab) {
				this.currentTab = tab;
			}
		}

		// set first tab active
		tabs.activateById(0);
		
		this.tabsbar = tabs;
 
		// if there are dynamic or static queries, display 
		if (!((this.table.preselectQueries.length == 0) && (this.table.preselectedDynamicQueries.length == 0)))
			this.window.addToolbar(tabs.dom);
		
		this.updateDynamicQueryTabs = function()
		{
			if (this.table) {
				if (this.table.preselectedDynamicQueries) {
					if (this.table.preselectedDynamicQueries.length > 0) {
						for (var qCount = 0; qCount < this.table.preselectedDynamicQueries.length; qCount++) {
							// check for valid string
							if (this.table.preselectedDynamicQueries[qCount].fieldId.indexOf('][') != -1) {
								// generate a call to backend to get the list of unique values
								var url = dax_bridge.url + "GetUniqueQueries?sessionId=" + dax_bridge.sessionId + "&tableid=" + this.table.tableid +
									"&fieldid=" + this.table.preselectedDynamicQueries[qCount].fieldId;
								makeCall (url, this.updateDynamicQueryTabs_handler, this);
							}
						}
					}
				}
			}
		}
		
		this.updateDynamicQueryTabs_handler = function(http_result, aRef) {
		
			if (http_result.readyState != 4)
				return;

			var responseError = checkResponse (http_result);
				
			if (responseError)	
				return;
			
			var queryNode = http_result.responseXML.getElementsByTagName("queries").item(0);
			var queryNames = queryNode ? queryNode.getElementsByTagName("query") : null;		
			var fieldId = queryNode.getAttribute("fieldid");

			if (!queryNames)
				return;
			
			if (queryNames.length == 0)
				return;

			if (aRef.updatingTabsFieldId) {
				if (aRef.updatingTabsFieldId == fieldId) {
					return;
				} else {
					setTimeout(function() {DataWindowPartThree_updateDynamicQueryTabs_handler(http_result, aRef) }, 1000);
					return;
				}
			} else {
				aRef.updatingTabsFieldId = fieldId;
			}				
				
			for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
				// mark all tabs as removable until proven otherwise later
				aRef.tabsbar.tabs[tabCount].keepTab = false;
			}
				
			for (qnCount = 0; qnCount < queryNames.length; qnCount++) {	
				var queryName = queryNames.item(qnCount).getAttribute('name');
				if (queryName) {
					if (queryName != '4daf_unique_') {
						var tabExists = false;
						// check if the tab exists before adding it
						for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
							if (aRef.tabsbar.tabs[tabCount].queryName == queryName) {
								var tabExists = true;
								// so we won't erase it later when purging the non-existent tabs
								aRef.tabsbar.tabs[tabCount].keepTab = true;
							}						
						}

						if (!tabExists) {
							// add tab
							var tabs = aRef.tabsbar;
							var tab = tabs.insert(queryName.substr(12));
							tab.owner = aRef;
						//	tabs.dom.className = "tabs_tb";
							tab.firstTab = false;
							tab.queryId = null;
							tab.queryName = queryName;
							tab.keepTab = true;

							tab.onswitch = function()
							{
								this.owner.imageQuery.queryId = null;							
								this.owner.queryName = this.queryName;
								this.owner.refresh(true);	
							}
							
							tab.tooltip = '';
						}
					}
				}
			}
			// erase tabs
			var eraseTabsArray = new Array();
			for (var tabCount = 0; tabCount < aRef.tabsbar.tabs.length; tabCount++) {
				if (!aRef.tabsbar.tabs[tabCount].firstTab) {
					if ((!aRef.tabsbar.tabs[tabCount].keepTab) && (aRef.tabsbar.tabs[tabCount].queryName)) {
						if ((aRef.tabsbar.tabs[tabCount].queryName.indexOf('4daf_unique_') != -1) && (aRef.tabsbar.tabs[tabCount].fieldId == fieldId))
							eraseTabsArray.push(aRef.tabsbar.tabs[tabCount]);
					}
				}
			}
			if (eraseTabsArray) {
				if (eraseTabsArray.length > 0) {
					for (var tabCount = 0; tabCount < eraseTabsArray.length; tabCount++) {
						var clearCurrentTab = false;
						// remove current tab manually from data window
						if (aRef.currentTab = eraseTabsArray[tabCount])
							clearCurrentTab = true;
						// remove tab
						aRef.tabsbar.remove(eraseTabsArray[tabCount].index);
						if (clearCurrentTab)
							aRef.currentTab = null;
					}
				}			
			}
			
			// current tab still exists, reselect it
			if (aRef.currentTab) {
				aRef.tabsbar.activate(aRef.currentTab);
			} else {
				aRef.currentTab = aRef.tabsbar.tabs[0];
				aRef.currentTab.dom.onclick();
			}

			// update tabs
			aRef.tabsbar.refresh();
			
			// if object is in init state, open first dynamic tab
			if (aRef.initializing == true) {
				aRef.initializing = false;
				if (aRef.tabsbar.tabs[0])
					aRef.tabsbar.tabs[0].dom.onclick();
			}
			
			// release lock
			aRef.updatingTabsFieldId = null;
		}
		
		this.refreshDynamicTabs = function()
		{
			if (this.dynamicTabTimeout) {
				clearTimeout(this.dynamicTabTimeout);
			}

			this.updateDynamicQueryTabs();
			
			var thisWindow = this;
						
			this.dynamicTabTimeout = setTimeout(function() { thisWindow.refreshDynamicTabs(); }, 60 * 3 * 1000);

		}
		
		// setup window buttons
		this.window.refreshBox.boundTo = this;
		this.window.refreshBox.allowRefresh = true;
		this.window.refreshBox.onmouseup = function(event)
		{

			this.className = "window_refreshbox";
			
			// to allow refresh every few seconds, and not more often to avoid memory leak
			if (!this.allowRefresh)
				return false;
				
			this.allowRefresh = false;
			var event = xpEvent(event, true);
			
			var refreshButton = this;
			
			this.resetRefreshTimeout = setTimeout(function() { refreshButton.allowRefresh = true; }, 7500);
			
			this.boundTo.refresh();
			return false;
		};

		this.window.onclose = function() {
			// erase taskbar item
			if (this.tbitem)
				this.tbitem.remove();
		
			// get rid of the editor
			if (this.editor)
				this.editor.destroy();
				
			// call image browser destroy function
			if (this.boundTo)
				this.boundTo.destroy();
		}
		
		// if taskbar exists, add taskbar icon
		if (taskbar) {
			var ti = new taskbarItem(taskbar);
			ti.setLabel(this.window.titleBar.innerHTML);
			ti.setIcon("datamatrix");
			setStyle (ti.dom, 'display', 'block');
			// create a handler for clicking
			var winRef = this.window;
			ti.dom.onclick = function() { 
				this.parent.setSelected (true, true);
				winRef.activate(); 
			};

			ti.setSelected (true, true);
			ti.parent.refresh(true);

			this.window.tbitem = ti;
		}
		
		this.window.activate();
		this.window.open();	
		
		this.window.tableId = this.table.tableid;		

		// editor function for window
		this.getEditor = function()	{
			if (this.editor)
				return this.editor;
		
			// create editor object
			this.editor = new Editor();
			var dom = this.editor.getDOM();
			dom.style.zIndex = ++WindowManager.maxZIndex;
			this.window.dom.m.insertBefore(dom, this.window.toolbar);
			var w_off =  (this.window.dom.offsetWidth / 2) -
				(dom.offsetWidth / 2);

			setStyle(dom, "position", "absolute");
			setStyle(dom, "left", w_off + "px");
			
			this.editor.owner = this;
			
			var w_ref = this; // use this for when editor is detatched
			this.editor.onaftersave = function()
			{
				this.owner.refresh(true);
			}

			this.editor.onafterdelete = function()
			{
				this.owner.refresh(true);
				this.getTearAway().contract(1, 50);
				this.destroy();
			}
			
			if (browser.ie7) {
				this.editor.onafterrefresh = function()
				{
					//setStyle(this.owner.editor.prv.mTearAway.dom, "display", "block");
					if ((this.owner) && (this.owner.editor)) {
						setStyle(this.prv.mTearAway.dom, "top", "29px");
						setStyle(this.prv.mTearAway.dom, "position", "absolute");
					}
				}
			}
			
			// customize
			dom.ondragstop = function(aEvent)
			{
				this.className = "tearaway_torn";
				var editor = this.owner;
				
				if (browser.ie) {
					this.top.t.style.fontSize = '1pt';
					this.top.tl.style.fontSize = '1pt';
					this.top.tr.style.fontSize = '1pt';
					this.top.style.fontSize = '1pt';

				}
			 
				// create new window when dragging is done
				if (editor.owner) {
					var winObj = editor.owner;
					
					if (!winObj.editor)
						return false;
						
					var dom = editor.getDOM();
					var pos = getObjectPosition(this); // global coordinates

					if (taskbar) {
					
						// add taskbar item for new window
						var ti = new taskbarItem(taskbar);

						// make unique label based on table name
						i = 1;
						while ($('dax_taskbar_button' + winObj.window.titleBar.innerHTML + i) != null) {
							i++;
						}
						// set taskbar button title
						ti.dom.id = 'dax_taskbar_button' + winObj.window.titleBar.innerHTML + i;

						ti.setLabel("Edit " + winObj.window.titleBar.innerHTML + " " + i);
						ti.setIcon("editor");
					
						// create a handler for clicking
						var ref = this;
						ti.dom.onclick = function() { 
							this.parent.setSelected (true, true);
							ref.onclick();
						};
						setStyle (ti.dom, 'display', 'block');
						ti.setSelected (true, true);
						// refresh taskbar
						ti.parent.refresh();
					}
					
					// override cancel button
					if (dom.cancelBtn) {
						dom.cancelBtn.onclick = function(aEvent) {
							if (ti)
								ti.remove(); // remove taskbar item
							document.body.removeChild(this.parent.prv.mDOM);
							this.parent.destroy();
							delete this.parent;
						};
					}
					
					// override the focusing
					this.onclick = function(aEvent) {
						if (WindowManager.active)
							WindowManager.active.deactivate();
							setStyle(this, "zIndex", ++WindowManager.maxZIndex);
							if (ti) {
								ti.setSelected(true,true);
							}
					};

					// release
					if (this.owner.owner.window)
						winObj.window.deactivate();
					this.owner.owner.editor = null;
					//this.parent.owner = null;
					
					// reposition
					document.body.insertBefore(this, null);
					setStyle(this, "left", pos.x + "px");
					setStyle(this, "top", pos.y + "px");
					
				}
				
			};
			return this.editor;
		};		

		
		// update window on close box method
//		aWindow.closeBox.onmouseup = function (aEvent) {
//			WindowManager.hideWindow(this.parent);
//			this.parent.tbitem.dom.style.display = 'none';
//			return false;
//		}

		// setup viewport
		this.viewport.setContent (this.container);
		this.viewport.isStatic = true;
		this.viewport.hscrollbar.isStatic = true;
		this.viewport.vscrollbar.isStatic = true;

		//this.viewport.setOverflow (0,0);
		this.viewport.boundTo = this;
		this.viewport.resizeOnRefresh = false;
		this.viewport.owner = this;	

		function dataMatrix_doVScroll(aTop, aIsScrollDown)
		{
			// calculate height
			var calcHeight = (aTop * this.viewport.vscrollbar.ratio);
			
			var cellHeight = +this.cellHeight + (+this.cellMargin * 2);
			
			// round it up to the nearest cell height
			var moduo = calcHeight % cellHeight;
			// var newHeight = (moduo > (cellHeight / 2)) ? (calcHeight + moduo) : (calcHeight  - moduo);
			var newHeight = (moduo > (cellHeight / 2)) ? (calcHeight + moduo) : (calcHeight + moduo);
			
			// reposition the contents
			//setStyle(this.boundTo.container, "top", newHeight + "px");

			// shift occured, populate
			//if ((newHeight + 'px') != this.boundTo.container.style.top) {
			var firstRecord = ((Math.abs(Math.floor(newHeight / cellHeight))) * this.rowCount) + 1;
			if (firstRecord != this.queryStart) {
				this.queryStart = firstRecord;
				//console.log ('first record:' + firstRecord);
				this.populate();
			}		
		}
		
		// scroll function
		this.doVScroll = dataMatrix_doVScroll;


		// set viewport scroll function
		this.viewport.onvscroll = function(aTop, aIsScrollDown)
		{
			if (!this.boundTo.populating)
				this.boundTo.doVScroll(aTop, aIsScrollDown);
		};

		// set viewport scroll function
		this.viewport.onvscrollstop = function(aTop, aIsScrollDown)
		{
			// if there is a timer to trigger this event, stop it
			if (this.boundTo.vScrollStopTimeout)
				window.clearTimeout(this.boundTo.vScrollStopTimeout);
			
			// if data matrix is busy populating, schedule this call for later
			if (this.boundTo.populating) {
				var datMatRef = this.boundTo;
				var datMatTop = aTop;
				var datMatScrollDown = aIsScrollDown;
				this.boundTo.vScrollStopTimeout = setTimeout(function() { datMatRef.viewport.onvscrollstop(datMatTop, datMatScrollDown); }, 500);			
			} else {
				this.boundTo.doVScroll(aTop, aIsScrollDown);
			}
		};

		
		
		this.viewport.onhscroll = function(aLeft, aIsScrollRight)
		{
			
			// calculate width
			var calcWidth = (aLeft * (-1) * this.hscrollbar.ratio);
			
			var cellWidth = +this.boundTo.cellWidth + (+this.boundTo.cellMargin * 2);
			
			// round it up to the nearest cell height
			var moduo = calcWidth % cellWidth;
			var newWidth = (moduo > (cellWidth / 2)) ? (calcWidth + moduo) : (calcWidth  + moduo);

			/*
			
			if (!(isNaN(aLeft * (-1) * this.hscrollbar.ratio))) {
				setStyle(this.content, "left", (aLeft * (-1) * this.hscrollbar.ratio)+ "px");
			}

			*/
			

			// shift occured, populate
			//if ((newHeight + 'px') != this.boundTo.container.style.top) {
			var firstRecord = ((Math.abs(Math.floor(newWidth / cellWidth))) * this.boundTo.colCount) + 1;
			if (firstRecord != this.boundTo.queryStart) {
				this.boundTo.queryStart = firstRecord;
				//console.log ('first record:' + firstRecord);
				this.boundTo.populate();
			}
		};

		// if target div is used, place the image browser there
		if (targetNode) {
			// this.isWindow = false;
			// if location exists, insert the data window into the target element
			var targetPosition = getObjectPosition (targetNode);
			// position the window where target element is
			this.window.dom.style.top = targetPosition.y + 'px';
			this.window.dom.style.left = targetPosition.x + 'px';
			this.window.size (targetNode.offsetWidth, targetNode.offsetHeight, true);
			// hide window based doms
			this.window.dom.top.style.display = 'none';
			this.window.dom.btm.style.display = 'none';
			this.window.dom.l.style.display = 'none';
			this.window.dom.r.style.display = 'none';
			this.window.resize(1,1);
		}
		
		this.container.style.display = 'block';
		this.viewport.dom.style.display = 'block';
		
		// create query
		this.imageQuery = new Query(this.table.tableid);		
		
		if (!this.skipQueryOnStart)
			this.refresh();
			
		this.updateToolbarDropdowns();
		
		var aThisObject = this;
		if (!this.initializing)
			this.tabsbar.refresh();
		this.dynamicTabTimeout = setTimeout(function() { aThisObject.refreshDynamicTabs(); }, 2500);

		this.loaded = true;
	}
}