/**
 * DDW
 *
 * @author Rudolf Psenicnik
 *
 */

// developer defined window
function DDW(ddwContents, ddwType, ddwTitleOrTarget, ddwParameters, ddwSelectedRecords) {

	if ((!ddwContents) || (!ddwType))
		return false;
		
	this.fullUrl = null;	
		
	if (!ddwParameters)
		this.ddwParameters = "";
	else {
		// make sure that ddw doesn't have /DAX/getDDW
		if (ddwContents.indexOf('/DAX/GetDDW') != -1) {
			this.ddwParameters = ddwParameters;
		} else {
			this.ddwParameters = "?";
		}
	}
	
	// list of selected fields
	if (!ddwSelectedRecords)
		this.ddwSelectedRecords = "";
	else {
		this.ddwSelectedRecords = "&recordid=";
		for (o = 0; o < ddwSelectedRecords.length; o++) {
			this.ddwSelectedRecords += ddwSelectedRecords[o] + ',';
		}
	}
	
	// set window title
	if (ddwType.indexOf('message') == -1) {
		this.messageTarget = null;
		if (ddwTitleOrTarget) 
			this.windowTitle = ddwTitleOrTarget;
		else
			this.windowTitle = 'Developer Defined Window';
	} else {
		if (ddwTitleOrTarget)
			this.messageTarget = ddwTitleOrTarget;
		else
			return false;
	}

	// save parameters
	this.ddwContents = ddwContents;
	this.ddwType = ddwType;
	
	// message type doesn't use a window
	if (!this.messageTarget) {
	
		this.loadWindow = function() {
	
			// create a window
			this.window = WindowManager.createNewWindow();
			this.window.owner = this;
			this.window.sizeCount = 2;
			
			// create an iframe node
			this.iframe = document.createElement ('iframe');
			this.iframe.owner = this;

			this.iframe.className = 'iframeddw';

			// create an iframe cover node, this is to prevent js from halting when mouse enter the iframe zone
			this.iframecover = getTemplate ('iframecover').cloneNode(true);
			this.iframecover.owner = this;
			
			// give unique id
			i = 1;
			while ($('iframecover' + i) != null) {
				i++;
			}
			
			this.iframecover.id = 'iframecover' + i;		

			// append the iframe
			this.window.contentArea.appendChild(this.iframe);	
			this.window.contentArea.appendChild(this.iframecover);	

			// position adjustments
			if (browser.ie) {
				this.iframe.style.left = '14px';
				this.iframe.style.top = '30px';
				this.window.horOffset = 2;
				this.window.verOffset = 0;
			} else if (browser.safari) {
				this.window.horOffset = 1;
				this.window.verOffset = 1;
			} else {
				this.window.horOffset = 3;
				this.window.verOffset = 3;
			}

		
		// set window properties
		this.window.titleBar.innerHTML = this.windowTitle;
		this.window.logo.className = 'window_logo_ddw';
		
		}
		
		// handles dynamic urls
		this.dynamicUrlHandler = function (http_response, callRef) {
			// check if it is proper return state
			if (http_response.readyState !=4)
				return;

			//if (!callRef)
			//	return;		
				
			// extract the url from the reply
			var response = http_response.responseXML.getElementsByTagName("GetDDW").item(0);
			var responseUrl = response.getAttribute('url');
			
			if (responseUrl) {
				//is it a javascript code? 
				if (responseUrl.substr(0,3) == 'JS:') {
					//callRef.owner.window.closeBox.onmouseup();
					//alert (responseUrl.substr(3));
					eval (responseUrl.substr(3));
				} else {
					// assign the url to the iframe
					callRef.loadWindow();
					callRef.initWindow();
					callRef.fullUrl = responseUrl;
					callRef.iframe.src = responseUrl;
				}
			}
		}
		
		this.initWindow = function () {
		
			// set window functions
			// window resize
			this.window.onsize = function () {

				if (!this.sizeCount)
					this.owner.iframecover.style.display = 'block';
				else
					this.sizeCount--;
					
				this.owner.iframe.width = this.contentArea.offsetWidth - this.horOffset;
				this.owner.iframe.height = this.contentArea.offsetHeight - this.verOffset;

				this.owner.iframecover.style.width = this.contentArea.offsetWidth + 'px';
				this.owner.iframecover.style.height = this.contentArea.offsetHeight + 'px';
			
			}

			// window resize stop
			this.window.onsizestop = function () {
				this.owner.iframecover.style.display = 'none';
			}

			// drag options for ie
			if (browser.ie) {
				this.window.ondragstart = function () {
					this.owner.iframecover.className = 'iframecover_inv';
					this.owner.iframecover.style.display = 'block';
				}

				this.window.ondragstop = function () {
					this.owner.iframecover.className = 'iframecover';
					this.owner.iframecover.style.display = 'none';
				}
			}
			
			// on close function
			this.window.onclose = function() {
				// erase taskbar item
				// 
				if (this.tbitem) {
					this.tbitem.remove();
					this.tbitem = null;
				}
			}
			
			
			// update window prototypes
			this.window.activate = function()
			{
				// window object
				var win = this;
				
				// current active window was clicked again
				if (win == WindowManager.active)
					return;
			
				// unfocus the last window
				if (WindowManager.active) {
					WindowManager.active.deactivate();
				}
				
				// change the style
				with (this) {
					dom.t.className = "window_t_a";
					dom.tl.className = "window_tl_a";
					dom.tr.className = "window_tr_a";
					dom.l.className = "window_l_a";
					dom.r.className = "window_r_a";
					dom.b.className = "window_b_a";
					dom.bl.className = "window_bl_a";
					dom.br.className = "window_br_a";
					closeBox.className = "window_closebox";
					minimizeBox.className = "window_minimizebox";
					refreshBox.className = "window_printpreviewbox";
					
					win.dom.style.zIndex = ++WindowManager.maxZIndex;
					//alert('Window Zindex1 = ' + win.dom.style.zIndex);
				}
				
				// set the current focus to this window
				WindowManager.active = win;
				
				setStyle(this.dom, 'display','block');
				
				// check if window has corresponding tab, if so select it
				if (this.tbitem) {
					this.tbitem.setSelected(true,true);
				}

				this.owner.iframecover.style.display = 'none';
				
				// enable scrollbars
				this.owner.iframe.style.overflow = 'auto';
			};
			
			this.window.deactivate = function()
			{
				this.dom.t.className = "window_t";
				this.dom.tl.className = "window_tl";
				this.dom.tr.className = "window_tr";
				this.dom.l.className = "window_l";
				this.dom.r.className = "window_r";
				this.dom.b.className = "window_b";
				this.dom.bl.className = "window_bl";
				this.dom.br.className = "window_br";
				this.closeBox.className = "window_closebox_o";
				this.minimizeBox.className = "window_minimizebox_o";
				this.refreshBox.className = "window_printpreviewbox_o";
				
				WindowManager.active = null;

				this.owner.iframecover.style.display = 'block';
				
				// disable scrollbars
				this.owner.iframe.style.overflow = 'hidden';			
			};
			
			
			// window print preview button replaces refresh box
			if (this.window.refreshBox) {
				this.window.refreshBox.parent = this.window;
				this.window.refreshBox.boundTo = this;	
				this.window.refreshBox.className = 'window_printpreviewbox';

				this.window.refreshBox.onmouseover = function() { this.className = "window_printpreviewbox_a"; };
				
				this.window.refreshBox.onmouseout =
					function() 
					{
						if (WindowManager.active == this.parent)
							this.className = "window_printpreviewbox";
						else
							this.className = "window_printpreviewbox_o";
					};		

				this.window.refreshBox.onmousedown = 
					function(event)
					{ 
						var event = xpEvent(event, true);
						this.className = "window_printpreviewbox_d";
						this.parent.activate();
					};
				
				this.window.refreshBox.onmouseup = function(event)
				{
					var event = xpEvent(event, true);
					// WindowManager.hideWindow(this.parent);
					this.className = "window_printpreviewbox";
					myWindow = window.open(this.boundTo.fullUrl);
					if (window.focus) { myWindow.focus() };	
					if (window.print) { myWindow.print() };
					return false;
				};

			}
			
			// if taskbar exists, add taskbar icon
			if (taskbar) {
				var ti = new taskbarItem(taskbar);
				ti.setLabel(this.window.titleBar.innerHTML);
				ti.setIcon("ddw");
				setStyle (ti.dom, 'display', 'block');
				// create a handler for clicking
				var winRef = this.window;
				ti.dom.onclick = function() { 
					this.parent.setSelected (true, true);
					winRef.activate(); 
				};

				ti.setSelected (true, true);
				ti.parent.refresh();

				this.window.tbitem = ti;
			}
			
			
			// set object functions
			this.redraw = function () {
				this.window.onsize();	
			}

			// display the window
			this.window.open();
			
			this.redraw();

			this.window.activate();
			
		}

		// set iframe properties
		this.fullUrl = this.ddwContents + this.ddwParameters + this.ddwSelectedRecords

		switch (this.ddwType) {
			case 'dynamic_url':
				makeCall (this.fullUrl, this.dynamicUrlHandler, this, null);
				break;
			case 'static_url':
				debugAlert ('DDW (iframe load):' + this.ddwContents + this.ddwParameters);
				this.loadWindow();
				this.initWindow();				
				if (browser.safari) {
					this.ddwContents = ReplaceString(this.ddwContents,"&#38;","&")
				}
				this.iframe.src = this.fullUrl;
				break;
		}
		
	} else {
		// message type
	
		// handles message urls
		this.messageUrlHandler = function (http_response, callRef) {
			// check if it is proper return state
			if (http_response.readyState !=4)
				return;

			if (!callRef)
				return;		
				
			// extract the message from the reply
			var response = http_response.responseXML.getElementsByTagName("GetDDW").item(0);
			var responseMessage = response.getAttribute('message');
			
			// check for object type
			if (callRef) {
				if ((callRef.toString().indexOf ('Window') != -1) || (callRef.toString().indexOf ('DataGrid') != -1)){
					// object is a window
					if (callRef.message) {
						callRef.message.innerHTML = responseMessage;
						if (browser.safari) 
							callRef.message.style.width = '100%';
					}					
				} else if (callRef.toString().indexOf ('Editor') != -1) {
					// object is an editor
					if (callRef.prv.mDOM.status) {
						callRef.prv.mDOM.status.innerHTML = responseMessage;
					}
				}
			}
		}

		// make a call based on message type
		switch (this.ddwType) {
			case 'message_window':
			case 'message_editor':
				if (browser.safari) {
					this.ddwContents = ReplaceString(this.ddwContents,"&#38;","&")
				}
				makeCall (this.ddwContents + this.ddwParameters + this.ddwSelectedRecords, this.messageUrlHandler, this.messageTarget, null);
				break;
		}
	}
		
	return this;
}