/**
 * Editor
 *
 * @author Jonathan Le
 */
 
 var dax_event_onload = 1;
 var dax_event_ondatachange = 20;
 
function Editor()
{
	var node = getTemplate("editor"); if (!node) return;
	// private
	this.prv = {
		mDOM : node.cloneNode(true),
		mTearAway : new TearAway(),
		mColumns : new Array(),
		mInputs : new Array(),
		mModified : false,
		mAutoReload : 60, // in seconds
		mTimeoutId	: null
	}; Function.decl.apply(this);

	// create properties out of HTML hierarchy
	probe(this.prv.mDOM);
	this.prv.mDOM.parent = this;
//	this.prv.mDOM.id = "editor"+getElementsByClassName(document.body, "editor").length;
	generateUniqueId(this.prv.mDOM);

	// override toString
	this.toString = function()
	{
		return "[object Editor]";
	};		
	
	// abort save flag
	this.abortSave = false;	
	this.clickFlag = false;	
	this.hasImages = false;
//	this.originalValue = "";
		
	this.getDOM = function()
	{
		return this.prv.mTearAway.getDOM();
	};
	
	this.getColumnDOM = function()
	{
		if (this.prv.mColumns.length <= 0) {
			return this.prv.mDOM.column;
		} else {
			return this.prv.mDOM.column.cloneNode(true);
		}
	};
	
	// redraw the editor area
	this.refresh = function()
	{
		// let the browser calculate the scroll height
		var style = this.prv.mDOM.style;
		var originalVisibility = style.visibility;
		
		style.visibility = "hidden";
		//if(browser.safari)
			//style.overflow = "hidden";
		//else
			//style.overflow = "scroll";
		style.overflow = "hidden";
		
		var width = Math.floor(this.prv.mDOM.clientWidth / this.prv.mColumns.length)-10;
		//var width = 400;
		for (var j = 0; j < this.prv.mInputs.length; j++) {
			var dom = this.prv.mInputs[j].prv.mDOM;
			if (browser.safari) {
				setStyle(dom, "width", "100%");
			} else {
				setStyle(dom, "width", width + "px");
			}
		}

		for (var i = 0; i < this.prv.mColumns.length; i++) {
			var dom = this.prv.mColumns[i].prv.mDOM;
			if (browser.safari) {
				setStyle(dom, "width", "100%");
			} else {
				setStyle(dom, "width", width + "px");
			}
		}

		// make the horizontal bar automatically visible/invisible
		//if(browser.safari)
			//style.overflow = "hidden";
		//else
		style.overflow = "hidden";
			
		style.visibility = originalVisibility;
		
		this.onafterrefresh();
		
	};
	
	// user customizable
	this.onafterrefresh = function () { };
	
	// set the number of inputs this editor will handle
	this.size = function(aNumberOfInputs, aNumberOfColumns)
	{
		// determine how many columns we want, default is 2
		var aNumberOfColumns = isDefined(aNumberOfColumns) ? aNumberOfColumns : 2;

		//if (this.prv.mColumns.length == aNumberOfColumns && this.prv.mInputs.length == aNumberOfInputs)
		//	return; // don't bother

		// add or remove columns as necessary
		var columnsToAdd = aNumberOfColumns - this.prv.mColumns.length;
		if (columnsToAdd < 0) {
			for (var i = 0; i < Math.abs(columnsToAdd); i++) {
				var col = this.prv.mColumns.pop();
				this.prv.mDOM.removeChild(col.prv.mDOM);
				delete col;
			}
		} else {
			for (var i = 0; i < columnsToAdd; i++) {
				var col = new EditorColumn(this);
				this.prv.mColumns.push(col);
				this.prv.mDOM.insertBefore(col.prv.mDOM, null);
			}
		}
		
		var inputsPerCol = 0;
		var length = aNumberOfColumns;
		var remainder = 0;
		if (aNumberOfInputs == aNumberOfColumns) {
			inputsPerCol = 1;
		} else if (aNumberOfColumns < aNumberOfInputs) {
			inputsPerCol = Math.floor(aNumberOfInputs / aNumberOfColumns);
			remainder = aNumberOfInputs % aNumberOfColumns;
		} else if (aNumberOfColumns > aNumberOfInputs) {
			inputsPerCol = 1;
			length = aNumberOfInputs;
		}
		
		// size each column
		for (var j = 0; j < length; j++) {
			this.prv.mColumns[j].size(remainder > 0 ? inputsPerCol + 1 : inputsPerCol);
			remainder--;
		}
		
		this.prv.mInputs = new Array();
		for (var k = 0; k < this.prv.mColumns.length; k++) {
			var inputs = this.prv.mColumns[k].getInputs();
			for (var l = 0; l < inputs.length; l++) {
				this.prv.mInputs.push(inputs[l]);
			}
		}
		
		// get an updated array of inputs
		//this.refresh();
		this.refresh(); // Double that. Bug Fix
			
	};
	
	// assign values into the inputs
	// loadArray is an array of {label, value, recordid} pair objects
	this.assign = function(aLoadArray)
	{
		var length = this.prv.mInputs.length < aLoadArray ?this.prv.mInputs.length :
			aLoadArray.length;
		for (var i = 0; i < length; i++) {
			if (!aLoadArray[i].skipfield) {
				this.prv.mInputs[i].prv.mFieldID = aLoadArray[i].fieldid;
				this.prv.mInputs[i].setLabel(aLoadArray[i].label);
				this.prv.mInputs[i].setValue(aLoadArray[i].value, aLoadArray[i].type, aLoadArray[i].picWidth, aLoadArray[i].picHeight, aLoadArray[i].format, aLoadArray[i].ddw, aLoadArray[i].ddwtype, aLoadArray[i].ddwtitle, aLoadArray[i].fieldtype);
				if ((aLoadArray[i].nonmodifiable != undefined) || (this.recordLocked)){
					if ((aLoadArray[i].nonmodifiable == "True")  || (this.recordLocked)){
						// check for type to disable non modifiable fields
						if (this.prv.mInputs[i].choicelist) {
							this.prv.mInputs[i].prv.mDOM.value.disabled = true;
							this.prv.mInputs[i].prv.mDOM.value.disableChoiceList = true;
						} else if (aLoadArray[i].type == 4) {
							//this.prv.mInputs[i].prv.mDOM.value.disabled = true;
							this.prv.mInputs[i].picker.value.disabled = true;
							this.prv.mInputs[i].picker.isDisabled = true;
						} else if (this.prv.mInputs[i].field.fieldtype == 'boolean'){
							this.prv.mInputs[i].isDisabled = true;
						} else {
							this.prv.mInputs[i].prv.mDOM.value.disabled = true;
						}	
					} else {
						// check for type to disable non modifiable fields
						if (this.prv.mInputs[i].choicelist) {
							this.prv.mInputs[i].prv.mDOM.value.disabled = false;
							this.prv.mInputs[i].prv.mDOM.value.disableChoiceList = false;
						} else if (aLoadArray[i].type == 4) {
							//this.prv.mInputs[i].prv.mDOM.value.disabled = true;
							this.prv.mInputs[i].picker.value.disabled = false;
							this.prv.mInputs[i].picker.isDisabled = false;
						} else if (this.prv.mInputs[i].field.fieldtype == 'boolean'){
							this.prv.mInputs[i].isDisabled = false;
						} else {
							this.prv.mInputs[i].prv.mDOM.value.disabled = false;
						}				
					}
				} else {
					// check for type to disable non modifiable fields
					if (this.prv.mInputs[i].choicelist) {
						this.prv.mInputs[i].prv.mDOM.value.disabled = false;
						this.prv.mInputs[i].prv.mDOM.value.disableChoiceList = false;
					} else if (aLoadArray[i].type == 4) {
						//this.prv.mInputs[i].prv.mDOM.value.disabled = true;
						this.prv.mInputs[i].picker.value.disabled = false;
						this.prv.mInputs[i].picker.isDisabled = false;
					} else if (this.prv.mInputs[i].field.fieldtype == 'boolean'){
						this.prv.mInputs[i].isDisabled = false;
					} else {
						this.prv.mInputs[i].prv.mDOM.value.disabled = false;
					}					
				}
				this.prv.mInputs[i].prv.mFieldRecordID = aLoadArray[i].fieldrecordid;
			}
		}
	};
	
	this.redraw = function()
	{
	
		this.addSelectionHeader();
	
		// set the buttons style to visible
		setStyle(this.prv.mDOM.bottom.buttons, "display", "block");

		var ta = this.getTearAway();
		ta.size(ta.prv.mDOM.offsetWidth, ta.prv.mDOM.offsetHeight);
		var buttonDelete = getElementByClassName(this.prv.mDOM,'deletebtn');
		//buttonDelete.disabled = false;
		buttonDelete.innerHTML = STR['button_delete'];
		
		var taDom = ta.getDOM();
		
		// fix for width issues
		taDom.top.style.width = '600px';
		taDom.middle.style.width = '600px';
		taDom.bottom.style.width = '600px';
		
		// ie7 vertical spacing bug fix
		//if (browser.ie7) {
		taDom.middle.ml.style.height = taDom.middle.m.offsetHeight + 'px';
		taDom.middle.mr.style.height = taDom.middle.m.offsetHeight + 'px';
		taDom.style.height = taDom.top.offsetHeight + taDom.middle.offsetHeight + taDom.bottom.offsetHeight + 'px';
		//}
		
		// fix for ACI0054603
		if (browser.safari3) {
			if (taDom.style.zIndex < 9000) {
				setStyle(taDom, "zIndex", ++WindowManager.maxZIndex);
			} else { 
				setStyle(taDom, "zIndex", ++WindowManager.maxZDashIndex);
			}		
		}
				
	};
	
	// gets data when selection picker is selected
	// record id is from parent table record
	this.callSelectionHeaderData = function (aRecordId) {
	
		// table id main table 
		var aTableId = this.record.tableId;
		
		// Async request
		var http_request = getXMLHttpRequest();
		
		if (http_request) {
			var ref = this;
			http_request.onreadystatechange = function() { ref.onparentpull(http_request); };
			
			try {
				var url = dax_bridge.url + "getParentRecord?tableid=" + aTableId +
					"&recordid=" + aRecordId + "&sessionId=" + dax_bridge.sessionId;
				debugAlert('<b>Editor (.parentPull):</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
				http_request.open("GET", url, true);
	  	  		http_request.send(null);
	   		} catch(e) {
				if (browser.ie6)
					http_request.onreadystatechange = function(){};
				else
					http_request.onreadystatechange = null;
	   			delete http_request;
		    }
		}		
		
	}
	
	this.addSelectionHeader = function () {

		var aTable = getTable(this.record.tableId);
	
		// do we need the picker?
		// yes: only for new records, and if there is a related table	
		// no: datatree shouldn't do use picker
		if ((this.record.recordId != -1) || (aTable.selectionviewid == 0) || ((this.owner.boundTo) && (this.owner.boundTo.spacer))) {
			// don't need it, hide it if it exists
			if (this.topContents)
				this.topContents.style.display = 'none';
			return;
		} else {
			// we need it, but return if it exists because it's already created
			if (this.topContents) {
				this.topContents.style.display = 'block';
				return;		
			}
		}
		
		// go ahead, create the picker		
		var selTable = getTable(aTable.selectionviewid);	
		//var selTable = getTable('Tasks');	
			
		var headerNode = getTemplate('selectionvieweditor');
		
		if (!headerNode)
			return;
		
		this.topContents = headerNode.cloneNode(true);
		this.topContents.parent = this;
		
		this.prv.mDOM.top.appendChild (this.topContents);
		this.topContents.style.display = 'block';
		
		probe (this.topContents);
	
		this.topContents.selButton.owner = this;
		this.topContents.selButton.value = STR['button_choose'] + ' '+ selTable.tablealias;
		
		//this.topContents.selInput.disabled = true;
		//this.topContents.selInput.style.display = 'none';
		
		// setup top contents functions

		this.topContents.selHelp.onmousemove = function (aEvent) {
			aEvent = xpEvent (aEvent);
			mousePos = getMousePosition (aEvent);
			showTooltip(STR['editor_selectionview'], mousePos.x, mousePos.y);
		}
		
		
		this.topContents.selHelp.onmouseout = function () {
			hideTooltip();
		}	
		
		this.topContents.selHelp.style.display = 'none';
		

		this.topContents.selButton.onclick = function () {
			
			if (!WindowManager.modalWindow) {
				WindowManager.modalWindow = new modalWindow();
				var firstLoad = true;
			} 

			this.owner.prv.mDOM.saveBtn.disabled = false;

			objectPos = getObjectPosition (this);
			
			WindowManager.modalWindow.show();
			WindowManager.modalWindow.setSize(400, 300);
			WindowManager.modalWindow.position(objectPos.x - 200, objectPos.y);
			
			//if (firstLoad) {
			this.owner.dataWindow = new DataWindow(selTable.tablename, WindowManager.modalWindow.contents, null, null, null);
			this.owner.dataWindow.customize (false, false, false, false);	
			this.owner.dataWindow.contentArea.style.overflow = 'hidden';
						
			if (browser.safari) {
				this.owner.dataWindow.contentArea.style.width = '400px';
				this.owner.dataWindow.contentArea.style.height = '300px';
				this.owner.dataWindow.contentArea.grid.refresh(); // contentArea.grid.reload();
				this.owner.dataWindow.activate = function() { return false;};

			}

			this.owner.dataWindow.grid.grid.boundToEditor = this.owner;
			this.owner.dataWindow.grid.grid.onrowselect = function (aRow) {
				
				// check for empty row
				if (aRow.prv.mElements[0].prv.mDOM.innerHTML != '') {
				
					this.boundToEditor.parentRecordId = aRow.recordId;
					//this.boundToEditor.topContents.selInput.value = aRow.element(0).prv.mContent;
					//this.boundToEditor.topContents.selLabel.innerHTML =
					//	aRow.prv.mParent.prv.mColumns[0].prv.mHeader.prv.mContent;
					//this.boundToEditor.topContents.selInput.style.display = 'block';
					WindowManager.modalWindow.hide();
					//this.boundToEditor.dataWindow.dom.style.display = 'none';
					WindowManager.destroyWindow(this.boundToEditor.dataWindow);	
					
					this.boundToEditor.callSelectionHeaderData(aRow.recordId);
				}
			}
			
			WindowManager.modalWindow.blackout.boundToEditor = this.owner;
			WindowManager.modalWindow.blackout.onclick = function () {
				WindowManager.modalWindow.hide();
				//this.boundToEditor.dataWindow.dom.style.display = 'none';
				WindowManager.destroyWindow(this.boundToEditor.dataWindow);	
			}

			this.owner.dataWindow.grid.grid.onrowdblclick = function (aRow) {	};			
		//} else {
		//	this.owner.dataWindow.dom.style.display = 'block';
		//}
			
			this.owner.dataWindow.dom.style.zIndex = ++WindowManager.maxZIndex + 25;
		};
		
		return false;
	}			
		
	
	// load the inputs with data from a database
	this.pull = function(aTableId, aRecordId)
	{
		// Async request
		var http_request = getXMLHttpRequest();
		
		if (http_request) {
			var ref = this;
			http_request.onreadystatechange = function() { ref.onpull(http_request); };
			
			try {
				var url = connection.command(DB.GET_RECORD) + "&tableid=" + aTableId +
					"&recordid=" + aRecordId;
				debugAlert('<b>Editor (.pull):</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
				http_request.open("GET", url, true);
	  	  		http_request.send(null);
	   		} catch(e) {
	   			delete http_request;
		    }
		}
	};
	
	
	this.onparentpull = function(http_request)
	{
		this.onpull(http_request, true)
	}
	
	this.onpull = function(http_request, isParentPull)
	{
		if (http_request.readyState != 4)
			return;
		var result = http_request.responseXML.getElementsByTagName("queryResult").item(0);
		var errorPresent = checkResponse(http_request);

		if ((errorPresent) || (!result))
			return;

		if (!isParentPull)
			isParentPull = false;
			
		// examine the structure
		var tableId = result.getAttribute(STR['tableid']);
		var s_table = structure.getTable(tableId);
			
		if (callback_onEditorPull)
			http_request = callback_onEditorPull(s_table.tablename, http_request);
		
		
		var row = result.getElementsByTagName(STR['rowtag']).item(0);
		
		if (!row)
			return;
		
		this.isBackendLocked = row.getAttribute('locked');
		
		var fields = row.getElementsByTagName(STR['fieldtag']);
		
		if (!fields)
			return;
		
		
		if (!isParentPull) {

			if ((this.record) && (this.isBackendLocked != 'True')) {
				// if we are already editing a record, we need to unlock it now
				if (!Controller.isLocked(this.record.tableId, this.record.recordId))
					Controller.unlockRecord(this.record.tableId, this.record.recordId);
			}
		
			// internal data
			this.record = {
				recordId : row.getAttribute(STR['recordid']),
				tableId  : tableId,
				map : new Array()
			};
			
		}
		
		// list of fields
		if (!s_table.fields)
			buildFieldList(tableId);
//		var s_fields = s_table.fields;
		var s_fields = [ ];
		for (var fieldCounter = 0; fieldCounter < s_table.fields.length; fieldCounter++) {
			var fieldRef = getField(s_table.tableid, s_table.fields[fieldCounter].fieldid);
			if (fieldRef.fieldindetail.toLowerCase() == 'true'){
				s_fields.push(fieldRef);
			}
		}

		this.size(s_fields.length, 1); // size the inputs

		// prepare the assignment array
		var assign = new Array();

		// create an array of id's that will be updated
		var updateFieldIds = new Array();

		// populate update fields array
		if (isParentPull) {
			// add fields to array for easy search later
			for (i = 0; i < fields.length; i++) {
				updateFieldIds.push (fields[i].getAttribute('id'));
			}		
		}

		// create a map
		for (var i = 0 ; i < s_fields.length; i++) {

			// checks if fields to update id array exists, and if it does only update those fields
			if ((updateFieldIds.length == 0) || (findInArray(updateFieldIds, s_fields[i].fieldid) != -1)) {
			
				this.record.map[s_fields[i].fieldid] = i;
				assign[i] = {
					label: s_fields[i].fieldalias,
					value: '',
					type: this.convertType(s_fields[i].fieldtype),
					fieldrecordid:  fields[i] ? fields[i].getAttribute('fieldrecordid') : null,
					fieldid: s_fields[i].fieldid,
					ddw: fields[i] ? fields[i].getAttribute('ddw') : null,
					ddwtype: fields[i] ? fields[i].getAttribute('ddwtype') : null,
					ddwtitle: fields[i] ? fields[i].getAttribute('ddwtitle') : null,
					skipfield: false,
					fieldtype: s_fields[i].fieldtype
				};

			} else {
				
				assign[i] = { skipfield: true };			
				
			}
			/*
			if (isDefined(http_request)) {
				http_request.onreadystatechange = null;
				delete http_request;
			}
			*/
		}

//		yourvariable: fieids[i].getAttribute('nameofattrib')

		var eTable = getTable(tableId);

		dax_bridge.cacheFields(eTable);
		
		// DDW's on selection level (disabled)
		
		/*
		
		if ((eTable) && (eTable.ddwOptions) && (eTable.ddwOptions.length > 0)) {
			
			// check for button type
			var ddwOption = eTable.ddwOptions[0];
			this.ddwButton1.style.display="block";
			this.ddwButton1.url = ddwOption.url;
			this.ddwButton1.title = ddwOption.title;
			this.ddwButton1.value = ddwOption.title;
			this.ddwButton1.ddwType = ddwOption.type;
			this.ddwButton1.innerHTML = ddwOption.title;
			this.ddwButton1.style.display="inline";
		} else {

			this.ddwButton1.style.display="none";
		
		}
		
		*/
		
		// fill the assignment array
		for (var i = 0; i < fields.length; i++) {
			var index = this.record.map[fields[i].getAttribute(STR['fieldid'])];
			if ((assign[index]) && (!assign[index].skipfield)) {
				var rField = getField(tableId, fields[i].getAttribute('id'));
				if (rField.fieldtype == 'picture') {
					// getting the picture dimensions
					var fieldWidth =  fields[i].getAttribute('width'); 
					var fieldHeight = fields[i].getAttribute('height');
				} else {
					var fieldWidth = null;
					var fieldHeight = null;
				}
				elValue = n2String(fields[i]);
				if(rField.fieldformat)
					var elFormat = rField.fieldformat;
				else{
					if(rField.fieldtype == 'date')
						var elFormat = 'd1';
					else
						var elFormat = rField.fieldformat;
				}
					
				var splitedFormat = elFormat.split(',');// breaks the formatting ins parts
				switch(elFormat.substr(0,1))
				{
					 case 'x':
					 case 'a':
					 break;
					 
					 case 'r':
						if(splitedFormat[2] == 'cm'){
							if(elValue.lastIndexOf('.') != -1)// replaces the '.' with commas ",' because of 4D
								elValue = elValue.replace(elValue.substr(elValue.lastIndexOf('.'),1),",");
						}
					break;
					
					case 'd':
				}
				
				if ((elFormat) && (assign[index].type != 4)) //apply the format if the type is not date
					elValue = format(elFormat, elValue);			

				assign[index].format = elFormat;
				assign[index].value = elValue;
				assign[index].picWidth = fieldWidth;
				assign[index].picHeight = fieldHeight;
				assign[index].nonmodifiable = "False";
				if (rField.fieldnonModifiable != undefined)
					if (rField.fieldnonModifiable == "True")
						assign[index].nonmodifiable = "True";
				if (rField.fieldnonEnterable != undefined)
					if (rField.fieldnonEnterable == "True")
						assign[index].nonmodifiable = "True";
				if (rField.fieldcallbackonload != undefined)
					if (rField.fieldcallbackonload) {
						var frid = fields[index].getAttribute('fieldrecordid'); 
						if (frid=="-1")
							frid = "-3";		
						ExecuteCallBack(this, dax_event_onload, tableId, rField.fieldid+"["+frid+"]", elValue, index, assign[index]);
					}
			}
		}

		
		
		// check the new record
		if (!isParentPull) {

			if ((this.isBackendLocked == 'True') || (Controller.isLocked(this.record.tableId, this.record.recordId))) {
				this.lock(); // lock the editor
			} else {
				Controller.lockRecord(this.record.tableId, this.record.recordId);
			}

		}
		
		//this.addSelectionHeader();
		this.assign(assign);
		this.redraw();
		
		if (!isParentPull) {		

			this.onAfterPull();	
		
			// start the first auto-reload
			var editor = this;
			var auto = editor.prv.mAutoReload;
			if (auto > 0) {
				this.prv.mTimeoutId = setTimeout(function() { editor.reload(); }, auto * 3000);
			}
		}
	};

	// user defined
	this.onAfterPull = function () { };
	
	// to base an editor on a table structure
	this.buildEditorByTable = function(aTableId)
	{
		// create a mapping if one hasn't already been
		if (!isDefined(this.record)) {
			this.record = {
				map : new Array()
			};
		} else if (!isDefined(this.record.map)) {
			this.record.map = new Array();
		}
		
		// list of fields
		var s_table = structure.getTable(aTableId);
		if (!s_table.fields) {
			buildFieldList(s_table.tableid);
		}
		
		var s_fields = [ ];
		for (var fieldCounter = 0; fieldCounter < s_table.fields.length; fieldCounter++) {
			var fieldRef = getField(s_table.tableid, s_table.fields[fieldCounter].fieldid);
			if (fieldRef.fieldindetail.toLowerCase() == 'true'){
				s_fields.push(fieldRef);
			}
		}
		
		//var s_fields = s_table.fields;
		this.size(s_fields.length, 1); // size the inputs
		dax_bridge.cacheFields(aTableId);
		
		this.parentRecordId = null;
		// temporary keep parent record id value
		this.parentRecordIdBuffer = null;
		
		// prepare the assignment array
		var assign = new Array();

		// create a map
		for (var i = 0 ; i < s_fields.length; i++) {
			var type = this.convertType(s_fields[i].fieldtype);
			var fieldtype = s_fields[i].fieldtype;
			this.record.map[s_fields[i].fieldid] = i;
			assign[i] = { label: s_fields[i].fieldalias, value: '', type: type , fieldid: s_fields[i].fieldid, fieldrecordid: '-1', format: s_fields[i].fieldformat, nonmodifiable: s_fields[i].fieldnonEnterable, fieldtype: fieldtype};
			var bridgefieldobj = getField(aTableId,s_fields[i].fieldid);
			if (bridgefieldobj.fieldcallbackonload != undefined)
				if (bridgefieldobj.fieldcallbackonload)
					ExecuteCallBack(this, dax_event_onload, aTableId, s_fields[i].fieldid+"[-3]", "", i, assign[i]);
		}
		
		this.assign(assign);
		//this.addSelectionHeader();
		this.redraw();
		this.onAfterBuild();
	};
	
	// user defined function
	this.onAfterBuild = function() {  };
	
	this.convertType = function(aTypeString)
	{
		switch (aTypeString)
		{
			case STR['type_alpha']:
				return 0;
			case STR['type_date']:
				return 4;
			case STR['type_text']:
				return 3;
			case STR['type_picture']:
				return 2;
			case STR['type_real']:
			case STR['type_integer']:
			case STR['type_longint']:
				return 1;
			default:
			{
//				alert('convertType alert:' + aTypeString)
				return 0;
			}
		};
	};
	
	// when a user clicks on the Editor's DOM
	this.prv.mDOM.onclick = function(aEvent)
	{
		//var event = xpEvent(aEvent, true);
		//Controller.setFocused(this.parent);
		//return false;
	};
	
	// the Controller object calls this function
	this.onkeydown = function(aEvent)
	{
		//alert('typing');
	};
	
	this.validateError = function(aInputRef)
	{
		var type = aInputRef.getType();
		var status = this.prv.mDOM.status;
		switch (type)
		{
			case 1: // number
			{
				status.innerHTML = STR['status_err_number'];
				aInputRef.prv.mDOM.value.focus();
				aInputRef.prv.mDOM.value.select();
			}
			break;
			default:
			{
				status.innerHTML = STR['status_err_default'];
				aInputRef.prv.mDOM.value.focus();
				aInputRef.prv.mDOM.value.select();
			}
			break;
		}
		
	};

	// delete the record
	this.deleteR = function()
	{
		// ask the user	
		var http_request = getXMLHttpRequest();
		if (http_request) {
			var ref = this;
			http_request.onreadystatechange = function() { ref.ondelete(http_request); };
			if (isDefined(this.record)) {
				if (this.record.recordId == -1){
					alert(STR['record_not_in_database']);
					return false;
				}
				else{
					var rv = confirm(STR['confirm_delete_records']);
					if (!rv)
						return;
				}
			}
			var tableId = this.record.tableId;
			var recordId = this.record.recordId;
			try {
				var command = recordId != -1 ? connection.command(DB.DELETE_RECORD) : connection.command(DB.DELETE_RECORD);
				var url = command + "&tableid=" + tableId + "&recordid=" + recordId;	
				http_request.open("GET", url, true);
				http_request.send(null);
			} 
			catch(e) {
				if (browser.ie6)
					http_request.onreadystatechange = function(){};
				else
					http_request.onreadystatechange = null;
				delete http_request;
			}
		}
	}
	
	this.ondelete = function(http_request)
	{
		if (http_request.readyState != 4)
			return;
		var result = http_request.responseXML.getElementsByTagName("recordErased").item(0);

		var errorResponse = checkResponse(http_request);
		if (!isDefined(result)) {
			
			//alert('check response ' + http_request.responseText);
			this.prv.mDOM.status.innerHTML = http_request.responseText;
			if (browser.ie6)
				http_request.onreadystatechange = function(){};
			else
				http_request.onreadystatechange = null;
			delete http_request;
			var buttonChange = getElementByClassName(this.getDOM(),'cancelbtn');
			return;
		}
		
		var success = result.getAttribute(STR['modify_success']) == STR['success_true'];
		var dom = this.prv.mDOM;
		var buttonChange = getElementByClassName(this.getDOM(),'cancelbtn');
		if (result) {
			if (this.owner)
				if (this.owner.contentArea)
					if (this.owner.contentArea.grid) {
						this.owner.contentArea.grid.reload();
					}
			this.onafterdelete();
		} 
		else {
			dom.status.innerHTML = STR['status_err_delete'];
		}
		
		// reset
		this.setModified(false);
		if (browser.ie6)
			http_request.onreadystatechange = function(){};
		else
			http_request.onreadystatechange = null;
		delete http_request;

	};

	this.onafterdelete = function() { 
		this.getTearAway().contract(1, 50);
		this.destroy();
	};

	
	// save the record
	this.save = function()
	{		
		// abort save if flag is set
		if (this.abortSave) {
			// disable button
			this.prv.mDOM.saveBtn.disabled = true;

			// reset flag
			this.abortSave = false;
			return;
		}
		
		if (isDefined(this.record)) {
			var http_request = getXMLHttpRequest();
			
			if (http_request) {
				var ref = this;
				http_request.onreadystatechange = function() { ref.onsave(http_request); };
				var dom = this.prv.mDOM;
				var tableId = this.record.tableId;
				var recordId = this.record.recordId;
				
				try {
					
					// var command = recordId != -1 ? connection.command(DB.MODIFY_RECORD) :
					//	 connection.command(DB.ADD_RECORD);
					var url = recordId != -1 ? connection.address.modifyrecord :
						connection.address.addrecord;
						
					var params = "sessionid=" + connection.sessionId + "&tableid=" + tableId +
						"&recordid=" + recordId;
					// upload binary queue
					this.record.uploads = new Array();
					var a = "Field(s) required(s)\n" // variable to keep the header of the alert box
					var alertErrors = "";

					var errorFlag = 0;
					
					// keep data in array for callback processing
					var dataArrays = new Object();
					
					dataArrays.fieldId = new Array();
					dataArrays.fieldName = new Array();
					dataArrays.value = new Array();

					
					for (var i in this.record.map) {

						var index = this.record.map[i];
						var input = this.prv.mInputs[index];
						var EmptyField = this.prv.mInputs[index].prv.mDOM.value;
						var sField = getField (this.record.tableId, i);
						
						// sets flag to check if there is any error on the 4 fields
						if(sField.fieldmandatory == 'True'){ // gets the field propertie from the database to check if field is mandotory or not
							if(input.getValue().length < 1){		
								setStyle(EmptyField,'border', 'solid 1px red');
								errorFlag = 1;
							}
						}
						
						if (input.getModified()) {
							if (input.getType() == 2) { // image
								this.record.uploads.push(i); // field to upload
							} 
							else{
								var newValue = "";
								var newA = sField.fieldformat.split(',');
								switch(sField.fieldformat.substr(0,1))
								{
									case 't': //time
										if(sField.fieldformat == "tMM:SS")
											newValue = "00:" + input.getValue();// formats the string in 4D format
										else
											newValue = input.getValue();
									break;
									
									case 'i': //integer
									case 'l': //long integer
									case 'r': // real
										if(newA[2] == "cm" && input.getValue().lastIndexOf(',') != -1){//check if the decimal separator is a comma ","	
											if (newA[1] == "per") {
												newValue = input.getValue();
												
												while (newValue.indexOf('.') != -1) {//replaces the "." by spaces and puts the period "." as a decimal separator accept by 4D
													newValue = newValue.replace('.',"");
												}
											}
											newValue = newValue.replace(newValue.substr(newValue.lastIndexOf(','),1),".");
										}
										else if(newA[2] == "per" && input.getValue().lastIndexOf('.') != -1){
											newValue = input.getValue();
											if(newValue.indexOf(' %') != -1)
												newValue = newValue.substr(0,newValue.indexOf(' %'));
											//replaces the "." by spaces and puts the period "." as a decimal separator accept by 4D
											while (newValue.indexOf(',') != -1) {
												newValue = newValue.replace(',',"");
											}
										}
										else
											newValue = input.getValue();
									break;
									
									case 'b': //boolean
										/*
										if(input.getValue() == "vrai" || input.getValue() == "yes" || input.getValue() == "male" || input.getValue() == "true" )
											newValue = "true";
										else
											newValue = "false";
										*/
										
										newValue = input.getValue();
									
									break;
									case 'd': //date
										newValue = input.getOriginalValue();//this.originalValue + 'T00:00:00';
									break;
									default:
										newValue = dax_bridge.encode(input.getValue());
								}
							}

							dataArrays.value.push (newValue);
							dataArrays.fieldName.push (getField(tableId, i).fieldname);
							
							if (!input.prv.mFieldRecordID) {
								dataArrays.fieldId.push (i);
							} else {
								dataArrays.fieldId.push (i + "[" + input.prv.mFieldRecordID + "]");
							}

						}
					}
					
					//before save callback
					dataArrays = callback_beforeRecordSave(getTable(tableId).tablename, recordId, dataArrays);

					// finish params building
					
					for (m = 0; m < dataArrays.fieldId.length; m++) {
						params += "&field" + dataArrays.fieldId[m] + "=" + dataArrays.value[m];
					}
					
					delete dataArrays;
					
					if (this.parentRecordId)
						params += '&parent=' + this.parentRecordId;
					
					if(errorFlag == 1){ // if any mandatory field was left blank gives the warn
						dom.status.innerHTML = STR['status_err_required'];
					}else{

					debugAlert ('<b>Editor (.save)</b>:' + url + params);

					http_request.open("POST", url, true);
					//http_request.setRequestHeader("Content-Type","text/html; charset=Shift_JIS");
					http_request.send(params);
					}
		   		} catch(e) { alert(e);
					if (browser.ie6)
						http_request.onreadystatechange = function(){};
					else
						http_request.onreadystatechange = null;
		   			delete http_request;
			    }
			}
		}
	};

	this.checkupload = function(new_iframe, fieldId, islast) {
	
		if (!browser.ie) {
			var aDoc = new_iframe.contentDocument;
		}
		else {
			if (document.frames["upload_" + fieldId])
				var aDoc = document.frames["upload_" + fieldId].document;
		}			
		
		if (aDoc) {
			if (aDoc.forms)
				var aform = aDoc.forms[0];
		}
		
		if (aform) {
			if ((aform.id == "binaryupload") & (aform.sessionid.value == "")) {
				this.uploading = false;
				if (islast) {
					var editor = this;
					setTimeout(function() {editor.pull(editor.record.tableId, editor.record.recordId);}, 500);
					//setTimeout(function() {editor.onaftersave();}, 500);
					setTimeout(function() {editor.reload();}, 500);	
					this.onaftersave();						
				}
			}
			else {
				var thisthis = this;
				setTimeout(function() {thisthis.checkupload(new_iframe, fieldId, islast);}, 500);
			}					
		}
		else {
			var thisthis = this;
			setTimeout(function() {thisthis.checkupload(new_iframe, fieldId, islast);}, 500);
		}		
	};
	
	this.uploadPic = function(aform, aframe, afieldid, islast) {
	
		if (!this.uploading) {
			this.uploading = true;
			aform.submit();
			var editor = this;
			setTimeout(function() {editor.checkupload(aframe, afieldid, islast);}, 500);
		}
		else {
			var editor = this;
			setTimeout(function() {editor.uploadPic(aform, aframe, afieldid, islast);}, 500);
		}
		
	};
	
	this.onsave = function(http_request)
	{
		
		var dom = this.prv.mDOM;
		var buttonChange = getElementByClassName(this.getDOM(),'cancelbtn');
		if (http_request.readyState != 4)
			return;
		// check for errors
		var errorResponse = checkResponse (http_request);

		var sField = "";
		var fieldErrors = http_request.responseXML.getElementsByTagName("errordetail"); // error created to handle unique fields
		var uniqueError = false;
		if (fieldErrors) {
			if (fieldErrors.length > 0) {
				for (i = 0; i < fieldErrors.length; i++) {
					for (var j in this.record.map) {
						var index = this.record.map[j];
						var eValue = fieldErrors[i].getAttribute("value"); // loops throught the fields to get the specific field with problem						
						if(j == eValue){ // compares if the field with error is the same of the field been checked 
							var DuplicatedField = this.prv.mInputs[index].prv.mDOM.value;
							setStyle(DuplicatedField,'border','solid 1px red');
							dom.status.innerHTML = STR['status_err_dup']; // display the message of duplicate record
							return false;
						}
					}
				}	
			}

		}
		

		var buttonSave = getElementByClassName(this.getDOM(),'savebtn');
		var results = http_request.responseXML.getElementsByTagName("addedRecord").item(0);

		if ((!DuplicatedField) && (!results)) {
			results = http_request.responseText;
			dom.status.innerHTML = results;
			return false;
		}

		
		var success = results.getAttribute(STR['modify_success']) == STR['success_true'];
		
		if (success) {
			dom.status.innerHTML = STR['status_save_success'];

			if ((this.record) && (this.isBackendLocked != 'True'))
				Controller.unlockRecord(this.record.tableId, this.record.recordId);
				
			buttonChange.innerHTML = STR['button_close'];
			buttonSave.disabled = true;

			
			// do only if we have the grid
			/*
			if (this.owner)
				if (this.owner.contentArea)
					if (this.owner.contentArea.grid) {
						this.owner.contentArea.grid.reload();
					}
				
			*/		
					
			var recordId = n2String(results.getElementsByTagName(STR['recordidtag']).item(0));
			
			// if it's a new record, update record id
			if (this.record)
				if (this.record.recordId == -1)
					this.record.recordId = recordId;
			
			if (!this.hasImages) {
				this.pull(this.record.tableId, this.record.recordId);
			}
			
			if (callback_afterRecordSaveSuccess) {
				callback_afterRecordSaveSuccess(getTable(this.record.tableId).tablename, recordId);
			}

			
			/*
			
			// do only if we have the grid
			if (this.owner)
				if (this.owner.contentArea)			
					if (this.owner.contentArea.grid) {			
						if (ref.owner.contentArea.grid.selection.range.length)
							ref.owner.contentArea.grid.selection.range.length = parseFloat(ref.owner.contentArea.grid.selection.range.length) + 1;
					}
			
			*/
			
			// update binaries
			
			var fieldrecords = results.getElementsByTagName('binaryfield');
			
			for (var j = 1; j <= fieldrecords.length; j++) {
				var index = this.record.map[fieldrecords.item(j-1).getAttribute("fieldid")];
				if (index) {
					var input = this.prv.mInputs[index];
					input.prv.mFieldRecordID = fieldrecords.item(j-1).lastChild.nodeValue;
				}
			}
			if (this.record.uploads) {
				if (this.record.uploads.length == 0)
					this.onaftersave();
				else {
					while (this.record.uploads.length > 0) {
						var fieldId = this.record.uploads.pop();
						var index = this.record.map[fieldId];
						var input = this.prv.mInputs[index];
						var type = input.getType();
						
						if (type == 2) { //image
							if (!browser.ie) {
								var aframe = input.an_iframe;
								var aform = input.an_iframe.contentDocument.forms[0];
								var aspinner = input.an_iframe.contentDocument.getElementById("spinner");
							}
							else {
								var aframe = document.frames["upload_" + fieldId];
								var aform = document.frames["upload_" + fieldId].document.forms[0];
								var aspinner = document.frames["upload_" + fieldId].document.getElementById("spinner");
							}
							if (aform) {
								aform.sessionid.value = connection.sessionId;
								aform.tableid.value = this.record.tableId;
								aform.fieldid.value = fieldId + "[" + input.prv.mFieldRecordID + "]";
								aform.recordid.value = this.record.recordId;			
								aspinner.style.display = "block";
								this.uploadPic(aform, aframe, fieldId, this.record.uploads.length == 0);
							}
							else
								alert("Sorry error: upload_" + fieldId);
						}		
					}
				}
			}
			else
				this.onaftersave();
		} else {
			dom.status.innerHTML = STR['status_save_fail'];
			buttonChange.innerHTML = STR['button_cancel'];
		}
		
		// reset
		this.setModified(false);
		
	//	if (browser.gecko) {
			// firefox refersh
			// this.refresh();
	//	}
		// tail end
		//
			//var editor = this;
			//setTimeout(function() {editor.pull(editor.record.tableId, editor.record.recordId);}, 5000);
			//setTimeout(function() {editor.onaftersave();}, 5000);
			//setTimeout(function() {editor.reload();}, 5000);
		
		if (browser.ie6)
			http_request.onreadystatechange = function(){};
		else
			http_request.onreadystatechange = null;
		delete http_request;
	};
	
	this.onaftersave = function() {};

	// this is the binary to upload
	this.upload = function()
	{
		debugAlert('Upload function');
		if (!this.record || !this.record.uploads || this.record.uploads.length <= 0)
			return;
		var fieldId = this.record.uploads.pop();
		var index = this.record.map[fieldId];
		var input = this.prv.mInputs[index];
		var type = input.getType();
		
		if (type == 2) { //image
			var path = input.prv.mDOM.value.browse.value;
			var upload = frames['upload'].document;
			var aform = upload.getElementById("binaryupload");
			var state = upload.getElementById("state");
			state.className = "inuse";
			state.innerHTML = "inuse";
			
			// duplicate file
			var afile = input.prv.mDOM.value.browse.cloneNode(true);
			afile.name = "DAX_BinaryFile";
			var bfile = upload.createElement("input");
			bfile = afile;
			//afile.setAttribute("type", "file");
			//afile.setAttribute("name", "DAX_BinaryFile");
			//afile.setAttribute("class", "browse");
			//afile.setAttribute("value", input.prv.mDOM.value.browse.value);
			//alert(afile.value);
			//afile.setAttribute("ownerDocument", upload);
			alert(upload.importNode);
			aform.sessionid.value = connection.sessionId;
			aform.tableid.value = this.record.tableId;
			aform.fieldid.value = fieldId + "[" + input.prv.mFieldRecordID + "]";
			aform.recordid.value = this.record.recordId;
			alert(bfile.ownerDocument);
			aform.appendChild(bfile);
			debugAlert('before submit');
			aform.submit();
			debugAlert('waiting for the file to be uploaded...');
			
		}
		debugAlert('after and before inqueue');
		Controller.uploading = false;
		// proceed to next upload by queueing ourselves
		if (this.record.uploads.length > 0) {
			Controller.enqueue(this);
		} else {
			// reload in 3 seconds
			var editor = this;
			debugAlert ('reload 962');
			setTimeout(function() {editor.pull(editor.record.tableId, aform.recordid.value);}, 500);
			setTimeout(function() {editor.onaftersave();}, 500);
			setTimeout(function() {editor.reload();}, 500);
		}
	};
	
	// clear the contents of all the inputs
	this.clear = function()
	{ 
		for (var i = 0; i < this.prv.mInputs.length; i++) {
			this.prv.mInputs[i].clear();
		}

		// reset the status
		this.prv.mDOM.status.innerHTML = "";
	};
		
	this.edit = function()
	{
		// clear the editor
		this.clear();

		// reset lock flag
		this.recordLocked = false;
		
		// open the editor
		this.getTearAway().expand();
		
		// reset modification history
		this.setModified(false);
				
		// reset the status
		this.prv.mDOM.status.innerHTML = "";
		var buttonChange = getElementByClassName(this.getDOM(),'cancelbtn');
		buttonChange.innerHTML = STR['button_cancel'];
		this.prv.mDOM.deleteBtn.disabled = false;

		//user defined function
		this.onAfterEditCall();
	};
	
	// user defined
	this.onAfterEditCall = function () { };
	
	// lock the editor
	this.lock = function()
	{
		//for (var i = 0; i < this.prv.mInputs.length; i++) {
		//	this.prv.mInputs[i].lock();
		//}

		this.recordLocked = true;
		this.prv.mDOM.status.innerHTML = STR['status_locked'];
		this.prv.mDOM.deleteBtn.disabled = true;
	};
	
	this.setModified = function(aModified)
	{
		if (aModified == this.prv.mModified)
			return;
		
		// set state
		this.prv.mModified = aModified;
		
		// enable or disable
		if (aModified) {
			// first, check if picker exists, check if it's a view, new record, and if it has no related record selected
			// if all three are true, save button shouldn't be enabled
			if ((this.topContents) && (this.record.tableId < 0) && (this.record.recordId.toString() == '-1') && (!this.parentRecordId)) {
				this.prv.mDOM.saveBtn.disabled = true;
			} else {
				this.prv.mDOM.saveBtn.disabled = false;
			}
		} else {
			this.prv.mDOM.saveBtn.disabled = true;
			// set modification to false
			for (var i = 0; i < this.prv.mInputs.length; i++) {
				this.prv.mInputs[i].setModified(false);
			}
		}
	};
	
	this.reload = function()
	{
		if (this.prv.mTimeoutId) {
			clearTimeout(this.prv.mTimeoutId);
		}
		if (!this.prv.mModified && this.record && this.record.recordId >= 0 && this.record.tableId >= 0) {
			this.pull(this.record.tableId, this.record.recordId);
		}

		var editor = this;
		var auto = editor.prv.mAutoReload;
		if (auto > 0) {
			this.prv.mTimeoutId = setTimeout(function() { editor.reload(); }, auto * 3000);
		}
	};
	
	// Frees resources
	this.destroy = function()
	{
		if (this.prv.mTimeoutId)
			clearTimeout(this.prv.mTimeoutId);

		if ((this.tbitem) && (this.tbitem.remove))
 			this.tbitem.remove();
			
		if (this.record){
			//unlock record function ( bridge.js)
			if ((this.isBackendLocked != 'True') && (!this.recordLocked)) {
				Controller.unlockRecord(this.record.tableId, this.record.recordId);
				if ((this.record.tableId) && (this.record.recordId) && (dax_bridge.sessionId)){
					var url = dax_bridge.url + "UnlockRecord?sessionId=" + dax_bridge.sessionId + "&tableId=" + this.record.tableId + "&recordId=" + this.record.recordId;	
					//unlock record on the back end
					makeCall(url,null,null,null);
				}
			}
		}
	};
	
	// constructor
	{
		// create the first column, input
		this.prv.mColumns.push(new EditorColumn(this));
		this.prv.mInputs.push(this.prv.mColumns[0].input(0));
		setStyle(this.prv.mDOM, "display", "block");
		
		// this bottom toolbar is for record saving
		var b = getTemplate("recordinputbottom", "dax");
		//b.id = "recordinputbottom"+getElementsByClassName(document.body, "recordinputbottom").length;
		generateUniqueId(b);
		
		
		// set the parent
		probe(b);
		for (property in b) {
			if (isDefined(b[property]) && b[property].nodeType == 1)
				b[property].parent = this;
		}

		b.deletebtn.innerHTML = STR['button_delete'];
		b.cancelbtn.innerHTML = STR['button_cancel'];
		b.savebtn.innerHTML   = STR['button_save'];
		
		this.ddwButton1 = b.ddwbtn1;
		this.ddwButton1.owner = this;
		this.ddwButton1.onclick = function () {
			var recordId = this.owner.record.recordId;
			var tableId = this.owner.record.tableId;
			// backend uses -3 as new record
			if (recordId == -1)
				recordId = -3;
			if (this.ddwType.indexOf ('Static') != -1)
				new DDW(this.url, 'static_url', this.title, '&recordid=' + recordId + '&tableid=' + tableId);
			else if (this.ddwType.indexOf("Dynamic") != -1)
				new DDW(this.url, 'dynamic_url', this.title, '&recordid=' + recordId + '&tableid=' + tableId);
			else if (this.ddwType.indexOf("Message") != -1)
				new DDW(this.url, 'message_editor', this.owner, '&recordid=' + recordId + '&tableid=' + tableId);
			};
		
		
		// define handlers
		var ref = this;
		b.cancelbtn.onclick = function(aEvent) {
			b.savebtn.disabled = true;
			// added on 10/05
			// call to back end to unlock the record when the cancel button is clicked
			// if record exists, crates the url to call the back end and unlock the record
			
			if ((ref.record) && ((ref.isBackendLocked != 'True'))  && (!ref.recordLocked))
				Controller.unlockRecord(ref.record.tableId, ref.record.recordId);
				
			//if ((ref.record.tableId) && (ref.record.recordId) && (dax_bridge.sessionId))
				//var url = dax_bridge.url + "UnlockRecord?sessionId=" + dax_bridge.sessionId + "&tableId=" + ref.record.tableId + "&recordId=" + ref.record.recordId;
			
			//unlock record function ( bridge.js)
			//makeCall(url,null,null,null);
			//editor.clear();
			
			//ref.getTearAway().contract(1, 50);
			
			ref.getDOM().style.display = 'none';			
			
			// free resources
			ref.destroy();
		};
		b.savebtn.onclick = function(aEvent) {
			ref.save();
		};
		
		b.deletebtn.onclick = function(aEvent) {
			ref.deleteR();
			
			//close the editor window after record deletion and grid refresh
			//ref.getTearAway().contract(1, 50);
		};		
//deleteBtn.onclick = function() { delete_records(this.prv.mDOM); };		
		// save for convenient referencing
		this.prv.mDOM.saveBtn 			= b.savebtn;
		this.prv.mDOM.cancelBtn 		= b.cancelbtn;
		this.prv.mDOM.deleteBtn 		= b.deletebtn;
		this.prv.mDOM.status			= b.status;
		this.prv.mDOM.bottom.buttons	= b;
		this.prv.mDOM.bottom.appendChild(b);
		
		// add the dom to the tearaway
		this.prv.mTearAway.setContent(this.prv.mDOM);
		this.prv.mTearAway.getDOM().owner = this;

		//hack for Safari
		if(browser.safari){
			var  hasStyle = getElementByClassName(this.prv.mDOM, 'recordinputbottom');
			if(hasStyle)
				hasStyle.style.display = "block";
		}		

	}
	
	this.saveImageUploadState = function() {
		if (!this.hasImages)
			return false;
		
		for (var inputCounter = 0; inputCounter < this.prv.mInputs.length; inputCounter++) {
			if (this.prv.mInputs[inputCounter].an_iframe) {
				if (!browser.ie)
					this.prv.mInputs[inputCounter].iFrameImageUrl = getElementByClassName(this.prv.mInputs[inputCounter].an_iframe.contentDocument.forms[0],'browse').value;
				else
					this.prv.mInputs[inputCounter].iFrameImageUrl = getElementByClassName(this.prv.mInputs[inputCounter].an_iframe.contentWindow.document.forms[0],'browse').value;
			}
		}		
		
	}
	
	this.restoreImageUploadState = function() {
		for (var inputCounter = 0; inputCounter < this.prv.mInputs.length; inputCounter++) {
			if (this.prv.mInputs[inputCounter].an_iframe) {
				this.prv.mInputs[inputCounter].set_iframe(this.prv.mInputs[inputCounter].an_iframe, this.prv.mInputs[inputCounter].prv.mFieldID);
			}
		}
	}
	
	return this;
}

/**
 * EditorColumn
 *
 * @author Jonathan Le
 */
function EditorColumn(aParent)
{
	// private
	this.prv = {
		mParent : aParent,
		mDOM : aParent.getColumnDOM(),
		mInputs : new Array()
	}; Function.decl.apply(this); probe(this.prv.mDOM);
	
	this.getInputDOM = function()
	{
		if (this.prv.mInputs.length <= 0) {
			return this.prv.mDOM.input;
		} else {
			return this.prv.mDOM.input.cloneNode(true);
		}
	};
	
	this.insertInput = function()
	{
		var input = new EditorInput(this);
		this.prv.mInputs.push(input);
		this.prv.mDOM.insertBefore(input.prv.mDOM, null);
		return input;
	};
	
	this.removeInput = function()
	{
		var ip = this.prv.mInputs.pop();
		this.prv.mDOM.removeChild(ip.prv.mDOM);
		delete ip;
	};
	
	// retrieve a column's input based on its index
	this.input = function(aPos)
	{
	};
	
	// scale the column
	this.size = function(aNumberOfInputs)
	{
		if (this.prv.mInputs.length == aNumberOfInputs)
			return;
			
		var inputCount = aNumberOfInputs - this.prv.mInputs.length;
		if (inputCount < 0) {
			for (var i = 0; i < Math.abs(inputCount); i++) {
				this.removeInput();
			}
		} else {
			for (var i = 0; i < inputCount; i++) {
				this.insertInput();
			}
		}
	};
		
	// constructor
	{
		// create the input
		this.prv.mInputs.push(new EditorInput(this));
		//this.prv.mDOM.deleteBtn.disabled = true;
	}
	
	return this;
}

/**
 * EditorInput
 *
 * @author Jonathan Le
 */
function EditorInput(aParent, aType)
{
	// private
	this.prv = {
		mParent : aParent,
		mDOM : aParent.getInputDOM(),
		mLabel : null,
		mType : -1,
		mValue : null,
		mModified: false
	}; Function.decl.apply(this);
	
	this.validate = function(aStr)
	{
		var aStr = isDefined(aStr) ? aStr : this.prv.mValue;
		
		var regex;
		switch (this.prv.mType)
		{
			case 0: // string
			break;
			case 1: // number
			{
				regex = /^\d*\.?\d*$/;
				if (!regex.test(aStr)) {
					return false;
				}
			}
			break;
			case 4: // date
			{	
				var validDate = validateDate(aStr);
				return validDate;
			}
			break;
			default:
			break;
		}

		return true;

	};
	
	// Set the input label.  acceptable types: strings, numbers, html elements
	this.setLabel = function(aLabel)
	{
		var type = typeof aLabel;
		switch (type)
		{
			case 'string':
			case 'number':
			{
				this.prv.mDOM.label.innerHTML = aLabel;
			}
			break;
			case 'object':
			{
				if (aLabel.nodeType == 1) {
					this.prv.mDOM.label.insertBefore(aLabel, null);
				}
			}
			break;
			default:
		};
	};
	
	this.setType = function(aType)
	{
		// remove the value if there is one
		if (this.prv.mDOM.value && this.prv.mDOM.value.nodeType == 1) {
			this.prv.mDOM.removeChild(this.prv.mDOM.value);
		}
		
		var node;
		var i_ref = this;
		
		// check if field has choicelist, if so, set appropriate type
 		if (!this.field)
			this.field = getField (this.prv.mParent.prv.mParent.record.tableId,this.prv.mFieldID);
		
		if (this.field.fieldchoicelist) {
			this.choicelist = this.field.fieldchoicelist;
			var aType = 6;
		} else {
			this.choicelist = null;
		}
		
		switch (aType)
		{
			case 6: // choice list
			{
				node = getTemplate("choicelistvalue").cloneNode(true);
				
				node.onchange =	
					function(skipCallback) {
						var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
						i_ref.prv.mValue = this.value;
						i_ref.setModified(true);
						
						var thisEditor = this.parentNode.parentNode.parentNode.parent;
						
						if (skipCallback != true) {
							for(i=0;i<thisEditor.prv.mInputs.length;i++) {
								if (thisEditor.prv.mInputs[i].prv.mDOM == this.parentNode) {
								
									for (j in thisEditor.record.map) {
										if (i == thisEditor.record.map[j])
											break;
									}
								
									var bridgefieldobj = getField(thisEditor.record.tableId,j);
									if (bridgefieldobj.fieldcallbackonchange != undefined)
										if (bridgefieldobj.fieldcallbackonchange) {
											var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
											if (frid=="-1")
												frid = "-3";
											ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.value, i, { value: this.value, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
										}
								}
							}	
						}
							//dom.status.innerHTML = '';
					};

				if (!browser.safari) {
					node.onmousedown = function() { DAX_Global_disableDragging = true; return false; };
					node.onmouseup = function() { return false; };
					node.onclick = function() { return false; };
					
					node.value.onmousedown = function() { DAX_Global_disableDragging = true; return false; };
					node.value.onmouseup = function() { return false; };
					node.value.onclick = function() { return false; };
				} else {
					node.onmousedown = function(aEvent) { xpEvent(aEvent, true); };
					node.value.onmousedown = function(aEvent) { xpEvent(aEvent, true); };
	
				}

				node.choicelistloaded = false;
				DAX_Bridge_GetChoiceList(this.choicelist, node);
				

			}
			break;		
			case 4: // date
			{
				// most of these are used for cross-browser debugging
				// leave them for now
				
				if (typeof(this.picker) == "undefined") {
					this.node = getTemplate("datevalue").cloneNode(true);
					this.picker = new PickerObject(30);
					this.node.appendChild(this.picker.dom);	
				
					this.node.input = getElementByClassName (this.node, 'dax_picker_value');
					this.node.table = getElementByClassName (this.node, 'dax_picker_table');
					
					this.node.popup = $('dax_picker_popup0');
					this.node.header = getElementByClassName (this.node.popup, 'dax_calendar_header');
					this.node.grid = this.picker.calendar.calGridObj.grid;

					if(isDefined(this.node.popup)){
					/*
						this.node.popup.onmouseover = 
							function() {
								i_ref.setModified(true);
							};		
					*/
						
					//	this.node.grid.setColumnWidth(30);
					//	this.node.grid.refresh();
			
						this.picker.calendar.drawCalendar();
						var flag = 0;
					}
				}				
				
				node = this.node;
				
				this.node.input.className = 'stringvalue dax_picker_value';
				
				this.node.input.onclick =
					function() { 
						if (!this.readOnly){
							this.className = "stringvalue_edit dax_picker_value"; 
							status = this.parent.parentNode.parentNode.parentNode.status.innerHTML = '';
							//this.parent.parentNode.parentNode.parentNode.saveBtn.disabled = false;
							this.style.border='1px solid black';
						}
					};
				this.node.input.onmouseover =
					function() {
						if (!this.readOnly)
							if (this.className != "stringvalue_edit dax_picker_value") {
									this.className = "stringvalue_over dax_picker_value";
								}
					};
				this.node.input.onmouseout = // depends on state
					function() {
						if (!this.readOnly)
							if (this.className != "stringvalue_edit dax_picker_value") {
								this.className = "stringvalue dax_picker_value";
							}
					};

				this.node.input.onkeypress = 
					function(e) {
						//i_ref.setModified(true);
						this.parent.parentNode.parentNode.parentNode.parent.clickFlag = true;
						var e = xpEvent(e, false);
						var unicode=e.charCode? e.charCode : e.keyCode
						status = this.parent.parentNode.parentNode.parentNode.status;		
						if (unicode!=8 && unicode!=9 && unicode!=44 && unicode!=46 && unicode != 47 && unicode!=45 && unicode!=37 && unicode!=39){ //if the key isn't the backspace key  or the tab key(which we should allow)
							if (unicode<48||unicode>57 && unicode!=47 && unicode!=45){ //if not a number
								this.style.border='1px solid red';
								status.innerHTML = STR['status_err_date']; //displays the error message close to the cancel buttom
								return false; //disable key press
							}
							else{
							if (!this.readOnly)
								i_ref.setModified(true);
								this.style.borderColor='#999999';
								status.innerHTML = '';
								return e;
							}
						}							
					};
				
				this.node.input.onchange =
					function(aPickerClicked) {
						//debugAlert ('onchange triggered');
						i_ref.setModified(true);
						
						if (aPickerClicked == true) {
						
							if(this.value.length){
								while(this.value.indexOf('\/') != -1){ // converts all the / to - the only two characters accept when typing the date value
									this.value = this.value.replace("/","-");
								}
									
								var originalValue = this.value;// keeps the original value before formated
								var valueLength = this.value.split('-');
								if(valueLength.length < 3){
									this.style.border='1px solid red';
									status.innerHTML = STR['status_err_date'];
									//abort save
									this.parent.parentNode.parentNode.parentNode.parent.abortSave = true;
									return false;
								}
								if(this.value.length < 10 ){
									this.value = fixValueSize(this.value);//make sure the date value is passed with the right length
									if(this.format){
										if(this.format == 'dDD-MMM-YYYY'){
											this.value = keepDetailDate(this.value);
										}
										else if(this.format == 'd1'){
											this.format = "dMM-DD-YYYY";
										}
										else{
											this.value = checkDateFormat(this.value, this.format.substr(1,this.format.length));						
										}
									}
									else{
										this.format = "dMM-DD-YYYY";
										this.value = checkDateFormat(this.value, this.format.substr(1,this.format.length));	
									}
								}
								else if(this.format){
									if(this.format == 'dDD-MMM-YYYY')
										this.value = keepDetailDate(this.value);
									else if(this.format == 'd1')
										this.format = 'dMM-DD-YYYY';
									else
										this.format = this.format;
								}
								else{
									this.format = "dMM-DD-YYYY";
								}
								this.value = checkDateFormat(this.value, this.format.substr(1,this.format.length));				
									
								this.parent.parentNode.parentNode.parentNode.parent.abortSave = false;
								//i_ref.setModified(true);
								i_ref.prv.mValue = this.value;
								i_ref.validate(this.value);
								i_ref.realDate = this.value;
		
								var newValue = this.value.split('-');
								status = this.parent.parentNode.parentNode.parentNode.status;
								if(this.value.length < 11){
									var checkDate = dateValidation(this.value, this.format.substr(1,this.format.length));// reverse-format the data
		
									if (newValue.length == 3 && checkDate == "True"){
										status.innerHTML = '';
										this.style.border='1px solid black';
										//this.parent.parentNode.parentNode.parentNode.saveBtn.disabled = false;
										this.originalValue = this.value;
										if(this.value && this.format){
											if(this.value.length > 0)
												this.value = format( this.format, this.value);
											else
												this.value = this.value;
											var thisEditor = this.parent.parentNode.parentNode.parentNode.parent;
											
											for(i=0;i<thisEditor.prv.mInputs.length;i++) {
												if (thisEditor.prv.mInputs[i].prv.mDOM == this.parent.parentNode) {
												
													for (j in thisEditor.record.map) {
														if (i == thisEditor.record.map[j])
															break;
													}
												
													var bridgefieldobj = getField(thisEditor.record.tableId,j);
													if (bridgefieldobj.fieldcallbackonchange != undefined)
														if (bridgefieldobj.fieldcallbackonchange) {
															var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
															if (frid=="-1")
																frid = "-3";
															ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.realDate, i, { value: this.realDate, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
														}
												}
											}	

											return;
										}
									}
									else{
										this.style.border='1px solid red';
										status.innerHTML = STR['status_err_date'];
										//abort save
										this.parent.parentNode.parentNode.parentNode.parent.abortSave = true;
										this.value = originalValue;
										return false;
									}
								}
								else if(this.value.length > 10 || this.value.length == 0)
									this.value = '00-00-00';
								else {						
									this.style.border='1px solid red';
									status.innerHTML = STR['status_err_date'];
									//abort save
									this.parent.parentNode.parentNode.parentNode.parent.abortSave = true;
									this.value = originalValue;
		
									return false;
								}
		
								var newValue = this.value.split('-');
								status = this.parent.parentNode.parentNode.parentNode.status;
				
								if(newValue.length == 3){
									this.value = checkDateFormat(this.value,this.format.substr(1,this.format.length)); // check the format passed and converts to 4d format
									var checkDate = dateValidation(this.value);// validates the data
									
									if(checkDate != 0){// if any problem
										this.style.border='1px solid red';
										status.innerHTML = STR['status_err_date'];
										//this.parent.parentNode.parentNode.parentNode.saveBtn.disabled = true;
										this.value = originalValue;
									}
									else{
										this.style.border='1px solid black';
										if(this.parent.calendar.value && this.format)
											this.value = format( this.format, this.parent.calendar.value);
									}
								}
							}
							else{
								this.value = '00-00-00';
								//i_ref.setModified(true);
								i_ref.prv.mValue = this.value;
								i_ref.validate(this.value);
								i_ref.realDate = this.value;
							}
							
						} else {

							// convert date to 4D format
							var processedDate = doFormat (this.value, this.format, '4daf_default');
							
							if (!processedDate) {
								this.style.border='1px solid red';
								status.innerHTML = STR['status_err_date'];
								//abort save
								this.parent.parentNode.parentNode.parentNode.parent.abortSave = true;
								return false;
							}
							
							i_ref.realDate = processedDate;
							i_ref.picker.value.originalValue = processedDate;
							
							this.value = doFormat (processedDate, '4daf_default', this.format, 'date');
							this.parent.parentNode.parentNode.parentNode.parent.abortSave = false;
							//i_ref.setModified(true);
							i_ref.prv.mValue = this.value;
							//i_ref.validate(this.value);
						
						}
						
						var thisEditor = this.parent.parentNode.parentNode.parentNode.parent;
						
						for(i=0;i<thisEditor.prv.mInputs.length;i++) {
							if (thisEditor.prv.mInputs[i].prv.mDOM == this.parent.parentNode) {
							
								for (j in thisEditor.record.map) {
									if (i == thisEditor.record.map[j])
										break;
								}
							
								var bridgefieldobj = getField(thisEditor.record.tableId,j);
								if (bridgefieldobj.fieldcallbackonchange != undefined)
									if (bridgefieldobj.fieldcallbackonchange) {
										var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
										if (frid=="-1")
											frid = "-3";
										ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.originalValue, i, { value: this.originalValue, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
									}
							}
						}	
							
						this.parent.parentNode.parentNode.parentNode.parent.clickFlag = false;
					};
				this.node.input.parent = node;
				this.node.input.owner = this;					
			}
			this.node.input.onblur =
				function() {
					//debugAlert ('onblur triggered');
					//i_ref.setModified(true);
					//this.onchange();
				};
			break;
			case 2: // image
			{
				node = getTemplate("imagevalue").cloneNode(true);
				probe(node);
				node.pic.link.onmousedown = function(aEvent) {xpEvent(aEvent, true);};
				
				this.prv.mParent.prv.mParent.hasImages = true;
				
				//node.browse.onmousedown = node.pic.link.onmousedown;
				//node.pic.src = node.browse.value;
				
				//node.browse.onchange =
				//	function() {  
				//		node.pic.link.image.src = 'file://'+node.browse.value;						
				//		i_ref.setModified(true); 
				//	};
			}
			break;
			case 3: // text
			{
				node = getTemplate("textvalue").cloneNode(true);
				
				node.onkeypress =	
					function() {
						var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
						i_ref.prv.mValue = this.value;
						i_ref.setModified(true);
						//dom.status.innerHTML = '';
					};				
					
					node.onchange =
						function() {

							var thisEditor = this.parentNode.parentNode.parentNode.parent;
							
							for(i=0;i<thisEditor.prv.mInputs.length;i++) {
								if (thisEditor.prv.mInputs[i].prv.mDOM == this.parentNode) {
								
									for (j in thisEditor.record.map) {
										if (i == thisEditor.record.map[j])
											break;
									}
								
									var bridgefieldobj = getField(thisEditor.record.tableId,j);
									if (bridgefieldobj.fieldcallbackonchange != undefined)
										if (bridgefieldobj.fieldcallbackonchange) {
											var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
											if (frid=="-1")
												frid = "-3";
											ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.value, i, { value: this.value, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
										}
								}
							}
						};
					// cancel propagation
					node.onmousedown = function(aEvent) {xpEvent(aEvent, true);};
					node.onmouseup = node.onmousedown;						
					}
	
			case 1: // number
			{
				if (!isDefined(node))
				{
					node = getTemplate("stringvalue").cloneNode(true);
					node.onblur = 
						function(){ 
							//debugAlert('Number type');
							this.style.borderColor='#999999';  // puts the border color to the original if the field was left with the non numeric character
							var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;		
							var status = this.parentNode.parentNode.parentNode.parent.prv.mDOM.status;
							this.className = 'stringvalue';
						}
								  
					node.onkeypress =
						function(e) {
							var e = xpEvent(e, false);
							var unicode=e.charCode? e.charCode : e.keyCode
//							alert(unicode);
							var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;		
							var status = this.parentNode.parentNode.parentNode.parent.prv.mDOM.status;
							//dom.saveBtn.disabled = false;
							i_ref.setModified(true);
							
							if (unicode!=8 && unicode!=9 && unicode!=44 && unicode!=46 && unicode!=45 && unicode!=37 && unicode!=39 && unicode!=99 && unicode!=118 && unicode!=120){ //if the key isn't the backspace key  or the tab key(which we should allow)
								if (unicode<48||unicode>57 && unicode!=44 && unicode!=46 && unicode!=45 && unicode!=99 && unicode!=118 && unicode!=120){ //if not a number
									this.style.border='1px solid red';
									dom.status.innerHTML = status.innerHTML = STR['status_err_number']; //displays the error message close to the cancel buttom
									return false; //disable key press
								}
								else{
								if (!this.readOnly)
									i_ref.setModified(true);
									this.style.borderColor='#999999';
									this.className = 'stringvalue';
									dom.status.innerHTML = '';
									return e;
								}
							}
						};
					node.onchange =
						function() {

							var thisEditor = this.parentNode.parentNode.parentNode.parent;
							
							for(i=0;i<thisEditor.prv.mInputs.length;i++) {
								if (thisEditor.prv.mInputs[i].prv.mDOM == this.parentNode) {
								
									for (j in thisEditor.record.map) {
										if (i == thisEditor.record.map[j])
											break;
									}
								
									var bridgefieldobj = getField(thisEditor.record.tableId,j);
									if (bridgefieldobj.fieldcallbackonchange != undefined)
										if (bridgefieldobj.fieldcallbackonchange) {
											var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
											if (frid=="-1")
												frid = "-3";
											ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.value, i, { value: this.value, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
										}
								}
							}
						};
					// cancel propagation
					node.onmousedown = function(aEvent) {xpEvent(aEvent, true);};
					node.onmouseup = node.onmousedown;						
				}
			}
			break;
			case 5: // boolean
			{
				if (!isDefined(node))
					node = getTemplate("booleancheckboxvalue").cloneNode(true);
				
				node.onclick =
					function()	{
						if (i_ref.isDisabled)
							return false;
						if (this.value.toLowerCase() == 'true') {
							getElementByClassName (this, 'fourdaf_checkbox', true).className = 'fourdaf_checkbox_uncheck';
							this.value = 'False';
						} else {
							getElementByClassName (this, 'fourdaf_checkbox', true).className = 'fourdaf_checkbox_check';
							this.value = 'True';
						}
						
						i_ref.setModified(true);
						
						var thisEditor = this.parentNode.parentNode.parentNode.parent;
						
						for(i=0;i<thisEditor.prv.mInputs.length;i++) {
							if (thisEditor.prv.mInputs[i].prv.mDOM == this.parentNode) {
							
								for (j in thisEditor.record.map) {
									if (i == thisEditor.record.map[j])
										break;
								}
							
								var bridgefieldobj = getField(thisEditor.record.tableId,j);
								if (bridgefieldobj.fieldcallbackonchange != undefined)
									if (bridgefieldobj.fieldcallbackonchange) {
										var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
										if (frid=="-1")
											frid = "-3";
										ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.value, i, { value: this.value, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format})
									}
							}
						}
					};

					
				// cancel propagation
				node.onmousedown = function(aEvent) {xpEvent(aEvent, true);};
				node.onmouseup = node.onmousedown;
			}
			break;
			case 0: // string
			default:
			{
				if (!isDefined(node))
					node = getTemplate("stringvalue").cloneNode(true);
				
				node.onclick =
					function() {
						if (!this.readOnly)
							this.className = "stringvalue_edit";
					};
				node.onmouseover =
					function() {
						if (!this.readOnly) {
							if (this.className != "stringvalue_edit")
								this.className = "stringvalue_over";
						}
					};
				node.onmouseout = // depends on state
					function() {
						if (!this.readOnly) {
							if (this.className != "stringvalue_edit")
								this.className = "stringvalue";
						}
					};
				if (!isDefined(node.onkeypress))
				{
					node.onkeypress =
					function(e) {
						var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
						i_ref.setModified(true);
						//dom.saveBtn.disabled = false;
						dom.status.innerHTML =  '';
						if(isDefined(this.format)){
							if(this.format.substr(0,1) == 't'){
								var e = xpEvent(e, false);
								var unicode=e.charCode? e.charCode : e.keyCode
										
								var status = this.parentNode.parentNode.parentNode.parent.prv.mDOM.status;
								
								if (unicode!=8 && unicode!=9 && unicode!=37 && unicode!=39){ //if the key isn't the backspace key  or the tab key(which we should allow)
									if (unicode<48||unicode>57 && unicode!=44 && unicode!=46 && unicode!=45 && unicode!=58){ //if not a number
										this.style.border='1px solid red';
										dom.status.innerHTML = status.innerHTML = STR['status_err_time']; //displays the error message close to the cancel buttom
										return false; //disable key press
									}
									else{
									if (!this.readOnly)
										i_ref.setModified(true);
										this.style.borderColor='#999999';
										this.className = 'stringvalue';
										dom.status.innerHTML = '';
										return e;
									}
								}
							}
						}
					};
				}
				node.onchange =	
					function() {
						var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
						i_ref.prv.mValue = this.value;
						i_ref.setModified(true);
						//dom.status.innerHTML = '';

						var thisEditor = this.parentNode.parentNode.parentNode.parent;
						
						for(i=0;i<thisEditor.prv.mInputs.length;i++) {
							if (thisEditor.prv.mInputs[i].prv.mDOM == this.parentNode) {
							
								for (j in thisEditor.record.map) {
									if (i == thisEditor.record.map[j])
										break;
								}
							
								var bridgefieldobj = getField(thisEditor.record.tableId,j);
								if (bridgefieldobj.fieldcallbackonchange != undefined)
									if (bridgefieldobj.fieldcallbackonchange) {
										var frid = thisEditor.prv.mInputs[i].prv.mFieldRecordID; 
										if (frid=="-1")
											frid = "-3";
										ExecuteCallBack (thisEditor, dax_event_ondatachange, thisEditor.record.tableId, j + "[" + frid + "]", this.value, i, { value: this.value, type: thisEditor.prv.mInputs[i].prv.mType , format: this.format});
									}
							}
						}	
					};
				node.onblur =	function() {
					var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
					i_ref.prv.mValue = this.value;
					i_ref.setModified(true);

					this.className = 'stringvalue';
					//i_ref.prv.mValue = this.value;
					if(this.format){
						if(this.format.substr(0,1) != 'x' && this.format.substr(0,1) != 'a'){// doesn't do formatting if the formatting is uppercase or lowercase
							if(this.format.substr(0,1) == 'i'){ //check the length of the string if is an integer fiels just 5 digists are alowed
								var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
								if(parseInt(this.value) > parseInt(32767) || parseInt(this.value) < '-32768'){
									this.style.border='1px solid red';
									dom.status.innerHTML = STR['editor_error_int'];// display error message
									this.parentNode.parentNode.parentNode.parent.abortSave = true;
								}
								else if(this.value.substr(0,1) == ',' || this.value.substr(0,1) == '.')
									this.value = "";
							}
						else if(this.format.substr(0,1) == 'l'){ //check the length of the string if is an integer fiels just 5 digists are alowed
							var dom = this.parentNode.parentNode.parentNode.parent.prv.mDOM;
							if(parseInt(this.value) > parseInt(2147483648) || parseInt(this.value) < '-2147483648'){
								this.style.border='1px solid red';
								dom.status.innerHTML = STR['editor_error_longint'];// display error message
								this.parentNode.parentNode.parentNode.parent.abortSave = true;
								return false;
							}
						}
						else if(this.format.substr(0,1) == 'r'){ //check the length of the string if is an integer fiels just 5 digists are alowed	
							if(this.value.substr(0,1) == ',')
								this.value = '0' + this.value;						}
						else
							this.value = this.value;
						}
						
						if( this.value.length){
							//clears the not valid characters if the formatting is not string or text
							if(this.format.substr(0,1) != 'a' && this.format.substr(0,1) != 'x'){
								var cleanValue = cleanUp(this.value, this.format);			
								debugAlert('clean value ' + cleanValue);}
							else
								var cleanValue = this.value;
							
							this.value = format( this.format, cleanValue );
						}						
						else
							this.value = this.value;
					}
					else
						this.value = this.value;

					
						
				};
				
					
				// cancel propagation
				node.onmousedown = function(aEvent) {xpEvent(aEvent, true);};
				node.onmouseup = node.onmousedown;
			}
			break;
		}
		
		node.id = node.className + this.prv.mParent.getInputs().length + this.prv.mParent.prv.mParent.getDOM().id;
		setStyle(node, "display", "block");
		this.prv.mDOM.insertBefore(node, null);
		this.prv.mDOM.value = node;
		this.prv.mType = aType;

	};
	
	this.set_iframe = function(new_iframe, fieldId) {
	
		if (!browser.ie) {
			var aDoc = new_iframe.contentDocument;
		}
		else {
			if (document.frames["upload_" + fieldId])
				var aDoc = document.frames["upload_" + fieldId].document;
		}			
		
		if (aDoc) {
			if (aDoc.forms)
				var aform = aDoc.forms[0];
		}
		
		if (aform) {
			if ((aform.id == "binaryupload") & (aform.sessionid.value == "")) {
				aDoc.i_ref = this;
				if ((this.an_iframe) && (this.iFrameImageUrl))
					getElementByClassName(aform, 'browse').setAttribute('value', this.iFrameImageUrl);

			}
			else {
				var thisthis = this;
				setTimeout(function() {thisthis.set_iframe(new_iframe, fieldId);}, 500);
			}					
		}
		else {
			var thisthis = this;
			setTimeout(function() {thisthis.set_iframe(new_iframe, fieldId);}, 500);
		}		
	};
		
	this.setValue = function(aValue, aType, aWidth, aHeight, aFormat, aDDW, aDDWType, aDDWTitle, aFieldType, aDontUpdateType, aDontRemoveDDWs)
	{
	
		if (aFieldType == 'boolean')
			aType = 5;
	
		//`debugAlert('Set Value a value = ' + aValue);
		if (!aDontUpdateType) {
			if (isDefined(aType)) {
				this.setType(aType);
			} else {
				var aType = this.prv.mType;
			}
		}
		if (aType < 0) { // no type defined
			var type = typeof aValue;
			switch (type)
			{
				case 'string':
				{
					// read the string to detect a 4D IMG tag
					var re = /^\/DAX\/GetImage\//;
					if (re.test(aValue)) {
						this.setType(2); // image
					} else {
						this.setType(0);
					}
				}
				break;
				case 'number':
				{
					this.setType(1);
				}
				break;
				case 'object':
				{
					if (aValue instanceof String) {
						var aValue = aValue.valueOf();
					} else {
						var aValue = "";
					}
				}
				break;
				default:
					var aValue = "";
				break;
			}
			
			// set the value, finally
			this.setValue(aValue);
		} else { // type already defined

			// check if choice list is present, if so set the value there
			if (this.choicelist) {
				// if value is in the list, select it
				var addPosition = this.prv.mDOM.value.length;
				var valueInList = false;
				for (var listCount = 0; listCount < this.prv.mDOM.value.length; listCount++) {
					/*
					if (this.prv.mDOM.value.options[listCount].value == '4daf_default') {
						addPosition = listCount;
					}
					*/
					if (this.prv.mDOM.value.options[listCount].value == aValue) {
						valueInList = true;
						this.prv.mDOM.value.selectedIndex = listCount;
						break;
					}
				}
				// if value is not in the list, add it
				// only if it is not a callback
				if ((!valueInList) && (!aDontUpdateType))
					this.prv.mDOM.value.options[addPosition] = new Option(aValue, aValue);
				aType = 6;
			}

			switch (aType)
			{
			
				case 6: // choice list
				{
					this.prv.mDOM.value.value = aValue;
					this.prv.mValue = aValue;
					this.prv.mDOM.value.format = aFormat;
					if (aFormat)
						this.prv.mDOM.format = aFormat;
				}
				break;
			
				case 5: // boolean
				{
					if (aValue.toLowerCase() == 'true') {
						getElementByClassName (this.prv.mDOM, 'fourdaf_checkbox', true).className = 'fourdaf_checkbox_check';
						this.prv.mDOM.value.value = 'True';
					} else {
						getElementByClassName (this.prv.mDOM, 'fourdaf_checkbox', true).className = 'fourdaf_checkbox_uncheck';
						this.prv.mDOM.value.value = 'False';
					}
				}
				break;
			
				case 4: // date
				{
					// take out T00:00:00 string
					if (aValue) {
						if(aValue.length > 0){
							if (aFormat)
								this.prv.mDOM.value.input.value = format(aFormat, aValue.substring(0,11));
							else
								this.prv.mDOM.value.input.value = aValue.substring(0,11);
							this.realDate = aValue.substring(0,11);
							this.prv.mDOM.value.input.realDate = aValue.substring(0,11);
						}
						else{
							this.realDate = '';
							this.prv.mDOM.value.input.value = '';
						}
						
						if (aFormat) {
							this.prv.mDOM.value.format = aFormat;
							this.prv.mDOM.value.input.format = aFormat;
						} else {
							// use default date format, which is first one in localization file
							this.prv.mDOM.value.format = 'd1';
							this.prv.mDOM.value.input.format = 'd1';
						}
					}
				}
				break;
				case 2: // image
				{
					this.prv.mDOM.value.pic.link.href = "#";
					var dom = this.prv.mDOM;
					this.prv.mDOM.value.pic.link.image.src = aValue;
					
					// adding the iframe for the picture upload
					// disable for DCS
					if (+this.prv.mParent.prv.mParent.record.tableId < 32000) { 
						var thisdoc = window.parent.document;
						var new_iframe = thisdoc.createElement('iframe');
						new_iframe.src = '/dax/js/templates/upload.html?random=' + Math.random();;
						new_iframe.height = "40px";
						new_iframe.scrolling = "no";
						new_iframe.id = "upload_" + this.prv.mFieldID;
						new_iframe.frameBorder = '0';
						this.prv.mDOM.value.appendChild(new_iframe);
						this.set_iframe(new_iframe, this.prv.mFieldID);
						this.an_iframe = new_iframe;
					}
					// setting the onclick event on the picture
					this.prv.mDOM.value.pic.link.onclick = function(aEvent) { 
						xpEvent(aEvent, true);
						whatever = getTemplate('image_preview_container');
						imageNode = whatever.cloneNode(true);
						var newWindow = WindowManager.createNewWindow();
						newWindow.titleBar.innerHTML = STR['editor_imageviewer'];
						if(aWidth < 300)
							aWidth = 300;
						if(aHeight < 200)
							aHeight = 200;							
						newWindow.size(aWidth, aHeight); // get the image size. when the window opens, the window will fit the image size
						newWindow.image = imageNode;
						newWindow.contentArea.appendChild(imageNode);							
						newWindow.activate();
						newWindow.open();
						
						newWindow.onsize = function () {	// get the new window dimensions and assign to the image dimensions
							var hImage = getElementByClassName (this.image, 'image_preview'); 
							hImage.widht = this.contentArea.offsetWidth;
							hImage.height = this.contentArea.offsetHeight;
						}
									
						// make unique label based on table name
						i = 1;
						while ($('image_preview_container' + i) != null) {
							i++;
						}
						
						imageNode.id = 'image_preview_container' + i;
						var ti = new taskbarItem(taskbar);
	
						// make unique label based on table name
						i = 1;
						while ($('dax_taskbar_button_image' + i) != null) {
							i++;
						}
						
						ti.dom.id = 'dax_taskbar_button_image' + i;
	
						ti.setLabel(STR['image'] + i);
						ti.setIcon("editor");
				
						// create a handler for clicking
						var ref = this
						ti.dom.winRef = newWindow;
						ti.dom.onclick = function() { 
							this.parent.setSelected (true, true);
							this.winRef.activate();
						};
						
						setStyle (ti.dom, 'display', 'block');
						ti.setSelected (true, true);
						
						ti.parent.refresh();
						newWindow.tbitem = ti;
					
						var hImage = getElementByClassName (imageNode, 'image_preview');
						hImage.src = aValue;
						//newWindow.hImage = hImage;
						newWindow.onclose = function() {
							if (this.tbitem)
								this.tbitem.remove();
							
							// get rid of the grid
							if (this.contentArea.grid)
								this.contentArea.grid.destroy();
							
							if (this.editor)
								this.editor.destroy();
						}	
					}
					return false;
				}
				break;
				case 3: // text
					this.prv.mDOM.value.value = aValue;
					if (aFormat)
						this.prv.mDOM.value.format = aFormat;
				case 0: // string, number
				case 1:
					this.prv.mDOM.value.value = aValue;
					if (aFormat)
						this.prv.mDOM.value.format = aFormat;
				break;
				default:
				{
					this.prv.mDOM.value.value = aValue;
					this.prv.mValue = aValue;
					this.prv.mDOM.value.format = aFormat;
					if (aFormat)
						this.prv.mDOM.format = aFormat;
					
					if (!isDefined(aValue) || aValue == "")
						this.prv.mDOM.value.className = "stringvalue_edit";
					else
						this.prv.mDOM.value.className = "stringvalue";
				}
				break;
			};
		// type is known, assign the DDW if it exists
			if ((aDDW) && (aDDWType)) {
				if (!aDDWTitle)
					aDDWTitle = 'DDW';
				switch (aType)
				{
					case 4: // date
						// check if ddw button is already there
						//var calNode = getElementByClassName (this.prv.mDOM, 'dax_picker_button');
						var calParentNode = getElementByClassName (this.prv.mDOM, 'dax_picker_aux');
						if (this.prv.ddwButton) {
							calParentNode.removeChild (this.prv.ddwButton);
							this.prv.ddwButton = null;
						}
						// create a ddw button
						var ddwButton = getTemplate('editorddwinline');
						var ddwNode = ddwButton.cloneNode (true);
						// get unique id
						v = 1;
						while ($('editorddwinline' + v) != null) {
							v++;
						}
						// insert button
						calParentNode.appendChild (ddwNode);
						// assign attributes
						ddwNode.id = 'editorddwinline' + v;
						ddwNode.innerHTML = aDDWTitle;	
						this.prv.ddwButton = ddwNode;
						ddwNode.parent = this.prv.mDOM;
						ddwNode.owner = this;
						ddwNode.url = aDDW;
						ddwNode.title = aDDWTitle;
						ddwNode.style.display = 'inline';

						// resize the input field
						var newInputSize = this.prv.mDOM.value.offsetWidth - ddwNode.offsetWidth - 10 + 'px';
						this.prv.mDOM.value.style.width = newInputSize;

						// assign values to the ddw button
						ddwNode.tableId = this.prv.mParent.prv.mParent.record.tableId;
						ddwNode.recordId = this.prv.mParent.prv.mParent.record.recordId;

					break;
					case 2: // image
						/*
						// check if ddw button is already ther
						if (this.prv.ddwButton) {
							this.prv.ddwButton.parent.removeChild (this.prv.ddwButton)
						}
						// create a ddw button
						var ddwButton = getTemplate('editorddwinline');
						var ddwNode = ddwButton.cloneNode (true);
						// get unique id
						v = 1;
						while ($('editorddwinline' + v) != null) {
							v++;
						}
						// insert button
						this.prv.mDOM.appendChild (ddwNode);
						// assign attributes
						ddwNode.id = 'editorddwinline' + v;
						ddwNode.innerHTML = aDDWTitle;	
						this.prv.ddwButton = ddwNode;
						ddwNode.parent = this.prv.mDOM;
						ddwNode.owner = this;
						ddwNode.url = aDDW;
						ddwNode.title = aDDWTitle;
						// resize the input field
						var newInputSize = this.prv.mDOM.value.offsetWidth - ddwNode.offsetWidth - 10 + 'px';
						this.prv.mDOM.value.style.width = newInputSize;

						// assign values to the ddw button
						ddwNode.tableId = this.prv.mParent.prv.mParent.record.tableId;
						ddwNode.recordId = this.prv.mParent.prv.mParent.record.recordId;
						*/
	
					break;
					case 3: // text
						// check if ddw button is already there
						if (this.prv.ddwButton) {
							this.prv.ddwButton.parent.removeChild (this.prv.ddwButton);
							this.prv.ddwButton = null;
						}
						// create a ddw button
						var ddwButton = getTemplate('editorddwinline');
						var ddwNode = ddwButton.cloneNode (true);
						// get unique id
						v = 1;
						while ($('editorddwinline' + v) != null) {
							v++;
						}
						// insert button
						this.prv.mDOM.appendChild (ddwNode);
						// assign attributes
						ddwNode.id = 'editorddwinline' + v;
						ddwNode.innerHTML = aDDWTitle;	
						this.prv.ddwButton = ddwNode;
						ddwNode.parent = this.prv.mDOM;
						ddwNode.owner = this;
						ddwNode.url = aDDW;
						ddwNode.title = aDDWTitle;
						ddwNode.style.display = 'inline';
						// resize the input field
						var newInputSize = this.prv.mDOM.value.offsetWidth - ddwNode.offsetWidth - 10 + 'px';
						this.prv.mDOM.value.style.width = newInputSize;

						// assign values to the ddw button
						ddwNode.tableId = this.prv.mParent.prv.mParent.record.tableId;
						ddwNode.recordId = this.prv.mParent.prv.mParent.record.recordId;

					break;
					case 6: // 
					case 0: // string, number
					case 1:
						// check if ddw button is already there
						if (this.prv.ddwButton) {
							this.prv.ddwButton.parent.removeChild (this.prv.ddwButton);
							this.prv.ddwButton = null;
						}
						// create a ddw button
						var ddwButton = getTemplate('editorddwinline');
						var ddwNode = ddwButton.cloneNode (true);
						// get unique id
						v = 1;
						while ($('editorddwinline' + v) != null) {
							v++;
						}
						// insert button
						this.prv.mDOM.appendChild (ddwNode);
						// assign attributes
						ddwNode.id = 'editorddwinline' + v;
						ddwNode.innerHTML = aDDWTitle;	
						this.prv.ddwButton = ddwNode;
						ddwNode.parent = this.prv.mDOM;
						ddwNode.owner = this;
						ddwNode.url = aDDW;
						ddwNode.title = aDDWTitle;
						ddwNode.style.display = 'inline';
						// resize the input field
						var newInputSize = this.prv.mDOM.value.offsetWidth - ddwNode.offsetWidth - 10 + 'px';
						this.prv.mDOM.value.style.width = newInputSize;

						// assign values to the ddw button
						ddwNode.tableId = this.prv.mParent.prv.mParent.record.tableId;
						ddwNode.recordId = this.prv.mParent.prv.mParent.record.recordId;
					break;
					default:

					break;
				};	
				
				if (ddwNode) {
					// backend uses -3 as new record
					if (ddwNode.recordId == -1)
						ddwNode.recordId = -3;
					if (aDDWType.indexOf ('Static') != -1) {
						ddwNode.onclick = function () {
							var recordId = this.recordId;
							recordId = ReplaceString(recordId, '[' + this.tableId + ']', this.owner.field.fieldid);
							var buttonDDW = new DDW(this.url, 'static_url', this.title, '&recordid=' + recordId + '&tableid=' + this.tableId+ '&data=' + this.owner.getValue());
							setTimeout(function() { WindowManager.active = null; if (buttonDDW.window) buttonDDW.window.activate(); }, 100);
						}					
					} else if (aDDWType.indexOf("Dynamic") != -1) {
						ddwNode.onclick = function () {
							var recordId = this.recordId;
							recordId = ReplaceString(recordId, '[' + this.tableId + ']', this.owner.field.fieldid);
							var buttonDDW = new DDW(this.url, 'dynamic_url', this.title, '&recordid=' + recordId + '&tableid=' + this.tableId + '&data=' + this.owner.getValue());
							setTimeout(function() { WindowManager.active = null; if (buttonDDW.window) buttonDDW.window.activate(); }, 100);
						}
					} else if (aDDWType.indexOf("Message") != -1) {
						ddwNode.onclick = function () {
							var recordId = this.recordId;
							recordId = ReplaceString(recordId, '[' + this.tableId + ']', this.owner.field.fieldid);
							var buttonDDW = new DDW(this.url, 'message_editor', this.owner.prv.mParent.prv.mParent, '&recordid=' + recordId + '&tableid=' + this.tableId + '&data=' + this.owner.getValue());
							setTimeout(function() { WindowManager.active = null; if (buttonDDW.window) buttonDDW.window.activate(); }, 100);
						}
					}
				}
			} else {
			// remove ddw if they exists

				if (!aDontRemoveDDWs) {
				
					switch (aType)
					{
						case 6: // choice list
							// check if ddw button is already there
							if (this.prv.ddwButton) {
								this.prv.ddwButton.parent.removeChild (this.prv.ddwButton);
								this.prv.ddwButton = null;
							}
						break;
					
						case 4: // date
							// check if ddw button is already there
							//var calNode = getElementByClassName (this.prv.mDOM, 'dax_picker_button');
							var calParentNode = getElementByClassName (this.prv.mDOM, 'dax_picker_aux');
							if (this.prv.ddwButton) {
								calParentNode.removeChild (this.prv.ddwButton);
								this.prv.ddwButton = null;
							}

						break;
						case 2: // image

						break;
						case 3: // text
							// check if ddw button is already there
							if (this.prv.ddwButton) {
								this.prv.ddwButton.parent.removeChild (this.prv.ddwButton);
								this.prv.ddwButton = null;
							}

						break;
						case 0: // string, number
						case 1:
							// check if ddw button is already there
							if (this.prv.ddwButton) {
								this.prv.ddwButton.parent.removeChild (this.prv.ddwButton);
								this.prv.ddwButton = null;
							}
						break;
						default:

						break;
					};	
				}
			}
		}
	};
	
	this.getValue = function()
	{
		if (this.choicelist)
			if (this.prv.mType != 4)
				return this.prv.mDOM.value.options[this.prv.mDOM.value.selectedIndex].value;
			else {
				// convert date to 4D format
			return doFormat(this.prv.mDOM.value.options[this.prv.mDOM.value.selectedIndex].value, this.field.fieldformat, '4daf_default', 'date');		
			}
			
		switch (this.prv.mType)
		{
			case 5: // boolean
			case 0: // string
			case 1: // number
			case 3: // text
				return this.prv.mDOM.value.value;
			case 4: // date
				return (this.realDate);
		}
	};
	
	this.getOriginalValue = function()
	{
		if (this.choicelist)
			return doFormat(this.prv.mDOM.value.options[this.prv.mDOM.value.selectedIndex].value, this.field.fieldformat, '4daf_default', 'date');		
			
		if(isDefined(this.picker)) {
			if(isDefined(this.picker.value.originalValue))
				return this.picker.value.originalValue;
			else if (this.realDate)
				return this.realDate;
			else
				return this.prv.mValue;
		} else {
			return this.realDate;
		}
	};	
	
	this.setModified = function(aModified)
	{
		if (aModified == this.prv.mModified)
			return;
		
		this.prv.mModified = aModified;
		
		if (aModified) {
			var editor = this.prv.mParent.getParent();
			editor.setModified(aModified);
		}
	}
	
	this.lock = function()
	{
		switch (this.prv.mType)
		{
			// strings & numbers
			case 3: // text
			case 0: // string
			case 1: // number
			case 4: //date
			default:
			{
				this.prv.mDOM.value.readOnly = true;
			}
			break;
		}
	};
	
	this.clear = function()
	{
		switch (this.prv.mType)
		{
			case 3:
				this.setValue(null);
			case 0:
			case 1:
			{
				this.setValue("");
			}
			break;
			case 2:
			{
				this.prv.mDOM.value.pic.link.href = "";
				this.prv.mDOM.value.pic.link.image.src = "";
			}
			break;
			case 4:
			{
				this.realDate = '';
				this.prv.mDOM.value.input.value = '';			
			}
			default:
			break;
		}
	};
	
	// constructor
	{
		var aType = isDefined(aType) ? aType : this.prv.mType;
		probe(this.prv.mDOM);
	}
	
	return this;
}

function setIframe() {

	if (this.document.i_ref!=undefined) {
//		if (navigator.userAgent.indexOf("WebKit") < 0)
		if (!browser.safari2)
			this.document.i_ref.prv.mDOM.value.pic.link.image.src = "";
		else
			this.document.i_ref.prv.mDOM.value.pic.link.image.src = "file://" + this.document.forms[0].DAX_BinaryFile.value;
		
		this.document.i_ref.setModified(true);
	}

}

// ExecuteCallBack

function ExecuteCallBack (InputObj, eventid, tableid, fieldid, value, InputID, Assign) {

	var sessionId = connection.sessionId;
 
 	if (sessionId == "")
 		return;
 		
 	var http_request = getXMLHttpRequest();
		
	if (http_request) {
		var url;
		if (value != "")
			url = connection.address.executecallback+"?sessionId="+sessionId+"&eventid="+eventid+"&tableid="+tableid+"&fieldid="+fieldid+"&value="+dax_bridge.encode(value);
		else
			url = connection.address.executecallback+"?sessionId="+sessionId+"&eventid="+eventid+"&tableid="+tableid+"&fieldid="+fieldid;
			
		http_request.onreadystatechange = 
			function() {
				OnReceiveCallBackResult(http_request, InputObj, InputID, Assign);
			};
			
		try {
		
			debugAlert('<b>Callback:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
		
			http_request.open("GET", url, true);
	  	  	http_request.send(null);
	   	} catch(e) {
			if (browser.ie6)
				http_request.onreadystatechange = function(){};
			else
				http_request.onreadystatechange = null;
	   		delete http_request;
		}
	}
	
	return true;
}

function OnReceiveCallBackResult(http_request, inputObj, id, Assign)
{
	if (http_request.readyState != 4)
		return;
		
	// check if lists are populated
	var listsLoaded = true;
	for (var inputCount = 0; inputCount < inputObj.prv.mInputs.length; inputCount++) {
		if (inputObj.prv.mInputs[inputCount].choicelist) {
			debugAlert (inputObj.prv.mInputs[inputCount].choicelist + ' loaded state:' + inputObj.prv.mInputs[inputCount].prv.mDOM.value.choicelistloaded);
			if (!inputObj.prv.mInputs[inputCount].prv.mDOM.value.choicelistloaded) {
				listsLoaded = false;
			}
		}
	}
	
	debugAlert ('Lists loaded: ' + listsLoaded)
	if (!listsLoaded) {
		inputObj.callbackTimer = window.setTimeout(function() {OnReceiveCallBackResult(http_request, inputObj, id, Assign); }, 500);
		return false;
	}
		
	if (checkResponse(http_request)) {
		// return this.onerror(http_request); // won't work
		return false;
	}

	// parse through the XML
	var result = http_request.responseXML.getElementsByTagName("Callback").item(0);

	if (!result)
		return false;
	
	var CallBackElems= result.getElementsByTagName("Result");
	
	var CallBackExecStatus = result.getAttribute("executionstatus");
	
	// get other callback values
	var callBackElements = result.getElementsByTagName("fieldvalue");
	var callBackLists = result.getElementsByTagName("list");
	var useLegacySystem = true;	

	// process other field values
	for (var elementCount = 0; elementCount < callBackElements.length; elementCount++) {
		useLegacySystem = false;
		var fieldId = callBackElements[elementCount].getAttribute('fieldid');
		var value = callBackElements[elementCount].getAttribute('value');
		if (inputObj.prv.mInputs[inputObj.record.map[fieldId]].field.fieldtype != 'date')
			inputObj.prv.mInputs[inputObj.record.map[fieldId]].setValue(value, null, null, null, null, null, null, null, null, null, null, true);
		else {
			inputObj.prv.mInputs[inputObj.record.map[fieldId]].setValue(doFormat(value, 'dYYYY-MM-DD', inputObj.prv.mInputs[inputObj.record.map[fieldId]].fieldformat), null, null, null, null, null, null, null, null, null, null, true);
			inputObj.prv.mInputs[inputObj.record.map[fieldId]].realDate = value;
		}
		inputObj.prv.mInputs[inputObj.record.map[fieldId]].setModified(true);
	}

	// process lists
	for (var listCount = 0; listCount < callBackLists.length; listCount++) {
		useLegacySystem = false;
		var listName = callBackLists[listCount].getAttribute('choicelist');
		var listValues = [ ];
		var listElements = callBackLists[listCount].getElementsByTagName('listitem');
		
		// find list
		for (var inputCounter = 0; inputCounter < inputObj.prv.mInputs.length; inputCounter++) {
			if ((inputObj.prv.mInputs[inputCounter].choicelist == listName) && (listElements.length > 0)) {
				var listToPopulate = inputObj.prv.mInputs[inputCounter].prv.mDOM.value;
				// get original value
				if (listToPopulate.options.length > 0) {
					var originalValue = listToPopulate.options[listToPopulate.selectedIndex].value;				
				} else {
					var originalValue = null;				
				}
				listToPopulate.options.length = 0;
				var selectedIndex = 0;
				for (listValueCount = 0; listValueCount < listElements.length; listValueCount++) {
					var value = (listElements[listValueCount].firstChild) ? (listElements[listValueCount].firstChild.nodeValue) : "";
					if (originalValue == value)
						selectedIndex = listValueCount;
					listToPopulate.options[listValueCount] = new Option(value, value);
				}
				listToPopulate.selectedIndex = selectedIndex;
				inputObj.prv.mInputs[inputCounter].setModified(true);
			}	
		}
	}

	// reset warning system
	inputObj.prv.mDOM.status.innerHTML = '';
	for (var inputCounter = 0; inputCounter < inputObj.prv.mInputs.length; inputCounter++){
		if (inputObj.prv.mInputs[inputCounter].prv.mDOM.value.className.indexOf('stringvalue_alert') != -1)
			inputObj.prv.mInputs[inputCounter].prv.mDOM.value.className = inputObj.prv.mInputs[inputCounter].prv.mDOM.value.oldClassName;
	}	

	if (useLegacySystem) {
		for (var i = 0; i < CallBackElems.length; i++) {
			var CallBackElem = CallBackElems.item(i);
			
			if (CallBackElem) {
				var newvalue = CallBackElem.getAttribute("value");

				inputObj.prv.mInputs[id].setValue(newvalue, Assign.type, Assign.picWidth, Assign.picHeight, Assign.format, null, null, null, null, true, true);
				
				var message = CallBackElem.getAttribute("message");
				
				if (message!="")
					inputObj.prv.mDOM.status.innerHTML = message;
					
				if (CallBackExecStatus != '1') {
					if (inputObj.prv.mInputs[id].prv.mType != 5)
						inputObj.prv.mInputs[id].prv.mDOM.value.focus();
					inputObj.prv.mInputs[id].prv.mDOM.value.oldClassName = inputObj.prv.mInputs[id].prv.mDOM.value.className;
					inputObj.prv.mInputs[id].prv.mDOM.value.className += ' stringvalue_alert';
				}
			}
		}
	}

	
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;

}