// custom editor
 
var rudiDebug = false; 
 
function CustomEditor()
{
	var node = getTemplate("customeditor"); if (!node) return;

	this.content = node.cloneNode(true);
	this.tearaway = new TearAway();
	this.dom = this.tearaway.getDOM();

	// create properties out of HTML hierarchy
	this.content.parent = this;
	this.content.id = "customeditor"+getElementsByClassName(document.body, "customeditor").length;

	// compability commands
	this.getDOM = function () {return this.dom; };
	//this.getAutoReload = function () {return 0; };
	//this.pull = function () {return null; };
	
	// visual options
	this.formMargin = 10;
	
	this.redraw = function () {

		var ta = this.tearaway;
		ta.size(this.content.offsetWidth + 20, this.content.offsetHeight + 29);

		var taDom = this.dom;
		
		// ie7 vertical spacing bug fix
		taDom.middle.ml.style.height = taDom.middle.m.offsetHeight + 'px';
		taDom.middle.mr.style.height = taDom.middle.m.offsetHeight + 'px';
		taDom.style.height = taDom.top.offsetHeight + taDom.middle.offsetHeight + taDom.bottom.offsetHeight + 'px';	
	
	}

	// builds editor fields
	this.construct = function () {
	
		makeCall ('/osxdbcustomform.xml?formname=[People]WidgetForm', this.construct_handler, this, null);
	
	}
	
	// gets editor fields from backend and constructs the editor contents
	this.construct_handler = function (http_response, aEditor) {
	
		if (http_response.readyState !=4)
			return;	

		// set appropriate editor size
		var formSize = http_response.responseXML.getElementsByTagName('formsize').item(0);
		
		var formHeight = formSize.getAttribute("height");
		var formWidth = formSize.getAttribute("width");

		aEditor.content.style.width = +formWidth + (+aEditor.formMargin * 2) + 'px';
		aEditor.content.style.height = +formHeight + (+aEditor.formMargin * 2) + 'px';
		aEditor.redraw();

		// select form elements
		var formElementRoot = http_response.responseXML.getElementsByTagName('page').item(1);
		var formElements = formElementRoot.childNodes;

		// acquire nodes
		var textTemplateNode = getTemplate('customeditor_text');
		var inputTemplateNode = getTemplate('customeditor_input');
		var imageTemplateNode = getTemplate('customeditor_image');
		var radioTemplateNode = getTemplate('customeditor_radio');
		var checkTemplateNode = getTemplate('customeditor_check');
		
		// loop through elements and draw them
		for (fel = 0;fel < formElements.length; fel++) {
			var currentElement = formElements.item(fel);
		
			switch (currentElement.nodeName) {

				case 'text':
				
					// add text node to editor
					var textNode = textTemplateNode.cloneNode(true);
					aEditor.content.appendChild(textNode);
					
					// set coordinates
					var coordNode = currentElement.getElementsByTagName('coord').item(0);
					textNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
					textNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
					
					// set text properties
					var formatNode = currentElement.getElementsByTagName('format').item(0);
					textNode.style.fontFamily = formatNode.getAttribute('font');
					textNode.style.fontSize = formatNode.getAttribute('size') + 'px';
					textNode.style.fontWeight = formatNode.getAttribute('weight');
					textNode.style.fontStyle = formatNode.getAttribute('style');
					textNode.style.color = formatNode.getAttribute('color');
					
					// set text
					textNode.innerHTML = currentElement.lastChild.nodeValue;
					
					// make node visible
					textNode.style.display = 'block';
				
				break;

				// node name is variable, displayed as input field, could be field, variable, or picture
				case 'variable':
				
					var variableType = currentElement.getAttribute('type');
				
					// branch off based on variable type
							
					switch (variableType) {
				
						case 'Field':
						case 'Variable':
				
							// add text node to editor
							var inputNode = inputTemplateNode.cloneNode(true);
							aEditor.content.appendChild(inputNode);
							
							// set coordinates
							var coordNode = currentElement.getElementsByTagName('coord').item(0);
							inputNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
							inputNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
							inputNode.style.width = (+coordNode.getAttribute('width')) + 'px';
							inputNode.style.height = (+coordNode.getAttribute('height')) + 'px';				

							// set text
							inputNode.value = currentElement.lastChild.nodeValue;
							
							// make node visible
							inputNode.style.display = 'block';
						
						break;
						
						case 'PictureLibrary':

							// add text node to editor
							var imageNode = imageTemplateNode.cloneNode(true);
							aEditor.content.appendChild(imageNode);

							// set coordinates
							var coordNode = currentElement.getElementsByTagName('coord').item(0);
							imageNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
							imageNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
							imageNode.style.width = (+coordNode.getAttribute('width')) + 'px';
							imageNode.style.height = (+coordNode.getAttribute('height')) + 'px';				

							// set text
							imageNode.src = '/osxdbcustomform.xml?librarypicture=' + currentElement.lastChild.nodeValue + 
							'&formname=[People]WidgetForm' +
							'&key=' + 1;
							
							// make node visible
							imageNode.style.display = 'block';
							
						break;
				
					}
				
				break;

				case 'radiobutton':
				
					// add text node to editor
					var radioNode = radioTemplateNode.cloneNode(true);
					aEditor.content.appendChild(radioNode);
					
					// actual radio element
					var radioRadioNode = getElementByClassName (radioNode, 'customeditor_radio_radio');
					
					// text element
					var radioTextNode = getElementByClassName (radioNode, 'customeditor_radio_text');
					
					// set coordinates
					var coordNode = currentElement.getElementsByTagName('coord').item(0);
					radioNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
					radioNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
					
					// set text
					radioTextNode.innerHTML = currentElement.lastChild.nodeValue;
					
					// make node visible
					radioNode.style.display = 'block';
				
				break;

				case 'checkbox':
				
					// add text node to editor
					var checkNode = checkTemplateNode.cloneNode(true);
					aEditor.content.appendChild(checkNode);
					
					// actual radio element
					var checkCheckNode = getElementByClassName (checkNode, 'customeditor_check_check');
					
					// text element
					var checkTextNode = getElementByClassName (checkNode, 'customeditor_check_text');
					
					// set coordinates
					var coordNode = currentElement.getElementsByTagName('coord').item(0);
					checkNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
					checkNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
					
					// set text
					checkTextNode.innerHTML = currentElement.lastChild.nodeValue;
					
					// make node visible
					checkNode.style.display = 'block';
				
				break;

				case 'checkbox':
				
					// add text node to editor
					var checkNode = checkTemplateNode.cloneNode(true);
					aEditor.content.appendChild(checkNode);
					
					// actual radio element
					var checkCheckNode = getElementByClassName (checkNode, 'customeditor_check_check');
					
					// text element
					var checkTextNode = getElementByClassName (checkNode, 'customeditor_check_text');
					
					// set coordinates
					var coordNode = currentElement.getElementsByTagName('coord').item(0);
					checkNode.style.top = (+coordNode.getAttribute('top')) + (+aEditor.formMargin) + 'px';
					checkNode.style.left = (+coordNode.getAttribute('left')) + (+aEditor.formMargin) + 'px';				
					
					// set text
					checkTextNode.innerHTML = currentElement.lastChild.nodeValue;
					
					// make node visible
					checkNode.style.display = 'block';
				
				break;

				
			}		
		}
	}
	
	// constructor
	{
		setStyle(this.content, "display", "block");
		
		// add the dom to the tearaway
		this.tearaway.setContent(this.content);
		this.tearaway.getDOM().owner = this;
	}
	
	return this;
}
