/**
 * A grid object is composed of a viewport with a grid inside.
 *
 * @author Jonathan Le
 */

function GridObject_toString()
{
	return "[object GridObject]";
};

function GridObject_setParent(aParentElement) 
{
	this.viewport.parent = aParentElement;
	this.viewport.refresh();
};

function GridObject_clear()
{
	this.grid.clear();
};

function GridObject_setFocusAll(aBoolean)
{
	this.grid.setFocusAll(aBoolean);
};

function GridObject_populate(aPopulateObj)
{

	this.owner.search.saveButton.disable();
	this.owner.populating = true;
	
	if (this.owner) {
		// if this is initial loding, check if allrecords, static, or dynamic tab should be opened
		if (this.owner.initializing) {
			// initializing done
			this.owner.initializing = false;
	
			if (this.owner.table.showallrecordstab.toLowerCase() == 'true') {
				// show all records
				// ie, do nothing at this point
			} else if (this.owner.table.preselectQueries.length > 0) {
				// static preselected queries, no all records tab
				if (this.owner.tabsbar.tabs[0])
					this.owner.tabsbar.tabs[0].dom.onclick();
				return true;
			} else if (this.owner.table.preselectedDynamicQueries.length > 0) {
				// call for tabs right now, we need them for data
				this.owner.initializing = false;
				this.owner.refreshDynamicTabs();
				return true;
			} else {
			
			}
		}
	}
		
	var vp = this.viewport;
	try {
		var comparison = isDefined(aPopulateObj.comparison) ? aPopulateObj.comparison :
			null;
		var start = isDefined(aPopulateObj.startIndex) && aPopulateObj.startIndex > 0 ?
			aPopulateObj.startIndex : 0;
		var length = isDefined(aPopulateObj.length) && aPopulateObj.length > 0 ?
			aPopulateObj.length : Math.ceil(vp.dom.offsetHeight / this.prv.mRowHeight);
		var queryName = isDefined(aPopulateObj.queryName) ? aPopulateObj.queryName :
			null;		
		var keepOldQueryId = isDefined(aPopulateObj.keepOldQueryId) ? aPopulateObj.keepOldQueryId : false;
		
		// we will load as many records as it the view fits
		var sessionId = connection.sessionId;

		// if we need to store a query id
		var queryid = isDefined(aPopulateObj.queryId) ? aPopulateObj.queryId : null;
		if (queryid) {			
			var url = connection.address.query+"?sessionId="+sessionId+"&queryid="+
				queryid+"&start="+start+"&length="+length;
		} else if (queryName) {
			var url = connection.address.query+"?sessionId="+sessionId+"&tableid="+
				aPopulateObj.tableId+"&start="+start+"&length="+length+"&queryname="+dax_bridge.encode(queryName);
		} else if (isDefined(comparison)) {
			var url = connection.address.query+"?sessionId="+sessionId+"&tableid="+
				aPopulateObj.tableId+"&start="+start+"&length="+length;
		} else {
			var url = connection.address.allrecords+"?sessionId="+sessionId+
				"&tableid="+aPopulateObj.tableId+"&start="+start+"&length="+length
		}
		
		if (!aPopulateObj.rowRef) {
			aPopulateObj.rowRef = new Object();
			aPopulateObj.rowRef.eraseAfter = true;
		} else {
			aPopulateObj.rowRef.eraseAfter = false;
		}
			
		aPopulateObj.rowRef.keepOldQueryId = keepOldQueryId;
		
		if (isDefined(comparison) && comparison != "" )
			url += comparison;
		//assert('testA',url);
		var http_request = getXMLHttpRequest();
		
		if (http_request) {
			if (!isDefined(aPopulateObj.onReadyStateFunc)) {
				var obj = this; // explicitly reference 'this'
				aPopulateObj.onReadyStateFunc =
					function() { obj.onpopulate(http_request, aPopulateObj.rowRef);};
			}
			
			debugAlert('<b>Grid:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
			
			http_request.onreadystatechange = aPopulateObj.onReadyStateFunc;
			http_request.open("GET", url, true);
	  	  	http_request.send(null);
	  	  	
	  	  	this.onbeforeload();
		}
	} catch (e) { }
};

function GridObject_reload()
{
	if (!this.selection)
		return;

	if (this.prv.mTimeoutId) {
		clearTimeout(this.prv.mTimeoutId);
	}
	
	var grid = this;
	
	var pop = {
//		queryName : grid.owner.currentTab.queryName ? grid.owner.currentTab.queryName : null,
//		queryId : grid.owner.currentTab.queryName ? null : grid.selection.queryId,
		queryId : grid.selection.queryId,
		tableId : grid.selection.tableId,
		startIndex : grid.selection.range.start,
		//length : grid.selection.range.length,
		comparison: grid.selection.comparison,
		keepOldQueryId : true
	};
	
	// populate the grid
	//grid.clear();
	grid.populate(pop);


	var auto = grid.prv.mAutoReload;
	if (auto > 0) {
		this.prv.mTimeoutId = setTimeout(function() { grid.reload(); }, auto * 3000);
	}
	
};

function GridObject_null() { };

function GridObject_onpopulate(http_request, aRowRef)
{
	if (http_request.readyState != 4)
		return;

	if (checkResponse(http_request))
		return;	
		
	// to reference this object
	var ref = this;
	
	var tablename = http_request.responseXML.getElementsByTagName("queryResult").item(0).getAttribute(STR['tablename']);
	
	if (callback_onViewQuery)
		http_response = callback_onViewQuery(tablename, http_request, 'grid');		
	
	// parse through the XML
	var result = http_request.responseXML.getElementsByTagName("queryResult").item(0);

	if (!isDefined(result)) {
		return this.onerror(http_request);
	}
	
	// get sorting value
	var sortingFieldId = result.getAttribute('sortfield');
	
	// get passed data
	var keepOldQueryId = aRowRef.keepOldQueryId;
	
	if (aRowRef.eraseAfter) {
		aRowRef = null;
	}

	if (!isDefined(this.firstLoad) && !this.firstLoad) {

		this.selection = {
			tableId : result.getAttribute(STR['tableid']),
			queryId : result.getAttribute(STR['queryid']),
			tableName : result.getAttribute(STR['tablename']),
			map : new Array(),
			formatMap : new Array(),
			fieldTypeMap : new Array(),
			range : {start : 0, length : result.getAttribute('size')},
			toString : function() { return this.tableId + " " + this.queryId + " " +
				this.tableName + " " + this.total; }
		};

		var s_table = structure.getTable(this.selection.tableId);

		if (!s_table.cached)
			buildFieldList(this.selection.tableId);
		
		// override for hiding breaklevel fields
		if (!this.hGridFields) {
		
			var s_fields = [ ];
			for (var fieldCounter = 0; fieldCounter < s_table.fields.length; fieldCounter++) {
				var fieldRef = getField(s_table.tableid, s_table.fields[fieldCounter].fieldid);
				if (fieldRef.fieldinlist.toLowerCase() == 'true'){
					s_fields.push(fieldRef);
				}
			}		
		
			gFields = s_fields;
		} else {
			
			var s_fields = [ ];
			for (var fieldCounter = 0; fieldCounter < this.hGridFields.length; fieldCounter++) {
				var fieldRef = getField(s_table.tableid, this.hGridFields[fieldCounter].fieldid);
				if (fieldRef.fieldinlist.toLowerCase() == 'true'){
					s_fields.push(fieldRef);
				}
			}				

			gFields = s_fields;
		}
		
		if (gFields.length > 0 && gFields.length < this.grid.prv.mColCount) {
			this.grid.setColumnWidth(
				Math.floor(this.viewport.dom.offsetWidth / gFields.length));
		}
		
		// resize the grid according to field count
		if (gFields.length > 0)
			this.grid.size(gFields.length); 
		
		// create a mapping and set Headers
		for (var i = 0; i < gFields.length; i++) {
			this.selection.map[gFields[i].fieldid] = i;
			this.grid.column(i).setHeader(gFields[i].fieldalias); //fieldname
			var colWidth = getField(this.selection.tableId, gFields[i].fieldid).fieldcolumnwidth;
			if (colWidth)
				this.grid.column(i).setWidth(colWidth); // column width
			this.grid.column(i).fieldId = gFields[i].fieldid; // reverse lookup
			// formatting map
			this.selection.formatMap[gFields[i].fieldid] = gFields[i].fieldformat;
			this.selection.fieldTypeMap[gFields[i].fieldid] = gFields[i].fieldtype;
		}
		
		this.grid.refresh();
		
	}

	if (this.recreateSelection) {
	
		this.selection.range.start = 0;
		this.selection.range.length = result.getAttribute('size');
		this.selection.queryId = result.getAttribute(STR['queryid']);
		
		/*

		tableId : result.getAttribute(STR['tableid']),
			queryId : result.getAttribute(STR['queryid']),
			tableName : result.getAttribute(STR['tablename']),
			map : new Array(),
			formatMap : new Array(),
			range : {start : 0, length : result.getAttribute('size')},
			toString : function() { return this.tableId + " " + this.queryId + " " +
				this.tableName + " " + this.total; }
		};	
		
		*/
		
		this.recreateSelection = false;
	
	}
	
	if (!keepOldQueryId) {
		this.currentQueryId = result.getAttribute(STR['queryid']);
		this.selection.queryId = this.currentQueryId;
	}
	
	this.receivedQueryId = result.getAttribute(STR['queryid']);
	
	
	// if named query is used, make sure that query id is placed
	/*
	if (this.owner.currentTab) {
		if ((this.owner.currentTab.queryName) && (this.owner.currentTab.queryId)){
			this.owner.currentTab.queryId = this.currentQueryId;		
		}
	}
	*/
	
	// store selection total
	this.selection.total = parseInt(result.getAttribute('recordsinselection'));
	
	this.beforeOnPopulate();
	
	var resrows = result.getElementsByTagName(STR['rowtag']);
	
	for (var i = 0; i < resrows.length; i++) {
		var resrow = resrows.item(i);
		var fields = resrow.getElementsByTagName(STR['fieldtag']);
		var row = (i == 0 && isDefined(aRowRef)) ? aRowRef : this.grid.row(i);
		
		if (row) {
			// if the record was previously selected
			row.recordId = resrow.getAttribute(STR['recordid']);
			if (isDefined(this.selection.selected) && isDefined(this.selection.selected[row.recordId]))
				row.setFocus(this.selection.selected[row.recordId]);
			else
				row.setFocus(false);
							
			for (var j = 0; j < fields.length; j++) {
				var id = fields[j].getAttribute(STR['fieldid']);
				var fWidth = fields[j].getAttribute('width');
				var fHeight = fields[j].getAttribute('height');
				var fddw = fields[j].getAttribute('ddw');
				var ddw_type = fields[j].getAttribute('ddwtype');
				var index = this.selection.map[id];
				var recordID = fields[j].getAttribute('fieldrecordid');
				
				var element = row.element(index);
				
				//gets the image dimensions from the back end
				if(isDefined(fWidth) && isDefined(fHeight)){
					element.picWidth = fWidth;
					element.picHeight = fHeight;	
				}	
		
				// formatting					
				if (isDefined(element)) {
					elValue = n2String(fields[j]);
					// check if there is a ddw
					
					if(fddw) {
						element.ddw = fddw;
						// creates the link for the ddw
						if (ddw_type.indexOf ('Static') != -1)
							elValue = "<a class='ddw_link' href='#' onclick='new DDW(\""+fddw+"\", \"static_url\", this.firstChild.nodeValue, \"&recordid="+id+"["+recordID+"]"+"&tableid="+this.selection.tableId+"\"); return false;'>"+elValue+"</a>";
						else if (ddw_type.indexOf ('Dynamic') != -1)
							elValue = "<a class='ddw_link' href='#' onclick='new DDW(\""+fddw+"\", \"dynamic_url\", this.firstChild.nodeValue, \"&recordid="+id+"["+recordID+"]"+"&tableid="+this.selection.tableId+"\"); return false;'>"+elValue+"</a>";								
						else if (ddw_type.indexOf ('Message') != -1)
							elValue = "<a class='ddw_link' href='#' onclick='new DDW(\""+fddw+"\", \"message_window\", this.parentNode.parent.prv.mParent.prv.mParent.owner.owner, \"&recordid="+id+"["+recordID+"]"+"&tableid="+this.selection.tableId+"\"); return false;'>"+elValue+"</a>";								
						}
					else {	
						var elFieldType = this.selection.fieldTypeMap[id];
						var elFormat = this.selection.formatMap[id];
						if ((elFieldType == 'boolean') || (elFieldType == 'date')) {
							elValue = doFormat(elValue, '4daf_default', elFormat, elFieldType, 'grid_cell'); //format 
						} else {
							if (elFormat) {
								elValue = format(elFormat, elValue); //old format
							}
						}
					}
					element.setContent(elValue);
				}
			}
		}
	}

	// clear blank rows
	var totalRows =  this.grid.prv.mRowCount;
	if (resrows.length < totalRows) {
		for (var i = resrows.length; i <= totalRows; i++) {
			var row = this.grid.row(i);
			if (row) {
				row.clear();
				row.setFocus(false);
			}
		}
	}
	
	if (!isDefined(this.firstLoad) && !this.firstLoad) {
		this.firstLoad = true;
		if (this.owner.target) {
			this.nowPopulating = false;
			this.reload();
			return;
		}
	}
	
	// free
	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;
	
	this.onafterload();

	var colCount = this.grid.prv.mColCount;

	// hide & show columns
	if (this.showColumnsArray.length > 0) {
		for (var arrCount = 0; arrCount < this.showColumnsArray.length; arrCount++) {
			if (!isNaN(this.showColumnsArray[arrCount]))
				if (this.showColumnsArray[arrCount] < colCount)
					this.grid.column(this.showColumnsArray[arrCount]).show();
		}
		this.showColumnsArray = new Array();
	}
	
	if (this.hideColumnsArray.length > 0) {
		for (var arrCount = 0; arrCount < this.hideColumnsArray.length; arrCount++) {
			if (!isNaN(this.hideColumnsArray[arrCount]))
				if (this.hideColumnsArray[arrCount] < colCount)
					this.grid.column(this.hideColumnsArray[arrCount]).hide();
		}
		this.hideColumnsArray = new Array();
	}
	
	this.nowPopulating = false;
	
	var aWindowRef = this.owner;

	// highlight sorted field 
	for (var colCount = 0; colCount < this.grid.prv.mColumns.length; colCount++) {
		if (this.grid.prv.mColumns[colCount].fieldId == sortingFieldId) {
			this.grid.prv.mColumns[colCount].prv.mHeader.prv.mDOM.className = 'cellAsc';
			//this.grid.prv.mColumns[colCount].prv.mHeader.prv.mOrderAsc = true;
			this.grid.prv.mColumns[colCount].prv.mHeader.prv.mSorted = true;
		} else {
			this.grid.prv.mColumns[colCount].prv.mHeader.prv.mDOM.className = 'cell';
			//this.grid.prv.mColumns[colCount].prv.mHeader.prv.mOrderAsc = false;
			this.grid.prv.mColumns[colCount].prv.mHeader.prv.mSorted = false;		
		}
	}
	
	if (!this.startedDynamicTabUpdate) {
		setTimeout (function() {aWindowRef.refreshDynamicTabs(); }, 2500);
		this.startedDynamicTabUpdate = true;
	}
	
	this.owner.search.saveButton.enable();
	this.owner.populating = false;
	
	// IE7 needs few seconds after sorting
	if (this.sortLock) {
		if (browser.ie){
			var sortGrid = this;
			this.sortLockTimer = setTimeout (function() {sortGrid.sortLock = false;}, 2000);
		} else {
			this.sortLock = false;
		}
	}

};

function GridObject_onheaderfocus(aHeaderRef)
{
	if (this.owner.sortLock)
		return false;

	if (!this.currentQueryId)	
		return false;
		
	if (this.owner.populating)
		return false;
	else
		this.owner.populating = true;

	
		
	var column = aHeaderRef.prv.mColumn;
	var grid = aHeaderRef.prv.mParent;
	
	if (this.selection) { // remote sort
		aHeaderRef.prv.mOrderAsc = !aHeaderRef.prv.mOrderAsc; // set order
		this.sort(column.fieldId, aHeaderRef.prv.mOrderAsc);
	} else { // local sort
		// Not implemented, blur the header
		aHeaderRef.prv.mDOM.onblur();
	}
};

function GridObject_goToPage(aPageNumber)
{
	// dealing with a selection
	if (this.selection) {
		var riv = this.selection.range.length;
		var start = (riv * (aPageNumber-1)) + 1;
		
		var pop = {
			queryId : this.selection.queryId,
			tableId : this.selection.tableId,
			startIndex : start,
			//length : this.selection.range.length,
			comparison: this.selection.comparison
		};

		// populate the grid
		this.clear();
		this.populate(pop);
		
		this.prv.mCurrentPage = aPageNumber;
		
		// move the slider's position
		var sh = this.viewport.vscrollbar.slider.top.clientHeight +
			this.viewport.vscrollbar.slider.mid.clientHeight + 
			this.viewport.vscrollbar.slider.bottom.clientHeight;
		var pos = Math.floor(((this.viewport.vscrollbar.slideArea.clientHeight - sh) /
			this.prv.mPageCount) * (aPageNumber-1)+1);
		setStyle(this.viewport.vscrollbar.slider, "top", pos + "px");
	}
};

function GridObject_previousPage()
{
	if (this.prv.mCurrentPage > 1)
		this.goToPage(--this.prv.mCurrentPage);
};

function GridObject_nextPage()
{
	if (this.prv.mCurrentPage < this.prv.mPageCount)
		this.goToPage(++this.prv.mCurrentPage);
};

function GridObject_onsort(http_request)
{
	if (http_request.readyState != 4)
		return;
		
	var result = http_request.responseXML.getElementsByTagName("querySort").item(0);
	checkResponse(http_request);

	if (!isDefined(result)) {
		return this.onerror(http_request)
	}
	
	var queryId = result.getAttribute(STR['queryid']);
	
	this.grid.clear();
	
	var pop = {
		queryId : queryId,
		tableId : this.selection.tableId,
//		length : this.selection.range.length,
		length : this.grid.prv.mRowCount,
		startIndex : this.selection.range.start,
		comparison : this.selection.comparison
	};
	
	this.populate(pop);

	if (browser.ie6)
		http_request.onreadystatechange = function(){};
	else
		http_request.onreadystatechange = null;
	delete http_request;

};

function GridObject_sort(aFieldId, aIsAscending)
{
	this.sortLock = true;	

	var direction = aIsAscending ? "asc" : "desc";
	var url = connection.command(DB.SORT) + "&tableid="+this.selection.tableId+
		"&fieldid="+aFieldId+"&direction="+direction+"&queryid="+
		this.currentQueryId;
		//	this.selection.queryId;

	var http_request = getXMLHttpRequest();
	if (http_request) {
		var this_ref = this;
		
		debugAlert('<b>Grid:</b>'+'<a href="'+url+'" target="_blank">'+url+'</a>');
		
		http_request.onreadystatechange = function() {
			this_ref.onsort(http_request);
		};
		http_request.open("GET", url, true);
  	  	http_request.send(null);
  	  				
  	  	this.onbeforeload();
	}
	
};

function GridObject_insertRecord(aSelectionId, aRowRef)
{
	if (!isDefined(this.selection))
		return;
	
	var obj = this;
	this.populate({queryId : this.selection.queryId, startIndex: aSelectionId,
		tableId : this.selection.tableId, length : 1, rowRef : aRowRef});
};

function GridObject_viewport_onvscrollstart(aTop)
{
};

/*

function GridObject_viewport_onvscroll(aTop, aIsScrollDown)
{
	var go = this.owner;
	if (go.selection) {
		var content = go.grid.body;
		var unit = go.prv.mRowHeight / this.vscrollbar.ratio; // units per record
		var riv = this.dom.offsetHeight / go.prv.mRowHeight;
		var n = Math.round(aTop / unit);
		
		if ((aTop % Math.round(unit)) == 0) {
			var a = 0;
			var b = 0;
			var index = 0;
			var top = 0;
			
			if (aIsScrollDown) {
				index = n+riv;
				b = go.grid.prv.mRowCount;
			} else {
				if (n > 0)
					top = (go.prv.mRowHeight * -1);
				index = n;
				a = go.grid.prv.mRowCount-1;
			}

			// here's where the trick is.  we swap rows, then reposition
			// this gives the illusion that there are an infinite number of rows
			content.style.top = top + "px";
			
			var row = go.grid.row(a);
			if (n > 0) {
				row.clear();
				go.grid.moveRow(a, b);
			
				if (!aIsScrollDown || (n+riv) < (go.selection.total+1))
					go.insertRecord(index, row);
			}
		} else if (unit < 1) {
			//go.grid.clear();
		} else {
			content.style.top = ((aTop % unit) * (-1) * this.vscrollbar.ratio)+ "px";
		}
	}
 };

*/

function GridObject_viewport_onvscroll(aTop, aIsScrollDown, aGo, bThis)
{

	// first clear the grid
	if (aGo) {
		var go = aGo;
	} else {
		var go = this.owner; // the grid object
		var bThis = this;
	}

	if (go.nowPopulatingTimer) {
		window.clearTimeout(go.nowPopulatingTimer);
		go.nowPopulatingTimer = null;
	}
	
	//debugAlert ('go.nowPopulating:' + go.nowPopulating);
	
	if (go.nowPopulating) {
//		if (aGo) {
//			debugAlert ('populating - refiring timer');
			go.nowPopulatingTimer = window.setTimeout(function() {GridObject_viewport_onvscroll(aTop, aIsScrollDown, go, bThis);}, 500);
//		}
		return;
	} else
		go.nowPopulating = true;
	
	if (go.selection) {
		//go.grid.clear();
		
		//assert ('testA', aTop);
		var unit = go.prv.mRowHeight / bThis.vscrollbar.ratio;
		
		var riv = Math.ceil(bThis.dom.offsetHeight / go.prv.mRowHeight);
		var n = Math.round(aTop / unit);

		// update the range
		go.selection.range.start = n;
		go.selection.range.length = riv;
		
		// populate those records
		var pop = {
			comparison : go.selection.comparison,
			queryId : go.selection.queryId,
			tableId : go.selection.tableId,
			startIndex : n
		};

		//length : (((n+riv) < go.selection.total || (go.selection.total - n + 1) > riv) ?
			//	riv : go.selection.total - n + 1)

		
		go.populate(pop);
		
		if (!aGo)
			go.nowPopulatingTimer = window.setTimeout(function() {GridObject_viewport_onvscroll(aTop, aIsScrollDown, go, bThis);}, 500);
		
		/*
		
		// calculate pages
		var curPage = 1;
		var pageCount = go.prv.mPageCount;
		
		// var pageHeight = Math.floor(this.vscrollbar.slideArea.offsetHeight / pageCount);
		var pageHeight = Math.floor((this.vscrollbar.slideArea.offsetHeight - this.vscrollbar.slider.offsetHeight)  / pageCount)
		if (aTop>0)
			for (curPage ; curPage < pageCount; curPage++) {
				if (aTop >= ((pageHeight * (curPage-1))+1) && aTop <= (pageHeight * curPage))
					break;
			}
		// go.setCurrentPage(curPage);
		
		*/
	}
};

 
function GridObject_viewport_onvscrollstop(aTop)
{
		// first clear the grid
	var go = this.owner; // the grid object
	if (go.selection) {
		//go.grid.clear();
		
		//assert ('testA', aTop);
		var unit = go.prv.mRowHeight / this.vscrollbar.ratio;
		
		var riv = Math.ceil(this.dom.offsetHeight / go.prv.mRowHeight);
		var n = Math.round(aTop / unit);

		// update the range
		go.selection.range.start = n;
		go.selection.range.length = riv;
		
		// populate those records
		var pop = {
			comparison : go.selection.comparison,
			queryId : go.selection.queryId,
			tableId : go.selection.tableId,
			startIndex : n
		};

		//length : (((n+riv) < go.selection.total || (go.selection.total - n + 1) > riv) ?
			//	riv : go.selection.total - n + 1)

		
		go.populate(pop);
		
		/*
		
		// calculate pages
		var curPage = 1;
		var pageCount = go.prv.mPageCount;
		
		// var pageHeight = Math.floor(this.vscrollbar.slideArea.offsetHeight / pageCount);
		var pageHeight = Math.floor((this.vscrollbar.slideArea.offsetHeight - this.vscrollbar.slider.offsetHeight)  / pageCount)
		if (aTop>0)
			for (curPage ; curPage < pageCount; curPage++) {
				if (aTop >= ((pageHeight * (curPage-1))+1) && aTop <= (pageHeight * curPage))
					break;
			}
		go.setCurrentPage(curPage);
		
		*/
	}
};

function GridObject_load(aArray)
{
	// resize the grid
	this.grid.size(aArray[0].length, aArray.length);
	
	for (var i = 0; i < aArray.length; i++) {
		this.grid.setRow(i, aArray[i]);
	}
};

function GridObject_refresh()
{
	var grid = this.grid;
	var vp = this.viewport;
	
	var rows = Math.ceil(xpGetSize(vp.parent).height / this.prv.mRowHeight);
	var cols = Math.ceil(xpGetSize(vp.parent).width / this.prv.mColWidth);
	
	var overflow = null;
	
	// we have a selection and we've loaded it already
	if (this.selection && this.firstLoad) {
		cols = null;

		// recalculate overflow
		overflow = ((this.selection.total+1) * (this.prv.mRowHeight)) -
			vp.dom.offsetHeight + this.grid.header.offsetHeight;
	}
	
	// size the grid
	if ((cols && grid.prv.mColCount < cols) || grid.prv.mRowCount < rows)
		grid.size(cols, rows);
	
	// resize columns, if necessary
	var lastCol = grid.column(grid.prv.mColCount - 1);
	var hOverflow = Math.floor(grid.getWidth() - grid.prv.mPadding - xpGetSize(vp.parent).width);

	// increases overflow space if grid vertical scrollbar is present
	if (overflow > 0)
		hOverflow += vp.vscrollbar.dom.offsetWidth;
	
	if (hOverflow <= 0) {
		// resize the column
		var diff = xpGetSize(vp.dom).width - (grid.getWidth() - grid.prv.mPadding);
		lastCol.setWidth(lastCol.getWidth() + diff);
		grid.refresh();
		
		hOverflow = 0;
		//grid.setColumnWidth(Math.floor((xpGetSize(vp.dom).width) / grid.prv.mColCount));
		//grid.refresh();
	} else {
		// HACK
		if ((lastCol.getWidth() > lastCol.prv.mMinimumWidth) && (hOverflow < lastCol.getWidth() - lastCol.prv.mMinimumWidth)) {
			
			// ACI0053001
			vp.hscrollbar.scroll (0, true);
			
			// resize the column
			lastCol.setWidth(lastCol.getWidth() - hOverflow);
			grid.refresh();
			
			hOverflow = 0;
		}
	}
	// redraw
	vp.setOverflow(overflow, hOverflow);
	vp.refresh();
	
	// set vscrollbar position based on selection
	if ((this.selection) && (!vp.vscrollbar.isScrolling)) {
		var selStart = (this.selection.range.start == 1) ? 0 : this.selection.range.start; 
		vp.vscrollbar.scroll(((selStart) * (this.prv.mRowHeight) / vp.vscrollbar.ratio), true);
	}
	
	this.onAfterRefresh();
	
};

function GridObject_destroy()
{
	// clear the query
	
	// destroy the grid
	if (this.prv.mTimeoutId)
		clearTimeout(this.prv.mTimeoutId);
	
	this.grid.destroy();
	this.viewport.destroy();
		
	//delete this;
};

function GridObject_onkeypress(aEvent)
{
	var event = xpEvent(aEvent);
	var keynum = event.which ? event.which : event.keyCode;
};

function GridObject_grid_onrowselect(aRow)
{
	var go = this.owner;
	if (go && go.selection) {
		if (!isDefined(go.selection.selected))
			go.selection.selected = new Array();
		
		go.selection.selected[aRow.recordId] = aRow.prv.mFocus;
		//Controller.setFocused(go);
	}
};

function GridObject_onerror(aErrorObj)
{
	return false;
};

// GridObject_

function GridObject(aParentElement)
{
	this.grid = new NGrid();
	this.viewport = new ViewPort();
//	alert('New index = ' + ++WindowManager.maxZIndex);
	// these objects are bound to 'this'
	this.grid.owner = this;
	this.viewport.owner = this;
	
	// needs a double viewport refresh first time to display properly
	// this keeps track if it is a first launch
	this.firstLaunch = true;

	// is on populate function waiting for reply?
	this.nowPopulating = false;
	this.sortLock = false;
	
	this.prv = {
		mRowHeight		: 0,
		mColWidth		: 0,
		mCurrentPage	: 1,
		mPageCount		: 0,
		mAutoReload		: 60, // value in seconds
		mTimeoutId		: null
	}; 

	// is this object part of HGrid?
	this.hGridFields = null;
	
	// override toString
	this.toString = GridObject_toString;
	
	// control the parent
	this.setParent = GridObject_setParent;
	
	this.clear = GridObject_clear;

	// just reload whatever selection is currently in view
	this.reload = GridObject_reload;

	// function to populate the grid with a table's data
	this.populate = GridObject_populate;

	// custom defined calls
	this.beforeOnPopulate = GridObject_null;
	
	this.onpopulate = GridObject_onpopulate;

	// override.  header is focused
	this.onheaderfocus = GridObject_onheaderfocus;
	
	// this happens before & after a populate is received & processed
	this.onbeforeload = GridObject_null;
	this.onafterload = GridObject_null;
	
	// Page manipulation
	this.goToPage = GridObject_goToPage;
	
	this.previousPage = GridObject_previousPage;
	
	this.nextPage = GridObject_nextPage;

	// Sorting on the backend
	this.sort = GridObject_sort;
	
	// Handler for sorting remotely
	this.onsort = GridObject_onsort;

	// This method is meant to be privately used for now
	// insert a record into a specific row
	this.insertRecord = GridObject_insertRecord;

	// JL
	// Overwrite these scrollbar methods
	this.viewport.onvscrollstart = GridObject_viewport_onvscrollstart;
	
	this.viewport.onvscroll = GridObject_viewport_onvscroll;
	
	this.viewport.onvscrollstop = GridObject_viewport_onvscrollstop;
	
	// user-defined
	//this.viewport.onhscroll = function(aLeft) {};

	// an array of arrays
	this.load = GridObject_load;
	
	// Refresh the viewport, and resize the grid if necessary
	this.refresh = GridObject_refresh;
	
	// user customizable
	this.onAfterRefresh = GridObject_null;
	
	// Unload the grid
	this.destroy =GridObject_destroy;
	
	this.onkeypress = GridObject_onkeypress;

	// override the on select
	this.grid.onrowselect = GridObject_grid_onrowselect;
	
	// handle any XmlHttpRequest errors here
	this.onerror = GridObject_onerror;
	
	this.setFocusAll = GridObject_setFocusAll;
	
	this.recreateSelection = false;
	
	this.startedDynamicTabUpdate = false;
	
	this.showColumnsArray = new Array();
	this.hideColumnsArray = new Array();
	
	// initializer
	{
		var vp = this.viewport;
		vp.setContent(this.grid.dom);
		
		// insert the viewport into the container
		aParentElement.insertBefore(vp.dom, null);
		this.setParent(aParentElement);
		
		try {
			// set the dimensions of cells in the grid
			var grid = this.grid;
	
			// calculate how many rows and cols fit into the viewable area
			grid.size(1, 1);
			
			this.prv.mRowHeight = grid.row(0).getHeight();
			this.prv.mColWidth = grid.column(0).getWidth();
			
			this.refresh();
		} catch(e) {}
	}
	
	return this;
}

function NGrid_getHeaderFrame()
{
	var cell = getElementByClassName(this.header, "cell");
	if (this.prv.mColumns.length <= 0) {
		return cell;
	} else {
		return cell.cloneNode(true);
	}
};

function NGrid_getRowDOM()
{
	var row = getElementByClassName(this.body, "row");
	if (this.prv.mRowCount <= 0) {
		return row;
	} else {
		var newrow = row.cloneNode(false);
		newrow.appendChild(getElementByClassName(row, "cell").cloneNode(true));
		return newrow;
	}
};

function NGrid_insertRow(aPos)
{
	var row = new NGridRow(this);
	var next = null;
	
	if (isDefined(aPos) && aPos >= 0 && aPos < this.prv.mRowCount) {
		next = this.prv.mRows[aPos].prv.mDOM;
		this.prv.mRows.splice(aPos, 0, row);
		
		// reset indices
		row.setIndex(aPos);
		for (var i = aPos + 1; i < this.prv.mRows.length; i++) {
			this.prv.mRows[i].setIndex[i];
		}
	} else {
		this.prv.mRows.push(row);
	}			
	
	this.body.insertBefore(row.prv.mDOM, next);
	this.prv.mRowCount++;
	return row;
};

function NGrid_setRow(aPos, aArray)
{
	var row = this.prv.mRows[aPos];
	if (!isDefined(row))
		return;
	
	row.setRow(aArray);
};

function NGrid_moveRow(aPos, aNewPos)
{
	if (aPos < 0 || aPos > this.prv.mRows.length)
		return;
	
	// move HTML element
	var row = this.prv.mRows[aPos];
	if (aNewPos+1 >= this.prv.mRows.length) {
		this.body.insertBefore(row.prv.mDOM, null);
	} else {
		var next = this.prv.mRows[aNewPos];
		this.body.insertBefore(row.prv.mDOM, next.prv.mDOM);
	}
	
	// re-insert row into array
	this.prv.mRows.splice(aPos, 1);
	this.prv.mRows.splice(aNewPos, 0, row);
};

function NGrid_insertColumn(aPos)
{
	var column = new NGridColumn(this);
	this.prv.mColCount++;
	
	// no rows? we create 1,1 grid
	if (this.prv.mRows.length <= 0) {
		this.insertRow();
	} else {
		// add elements to the rows
		for (var i = 0; i < this.prv.mRows.length; i++) {
			var row = this.prv.mRows[i];
			row.insertElement();
		}
	}
};

function NGrid_setColumn(aPos, aArray)
{
	if (aPos < 0 || aPos >= this.prv.mColCount)
		return;
	
	this.prv.mColumns[aPos].prv.mColumn = aArray;
	
};

function NGrid_moveColumn(aPos, aNewPos)
{
	if (aPos < 0 || aPos >= this.prv.mColCount ||
		aNewPos < 0 || aNewPos >= this.prv.mColCount)
		return;
	
	var col = this.prv.mColumns[aPos];
	var elems = col.prv.mElements;
	var newcol = null;
	var newelems = null;
	
	if (isDefined(this.prv.mColumns[aNewPos+1])) {
		newcol = this.prv.mColumns[aNewPos+1];
		newelems = newcol.prv.mElements;
	}

	var header = col.prv.mHeader;

	// move and reorder the elements
	if (newcol) {
		for (var i = 0; i < elems.length; i++) {
			var parentFrame = elems[i].prv.mParent.prv.mDOM;
			parentFrame.insertBefore(elems[i].prv.mDOM, newelems[i].prv.mDOM);
		}
		
		this.header.insertBefore(header.prv.mDOM, newcol.prv.mHeader.prv.mDOM);
	} else {
		for (var i = 0; i < elems.length; i++) {
			var parentFrame = elems[i].prv.mParent.prv.mDOM;
			parentFrame.insertBefore(elems[i].prv.mDOM, null);
		}
					
		this.header.insertBefore(header.prv.mDOM, null);
	}
	
	this.prv.mColumns.splice(aPos, 1);
	this.prv.mColumns.splice(aNewPos, 0, col);
	
	// set indices for columns
	for (var i = 0; i < this.prv.mColCount; i++) {
		this.prv.mColumns[i].setIndex(i);		
	}
};

function NGrid_removeColumn(aPos)
{
	if (aPos < 0 || aPos >= this.prv.mColCount)
		return;
	
	aPos = isDefined(aPos) ? aPos : this.prv.mColCount - 1;
	var column = this.prv.mColumns[aPos];
	
	// remove from list
	this.prv.mColumns.splice(aPos, 1);
	this.prv.mColCount--;
	
	var elems = column.prv.mElements;
	for (var i = 0; i < elems.length; i++) {
		if (elems[i].prv.mParent) {
			var parentFrame = elems[i].prv.mParent.prv.mDOM;
			parentFrame.removeChild(elems[i].prv.mDOM);
			elems[i].destroy();
		}
	}
	
	// remove the header		
	var header = column.prv.mHeader;
	this.header.removeChild(header.prv.mDOM);
	header.destroy();

	// remove from memory
	column.destroy();
	delete column;
	
	// set indices for columns
	for (var i = 0; i < this.prv.mColCount; i++) {
		this.prv.mColumns[i].setIndex(i);		
	}
}

function NGrid_removeRow(aPos)
{
	if (aPos < 0 || aPos >= this.prv.mRowCount)
		return;

	var aPos = isDefined(aPos) ? aPos : this.prv.mRowCount - 1;		
	var row = this.prv.mRows[aPos];

	if (row) {
		this.prv.mRows.splice(aPos, 1);
		this.prv.mRowCount--;
		
		this.body.removeChild(row.prv.mDOM);
		//row.destroy();
		delete row;
		
		// we have to remove the references in the columns
		for (var i = 0; i < this.prv.mColumns.length; i++) {
			var col = this.prv.mColumns[i];
			col.prv.mElements.splice(aPos, 1);
		}
		
		// reindex
		for (var i = 0; i < this.prv.mRowCount; i++) {
			this.prv.mRows[i].setIndex(i);
		}
	}
};

function NGrid_row(aPos)
{
	return (aPos >= 0 && aPos < this.prv.mRows.length) ?
		this.prv.mRows[aPos] : null;
};

function NGrid_column(aPos)
{
	return (aPos >= 0 && aPos < this.prv.mColumns.length) ?
		this.prv.mColumns[aPos] : null;
};

function NGrid_clear()
{
	for (var i = 0; i < this.prv.mRowCount; i++) {
		this.prv.mRows[i].clear();
	}
	//this.prv.mRowCount = 0;
};

function NGrid_setFocusAll(aBoolean)
{
	for (var i = 0; i < this.prv.mRowCount; i++) {
		this.prv.mRows[i].setFocus(aBoolean);
	}
	//this.prv.mRowCount = 0;
};

function NGrid_size(aCols, aRows)
{
	// determine how many rows to add
	var rowCount = isDefined(aRows) ? aRows - this.prv.mRowCount : 0;
	if (rowCount < 0) {
		// we have to delete some rows
		for (var i = 0; i < Math.abs(rowCount); i++) {
			this.removeRow();
		}
	}
	
	// determine the number of cols
	var colCount = isDefined(aCols) ? aCols - this.prv.mColCount : 0;
	if (colCount < 0) {
		// we have to delete some columns
		for (var i = 0; i < Math.abs(colCount); i++) {
			this.removeColumn();
		}
	}
	
	this.prv.mColCount = colCount > 0 ? this.prv.mColCount + colCount :
		this.prv.mColCount;

	// go about adding column and rows if necessary
	for (var i = 0; i < this.prv.mRowCount; i++) {
		this.prv.mRows[i].size(this.prv.mColCount);
	}
	
	for (var i = 0; i < rowCount; i++) {
		this.insertRow();
	}
	
	// now go through the rows and assimilate columns
	for (var i = 0; i < this.prv.mColCount; i++) {
		if (i < this.prv.mColumns.length) {
			var column = this.prv.mColumns[i];
		} else {
			var column = new NGridColumn(this);
			this.prv.mColumns.push(column);
		}
		
		var elements = column.prv.mElements;
		
		for (var j = 0; j < this.prv.mRowCount; j++) {
			var row = this.prv.mRows[j];
			if (j < elements.length) {
				elements[j] = row.element(i);
			} else {
				elements.push(row.element(i));
			}				
		}
		
		var h = column.prv.mHeader;
		if (h.prv.mContent == null)
			column.setHeader("");
	}
	
	// reset indices
	for (var i = 0; i < this.prv.mRows.length; i++)	this.prv.mRows[i].setIndex(i);
	for (var i = 0; i < this.prv.mColumns.length; i++) this.prv.mColumns[i].setIndex(i);
	
	this.refresh();
};

function NGrid_setColumnWidth(aWidth)
{

	for (var i = 0; i < this.prv.mColCount; i++) {
		this.prv.mColumns[i].setWidth(aWidth);
	}
};

function NGrid_getWidth()
{
	var rowWidth = this.prv.mPadding; // free space
	for (var i = 0; i < this.prv.mColCount; i++) {
		rowWidth += this.column(i).getWidth();
	}
	return rowWidth;
};

function NGrid_onheaderfocus(aHeaderRef)
{
	// blur the rest of the headers
	for (var i = 0; i < this.prv.mColumns.length; i++) {
		var col = this.prv.mColumns[i];
		var header = col.prv.mHeader;
		
		if (header != aHeaderRef) {
			header.prv.mDOM.onblur();
		}
	}
	
	// if we belong to a gridObject...propagate our event
	if (this.owner && this.owner.onheaderfocus) {
		this.owner.onheaderfocus(aHeaderRef);
	}
};

function NGrid_refresh()
{
	// try setting the widths
	try {
		var rowWidth = this.getWidth();
					
		// set HTML styles
		this.body.style.width = rowWidth +"px";
		this.header.style.width = rowWidth +"px";
	} catch (e) {}
};

function NGrid_null() { };

function NGrid_asyncLoad(table, aQuery) {
	// initialize query if not set by user
	if (!aQuery) {
		aQuery = new Query(table);
		aQuery.batchSize = 10;
	}
	
	aHandler = this.asyncLoad_handler;
	aGrid = this;
	
	// initialize grid
	var aTableObj = getTable(table);
	dax_bridge.cacheFields (aTableObj);
	var columns = aTableObj.fields.length;
	// get number of fields with break levels different that 0 
	// those fields won't be displayed
	for (i = 0; i < aTableObj.fields.length; i++) {
		if (aTableObj.fields[i].fieldbreaklevel != 0) {
			columns--;
		}
	}
	this.firstRowRemoved = false;
			
	if (columns == 0)
		return;
	
	this.size (columns,1);
	
	// hide headers
	//setStyle (this.header, 'display', 'none');
	this.refresh();

	// set loading message
	this.row(0).element(0).setContent(STR['loading']);

	// start batch calls
	aGrid.currentQuery = aQuery;
	aQuery.callRef = this;
	aQuery.nextBatch(aHandler, aGrid);
};

function NGrid_asyncLoad_handler(http_response, aGrid) {
	if (http_response.readyState !=4)
		return;
	aQuery = aGrid.currentQuery;
	aQuery.parseResult (http_response);
	if (aQuery.size > 0) {
		for (g = 0; g < aQuery.result.length; g++) {
			aRow = aGrid.insertRow();
			aData = new Array();
			for (h = 0; h < aQuery.result[g].fields.length; h++) {
				var aField = getField(aQuery.result[g].fields[h].id, '');
				if (aField.fieldbreaklevel == 0) {
					aData.push(aQuery.result[g].fields[h].value);
				}
			}
			aRow.setRow(aData);
		}
	}

	// remove first row
	if (!(aGrid.firstRowRemoved)) {
		aGrid.removeRow (0);
		aGrid.firstRowRemoved = true;
	}
	
	aGrid.asyncLoad_refresh();
	
	if (!(aQuery.lastRecord)) {
		aQuery.nextBatch(aHandler);
	}
};

function NGrid_destroy() {
	purgeparents(this);
	for (var i in this.prv.mColumns)
		this.prv.mColumns[i].destroy();
	for (var i in this.prv.mRows)
		this.prv.mRows[i].destroy();
	if (this.owner)
		this.owner = null;
};

// NGrid_

/**
 * Grid
 *
 * @author Jonathan Le
 */
function NGrid(colCount, rowCount)
{

	var node = getTemplate("newgrid");
	if (!node)
		return null;

	//alert('Ngrid = ' + ++WindowManager.maxZIndex);
	// presentation properties
	this.dom	= node.cloneNode(true);
//	this.dom.id = "newgrid" + getElementsByClassName(document.body, "newgrid").length;
	generateUniqueId(this.dom);
	this.header	= getElementByClassName(this.dom, "header");
	this.body	= getElementByClassName(this.dom, "body");

	// set parent property
	for (var property in this) {
		if (this[property])
			this[property].parent = this;
	}

	// prv
	this.prv = {
		mColCount : 0,
		mRowCount : 0,
		mRows : new Array(),
		mColumns : new Array(),
		mPadding : 10 // extra space to expand the body & header
	};

	this.destroy = NGrid_destroy;
	
	// factory method (header)
	this.getHeaderFrame = NGrid_getHeaderFrame;
	
	// factory method
	this.getRowDOM = NGrid_getRowDOM;
	
	this.insertRow = NGrid_insertRow;
	
	this.setRow = NGrid_setRow;
	
	this.moveRow = NGrid_moveRow;
	
	this.insertColumn = NGrid_insertColumn;
	
	this.setColumn = NGrid_setColumn;
	
	this.moveColumn = NGrid_moveColumn;
	
	this.removeColumn = NGrid_removeColumn;
	
	this.removeRow = NGrid_removeRow;
	
	this.row = NGrid_row;
	
	this.column = NGrid_column;
	
	// Clears the grid and deletes all elements
	this.clear = NGrid_clear;
	
	this.size = NGrid_size;

	this.setFocusAll = NGrid_setFocusAll;
	
	this.setColumnWidth = NGrid_setColumnWidth;
	
	// calculate the grid's with based on the width of the columns
	this.getWidth = NGrid_getWidth;
	
	// Event propagation - a Header has been clicked, handle it.
	this.onheaderfocus = NGrid_onheaderfocus;
	
	this.refresh = NGrid_refresh;
	
	this.onrowselect = NGrid_null;
	this.onrowdblclick = NGrid_null;
	
	// added by Rudolf Psenicnik
	// will populate grid with table data, looping until all records within specified query are downloaded
	this.asyncLoad = NGrid_asyncLoad;
	
	this.asyncLoad_handler = NGrid_asyncLoad_handler;

	// automatically called after each async refresh cycle
	// put user defined functions here
	this.asyncLoad_refresh = NGrid_null;
	
	// init me
	{
		// resize the grid		
		this.size(colCount, rowCount);
		setStyle(this.header, "zIndex", ++WindowManager.maxZIndex);
		setStyle(this.dom, "display", "block");
	}
}

function NGridHeader_setContent(aContent)
{
	var type = typeof aContent;
	switch (type)
	{
		case 'number':
		case 'string':
		{
			this.prv.mDOM.innerHTML = aContent;
		}
		default:
		{
			this.prv.mContent = aContent;
		}
		break;
	}
};

function NGridHeader_setWidth(aWidth)
{
	this.prv.mWidth = aWidth;
	setStyle(this.prv.mDOM, "width", aWidth + "px");
};

function NGridHeader_prv_mDOM_onmousedown(aEvent)
{

	Controller.dragHeader = this;
		
	if (!Controller.isInHeaderResizeArea) {
	
		if (!this.parent.prv.mParent.owner.currentQueryId)	
			return false;
		
		if (this.parent.prv.mParent.owner.sortLock)
			return false;
	
		var header = this.parent;
		if (header.prv.mOrderAsc) {
			this.className = "cellAsc";
		} else {
			this.className = "cellDesc";
		}
		
		header.prv.mSorted = true;
		header.prv.mParent.onheaderfocus(header);
	} else {
		if (document.addEventListener) {
			document.addEventListener("mousemove", headerResizing, true);
			document.addEventListener("mouseup",  headerResizeStop, true);
		} else { // IE
			document.onmousemove = headerResizing;
			document.onmouseup   = headerResizeStop;
		}	
	}
		
//	if (this._resize) {
//	var event = xpEvent(aEvent, true);
//	var header = this.parent;
	
	// Gecko, Safari

	    
//	}

	return false;
};

function NGridHeader_prv_mDOM_onmouseup(aEvent)
{

	// stop resizing when done
	if(!Controller.isInHeaderResizeArea) {
		this.onfocus();
	}
};

function NGridHeader_prv_mDOM_onmousemove(aEvent)
{
	if ((!aEvent) && (!browser.ie7))
		return;
		
	//debugAlert ('mouse move, dragHeader:' + Controller.dragHeader);		
		
	var event = xpEvent(aEvent);
	var coord = getMousePosition(event);
	var gPos = getObjectPosition(this);

	var column = this.parent.prv.mColumn;
	var grid = column.prv.mParent;

	// get the local x coordinate
	var x = coord.x - gPos.x;
	var margin = 5;
	var style = this.style;
	
	if (!isDefined(this._originalCursor)) {
		this._originalCursor = style.cursor;
	}
	
	// set the resize flag
	this._resize = true;
	Controller.isInHeaderResizeArea = true;
	this._right = false;
	
	// touched the left-most side
	//if (x > 0 && x <= margin && column.getIndex() > 0) {
	//	style.cursor = "w-resize";
	//} else 

	if (x < this.offsetWidth && x >= (this.offsetWidth - margin) && (column.prv.mIndex < grid.prv.mColCount-1)) {
		style.cursor = "e-resize";
		this._right = true;
	} else {
		style.cursor = this._originalCursor;
		this._resize = false;
		Controller.isInHeaderResizeArea = false;
	}
};

function NGridHeader_prv_mDOM_onmouseover(aEvent)
{
	this.className = "cellOver";
};

function NGridHeader_prv_mDOM_onmouseout(aEvent)
{
	var header = this.parent;
	if (header.prv.mSorted) {
		if (header.prv.mOrderAsc) {
			this.className = "cellAsc";
		} else {
			this.className = "cellDesc";
		}
	} else {
		this.className = "cell";
	}
};

function NGridHeader_prv_mDOM_onfocus(aEvent)
{

	//if(Controller.isInHeaderResizeArea)
	//	return false;
	
	return false;
	
	var header = this.parent;
	if (header.prv.mOrderAsc) {
		this.className = "cellAsc";
	} else {
		this.className = "cellDesc";
	}
	
	header.prv.mSorted = true;
	header.prv.mParent.onheaderfocus(header);
};

function NGridHeader_prv_mDOM_onblur(aEvent)
{
	return false;
	
	this.parent.prv.mSorted = false;
	this.className = "cell";
};

// NGridHeader_prv_mDOM_
// NGridHeader_

function NGridHeader_destroy() {
	this.prv.mDOM.parent = null;
	this.prv.mParent = null;
	this.prv.mColumn = null;
};

/**
 * Object for the Grid header.
 *
 * @author Jonathan Le
 */
function NGridHeader(parent)
{
	// prv
	this.prv = {
		mParent : parent,
		mDOM : parent.getHeaderFrame(),
		mContent : null,
		mColumn : null,
		mWidth : 50,
		mOrderAsc : false,
		mSorted : false
	}; //Function.decl.apply(this);
	
	this.destroy = NGridHeader_destroy;
	
	this.prv.mDOM.parent = this;
	
	this.setContent = NGridHeader_setContent;
	
	this.setWidth = NGridHeader_setWidth;
	
	// events
	this.prv.mDOM.onmousedown = NGridHeader_prv_mDOM_onmousedown;
	
	this.prv.mDOM.onmouseup = NGridHeader_prv_mDOM_onmouseup;
	
	this.prv.mDOM.onmousemove = NGridHeader_prv_mDOM_onmousemove;
	
	this.prv.mDOM.onmouseover = NGridHeader_prv_mDOM_onmouseover;
	
	this.prv.mDOM.onmouseout = NGridHeader_prv_mDOM_onmouseout;

	this.prv.mDOM.onfocus = NGridHeader_prv_mDOM_onfocus;
	
	this.prv.mDOM.onblur = NGridHeader_prv_mDOM_onblur;
	
	// init
	{
		this.setWidth(this.prv.mWidth);
	}
}

function headerResizing(aEvent)
{

	if (!Controller.dragHeader)
		return false;

	Controller.isInHeaderResizeArea = true;		
		
	var event = xpEvent(aEvent, true);
//	var dom = event._target;
	var dom = Controller.dragHeader;
	var header = dom.parent;

	// abort if necessary
	/*
	if (!dom || !header || !dom._resize) {
		return headerResizeStop(aEvent);
	}
		
	*/
	
	var mpos = getMousePosition(event);
	var pos = getObjectPosition(dom);
	
	// calculate the difference
	var dx = mpos.x - pos.x;
	var column = header.prv.mColumn;
	var grid = column.prv.mParent;

	column.setWidth(dx);
	grid.refresh();
	
	// for grids belonging to a gridObject
	if (grid.owner) {
		grid.owner.refresh();
	}
	
	dom._resize = true;
	
	return false;
};

function headerResizeStop(aEvent)
{

	Controller.dragHeader = null;
	//Controller.isInHeaderResizeArea = true;
	
	var event = xpEvent(aEvent, true);
	var dom = event._target;

	// Gecko
	if (document.removeEventListener) {
		document.removeEventListener("mousemove", headerResizing, true);
		document.removeEventListener("mouseup",   headerResizeStop, true);
	} else {
		document.onmousemove = null;
 		document.onmouseup   = null;
	}
	
	dom._resize = false;
	
	return false;
};
	
function NGridRow_toString()
{
	return "[object GridRow]";
};

function NGridRow_getElementDOM()
{
	var element = getElementByClassName(this.prv.mDOM, "cell");
	if (this.prv.mElements.length <= 0) {
		return element;
	} else {
		return element.cloneNode(true);
	}
};
	
function NGridRow_setRow(aArray)
{
	// set content for all row elements
	for (var i = 0; i < this.prv.mElements.length; i++) {
		if (isDefined(aArray[i])) {
			this.prv.mElements[i].setContent(aArray[i]);
		}
	}
};
	
function NGridRow_insertElement(aPos)
{
	var next = null;
	
	if (isDefined(aPos)) {
		var aPos = (aPos >= 0 && aPos < this.prv.mElements.length) ? aPos :
			this.prv.mElements.length;
	} else {
		var aPos = this.prv.mElements.length;
	}
	var element = new NGridElement(this);
	this.prv.mElements.splice(aPos, 0, element);
	
	element.setPos(aPos, this.prv.mIndex);
	this.prv.mDOM.insertBefore(element.prv.mDOM, next);

	element.clear();
};
	
function NGridRow_element(aPos)
{
	return (aPos >= 0 && aPos < this.prv.mElements.length) ?
		this.prv.mElements[aPos] : null;
};
	
function NGridRow_prv_mDOM_onmouseover(aEvent)
{
	if (this.parent.prv.mFocus)
		this.className = "rowfocushover";
	else
		this.className = "rowhover";
};

function NGridRow_prv_mDOM_onmouseout(aEvent)
{
	if (this.parent.prv.mFocus)
		this.className = "rowfocused";
	else
		this.className = "row";
};

function NGridRow_prv_mDOM_onclick(aEvent)
{
	var event = xpEvent(aEvent, true);
	//hack for safari, it doesn't work if the  _target.textContent is blank
	var row = this.parent;
	if (row.prv.mParent.owner.owner != undefined) {
		if (!row.prv.mParent.owner.owner.selectOneRow) {
	
			if(browser.safari || browser.ie7){
				if(event._target.innerHTML != '')
					row.setFocus(!row.prv.mFocus);
			}
			else{
				if(event._target.textContent != '')
					row.setFocus(!row.prv.mFocus);
			}
	
		} else {
		
			row.prv.mParent.setFocusAll(false);	// Deselect all rows
			row.setFocus(true);	// Select the row that the user clicked
		
		}
	}
	// propagate some events
	this.onmouseover();
	row.prv.mParent.onrowselect(row, aEvent);

};

function NGridRow_prv_mDOM_ondblclick(aEvent)
{
	if(this.firstChild.innerHTML != ''){ // make sure to get the record if the dbl click happens on the record and not on the empty field
		var row = this.parent;		
		row.prv.mParent.onrowdblclick(row);
	}
};

function NGridRow_setFocus(aFocus)
{
	if (!isDefined(aFocus))
		var aFocus = false;
		
	if (aFocus) {
		this.prv.mDOM.className = "rowfocused";
	} else {
		this.prv.mDOM.className = "row";	
	}

	this.prv.mFocus = aFocus;
};

function NGridRow_getCellValue(colNumber) {

	if (this.element(colNumber))
		return this.element(colNumber).prv.mContent;
	else
		return null;
}


function NGridRow_getHeight()
{
	if (this.prv.mElements.length > 0) {
		return this.prv.mElements[0].getHeight();
	} else {
		return 0;
	}
};

function NGridRow_removeElement(aPos)
{
};

function NGridRow_size(aWidth)
{
	// generate the frame of this row
	var count = aWidth - this.prv.mElements.length;
	if (count < 0) {
		for (var i = 0; i < Math.abs(count); i++) {
			this.removeElement();
		}
	} else {
		for (var i = 0; i < count; i++) {
			this.insertElement();
			if (this.prv.mParent.column(i)) {
			//	this.element(i).prv.mWidth=this.prv.mParent.column(i).prv.mWidth;
				this.element(i).setWidth(this.prv.mParent.column(i).prv.mHeader.prv.mWidth);
			}
		}
	}
};

function NGridRow_setIndex(aIndex)
{
	if (aIndex >= this.prv.mElements.length)
		return;
		
	for (var i = 0; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].setPos(null, aIndex);
	}
	
	this.prv.mIndex = aIndex;
};

function NGridRow_clear()
{
	for (var i = 0; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].clear();
	}
};

function NGridRow_destroy() {
	this.prv.mDOM.parent = null;
	this.prv.mParent = null;
	for (var i in this.prv.mElements)
		this.prv.mElements[i].destroy();
};
	
// NGridRow_prv_mDOM_
// NGridRow_
	
function NGridRow(parent)
{
	// prv
	this.prv = {
		mParent : parent,
		mDOM : parent.getRowDOM(),
		mElements : new Array(),
		mIndex : parent.prv.mRowCount,
		mFocus : false
	};

	//Function.decl.apply(this);
	this.prv.mDOM.parent = this;

	this.toString = NGridRow_toString;
	
	// factory method
	this.getElementDOM = NGridRow_getElementDOM;
	
	this.setRow = NGridRow_setRow;
	
	this.insertElement = NGridRow_insertElement;
	
	this.element = NGridRow_element;
	
	this.prv.mDOM.onmouseover = NGridRow_prv_mDOM_onmouseover;
	
	this.prv.mDOM.onmouseout = NGridRow_prv_mDOM_onmouseout;
	
	this.prv.mDOM.onclick = NGridRow_prv_mDOM_onclick;
	
	this.prv.mDOM.ondblclick = NGridRow_prv_mDOM_ondblclick;
	
	this.setFocus = NGridRow_setFocus;

	this.getCellValue = NGridRow_getCellValue;
	
	this.getHeight = NGridRow_getHeight;
	
	this.removeElement = NGridRow_removeElement;
	
	// set the column size
	this.size = NGridRow_size;
	
	this.setIndex = NGridRow_setIndex;
	
	// clear out the elements in the row
	this.clear = NGridRow_clear;
	
	this.destroy = NGridRow_destroy;

	// initializer
	{
		this.size(parent.prv.mColCount);
	}
	return this;
}


function NGridColumn_element(aPos)
{
	return (aPos >= 0 && aPos < this.prv.mElements.length) ?
		this.prv.mElements[aPos] : null;
};

function NGridColumn_header()
{
	return this.prv.mHeader;
};

function NGridColumn_setHeader(aContent) {
	var header = this.prv.mHeader;
	header.setContent(aContent);
	header.prv.mColumn = this;
};

function NGridColumn_setColumn(aArray)
{
	// set content for all row elements
	for (var i = 0; i < this.prv.mElements.length; i++) {
		if (isDefined(aArray[i])) {
			this.prv.mElements[i].setContent(aArray[i]);
		}
	}
};

function NGridColumn_getWidth()
{
	if (this.prv.mElements.length > 0) {
		return this.prv.mElements[0].getWidth();
	} else {
		return this.prv.mWidth;
	}
};

function NGridColumn_setWidth(aWidth, overrideMin)
{
	//alert (aWidth);
	var aWidth = ((aWidth > this.prv.mMinimumWidth) || (overrideMin)) ? aWidth : this.prv.mMinimumWidth;
	// set individual elements
	for (var i = 0; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].setWidth(aWidth);
	}
	
	this.prv.mWidth = aWidth;
	
	// set header
	this.prv.mHeader.setWidth(aWidth);
};

function NGridColumn_setIndex(aIndex)
{
	if (aIndex >= this.prv.mElements.length)
		return;
		
	for (var i = 0 ; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].setPos(aIndex, null);
	}
	
	this.prv.mIndex = aIndex;
};


function NGridColumn_destroy() {
	this.prv.mParent = null;
	this.prv.mHeader.destroy();
	for (var i in this.prv.mElements)
		this.prv.mElements[i].destroy();
};

function NGridColumn_hide() {

	//alert (aWidth);
	var aWidth = 0;
	// set individual elements
	for (var i = 0; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].setWidth(aWidth);
		this.prv.mElements[i].prv.mDOM.className = 'cell cell_hidden';
	}
	
	this.prv.mWidth = aWidth;
	
	// set header
	this.prv.mHeader.setWidth(aWidth);
	this.prv.mHeader.prv.mDOM.style.display = 'none';
	
	this.prv.mVisible = false;

	if (this.prv.mParent.refresh)
		this.prv.mParent.refresh();
	
	// for grids belonging to a gridObject
	if (this.prv.mParent.owner)
		if (this.prv.mParent.owner.refresh)
			this.prv.mParent.owner.refresh();
	
};

function NGridColumn_show() {

	//alert (aWidth);
	var aWidth = this.prv.mMinimumWidth;
	// set individual elements
	for (var i = 0; i < this.prv.mElements.length; i++) {
		this.prv.mElements[i].setWidth(aWidth);
		this.prv.mElements[i].prv.mDOM.className = 'cell';
	}
	
	this.prv.mWidth = aWidth;
	
	// set header
	this.prv.mHeader.setWidth(aWidth);
	this.prv.mHeader.prv.mDOM.style.display = '';
	
	this.prv.mVisible = true;

	
	if (this.prv.mParent.refresh)
		this.prv.mParent.refresh();
	
	// for grids belonging to a gridObject
	if (this.prv.mParent.owner)
		if (this.prv.mParent.owner.refresh)
			this.prv.mParent.owner.refresh();
	
};

// NGridColumn_

function NGridColumn(parent)
{
	// prv
	this.prv = {
		mParent			: parent,
		mElements		: new Array(),
		mHeader			: new NGridHeader(parent),
		mIndex			: 0,
		mWidth			: 50, // Reverting to 50 - Rodrigo had it changed to 65 - Was breaking the admin and no comment
		mMinimumWidth	: 50,
		mVisible		: true
	}; //Function.decl.apply(this);
	
	this.prv.mHeader.prv.mColumn = this;
	
	this.element = NGridColumn_element;
	
	// just an alias for getHeader
	this.header = NGridColumn_header;
	
	// override
	this.setHeader = NGridColumn_setHeader;
	
	this.setColumn = NGridColumn_setColumn;

	this.getWidth = NGridColumn_getWidth;
	
	this.setWidth = NGridColumn_setWidth;
	
	this.setIndex = NGridColumn_setIndex;
	
	this.destroy = NGridColumn_destroy;
	
	this.hide = NGridColumn_hide;

	this.show = NGridColumn_show;
	
	// initializer
	{
		var header = this.prv.mHeader;
		this.prv.mParent.header.insertBefore(header.prv.mDOM, null);
	}
	return this;
}

function NGridElement_getWidth()
{
	if (this.prv.mWidth > 0 && this.prv.mWidth > this.prv.mDOM.offsetWidth) {
		return this.prv.mWidth;
	} else {
		return this.prv.mDOM.offsetWidth;
	}
};

function NGridElement_getHeight()
{
	if (this.prv.mDOM.offsetHeight) {
		if (this.prv.mHeight > 0 && this.prv.mHeight > this.prv.mDOM.offsetHeight) {
			return this.prv.mHeight;
		} else {
			return this.prv.mDOM.offsetHeight;
		}
	} else {
		return this.prv.mHeight;
	}
};

function NGridElement_setWidth(aWidth)
{
	this.prv.mWidth = aWidth;

	setStyle(this.prv.mDOM, "width", aWidth + "px");
};

function NGridElement_setHeight(aHeight)
{
	this.prv.mHeight = aHeight;
	setStyle(this.prv.mDOM, "height", aHeight + "px");
};

function NGridElement_setPos(aX, aY)
{
	if (isDefined(aX))
		this.prv.mPosX = aX;
	
	if (isDefined(aY))
		this.prv.mPosY = aY;
};

function NGridElement_prv_mDOM_onmousedown()
{
	return false;
};

function NGridElement_prv_mDOM_onmouseup()
{
	return true;
};

function NGridElement_setType(aType)
{
	this.prv.mType = aType;
};

function NGridElement_link_ti_dom_onclick() { 
	this.parent.setSelected (true, true);
	this.winRef.activate();
};

function NGridElement_link_newWindow_onsize() {
	// image resize
	
	// get the new window dimensions and assign to the image dimensions
	var hImage = getElementByClassName (this.image, 'image_preview');
	hImage.widht = this.contentArea.offsetWidth;
	hImage.height = this.contentArea.offsetHeight;
};

function NGridElement_link_newWindow_onclose() {
	if (this.tbitem) {
		this.tbitem.remove();
		this.tbitem = null;
	}
	
	// get rid of the grid
	if (this.contentArea.grid)
		this.contentArea.grid.destroy();
	
	if (this.editor)
		this.editor.destroy();
}							


function NGridElement_link_onclick(aEvent) { 

	var picWidth = this.parent.picWidth;
	var picHeight = this.parent.picHeight;
	if (aEvent)
		xpEvent(aEvent, false);
	imageNode = getTemplate('image_preview_container');
	var newWindow = WindowManager.createNewWindow();
	newWindow.titleBar.innerHTML = STR['image_viewer'];
	if(picWidth < 300)
		picWidth = 300;
	if(picHeight < 200)
		picHeight = 200;								
	newWindow.size(picWidth, picHeight);
	newWindow.contentArea.appendChild(imageNode);							
	newWindow.activate();
	newWindow.open();
	
	i = 1;
	while ($('image_preview_container' + i) != null) {
		i++;
	}

	imageNode.id = 'image_preview_container' + i;
	
	if (taskbar) {
	
		var ti = new taskbarItem(taskbar);

		// make unique label based on table name
		i = 1;
		while ($('dax_taskbar_button_image' + i) != null) {
			i++;
		}
		
		ti.dom.id = 'dax_taskbar_button_image' + i;

		ti.setLabel("Image" + i);
		ti.setIcon("editor");

		// create a handler for clicking
		var ref = this
		ti.dom.winRef = newWindow;
		ti.dom.onclick = NGridElement_link_ti_dom_onclick;
		
		setStyle (ti.dom, 'display', 'block');
		ti.setSelected (true, true);
		
		ti.parent.refresh();
		newWindow.tbitem = ti;
		
	}
	
	newWindow.image = imageNode;
	newWindow.onsize = NGridElement_link_newWindow_onsize;

	this.image = getElementByClassName (imageNode, 'image_preview');
	this.image.src = this.aContent;								
	
	newWindow.onclose = NGridElement_link_newWindow_onclose;
								
	return false;
};


function NGridElement_setContent(aContent)
{
	// examine the content type
	var type = typeof aContent;
	
	switch (type) {
		case 'string':
		case 'number':
		{
			var dom = this.prv.mDOM;
			
			//check if it is a ddw
			if((aContent.indexOf) && (aContent.indexOf('ddw_link') != -1) && (aContent.indexOf('http:') != -1)){
				dom.innerHTML = aContent;
			}
			else{
				var regex = /^\/DAX\/GetImage\/|http:\/\//;	

				if (regex.test(aContent)) {
					var image = /^\/DAX\/GetImage\//;
					var dom = this.prv.mDOM;
					
					//if (!dom.link) {
						link = document.createElement("a");
						dom.innerHTML = "";
						dom.appendChild(link);
						dom.link = link; // save the link
						link.className = "dax_grid_imagelink";
					//} else {
					//	link = dom.link;
					//}
					
					link.href = "#";
					link.parent = this;
					link.aContent = aContent;
					link.onclick = NGridElement_link_onclick;
					
					if (image.test(aContent)) {
						link.innerHTML = STR['view_image'];
					} else {
						link.innerHTML = aContent;
					}
		
				} else {
					this.prv.mDOM.innerHTML = aContent;
				}
			}
		}
		break;
		
		case 'object':
		{
			if (aContent instanceof String) {
				this.setContent(aContent.valueOf());
			} else if (aContent.nodeType == 1) { // is content a DOM element
				this.clear();
				this.prv.mDOM.appendChild(aContent);
			}
		}
		break;
		default:
		// unsupported
		break;
	}
	
	this.prv.mContent = aContent;
};

function NGridElement_prv_mDOM_nullevent(event)
{
};

function NGridElement_clear()
{
	this.setContent('');
};

function NGridElement_destroy() {
	this.prv.mDOM.parent = null;
	this.prv.mParent = null;
};

// NGridElement_prv_mDOM_
// NGridElement_

function NGridElement(parent)
{
	// private
	this.prv = {
		mParent : parent,
		mDOM : parent.getElementDOM(),
		mPosX : 0,
		mPosY : 0,
		mType : 1,
		mWidth : 50,
		mHeight : 20,
		mContent : null
	};
	//Function.decl.apply(this);
	
	this.prv.mDOM.parent = this;
	
	this.getWidth =	NGridElement_getWidth;
		
	this.getHeight = NGridElement_getHeight;
		
	this.setWidth = NGridElement_setWidth;
		
	this.setHeight = NGridElement_setHeight;
		
	this.setPos = NGridElement_setPos;

	this.prv.mDOM.onmousedown = NGridElement_prv_mDOM_onmousedown;
		
	this.prv.mDOM.onmouseup = NGridElement_prv_mDOM_onmouseup;

	this.setType = NGridElement_setType;
	
	this.setContent = NGridElement_setContent;
	
	this.prv.mDOM.onmouseover = NGridElement_prv_mDOM_nullevent;
	
	this.prv.mDOM.onmouseout = NGridElement_prv_mDOM_nullevent;
	
	this.prv.mDOM.onclick = NGridElement_prv_mDOM_nullevent;
	
	this.clear = NGridElement_clear;
	
	this.destroy = NGridElement_destroy;
	
	// stat initializer
	{
		this.setWidth(this.prv.mWidth);
		this.setHeight(this.prv.mHeight);
		this.setContent("");
	}
	
	return this;
}

// delay inline query for live search
function inline_query_delay(obj, isKeyPressed, fieldid, value, queryId, queryName, dataWindowRef, keepOldQueryId)
{
	
	if (!obj.delaySearchObj) {
		obj.delaySearchObj = {
			fieldid			: fieldid,
			value			: value,
			queryId			: queryId,
			queryName		: queryName,
			dataWindowRef	: dataWindowRef,
			keepOldQueryId	: keepOldQueryId,
			isDelayed		: false
		}	
	}
	
	if (isKeyPressed)
		obj.delaySearchObj.value = value;
	
	if (obj) {
		if (obj.boundTo)
			obj.boundTo.search.saveButton.disable();
		else if (obj.saveButton)
			obj.saveButton.disable();
		else {
			var objRef = getParentByClassName(obj, 'searchwidget');
			if (objRef)
				objRef.owner.search.saveButton.disable();
			
		}
	}
	
	var aDelay = obj.delaySearchObj;
	
	debugAlert(aDelay.value + '-- isDelayed:' + aDelay.isDelayed + ' delayInt:' + aDelay.delayInt);
	
	if (isKeyPressed) {
		// check if interval is already in place, if so clear it and start a new one
		window.clearTimeout (aDelay.delayInt);
		//aDelay.isDelayed = false;
		aDelay.delayInt = window.setTimeout (function() {inline_query_delay(obj, false); }, 1000);
	} else {
		window.clearTimeout (aDelay.delayInt);
		obj.delaySearchObj = null;
		inline_query(obj, aDelay.fieldid, aDelay.value, aDelay.queryId, aDelay.queryName, aDelay.dataWindowRef, aDelay.keepOldQueryId);
	}
}

function inline_query(obj, fieldid, value, queryId, queryName, dataWindowRef, keepOldQueryId)
{

	if (obj) {
		if (obj.boundTo)
			obj.boundTo.search.saveButton.disable();
		else if (obj.saveButton)
			obj.saveButton.disable();
		else {
			var objRef = getParentByClassName(obj, 'searchwidget');
			if (objRef)
				objRef.owner.search.saveButton.disable();
			
		}
	}
		
	if (!dataWindowRef) {

		var toolbar = null;
		if (!(toolbar = getParentByClassName(obj, "window_toolbar")))
			return;
				
		var content = toolbar.parent.contentArea;
	
	} else {
	
		var content = dataWindowRef.contentArea;
		
	}
	var comparison = null;
	
	if (value != "*")
		comparison = "&fieldid="+fieldid + "&comp=equal&value=" + dax_bridge.encode(value) + "@";

	// check for proper comparison criteria (real and int numbers should be cleared if only @ is left)

	var grid = content.grid;	
	
	if (!grid.selection) {
		grid.selection = {
			tableId : grid.owner.table.tableid,
			queryId : null,
			tableName : grid.owner.table.tablename,
			map : new Array(),
			formatMap : new Array(),
			fieldTypeMap : new Array(),
			range : {start : 0, length : 0},
			toString : function() { return this.tableId + " " + this.queryId + " " +
				this.tableName + " " + this.total; }
		};
	
	}
	
	// set comparison
	grid.selection.comparison = comparison;

	if (queryName) {

		grid.recreateSelection = true;
	
		var pop = {
				queryName : queryName,
				tableId : grid.selection.tableId,
				startIndex : 0,
				// length : grid.selection.range.length,
				comparison: grid.selection.comparison
			};	

	} else {
	
		if (!queryId) {
		
			var pop = {
				queryId : grid.selection.queryId,
				tableId : grid.selection.tableId,
				startIndex : grid.selection.range.start,
				// length : grid.selection.range.length,
				comparison: grid.selection.comparison,
				keepOldQueryId : keepOldQueryId
			};
		
		} else {

		var pop = {
				queryId : queryId,
				tableId : grid.selection.tableId,
				startIndex : grid.selection.range.start,
				// length : grid.selection.range.length,
				comparison: grid.selection.comparison,
				keepOldQueryId : keepOldQueryId
			};	
		}
	}

	// populate the grid
	grid.clear();
	grid.populate(pop);
	
	//grid.viewport.vscrollbar.scroll(0);
}


// quick launch wrappers

// loads async grid
function addAsyncGrid (table, node)
{
	nGrid = new NGrid();
	node.appendChild (nGrid.dom);
	nGrid.asyncLoad (table);
	
	return grid;
}

function ListBox_insertRow(dataArray) {

	aRow = this.grid.insertRow();
	aRow.setRow(dataArray);

};

function ListBox_redraw() {

	// update width and height
	this.dom.style.width = this.width + 'px';
	this.dom.style.height = this.height + 'px';

	// redraw grid
	this.grid.setColumnWidth (this.dom.offsetWidth / this.grid.prv.mColCount );
	this.grid.refresh();	

};

// ListBox_

// list box wrapper
function ListBox (aTarget, headersArray, aWidth, aHeight)
{
	if (!aTarget)
		return;

	// div to encapsulate grid
	var listBox = document.createElement('div');

	this.target = aTarget;
	this.dom = listBox;
	this.grid = new NGrid (headersArray.length, 0);

	this.target.appendChild (this.dom);
	
	// set width
	if (!aWidth)
		this.width = this.target.offsetWidth;
	else
		this.width = aWidth;

	// set height
	if (!aWidth)
		this.height = this.target.offsetHeight;
	else
		this.height = aHeight;


	this.dom.appendChild(this.grid.dom);
	this.dom.style.overflow = 'auto';
	
	// set column headers
	for (i = 0; i < headersArray.length; i++) {
		this.grid.column(i).setHeader(headersArray[i]);
		var header = this.grid.column(i).prv.mHeader;		
		// remove on click functions
		header.prv.mDOM.onmouseover = '';
		header.prv.mDOM.onmouseout = '';
		header.prv.mDOM.onclick = '';
		header.prv.mDOM.onmouseup = '';
		header.prv.mDOM.onmousedown = '';			
	}

	// add new row	
	this.insertRow = ListBox_insertRow;
	
	// redraws the graphics
	this.redraw = ListBox_redraw;
	
	this.redraw();
	
	return this;

}