function dax_superGrid(targetNode, lockedTop, lockedLeft, lockedRight, lockedBottom) {

	// get template nodes
	this.nodes = {
		grid				: getTemplate('dax_grid'),
		gridContainer		: getTemplate('dax_grid_container'),
		row					: getTemplate('dax_grid_row'),
		cell				: getTemplate('dax_grid_cell'),		
		cellInput			: getTemplate('dax_grid_cell_input'),
		cellSelect			: getTemplate('dax_grid_cell_select'),
		cellCheckBox		: getTemplate('dax_grid_cell_checkbox'),
		cellTextArea		: getTemplate('dax_grid_cell_textarea')
	}

	// 
	this.targetNode = targetNode;
	
	// properties
	if (!lockedTop)
		lockedTop = 0;
	if (!lockedLeft)
		lockedLeft = 0;
	
	// internal properties
	this.rows = [ ];
	this.cols = [ ];	
	
	// ok to change properties
	this.gridProperties = {
		useAlternateRowColor	: true,					// use alternate row color	
		lockedTop				: lockedTop,			// rows outside viewport
		lockedLeft				: lockedLeft,			// columns outside viewport
		lockedBottom			: lockedBottom,			// rows outside viewport
		lockedRight				: lockedRight,			// columns outside viewport
		headerRows				: lockedTop,			// rows to inherit header style and functions (drag, resize)
		headerColumns			: lockedLeft,			// columns to inherit header style and functions (drag, resize)
		defaultRowHeight		: 25,
		allowColumnResize		: true,
		allowRowResize			: true,
		allowColumnSelection	: true,
		allowRowSelection		: true,
		allowCellSelection		: true,
		useMultipleSelection	: true,
		resizeViewportToGrid	: false,				// if true, viewport will size to fit grid height if grid is smaller than targetNode
		disableVisualRefresh	: false,
		className				: '',					// dev defined class name
		internalClassName		: '',					// internal class name, used for DAX objects, ex: 'dax_reportviewer'
		allowColumnSwap			: true
	};
	
	// read only properties, these are actual values
	this.dimensions = {
		gridWidth		: 0,
		gridHeight		: 0,
		columns			: 0,
		rows			: 0,
		lockedTop		: 0,
		lockedBottom	: 0,
		lockedRight		: 0,
		lockedLeft		: 0
	}
	
	this.dragging = {
		headerMode			: 'move',	// header can be 'move'd or 'resize'd
		columnNumber		: null		// number of the column that is resized or moved
	}
	
	// applied to new rows & columns
	this.defaultStyle = {
		grid	: '',		// style for entire grid
		row		: '',
		rowAlt	: '',
		column	: '',
		cell	: ''	
	}

	// applied to new rows & columns
	this.defaultInternalStyle = {
		grid	: '',		// style for entire grid
		row		: '',
		rowAlt	: '',
		column	: '',
		cell	: ''	
	}	
	
	// applied to new rows & columns, for selected elements
	this.defaultSelectedStyle = {
		grid	: 'dax_grid_cell_selected',		// style for entire grid
		row		: 'dax_grid_cell_selected',
		rowAlt	: 'dax_grid_cell_selected',
		column	: 'dax_grid_cell_selected',
		cell	: 'dax_grid_cell_selected'	
	}
	
	this.onColumnClick = function(column) { };
	this.onRowClick = function(row) { };
	this.onCellClick = function(row, column) { };
	this.onMoveColumn = function(col1, col2) { };
	
	this.onColumnDblClick = function(column) { };
	this.onRowDblClick = function(row) { };
	this.onCellDblClick = function(row, column) { };
	
	this.onCellHover = function(row, column) { };
	this.onCellOut = function(row, column) { };
	
	this.onMouseColumnResizeStop = function(column, width) { };
	
	this.onRowFooterChange = function() { };
	
	/*
	// Drag & drop support
	*/
	
	// event handlers
	this.ondragover = null;
	this.ondragout = null;
	this.ondragrelease = null;

	// mouse over, mouse out , and on drag release functions
	function dax_grid_cell_onmouseover(e){

		if (typeof(xpEvent) == 'undefined')
			return false;
	
		if (!dax_bridge)
			return false;
	
		if (this.allowDragDrop) {
			if (dax_bridge.hoverDragging) {
				dax_bridge.hoverDragging.target = this;
				this.originalClassName = this.className;
				var hoverStyle = 'dax_grid_cell_drag_hover';
				if (this.grid.ondragover)
					hoverStyle = this.grid.ondragover(this,e);
				if (hoverStyle) {
					this.className += ' ' + hoverStyle;
				}
			}
		}
		
		// single cell hover for the header
		if (!this.grid.rows[this.row].isHeader) {
			this.grid.rows[this.row].dom.originalRowName = this.grid.rows[this.row].dom.className;
			this.grid.rows[this.row].leftDom.originalRowName = this.grid.rows[this.row].leftDom.className;
			this.grid.rows[this.row].rightDom.originalRowName = this.grid.rows[this.row].leftDom.className;
			this.grid.rows[this.row].dom.className += ' dax_grid_row_hover';
			this.grid.rows[this.row].leftDom.className += ' dax_grid_row_hover';
			this.grid.rows[this.row].rightDom.className += ' dax_grid_row_hover';
		} else {
			this.originalClassName = this.className;
			this.className += ' dax_grid_cell_firstheader_hover';
		}
	}	
	
	function dax_grid_cell_onmouseout(){

		if (typeof(xpEvent) == 'undefined')
			return false;
	
		if (!dax_bridge)
			return false;

		if (this.allowDragDrop) {
			if (dax_bridge.hoverDragging) {
				dax_bridge.hoverDragging.target = null;
				if (this.grid.ondragout)
					this.grid.ondragout(this);
				this.className = this.originalClassName;
			}
		}
		
		if (!this.grid.rows[this.row].isHeader) {
			this.grid.rows[this.row].dom.className = this.grid.rows[this.row].dom.originalRowName;
			this.grid.rows[this.row].leftDom.className = this.grid.rows[this.row].leftDom.originalRowName;
			this.grid.rows[this.row].rightDom.className = this.grid.rows[this.row].rightDom.originalRowName;
		} else {
			this.className = this.originalClassName;
		}
	}
	
	function dax_grid_cell_ondragrelease() {
		if (this.allowDragDrop) {
			if (dax_bridge.hoverDragging) {
				if (dax_bridge.hoverDragging.target) {
					this.className = this.originalClassName;
					if (this.grid.ondragrelease)
						this.grid.ondragrelease(this);	
					dax_bridge.hoverDragging = null;
				}
			}
		}
	}
	
	// allow for column resizing
	function dax_grid_allowColumnResize(col, allowResize) {
		this.cols[col].isResizable = allowResize;
	}
	
	this.allowColumnResize = dax_grid_allowColumnResize;
	
	// set draggable cells
	function dax_grid_setDropCells(row, col, isDroppable){
	
		if (!isDroppable)
			isDroppable = false;
	
		if (((!row) && (row != 0)) && ((!col) && (col != 0))) {
			// grid
			var rowCountStart = 0;
			var rowCountEnd = this.dimensions.rows;
			var cellCountStart = 0;
			var cellCountEnd = this.dimensions.columns;
		} else if ((!row) && (row != 0)) {
			// column
			var rowCountStart = 0;
			var rowCountEnd = this.dimensions.rows;
			var cellCountStart = col;
			var cellCountEnd = col + 1;
		} else if ((!col) && (col != 0)) {
			// row
			var rowCountStart = row;
			var rowCountEnd = row + 1;
			var cellCountStart = 0;
			var cellCountEnd = this.dimensions.columns;
		} else {
			// cell
			var rowCountStart = row;
			var rowCountEnd = row + 1;
			var cellCountStart = col;
			var cellCountEnd = col + 1;
		}			
	
		for (var rowCount = rowCountStart; rowCount < rowCountEnd; rowCount++) {
			for (var cellCount = cellCountStart; cellCount < cellCountEnd; cellCount++) {
				var currentCell = this.rows[rowCount].cells[cellCount];
				currentCell.allowDragDrop = isDroppable;
			}
		}	
	}

	/*
	// Changing grid properties
	*/
	
	// allow for row and column resizing
	function dax_grid_allowResize(resizeRows, resizeColumns) {
		if (resizeRows == false)
			this.gridProperties.allowRowResize = false;
		else if (resizeRows)
			this.gridProperties.allowRowResize = true;

		if (resizeColumns == false)
			this.gridProperties.allowColumnResize = false;
		else if (resizeColumns)
			this.gridProperties.allowColumnResize = true;
	}

	this.allowResize = dax_grid_allowResize;
	
	// set alternate style rows
	// start row 0 and endRow null do entire grid
	// start null and end null just affect defaults
	function dax_grid_setAltRowStyle(startRow, endRow, useAlt, useAsDefault) {
		if (useAsDefault)
			this.gridProperties.useAlternateRowColor = useAlt;
			
		if ((startRow) || (startRow == 0)) {
			if (!endRow)
				endRow = this.rows.length - 1;
				
			for (var rowCount = startRow; rowCount <= endRow; rowCount++) {
				this.rows[rowCount].useAltStyle = useAlt;
				this.refreshStyles(rowCount, null);
			}		
		}
	}

	this.setAltRowStyle = dax_grid_setAltRowStyle;

	// set header rows
	function dax_grid_setHeaderRows(startRow, endRow, isHeader) {
		if ((startRow) || (startRow == 0)) {
			if (!endRow)
				if (endRow != 0)
					endRow = this.rows.length - 1;
				
			for (var rowCount = startRow; rowCount <= endRow; rowCount++) {
				this.rows[rowCount].isHeader = isHeader;
				this.refreshStyles(rowCount, null);
			}		
		}
		this.primeGrid();
		this.redraw(true);
	}
	
	this.setHeaderRows = dax_grid_setHeaderRows;
	
	// set row as a block
	function dax_grid_setBlockRow(rowNumber, isBlock, rowHeight) {
	
		// update row height if entered
		if (rowHeight)
			this.setRowHeight(rowNumber, rowHeight);
			
		if (isBlock) {
			if (this.cols.length > this.gridProperties.lockedLeft) {
				var blockCellWidth = this.cols[this.gridProperties.lockedLeft].colWidth;
				// resize all cells except block one inside viewport to 0
				for (var cellCount = this.gridProperties.lockedLeft + 1; cellCount < this.rows[rowNumber].cells.length; cellCount++){
					this.rows[rowNumber].cells[cellCount].style.display = 'none';
					blockCellWidth += this.cols[cellCount].colWidth;
				}
				this.setCellWidth(this.rows[rowNumber].cells[this.gridProperties.lockedLeft], blockCellWidth);
			}
		} else {
			if (this.rows[rowNumber].isBlock) {
				for (var cellCount = this.gridProperties.lockedLeft; cellCount < this.rows[rowNumber].cells.length; cellCount++){
					this.rows[rowNumber].cells[cellCount].style.display = 'block';
					this.setCellWidth(this.rows[rowNumber].cells[cellCount], this.cols[cellCount].colWidth);
				}
			}		
		}
		
		// declare row block type
		this.rows[rowNumber].isBlock = isBlock;	
		this.redraw();
	}
	
	this.setBlockRow = dax_grid_setBlockRow;
	
	function dax_grid_setSelected(row, column, isSelected) {

		// apply inverse value if isSelected is not specified
		if ((!isSelected) && (isSelected != false)) {
			if (((!row) && (row != 0)) && ((!column) && (column != 0))){
				// entire grid
				// var isSelected = true;
			} else if ((!row) && (row != 0)) {
				// column
				var isSelected = !(this.cols[column].isSelected);
			} else if ((!column) && (column != 0)) {
				// row
				var isSelected = !(this.rows[row].isSelected);
			} else {
				// cell
				var isSelected = !(this.rows[row].cells[column].isSelected);
			}
		}

		if (!this.gridProperties.useMultipleSelection) {
		
			// deselect all
			for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {

				// rows
				if (this.rows[rowCount].isSelected) {
					this.rows[rowCount].isSelected = false;
					this.refreshStyles(rowCount, null);				
				}
				
				// cells
				for (var cellCount = 0; cellCount < this.rows[rowCount].cells.length; cellCount++) {

					var currentCell = this.getCell(rowCount, cellCount);
				
					if (currentCell.isSelected) {
						currentCell.isSelected = false;
						this.refreshStyles(rowCount, cellCount);				
					}
				}
			}

			// columns
			for (var colCount = 0; colCount < this.cols.length; colCount++) {

				if (this.cols[colCount].isSelected){
					this.cols[colCount].isSelected = false;
					this.refreshStyles(null, colCount);
				}			
			}
		}
				
		if (((!row) && (row != 0)) && ((!column) && (column != 0))) {
			// entire grid
		} else if ((!row) && (row != 0)) {
			// column
			this.cols[column].isSelected = isSelected;
		} else if ((!column) && (column != 0)) {
			// row
			this.rows[row].isSelected = isSelected;
		} else {
			// cell
			this.rows[row].cells[column].isSelected = isSelected;
		}
		
		this.refreshStyles(row, column);
	}
	
	this.setSelected = dax_grid_setSelected;
	
	function dax_grid_allowSelection(row, column, allowSelection, setAsDefault) {
		
		if (!row) {
			// column
			if (column != -1)
				this.cols[column].isSelectable = allowSelection;
			
			if (setAsDefault)
				this.gridProperties.allowColumnSelection = allowSelection;
			
		} else if (!column) {
			// row
			if (setAsDefault)
				this.gridProperties.allowRowSelection = allowSelection;

			if (row != -1)
				this.rows[row].isSelectable = allowSelection;
		} else {
			// cell
			if ((row != -1) && (column != -1))
				this.rows[row].cells[column].isSelectable = allowSelection;
			
			if (setAsDefault)
				this.gridProperties.allowCellSelection = allowSelection;
		}
	}
	
	this.allowSelection = dax_grid_allowSelection;
	
	function dax_grid_allowMultipleSelection(allowMultiSelection) {
	
		this.gridProperties.useMultipleSelection = allowMultiSelection;
	
	}
	
	this.allowMultipleSelection = dax_grid_allowMultipleSelection;
	
	function dax_grid_getSelected() {
	
		var selectedCells = [ ];
		
		// check for block rows
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			if (!this.rows[rowCount].isBlock)
				var lastCell = this.rows[rowCount].cells.length;
			else
				var lastCell = 1;

			for (var cellCount = 0; cellCount < lastCell; cellCount++) {
				var currentCell = this.rows[rowCount].cells[cellCount];
				if ((currentCell.isSelected) || (this.rows[rowCount].isSelected) || (this.cols[cellCount].isSelected)) {
					if ((!(this.rows[rowCount].isHeader)) && (!(this.cols[cellCount].isHeader)))
						selectedCells.push(currentCell);				
				}						
			}
		}

		return selectedCells;	
	}
	
	this.getSelected = dax_grid_getSelected;
	
	// change number of locked rows and columns
	// NOT FINISHED
	function dax_grid_setLocked(rows, columns){
	
		if ((rows) || (rows == 0)) {
			// move rows around if needed
			if (this.gridProperties.lockedTop < rows) {
				// check if we need to move more rows to locked section
				for (var rowCount = this.gridProperties.lockedTop; rowCount < (rows - this.gridProperties.lockedTop); rowCount++) {
					if (this.rows.length > rowCount) {
						// move rows
						this.topLeftContainer.appendChild(this.rows[rowCount].lockedDom);		
						this.topContainer.appendChild(this.rows[rowCount].dom);		
					}
				}				
			} else if (this.gridProperties.lockedTop > rows) {
				// check if we need to move more rows to locked section
				for (var rowCount = this.gridProperties.lockedTop; rowCount < rows; rowCount--) {
					if (this.rows.length > rowCount + 1) {
						// move rows
						this.leftContainer.insertBefore(this.rows[rowCount].lockedDom, this.rows[rowCount + 1].lockedDom);		
						this.gridContainer.insertBefore(this.rows[rowCount].dom, this.rows[rowCount + 1].dom);		
					} else if (this.rows.length > rowCount) {
						this.leftContainer.appendChild(this.rows[rowCount].lockedDom);		
						// move rows
						this.gridContainer.appendChild(this.rows[rowCount].dom);		
					}
				}				
			}
			this.gridProperties.lockedTop == rows;
		}
	
		if ((columns) || (columns == 0)) {
			// move cells around if needed
			if (this.gridProperties.lockedLeft < columns) {
				// move cells to locked row
				for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
					for (var cellCount = this.gridProperties.lockedLeft; cellCount < columns; cellCount++) {
						if (this.rows[rowCount].cells.length > cellCount) {
							this.rows[rowCount].lockedDom.appendChild(this.rows[rowCount].cells[cellCount]);
						}
					}
				}
			} else if (this.gridProperties.lockedLeft < columns) {
			
			}
		
			this.gridProperties.lockedLeft == rows;
		}
		
		// refresh block visibility
		var displayCorner = false;			// display upper left corner if either element is locked
		if (this.gridProperties.lockedLeft > 0) {
			displayCorner = true;
			this.leftContainer.style.display = 'block';
		} else {
			this.leftContainer.style.display = 'none';
		}
			
		if (this.gridProperties.lockedTop > 0) {
			displayCorner = true;		
			this.topContainer.style.display = 'block';
		} else {
			this.topContainer.style.display = 'none';
		}
		
		if (displayCorner) {
			this.topLeftContainer.style.display = 'block';	
		} else {
			this.topLeftContainer.style.display = 'none';	
		}
		

		
	}
	
	this.setLocked = dax_grid_setLocked;
	
	
	function dax_grid_size(rows, cols) {
	
		this.dimensions.columns = cols;
		this.dimensions.rows = rows;
	
		// add rows if needed
		while(this.dimensions.rows > this.rows.length) {
			this.addRow();
		}	
			
		// remove rows if needed
		while(this.dimensions.rows < this.rows.length) {
			//this.removeRow();
			this.removeRowInternal();
		}

		// update number of cells for existing rows
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			if (this.rows[rowCount].cells.length < this.dimensions.columns){
				// add cells
				while (this.dimensions.columns > this.rows[rowCount].cells.length) {
					this.rows[rowCount].initStyle = true;
					this.addCell(rowCount);
				}
			} else if (this.rows[rowCount].cells.length > this.dimensions.columns){
				// remove cells
				while (this.dimensions.columns < this.rows[rowCount].cells.length) {
					this.removeCell(rowCount);
				}				
			}
		}
		
		this.primeGrid();
		this.refreshStyles();
		this.redraw(true, true);
	};
	
	this.size = dax_grid_size;
	
	// outside call to insert a row
	function dax_grid_insertRow(rowNumber) {

		if (rowNumber < this.gridProperties.lockedTop)
			return false;
	
		this.addRow(rowNumber);

		this.dimensions.rows++;
		
		this.primeGrid();
		this.refreshStyles();
		this.redraw(true, true);
	}
	
	this.insertRow = dax_grid_insertRow;
	
	function dax_grid_addRow(rowNumber) {
	
		if (!rowNumber)
			rowNumber = this.rows.length;
	
		var newRow = this.nodes.row.cloneNode(true);
		var newLeftRow = this.nodes.row.cloneNode(true);
		var newRightRow = this.nodes.row.cloneNode(true);
		
		generateUniqueId(newRow);
		generateUniqueId(newLeftRow);
		generateUniqueId(newRightRow);
		
		var rowObject = {
			leftDom			: newLeftRow,
			rightDom		: newRightRow,
			dom				: newRow,
			cells			: [ ],
			styleName		: this.defaultStyle.row,
			styleNameAlt	: this.defaultStyle.rowAlt,
			selectedStyleName	: this.defaultSelectedStyle.row,
			internalStyleName	: this.defaultInternalStyle.row,
			initStyle		: true,
			isHeader		: (rowNumber < this.gridProperties.headerRows) ? true : false,
			rowHeight		: 1,		// in multiples of row height
			useAltStyle		: this.gridProperties.useAlternateRowColor,		// use alt row style
			isBlock			: false,										// consists of only one cell
			isSelectable	: this.gridProperties.allowRowSelection,
			isSelected		: false,
			type			: null		// header, footer, default
		}

		newRow.style.display = 'block';
		newLeftRow.style.display = 'block';
		newRightRow.style.display = 'block';
		
		// and to dom
		if (this.gridProperties.lockedTop > rowNumber) {
			// row goes to top locked section
			rowObject.type = 'header';
			if (this.rows.length > rowNumber) {
				this.topContainer.insertBefore(newRow, this.rows[rowNumber].dom);
				this.topLeftContainer.insertBefore(newLockedRow, this.rows[rowNumber].leftDom);
				this.topRightContainer.insertBefore(newLockedRow, this.rows[rowNumber].rightDom);
			} else {
				this.topContainer.appendChild(newRow);
				this.topLeftContainer.appendChild(newLeftRow);
				this.topRightContainer.appendChild(newRightRow);
			}

			this.dimensions.lockedTop++;

		} else if ((this.gridProperties.lockedBottom > 0) && ((rowNumber + 1)> (this.dimensions.rows - this.gridProperties.lockedBottom)) && (this.gridProperties.lockedBottom > this.dimensions.lockedBottom)){
			// row goes to bottom locked section
			rowObject.type = 'footer';
			if (this.rows.length > rowNumber) {
				this.bottomContainer.insertBefore(newRow, this.rows[rowNumber].dom);
				this.bottomLeftContainer.insertBefore(newLockedRow, this.rows[rowNumber].leftDom);
				this.bottomRightContainer.insertBefore(newLockedRow, this.rows[rowNumber].rightDom);
			} else {
				this.bottomContainer.appendChild(newRow);
				this.bottomLeftContainer.appendChild(newLeftRow);
				this.bottomRightContainer.appendChild(newRightRow);
			}
			
			this.dimensions.lockedBottom++;
			
		} else if (this.rows.length > rowNumber) {
			// add row to the unlocked (viewport) section
			rowObject.type = 'default';
			this.gridContainer.insertBefore(newRow, this.rows[rowNumber].dom);
			this.leftContainer.insertBefore(newLeftRow, this.rows[rowNumber].leftDom);
			this.rightContainer.insertBefore(newRightRow, this.rows[rowNumber].rightDom);
		} else {
			rowObject.type = 'default';
			// add row to the end of unlocked section
			this.gridContainer.appendChild(newRow);
			this.leftContainer.appendChild(newLeftRow);
			this.rightContainer.appendChild(newRightRow);
		}

		// add row to array
		this.rows.splice(rowNumber, 0, rowObject); 
		
		// add cells
		while (this.dimensions.columns > this.rows[rowNumber].cells.length) {
			this.rows[rowNumber].initStyle = true;
			this.addCell(rowNumber);
		}
		
		// remove cells
		while (this.dimensions.columns < this.rows[rowNumber].cells.length) {
			this.removeCell(rowNumber);
		}

		return;
	}
	
	this.addRow = dax_grid_addRow;

	function dax_grid_removeRowInternal(rowNumber) {

		if ((!rowNumber) && (rowNumber != 0))
			rowNumber = this.rows.length - 1;

		for (var cellCount = 0; cellCount < this.rows[rowNumber].cells.length; cellCount++) {
			this.removeCell(rowNumber, cellCount);
		}
		if (rowNumber < this.gridProperties.lockedTop) {
			this.topLeftContainer.removeChild(this.rows[rowNumber].leftDom);
			this.topContainer.removeChild(this.rows[rowNumber].dom);
			this.topRightContainer.removeChild(this.rows[rowNumber].rightDom);

			this.dimensions.lockedTop--;

		} else if ((this.gridProperties.lockedBottom > 0) && ((rowNumber + 1) > (this.dimensions.rows - this.gridProperties.lockedBottom))) {
			this.bottomLeftContainer.removeChild(this.rows[rowNumber].leftDom);
			this.bottomContainer.removeChild(this.rows[rowNumber].dom);
			this.bottomRightContainer.removeChild(this.rows[rowNumber].rightDom);
			
			this.dimensions.lockedBottom--;
			
		} else {
			this.leftContainer.removeChild(this.rows[rowNumber].leftDom);
			this.gridContainer.removeChild(this.rows[rowNumber].dom);
			this.rightContainer.removeChild(this.rows[rowNumber].rightDom);
		}
		this.rows[rowNumber].dom = null;
		this.rows[rowNumber].lockedDom = null;
		this.rows.splice(rowNumber, 1);
	}
	
	this.removeRowInternal = dax_grid_removeRowInternal;
	
	// outside call to remove a row
	function dax_grid_removeRow(rowNumber) {

		if ((!rowNumber) && (rowNumber != 0))
			rowNumber = this.rows.length - 1;
		
		if (rowNumber < 0)
			return false;		
	
		this.removeRowInternal(rowNumber);

		this.dimensions.rows--;
		
		this.primeGrid();
		this.refreshStyles();
		this.redraw(true);
	}
	
	this.removeRow = dax_grid_removeRow;
	
	// internal functio to remove a column
	function dax_grid_removeColumnInternal(colNumber) {
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			// don't remove cell if row is block and it's a first unlocked cell
			if ((this.rows[rowCount].isBlock) && (colNumber >= this.gridProperties.lockedLeft)) {
				// is block and cell that shouldn't be removed
		
			} else {
				this.removeCell(rowCount, colNumber);
				this.rows[rowCount].cells.splice(colNumber, 1);
			}
		}
		this.cols[colNumber].cells = null;
		this.cols.splice(colNumber, 1);
	}
	
	this.removeColumnInternal = dax_grid_removeColumnInternal;
	
	// outside call to remove a column
	function dax_grid_removeColumn(colNumber){
	
		if ((!colNumber) && (colNumber != 0))
			colNumber = this.cols.length - 1;	
	
		this.removeColumnInternal(colNumber);

		this.dimensions.columns--;
		
		this.primeGrid();
		this.refreshStyles();
		this.redraw(true, true);	
	}
	
	this.removeColumn = dax_grid_removeColumn;
	
	function dax_grid_addColumnInternal(columnNumber) {
		var newColumn = {
			cells 			: [ ],
			styleName		: this.defaultStyle.column,
			selectedStyleName		: this.defaultSelectedStyle.column,
			internalStyleName		: this.defaultInternalStyle.column,
			colWidth		: null,
			colType			: (columnNumber < this.gridProperties.headerColumns) ? true : false,
			isResizable		: true,
			isSelectable	: this.gridProperties.allowColumnSelection,
			isSelected		: false,
			visible			: true
		}

		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			this.addCell(rowCount, columnNumber);
		}
		
		this.cols.splice(columnNumber, 0, newColumn);
	}
	
	this.addColumnInternal = dax_grid_addColumnInternal;

	function dax_grid_addColumn(colNumber){
	
		if (colNumber < this.gridProperties.lockedLeft)
			return false;
	
		if ((!colNumber) && (colNumber != 0))
			colNumber = this.cols.length;	
	
		this.addColumnInternal(colNumber);

		this.dimensions.columns++;
		
		this.primeGrid();
		this.refreshStyles();
		this.redraw(true, true);	
	}
	
	this.addColumn = dax_grid_addColumn;
	
	function dax_grid_addCell(rowNumber, cellPosition) {
		if ((!cellPosition) && (cellPosition != 0))
			cellPosition = this.rows[rowNumber].cells.length;
		var newCell = this.nodes.cell.cloneNode(true);
		generateUniqueId(newCell);
		if (this.rows[rowNumber].isBlock)
			newCell.style.display = 'none';
		else
			newCell.style.display = 'block';
		// and to dom		
		if (cellPosition < this.gridProperties.lockedLeft) {
			// locked left area, here we just add cells
			this.rows[rowNumber].leftDom.appendChild(newCell);	
			
			this.dimensions.lockedLeft++;
			
		} else if ((this.gridProperties.lockedRight > 0) && ((cellPosition + 1) > (this.dimensions.columns - this.gridProperties.lockedRight))) {
			// locked right area, here we just add cells
			this.rows[rowNumber].rightDom.appendChild(newCell);		

			this.dimensions.lockedRight++;

		} else {
			// unlocked row area
			if (cellPosition < this.rows[rowNumber].cells.length) {
				// insert cell
				this.rows[rowNumber].dom.insertBefore(newCell, this.rows[rowNumber].cells[cellPosition]);	
			} else {
				// cell goes at the end
				this.rows[rowNumber].dom.appendChild(newCell);	
			}		
		}

		// add cell to array
		this.rows[rowNumber].cells.splice(cellPosition, 0, newCell);
		
		newCell.styleName = this.defaultStyle.cell;
		newCell.selectedStyleName = this.defaultSelectedStyle.cell;
		newCell.internalStyleName = this.defaultInternalStyle.cell;
		newCell.parent = this;
		newCell.cellType = 'text';
		newCell.value = null;
		newCell.selectable = this.gridProperties.allowCellSelection;
		newCell.selected = false;
	}
	
	this.addCell = dax_grid_addCell;	
	
	function dax_grid_removeCell(rowNumber, cellNumber){
		this.rows[rowNumber].cells[cellNumber].grid = null;
		this.rows[rowNumber].cells[cellNumber].parent = null;
		if (cellNumber < this.gridProperties.lockedLeft) {
			this.rows[rowNumber].leftDom.removeChild(this.rows[rowNumber].cells[cellNumber]);
			this.dimensions.lockedLeft--;
		} else if ((this.gridProperties.lockedRight > 0) && ((cellPosition + 1) > (this.dimensions.columns - this.gridProperties.lockedRight))) {
			this.rows[rowNumber].rightDom.removeChild(this.rows[rowNumber].cells[cellNumber]);
			this.dimensions.lockedRight--;
		} else {
			this.rows[rowNumber].dom.removeChild(this.rows[rowNumber].cells[cellNumber]);
		}	
	}
	
	this.removeCell = dax_grid_removeCell;
	
	// for internal use
	function dax_grid_getCellWidth(cellRef) {
	
		return cellRef.offsetWidth;
	
	}
	
	function dax_grid_getCellHeight(cellRef) {
	
		return cellRef.offsetHeight;
	
	}
	
	function dax_grid_setCellWidth(cellRef, width) {
		
		var outputString = 'Cell (' + cellRef.row + ',' + cellRef.column + '):';

		var padding = parseInt(getStyle(cellRef,'padding-left', 'paddingLeft'),10) + parseInt(getStyle(cellRef,'padding-right', 'paddingRight'),10);
		var margin = parseInt(getStyle(cellRef,'margin-left', 'marginLeft'),10) + parseInt(getStyle(cellRef,'margin-right', 'marginLeft'),10);
		var border = ((getStyle(cellRef,'border-left-style', 'borderLeftStyle') != 'none') ? parseInt(getStyle(cellRef,'border-left-width', 'borderLeftWidth'),10) : 0) + 
			((getStyle(cellRef,'border-right-style', 'borderRightStyle') != 'none') ? parseInt(getStyle(cellRef,'border-right-width', 'borderRightWidth'),10) : 0);
		
		// for IE7
		if (!padding)
			padding = 0;
		if (!margin)
			margin = 0;
		if (!border)
			border = 0;
			
		width = width - padding - margin - border;

		outputString += 'pad:' + padding + ' mar:' + margin + ' bor:' + border;
		
		if (width >= 0)
			cellRef.style.width = width + 'px';

		outputString +=' -> width:' + width + ' real:' + cellRef.offsetWidth + ' diff' + (cellRef.offsetWidth - width - padding - margin - border);
		
		outputString += '<br />';
		
		debugAlert(outputString);

			
	}

	function dax_grid_setCellHeight(cellRef, height) {

		var padding = parseInt(getStyle(cellRef,'padding-top', 'paddingTop'),10) + parseInt(getStyle(cellRef,'padding-bottom', 'paddingBottom'),10);
		var margin = parseInt(getStyle(cellRef,'margin-top', 'marginTop'),10) + parseInt(getStyle(cellRef,'margin-bottom', 'marginBottom'),10);
		var border = ((getStyle(cellRef,'border-top-style', 'borderTopStyle') != 'none') ? parseInt(getStyle(cellRef,'border-top-width', 'borderTopWidth'),10) : 0 )+ 
			((getStyle(cellRef,'border-bottom-style', 'borderBottomStyle') != 'none') ? parseInt(getStyle(cellRef,'border-bottom-width', 'borderBottomWidth'),10) : 0);
		
		// for IE7
		if (!padding)
			padding = 0;
		if (!margin)
			margin = 0;
		if (!border)
			border = 0;
			
		if (height >= 0)
			height = height - padding - margin - border;
	
		cellRef.style.height = height + 'px';
	
	}
	
	// again, internal use onlyaz
	this.getCellWidth = dax_grid_getCellWidth;
	this.getCellHeight = dax_grid_getCellHeight;
	this.setCellWidth = dax_grid_setCellWidth;
	this.setCellHeight = dax_grid_setCellHeight;

	function dax_grid_cell_onmousedown(event) {

		var clickEvent = xpEvent(event);
	
		var abortDefault = this.parent.onCellClick(this.row, this.column, clickEvent);

		if (abortDefault == false)
			return false;
		
		if (this.selectable) 
			this.parent.setSelected(this.row, this.column);
	}

	function dax_grid_cell_ondblclick() {

		var abortDefault = this.parent.onCellDblClick(this.row, this.column);

		if (abortDefault == false)
			return false;
		
	}	
	
	// optional parameters select which additional redraw functions are performed, the less the quicker
	function dax_grid_redraw(updateRowHeights, updateColumnWidths) {

		if (this.gridProperties.disableVisualRefresh)
			return;

		// since viewport scrollbars are 'forced' outside viewport, we need to do some manual work here instead of viewport
		// check if scrollbar should be displayed
		var showHorScrollbar = (this.viewport.dom.contents.offsetWidth > this.viewport.dom.main.offsetWidth) ? true : false;
		var showVerScrollbar = (this.viewport.scrollbars.ver.visibleHeight < this.viewport.scrollbars.ver.height) ? true : false;
			
		// modify horizonal and vertical viewport size based on scrollbar visibility	
		var horViewportModifier = showVerScrollbar ? -17 : 0;
		var verViewportModifier = showHorScrollbar ? -17 : 0;
		
		// show and hide appropriate sections
		this.topLeftViewport.style.display = ((this.gridProperties.lockedTop > 0) && (this.gridProperties.lockedLeft > 0)) ? 'block' : 'none';
		this.topViewport.style.display = (this.gridProperties.lockedTop > 0) ? 'block' : 'none';
		this.topRightViewport.style.display = ((this.gridProperties.lockedTop > 0) && (this.gridProperties.lockedRight > 0)) ? 'block' : 'none';

		this.leftViewport.style.display = (this.gridProperties.lockedLeft > 0) ? 'block' : 'none';
		this.rightViewport.style.display = (this.gridProperties.lockedRight > 0) ? 'block' : 'none';

		this.bottomLeftViewport.style.display = ((this.gridProperties.lockedBottom > 0) && (this.gridProperties.lockedLeft > 0)) ? 'block' : 'none';
		this.bottomViewport.style.display = (this.gridProperties.lockedBottom > 0) ? 'block' : 'none';
		this.bottomRightViewport.style.display = ((this.gridProperties.lockedBottom > 0) && (this.gridProperties.lockedRight > 0)) ? 'block' : 'none';
	
		this.viewportContainer.style.display = 'block';
		
		// doesn't include locked columns
		var gridLeftWidth = 0;
		var gridContainerWidth = 0;
		var gridRightWidth = 0;

		for (var colCount = 0; colCount < this.cols.length; colCount++) {
			if (colCount < this.gridProperties.lockedLeft)
				gridLeftWidth += (this.rows[0]) ? (this.getCellWidth(this.rows[0].cells[colCount])) : 0;
			else if ((this.gridProperties.lockedRight > 0) && ((colCount + 1) > (this.dimensions.columns - this.gridProperties.lockedRight)))
				gridRightWidth += (this.rows[0]) ? (this.getCellWidth(this.rows[0].cells[colCount])) : 0;
			else
				gridContainerWidth += (this.rows[0]) ? (this.getCellWidth(this.rows[0].cells[colCount])) : 0;
			
			if (updateColumnWidths)			
				this.setColumnWidthInternal(colCount, this.cols[colCount].colWidth);
		}
		
		// update grid width
		var targetNodeOffsetWidth = this.targetNode.offsetWidth;
		
		this.gridContainer.style.width = gridContainerWidth + 'px';

		this.topLeftContainer.style.width = gridLeftWidth + 'px';		
		this.topContainer.style.width = gridContainerWidth + 'px';
		this.topRightContainer.style.width = gridRightWidth + 'px';

		this.leftContainer.style.width = gridLeftWidth + 'px';
		this.rightContainer.style.width = gridRightWidth + 'px';
	
		this.bottomLeftContainer.style.width = gridLeftWidth + 'px';		
		this.bottomContainer.style.width = gridContainerWidth + 'px';
		this.bottomRightContainer.style.width = gridRightWidth + 'px';
		
		var viewportWidth = Math.max(targetNodeOffsetWidth - (gridLeftWidth + gridRightWidth) + horViewportModifier, 0)
		
		this.viewportContainer.style.width = viewportWidth + 'px';		
		this.topViewport.style.width = viewportWidth + 'px';
		this.bottomViewport.style.width = viewportWidth + 'px';
		
		// if horizontal scrollbar is visible, position the div container
		if (showHorScrollbar) {
			this.bottomScrollbar.style.display = 'block';
			this.bottomScrollbar.style.left = gridLeftWidth + 'px';
		} else {
			this.bottomScrollbar.style.display = 'none';
		}
		
		// doesn't include locked rows (and header)
		var gridTopHeight = 0;
		var gridContainerHeight = 0;
		var gridBottomHeight = 0;
		
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			if (rowCount < this.gridProperties.lockedTop)
				gridTopHeight += this.rows[rowCount].rowHeight * this.gridProperties.defaultRowHeight;
			else if ((this.gridProperties.lockedBottom > 0) && ((rowCount + 1) > (this.dimensions.rows - this.gridProperties.lockedBottom)))
				gridBottomHeight += this.rows[rowCount].rowHeight * this.gridProperties.defaultRowHeight;
			else
				gridContainerHeight += this.rows[rowCount].rowHeight * this.gridProperties.defaultRowHeight;

			if (updateRowHeights)
				this.setRowHeight(rowCount, this.rows[rowCount].rowHeight);
		}

		// set viewport height
		if (this.targetNode.offsetHeight - (gridTopHeight + gridBottomHeight) < 0) {
			// no space for viewport
			var viewportHeight = 0;
		} else if (((gridTopHeight + gridBottomHeight) + gridContainerHeight > this.targetNode.offsetHeight) || (!this.gridProperties.resizeViewportToGrid)){
			// grid space is larger than target node, viewport is tall as target node
			var viewportHeight = this.targetNode.offsetHeight - (gridTopHeight + gridBottomHeight);
		} else {
			// grid space is smaller than target node, viewport is tall as grid
			var viewportHeight = (gridTopHeight + gridBottomHeight) + gridContainerHeight;
		}

		// update grid height
		var targetNodeOffsetHeight = this.targetNode.offsetHeight;

		this.topLeftContainer.style.height = gridTopHeight + 'px';	
		this.topContainer.style.height = gridTopHeight + 'px';
		this.topRightContainer.style.height = gridTopHeight + 'px';	

		this.leftContainer.style.height = viewportHeight + 'px';
		this.rightContainer.style.height = viewportHeight + 'px';
		
		this.bottomContainer.style.height = gridBottomHeight + 'px';
		this.bottomLeftContainer.style.height = gridBottomHeight + 'px';	
		this.bottomRightContainer.style.height = gridBottomHeight + 'px';	
		
		this.viewportContainer.style.height = viewportHeight + verViewportModifier + 'px';
		this.gridContainer.style.height = gridContainerHeight + 'px';
		
		var viewportHeight = Math.max(targetNodeOffsetHeight - (gridBottomHeight + gridTopHeight) + verViewportModifier, 0);
		
		this.leftViewport.style.height = viewportHeight + 'px';
		this.rightViewport.style.height = viewportHeight + 'px';

		// if horizontal scrollbar is visible, position the div container
		if (showVerScrollbar) {
			this.rightScrollbar.style.display = 'block';
			this.rightScrollbar.style.top = gridTopHeight + 'px';
		} else {
			this.rightScrollbar.style.display = 'none';
		}
		
		// redraw
		this.viewport.redraw();
		
		// update dimensions
		this.dimensions.width = (gridRightWidth + gridLeftWidth) + gridContainerWidth;
		this.dimensions.height = (gridTopHeight + gridBottomHeight) + gridContainerHeight;

		this.gridMainContainer.style.width = this.targetNode.offsetWidth + horViewportModifier + 'px';
		this.gridMainContainer.style.height = this.targetNode.offsetHeight + verViewportModifier + 'px';
		
		this.grid.style.width = this.targetNode.offsetWidth  + 'px';
		this.grid.style.height = this.targetNode.offsetHeight  +'px';	
	}
		
	this.redraw = dax_grid_redraw;

	// column events
	function dax_grid_column_setWidthInternal(colNumber, width){
		this.cols[colNumber].colWidth = width;
		for (var cellCount = 0; cellCount < this.cols[colNumber].cells.length; cellCount++) {
			if (this.rows[cellCount].isBlock) {
				// width for block rows
				var totalViewportWidth = 0;
				var doNormalResize = false;
				if (this.gridProperties.lockedLeft > colNumber) {
					// column belongs to locked section on the left side, normal resize is fine
					doNormalResize = true;
				} else if (((this.dimensions.columns - this.gridProperties.lockedRight) < (colNumber + 1)) && (this.gridProperties.lockedRight > 0)){	
					// column belongs to locked section on the right side, normal resize is fine
					doNormalResize = true;
				} else {
					// don't have total width yet, calculate it
					if (!totalViewportWidth) {
						var blockCellWidth = this.cols[this.gridProperties.lockedLeft].colWidth;
						// resize all cells except block one inside viewport to 0
						for (var cellCountA = this.gridProperties.lockedLeft + 1; cellCountA < this.cols.length; cellCountA++){
							blockCellWidth += this.cols[cellCountA].colWidth;
						}
						totalViewportWidth = blockCellWidth;
					}
					this.setCellWidth(this.rows[cellCount].cells[this.gridProperties.lockedLeft], totalViewportWidth);
				}
			} else {
				doNormalResize = true;
			}
			
			if (doNormalResize) {
				var currentCell = this.cols[colNumber].cells[cellCount];
				this.setCellWidth(currentCell, width);
				currentCell.width = this.getCellWidth(currentCell);	

				// resize contents based on type
				switch (currentCell.cellType) {
					case 'html':
					break;
					case 'checkbox':
						// should center automatically
					break;
					case 'input':
						if (currentCell.contentObject) {
							var inputObj = currentCell.contentObject;
							inputObj.style.width = this.getCellWidth(currentCell) - 10 + 'px'; // compensate for padding
						}							
					break;
					case 'dropdown':
						if (currentCell.contentObject) {
							var selectObj = currentCell.contentObject;
							selectObj.style.width = this.getCellWidth(currentCell) - 10 + 'px'; // compensate for padding
						}							
					break;
					case 'image':
					default:
					break;
				}			
			}
		}
	}
	
	this.setColumnWidthInternal = dax_grid_column_setWidthInternal;

	function dax_grid_setColumnWidth(columnNumber, width) {
	
		this.setColumnWidthInternal(columnNumber, width);
		this.redraw();
	}
	
	this.setColumnWidth = dax_grid_setColumnWidth;
	
	function dax_grid_getColumnWidth(columnNumber) {
	
		return this.cols[columnNumber].colWidth;
	
	}
	
	this.getColumnWidth = dax_grid_getColumnWidth;
		
	// row events
	// height is set in row multiples (1, 2, 3...n)
	function dax_grid_row_setHeight(rowNumber, height){
		for (var cellCount = 0; cellCount < this.rows[rowNumber].cells.length; cellCount++) {
			var currentCell = this.rows[rowNumber].cells[cellCount];
			this.rows[rowNumber].rowHeight = height;
			this.setCellHeight(currentCell, height * this.gridProperties.defaultRowHeight);
			currentCell.height = this.getCellHeight(currentCell);	

			// resize contents based on type
			switch (currentCell.cellType) {
				case 'html':
				break;
				case 'checkbox':
				break;
				case 'input':
				break;
				case 'dropdown':
				break;
				case 'image':
				break;
				default:
				break;
			}
		}
	}
	
	this.setRowHeight = dax_grid_row_setHeight;
	
	function dax_grid_getRowHeight(rowNumber) {
		return this.rows[rowNumber].rowHeight;
	}
	
	this.getRowHeight = dax_grid_getRowHeight;
	
	// return cell reference
	function dax_grid_getCell(rowNumber, colNumber) {
		if (this.rows[rowNumber].isBlock) {
			// if row is block
			if (colNumber < this.gridProperties.lockedLeft)
				return this.rows[rowNumber].cells[colNumber];
			else if ((this.gridProperties.lockedRight > 0) && ((colNumber + 1) > (this.dimensions.columns - this.gridProperties.lockedRight)))
				return this.rows[rowNumber].cells[colNumber];
			else
				return this.rows[rowNumber].cells[this.gridProperties.lockedLeft];
		} else
			return this.rows[rowNumber].cells[colNumber];
	}
	
	this.getCell = dax_grid_getCell;
	
	
	// header events
	function dax_grid_row_header_onmousedown() {

		if (this.parent.dragging.headerMode == 'resize') {
			if (this.parent.gridProperties.allowRowResize) {
		
				dax_bridge.dragObject = this.parent;
				this.parent.dragging.rowNumber = this.row;
			
				// resizing row
				if (document.addEventListener) {
					document.addEventListener("mousemove", dax_grid_header_rowResize, true);
					document.addEventListener("mouseup",  dax_grid_header_rowResizeStop, true);
				} else { // IE
					document.onmousemove = dax_grid_header_rowResize;
					document.onmouseup   = dax_grid_header_rowResizeStop;
				}	
			}
		} else {
			var abortDefault = this.parent.onRowClick(this.row, this.column);
			
			if (abortDefault == false)
				return false;

			this.parent.setSelected(this.row);
		}
	}
	
	function dax_grid_row_header_onmousemove(aEvent) {

		if (this.parent.gridProperties.allowRowResize) {
			var event = xpEvent(aEvent);
			var coord = getMousePosition(event);
			var gPos = getObjectPosition(this);
	
			// get the local x coordinate
			var y = coord.y - gPos.y;
			var margin = 10;
			var style = this.style;
	
			if (!isDefined(this._originalCursor)) {
				this._originalCursor = style.cursor;
			}
	
			if (y < this.offsetHeight && y >= (this.offsetHeight - margin)) {
				style.cursor = "n-resize";
				this.parent.dragging.headerMode = 'resize';
			} else {
				style.cursor = this._originalCursor;
				this.parent.dragging.headerMode = 'move';
			}
		}
	}
	
	function dax_grid_header_rowResize(aEvent) {
	
		if (!dax_bridge.dragObject)
			return false;

		var event = xpEvent(aEvent, true);
		var grid = dax_bridge.dragObject;
		var headerCell = grid.rows[grid.dragging.rowNumber].cells[0];
		
		var mpos = getMousePosition(event);
		var pos = getObjectPosition(headerCell);

		// calculate the difference
		var dy = mpos.y - pos.y;

		// find the nearest height multiple
		var rowHeight = Math.round(dy / grid.gridProperties.defaultRowHeight);
		if (rowHeight < 1)
			rowHeight = 1;
		
		grid.setRowHeight(grid.dragging.rowNumber, rowHeight);
		grid.redraw();

		return false;
	}
	
	function dax_grid_header_rowResizeStop() {
		// Gecko
		if (document.removeEventListener) {
			document.removeEventListener("mousemove", dax_grid_header_rowResize, true);
			document.removeEventListener("mouseup",   dax_grid_header_rowResizeStop, true);
		} else {
			document.onmousemove = null;
	 		document.onmouseup   = null;
		}		
	}

	
	// header events
	function dax_grid_col_header_onmousedown() {

		dax_bridge.dragObject = this.parent;
		this.parent.dragging.columnNumber = this.column;
		this.parent.dragging.rowNumber = this.row;
		
		if (this.parent.dragging.headerMode == 'resize') {
		
			if ((this.parent.gridProperties.allowColumnResize) && (this.parent.cols[this.column].isResizable)){
				// resizing header
				if (document.addEventListener) {
					document.addEventListener("mousemove", dax_grid_header_columnResize, true);
					document.addEventListener("mouseup",  dax_grid_header_columnResizeStop, true);
				} else { // IE
					document.onmousemove = dax_grid_header_columnResize;
					document.onmouseup   = dax_grid_header_columnResizeStop;
				}
			}
		
		} else {

		
			// movable only if it's not locked
			if ((this.grid.gridProperties.lockedLeft <= this.column) && (this.parent.gridProperties.allowColumnSwap)) {
			
				// moving header
				this.parent.updateColumnCoordinates();
				
				if (document.addEventListener) {
					document.addEventListener("mousemove", dax_grid_header_columnMove, true);
					document.addEventListener("mouseup",  dax_grid_header_columnMoveStop, true);
				} else { // IE
					document.onmousemove = dax_grid_header_columnMove;
					document.onmouseup   = dax_grid_header_columnMoveStop;
				}	
			}

			var abortDefault = this.parent.onColumnClick(this.column);
			
			if (abortDefault == false)
				return false;

			if (this.parent.cols[this.column].isSelectable)
				this.parent.setSelected(null, this.column);

		}
	}
	
	function dax_grid_col_header_onmousemove(aEvent) {

		if (typeof(xpEvent) == 'undefined')
			return false;
	
		if ((this.parent.gridProperties.allowColumnResize) && (this.parent.cols[this.column].isResizable)) {

			var event = xpEvent(aEvent);
			var coord = getMousePosition(event);
			var gPos = getObjectPosition(this);
	
			// get the local x coordinate
			var x = coord.x - gPos.x;
			var margin = 10;
			var style = this.style;
	
			if (!isDefined(this._originalCursor)) {
				this._originalCursor = style.cursor;
			}
	
			if (x < this.offsetWidth && x >= (this.offsetWidth - margin)) {
				style.cursor = "e-resize";
				this.parent.dragging.headerMode = 'resize';
			} else {
				style.cursor = this._originalCursor;
				if (this.parent.gridProperties.allowColumnSwap)
					this.parent.dragging.headerMode = 'move';
				else
					this.parent.dragging.headerMode = 'nomove';
			}
		}
	}
	
	function dax_grid_header_columnResize(aEvent) {

		if (!dax_bridge.dragObject)
			return false;
			
		var event = xpEvent(aEvent, true);
		var grid = dax_bridge.dragObject;
		var headerCell = grid.cols[grid.dragging.columnNumber].cells[grid.dragging.rowNumber];
		
		var mpos = getMousePosition(event);
		var pos = getObjectPosition(headerCell);

		// calculate the difference
		var dx = mpos.x - pos.x;

		grid.setColumnWidthInternal(grid.dragging.columnNumber, dx);
//		grid.onMouseColumnResize(grid.dragging.columnNumber, dx);
		grid.redraw();

		return false;	
	}
	
	function dax_grid_header_columnResizeStop() {

		var grid = dax_bridge.dragObject;
		grid.onMouseColumnResizeStop(grid.dragging.columnNumber, grid.cols[grid.dragging.columnNumber].colWidth);
		
		// Gecko
		if (document.removeEventListener) {
			document.removeEventListener("mousemove", dax_grid_header_columnResize, true);
			document.removeEventListener("mouseup",   dax_grid_header_columnResizeStop, true);
		} else {
			document.onmousemove = null;
	 		document.onmouseup   = null;
		}		
	}
	
	function dax_grid_header_columnMove(event) {

		if (!dax_bridge.dragObject)
			return false;
			
		var grid = dax_bridge.dragObject;
		var mpos = getMousePosition(event);

		// find if dragging is needed
		var originalColumn = grid.dragging.columnNumber;
		var originalColumnX = grid.cols[originalColumn].posX;

		grid.swapOriginalColumn = originalColumn;
		grid.swapTargetColumn = originalColumn;
		
		if (mpos.x < grid.cols[0].posX + (grid.cols[0].colWidth / 2)) {
			// first column
			var targetColumn = 0;
		} else if (mpos.x > grid.cols[grid.cols.length - 1].posX + (grid.cols[grid.cols.length - 1].colWidth / 2)) {
			// last column
			var targetColumn = grid.cols.length - 1;
		} else {
			// find the column
			var targetColumn = originalColumn;
			// before the current column
			for (var colCount = 0; colCount < originalColumn; colCount++) {
				if ((mpos.x > grid.cols[colCount].posX - (grid.cols[colCount].colWidth / 2)) && (mpos.x < grid.cols[colCount + 1].posX - (grid.cols[colCount + 1].colWidth / 2))) {
					var targetColumn = colCount;
				}
			}
			// after the current column
			for (var colCount = originalColumn; colCount < (grid.cols.length - 1); colCount++) {
				if ((mpos.x > grid.cols[colCount].posX + (grid.cols[colCount].colWidth / 2)) && (mpos.x < grid.cols[colCount + 1].posX + (grid.cols[colCount + 1].colWidth / 2))) {
					var targetColumn = colCount;
				}
			}
		}
		
		// if numbers are not the same, update visual indicator
		if (originalColumn != targetColumn) {
			grid.swappingNeeded	= true;		
			grid.swapOriginalColumn = originalColumn;
			grid.swapTargetColumn = targetColumn;
			// update original column style
			if (!grid.swapOriginalColumnStyleSet) {
				grid.swapOriginalColumnStyleSet = originalColumn;
				// make current column semi transparent
				for (var rowCount = 0; rowCount < grid.cols[originalColumn].cells.length; rowCount++) {
				if (grid.cols[targetColumn].cells[rowCount].className.indexOf('dax_grid_dragging_column_origin') == -1)
					grid.cols[originalColumn].cells[rowCount].className += ' dax_grid_dragging_column_origin';
				}
			}
			// set indicator for target column
			if (grid.swapTargetColumnStyle != targetColumn) {
				if (grid.swapTargetColumnStyle) {
					grid.refreshStyles(null, grid.swapTargetColumnStyle);
				}
				grid.swapTargetColumnStyle = targetColumn;
				// make current column semi transparent
				for (var rowCount = 0; rowCount < grid.cols[targetColumn].cells.length; rowCount++) {
					if (grid.cols[targetColumn].cells[rowCount].className.indexOf('dax_grid_dragging_column_target') == -1)
						grid.cols[targetColumn].cells[rowCount].className += ' dax_grid_dragging_column_target';
				}
			grid.redraw();				
			}
		} else {
			grid.swappingNeeded	= false;				
		}
	
		return false;
	}

	function dax_grid_header_columnMoveStop() {

		// Gecko
		if (document.removeEventListener) {
			document.removeEventListener("mousemove", dax_grid_header_columnMove, true);
			document.removeEventListener("mouseup",   dax_grid_header_columnMoveStop, true);
		} else {
			document.onmousemove = null;
	 		document.onmouseup   = null;
		}	
	
		var grid = dax_bridge.dragObject;

		if (grid.swappingNeeded) {
			// remove visual styling
			if ((grid.swapTargetColumnStyle) || (grid.swapTargetColumnStyle == 0))
				grid.refreshStyles(null, grid.swapTargetColumnStyle);
			if ((grid.swapOriginalColumn) || (grid.swapOriginalColumn == 0))
				grid.refreshStyles(null, grid.swapOriginalColumn);
			if (grid.swappingNeeded) {
				grid.moveColumn(grid.swapOriginalColumn, grid.swapTargetColumn);
			}
		}

		grid.swappingNeeded	= false;	
		grid.swapTargetColumn = null;
		grid.swapOriginalColumn = null;
		grid.swapOriginalColumnStyleSet = false;
		grid.swapTargetColumnStyle = false;
		dax_bridge.dragObject = null;
		
	return false;
	}

	// return a footer row number, starting from 1 for footerNumber
	function dax_grid_getFooterRowNumber(footerNumber) {
		return (this.dimensions.rows - this.gridProperties.lockedBottom - 1 + footerNumber);		
	}
	
	this.getFooterRowNumber = dax_grid_getFooterRowNumber;

	// set number of footer rows
	function dax_grid_setFooterRows(numberOfRows) {
		this.gridProperties.lockedBottom = numberOfRows;
		this.onRowFooterChange();
	}
	
	this.setFooterRows = dax_grid_setFooterRows;

	// set number of right locked columns
	function dax_grid_setRightLockedColumns(numberOfColumns) {
		this.gridProperties.lockedRight = numberOfColumns;
	}
	
	this.setRightLockedColumns = dax_grid_setRightLockedColumns;
	
	// updates row, column & cell properties
	function dax_grid_primeGrid() {

		if (this.rows.length > 0) {
			// update columns
			for (var cellCount = 0; cellCount < this.rows[0].cells.length; cellCount++) {
				if (!this.cols[cellCount]) {
					// column doesn't exist
					this.cols[cellCount] = {
						cells			: [ ],
						styleName		: this.defaultStyle.column,
						colWidth		: null,
						isHeader		: (cellCount < this.gridProperties.headerColumns) ? true : false,
						isResizable		: true,
						isSelectable	: this.gridProperties.allowColumnSelection,
						isSelected		: false,
						visible			: true
					}
				} else {
					this.cols[cellCount].cells = [ ];
				}
			}
		} else if (this.cols) {
			// check if cols array exist, if so, reset cells
			// this happens when there are 0 rows in the grid
			for (var colCount = 0; colCount < this.cols.length; colCount++) {
				this.cols[colCount].cells = [ ];
			}
		}
	
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			for (var cellCount = 0; cellCount < this.rows[rowCount].cells.length; cellCount++) {
				// check if is this is a cell that belongs to the block
				if ((this.rows[rowCount].isBlock) && (cellCount > this.gridProperties.lockedLeft)) {
					var currentCell = null;
				} else {
					var currentCell = this.rows[rowCount].cells[cellCount];
				}
				
				// setup header functions
				if (currentCell) {
					if (this.rows[rowCount].isHeader) {
						currentCell.onmousedown = dax_grid_col_header_onmousedown;
						currentCell.onmousemove = dax_grid_col_header_onmousemove;
					} else if (this.cols[cellCount].isHeader) {
						currentCell.onmousedown = dax_grid_row_header_onmousedown;
						currentCell.onmousemove = dax_grid_row_header_onmousemove;
					} else {
						currentCell.onmousedown = dax_grid_cell_onmousedown;
					}

					currentCell.ondblclick = dax_grid_cell_ondblclick;
					
					currentCell.onmouseover = dax_grid_cell_onmouseover;
					currentCell.onmouseout = dax_grid_cell_onmouseout;
					currentCell.ondragrelease = dax_grid_cell_ondragrelease;
					// add cell to column array
					this.cols[cellCount].cells[rowCount] = currentCell;
					// set cell numbers
					currentCell.column = cellCount;
					currentCell.row = rowCount;
					
					currentCell.allowDragDrop = true;
					currentCell.grid = this;
				}
			}
		}
		
		// set column width for columns that don't have it set
		for (var colCount = 0; colCount < this.cols.length; colCount++) {
			if (!this.cols[colCount].colWidth)
				this.setColumnWidthInternal(colCount, 75);
		}
		
		// erase any extra columns
		if (this.rows.length > 0) {
			while (this.cols.length > this.rows[0].cells.length) {
				var removedElement = this.cols.pop();
			}					
		}
	}
	
	function dax_grid_updateColumnCoordinates() {
	
		// get column coordinates
		for (var colCount = 0; colCount < this.cols.length; colCount++) {
			var column = this.cols[colCount];
			var pos = getObjectPosition(column.cells[0]);
			column.posX = pos.x;
			column.colWidth = this.getCellWidth(column.cells[0]);
		}	
	}
	
	this.updateColumnCoordinates = dax_grid_updateColumnCoordinates;
	
	this.primeGrid = dax_grid_primeGrid;

	// secondColumn is target column
	function dax_grid_moveColumn(firstColumn, secondColumn) {
	
		if (this.swappingColumns)
			return false;
			
		this.swappingColumns = true;
	
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			// get first column cell
			var firstCell = this.rows[rowCount].cells.splice(firstColumn, 1)[0];
			this.rows[rowCount].dom.insertBefore(firstCell, this.rows[rowCount].cells[secondColumn]);
			this.rows[rowCount].cells.splice(secondColumn, 0, firstCell);

			firstCell.column = secondColumn;
		}

		// swap columns
		var firstColumnAr = this.cols.splice(firstColumn, 1)[0];
		this.cols.splice(secondColumn, 0, firstColumnAr);
		
		// update cell numbers and coordinates
		this.updateColumnCoordinates();
		
		for (var colCount = 0; colCount < this.cols.length; colCount++) {
			for (var cellCount = 0; cellCount < this.cols[colCount].cells.length; cellCount++) {
				var currentCell = this.cols[colCount].cells[cellCount];
				currentCell.column = colCount;
			}			
		}
				
		this.swappingColumns = false;
		this.onMoveColumn(firstColumn, secondColumn);
	}
	
	this.moveColumn = dax_grid_moveColumn;
	
	// sets grid, row, column, or cell style
	function dax_grid_setStyleClass(row, col, styleName, altStyleName, setAsDefault){

		if (!altStyleName)
			var altStyleName = styleName;

		if ((!row) && (row != 0) && (!col) && (col != 0)) {
			// grid
			this.styleName = styleName;
			if (setAsDefault)
				this.defaultStyle.grid = styleName;
		} else if ((!row) && (row != 0)) {
			// column
			this.cols[col].styleName = styleName;
			if (setAsDefault)
				this.defaultStyle.column = styleName;
		} else if ((!col) && (col != 0)){
			// row
			this.rows[row].styleName = styleName;
			this.rows[row].styleNameAlt = altStyleName;
			if (setAsDefault) {
				this.defaultStyle.row = styleName;
				this.defaultStyle.rowAlt = altStyleName;
			}
		} else {
			// cell
			this.rows[row].cells[col].styleName = styleName;
			if (setAsDefault)
				this.defaultStyle.cell = styleName;
		}	
		this.refreshStyles(row, col);
	}
	
	this.setStyleClass = dax_grid_setStyleClass;

	// wrappers
	function dax_grid_setColumnStyle(col, styleName, setAsDefault) {
		this.setStyleClass(null, col, styleName, null, setAsDefault);
	}
	
	this.setColumnStyle = dax_grid_setColumnStyle;
	
	function dax_grid_setRowStyle(row, styleName, altStyleName, setAsDefault) {
		this.setStyleClass(row, null, styleName, altStyleName, setAsDefault);
	}
	
	this.setRowStyle = dax_grid_setRowStyle;

	function dax_grid_setCellStyle(row, col, styleName, altStyleName, setAsDefault) {
		this.setStyleClass(row, col, styleName, altStyleName, setAsDefault);
	}
	
	this.setCellStyle = dax_grid_setCellStyle;
	
	// sets grid, row, column, or cell style
	function dax_grid_setSelectedStyleClass(row, col, styleName, altStyleName, setAsDefault){

		if (!altStyleName)
			var altStyleName = styleName;
		
		if ((!row) && (row != 0) && (!col) && (col != 0)) {
			// grid
			this.styleName = styleName;
			if (setAsDefault)
				this.defaultSelectedStyle.grid = styleName;
		} else if ((!row) && (row != 0)) {
			// column
			this.cols[col].selectedStyleName = styleName;
			if (setAsDefault)
				this.defaultSelectedStyle.column = styleName;
		} else if ((!col) && (col != 0)){
			// row
			this.rows[row].selectedStyleName = styleName;
			this.rows[row].selectedStyleNameAlt = altStyleName;
			if (setAsDefault) {
				this.defaultSelectedStyle.row = styleName;
				this.defaultSelectedStyle.rowAlt = altStyleName;
			}
		} else {
			// cell
			this.rows[row].cells[col].selectedStyleName = styleName;
			if (setAsDefault)
				this.defaultSelectedStyle.cell = styleName;
		}	
		this.refreshStyles(row, col);
	}
	
	this.setSelectedStyleClass = dax_grid_setSelectedStyleClass;	
	
	// sets grid, row, column, or cell style
	function dax_grid_setInternalStyleClass(row, col, styleName, altStyleName, setAsDefault){

		if (!altStyleName)
			var altStyleName = styleName;
		
		if ((!row) && (row != 0) && (!col) && (col != 0)) {
			// grid
			this.internalStyleName = styleName;
			if (setAsDefault)
				this.defaultInternalStyle.grid = styleName;
		} else if ((!row) && (row != 0)) {
			// column
			this.cols[col].internalStyleName = styleName;
			if (setAsDefault)
				this.defaultInternalStyle.column = styleName;
		} else if ((!col) && (col != 0)){
			// row
			this.rows[row].internalStyleName = styleName;
			this.rows[row].internalStyleNameAlt = altStyleName;
			if (setAsDefault) {
				this.defaultInternalStyle.row = styleName;
				this.defaultInternalStyle.rowAlt = altStyleName;
			}
		} else {
			// cell
			this.rows[row].cells[col].internalStyleName = styleName;
			if (setAsDefault)
				this.defaultInternalStyle.cell = styleName;
		}	
		this.refreshStyles(row, col);
	}
	
	this.setInternalStyleClass = dax_grid_setInternalStyleClass;	


	// no row or col refreshes everything
	function dax_grid_refreshStyles(row, col, overrideDisableVisualRefresh) {
	
		if ((this.gridProperties.disableVisualRefresh) && (!overrideDisableVisualRefresh))
			return;
	
		if (((!row) && (row != 0)) && ((!col) && (col != 0))) {
			// grid
			var rowCountStart = 0;
			var rowCountEnd = this.dimensions.rows;
			var cellCountStart = 0;
			var cellCountEnd = this.dimensions.columns;
		} else if ((!row) && (row != 0)) {
			// column
			var rowCountStart = 0;
			var rowCountEnd = this.dimensions.rows;
			var cellCountStart = col;
			var cellCountEnd = col + 1;
		} else if ((!col) && (col != 0)) {
			// row
			var rowCountStart = row;
			var rowCountEnd = row + 1;
			var cellCountStart = 0;
			var cellCountEnd = this.dimensions.columns;
		} else {
			// cell
			var rowCountStart = row;
			var rowCountEnd = row + 1;
			var cellCountStart = col;
			var cellCountEnd = col + 1;
		}			
	
		for (var rowCount = rowCountStart; rowCount < rowCountEnd; rowCount++) {
			if (this.rows[rowCount]) {
			for (var cellCount = cellCountStart; cellCount < cellCountEnd; cellCount++) {
				var currentCell = this.rows[rowCount].cells[cellCount];
	
				// check for alternate row
				if ((rowCount % 2 != 0) && (this.rows[rowCount].useAltStyle)) {
					// use alt row style
					var rowStyleName = this.rows[rowCount].styleNameAlt + ' ' + this.rows[rowCount].internalStyleNameAlt;

					// assign class name based on locked rows & columns
					if ((this.rows[rowCount].isHeader) && (this.cols[cellCount].isHeader))
						var startClassName = 'dax_grid_cell_colrow_header_alt';				
					else if (this.cols[cellCount].isHeader)
						var startClassName = 'dax_grid_cell_row_header_alt';				
					else if (this.rows[rowCount].isHeader)
						var startClassName = 'dax_grid_cell_col_header_alt';
					else
						var startClassName = 'dax_grid_cell_alt';				
					
				} else {

				// not alt row style
					
					var rowStyleName = this.rows[rowCount].styleName + ' ' + this.rows[rowCount].internalStyleName;

					// assign class name based on locked rows & columns
					if ((this.rows[rowCount].isHeader) && (this.cols[cellCount].isHeader))
						var startClassName = 'dax_grid_cell_colrow_header';
					else if (this.cols[cellCount].isHeader)
						var startClassName = 'dax_grid_cell_row_header';				
					else if (this.rows[rowCount].isHeader)
						var startClassName = 'dax_grid_cell_col_header';
					else
						var startClassName = 'dax_grid_cell';				
				
				}

				// check for selection
				if ((this.cols[cellCount].isSelected) || (this.rows[rowCount].isSelected) || (currentCell.isSelected)) {
					var selectedClassName = (this.rows[rowCount].selectedStyleName  ? (this.rows[rowCount].selectedStyleName + ' ') : '') +
						(this.cols[cellCount].selectedStyleName  ? (this.cols[cellCount].selectedStyleName + ' ') : '')  + 
						(this.selectedStyleName  ? (this.selectedStyleName + ' ') : '');

					// check for block
					if ((this.cols[cellCount].isSelected) && (this.rows[rowCount].isBlock) && (cellCount >= this.gridProperties.lockedLeft))
						var selectedClassName = '';
				} else {
					var selectedClassName = '';
				}
				
				
				
				// grid style + row style + column style + cell style
				currentCell.className = startClassName + ' ' +
					rowStyleName + ' ' +
					this.cols[cellCount].styleName + ' ' +
					this.cols[cellCount].internalStyleName + ' ' +
					currentCell.styleName + ' ' + 
					currentCell.internalStyleName + ' ' + 
					selectedClassName;
				}
			}
		}	
	}
	
	this.refreshStyles = dax_grid_refreshStyles;
	
	function dax_grid_setCellValue(row, column, contentValue, type){
		if (!this.rows[row])
			return false;
		if (!this.rows[row].cells[column])
			return false;
		var currentCell = this.rows[row].cells[column];
		
		if (!type)
			var type = currentCell.cellType;
		
		switch (type) {
			case 'html':
				currentCell.innerHTML = contentValue;
			break;
			case 'checkbox':
				if (contentValue == true) {
					currentCell.contentObject.className = 'fourdaf_checkbox_check';
				} else {
					currentCell.contentObject.className = 'fourdaf_checkbox_uncheck';
				}
			break;
			case 'input':
				currentCell.contentObject.value = contentValue;
			break;
			case 'textarea':
				currentCell.contentObject.value = contentValue;
			break;
			case 'dropdown':
				
				if (contentValue.length == 1) {
					if (currentCell.contentObject.options.length == 0) {
						// no elements present, create and select one
						currentCell.contentObject.options[0] = new Option(contentValue[0], contentValue[0]);
						currentCell.contentObject.selectedIndex = 0;
					} else {
						// single element, find and select it
						for (var selectCount = 0; selectCount < currentCell.contentObject.options.length; selectCount++) {
							if (currentCell.contentObject.options[selectCount].value == contentValue[0])
								currentCell.contentObject.selectedIndex = selectCount;								
						}
					}				
				} else if (contentValue.length > 0) {

					// clear current options
					currentCell.contentObject.options.length = 0;
					// add values of content array into the dropdown menu
					for (var arrayCount = 0; arrayCount < contentValue.length; arrayCount++) {
						currentCell.contentObject.options[arrayCount] = new Option(contentValue[arrayCount], contentValue[arrayCount]);
					}

					currentCell.selectedIndex = 0;
				}
			break;
			case 'image':
			break;
			case 'node':
			break;
			case 'text':
				currentCell.innerHTML = contentValue;
			break;
			default:
				currentCell.innerHTML = contentValue;
			break;
		}
		currentCell.value = contentValue;
		
		return currentCell;
	}
	
	this.setCellValue = dax_grid_setCellValue;
	
	// keep value preserves old value
	function dax_grid_setCellType(row, column, type, keepValue){
		var currentCell = this.rows[row].cells[column];
		if (keepValue)
			var currentValue = this.getCellValue(row, column);
		
		// return if type is the same
		if (type == currentCell.type)
			return;
			
		if (currentCell.contentObject)
			currentCell.contentObject = null;
			
		// purge cell contents
		removeChildren(currentCell);
		currentCell.cellType = type;
		
		switch (type) {
			case 'html':
			break;
			case 'checkbox':
				currentCell.contentObject = this.nodes.cellCheckBox.cloneNode(true);
				generateUniqueId(currentCell.contentObject);
				currentCell.appendChild(currentCell.contentObject);
				currentCell.contentObject.style.display = 'block';
				currentCell.contentObject.boundTo = this;
				currentCell.contentObject.parentCell = currentCell;
				
				// update style and redraw column
				this.refreshStyles(row, column);
				this.setColumnWidthInternal(column, this.cols[column].columnWidth);
				this.redraw();

			break;
			case 'input':
				// add and link input field
				currentCell.contentObject = this.nodes.cellInput.cloneNode(true);
				generateUniqueId(currentCell.contentObject);
				currentCell.appendChild(currentCell.contentObject);
				currentCell.contentObject.style.display = 'block';
				currentCell.contentObject.boundTo = this;
				currentCell.contentObject.parentCell = currentCell;
				
				currentCell.contentObject.onmousedown = this.inlineEdit_inputOnMouseDown;
				
				if (keepValue)
					currentCell.contentObject.value = currentValue;
				
				// update style and redraw column
				this.refreshStyles(row, column);
				this.setColumnWidthInternal(column, this.cols[column].columnWidth);
				this.redraw();
			break;
			case 'textarea':
				// add and link input field
				currentCell.contentObject = this.nodes.cellTextArea.cloneNode(true);
				generateUniqueId(currentCell.contentObject);
				currentCell.appendChild(currentCell.contentObject);
				currentCell.contentObject.style.display = 'block';
				currentCell.contentObject.boundTo = this;
				currentCell.contentObject.parentCell = currentCell;
				
				currentCell.contentObject.onmousedown = this.inlineEdit_inputOnMouseDown;
				
				if (keepValue)
					currentCell.contentObject.value = currentValue;
				
				// update style and redraw column
				this.refreshStyles(row, column);
				this.setColumnWidthInternal(column, this.cols[column].columnWidth);
				this.redraw();
			break;
			case 'dropdown':
				currentCell.contentObject = this.nodes.cellSelect.cloneNode(true);
				generateUniqueId(currentCell.contentObject);
				currentCell.appendChild(currentCell.contentObject);
				
				if (browser.ie7)
					currentCell.contentObject.className = 'dax_grid_cell_select_ie7';
				
				currentCell.contentObject.style.display = 'block';
				currentCell.contentObject.boundTo = this;
				currentCell.contentObject.parentCell = currentCell;

				currentCell.contentObject.onmousedown = this.inlineEdit_inputOnMouseDown;
				
				if (keepValue)
					currentCell.contentObject.options[0] = new Option(currentValue, currentValue);
				
				// update style and redraw column
				this.refreshStyles(row, column);
				this.setColumnWidthInternal(column, this.cols[column].columnWidth);
				this.redraw();
			break;
			case 'image':
			break;
			case 'node':
			break;
			case 'text':
			break;
			default:
			break;
		}
	}

	this.setCellType = dax_grid_setCellType;
	
	function dax_grid_inlineEdit_inputOnMouseDown(event) { xpEvent(event, true); }
	
	this.inlineEdit_inputOnMouseDown = dax_grid_inlineEdit_inputOnMouseDown;
	
	function dax_grid_getCellValue(row, column){

		var currentCell = this.getCell(row, column);
		
		if (!currentCell)
			return false;
	
		switch (currentCell.cellType) {
			case 'html':
				var currentValue = currentCell.innerHTML;
			break;
			case 'checkbox':
				if (currentCell.contentObject.className == 'fourdaf_checkbox_check') {
					var currentValue = 'true';
				} else {
					var currentValue = 'false';
				}			
			break;
			case 'textarea':
				var currentValue = currentCell.contentObject.value;
			break;
			case 'input':
				var currentValue = currentCell.contentObject.value;
			break;
			case 'dropdown':
				var currentValue = currentCell.contentObject.options[currentCell.contentObject.selectedIndex].value;
			break;
			case 'image':
			break;
			case 'node':
			break;
			case 'text':
				var currentValue = currentCell.innerHTML;
			break;
			default:
				var currentValue = currentCell.innerHTML;
			break;
		}
		return currentValue;
	}
	
	this.getCellValue = dax_grid_getCellValue;
	
	function dax_grid_enableVisualRefresh (doRefresh) {

		this.gridProperties.disableVisualRefresh = false;

		if (doRefresh) {
			this.refreshStyles();
			this.redraw(true, true);
		}
	}

	function dax_grid_disableVisualRefresh () {

		this.gridProperties.disableVisualRefresh = true;

	}

	this.enableVisualRefresh = dax_grid_enableVisualRefresh;
	this.disableVisualRefresh = dax_grid_disableVisualRefresh;
	
	function dax_grid_setRowHeightInPx(sizeInPx) {
		this.gridProperties.defaultRowHeight = sizeInPx;
		this.redraw(true);
	
	}
	
	this.setRowHeightInPx = dax_grid_setRowHeightInPx;
	
	function dax_grid_onVScroll() {
		var vPosition = this.scrollbars.ver.position;
		this.boundTo.leftContainer.style.top = 0 - vPosition + 'px';
		this.boundTo.rightContainer.style.top = 0 - vPosition + 'px';
	}
	
	function dax_grid_onHScroll() {
		var hPosition = this.scrollbars.hor.position;
		this.boundTo.topContainer.style.left = 0 - hPosition + 'px';
		this.boundTo.bottomContainer.style.left = 0 - hPosition + 'px';
	}
	
	function dax_grid_setGridClass(className, internalClassName) {
	
		if (className)
			this.gridProperties.className = className;
			
		if (internalClassName)
			this.gridProperties.internalClassName = internalClassName;
			
		this.grid.className = 'dax_grid ' + this.gridProperties.className + ' ' + this.gridProperties.internalClassName;
	}
	
	this.setGridClass = dax_grid_setGridClass;
	
	function dax_grid_animateColumnResizing(columnNumber, targetWidth, incrementInPx, intervalRate, gridRef) {
		if (targetWidth > gridRef.cols[columnNumber].colWidth) {
			var newWidth = ((gridRef.cols[columnNumber].colWidth + incrementInPx) > targetWidth ? targetWidth : (gridRef.cols[columnNumber].colWidth + incrementInPx));
			gridRef.setColumnWidth(columnNumber, newWidth);
			gridRef.columnAnimationTimeout = setTimeout(function() {gridRef.animateColumnResizing(columnNumber, targetWidth, incrementInPx, intervalRate, gridRef)});
		} else if (targetWidth < gridRef.cols[columnNumber].colWidth){
			var newWidth = ((gridRef.cols[columnNumber].colWidth - incrementInPx) < targetWidth ? targetWidth : (gridRef.cols[columnNumber].colWidth - incrementInPx));
			gridRef.setColumnWidth(columnNumber, newWidth);
			gridRef.columnAnimationTimeout = setTimeout(function() {gridRef.animateColumnResizing(columnNumber, targetWidth, incrementInPx, intervalRate, gridRef)});
		}	
	}
	
	this.animateColumnResizing = dax_grid_animateColumnResizing;
	
	function dax_grid_allowColumnSwap(allowSwap) {
		this.gridProperties.allowColumnSwap = allowSwap;
	}
	
	this.allowColumnSwap = dax_grid_allowColumnSwap;
	
	// scroll viewport to focus on the cell
	function dax_grid_focusCell(row, column) {

		// does cell exists?
		var cell = this.getCell(row, column);
		
		if (!cell)
			return true;

		// if row or column are locked, don't do anything
		if (this.gridProperties.lockedTop > row)
			return true;

		if (this.gridProperties.lockedLeft > column)
			return true;

		if ((row + 1) > (this.dimensions.rows - this.gridProperties.lockedBottom))
			return true;
			
		if ((column + 1) > (this.dimensions.columns - this.gridProperties.lockedRight))
			return true;
		
		
		// is cell horizontally in visible range? if so, don't do anything
		var cellOffsetLeft = cell.offsetLeft;
		var cellWidth = cell.offsetWidth;
		var notVisibleLeft = false;
		var notVisibleRight = false;
		
		// is cell hidden on left side?
		if (cellOffsetLeft < this.viewport.scrollbars.hor.position)
			notVisibleLeft = true;
		
		// is cell hidden on right side?
		if ((cellOffsetLeft + cellWidth) > this.viewport.scrollbars.hor.position + this.viewport.scrollbars.hor.visibleWidth)
			notVisibleRight = true;

		// return if cell if visible
		if (!((notVisibleLeft) || (notVisibleRight)))
			return true;
			
		// scroll to position where cell will be visible
		if (notVisibleRight)
			this.viewport.hscroll(cellOffsetLeft + cellWidth - this.viewport.scrollbars.hor.visibleWidth + 15);
		else if (notVisibleLeft)
			this.viewport.hscroll(cellOffsetLeft - 15);

	}
	
	this.focusCell = dax_grid_focusCell;
	
	// show column
	function dax_grid_showColumn(columnNumber) {

		// check if column exists
		if (!this.cols[columnNumber])
			return false;
		else
			var column = this.cols[columnNumber];
			
		// mark column as invisible
		this.cols[columnNumber].visible = true;
		
		// hide all cells that belong to the column
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			// skip block rows
			if (!this.rows[rowCount].isBlock) {
				// check if cell exists
				if (this.rows[rowCount].cells[columnNumber]) {
					var currentCell = this.rows[rowCount].cells[columnNumber];
					currentCell.style.display = 'block';
				}			
			}		
		}
		
		// don't redraw if visual refresh is disabled
		if (!this.gridProperties.disableVisualRefresh)
			this.redraw();

	
	}
	
	this.showColumn = dax_grid_showColumn;

	// hide column
	function dax_grid_hideColumn(columnNumber) {

		// check if column exists
		if (!this.cols[columnNumber])
			return false;
		else
			var column = this.cols[columnNumber];
			
		// mark column as invisible
		this.cols[columnNumber].visible = false;
		
		// hide all cells that belong to the column
		for (var rowCount = 0; rowCount < this.rows.length; rowCount++) {
			// skip block rows
			if (!this.rows[rowCount].isBlock) {
				// check if cell exists
				if (this.rows[rowCount].cells[columnNumber]) {
					var currentCell = this.rows[rowCount].cells[columnNumber];
					currentCell.style.display = 'none';
				}			
			}		
		}
		
		// don't redraw if visual refresh is disabled
		if (!this.gridProperties.disableVisualRefresh)
			this.redraw();
		
	}
	
	this.hideColumn = dax_grid_hideColumn;

	/*
	// User defined functions
	*/
	
		
	/*
	// Initialize object
	*/
	
	// create window, viewport, grid
	this.grid = this.nodes.grid.cloneNode(true);
	generateUniqueId(this.grid);

	this.topLeftContainer = getElementByClassName(this.grid, 'dax_grid_topleft_container');
	this.topContainer = getElementByClassName(this.grid, 'dax_grid_top_container');
	this.topRightContainer = getElementByClassName(this.grid, 'dax_grid_topright_container');
	
	this.leftContainer = getElementByClassName(this.grid, 'dax_grid_left_container');
	this.rightContainer = getElementByClassName(this.grid, 'dax_grid_right_container');

	this.bottomLeftContainer = getElementByClassName(this.grid, 'dax_grid_bottomleft_container');
	this.bottomContainer = getElementByClassName(this.grid, 'dax_grid_bottom_container');
	this.bottomRightContainer = getElementByClassName(this.grid, 'dax_grid_bottomright_container');
	
	this.topLeftViewport = getElementByClassName(this.grid, 'dax_grid_topleft_viewport');
	this.topViewport = getElementByClassName(this.grid, 'dax_grid_top_viewport');
	this.topRightViewport = getElementByClassName(this.grid, 'dax_grid_topright_viewport');

	this.leftViewport = getElementByClassName(this.grid, 'dax_grid_left_viewport');
	this.rightViewport = getElementByClassName(this.grid, 'dax_grid_right_viewport');
	
	this.bottomLeftViewport = getElementByClassName(this.grid, 'dax_grid_bottomleft_viewport');
	this.bottomViewport = getElementByClassName(this.grid, 'dax_grid_bottom_viewport');
	this.bottomRightViewport = getElementByClassName(this.grid, 'dax_grid_bottomright_viewport');
	
	this.viewportContainer = getElementByClassName(this.grid, 'dax_grid_viewport_container');
	this.gridContainer = this.nodes.gridContainer.cloneNode(true);

	this.rightScrollbar = getElementByClassName(this.grid, 'dax_grid_scrollbar_right');
	this.bottomScrollbar = getElementByClassName(this.grid, 'dax_grid_scrollbar_bottom');
	this.gridMainContainer = getElementByClassName(this.grid, 'dax_grid_main');
	
	generateUniqueId(this.gridContainer);
	
	// setup viewport
	this.viewport = new dax_viewport(this.viewportContainer);
	this.viewport.boundTo = this;
	this.viewport.setContents(this.gridContainer);

	this.viewport.onvscroll = dax_grid_onVScroll;
	this.viewport.onhscroll = dax_grid_onHScroll;

	this.viewport.isSuperGridChild = true;
	
	// move viewport scrollbars into designated containers
	this.rightScrollbar.appendChild(this.viewport.nodes.vscrollbar);
	this.bottomScrollbar.appendChild(this.viewport.nodes.hscrollbar);
	
	this.grid.style.display = 'block';
	this.gridContainer.style.display = 'block';
	
	if (this.targetNode)
		this.targetNode.appendChild(this.grid);

	this.activate = function(){
	
		this.primeGrid();
		this.redraw(true, true);

	}	

	this.setLocked();
	this.size(1, 1);
	this.refreshStyles();	
	
}

function dax_superGridObject(targetNode, lockedTop, lockedLeft, lockedRight, lockedBottom) {

	/*
	// Properties
	*/ 

	this.windowSettings = {
		addRowsOnResize		: false,		// add rows if window is resized beyond grid height
		windowMode			: true,
		parentContainer		: null
	};

	this.window = WindowManager.createNewWindow();
	dax_superGrid.apply(this, [this.window.contentArea, lockedTop, lockedLeft, lockedRight, lockedBottom]);
	this.targetNode = this.window.contentArea;
	// display and refresh window and viewport
	this.window.contentArea.appendChild(this.grid);		
	
	this.window.onsize = function(){
		this.parent.onWindowSize();
		this.parent.onWindowSizeExt();	// defined in data views, extensions, etc
	}

	function dax_superGridObject_onWindowSize() {
		this.grid.style.width = this.window.contentArea.offsetWidth + 'px';
		this.redraw();
		// add rows until viewport is filled
		if (this.windowSettings.addRowsOnResize) {
			if (this.dimensions.height < this.gridMainContainer.offsetHeight) {
				var rowsToAdd = Math.ceil((this.gridMainContainer.offsetHeight - this.dimensions.height) / this.gridProperties.defaultRowHeight);
				// add extra rows
				this.size(this.dimensions.rows + rowsToAdd, this.dimensions.columns);
			}
		}
	}
	
	this.onWindowSize = dax_superGridObject_onWindowSize;
	
	function dax_superGridObject_blank() { };
	
	this.onWindowSizeExt = dax_superGridObject_blank;
	
	this.window.parent = this;

	if (targetNode) {
		this.windowSettings.parentContainer = targetNode;
		this.windowSettings.windowMode = false;
	}
		
	function dax_superGridObject_activate(){
		this.window.activate();
		this.window.open();	
	
		// add window to node
		if (!this.windowSettings.windowMode) {

			// clear the node contents
			while (this.windowSettings.parentContainer.firstChild)
				this.windowSettings.parentContainer.removeChild(this.windowSettings.parentContainer.firstChild);
		
			this.windowSettings.parentContainer.appendChild(this.window.dom);
			this.window.dom.style.position = 'relative';
			this.window.dom.style.top = '0px';
			this.window.dom.style.left = '0px';
			this.window.dom.top.style.display = 'none';
			this.window.dom.btm.style.display = 'none';
			this.window.dom.l.style.display = 'none';
			this.window.dom.r.style.display = 'none';
		}

		if (!this.windowSettings.windowMode) {
			//this.window.dom.style.display = 'none';
			this.window.size(this.windowSettings.parentContainer.clientWidth, this.windowSettings.parentContainer.clientHeight, true);
		}
	
		this.window.onsize();
		this.primeGrid();
		this.redraw(true);		
	}
	
	this.activate = dax_superGridObject_activate;
	
}