/**
 * HListItem
 *
 * @author Jonathan Le
 * @author Julien Feasson
 */
function HList()
{
	var node = getTemplate("hl");
	if (!node)
		return null;
		
	// prv data
	this.items = new Array();
	this.dom = node.cloneNode(true);
	this.expanded = true;
	this.isChild = false;
	this.length = 0;
	this.draganddrop = false;
	this.draging = false;

	// assign unique id
	var i = 1;
	while ($('hl' + i) != null) {
		i++;
	}
	
	this.idNumber = i;
	this.dom.id = 'hl' + this.idNumber;
	
	this.build =
		function(list)
		{
			var items = list.childNodes;
			
			for (var i = 0; i < items.length; i++) {
				var item  = items[i];
				
				if (item.nodeName.toLowerCase() == "li") {
					var tn = getTextNode(item);
					var li = this.insert(tn.nodeValue);
					
					var children = item.childNodes;
					
					for (var j = 0; j < children.length; j++) {
						var nodeName = children[j].nodeName.toLowerCase();
						if (nodeName == "ul" || nodeName == "ol") {
							var hl = new HList();
							li.attach(hl.build(children[j]));
						}
					}
				}
			}
			
			return this;
		};
		
	this.load =
		function(array)
		{
			this.items = new Array();
			for (var i = 0; i < array.length; i++) {
				this.insert(array[i]);
			}
		};
	
	this.insert =
		function(label, img_url, index)
		{
			var li = new HListItem(label, img_url);
			
			if (index != null && index != undefined) {
				this.items.splice(index, 0, li);
			} else {
				index = this.items.push(li) - 1;
			}
			
			this.dom.insertBefore(li.dom,
				(this.items[index+1] ? this.items[index+1].dom : null));
			
			li.parent = this;
			
			if (img_url != null && img_url != undefined && img_url != "") {
				li.setIcon(img_url);
			}
			
			this.length++;
			
			return li;
		};

	this.rename =
		function(label, index)
		{
			if (index > 0 && index < this.items.length() && index != undefined) {
				this.items[index].label = label;
				return 1;
			}
			else
				return 0;
		};
	
	this.remove =
		function(index)
		{
			if (index > 0 && (index - 1) < this.items.length && index != undefined) {
				this.items.splice(index, 1);
				return 1;
			}
			else
				return 0;		
		};
	
	this.move =
		function(index_from,index_to) // Move the item at index_from to just before the item at index_to
		{			
			if ((index_from >= 0 && index_from < this.items.length && index_from != undefined) && (index_to >= 0 && index_to <= this.items.length && index_to != undefined)) {
				
				// remove item from index_from
				var li = this.items[index_from];
				this.items.splice(index_from, 1);
				this.dom.removeChild(li.dom);
				
				// insert item to before index_to
				if (index_to == this.items.length) {
					this.dom.appendChild(li.dom);
					this.items.splice(index_to, 0, li);
				}
				else {
					this.dom.insertBefore(li.dom, this.items[index_to].dom);
					this.items.splice(index_to, 0, li);
				}
				
				return 1;
				
			}
			else
				return 0;		
		};
		
	this.refresh =
		function()
		{
			for (var i in this.items) {
				this.items[i].setExpander();
			}
		};
	
	// init
	{
		setStyle(this.dom, "display", "block");
	}
	
	return this;
}

/**
 * HListItem
 *
 * @author Jonathan Le
 * @author Julien Feasson
 */
function HListItem(label, img_url)
{
	var node = getTemplate("hl_item");
	if (!node)
		return null;
	if (browser.ie)
		node.style.styleFloat = "none";
	
	this.dom 		= node.cloneNode(true);
	this.expander	= getElementByClassName(this.dom, "hl_item_expander");
	this.icon 		= getElementByClassName(this.dom, "hl_item_icon");
	this.label 		= getElementByClassName(this.dom, "hl_item_label");
	this.sublist 	= getElementByClassName(this.dom, "hl_item_sublist");

	// assign unique id
	var i = 1;
	while ($('hl_item' + i) != null) {
		i++;
	}
	
	this.idNumber = i;
	this.dom.id = 'hl_item' + this.idNumber;
	
	this.expander.parent = this;
	this.sublist.parent = this;
	this.label.parent = this;
	this.icon.parent = this;
	this.dom.parent = this;
	
	this.list = null;
	this.parent = null;
	
	this.setIcon =
		function(url)
		{
			this.icon.style.backgroundImage = "url(" + url + ")";
			this.icon.style.cursor = "pointer";
		};
		
	this.setExpander =
		function()
		{
			var isFirst = this.parent.items[0] == this;
			var isLast = this.parent.items[this.parent.items.length - 1] == this;
			
			if (this.list && this.list.length > 0) {
				if (this.list.expanded) {
					this.expander.className = "hl_item_expander_a";
				} else {
					this.expander.className = "hl_item_expander";
				}
			} else {
				if (isFirst && !isLast) {
					if (this.parent.isChild) {
						this.expander.className = "hl_item_expander_t";
					} else {
						this.expander.className = "hl_item_expander_h";
					}
				} else if (isLast && !isFirst) {
					this.expander.className = "hl_item_expander_l";
				} else if (!isFirst && !isLast) {
					this.expander.className = "hl_item_expander_t";
				} else if (isFirst && isLast) {
					this.expander.className = "hl_item_expander_l";
				}
			}
			
			if (isLast) {
				this.sublist.className = "hl_item_sublist_a";
			} else {
				this.sublist.className = "hl_item_sublist";
			}
		};
	
	this.expand =
		function()
		{
			if (this.list) {
				if (this.list.expanded) {
					this.sublist.style.display = "none";
					this.list.expanded = false;
				} else {
					this.sublist.style.display = "block";
					this.list.expanded = true;
				}
				
				this.list.refresh();
				this.setExpander();
			}
		};
	
	this.attach =
		function(hl)
		{
			this.list = hl;
			this.list.isChild = true;
			this.list.parent = this;
			this.sublist.insertBefore(hl.dom, null);
			
			// list starts out as expanded so collapse it
			this.expand();
		};
	
	this.expander.onclick =
		function()
		{
			var li = this.parent;
			li.expand();
			this.parent.parent.draganddrop = false;
		};
	
	this.icon.onmousedown =
		function()
		{
	/*
			this.parent.parent.draganddrop = true;
			for (var i in this.parent.parent.items) {
				if (this.parent == this.parent.parent.items[i])
					itemfrom = i;
			}
	*/
			
			return false;
		};
	
	this.icon.onmouseup =
		function()
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto>itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemfrom != itemto)
					this.parent.parent.move(itemfrom, itemto);
			}
			this.parent.parent.draganddrop = false;
			if (itemto!=itemfrom)
				this.parent.parent.draging = false;
			return true;
		};
	/*
	this.label.disableUnderline = false;
	this.label.onmouseover =
		function(aEvent)
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto!=itemfrom)
					this.parent.parent.draging = true;
				if (itemto>itemfrom) {
					if (this.parent.parent.items[itemto].list)
						if (this.parent.parent.items[itemto].list.expanded)
							this.parent.parent.items[itemto].expand();
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.innerHTML + "<br><img src='images/insert.png>";
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = "<img src='images/insert.png><br>" + this.parent.parent.items[itemto].label.innerHTML;
				}
				return true;
			}
			else {
				var event = xpEvent(aEvent, true);
				
				try {
					Controller.focusedObject.style.textDecoration = "";
	   			} catch(e) {}
	   			
				Controller.setFocused(this);
				if (!this.disableUnderline) {
					this.style.textDecoration = "underline";
					debugAlert('underline');
				}
				return false;
			}
		};
	this.label.onkeydown = 
		function(aEvent)
		{
			var keypressed = getkey(aEvent);
			
			switch (keypressed)
			{
				case 13:
					this.parent.icon.onclick();
				case 37:
					// Arrow left
					if (this.parent.list.expanded)
						this.parent.expander.onclick();
					break;
				case 38:
					// Arrow up
					
					// Could be replaced by an object property
					for (var i in this.parent.parent.items) {
						if (this.parent == this.parent.parent.items[i])
							itemnb = i;
					}
					
					mythis = null;
					
					if (itemnb!=0)
					{
						mythis = this.parent.parent.items[itemnb-1].label;
						this.style.textDecoration = "";
						
						if (mythis.parent.list)
							if (mythis.parent.list.expanded)
								mythis = mythis.parent.list.items[mythis.parent.list.items.length-1].label;
					}
					else
						if (this.parent.parent.isChild)
						{
							this.style.textDecoration = "";
							mythis = this.parent.parent.parent.label;
						}
							
					if (mythis)
					{	
						Controller.setFocused(mythis);
						mythis.style.textDecoration = "underline";
					}
						
					break;
				case 39:
					// Arrow right
					if (!this.parent.list.expanded)
						this.parent.expander.onclick();
					break;
				case 40:
					// Arrow down
					
					// Could be replaced by an object property
					for (var i in this.parent.parent.items) {
						if (this.parent == this.parent.parent.items[i])
							itemnb = i;
					}
					
					mythis = null;
					
					if (itemnb!=this.parent.parent.length-1)
					{
						mythis = this.parent.parent.items[itemnb-1+2].label;
					}
					else
						if (this.parent.parent.isChild)
						{
							// Could be replaced by an object property
							for (var i in this.parent.parent.parent.parent.items) {
								if (this.parent.parent.parent == this.parent.parent.parent.parent.items[i])
									itemnb = i;
							}
							
							//if (itemnb==this.parent.parent.parent.list.length-2)
							mythis = this.parent.parent.parent.parent.items[itemnb-1+2].label;
						}
					
					if (this.parent.list)
						if (this.parent.list.expanded)
							mythis = this.parent.list.items[0].label;
							
					if (mythis)
					{	
						this.style.textDecoration = "";
						Controller.setFocused(mythis);
						mythis.style.textDecoration = "underline";
					}
					break;
				default:
					break;
			}
		};
	
	this.label.onmouseout =
		function()
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto>itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
			}
			else
			{
				try {
					if (Controller.focusedObject.parent.constructor == HListItem)
						Controller.focusedObject.style.textDecoration = "";	
	   			} catch(e) {}
	   			
				this.style.textDecoration = "";
			}
			return true;
		};
	*/
	
	this.icon.onmouseover =
		function()
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto!=itemfrom)
					this.parent.parent.draging = true;
				if (itemto>itemfrom) {
					if (this.parent.parent.items[itemto].list)
						if (this.parent.parent.items[itemto].list.expanded)
							this.parent.parent.items[itemto].expand();
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.innerHTML + "<br><img src='images/insert.png>";
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = "<img src='images/insert.png><br>" + this.parent.parent.items[itemto].label.innerHTML;
				}
			}
			return true;
		};
	
	this.icon.onmouseout =
		function()
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto>itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
			}
			return true;
		};
	
	/*
	this.icon.onclick =
		function()
		{
			if (!labeledit &!this.parent.parent.draging) {
				editedobject = this.parent.label;
				this.parent.label.innerHTML = "<form action='' method='post' onsubmit='editedobject.innerHTML=this.label.value;labeledit=false;return false;'><input name='label' type='text' value='"+this.parent.label.innerHTML+"' style='width:100px;height:10px;font-size:8pt;'></form>";
				labeledit = true;
			}
			//if (this.parent.parent.draging)
			//	this.parent.parent.draging = true;
			return true;
		};
	*/		
	this.label.onmouseup =
		function()
		{
			if (this.parent.parent.draganddrop) {
				for (var i in this.parent.parent.items) {
					if (this.parent == this.parent.parent.items[i])
						itemto = i;
				}
				if (itemto>itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemto<itemfrom) {
					this.parent.parent.items[itemto].label.innerHTML = this.parent.parent.items[itemto].label.textContent;
				}
				if (itemfrom != itemto)
					this.parent.parent.move(itemfrom, itemto);
			}
			this.parent.parent.draganddrop = false;
			this.parent.parent.draging = false;
			return true;
		};
	
	this.label.onmousedown =
		function()
		{
			if (labeledit)
				return true;
			else
				return false;
		};
	
	// initializer
	{
		setStyle(this.dom, "display", "block");
		this.label.innerHTML = label;
	}

}




/**
 * HGrid
 *
 * @author Rudolf Psenicnik
 *
 * @param aParent
 * @param table name or number
 * 
 * creates hierarchical grid
 * breakpoints are set on the backend
 */
function HGrid (aParent, table, selectOneRow) {

	this.loaded = false;

	// get template
	var node = getTemplate("hgrid");
	var nodeBody = getTemplate("hgrid_body");
	var spacer = getTemplate("hgrid_spacer");
	
	// get table reference 
	this.table = getTable (table);

	if ((!node) || (!this.table) || (!nodeBody) || (!spacer))
		return null;

	// cache field names if not loaded
	this.table.cacheFields();
		
	if (this.table.showallrecordstab == 'false')
		return false;
		
	//assign pointers
	this.dom 			= node.cloneNode(true);
	this.body			= nodeBody.cloneNode(true);
	this.header			= getElementByClassName (this.dom, 'hgrid_header');
	this.container		= getElementByClassName (this.dom, 'hgrid_container');
	this.spacer			= spacer.cloneNode(true);
	this.viewport		= new ViewPort();
	this.window			= null;
	
	// parameters
	// lots of these are depreciated, but I'll keep them for now just in case
	this.horOffset		= 32; 				// used for first column offset
	this.maxLevel		= 0; 				// last break level
	this.firstColWidth	= 32;				// this.horOffset * this.maxLevel, width of first column
	this.levelField		= new Array(); 		// maps levels with fields: this.levelField[level] = field
	this.hlists			= new Array();		// array of all lists used in objects for easy reference
	this.hitems			= new Array();		// array of all items used in objects for easy reference
	this.lastLists		= new Array(); 		// array of all last lists (easy access to spacers)
	this.ignoreFields	= new Array();		// fields that are included in the hlist and therefore not displayable on the grid
	this.columns		= 0;
	this.width			= 0;
	this.height			= 0;
	this.maxGridRows	= 4;				// depreciated
	this.cellHeight		= 25;				// height of one cell, used for various visual positioning calculations
	this.vScrollPos		= 0;				// used to detect vertical scrolling
	this.gridRows		= 1;				// total number of grid rows to display
	this.populateFunc	= null;				// track populate function
	this.selectedItem	= null;				// selected list item
	this.selectedRec	= new Array();		// array of selected record id's
	this.selectedQue	= new Array();		// array of selected record queries
	this.queryIds		= new Array();		// list of queries to clear when window is closed
	this.isWindow		= true;				// window mode?
	this.isWindowMode 	= true;
	this.predefinedQuery = null;			// name of the predefined query to use instead of all records
	
	// preferences
	this.breakCount		= false;			// include record count in the break level label
	this.quickScroll	= false;			// quick scroll accelerates the redraw, useful with large number of records
	
	// custom actions enablers
	this.customActions = {
		gridClick :		false,
		gridDblClick :	false,
		headerClick :	false
	}
	
	// custom functions
	this.onBreakLevelHeaderClick = function () { };
	
	//assign unique id
	i = 1;
	while ($('hgrid' + i) != null) {
		i++;
	}
	this.idNumber = i;
	this.dom.id = 'hgrid' + this.idNumber;
	this.body.id = 'hgrid_body' + this.idNumber;
	this.spacer.id = 'hgrid_spacer' + this.idNumber;
	
	// cycle through all fields, set highest level and set levelField mapping
	// also build ignore fields array
	for (i = 0; i < this.table.fieldsList.length; i++) {
		// get breaklevel from field
		var level = this.table.fieldsList[i].fieldbreaklevel;
		if (level > 0) {
			// append ignore fields array
			this.ignoreFields.push(this.table.fieldsList[i]);
			// map this level to field
			this.levelField[level] = this.table.fieldsList[i];
			// set maximum level
			if (level > this.maxLevel) {
				this.maxLevel = level;
				this.firstColWidth = this.maxLevel * this.horOffset;
			}
		}
	}
	
	// abort if maximum level doesn't exist
	if (this.maxLevel == 0)
		return false;

	this.selectOneRow = false;
	
	if (selectOneRow)
		if (selectOneRow == true)
			this.selectOneRow = true;
		
	// show or hide breakpoint display
	this.showBreakCount = function (showBreak) {
		if (showBreak)
			this.breakCount == true;
		else
			this.breakCount == false;
	}
	
	// set quick scroll preference
	this.setQuickScroll = function (quickScroll) {
		if (quickScroll)
			this.quickScroll = true;
		else
			this.quickScroll = false;
	}

	if ((this.table.showallrecordstab.toLowerCase() == 'true') || (this.table.preselectQueries.length == 0)) {
	
		// don't use predefined query
		this.predefinedQuery = null;
	
	} else {
	
		this.predefinedQuery = this.table.preselectQueries[0].name;
	
	}

	
	// if pParent is passed, insert hgrid inside, otherwise create new window
//	if (aParent) {
//		this.parent	= aParent;
//		// add node
		//this.parent.appendChild(this.dom);
//	} else {
		var aWindow = WindowManager.createNewWindow();
			
		// set window title to "table name - display field (date field)"
		aWindow.titleBar.innerHTML = this.table.tablealias; 
		aWindow.contentArea.appendChild(this.dom);

		// set minimum sizes
		aWindow.minimumWidth = 250;
		aWindow.minimumHeight = 175;

		aWindow.activate();
		aWindow.open();	

		aWindow.status.style.height = '19px';

		aWindow.tableId = this.table.tableid;		
		
		// update window on close box method
		aWindow.closeBox.onmouseup = function (aEvent) {
			WindowManager.hideWindow(this.parent);
			window.clearTimeout(this.parent.boundTo.refreshTimeout);
			this.parent.tbitem.dom.style.display = 'none';
			return false;
		}
		
		// set window as parent
		this.parent	= aWindow.contentArea;
		this.window = aWindow;
		// IE fix
		if (browser.ie) {
			setStyle (aWindow.contentArea, 'position', 'relative');
		}

		//this.container = aWindow;
		// set  redraw on window resize
		aWindow.boundTo = this;
		aWindow.onsize = function () {
			this.boundTo.redraw();
			//this.boundTo.resizing = true;
		}
		aWindow.onsizestop = function () {
			this.boundTo.getRecords();
			//this.boundTo.resizing = false;
		}
		
		// if taskbar exists, add taskbar icon
		if (taskbar) {
			var ti = new taskbarItem(taskbar);
			ti.setLabel(aWindow.titleBar.innerHTML);
			ti.setIcon("datawindow");
			setStyle (ti.dom, 'display', 'block');
			// create a handler for clicking
			var winRef = aWindow;
			ti.dom.onclick = function() { 
				this.parent.setSelected (true, true);
				winRef.activate(); 
			};

			ti.setSelected (true, true);
			ti.parent.refresh();

			aWindow.tbitem = ti;
		}

		// update window title bar refresh button action
		aWindow.refreshBox.boundTo = this;
		aWindow.refreshBox.onmouseup = function(event)
		{
			var event = xpEvent(event, true);
			this.className = "window_refreshbox";
			this.boundTo.redraw();
			this.boundTo.updateLists();
			return false;
		};
		// window on close method
		aWindow.onclose = function() {
			// erase taskbar item
			if (this.tbitem)
				this.tbitem.remove();
		
			// get rid of the editor
			if (this.editor)
				this.editor.destroy();
				
			// call hgrid destroy function
			if (this.boundTo)
				this.boundTo.destroy();
		}
		
		// let's write a function dynamically
		aWindow.getEditor = function()
		{
			if (this.editor)
				return this.editor;
		
			this.editor = new Editor();
			var dom = this.editor.getDOM();
			this.dom.style.zIndex = ++WindowManager.maxZIndex;
		//	alert('Editor = ' + this.dom.style.zIndex);
			this.dom.m.appendChild(dom);
		
			var w_off =  (this.dom.offsetWidth / 2) -
				(dom.offsetWidth / 2);
		
			setStyle(dom, "position", "absolute");
			setStyle(dom, "left", w_off + "px");

			if (browser.ie)
				setStyle(dom, "top", "30px");
			
			this.editor.owner = this;
			
			this.editor.onaftersave = function()
			{
				this.owner.boundTo.updateLists();				
				this.owner.boundTo.redraw();
			}

			this.editor.onafterdelete = function()
			{
				this.owner.boundTo.updateLists();				
				this.owner.boundTo.redraw();
			}
			
			
			//this.editor.record = { recordId : -1, tableId: this.boundTo.table.tableid };
			//this.editor.buildEditorByTable(this.boundTo.table.tableid);

			
			// customize
			dom.ondragstop = function(aEvent)
			{
				this.className = "tearaway_torn";
				var editor = this.owner;
				
				if (editor.owner) {
					var winObj = editor.owner;
					
					if (!winObj.editor)
						return;
					
					var dom = winObj.editor.getDOM();
					var pos = getObjectPosition(this); // global coordinates

					if (taskbar) {
						
						var ti = new taskbarItem(taskbar);

						// make unique label based on table name
						i = 1;
						while ($('dax_taskbar_button' + winObj.titleBar.innerHTML + i) != null) {
							i++;
						}
						
						ti.dom.id = 'dax_taskbar_button' + winObj.titleBar.innerHTML + i;

						ti.setLabel(STR['taskbar_edit'] + " " + winObj.titleBar.innerHTML + " " + i);
						ti.setIcon("editor");
					
						// create a handler for clicking
						var ref = this;
						ti.dom.onclick = function() { 
							this.parent.setSelected (true, true);
							ref.onclick();
						};
						setStyle (ti.dom, 'display', 'block');
						ti.setSelected (true, true);
						
						ti.parent.refresh();
					}
					// handler for closing
					// ticlose = getElementByClassName (ti.dom, 'dax_taskbar_close', true);
					// ticlose.cancelBtnRef = dom.cancelBtn;
					// ticlose.onclick = function () {this.cancelBtnRef.onclick(null); };
					
					
					// override cancel button
					editor.prv.mDOM.cancelBtn.onclick = function(aEvent) {
						if (taskbar) {
							ti.remove(); // remove taskbar item
						}
						document.body.removeChild(this.parent.prv.mTearAway.dom);
						this.parent.destroy();
						//delete this.parent;
					}
					
					// override the focusing
					this.onclick = function(aEvent) {
						if (WindowManager.active)
							WindowManager.active.deactivate();
							setStyle(this, "zIndex", ++WindowManager.maxZIndex);
							if (ti) {
								ti.setSelected(true,true);
							}
					};
	
					// release
					winObj.deactivate();
					winObj.editor = null;
					this.parent.owner = null;
					
					// reposition
					document.body.insertBefore(this, null);
					setStyle(this, "left", pos.x + "px");
					setStyle(this, "top", pos.y + "px");
				}
			};
			return this.editor;
		};		

	//}
	

	// add toolbar
	this.toolbar = new Toolbar();
	
	if (this.toolbar) {
		// add the toolbar to the window
		if (aWindow) {
			aWindow.addToolbar(this.toolbar.prv.mDOM);
			aWindow.resizeToolbar(false);
			// update toolbar style
			var toolbarNode = getElementByClassName (aWindow.dom, 'window_toolbar');
			toolbarNode.className = 'window_toolbar hgrid_view_toolbar';
		} else {
			// simulate toolbar
			var toolbarNode = getElementByClassName (this.container, 'calendar_view_toolbar');
			if (toolbarNode) {
				toolbarNode.appendChild (this.toolbar.prv.mDOM);
				// adjust sizes for fixed container
				setStyle (this.parent, 'height', this.container.offsetHeight - this.toolbar.prv.mDOM.offsetHeight + 'px');
				setStyle (toolbarNode, 'width', this.container.offsetWidth + 'px');
			} else {
				// toolbar failed to load, use different resizing
				setStyle (this.parent, 'height', this.container.offsetHeight + 'px');
			}
		}
		// add toolbar buttons
//		var createBtn = this.toolbar.addButton(STR['button_create']);
		var deleteBtn = this.toolbar.addButton(STR['button_delete']);
		
		
		/*
		
		// modify create button parameters
		createBtn.boundTo = this;
		createBtn.onclick = function() { 
			// create is possible only if last list level is selected
			if (this.boundTo.selectedItem) {
				if (this.boundTo.selectedItem.parent.level == this.boundTo.maxLevel) {
					// find the editor if we have one, otherwise create a new one
					var editor = this.boundTo.parent.parent.getEditor();
					// clear the editor, set its record to -1 to signify that it is a new record
					if (isDefined(editor.record)) {
						// editor exists, use it
						editor.record.recordId = -1;
						editor.getDOM().deleteBtn.disabled = false; // make sure an empty record can't be deleted
					} else {
						// editor doesn't exists, create new one and use it
						//var grid = this.boundTo.contentArea.grid;
						editor.bRef = this;
						
						editor.record = { recordId : -1, tableId: this.boundTo.table.tableid };
						editor.buildEditorByTable(this.boundTo.table.tableid);
					}

					// now we are ready to edit
					editor.clear();
					// editor.queryId = aDataList.queryId;
					editor.edit();
					// populate editor data
					var aDataList = this.boundTo.selectedItem;
					while (aDataList) {
						// get the data from the list item
						var eValue = aDataList.data.value;
						var eFieldId = aDataList.data.fieldid;
						var eTableId = this.boundTo.table.tableid;
						// check if field type is date
						if (getField(eTableId, eFieldId).fieldtype == 'date') {
							editor.prv.mInputs[editor.record.map[eFieldId]].prv.mValue = eValue;
							editor.prv.mInputs[editor.record.map[eFieldId]].picker.value.value = eValue;
							// disable entry object
							editor.prv.mInputs[editor.record.map[eFieldId]].picker.value.disable = true;
						} else {
							// place the value into the editor
							editor.prv.mInputs[editor.record.map[eFieldId]].prv.mValue = eValue;
							editor.prv.mInputs[editor.record.map[eFieldId]].prv.mDOM.value.value = eValue;
							// disable entry object
							editor.prv.mInputs[editor.record.map[eFieldId]].prv.mDOM.value.disabled = true;
						}
						// set field as modified
						editor.prv.mInputs[editor.record.map[eFieldId]].prv.mModified = true;
						
						// go one level higher
						if (aDataList.parent.parent.data)
							aDataList = aDataList.parent.parent;
						else
							aDataList = null;
					}
				} else {
					alert (STR['select_branch']);
				}
			} else {
				alert (STR['select_branch']);
			}
		};

		this.createButton = createBtn;
		
		*/
		
		// modify delete button parameters
		deleteBtn.boundTo = this;
		deleteBtn.onclick = function() { 
			// number of selected records
			if (!this.boundTo.selectedRec)
				return;
			var selRecCount = this.boundTo.selectedRec.length;
			// check if there are any selected records
			if (selRecCount == 0) {
				// no records selected
				alert (STR['delete_records']);
			} else if (selRecCount > 0) {
				var eraseRec = confirm (STR['delete_records1'] + " " + selRecCount + " " + STR['delete_records2']);
				if (eraseRec) {
					deleteRecords (this.boundTo.table.tableid, this.boundTo.selectedRec);
					var deleteQueries = new Array();
					for (r = 0; r < this.boundTo.selectedQue.length; r++) {
						// array length check is for IE
						if (deleteQueries.length > 0) {
							if (deleteQueries.indexOf(this.boundTo.selectedQue[r]) == -1) {
								deleteQueries.push (this.boundTo.selectedQue[r]);
							}
						}
					}
					this.boundTo.selectedRec = new Array();
					this.boundTo.selectedQue = new Array();
					// clear all selections

					this.boundTo.updateLists(deleteQueries);
				}
			}
			
		};

		this.deleteButton = deleteBtn;

		
		// add ddw button to the toolbar
		var eTable = this.table;
		if ((eTable) && (eTable.ddwOptions) && (eTable.ddwOptions.length > 0)) {
			// add a ddw button
			var ddwOption = eTable.ddwOptions[0];
			var ddwButton = this.toolbar.addButton (ddwOption.title);
			ddwButton.url = ddwOption.url;
			ddwButton.title = ddwOption.title;
			ddwButton.owner = this;
			if (ddwOption.type.indexOf("Static") != -1) {
				ddwButton.onclick = function () {
					//var queryId = this.owner.contentArea.grid.selection.queryId;
					var tableId = this.owner.table.tableid;
					new DDW(this.url, 'static_url', this.window, "&tableid=" + tableId, this.owner.selectedRec);
				};
			} else if (ddwOption.type.indexOf("Dynamic") != -1) {
				ddwButton.onclick = function () {
					//var queryId = this.owner.contentArea.grid.selection.queryId;
					var tableId = this.owner.table.tableid;
					new DDW(this.url, 'dynamic_url', this.window,  "&tableid=" + tableId, this.owner.selectedRec);	
				};
			} else if (ddwOption.type.indexOf("Message") != -1) {
				ddwButton.onclick = function () {
					//var queryId = this.owner.contentArea.grid.selection.queryId;
					var tableId = this.owner.table.tableid;
					new DDW(this.url, 'message_window', this.owner.window,  "&tableid=" + tableId, this.owner.selectedRec);	
				};
			}
		}
	}
	
	// create grid
	this.grid = new NGrid();
	this.grid.owner = this;
	// get number of fields that are visible
	var s_fields = [ ];
	for (var fieldCounter = 0; fieldCounter < this.table.fieldsList.length; fieldCounter++) {
		var fieldRef = getField(this.table.tableid, this.table.fieldsList[fieldCounter].fieldid);
		if (fieldRef.fieldinlist.toLowerCase() == 'true'){
			s_fields.push(fieldRef);
		}
	}	
	this.grid.size ((s_fields.length - this.maxLevel) + 1,30);

	// add header to the hgrid
	this.container.appendChild (this.grid.dom);
	this.grid.refresh();
	
	if (browser.ie) {
		// resize grid
		for (i = 0; i < this.grid.prv.mRowCount; i++) {
			for (j = 0; j < this.grid.row(i).prv.mElements.length; j++) {
				if (this.isWindow)
					this.grid.row(i).element(j).prv.mDOM.style.height = '21px';
				else
					this.grid.row(i).element(j).prv.mDOM.style.height = this.cellHeight + 'px';
			}
		}
	}

	
	// setup viewport
	this.container.appendChild (this.viewport.dom);
	this.viewport.parent = this.container;
	this.viewport.setContent (this.body);
	this.viewport.setOverflow (0,0);
	this.viewport.boundTo = this.body;
	this.viewport.resizeOnRefresh = false;
	this.viewport.owner = this;
	

	// erase grid if quick scroll is enabled
	this.viewport.onvscrollstart = function(aTop) { 
		if (this.owner.quickScroll) {
			for (i = 0; i < this.owner.grid.prv.mRowCount; i++) {
				var rowRef = this.owner.grid.row(i);
				rowRef.clear();
				if (rowRef.prv.mFocus)
					rowRef.setFocus(false);
			}
		}
	};
	
	// scroll in cell height increments
	this.viewport.cellHeight = this.cellHeight;
	this.viewport.onvscroll = function(aTop, aIsScrollDown)
	{
		// calculate height
		var calcHeight = (aTop * (-1) * this.vscrollbar.ratio);
		
		var cellHeight = this.cellHeight;
		
		if ((browser.ie) && (this.owner.isWindow))
			cellHeight = 26;
		
		// round it up to the nearest cell height
		var moduo = calcHeight % cellHeight;
		var newHeight = (moduo > (cellHeight / 2)) ? (calcHeight + moduo) : (calcHeight  - moduo);
		// reposition the contents
		setStyle(this.boundTo, "top", newHeight + "px");
		if (!this.owner.quickScroll) {
			// if vscroll position is updated, move the grid
			if (this.owner.vScrollPos != newHeight) {
				
				var rowsToMove = (newHeight - this.owner.vScrollPos) / cellHeight;
				if (rowsToMove > 0)
					for (b = 0; b < rowsToMove; b++) {
						this.owner.grid.insertRow (0);
						for (j = 0; j < this.owner.grid.row(0).prv.mElements.length; j++) {
							if ((this.owner.isWindow) && (browser.ie))
								this.owner.grid.row(0).element(j).prv.mDOM.style.height = '21px';
						}
					}				
				else
					for (b = 0; b > rowsToMove; b--) {
						this.owner.grid.removeRow (0);
						this.owner.grid.insertRow ();
						for (j = 0; j < this.owner.grid.row(this.owner.grid.prv.mRowCount - 1).prv.mElements.length; j++) {
							if ((this.owner.isWindow) && (browser.ie))
								this.owner.grid.row(this.owner.grid.prv.mRowCount - 1).element(j).prv.mDOM.style.height = '21px';
						}
					}
				// update vscroll position
				this.owner.vScrollPos = newHeight;
			}
		}
	};

	this.viewport.onvscrollstop = function() { 
		// if quick scroll is enabled, align the hlist with grid
		/*
		if (this.owner.quickScroll) {
				
			var rowsToMove = (newHeight - this.owner.vScrollPos) / this.cellHeight;
			if (rowsToMove > 0)
				for (b = 0; b < rowsToMove; b++) {
					this.owner.grid.insertRow (0);
				}				
			else
				for (b = 0; b > rowsToMove; b--) {
					this.owner.grid.removeRow (0);
					this.owner.grid.insertRow ();
				}
			

			// update vscroll position
			this.owner.vScrollPos = newHeight;
		}		
		*/
		this.owner.getRecords();
	};	
	
	// set increment when scrollbar buttons are pressed
	this.viewport.vscrollbar.increment = this.cellHeight;
	
	this.columns = (s_fields.length - this.maxLevel);
	
	// create header
	// header is separate grid that controls all other grids
	
	var headerCounter = 1;
	
	// set grid labels and override on click and mouse over methods
	for (i = 0; i <= s_fields.length; i++) {
		// skip fields with break level set
		if (i == 0) {
			this.grid.column(i).setHeader(STR['hgrid_first_column']);
				this.grid.column(i).prv.mHeader.prv.mDOM.onmouseover = '';
				this.grid.column(i).prv.mHeader.prv.mDOM.onmouseout = '';
				this.grid.column(i).prv.mHeader.prv.mDOM.onmouseup = '';
				this.grid.column(i).prv.mHeader.prv.mDOM.owner = this;
			} else {
			// skip if field has break level as it is not displayable in grid
			if (s_fields[i-1].fieldbreaklevel == 0) {
				this.grid.column(headerCounter).setHeader(s_fields[i-1].fieldalias);
				this.grid.column(headerCounter).prv.mHeader.prv.mDOM.onmouseover = '';
				this.grid.column(headerCounter).prv.mHeader.prv.mDOM.onmouseout = '';
				this.grid.column(headerCounter).prv.mHeader.prv.mDOM.onmouseup = '';
				this.grid.column(headerCounter).prv.mHeader.prv.mDOM.owner = this;
				headerCounter++;
			}
		}
	}

	/*
	
	// override header grid refresh method
	this.headerGrid.refresh = function()
	{
		// try setting the widths
		try {
			var rowWidth = this.prv.mWidth;
			// set HTML styles
			this.body.style.width = rowWidth + 15 + "px";
			this.header.style.width = rowWidth + 15 + "px";
		} catch (e) {}
	};	

	*/
	
	// called when window is closed
	this.destroy = function () {
		if ((this.queryIds) && (this.queryIds.length > 0)){
			clearQuery (this.queryIds, true);
		}
	}
	
	// redraw function is used for visually updating the grid when resizing
	this.redraw = function () {
		// calculate new viewport width and height
		var newHeight = this.parent.offsetHeight - 24 + 'px';  //24
		var newWidth = this.parent.offsetWidth - 4 + 'px';
		setStyle (this.viewport.dom, 'width', newWidth);
		setStyle (this.viewport.dom, 'height', newHeight);
		
		// update viewport
		var overflow = this.body.offsetHeight + 25 - this.viewport.dom.offsetHeight;

		// update viewport vertical scrollbar
		this.viewport.vscrollbar.isStatic = true;
		this.viewport.vscrollbar.areaHeight = this.body.offsetHeight + 25;
		
		this.viewport.setOverflow(overflow, 0);
		
		// redraw viewport
		this.viewport.refresh();
		
		// if body height is larger than 2000, enter quick scroll mode
		if (this.body.offsetHeight > 2000)
			this.quickScroll = true;
		else
			this.quickScroll = false;

		// Omid window resize fix
		if (browser.gecko) {
			if (this.isWindow) {
				// reposition viewport
				this.viewport.dom.style.top = '79px'; // 19px
				this.viewport.vscrollbar.dom.style.height = this.viewport.dom.offsetHeight + 'px'; 
			} else {
				// reposition viewport
				this.viewport.dom.style.left = '0px'; // 19px
				this.viewport.dom.style.top = '49px'; // 19px
				this.viewport.vscrollbar.dom.style.height = this.viewport.dom.offsetHeight + 'px'; 
			}
		}
			
		// IE fixes
		if (browser.ie6) {
			// reposition viewport
			this.viewport.dom.style.top = '20px'; // 19px
			this.viewport.vscrollbar.dom.style.height = this.viewport.dom.offsetHeight + 'px'; 
		}
		
		if (browser.ie7) {
			// reposition viewport
			if (this.isWindow)
				this.viewport.dom.style.top = '24px'; //24px
			else
				this.viewport.dom.style.top = '19px'; //24px
		}		
		
		if ((browser.safari) && (!this.isWindow))
			this.viewport.dom.style.top = '49px'; // 19px
		
		/*
		
		//if (!this.isWindow) {	
			if (browser.ie) {
				// resize grid
				for (i = 0; i < this.grid.prv.mRowCount; i++) {
					for (j = 0; j < this.grid.row(i).prv.mElements.length; j++) {
						if (this.isWindow)
							this.grid.row(i).element(j).prv.mDOM.style.height = '21px';
						else
							this.grid.row(i).element(j).prv.mDOM.style.height = this.cellHeight + 'px';
					}
				}
			}
		//}
		
		*/
		
		// update selected elements
		// first if statement is IE bug fix
		for (f = 0; f < this.grid.prv.mRowCount; f++) {
			if (this.grid.row(f).recordId) {
//				if ((this.selectedRec.length > 0) && (this.selectedRec.indexOf(this.grid.row(f).recordId) != -1)) {
				if ((this.selectedRec.length > 0) && (findInArray(this.selectedRec, this.grid.row(f).recordId) != -1)) {
					// record exists in selection
					this.grid.row(f).setFocus(true);
				} else {
					// record is not selected
					this.grid.row(f).setFocus(false);
				}
			}
		}
		// resize grid
		//var gridRows = this.viewport.dom.offsetHeight / this.cellHeight;
		//this.grid.size (null, gridRows + 1);
		
		// ie and safari fix for label element width
		if (!browser.gecko) {
			var viewportSize = this.parent.offsetWidth;
			for (g = 0; g < this.hitems.length; g++) {
				var gItem = this.hitems[g];
				var labelLeft = 0;
				var loopObject = gItem.label;
				if (browser.ie) {
					while (loopObject.className != 'hgrid_body') {
						labelLeft += loopObject.offsetLeft;
						loopObject = loopObject.parentNode;
					}
				}
				// now that we have appropriate width, resize the label
				var gWidth = this.viewport.dom.offsetWidth - labelLeft;
				setStyle (gItem.label, 'width', gWidth + 'px');
				if ((gWidth + 100) > viewportSize)
					viewportSize = gWidth + 100;
			}
			setStyle (this.viewport.content, 'width', viewportSize);
		}
		
		// manually reposition the scroll bar
		this.viewport.vscrollbar.dom.style.top = "0px";
		//if (browser.gecko)
		//this.viewport.vscrollbar.dom.style.left = parseInt(this.viewport.vscrollbar.dom.style.left) - 14 + 'px';
		var leftStyle = this.viewport.vscrollbar.dom.style.left;
		if (this.isWindow)
			if ((browser.ie) && (!this.isWindowMode)) {
				this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) - 10 + 'px';
			} else {
				this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) - 14 + 'px';
			}
		else {
			if (browser.ie) {
				if (this.isWindowMode) {
					this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) - 10 + 'px';
				} else {
					this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) - 6 + 'px';
				}
			} else if (browser.safari) {
				this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) - 26 + 'px';
			} else {
				this.viewport.vscrollbar.dom.style.left = +leftStyle.substr(0, leftStyle.length - 2) + 'px';
			}
		}
	}
	
	
	this.refreshHeader = function () {
		/*
		this.headerGrid.column(0).setMinimumWidth (this.maxLevels * this.horOffset);		
		if (this.headerGrid.column(0).prv.mHeader.prv.mWidth < (this.maxLevels * this.horOffset)) {
			this.headerGrid.column(0).prv.mHeader.setWidth (this.maxLevels * this.horOffset);		
		};
		*/
	}
	
	// loads visible records
	this.getRecords = function () {
		// figure out which elements are visible
		// boundaries of the viewport
		var topBoundary = Math.abs(this.body.offsetTop);
		var bottomBoundary = topBoundary + this.viewport.dom.offsetHeight;
		
		// if there is populate function already, clear it
		if (this.populateFunc) {
			this.populateFunc.abort = true;
			this.populateFunc = null;
		}
		// object that holds information for record population
		var aPopulate = new Object();
		aPopulate.hGrid = this;
		aPopulate.abort = false;
		aPopulate.handler = this.getRecords_handler;
		this.populateFunc = aPopulate;
		
		// create array with queries to perform
		var aQueryCycle = new Array();
		
		// clear grid
		this.gridRows = 1;
		for (i = 0; i < this.grid.prv.mRowCount; i++) {
			var rowRef = this.grid.row(i);
			rowRef.queryId = null;
			rowRef.recordPosition = null;
			rowRef.recordId = null;
			rowRef.toString = function() {return '[object DataTreeGridRow]';};
		}

		// loop through lists and check if they are on the screen
		for (i = 0; i < this.lastLists.length; i++) {
			// (this.lastLists.length + ' lists found');
			var aLastList = this.lastLists[i];

			if ((!aLastList.recordCount) || (aLastList.recordCount > 0)) {

			
			if (browser.ie) {
				var listTop = 0;
				var loopObject = aLastList.dom;
				if (loopObject) {
					while (loopObject.className != 'hgrid_body') {
						listTop += loopObject.offsetTop;
						loopObject = loopObject.parentNode;
					}
				}
			} else {
				listTop = aLastList.dom.offsetTop;
			}
			
			
			debugAlert ('lOffTop:' + aLastList.dom.offsetTop + 
				' topBound:' + topBoundary +
				' lOffHeight:' + aLastList.dom.offsetHeight +
				' btmBound:' +  bottomBoundary);
			
			if ((listTop <= bottomBoundary) && ((listTop + aLastList.dom.offsetHeight) >= topBoundary)) {
			// check if we need to abort the process
				if (aPopulate.abort)
					return;
				
				var cellHeight = this.cellHeight;
				
				if (this.lastLists[i].spacer) {
					var aSpacer = this.lastLists[i].spacer;
					if (!browser.ie) {
						var spacerTop = aSpacer.offsetTop;
					} else {
						// IE likes to complicate things
						var spacerTop = 0;
						var loopObject = aSpacer;
						if (loopObject) {
							while (loopObject.className != 'hgrid_body') {
								spacerTop += loopObject.offsetTop;
								loopObject = loopObject.parentNode;
							}
						}
					}
					var spacerBottom = spacerTop + aSpacer.offsetHeight;
					if ((spacerTop <= bottomBoundary) && (spacerBottom >= topBoundary)) {
						// list is visible
						// set query parameters
						// debugger;
						aLastList.query.currentPos = null;
						aLastList.query.batchSize = null;
						aQueryCycle.push (aLastList.query);
						// get the visible limits for the grid area
						var upperLimit = (topBoundary <= spacerTop) ? spacerTop : topBoundary;
						var lowerLimit = (bottomBoundary >= spacerBottom) ? spacerBottom : bottomBoundary;
						var gridCount = (lowerLimit - upperLimit) / cellHeight;
						var firstSpacerTop = Math.floor(((upperLimit - topBoundary) / cellHeight));
						//debugAlert ('upperLimit:' + upperLimit + ' lowerLimit:' + lowerLimit + ' gridCount:' + gridCount +
						//	' this.cellHeight' + this.cellHeight + ' spacerTop:' + spacerTop + ' spacerBottom:' + spacerBottom);
						//debugAlert (firstSpacerTop);
						var recordPosition = (topBoundary <= spacerTop) ? 1 : ((topBoundary - spacerTop) / cellHeight);
						//debugAlert ("upLim:" + upperLimit + " lowLim:" + lowerLimit + " gridCount:" + gridCount + " firstSpacerTop:"
						//	+ firstSpacerTop + " recPos:" + recordPosition);
						// loop through background grid elements
						for (j = 0; j < gridCount; j++) {
							var spacerPosition = firstSpacerTop + j;
							this.grid.row(spacerPosition).queryId = aLastList.query.queryId;
							this.grid.row(spacerPosition).recordPosition = recordPosition + j;
							/*
							rowContents = [spacerPosition,this.grid.row(spacerPosition).queryId,
							this.grid.row(spacerPosition).recordPosition + " firstSp:"+ firstSpacerTop +" gridCount:" + gridCount + " j:" + j, 
							"rec pos:" + recordPosition + "+" + j];
							this.grid.row(spacerPosition).setRow(rowContents);
							*/
							//var debugRow = [" ", ];
							//debugAlert ("j:" + j + " spacPos:" + spacerPosition + " record:" + this.grid.row(spacerPosition).recordPosition);
							//this.grid.row(spacerPosition).element(0).setContent (spacerPosition);
							// update number of rows
							if ((spacerPosition + 1) > this.gridRows) {
								this.gridRows = spacerPosition + 1;
							}
						}
						//debugAlert ('gridTop:'+this.grid.dom.offsetTop);
						//debugAlert ('bodyTop:'+this.grid.body.offsetTop);
						// create query
						//debugAlert ('<b>queryAdded</b>: recPos:' + recordPosition + ' batch:' + gridCount);
						aLastList.query.currentPos = recordPosition;
						aLastList.query.batchSize = gridCount;
						aLastList.query.callRef = aPopulate;
						
						
						/*
						
						// find which grid row this spacer relates to
						var spacerPosition = (aSpacer.offsetTop - topBoundary) / this.cellHeight;
						// update number of rows
						if ((spacerPosition + 1) > this.gridRows) {
							this.gridRows = spacerPosition + 1;
						}
						// find which record in selection this spacer relates to
						var recordPosition = aSpacer.spacerNumber;
						// find query id in which record belongs
						var spacerQuery = aSpacer.parent.parent.query;
						// update query specs
						if (!aLastList.query.currentPos) {
							aLastList.query.currentPos = recordPosition;
							aLastList.query.batchSize = 1;
							aLastList.query.callRef = aPopulate;
						} else {
							aLastList.query.batchSize++;
						}
						// set grid row properties
						// we'll use these later to populate the rows
						this.grid.row(spacerPosition).queryId = aLastList.query.queryId;
						this.grid.row(spacerPosition).recordPosition = recordPosition;
						*/
						}
					}
				}
			}
		}
		// clear blank rows
		for (q = (this.gridRows - 1); q < this.grid.prv.mRowCount; q++) {
			this.grid.row(q).clear();
		}
		
		// we're determined which records should be downloaded, now loop through queries, execute them, and populate the appropriate rows
		aPopulate.queries = aQueryCycle;
		this.getRecords_handler (null, aPopulate);
	}
	
	this.getRecords_handler = function (http_response, aPopulate) {
		// check if we need to abort the process
		if (aPopulate.abort)
			return;
		// if this is a first call, http_response will be blank
		if (http_response) {
			// not a first call
			if (http_response.readyState !=4)
				return;
			// check for errors
			if (checkResponse (http_response))
				return;
			
			if (callback_onViewQuery)
				http_response = callback_onViewQuery(aPopulate.hGrid.table.tablename, http_response, 'datatree');				
			// parse reply
			aPopulate.currentQuery.parseResult(http_response);
			var queryResult = aPopulate.currentQuery.result;
			// populate grid with results
			if (queryResult) {
				// loop through results
				if (queryResult.length > 0) {
					for (t = 0; t < queryResult.length; t++) {
						var aQueryId = aPopulate.currentQuery.queryId;
						// save queryId for clearQuery call when window is closed
						if (aPopulate.hGrid.queryIds.length > 0) {
							if (findInArray(aPopulate.hGrid.queryIds,aQueryId) == -1)
								aPopulate.hGrid.queryIds.push (aQueryId);
						} else {
							aPopulate.hGrid.queryIds.push (aQuery.queryId);
						}
						var selectionId = queryResult[t].selectionid;
						// loop though the grid to find the appropriate row
						// this process could probably be more optimized
						for (u = 0; u < aPopulate.hGrid.grid.prv.mRowCount; u++) {
							var aRow = aPopulate.hGrid.grid.row(u);
							if (aRow.queryId) {
								if ((aRow.queryId == aQueryId) && (aRow.recordPosition == selectionId)) {
									// row and populate record match
									// filter results for breakpoints
									var rowContents = new Array();
									// first array element should be blank
									rowContents.push ("");
									var ddwElements = new Array();
									var elementCount = 0;
									for (v = 0; v < queryResult[t].fields.length; v++) {
										// don't add break level elements
										if (getField(aPopulate.hGrid.table.tableid, queryResult[t].fields[v].id).fieldbreaklevel == 0) {
											/*
											// format value
											var aFormat = getField(aPopulate.hGrid.table.tableid, queryResult[t].fields[v].id).fieldformat;
											if (aFormat){
												//debugAlert (queryResult[t].fields[v].value + " " + aFormat);
												var value = format(aFormat, queryResult[t].fields[v].value);
											} else {
												var value = queryResult[t].fields[v].value;
											}
											*/
											
											value = queryResult[t].fields[v].value;

											// get ddw properties
											var ddw = queryResult[t].fields[v].ddw ? queryResult[t].fields[v].ddw : null;
											var ddwType = queryResult[t].fields[v].ddwtype ? queryResult[t].fields[v].ddwtype : null;
											// add ddw if it exists
											if ((ddw) && (ddwType)) {
												var recordId = queryResult[t].fields[v].id + '[' + queryResult[t].fields[v].fieldrecordid + ']';
												if (ddwType.indexOf ('Static') != -1)
													value = "<a class='ddw_link' href='#' onclick='new DDW(\""+ddw+"\", \"static_url\",this.textContent, \"&recordid="+recordId+"\&tableid="+aPopulate.hGrid.table.tableid+"\"); return false;'>"+value+"</a>";
												else if (ddwType.indexOf ('Dynamic') != -1)
													value = "<a class='ddw_link' href='#' onclick='new DDW(\""+ddw+"\", \"dynamic_url\",this.textContent, \"&recordid="+recordId+"\&tableid="+aPopulate.hGrid.table.tableid+"\"); return false;'>"+value+"</a>";
												else if (ddwType.indexOf ('Message') != -1)
													value = "<a class='ddw_link' href='#' onclick='new DDW(\""+ddw+"\", \"message_window\", this.parentNode.parent.prv.mParent.prv.mParent.owner.window , \"&recordid="+recordId+"\&tableid="+aPopulate.hGrid.table.tableid+"\"); return false;'>"+value+"</a>";
												ddwElements.push(elementCount);
											}
											
											// add values to the array that will later populate the row
											rowContents.push (value);
											elementCount++;
										}
									}
									// populate the row
									//debugAlert (rowContents[1]);
									//rowContents = ["",aRow.recordPosition,aRow.queryId,rowContents[1]];
									aRow.setRow(rowContents);
									aRow.ddwLinks = new Array();
									if (ddwElements.length > 0) {
										for (y = 0; y < ddwElements.length; y++) {
											aRow.ddwLinks.push (getElementByClassName(aRow.prv.mDOM, 'ddw_link'));
										}
									}
									// check if there are image links, and add them to the imageLinks array
									
									// create image link references
									aRow.imageLinks = new Array();
									var imageLinks = getElementsByClassName (aRow.prv.mDOM, 'dax_grid_imagelink');
									aRow.imageLinks = new Array();
									if (imageLinks.length > 0) {
										for (y = 0; y < imageLinks.length; y++) {
											aRow.imageLinks.push (imageLinks[y]);
										}
									}
									
									aRow.recordId = queryResult[t].recordid;
									aRow.toString = function() {return '[object DataTreeGridRow]';};
									break;
								}
							}
						}
					}
				}
			}
		}
		// check if there are more queries to process
		if (aPopulate.queries.length > 0) {
			// get query reference
			var aQuery = aPopulate.queries.pop();
			aPopulate.currentQuery = aQuery;
			aQuery.nextBatch(aPopulate.handler);
		} else {
			aPopulate.hGrid.redraw();
			var ehGrid = aPopulate.hGrid;
			if (ehGrid.refreshTimeout)
				clearTimeout(ehGrid.refreshTimeout);
			ehGrid.refreshTimeout = setTimeout(function() { ehGrid.redraw(); ehGrid.updateLists(); }, 3000 * 60);	
	
		}
	}
	
	// enables custom actions for data tree
	this.customize = function (useGridClick, useDblClick, useHeaderClick, showButtons) {
	
		if (useGridClick == true)
			this.customActions.gridClick = true;
		else
			this.customActions.gridClick = false;

		if (useDblClick == true)
			this.customActions.gridDblClick = true;
		else
			this.customActions.gridDblClick = false;

		if (useHeaderClick == true)
			this.customActions.headerClick = true;
		else
			this.customActions.headerClick = false;		

		if (showButtons == true) {
			this.deleteButton.prv.mDOM.style.display = 'block';
//			this.createButton.prv.mDOM.style.display = 'block';
		 } else {
			this.deleteButton.prv.mDOM.style.display = 'none';
//			this.createButton.prv.mDOM.style.display = 'none';
		}	
	}
	
	// redraws grids to match header columns	
	this.refresh = function () {
	
		/*
	
		// resize the hgrid and container to fit the parent
		this.height = this.parent.offsetHeight - this.header.offsetHeight;
		this.width = this.parent.offsetWidth;
		setStyle (this.container, 'width', this.width + 'px');
		setStyle (this.container, 'height', this.height + 'px');
		// body dimensions are larger to avoid div overflow issues
		// setStyle (this.body, 'width', this.width + 500 + 'px');
		//setStyle (this.viewport.dom, 'width', this.width + 500 + 'px');
		//setStyle (this.viewport.dom, 'height', this.height + 'px');
		
		// get header column widths
		var colWidths 		= new Array();
		var colWidthTotal 	= 0;
		var parentWidth		= this.parent.offsetWidth;
		// get column sizes
		for (i = 0; i < this.columns; i++) {
			if (this.headerGrid.column(i+1).prv.mHeader) {
				colWidths.push(this.headerGrid.column(i+1).prv.mHeader.prv.mWidth);
				colWidthTotal = colWidthTotal + this.headerGrid.column(i+1).prv.mHeader.prv.mWidth;
			}
		}
		
		// last column size override to fit total width
		var extendLastColumnBy = this.width - colWidthTotal;
		var newLastColumnWidth = extendLastColumnBy;
		setStyle ((this.headerGrid.column(this.columns).prv.mHeader.prv.mDOM), 'width', newLastColumnWidth + 'px');
				
		// set new column sizes for every grid
		for (i = 0; i < this.childGrids.length; i++) {
			for (j = 0; j < this.childGrids[i].grid.prv.mRowCount; j++) {
				for (k = 0; k < this.childGrids[i].grid.prv.mColCount; k++) {
					this.childGrids[i].grid.row(j).element(k).setWidth(colWidths[k]);
				}
			}
			
			// set datawindow viewport size
			var gridViewportWidth = this.width - 100;
			this.childGrids[i].viewport.dom.style.width = gridViewportWidth + 'px';
			
			// grid offset - get first header size and move grid accordingly
			var gridOffset = this.headerGrid.column(0).prv.mHeader.prv.mWidth - (this.horOffset * 2);
			setStyle (this.childGrids[i].viewport.dom, 'left', gridOffset + 'px');
			setStyle (this.childGrids[i].viewport.dom, 'position', 'relative');
			this.childGrids[i].refresh();
			
			// reposition child grid viewport scrollbar
			this.childGrids[i].viewport.vscrollbar.dom.style.top = "0px";
			this.childGrids[i].viewport.vscrollbar.dom.style.left = this.childGrids[i].viewport.dom.offsetWidth - this.childGrids[i].viewport.vscrollbar.dom.offsetWidth + "px";
		}
	
		// allow for expansion space in the header
		this.headerGrid.header.style.width = (this.width * 2.5)+ 'px';
		
		// update viewport
		var overflow = this.body.offsetHeight - this.viewport.dom.offsetHeight;

		this.viewport.vscrollbar.isStatic = true;
		this.viewport.vscrollbar.areaHeight = this.body.offsetHeight;
		
		this.viewport.setOverflow(overflow, 0);

		this.viewport.refresh();
		
		//this.viewport.vscrollbar.dom.style.top = this.viewport.content.offsetTop + "px";
		//this.viewport.vscrollbar.dom.style.left = this.viewport.content.offsetLeft + "px";

		*/
		
	}
	
	// headers are done
	// attach hgrid to the parent
	//this.body.appendChild (this.dom);

	// initialize
	this.initialize = function () {
		// create new list
		aList = new HList();
		// root list
		this.body.appendChild (aList.dom);
		setStyle (aList.dom.style, 'float', 'none');
		aList.level = 1;
		aList.parent = this;
		// root list settings
		aList.selected = null;
		aList.queryId = null;
		// set properties
		aList.owner = this;
		// keep track of lists
		this.hlists.push (aList);
		this.populate (aList);
		
		// set all grid columns to be of equal length
		if (this.grid) {
			var gridWidth = this.grid.dom.offsetWidth;
			var gridColumnWidth = ((gridWidth - this.firstColWidth) / this.columns); 
			this.grid.setColumnWidth (gridColumnWidth);
			// set first column width
			this.grid.column(0).setWidth (this.firstColWidth);
			this.grid.column(0).prv.mMinimumWidth = this.firstColWidth;
			// redraw
			this.grid.refresh();
		}		

		//this.refresh();
		//this.redraw();
	}

	// calls backend to check the query size
	this.primeQuery = function (list) {
		li = list.parent;
		aQuery = new Query(li.parent.owner.table.tableid);
		// link query object to list
		li.query = aQuery;
		aQuery.batchSize = 0;
		aQuery.callRef = list; // reference to the list for later
		// break level settings
		//aQuery.breakLevel = aList.level; // what break level are we on?
		aQuery.breakSelected = dax_bridge.encode(li.selected); // selected item to expand on
		aQuery.breakQueryId = li.queryId; // queryid from parent list
		aQuery.breakGetLevel = li.parent.level;
		aQuery.queryName = li.parent.owner.predefinedQuery;
		// using variables because 'this' reference will be lost when processing reply
		aHandler = this.primeQuery_handler;
		// start batch calls
		aQuery.nextBatch(aHandler);			
	}

	// gets backend query size, resize the appropriate sublist
	this.primeQuery_handler = function (http_response, callRef) {
		// proceed only if http call state is 4
		if (http_response.readyState !=4)
			return;
		// check if list was passed
		if (!callRef)
			return;
		aList = callRef;
		// check for errors
		var errorResponse = checkResponse (http_response);
		// parse response
		aQuery.parseBreak (http_response);
		var hGrid = aList.parent.parent.owner;
		// get number of records in selection
		var gridSize = aQuery.recordsInSelection;
		aList.parent.spacing.innerHTML = '';
		// save queryId for clearQuery call when window is closed
		if (hGrid.queryIds.length > 0) {
			if (findInArray(hGrid.queryIds, aQuery.queryId) == -1)
				hGrid.queryIds.push (aQuery.queryId);
		} else {
			hGrid.queryIds.push (aQuery.queryId);
		}
		if (gridSize > 0) {
			// add spacer elements
			var aSpacer = hGrid.spacer.cloneNode(true);
			aSpacer.boundTo = hGrid;
			aSpacer.parent = aList;
			aList.parent.spacing.appendChild (aSpacer);
			// make sure spacer is long enough
			setStyle (aSpacer, 'width', hGrid.viewport.dom.offsetWidth + 'px');
			var newHeight = ((browser.ie) && (hGrid.isWindow)) ? (((hGrid.cellHeight + 1) * gridSize) + 'px') : (hGrid.cellHeight * gridSize + 'px');
			//setStyle (aSpacer, 'height', newHeight + 'px');
			aSpacer.style.height = newHeight;
			// with IE, height set must be explicit
			if (browser.ie) {
				aList.parent.spacing.style.height = newHeight;
			}
			// add the spacer to the array for easy reference
			aList.parent.spacer = aSpacer;
			// define spacer on double click event
			// edit record
			aSpacer.ondblclick = function (aEvent) {
				// get click position
				var aEvent = xpEvent (aEvent);
				var mousePos = getMousePosition (aEvent);
				var objectPos = getObjectPosition (hGrid.grid.body);
				// vertical click pos
				var clickedRow = parseInt ((mousePos.y - objectPos.y) / hGrid.cellHeight);
				// get grid row
				var aRow = hGrid.grid.row(clickedRow);
				// open the editor
				if (aRow) {
				
					if (hGrid.customActions.gridDblClick) {
					
						hGrid.grid.onrowdblclick(aRow);
					
					} else {
				
						if (hGrid.table.fieldsDetail.length == 0)
							return false;
				
						if (isDefined(hGrid.window.editor) && isDefined(hGrid.window.editor.record)) {
							hGrid.window.editor.edit();
						} else {
							hGrid.window.getEditor();
							setStyle(hGrid.window.editor.getDOM(), "display", "block");
							var editor = hGrid.window.editor;
						}
							
						//	var auto = hGrid.window.getAutoReload();
						//	if (auto > 0) {
						//		this.prv.mTimeoutId = setTimeout(function() { editor.reload(); }, auto * 1000);
						//	}
						
						// after data pull, disable editing of break level fields
						hGrid.window.editor.onAfterPull = function () {
							//loop through the map for field id's
							for (fieldId in this.record.map) {
								// if there is a breakpoint, disable the field
								var fieldRef = getField(this.record.tableId, fieldId);
								if (fieldRef.fieldbreaklevel != 0) {
									// check if field type is date
									if (fieldRef.fieldtype == 'date') {
										this.prv.mInputs[this.record.map[fieldId]].picker.value.disable = true;
									} else {
										// disable entry object
										this.prv.mInputs[this.record.map[fieldId]].prv.mDOM.value.disabled = true;
									}								
									this.prv.mInputs[this.record.map[fieldId]].prv.mModified = true;
								}
							}
						}					
						
						hGrid.window.editor.pull(hGrid.table.tableid, aRow.recordId);
						
						// populate editor data
						var aDataList = this.boundTo.selectedItem;
						while (aDataList) {
							// get the data from the list item
							var eValue = aDataList.data.value;
							var eFieldId = aDataList.data.fieldid;
							var eTableId = this.boundTo.table.tableid;
							// check if field type is date
							if (getField(eTableId, eFieldId).fieldtype == 'date') {
								hGrid.window.editor.prv.mInputs[editor.record.map[eFieldId]].picker.value.disable = true;
							} else {
								// disable entry object
								hGrid.window.editor.prv.mInputs[editor.record.map[eFieldId]].prv.mDOM.value.disabled = true;
							}
							// set field as modified
							hGrid.window.editor.prv.mInputs[editor.record.map[eFieldId]].prv.mModified = true;
							// go one level higher
							if (aDataList.parent.parent.data)
								aDataList = aDataList.parent.parent;
							else
								aDataList = null;
						}
					}
				}
			};
			// define spacer on click event
			// select record
			aSpacer.onclick = function (aEvent) {

				var cellHeight = browser.ie ? (hGrid.cellHeight + 1) : hGrid.cellHeight;
				
				// get click position
				var aEvent = xpEvent (aEvent);
				var mousePos = getMousePosition (aEvent);
				var objectPos = getObjectPosition (hGrid.grid.body);
				// vertical click pos
				var clickedRow = parseInt ((mousePos.y - objectPos.y) / cellHeight);
				// get grid row
				var aRow = hGrid.grid.row(clickedRow)
				var recordId = aRow.recordId;
				if (!recordId)
					return false;
				// IE fix
				if (hGrid.selectedRec.length > 0) {
//					if (browser.ie) {
						// IE doesn't support indexOf for arrays
						recordSelected = findInArray(hGrid.selectedRec, recordId);
//					} else {
//						recordSelected = hGrid.selectedRec.indexOf(recordId);
//					}
				} else {
					var recordSelected = -1;
				}
				
				// is ddw link clicked on?
				var ddwLinkClicked = null;
				var imageLinkClicked = null;
				if (aRow.ddwLinks.length > 0) {
					for (y = 0; y < aRow.ddwLinks.length; y++) {
						var linkPos = getObjectPosition (aRow.ddwLinks[y]);
						// check if click is in the link range
						if ((mousePos.x > linkPos.x) && (mousePos.x < linkPos.x + aRow.ddwLinks[y].offsetWidth)) {
							if ((mousePos.y > linkPos.y) && (mousePos.y < linkPos.y + aRow.ddwLinks[y].offsetHeight)) {
								ddwLinkClicked = aRow.ddwLinks[y];
								break;
							}
						}
					}
				}
				if (aRow.imageLinks.length > 0) {
					for (y = 0; y < aRow.imageLinks.length; y++) {
						var linkPos = getObjectPosition (aRow.imageLinks[y]);
						// check if click is in the link range
						if ((mousePos.x > linkPos.x) && (mousePos.x < linkPos.x + aRow.imageLinks[y].offsetWidth)) {
							if ((mousePos.y > linkPos.y) && (mousePos.y < linkPos.y + aRow.imageLinks[y].offsetHeight)) {
								imageLinkClicked = aRow.imageLinks[y];
								break;
							}
						}
					}
				}				
									
				if (ddwLinkClicked) {
					// simulate on link click for the element
					ddwLinkClicked.onclick();
				} else if (imageLinkClicked) {
					imageLinkClicked.onclick();
				} else {
					// simulate on click event for the row
					if (!(hGrid.customActions.gridClick)) {
						if (recordSelected == -1) {
							// record is not selected, select it
						aRow.setFocus (true);
							// add the record to the selected array
							hGrid.selectedRec.push (recordId);
							hGrid.selectedQue.push (aRow.queryId);
						} else {
							// record is selected, unselect it
							aRow.setFocus (false);
							// remove the record from the array
							hGrid.selectedRec.splice (recordSelected, 1);
							hGrid.selectedQue.splice (recordSelected, 1);
						}
					} else {
						if (hGrid.selectOneRow) {
							hGrid.grid.setFocusAll(false);	// Deselect all rows
							aRow.setFocus(true);	// Select the row that the user clicked
						} 
						hGrid.grid.onrowselect(aRow);
					}
				}
			};
		} else {
			// no records, remove the child
			var parentListItem = aList.parent;
			parentListItem.sublist.innerHTML = '';
			parentListItem.list = null;
			parentListItem.parent.refresh();			
		}
		// update label
		aList = aList.parent;
		if ((aList.data.ddw) && (aList.data.ddwtype)) {
			if (aList.data.ddwtype.indexOf ('Static') != -1)
				aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"static_url\",this.textContent, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
			else if (aList.data.ddwtype.indexOf ('Dynamic') != -1)
				aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"dynamic_url\",this.textContent, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
			else if (aList.data.ddwtype.indexOf ('Message') != -1)
				aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"message_window\",this.parentNode.parent.parent.owner.window, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
			} else {
			aList.label.innerHTML = aList.formattedValue + ' (' + gridSize + ')';
		}
		// get records
		hGrid.getRecords();
	}
	
	// populate list with data
	this.populate = function (aList) {
		// set query parameters
		aQuery = new Query(this.table.tableid);
		aQuery.batchSize = 20;
		aQuery.callRef = aList; // reference to the list for later
		// break level settings
		aQuery.breakLevel = aList.level; // what break level are we on?
		aQuery.breakSelected = dax_bridge.encode(aList.selected); // selected item to expand on
		aQuery.breakQueryId = aList.queryId; // queryid from parent list
		aQuery.queryName = this.predefinedQuery;
		// using variables because 'this' reference will be lost when processing reply
		aHandler = this.populate_handler;
		// start batch calls
		aQuery.nextBatch(aHandler);				
	}

	this.populate_handler = function (http_response, callRef) {
		// proceed only if http call state is 4
		if (http_response.readyState !=4)
			return;
		// check if list was passed
		if (!callRef)
			return;
		// check response for errors
		var errorResponse = checkResponse (http_response);
		var aList = callRef;
		var hGrid = aList.owner; 
		// parse response
		aQuery.parseBreak (http_response);
		if (hGrid.queryIds.length > 0) {
			if (findInArray (hGrid.queryIds, aQuery.queryId) == -1)
				hGrid.queryIds.push(aQuery.queryId);
		} else {
			hGrid.queryIds.push(aQuery.queryId);
		}				
		// loop through responses
		if (aQuery.size > 0) {
			for (b = 0; b < aQuery.result.length; b++) {
				// get values from query
				var queryId = aQuery.queryId;
				var tableId = aList.owner.table.tableid;
				var value = aQuery.result[b].value;
				var aDDW = aQuery.result[b].ddw;
				var aDDWType = aQuery.result[b].ddwtype;
				var aDDWTitle = aQuery.result[b].ddwtitle;
				// append values to the list
				if (value) {
					// format value
					var aFormat = getField(tableId, aQuery.fieldid).fieldformat;
					if (aFormat){
						// format value
						var formattedValue = format(aFormat, value);
					} else {
						var formattedValue = value;
					}
					var aItem = aList.insert (formattedValue);
					aList.owner.hitems.push (aItem);
					// set list item mouse methods
					aItem.icon.onmousedown = null;
					aItem.icon.onmouseup = null;
					aItem.icon.onmouseover = null;
					aItem.icon.onmouseout = null;
					aItem.label.onmouseup = null;
					aItem.label.onmousedown = null;
					aItem.label.disableUnderline = true;
					// set css style based on level
					aItem.label.className = aItem.label.className + ' level' + aList.level;
					if (browser.ie)
						setStyle (aItem.label, 'height', '25px');
					// create and attach sublist
					var aItemList = new HList();
					aItem.attach(aItemList);
					// save data with item for creating a new record
					aItem.data = new Object();
					aItem.data.tableid = aList.owner.table.tableid;
					aItem.data.fieldid = aQuery.fieldid;
					aItem.data.value = formattedValue;
					// are children cached?
					aItem.cached = false;
					// dummy item will insure that this aItemList looks expandable
					var dummyItem = aItemList.insert (STR['loading']);
					aList.dummy = dummyItem;
					aItemList.refresh();
					// hide dummy item
					setStyle(dummyItem.dom, 'display', 'none');
					// save parameters to the list item
					aItem.queryId = queryId;
					aItem.selected = value;
					aItem.formattedValue = formattedValue;
					// add ddw info
					aItem.data.ddw = aDDW;
					aItem.data.ddwtype = aDDWType;
					aItem.data.ddwtitle = aDDWTitle;
					// if ddw are used, add a link
					if ((aItem.data.ddw) && (aItem.data.ddwtype)) {
						if (aItem.data.ddwtype.indexOf ('Static') != -1)
							aItem.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aItem.data.ddw+"\", \"static_url\",this.textContent, \"&data="+encodeURIComponent(aItem.data.value)+"\&queryid="+aItem.queryId+"\"); return false;'>"+aItem.data.value+"</a>";
						else if (aItem.data.ddwtype.indexOf ('Dynamic') != -1)
							aItem.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aItem.data.ddw+"\", \"dynamic_url\",this.textContent, \"&data="+encodeURIComponent(aItem.data.value)+"\&queryid="+aItem.queryId+"\"); return false;'>"+aItem.data.value+"</a>";
						else if (aItem.data.ddwtype.indexOf ('Message') != -1)
							aItem.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aItem.data.ddw+"\", \"message_window\",this.parentNode.parent.parent.owner.window, \"&data="+encodeURIComponent(aItem.data.value)+"\&queryid="+aItem.queryId+"\"); return false;'>"+aItem.data.value+"</a>";
						}
					// set expander actions based on list level
					if (aList.level == aList.owner.maxLevel) {
						// last level, load grid
						aItem.expander.onclick =
							function()
							{								
								var li = this.parent;
								// reference to HGrid
								hGrid = li.parent.owner;					
								li.expand();
								li.labelText = li.label.innerHTML;
								// create contents only if not cached
								if (li.cached == false) {
									// purge aItemList children
									while (li.list.dom.firstChild)
										li.list.dom.removeChild(li.list.dom.firstChild);	
									// get node for hgrid object container and place grid object inside
									aGridNode = getTemplate ("hgrid_object");
									aGridParent = aGridNode.cloneNode(true);
									li.list.dom.appendChild (aGridParent);
									li.spacing = aGridParent;
									// assign unique id
									var i = 1;
									while ($('hgrid_object' + i) != null) {
										i++;
									}
									li.spacing.idNumber = i;
									li.spacing.id = 'hgrid_object' + li.spacing.idNumber;
									// place temporary message in spacer
									// spacer will later be populated with div elements that simulate extra space
									//li.spacing.innerHTML = 'Loading...';
									setStyle (li.spacing, 'height', '25px');

									// add spinner
									var spinner = getTemplate("spinner_left").cloneNode(true);
									// assign unique id to the spinner
									i = 1;
									while ($('spinner_left' + i) != null) {
										i++;
									}
									spinner.id = 'spinner_left' + i;				
									
									li.spacing.appendChild (spinner);
									setStyle(spinner, "display", "block");
									setStyle(spinner, "visibility", "visible");
									
									// update array of lists on the last level
									hGrid.lastLists.push (li);
									// get number of records in query
									hGrid.primeQuery (this);
									// set this level as initialized
									li.cached = true;
								} else {
									hGrid.getRecords();
								}
								hGrid.redraw();
							};
						aItem.icon.onclick = aItem.expander.onclick;
						// last list item, select it
						aItem.label.onclick = function () {
							var hGrid = this.parent.parent.owner;
							// if another list is selected, deselect it
							if (hGrid.selectedItem) {
								hGrid.selectedItem.label.className = 'hl_item_label level' + hGrid.selectedItem.parent.level;
							}
							// set this item as selected
							hGrid.selectedItem = this.parent;
							hGrid.selectedItem.label.className = 'hl_item_label level_sel';
							if (hGrid.customActions.headerClick) {
								hGrid.onBreakLevelHeaderClick(hGrid.selectedItem.selected, hGrid.selectedItem.parent.level);
							}
						};
					} else {
						// not last level, expand will load one more list
						aItem.expander.onclick =
							function()
							{
								// reference to HGrid
								var li = this.parent;
								hGrid = li.parent.owner;					
								// create contents only if not cached
								if (li.cached == false) {
									// add next level list
									nList = new HList ();
									li.attach (nList);
									// set new list parameters
									nList.level = (parseFloat(li.parent.level) + 1);
									nList.owner = hGrid;
									// set object as cached
									li.cached = true;
									// pass on parameters to new list
									nList.queryId = li.queryId;
									nList.selected = li.selected;
									// populate list
									hGrid.populate(nList);
									//li.expand();
								} else {
									li.expand();
									hGrid.getRecords();
								}
								hGrid.redraw();
							};
						aItem.icon.onclick = aItem.expander.onclick;
						aItem.label.onclick = function () {
							var hGrid = this.parent.parent.owner;
							// if another list is selected, deselect it
							if (hGrid.selectedItem) {
								hGrid.selectedItem.label.className = 'hl_item_label level' + hGrid.selectedItem.parent.level;
							}
							// set this item as selected
							hGrid.selectedItem = this.parent;
							hGrid.selectedItem.label.className = 'hl_item_label level_sel';
							if (hGrid.customActions.headerClick) {
								hGrid.onBreakLevelHeaderClick(hGrid.selectedItem.selected, hGrid.selectedItem.parent.level);
							}
						};
						aItem.label.onmouseover = null;
						aItem.label.onmouseout = null;
					}
				}
			}
			if (aList.level != 1) { 
				aList.parent.expand();
			}
		}
		// redraw the list
		aList.refresh();
		hGrid.redraw();
		
	}
	
	// update lists, use when adding or removing records
	this.updateLists = function (queryArray) {

		// if there is populate function already, clear it
		if (this.populateFunc) {
			this.populateFunc.abort = true;
			this.populateFunc = null;
		}
		
		// if query array is blank, update all lists
		if ((!queryArray) || (queryArray.length == 0)) {
			var updateLists = this.lastLists.slice();

		} else {
			// update only specific lists
			var updateLists = new Array();
			for (u = 0; u < this.lastLists.length; u++) {
				var queryPosition = queryArray.indexOf(this.lastLists[u].query.queryId);
				if (queryPosition != -1) {
					// query id matches one of the lists, add it to the array
					// but only if it's not already added
					if (updateLists.indexOf(this.lastLists[u].query.queryId) == -1) {
						updateLists.push (this.lastLists[u]);
					}
				}
			}
		}

		
		// cycle the list updates
		var aUpdate = updateLists;
		aUpdate.owner = this;
		aUpdate.handler = this.updateLists_handler;
		this.populateFunc = aUpdate;
		this.updateLists_handler (null, aUpdate);
	}
	
	
	// loop through the lists, and update them
	// initialize code
	this.updateLists_handler = function (http_response, callRef) {
		// proceed only if http call state is 4
		if (http_response)
			if (http_response.readyState !=4)
				return;
		
		// check if list was passed
		if (!callRef)
			return;

		// abort if needed
		if (callRef.abort)
			return;
			
		// if http_reqest came in, parse it
		if (http_response) {
			
			// check for errors
			if (checkResponse (http_response))
				debugAlert ('error in .updateLists_handler');
			
			var aList = callRef.currentList;
			var aQuery = callRef.currentQuery;
			
			aQuery.parseBreak (http_response);
			var hGrid = aList.parent.owner;
			// get number of records in selection
			var gridSize = aQuery.recordsInSelection;
			
			aList.recordCount = gridSize;
			
			if (gridSize > 0) {
				if (!aList.list) {
					// there is no list, create one
					aList.owner.primeQuery (aList);
				} else {
					// make sure spacer is long enough
					
					aList.spacer.style.width = hGrid.viewport.dom.offsetWidth + 'px';
					aList.spacer.style.height = (hGrid.cellHeight * gridSize) + 'px';
//					setStyle (aList.spacer, 'width', hGrid.viewport.dom.offsetWidth + 'px');
//					setStyle (aList.spacer, 'height', (hGrid.cellHeight * gridSize) + 'px');
				}
			} else {
				// no records, remove the child
				
//				if (!browser.ie) {
					//aList.spacing.innerHTML = '';
					//aList.sublist.innerHTML = '';
					setStyle (aList.spacing, 'width', '0px');
					setStyle (aList.spacing, 'height', '0px');
					setStyle (aList.sublist, 'width', '0px');
					setStyle (aList.sublist, 'height', '0px');
					aList.list = null;
					aList.parent.refresh();		
//				}
				// if spacer exists, minimize it
				if (aList.spacer) {
					setStyle (aList.spacer, 'width', '0px');
					setStyle (aList.spacer, 'height', '0px');
				}
			}
			
			// update label
			//aList = aList.parent;
			if ((aList.data) && (aList.data.ddw) && (aList.data.ddwtype)) {
				if (aList.data.ddwtype.indexOf ('Static') != -1)
					aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"static_url\",this.textContent, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
				else if (aList.data.ddwtype.indexOf ('Dynamic') != -1)
					aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"dynamic_url\",this.textContent, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
				else if (aList.data.ddwtype.indexOf ('Message') != -1)
					aList.label.innerHTML = "<a class='ddw_link' href='#' onclick='new DDW(\""+aList.data.ddw+"\", \"message_window\",this.parentNode.parent.parent.owner.window, \"&data="+encodeURIComponent(aList.data.value)+"\&queryid="+aList.queryId+"\"); return false;'>"+aList.data.value+"</a> (" + gridSize + ")";
				} else {
				aList.label.innerHTML = aList.formattedValue + ' (' + gridSize + ')';
			}
			
		}
			
		// make a next call
		if (callRef.length > 0) {
			var li = callRef.pop();
			aQuery = new Query(li.parent.owner.table.tableid);
			// link query object to list
			li.query = aQuery;
			aQuery.batchSize = 1;
			aQuery.callRef = callRef; // reference to the list for later
			// break level settings
			//aQuery.breakLevel = aList.level; // what break level are we on?
			aQuery.breakSelected = dax_bridge.encode(li.selected); // selected item to expand on
			aQuery.breakQueryId = li.queryId; // queryid from parent list
			aQuery.breakGetLevel = li.parent.level;
			aQuery.queryName = li.parent.owner.predefinedQuery;
			// using variables because 'this' reference will be lost when processing reply
			aHandler = this.primeQuery_handler;
			callRef.currentQuery = aQuery;
			callRef.currentList = li;
			// start batch calls
			aQuery.nextBatch(callRef.handler);	
		} else {
			callRef.owner.redraw();
			callRef.owner.getRecords();
		}
	}

	this.initialize();	

	if (aParent) {
//		this.isWindow = false;
		if ((browser.gecko) || (browser.safari))
			this.isWindow = false;
		else
			this.isWindow = true;
		
		this.isWindowMode = false;
		// if location exists, insert the data window into the target element
		var targetPosition = getObjectPosition (aParent);
		// position the window where target element is
		aWindow.dom.style.top = targetPosition.y + 'px';
		aWindow.dom.style.left = targetPosition.x + 'px';
		aWindow.size (aParent.offsetWidth, aParent.offsetHeight, true);
		// hide window based doms
		aWindow.dom.top.style.display = 'none';
		aWindow.dom.btm.style.display = 'none';
		aWindow.dom.l.style.display = 'none';
		aWindow.dom.r.style.display = 'none';

		// safari framework mode bug fix
		if (browser.safari) {
			aWindow.dom.style.position = 'absolute';
			aWindow.dom.style.overflow = 'hidden';
			aWindow.dom.m.style.width = aWindow.dom.offsetWidth + 'px';
			aWindow.dom.m.style.overflow = 'hidden';
			this.dom.style.width = aWindow.dom.offsetWidth - 12 + 'px';			
		}
	}
	
	// window border  bug fix
	if (aWindow)
		aWindow.resize (1,1);
		
	this.loaded = true;
		
}