// report viewer
function dax_reportViewer(reportTargetNode) {

	// sent as additional attributes in url
	this.customValues = {
		name		: [],
		value		: []
	}

	// report will be inserted into targetNode
	if (reportTargetNode)
		this.reportTargetNode = reportTargetNode;
	else
		this.reportTargetNode = null;

	// initialize super grid
	dax_superGridObject.apply(this, [this.reportTargetNode, 0, 0]);
	
	this.gridProperties.headerRows = 0;
	this.gridProperties.headerColumns = 0;

	this.setGridClass(null, 'dax_reportviewer');
	
	function dax_reportViewer_onWindowClose() {
		this.parent.onclose();
	}
	
	this.window.onclose = dax_reportViewer_onWindowClose;
	
	// create toolbar
	this.toolbar = new Toolbar();
	this.window.addToolbar(this.toolbar.prv.mDOM);
	this.window.resizeToolbar(false);
		
	this.toolbar.prv.mDOM.className += ' dax_grid_toolbar';

	// add create & delete record buttons
	this.toolbar.previousPage = this.toolbar.addButton('Previous');
	this.toolbar.nextPage = this.toolbar.addButton('Next');	
	
	this.toolbar.nextPage.owner = this;
	this.toolbar.previousPage.owner = this;
	
	// set button actions
	this.toolbar.nextPage.onclick = function (){
		this.owner.getNextPage();
	}

	this.toolbar.previousPage.onclick = function (){
		this.owner.getPreviousPage();
	}
	
	// add the spinner
	this.toolbar.spinner = getTemplate("dax_grid_spinner").cloneNode(true);
	this.toolbar.addContent(this.toolbar.spinner);
	generateUniqueId(this.toolbar.spinner);

	// refresh interval
	this.refreshInterval = 3000 * 60; // 3 minutes
	
	// activate report window
	this.activate();

	// define report
	this.report = {
		initialized		: false,
		objects			: new Array(),
		staticObjects	: {
			reportHeader	: {	visible	: false, rowNumber : null	},
			reportFooter	: {	visible	: false, rowNumber : null	},
			pageHeader		: {	visible	: false, rowNumber : null	},
			pageFooter		: {	visible	: false, rowNumber : null	},
			detailHeader	: {	visible	: false, rowNumber : null	},
			detailFooter	: {	visible	: false, rowNumber : null	},
			detail			: {	visible	: false, cached	: false	}, // same default style for all detail rows
			group			: new Array()		// styles for group headers, rest of the info is in .objects section
		},
		properties		: null,			// links to report in dax_bridge.reports
		data			: {
			moreRecords		: false,	// is there another page with records? returned as XML attribute
			recordCount		: 0,		// number of records received
			recordBatch		: 20,		// how many records to ask for
			currentRecord	: 0,		// first record number requested
			isFirstPage		: true,
			isLastPage		: true,
			queryId			: '',
			selection		: null,
			columns			: [ ]		// array of numbers of visible columns, determined by visibility in detailHeader elements
		},
		generatePosition	: null,		// report generates headers, then data rows, then footers, and this keeps track of current position
		useStylesFrom4D		: false,	// using styles from 4D will double the processing time
		rowDrawingBatchSize : 10,		// draw this many rows at once
		cssStyle			: null,
		isDashboard			: false
	}

	// form values sent to 4D, 4D gets them via GET WEB FORM VARIABLES 
	function dax_reportViewer_addCustomValue(customName, customValue) {
		this.customValues.name.push(customName);
		this.customValues.value.push(customValue);
	}
	
	this.addCustomValue = dax_reportViewer_addCustomValue;
	
	// clear custom values
	function dax_reportViewer_clearCustomValues() {
		this.customValues = {
			name	: [],
			value	: []
		}	
	}
	
	this.clearCustomValues = dax_reportViewer_clearCustomValues;
	
	function dax_reportviewer_onclose() {

		// stop refreshing
		if (this.refreshTimeout)
			clearTimeout(this.refreshTimeout);
	
	}
	
	this.onclose = dax_reportviewer_onclose;
	
	function dax_reportviewer_getAlign(value) {
		
		if (!value)
			return null;
			
		switch (value.toLowerCase()) {
			case 'left':
			case 'right':
			case 'center':
				return value.toLowerCase();
			break;
			default:
				return null;
			break;		
		}
	}
	
	this.getAlign = dax_reportviewer_getAlign;
	
	function dax_reportviewer_getreport(reportName, queryId, start, length) {
	
		this.toolbar.spinner.style.display = 'block';
	
		if (!this.report.properties) {
	
			// set window title
			this.window.setTitle(reportName);
	
			// find the report
			var report = null;
		
			for (var reportCount = 0; reportCount < dax_bridge.reports.length; reportCount++) {
				if (reportName == dax_bridge.reports[reportCount].name)
					report = dax_bridge.reports[reportCount];
			}
		
			if (!report) {
				debugAlert('report ' + reportName + ' not found.');
				return false;
			}

			this.report.properties = report;
		} else {
			var report = this.report.properties;

		}
	
		if (queryId)
			this.report.data.queryId = queryId;

		this.report.data.selection = report.table;

		if (start)
			this.report.data.currentRecord = start;

		if ((length) || (length == 0))
			this.report.data.recordBatch = length;
			
			
		var url = dax_bridge.url + "runReport?sessionid=" + dax_bridge.sessionId +
						"&tableid=" + this.report.data.selection.tableid +
						"&reportname=" + this.report.properties.name +
						"&queryid=" + this.report.data.queryId +
						"&start=" + ((this.report.data.currentRecord != 1) ? this.report.data.currentRecord : 0) +
						"&length=" + this.report.data.recordBatch +
						"&type=" + this.report.properties.type;

		// append custom values
		if (this.customValues.name.length > 0) {
			for (var customValueCount = 0; customValueCount < this.customValues.name.length; customValueCount++) {
				url += "&" + dax_bridge.encode(this.customValues.name[customValueCount]) + "=" + dax_bridge.encode(this.customValues.value[customValueCount]);
			}
		}
						
		makeCall (url, this.getReportHandler, this);
	}

	function dax_reportviewer_getreporthandler(http_result, reportObject) {
	
		if (http_result.readyState != 4)
			return;

		var responseError = checkResponse (http_result);
			
		if (responseError) {
			debugAlert ('Report loading error');
			return;
		}
			
		// parse XML
		var result = http_result.responseXML;
		var reportNode = result.getElementsByTagName('report')[0];
		var recordsReceived = 0;
		
		// report object
		var report = reportObject.report;
		
		report.cssStyle = (reportNode.getAttribute('csstheme') != 'default') ? reportNode.getAttribute('csstheme').toLowerCase() : null;
		
		var reportClassName = (reportObject.report.isDashboard ? 'dax_dashboard' : 'dax_reportviewer') + (report.cssStyle ? (' ' + report.cssStyle) : '');
		
		reportObject.setGridClass(null, reportClassName);

		
		// parse through report nodes
		for (var nodeCount = 0; nodeCount < reportNode.childNodes.length; nodeCount++) {
			var currentNode = reportNode.childNodes[nodeCount];
			switch (currentNode.nodeName) {
				case 'reportheader':
				case 'reportfooter':
				case 'pageheader':
				case 'pagefooter':
					var currentObject = {
						type		: currentNode.nodeName,
						style		: {
							align		: reportObject.getAlign(currentNode.getAttribute('align')),
							color		: currentNode.getAttribute('color'),
							font		: currentNode.getAttribute('font'),
							size		: currentNode.getAttribute('size'),
							className	: currentNode.nodeName,
							cached		: false
						},
						height		: currentNode.getAttribute('height'),
						visible		: ((currentNode.nodeName == 'pageheader') || (currentNode.nodeName == 'pagefooter')) ? currentNode.getAttribute('visible') : false,									// whether to currently display the object
						show		: currentNode.getAttribute('visible'),		// value sent from BE to indicate if object should be visible at all
						elements	: new Array(),
						cacheHTML	: null
					}
					for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
						var subNode = currentNode.childNodes[subNodeCount];
						switch (subNode.nodeName) {
							case 'graphic':
								var subObject = {
									style		: {
										align		: reportObject.getAlign(subNode.getAttribute('align')),
										inline		: subNode.getAttribute('inline'),
										className	: ''
									},
									type		: 'graphic',
									image		: subNode.getAttribute('image')
								}
								currentObject.elements.push(subObject);
							break;
							case 'chart':
								var subObject = {
									type		: 'chart',
									style		: {
										align		: reportObject.getAlign(subNode.getAttribute('align')),
										inline		: subNode.getAttribute('inline'),
										className	: ''
									},
									chartid		: subNode.getAttribute('chartid')
								}
								currentObject.elements.push(subObject);
							break;
							case 'line':
								var subObject = {
									type		: 'line',
									style		: {
										align		: reportObject.getAlign(subNode.getAttribute('align')),
										font		: subNode.getAttribute('font'),
										color		: subNode.getAttribute('color'),
										size		: subNode.getAttribute('size'),
										className	: ''
									},
									value		: reportObject.parseResponseText(getTextValue(subNode))
								}
								currentObject.elements.push(subObject);
							break;							
						}
					}
					if (currentNode.nodeName == 'reportheader')
						report.staticObjects.reportHeader = currentObject;
					else if (currentNode.nodeName == 'reportfooter')
						report.staticObjects.reportFooter = currentObject;
					else if (currentNode.nodeName == 'pageheader')
						report.staticObjects.pageHeader = currentObject;
					else if (currentNode.nodeName == 'pagefooter')
						report.staticObjects.pageFooter = currentObject;
				break;			
				case 'detailheader':
				case 'detailfooter':
					var currentObject = {
						type		: currentNode.nodeName,
						style		: {
							align		: reportObject.getAlign(currentNode.getAttribute('align')),
							className	: currentNode.nodeName,
							cached		: false
						},
						height		: currentNode.getAttribute('height'),
						visible		: currentNode.getAttribute('visible'),
						show		: currentNode.getAttribute('visible'),
						elements	: new Array()
					}
					for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
						var subNode = currentNode.childNodes[subNodeCount];
						switch (subNode.nodeName) {
							case 'field':
								var subObject = {
									value		: (!!subNode.getAttribute('value')) ? (subNode.getAttribute('value')) : reportObject.parseResponseText(getTextValue(subNode)),
									indicator	: subNode.getAttribute('class'),
									width		: subNode.getAttribute('width'),
									clickEvent	: subNode.getAttribute('iphonetype'),
									style		: {
										align		: reportObject.getAlign(subNode.getAttribute('align')),
										className	: currentNode.nodeName
									},
									visible		: (subNode.getAttribute('visible') ? subNode.getAttribute('visible') : 'true')
								}
																
								currentObject.elements.push(subObject);
							break;
						}
					}
					if (currentNode.nodeName == 'detailheader')
						report.staticObjects.detailHeader = currentObject;
					else if (currentNode.nodeName == 'detailfooter')
						report.staticObjects.detailFooter = currentObject;
				break;			
				case 'groupheader':
				case 'groupfooter':
					var currentObject = {
						type		: currentNode.nodeName,
						align		: reportObject.getAlign(currentNode.getAttribute('align')),
						fieldid		: currentNode.getAttribute('fieldid'),
						height		: currentNode.getAttribute('height'),
						id			: currentNode.getAttribute('id'),
						sort		: currentNode.getAttribute('sort'),
						visible		: currentNode.getAttribute('visible'),
						styleId		: currentNode.getAttribute('styleid'),
						elements	: new Array()
					}
					
					var objectExists = !!report.staticObjects.group[currentObject.styleId];
					
					if (!objectExists) {
						var staticObject = {
							style		: {
								align		: reportObject.getAlign(currentNode.getAttribute('align')),
								className	: currentNode.nodeName
							},
							elements	: new Array(),
							cached		: false
						}
					}

					if (currentObject.visible.toLowerCase() == 'true') {
					
						for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
							var subNode = currentNode.childNodes[subNodeCount];
							switch (subNode.nodeName) {
								case 'field':
									if (!objectExists) {
										var staticSubObject = {
											style		: {
												align			: reportObject.getAlign(subNode.getAttribute('align')),
												className		: currentNode.nodeName
											},
											cached		: false
										}
										staticObject.elements.push(staticSubObject);
									}
									var subObject = {
										formula		: subNode.getAttribute('formula'),
										label		: subNode.getAttribute('label'),
										value		: reportObject.parseResponseText(getTextValue(subNode)),
										clickEvent	: subNode.getAttribute('iphonetype'),
										indicator	: subNode.getAttribute('class'),
										visible		: subNode.getAttribute('visible')
									}

									currentObject.elements.push(subObject);

								break;
							}
						}
						report.objects.push(currentObject);
					}
					if (!objectExists)
						report.staticObjects.group[currentObject.styleId] = staticObject;
				break;			
				case 'detail':
					recordsReceived++;
					if (!report.staticObjects.detail.cached) {
						var currentStaticObject = {
							type		: 'detail',
							style		: {
								align			: reportObject.getAlign(currentNode.getAttribute('align')),
								className		: 'detail',
								altClassName	: 'detailalt',
								cached			: false
							},
							height		: currentNode.getAttribute('height'),
							visible		: currentNode.getAttribute('visible'),
							elements	: new Array(),
							cached		: true,
							rowNumber	: null		// first row number for detail section
						}
						for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
							var subNode = currentNode.childNodes[subNodeCount];
							switch (subNode.nodeName) {
								case 'field':
									var subObject = {
										clickEvent	: subNode.getAttribute('iphonetype'),
										type		: 'detail',
										style		: {
											align			: reportObject.getAlign(subNode.getAttribute('align')),
											className		: currentNode.nodeName
										}
									}
									currentStaticObject.elements.push(subObject);
								break;
							}
						}

						report.staticObjects.detail = currentStaticObject;
					}
					if (report.staticObjects.detail.visible.toLowerCase() == 'true') {
						var currentObject = {
							type		: 'detail',
							elements	: new Array()
						}
						for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
							var subNode = currentNode.childNodes[subNodeCount];
							switch (subNode.nodeName) {
								case 'field':
									var subObject = {
										type		: 'detail',
										value		: reportObject.parseResponseText(getTextValue(subNode)),
										indicator	: subNode.getAttribute('class')
									}
									currentObject.elements.push(subObject);
								break;
							}
						}
						report.objects.push(currentObject);
					}
				break;			
				case 'pagefooter':
					var currentStaticObject = {
						type		: 'detail',
						style		: {
							align			: reportObject.getAlign(currentNode.getAttribute('align')),
							className		: currentNode.nodeName
						},
						height		: currentNode.getAttribute('height'),
						visible		: currentNode.getAttribute('visible')
					}

					for (var subNodeCount = 0; subNodeCount < currentNode.childNodes.length; subNodeCount++) {
						var subNode = currentNode.childNodes[subNodeCount];
						switch (subNode.nodeName) {
							case 'pagenb':
							break;
							case 'currenttime':
							break;
							case 'currentdate':
							break;
						}
					}				

					report.staticObjects.pageFooter = currentStaticObject;
				break;			
			}
		}

		// record count
		report.data.moreRecords = reportNode.getAttribute('morerecords');
		
		report.data.recordCount = recordsReceived;

		// detail header & footer are visible on every page
		if (report.staticObjects.detailFooter.show == 'true')
			report.staticObjects.detailFooter.visible = 'true';

		if (report.staticObjects.detailHeader.show == 'true')
			report.staticObjects.detailHeader.visible = 'true';
		
		// update toolbar buttons
		// set visible elements
		if (report.data.moreRecords == 'false') {
			reportObject.toolbar.nextPage.disable();
			report.data.isLastPage = true;
			if (report.staticObjects.reportFooter.show == 'true')
				report.staticObjects.reportFooter.visible = 'true';
		} else {
			reportObject.toolbar.nextPage.enable();
			report.data.isLastPage = false;
			report.staticObjects.reportFooter.visible = 'false';
			//report.staticObjects.detailFooter.visible = 'false';
		}

		if (report.data.currentRecord < 2) {
			reportObject.toolbar.previousPage.disable();
			report.data.isFirstPage = true;
			if (report.staticObjects.reportHeader.show == 'true')
				report.staticObjects.reportHeader.visible = 'true';
		} else {
			reportObject.toolbar.previousPage.enable();
			report.data.isFirstPage = false;
			report.staticObjects.reportHeader.visible = 'false';
			//report.staticObjects.detailHeader.visible = 'false';
		}
		
		
		report.generatePosition = 'header';
		setTimeout(function() {reportObject.reportGenerate(reportObject)}, 0);

	}
	
	this.getReport = dax_reportviewer_getreport;
	this.getReportHandler = dax_reportviewer_getreporthandler;

	function dax_reportviewer_reportGenerate(reportObjectRef) {
	
		var report = reportObjectRef.report;
	
		if (report.generatePosition == 'header') {
			
			reportObjectRef.disableVisualRefresh();

			reportObjectRef.initializeReport('header');
			//reportObjectRef.generateStyles('header');
			reportObjectRef.populateReport('header');
			
			reportObjectRef.enableVisualRefresh(true);
			reportObjectRef.window.onsize();
		
			// start drawing data rows
			report.generatePosition = 0;

			setTimeout(function() {reportObjectRef.reportGenerate(reportObjectRef)}, 0);
			
		} else if (report.generatePosition == 'footer') {

			reportObjectRef.disableVisualRefresh();

			reportObjectRef.initializeReport('footer');
			//reportObjectRef.generateStyles('footer');
			reportObjectRef.populateReport('footer');
			
			reportObjectRef.enableVisualRefresh(true);
			reportObjectRef.window.onsize();		
		
			report.generatePosition = 'done';
			setTimeout(function() {reportObjectRef.reportGenerate(reportObjectRef)}, 0);
			
		} else if (report.generatePosition == 'done') {

			// set detail header as resizable
			var detailHeaderRowNumber = reportObjectRef.getRowNumber('detailheader');
			if ((detailHeaderRowNumber) || (detailHeaderRowNumber == 0))
				reportObjectRef.setHeaderRows(detailHeaderRowNumber, detailHeaderRowNumber, true);
		
			reportObjectRef.toolbar.spinner.style.display = 'none';
			report.initialized = true;
			reportObjectRef.setRefreshTimer();

		} else {

			var rowsToUpdate = (report.generatePosition + report.rowDrawingBatchSize > reportObjectRef.report.objects.length) ? (reportObjectRef.report.objects.length - report.generatePosition) : report.rowDrawingBatchSize;
			
			reportObjectRef.disableVisualRefresh();

			reportObjectRef.initializeReport(report.generatePosition, rowsToUpdate);
			//reportObjectRef.generateStyles(report.generatePosition, rowsToUpdate);
			reportObjectRef.populateReport(report.generatePosition, rowsToUpdate);

			reportObjectRef.enableVisualRefresh();
			reportObjectRef.redraw(false, true);
			reportObjectRef.window.onsize();

			if (reportObjectRef.report.objects.length > report.generatePosition) {
				// start drawing data rows
				report.generatePosition += ((rowsToUpdate + report.generatePosition) > reportObjectRef.report.objects.length) ? (reportObjectRef.report.objects.length - report.generatePosition) : rowsToUpdate;
			} else {
				report.generatePosition = 'footer';
			}
			
			setTimeout(function() {reportObjectRef.reportGenerate(reportObjectRef)}, 0);
			
		}
	}
	
	this.reportGenerate = dax_reportviewer_reportGenerate;
	
	// initialize report: grid size, styles
	function dax_reportviewer_initializeReport(populateArea, populateLength) {
		
		var report = this.report;
	
		if (populateArea == 'header') {

			// move the scroll position to the top
			this.viewport.vscroll(0, true);
		
			// size grid
			if (this.report.data.columns.length == 0) {
				var gridColCount = 0;
				for (var colCount = 0; colCount < report.staticObjects.detailHeader.elements.length; colCount++) {
					if (report.staticObjects.detailHeader.elements[colCount].visible.toLowerCase() == 'true') {
						gridColCount++;
						this.report.data.columns.push(colCount);
					}
				}
				var gridRowCount = (report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
					(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0) +
					(report.staticObjects.detailHeader.visible == 'true' ? 1 : 0);

				this.size(gridRowCount, this.report.data.columns.length);
			}
			
			var totalColWidth = 0;
			// set column widths
			for (var colCount = 0; colCount < this.report.data.columns.length; colCount++) {
				var colNumber = this.report.data.columns[colCount];
				var colWidth = +this.report.staticObjects.detailHeader.elements[colNumber].width;
				this.setColumnWidth(colCount, colWidth);
				totalColWidth += colWidth;
			}
			
			// resize if object is in window mode and on first load
			if (!this.report.initialized) {
				if (this.windowSettings.windowMode) {
					if (totalColWidth > 900) {
						var windowWidth = 900;
					} else if (totalColWidth < 300) {
						var windowWidth = 300;
					} else {
						var windowWidth = totalColWidth;
					}
				}
			}
			
			this.window.size(windowWidth + 47, null, true);
			
			// setup report header
			var reportHeaderRowNumber = this.getRowNumber('reportheader');
			if ((reportHeaderRowNumber) || (reportHeaderRowNumber == 0)) {
				this.setBlockRow(reportHeaderRowNumber, true, this.report.staticObjects.reportHeader.height);		
			}

			// setup page header
			var pageHeaderRowNumber = this.getRowNumber('pageheader');
			if ((pageHeaderRowNumber) || (pageHeaderRowNumber == 0)) {
				this.setBlockRow(pageHeaderRowNumber, true, this.report.staticObjects.pageHeader.height);		
			}

		} else if (populateArea == 'footer') {	

			var rowsToAdd =	(report.staticObjects.reportFooter.visible == 'true' ? 1 : 0) +
				(report.staticObjects.pageFooter.visible == 'true' ? 1 : 0) +
				(report.staticObjects.detailFooter.visible == 'true' ? 1 : 0);

			/*
			// add new rows if needed
			for (var rowCount = 0; rowCount < rowsToAdd; rowCount++) {
				if (!(this.rows[rowCount]))
					this.insertRow();				
			}
			*/	
			
			// setup report footer
			var reportFooterRowNumber = this.getRowNumber('reportfooter');
			if (reportFooterRowNumber) {
				
				if (!(this.rows[reportFooterRowNumber]))
					this.insertRow();
			
				this.setBlockRow(reportFooterRowNumber, true, this.report.staticObjects.reportFooter.height);		
			}

			// setup page footer
			var pageFooterRowNumber = this.getRowNumber('pagefooter');
			if (pageFooterRowNumber) {

				if (!(this.rows[pageFooterRowNumber]))
					this.insertRow();

				this.setBlockRow(pageFooterRowNumber, true, this.report.staticObjects.pageFooter.height);		
			}
		}
	}
	
	this.initializeReport = dax_reportviewer_initializeReport;

	// places data in cells
	function dax_reportviewer_populateReport(populateArea, populateLength) {
	
		var report = this.report;
	
		if (populateArea == 'header') {
	
			// report header
			var reportHeaderRowNumber = this.getRowNumber('reportheader');
			if ((reportHeaderRowNumber) || (reportHeaderRowNumber == 0)) {
				this.generateReportBlock(report.staticObjects.reportHeader, reportHeaderRowNumber);
			}		

			// page header
			var pageHeaderRowNumber = this.getRowNumber('pageheader');
			if ((pageHeaderRowNumber) || (pageHeaderRowNumber == 0)) {
				this.generateReportBlock(report.staticObjects.pageHeader, pageHeaderRowNumber);
			}		

			// detail header
			var detailHeaderRowNumber = this.getRowNumber('detailheader');
			if ((detailHeaderRowNumber) || (detailHeaderRowNumber == 0)) {

				// set row height
				if ((report.staticObjects.detailHeader.height) && (report.staticObjects.detailHeader.height != 1))
					this.setRowHeight(detailHeaderRowNumber, report.staticObjects.detailHeader.height);

				// populate	
				for (var columnCount = 0; columnCount < this.report.data.columns.length; columnCount++) {

					// not all columns are visible, get the next visible number
					var columnNumber = this.report.data.columns[columnCount];

					var cellRef = this.setCellValue(detailHeaderRowNumber, columnCount, report.staticObjects.detailHeader.elements[columnNumber].value);
					if (report.staticObjects.detailHeader.elements[columnNumber].style.align) {
						cellRef.style.textAlign = report.staticObjects.detailHeader.elements[columnNumber].style.align;
					} else if (report.staticObjects.detailHeader.style.align) {
						cellRef.style.textAlign = report.staticObjects.detailHeader.style.align;
					}
				}			
				// set style
				this.setStyleClass (detailHeaderRowNumber, null, report.staticObjects.detailHeader.style.className, report.staticObjects.detailHeader.style.className);			
			}			
		
		} else if (populateArea == 'footer') {

			// detail footer
			var detailFooterRowNumber = this.getRowNumber('detailfooter');
			if ((detailFooterRowNumber) || (detailFooterRowNumber == 0)) {

				// set row height
				if ((report.staticObjects.detailFooter.height) && (report.staticObjects.detailFooter.height != 1))
					this.setRowHeight(detailFooterRowNumber, report.staticObjects.detailFooter.height);

				// populate	
				for (var columnCount = 0; columnCount < this.report.data.columns.length; columnCount++) {

					// not all columns are visible, get the next visible number
					var columnNumber = this.report.data.columns[columnCount];

					if (!(this.rows[detailFooterRowNumber]))
						this.insertRow();
					
					var cellRef = this.setCellValue(detailFooterRowNumber, columnCount, report.staticObjects.detailFooter.elements[columnNumber].value);

					if (report.staticObjects.detailFooter.elements[columnNumber].clickEvent) {
						cellRef.clickEvent = report.staticObjects.detailFooter.elements[columnNumber].clickEvent;
						cellRef.style.textDecoration = 'underline';
					}

					if (report.staticObjects.detailFooter.elements[columnNumber].style.align)
						cellRef.style.textAlign = report.staticObjects.detailFooter.elements[columnNumber].style.align;
					else if (report.staticObjects.detailFooter.style.align)
						cellRef.style.textAlign = report.staticObjects.detailFooter.style.align;					
				}			
				// set style
				var indicatorStyle = report.staticObjects.detailFooter.indicator ? report.staticObjects.detailFooter.indicator : '';
				this.setStyleClass (detailFooterRowNumber, null, report.staticObjects.detailFooter.style.className + ' ' + indicatorStyle, report.staticObjects.detailFooter.style.className + ' ' + indicatorStyle);			


			}			
			
			// page footer
			var pageFooterRowNumber = this.getRowNumber('pagefooter');
			if ((pageFooterRowNumber) || (pageFooterRowNumber == 0)) {
				this.generateReportBlock(report.staticObjects.pageFooter, pageFooterRowNumber);
			}			
			// report footer
			var reportFooterRowNumber = this.getRowNumber('reportfooter');
			if ((reportFooterRowNumber) || (reportFooterRowNumber == 0)) {
				this.generateReportBlock(report.staticObjects.reportFooter, reportFooterRowNumber);
			}
		
		} else {
	
			// group header, footer and details
			var detailFirstRowNumber = this.getRowNumber('detail');
			if ((detailFirstRowNumber) || (detailFirstRowNumber == 0)) {
				// populate	
				for (var detailCount = populateArea; detailCount < populateLength + populateArea; detailCount++) {
					var rowNumber = detailFirstRowNumber + detailCount;
					
					if (!(this.rows[rowNumber]))
						this.insertRow();
					
					for (var columnCount = 0; columnCount < this.report.data.columns.length; columnCount++) {
						
						// not all columns are visible, get the next visible number
						var columnNumber = this.report.data.columns[columnCount];
						
						// set cell value
						var cellRef = this.setCellValue(rowNumber, columnCount, report.objects[detailCount].elements[columnNumber].value);
						
						// set cell styling and click events
						var styleClassName = report.objects[detailCount].type;
						var indicatorClass = report.objects[detailCount].elements[columnNumber].indicator ? report.objects[detailCount].elements[columnNumber].indicator : '';
						switch (report.objects[detailCount].type) {
							case 'groupheader':
							case 'groupfooter':
								if (report.objects[detailCount].elements[columnNumber].clickEvent) {
									cellRef.clickEvent = report.objects[detailCount].elements[columnNumber].clickEvent;
									cellRef.style.textDecoration = 'underline';
								}
									
								this.setStyleClass(rowNumber, columnCount, indicatorClass, indicatorClass);
								if (report.staticObjects.group[report.objects[detailCount].styleId].elements[columnNumber].style.align)
									cellRef.style.textAlign = report.staticObjects.group[report.objects[detailCount].styleId].elements[columnNumber].style.align;
								else if (report.staticObjects.group[report.objects[detailCount].styleId].style.align)
									cellRef.style.textAlign = report.staticObjects.group[report.objects[detailCount].styleId].style.align;
							break;
							case 'detail':
								if (report.staticObjects.detail.elements[columnNumber].clickEvent) {
									cellRef.clickEvent = report.staticObjects.detail.elements[columnNumber].clickEvent;
									cellRef.style.textDecoration = 'underline';
								}									
									
								this.setStyleClass(rowNumber, columnCount, indicatorClass, indicatorClass);
								if (report.staticObjects.detail.elements[columnNumber].style.align)
									cellRef.style.textAlign = report.staticObjects.detail.elements[columnNumber].style.align;
								else if (report.staticObjects.detail.style.align)
									cellRef.style.textAlign = report.staticObjects.detail.style.align;
							break;
						}
					}

					// set row height
					if ((report.objects[detailCount].height) && (report.objects[detailCount].height != 1))
						this.setRowHeight(rowNumber, report.objects[detailCount].height);

					if (report.objects[detailCount].type != 'detail')
						this.setStyleClass (rowNumber, null, report.objects[detailCount].type, report.objects[detailCount].type);			
					else
						this.setStyleClass (rowNumber, null, 'detail', 'detailalt');			

					/*	
					// set classes
					switch (report.objects[detailCount].type) {
						case 'groupheader':
						case 'groupfooter':
							break;
						case 'detail':
							this.setStyleClass (rowNumber, null, report.staticObjects.detail.style.className, report.staticObjects.detail.style.altClassName);			
						break;
					*/
					
				this.refreshStyles(rowNumber, null, true);
				}
			}			
		}
	}
	
	this.populateReport = dax_reportviewer_populateReport;

	function dax_reportviewer_populateReport_generateBlocks(staticBlockRef, rowNumber) {

		if (staticBlockRef.cacheHTML) {
		
			htmlToAdd = staticBlockRef.cacheHTML;
		
		} else {
	
			var htmlToAdd = '';
			// populate
			for (var elementCount = 0; elementCount < staticBlockRef.elements.length; elementCount++) {
				var elementNode = staticBlockRef.elements[elementCount];
				switch (elementNode.type) {
					case 'line':
						var lineString = '<div class="' + elementNode.style.className + '">' +
							elementNode.value + '</div>';
						htmlToAdd += lineString;
					break;
					case 'chart':
						var lineString = '<div style="font-size: 8pt">Chart goes here</div>';
						htmlToAdd += lineString;
					break;
					case 'graphic':
						var lineString = '<img src="' + dax_bridge.url + 'GetReportImage?image=' + elementNode.image + '&sessionid=' + dax_bridge.sessionId + '" />';
						htmlToAdd += lineString;
					break;
				}		
			}
			
			staticBlockRef.cacheHTML = htmlToAdd;
		}
		
		this.setCellValue(rowNumber, 0, htmlToAdd);
		this.setStyleClass(rowNumber, null, staticBlockRef.style.className, staticBlockRef.style.className);
	}
	
	this.generateReportBlock = dax_reportviewer_populateReport_generateBlocks;
	
	/*
	
	function dax_reportviewer_generateStyles(populateArea, populateLength) {
	
		// this number is used to generate unique style names
		var styleCount = 1;
	
		if (populateArea == 'header') {

			// styles for page & report header & footer elements
			var elementsToStyle = new Array();
		
			if (this.report.staticObjects.pageHeader.visible) {
				if (!this.report.staticObjects.pageHeader.style.cached) {
					this.report.staticObjects.pageHeader.style.cached = true;
					var className = 'dax_reportviewer_' + this.report.properties.id + '_pageheader pageheader';
					this.report.staticObjects.pageHeader.style.className = className;
					if (this.report.useStylesFrom4D)
						makeCSS('.' + className, 'border-width: 0px; background-color: #FFFFFF');

					for (var dataCount = 0; dataCount < this.report.staticObjects.pageHeader.elements.length; dataCount++) {
						elementsToStyle.push(this.report.staticObjects.pageHeader.elements[dataCount]);
					}
				}
			}
		
			// generate style for detail header
			if (!this.report.staticObjects.detailHeader.style.cached) {
				this.report.staticObjects.detailHeader.style.cached = true;
				var detailStyle = this.report.staticObjects.detailHeader.style;
				var styleName = 'dax_reportviewer_' + this.report.properties.id + '_detailheader detailheader';
				var styleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
					(detailStyle.backcolor ? 'background-color:' + detailStyle.backcolor + ';' : '') +
					(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
					(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
					(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
					'border-width: 0px;' +
					'white-space: nowrap;';

				detailStyle.className = styleName;
				if (this.report.useStylesFrom4D)
					makeCSS('.' + styleName, styleDef);
			}
			
			// styles for page & report header & footer elements
			if (this.report.staticObjects.reportHeader.visible) {
				if (!this.report.staticObjects.reportHeader.style.cached) {
					this.report.staticObjects.reportHeader.style.cached = true;
					var className = 'dax_reportviewer_' + this.report.properties.id + '_reportheader reportheader';
					this.report.staticObjects.reportHeader.style.className = className;
					if (this.report.useStylesFrom4D)
						makeCSS('.' + className, 'border-width: 0px; background-color: #FFFFFF');

					for (var dataCount = 0; dataCount < this.report.staticObjects.reportHeader.elements.length; dataCount++) {
						elementsToStyle.push(this.report.staticObjects.reportHeader.elements[dataCount]);
					}
				}
			}
			
			for (var dataCount = 0; dataCount < elementsToStyle.length; dataCount++) {
				var dataObject = elementsToStyle[dataCount];
				switch (dataObject.type) {
					case 'line':
						var detailStyle = dataObject.style;
						var styleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
							(detailStyle.backcolor ? 'background-color:' + detailStyle.backcolor + ';' : '') +
							(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
							(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
							(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
							'border-width: 0px;' +
							'white-space: nowrap;';
						detailStyle.className = 'dax_reportviewer_' + this.report.properties.id + '_' + dataObject.type + (styleCount++) + ' ' + dataObject.type;
						if (this.report.useStylesFrom4D)
							makeCSS('.' + detailStyle.className, styleDef);
					break;
				}
			}	

		} else if (populateArea == 'footer') {

			// generate style for detail footer
			if (!this.report.staticObjects.detailFooter.style.cached) {
				this.report.staticObjects.detailFooter.style.cached = true;
				var detailStyle = this.report.staticObjects.detailFooter.style;
				var styleName = 'dax_reportviewer_' + this.report.properties.id + '_detailfooter detailfooter';
				var styleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
					(detailStyle.backcolor ? 'background-color:' + detailStyle.backcolor + ';' : '') +
					(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
					(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
					(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
					'border-width: 0px;' +
					'white-space: nowrap;';

				detailStyle.className = styleName;
				if (this.report.useStylesFrom4D)
					makeCSS('.' + styleName, styleDef);
			}

			
			// styles for page & report header & footer elements
			var elementsToStyle = new Array();
			
			// styles for page & report header & footer elements

			if (this.report.staticObjects.reportFooter.visible) {
				if (!this.report.staticObjects.reportFooter.style.cached) {
					this.report.staticObjects.reportFooter.style.cached = true;
					var className = 'dax_reportviewer_' + this.report.properties.id + '_reportfooter reportfooter';
					this.report.staticObjects.reportFooter.style.className = className;
					if (this.report.useStylesFrom4D)
						makeCSS('.' + className, 'border-width: 0px; background-color: #FFFFFF');

					for (var dataCount = 0; dataCount < this.report.staticObjects.reportFooter.elements.length; dataCount++) {
						elementsToStyle.push(this.report.staticObjects.reportFooter.elements[dataCount]);
					}
				}
			}
			
			if (this.report.staticObjects.pageFooter.visible) {
				if (!this.report.staticObjects.pageFooter.style.cached) {
					this.report.staticObjects.pageFooter.style.cached = true;
					var className = 'dax_reportviewer_' + this.report.properties.id + '_pagefooter pagefooter';
					this.report.staticObjects.pageFooter.style.className = className;
					if (this.report.useStylesFrom4D)
						makeCSS('.' + className, 'border-width: 0px; background-color: #FFFFFF');

					for (var dataCount = 0; dataCount < this.report.staticObjects.pageFooter.elements.length; dataCount++) {
						elementsToStyle.push(this.report.staticObjects.pageFooter.elements[dataCount]);
					}
				}
			}
			
		} else {

			if (this.report.staticObjects.detail.visible) {
				if (!this.report.staticObjects.detail.style.cached) {
			
					this.report.staticObjects.detail.style.cached = true;
					// default detail style
					var detailStyle = this.report.staticObjects.detail.style;
					var styleName = 'dax_reportviewer_' + this.report.properties.id + '_detail detail';
					var altStyleName = 'dax_reportviewer_' + this.report.properties.id + '_detailalt detailalt';
					var styleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
						(detailStyle.backcolor ? 'background-color:' + detailStyle.backcolor + ';' : '') +
						(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
						(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
						(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
						'border-width: 0px;' +
						'white-space: nowrap;';
						
					var altStyleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
						(detailStyle.altbackcolor ? 'background-color:' + detailStyle.altbackcolor + ';' : '') +
						(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
						(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
						(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
						'border-width: 0px;' +
						'white-space: nowrap;';
						
					detailStyle.className = styleName;
					detailStyle.altClassName = altStyleName;

					if (this.report.useStylesFrom4D) {
						makeCSS('.' + styleName, styleDef);
						makeCSS('.' + altStyleName, altStyleDef);			
					}
					// generate styles for detail
					for (var cellCount = 0; cellCount < this.report.staticObjects.detail.elements.length; cellCount++) {
						cellStyle = this.report.staticObjects.detail.elements[cellCount].style;
						var styleDef = (cellStyle.align ? 'text-align:' + cellStyle.align + ';' : '') +
							(cellStyle.backcolor ? 'background-color:' + cellStyle.backcolor + ';' : '') +
							(cellStyle.color ? 'color:' + cellStyle.color + ';' : '') +	
							(cellStyle.font ? 'font-family:' + cellStyle.font + ';' : '') +	
							(cellStyle.size ? 'font-size:' + cellStyle.size + ';' : '');
						// if style exists, apply it
						if (styleDef) {
							var className = 'dax_reportviewer_' + this.report.properties.id + 'detail_cell' + (styleCount++);
							cellStyle.className = className;
							if (this.report.useStylesFrom4D)
								makeCSS('.' + className, styleDef);
						}
					}
				}
			}
			// generate styles for group header and footer
			
			for (var dataCount = 0; dataCount < this.report.objects.length; dataCount++) {
				var dataObject = this.report.objects[dataCount];
				switch (dataObject.type) {
					case 'groupheader':
					case 'groupfooter':
						var styleIsCached = this.report.staticObjects.group[dataObject.styleId].cached;
						if (!styleIsCached) {
							this.report.staticObjects.group[dataObject.styleId].cached = true;
							var detailStyle = this.report.staticObjects.group[dataObject.styleId].style;
							var styleDef = (detailStyle.align ? 'text-align:' + detailStyle.align + ';' : '') +
								(detailStyle.backcolor ? 'background-color:' + detailStyle.backcolor + ';' : '') +
								(detailStyle.color ? 'color:' + detailStyle.color + ';' : '') +	
								(detailStyle.font ? 'font-family:' + detailStyle.font + ';' : '') +	
								(detailStyle.size ? 'font-size:' + detailStyle.size + ';' : '') +
								'border-width: 0px;' +
								'white-space: nowrap;';
							detailStyle.className = 'dax_reportviewer_' + this.report.properties.id + '_group ' + dataObject.type;
							if (this.report.useStylesFrom4D)
								makeCSS('.' + detailStyle.className, styleDef);
							
							// cell styling
							for (var cellCount = 0; cellCount < dataObject.elements.length; cellCount++) {
								cellStyle = this.report.staticObjects.group[dataObject.styleId].elements[cellCount].style;
								this.report.staticObjects.group[dataObject.styleId].elements[cellCount].cached = true;
								var styleDef = (cellStyle.align ? 'text-align:' + cellStyle.align + ';' : '') +
									(cellStyle.backcolor ? 'background-color:' + cellStyle.backcolor + ';' : '') +
									(cellStyle.color ? 'color:' + cellStyle.color + ';' : '') +	
									(cellStyle.font ? 'font-family:' + cellStyle.font + ';' : '') +	
									(cellStyle.size ? 'font-size:' + cellStyle.size + ';' : '');
								// if style exists, apply it
								if (styleDef) {
									cellStyle.className = 'dax_reportviewer_' + this.report.properties.id + '_group_cell' + cellCount;
								if (!this.report.useStylesFrom4D)
									makeCSS('.' + cellStyle.className, styleDef);
								}
							}
						}
					break;
				}
			}
		}
	}
	
	this.generateStyles = dax_reportviewer_generateStyles;

	*/
	
	// shortcut to get row numbers
	function dax_reportviewer_getRowNumber(element) {
	
		var report = this.report;
	
		switch (element) {
			case 'reportheader':
				// row number cached
				if (report.staticObjects.reportHeader.rowNumber)
					return report.staticObjects.reportHeader.rowNumber;
				else {
					if (!report.data.isFirstPage)
						return null;
					// is row visible?
					if (report.staticObjects.reportHeader.visible != 'true')
						return null;
					// special case for 0
					if (report.staticObjects.reportHeader.rowNumber == 0)
						return 0;
					// assign row number
					report.staticObjects.reportHeader.rowNumber = 0;
					return report.staticObjects.reportHeader.rowNumber;
				}
			break;
			case 'reportfooter':
				// row number cached
				if (report.staticObjects.reportFooter.rowNumber)
					return report.staticObjects.reportFooter.rowNumber;
				else {
					if (!report.data.isLastPage)
						return null;
					// is row visible?
					if (report.staticObjects.reportFooter.visible != 'true')
						return null;
					// special case for 0
					if (report.staticObjects.reportFooter.rowNumber == 0)
						return 0;
					// assign row number
					report.staticObjects.reportFooter.rowNumber =
						(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.pageFooter.visible == 'true' ? 1 : 0) +
						(report.staticObjects.detailHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.detailFooter.visible == 'true' ? 1 : 0) +
						report.objects.length;

					return report.staticObjects.reportFooter.rowNumber;
				}
			break;
			case 'detail':
				// row number cached
				if (report.staticObjects.detail.rowNumber)
					return report.staticObjects.detail.rowNumber;
				else {
					// is row visible?
					//if (report.staticObjects.detail.visible != 'true')
					//	return null;
					// special case for 0
					if (report.staticObjects.detail.rowNumber == 0)
						return 0;
					// assign row number
					report.staticObjects.detail.rowNumber =
						(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.detailHeader.visible == 'true' ? 1 : 0);
					return report.staticObjects.detail.rowNumber;
				}
			break;
			case 'detailheader':

				// row number cached
//				if (report.staticObjects.detailHeader.rowNumber)
//					return report.staticObjects.detailHeader.rowNumber;
//				else {
				// is row visible?
				if (report.staticObjects.detailHeader.visible != 'true')
					return null;
				// special case for 0
				if (report.staticObjects.detailHeader.rowNumber == 0)
					return 0;
				// assign row number
				report.staticObjects.detailHeader.rowNumber =
					(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
					(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0);
				return report.staticObjects.detailHeader.rowNumber;
//				}
			break;
			case 'pageheader':
				if (report.staticObjects.pageHeader.rowNumber)
					return report.staticObjects.pageHeader.rowNumber;
				else {
					// is row visible?
					if (report.staticObjects.pageHeader.visible != 'true')
						return null;
					// special case for 0
					if (report.staticObjects.pageHeader.rowNumber == 0)
						return 0;
					// assign row number
					report.staticObjects.pageHeader.rowNumber =
						(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0);

					return report.staticObjects.pageHeader.rowNumber;
				}
			break;
			case 'pagefooter':
				if (report.staticObjects.pageFooter.rowNumber)
					return report.staticObjects.pageFooter.rowNumber;
				else {
					// is row visible?
					if (report.staticObjects.pageFooter.visible != 'true')
						return null;
					// special case for 0
					if (report.staticObjects.pageFooter.rowNumber == 0)
						return 0;
					// assign row number
					report.staticObjects.pageFooter.rowNumber =
						(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.detailHeader.visible == 'true' ? 1 : 0) +
						(report.staticObjects.detailFooter.visible == 'true' ? 1 : 0) +
						report.objects.length;

					return report.staticObjects.pageFooter.rowNumber;
				}
			break;
			case 'detailfooter':

//				if (report.staticObjects.detailFooter.rowNumber)
//					return report.staticObjects.detailFooter.rowNumber;
//				else {
					// is row visible?
				if (report.staticObjects.detailFooter.visible != 'true')
					return null;
				// special case for 0
				//if (report.staticObjects.detailFooter.rowNumber == 0)
				//	return 0;
				// assign row number
				report.staticObjects.detailFooter.rowNumber =
					(report.staticObjects.reportHeader.visible == 'true' ? 1 : 0) +
					(report.staticObjects.pageHeader.visible == 'true' ? 1 : 0) +
					(report.staticObjects.detailHeader.visible == 'true' ? 1 : 0) +
					report.objects.length;

				return report.staticObjects.detailFooter.rowNumber;
//				}
			break;
		}
	}
	
	this.getRowNumber = dax_reportviewer_getRowNumber;
	
	function dax_reportviewer_pageChangeCommon() {
	
		var staticObjects = this.report.staticObjects;

		if ((staticObjects.reportHeader.rowNumber) || (staticObjects.reportHeader.rowNumber == 0))
			if (!isNaN(staticObjects.reportHeader.rowNumber))
				if (this.rows[staticObjects.reportHeader.rowNumber].isBlock)
					this.setBlockRow(staticObjects.reportHeader.rowNumber, false, 1);		

		if ((staticObjects.reportFooter.rowNumber) || (staticObjects.reportFooter.rowNumber == 0))
			if (!isNaN(staticObjects.reportFooter.rowNumber))
				if (this.rows[staticObjects.reportFooter.rowNumber].isBlock)
					this.setBlockRow(staticObjects.reportFooter.rowNumber, false, 1);		

		if ((staticObjects.pageHeader.rowNumber) || (staticObjects.pageHeader.rowNumber == 0))
			if (!isNaN(staticObjects.pageHeader.rowNumber))
				if (this.rows[staticObjects.pageHeader.rowNumber].isBlock)
					this.setBlockRow(staticObjects.pageHeader.rowNumber, false, 1);		

		if ((staticObjects.pageFooter.rowNumber) || (staticObjects.pageFooter.rowNumber == 0))
			if (!isNaN(staticObjects.pageFooter.rowNumber))
				if (this.rows[staticObjects.pageFooter.rowNumber].isBlock)
					this.setBlockRow(staticObjects.pageFooter.rowNumber, false, 1);		
				
				
		// clear row number cache
		staticObjects.reportHeader.rowNumber = null;
		staticObjects.reportFooter.rowNumber = null;
		staticObjects.pageHeader.rowNumber = null;
		staticObjects.pageFooter.rowNumber = null;
		staticObjects.detailHeader.rowNumber = null;
		staticObjects.detail.rowNumber = null;

		this.report.objects = [ ];

	}
	
	this.pageChangeCommon = dax_reportviewer_pageChangeCommon;
	
	function dax_reportviewer_getNextPage() {
	
		this.pageChangeCommon();
		if (this.report.data.currentRecord < 1)
			this.report.data.currentRecord = 1;
		this.report.data.currentRecord += this.report.data.recordBatch;
		this.getReport();		
	}
	
	this.getNextPage = dax_reportviewer_getNextPage;

	function dax_reportviewer_getPreviousPage() {
	
		this.pageChangeCommon();
		this.report.data.currentRecord -= this.report.data.recordBatch;
		if (this.report.data.currentRecord < 1)
			this.report.data.currentRecord = 1;
		this.getReport();		
	}
	
	this.getPreviousPage = dax_reportviewer_getPreviousPage;
	
	function dax_reportviewer_parseResponseText(textIn) {
		if (!textIn)
			return '';
		var textOut = ReplaceString(textIn, '##BR', '<br />')
		return textOut;
	}
	
	this.parseResponseText = dax_reportviewer_parseResponseText;
	
	function dax_reportviewer_showToolbar(showToolbar){
		
		if (showToolbar)
			this.toolbar.prv.mDOM.style.display = 'block';
		else
			this.toolbar.prv.mDOM.style.display = 'none';
			
		this.window.resize();

	}
	
	this.showToolbar = dax_reportviewer_showToolbar;
	
	function dax_reportviewer_refresh() {

		this.pageChangeCommon();
		this.getReport();		

	}
	
	this.refresh = dax_reportviewer_refresh;
	
	function dax_reportviewer_setRefreshTimer(){
		// set refresh timer
		if (this.refreshInterval > 0) {
		
			if (this.refreshTimeout)
				clearTimeout(this.refreshTimeout);
		
			var reportViewerObject = this;
			var refreshInterval = this.refreshInterval;
		
			this.refreshTimeout = setTimeout(function() { reportViewerObject.refresh(); }, refreshInterval);

		}
	}
	
	this.setRefreshTimer = dax_reportviewer_setRefreshTimer;	
	
	function dax_reportViewer_setRefreshInterval(refreshInterval) {
		// if refresh time is less than 30sec, set it to 30 sec.
		this.refreshInterval = (refreshInterval < 10) ? (1000 * 10) : refreshInterval;

	}
	
	this.setRefreshInterval = dax_reportViewer_setRefreshInterval;
	
	function dax_reportviewer_refreshBox_onmouseup(event) {
		var event = xpEvent(event, true);
		// WindowManager.hideWindow(this.parent);
		this.className = "window_refreshbox";
		this.parent.parent.refresh();
		return false;
	};
	
	this.window.refreshBox.onmouseup = dax_reportviewer_refreshBox_onmouseup;
	
	
	function dax_reportViewer_onCellClick(row, column) {
		var clickedCell = this.getCell(row, column);
		
		if (!clickedCell)
			return false;
		
		switch(clickedCell.clickEvent) {
			case 'link':
				var goToUrl = clickedCell.value;
				window.open(goToUrl);
			break;
			case 'email':
				var goToUrl = 'mailto:' + clickedCell.value;
				window.open(goToUrl);
			break;
			case 'maps':
				var goToUrl = 'http://maps.google.com/maps?q=' + clickedCell.value;
				window.open(goToUrl);
			break;
		}
	}
	
	// click handler
	this.onCellClick = dax_reportViewer_onCellClick;
}

// Dashboard wrapper
function dax_dashboardViewer(reportName, targetNode){

	// intialize the report
	dax_reportViewer.apply(this, [targetNode]);
	this.report.isDashboard = true;
	this.setRowHeightInPx(27);
	
	// hide toolbar
	this.showToolbar(false);
	this.setGridClass(null, 'dax_dashboard');
	
	// load report
	this.getReport(reportName, null, null, 0);
}

// Chart
function dax_chartViewer(reportName, targetNode) {

	this.refreshInterval = 1000 * 60 * 3; // 3 minutes
	// refresh interval

	this.windowSettings = {
		windowMode			: true,
		parentContainer		: null
	};

	this.chart = {
		properties	: null,
		data		: {				// canvas properties
			x		: [ ],
			y		: [ ],
			yLabel	: [ ],
			minY	: 0,
			maxY	: 0
		},
		scale		: {				// canvas properties
			x			: 1000,
			y			: 500,
			thickLine	: 2,
			thinLine	: 2,
			chartLine	: 3
		},
		name			: reportName,
		type			: null,			// canvas, svg, or png
		initialized		: null,
		urls			: [ ],			// for non canvas type
		currentUrl		: null			// which url is currently displayed
	}

	// sent as additional attributes in url
	this.customValues = {
		name		: [],
		value		: []
	}	

	// form values sent to 4D, 4D gets them via GET WEB FORM VARIABLES 
	function dax_chartViewer_addCustomValue(customName, customValue) {
		this.customValues.name.push(customName);
		this.customValues.value.push(customValue);
	}
	
	this.addCustomValue = dax_chartViewer_addCustomValue;
	
	// clear custom values
	function dax_chartViewer_clearCustomValues() {
		this.customValues = {
			name	: [],
			value	: []
		}	
	}
	
	this.clearCustomValues = dax_chartViewer_clearCustomValues;
	
	function dax_chartviewer_initializeSVG() {

		this.nodes = {
			svg		: null
		}
	
		this.activate();
		
		this.chart.initialized = true;
		
	}
	
	this.initSVG = dax_chartviewer_initializeSVG;
	
	
	function dax_chartviewer_initializeImage() {

		this.nodes = {
			image			 : getTemplate('dax_chart_image').cloneNode(true)
		}
		
		var imageNode = this.nodes.image;
		
		generateUniqueId(imageNode);
		imageNode.style.display = 'block';
		
		removeChildren(this.window.contentArea);
		this.window.contentArea.appendChild(this.nodes.image);
		this.activate();
		
		this.chart.initialized = true;
		
	}
	
	this.initImage = dax_chartviewer_initializeImage;
	
	function dax_chartviewer_initializeCanvas() {

		this.nodes = {
			graphPatternImg : new Image(),
			chartNode 		: getTemplate('dax_chart').cloneNode(true),
			labelsY			: [ ]
		}

		this.window.contentArea.appendChild(this.nodes.chartNode);
		this.nodes.chartNode.style.display = 'block';
		
		var chartCanvasNode = document.createElement('canvas');
		chartCanvasNode.width = 1000;
		chartCanvasNode.height = 1000;
		chartCanvasNode.className = 'chart_canvas';
		generateUniqueId(this.nodes.chartNode);
		
		getElementByClassName(this.nodes.chartNode, 'chart_placeholder').appendChild(chartCanvasNode);

		if (browser.ie)
			G_vmlCanvasManager.initElement(chartCanvasNode);
		
		this.nodes.labelX = getElementByClassName(this.nodes.chartNode, 'chart_x_label', true);
		this.nodes.canvas = getElementByClassName(this.nodes.chartNode, 'chart_canvas', true);
		
		this.nodes.canvas.width = this.chart.scale.x;
		this.nodes.canvas.height = this.chart.scale.y;

		for (var labelCount = 0; labelCount < 5; labelCount++) {
			this.nodes.labelsY.push(getElementByClassName(this.nodes.chartNode, 'chart_y_label_' + (5 - labelCount), true));
		}
		
		// initialize chart
		this.activate();

		this.nodes.canvas.width = this.nodes.canvas.clientWidth;
		this.nodes.canvas.height = this.nodes.canvas.clientHeight;

		this.chart.scale.x = this.nodes.canvas.clientWidth;
		this.chart.scale.y = this.nodes.canvas.clientHeight;

		this.chart.initialized = true;
	
	}
	
	this.initCanvas = dax_chartviewer_initializeCanvas;
	
	this.window = WindowManager.createNewWindow();

	function dax_reportViewer_onWindowClose() {
		this.parent.onclose();
	}
	
	this.window.onclose = dax_reportViewer_onWindowClose;
	
	this.window.onsize = function(){
		this.parent.redraw();
	}

	this.window.onsizestart = function() {
		if (this.parent.chart.type == 'svg'){
			this.parent.nodes.svg.style.display = 'none';
		}	
	}
	
	this.window.onsizestop = function() {
	
		if (this.parent.chart.type == 'canvas') {
	
			if(browser.ie) {

				removeChildren(getElementByClassName(this.parent.nodes.chartNode, 'chart_placeholder'));
			
				var chartCanvasNode = document.createElement('canvas');
				chartCanvasNode.width = 1000;
				chartCanvasNode.height = 1000;
				chartCanvasNode.className = 'chart_canvas';
				generateUniqueId(this.parent.nodes.chartNode);

				getElementByClassName(this.parent.nodes.chartNode, 'chart_placeholder').appendChild(chartCanvasNode);

				if (browser.ie)
					G_vmlCanvasManager.initElement(chartCanvasNode);

				this.parent.nodes.labelX = getElementByClassName(this.parent.nodes.chartNode, 'chart_x_label', true);
				this.parent.nodes.canvas = getElementByClassName(this.parent.nodes.chartNode, 'chart_canvas', true);

				this.parent.nodes.canvas.width = this.parent.chart.scale.x;
				this.parent.nodes.canvas.height = this.parent.chart.scale.y;

				this.parent.redraw();
			
				this.parent.nodes.canvas.width = this.parent.nodes.canvas.clientWidth;
				this.parent.nodes.canvas.height = this.parent.nodes.canvas.clientHeight;

				this.parent.chart.scale.x = this.parent.nodes.canvas.clientWidth;
				this.parent.chart.scale.y = this.parent.nodes.canvas.clientHeight;	
				
				this.parent.getChart();
				
				
			}
		} else if (this.parent.chart.type == 'svg'){
			this.parent.nodes.svg.style.display = 'block';
		}	
	}
	
	function dax_chartviewer_onclose() {

		// stop refreshing
		if (this.refreshTimeout)
			clearTimeout(this.refreshTimeout);
	
	}
	
	this.onclose = dax_chartviewer_onclose;
	
	this.window.parent = this;

	if (targetNode) {
		this.windowSettings.parentContainer = targetNode;
		this.windowSettings.windowMode = false;
	}

	function dax_chartViewer_scale(x, y, chartLine, thickLine, thinLine) {

		if (x)
			this.chart.scale.x = x;

		if (y)
			this.chart.scale.y = y;

		if (chartLine)
			this.chart.scale.chartLine = chartLine;

		if (thickLine)
			this.chart.scale.thickLine = thickLine;

		if (thinLine)
			this.chart.scale.thinLine = thinLine;
			
		this.nodes.canvas.width = this.chart.scale.x;
		this.nodes.canvas.height = this.chart.scale.y;

		this.drawChart();
		
	}
	
	this.scale = dax_chartViewer_scale;
	
	this.activate = function(){
		this.window.activate();
		this.window.open();	
	
		// add window to node
		if (!this.windowSettings.windowMode) {
		
			// clear the node contents
			while (this.windowSettings.parentContainer.firstChild)
				this.windowSettings.parentContainer.removeChild(this.windowSettings.parentContainer.firstChild);
		
			this.windowSettings.parentContainer.appendChild(this.window.dom);
			this.window.dom.style.position = 'relative';
			this.window.dom.style.top = '0px';
			this.window.dom.style.left = '0px';
			this.window.dom.top.style.display = 'none';
			this.window.dom.btm.style.display = 'none';
			this.window.dom.l.style.display = 'none';
			this.window.dom.r.style.display = 'none';
		}

		if (!this.windowSettings.windowMode) {
			//this.window.dom.style.display = 'none';
			this.window.size(this.windowSettings.parentContainer.clientWidth, this.windowSettings.parentContainer.clientHeight, true);
		}
	
		this.window.onsize();
	}

	function dax_chartViewer_getChart(chartName) {

		if (!this.chart.properties) {
	
			// set window title
			this.window.setTitle(chartName);
	
			// find the report
			var report = null;
		
			for (var reportCount = 0; reportCount < dax_bridge.reports.length; reportCount++) {
				if (chartName == dax_bridge.reports[reportCount].name)
					report = dax_bridge.reports[reportCount];
			}
		
			if (!report) {
				debugAlert('report ' + reportName + ' not found.');
				return false;
			}

			this.chart.properties = report;
		} else {
			var chart = this.chart.properties;

		}
	
		var url = dax_bridge.url + "runReport?sessionid=" + dax_bridge.sessionId +
						"&tableid=" + this.chart.properties.table.tableid +
						"&reportname=" + dax_bridge.encode(this.chart.properties.name) +
						"&start=0" +
						"&length=1" +
						"&type=" + this.chart.properties.type;

		// append custom values
		if (this.customValues.name.length > 0) {
			for (var customValueCount = 0; customValueCount < this.customValues.name.length; customValueCount++) {
				url += "&" + dax_bridge.encode(this.customValues.name[customValueCount]) + "=" + dax_bridge.encode(this.customValues.value[customValueCount]);
			}
		}
						
		makeCall (url, this.getChartHandler, this);

	
	}
	
	this.getChart = dax_chartViewer_getChart;
	
	function dax_chartViewer_getChartHandler(http_result, chartObject) {

		if (http_result.readyState != 4)
			return;

		var responseError = checkResponse (http_result);
			
		if (responseError) {
			debugAlert ('Report loading error');
			return;
		}
			
		// parse XML
		var result = http_result.responseXML;
		var reportNode = result.getElementsByTagName('report')[0];
		
		var chart = chartObject.chart.data;
		var useChart = (reportNode.getAttribute('usechart') == 'true') ? true : false;
		
		if (useChart) {
			var chartXMLNode = reportNode.getElementsByTagName('dashchart')[0];
			
			// did chart XML arrive?
			if (!chartXMLNode)
				return false;
		
			var canvasType = chartXMLNode.getAttribute('type');

			chartObject.chart.type = canvasType;
			
			// branch out based on the chart type
			switch (chartObject.chart.type) {
			
				case 'canvas':
					if (!chartObject.chart.initialized)
						chartObject.initCanvas();

					chart.x = (chartXMLNode.getElementsByTagName('Xlabel')[0]).firstChild.nodeValue.split(',');
					chart.y = (chartXMLNode.getElementsByTagName('value')[0]).firstChild.nodeValue.split(',');
					chart.yLabel = (chartXMLNode.getElementsByTagName('Ylabel')[0]).firstChild.nodeValue.split(',');
					chart.minY = +chartXMLNode.getElementsByTagName('value')[0].getAttribute('min');
					chart.maxY = +chartXMLNode.getElementsByTagName('value')[0].getAttribute('max');

						
				break;
				
				case 'png':
				case 'urlimage':
				case 'svg':

					if (chartObject.chart.type != 'svg')
						chartObject.initImage();
					else
						chartObject.initSVG();
					
					// get list of images
					chartObject.chart.urls = [ ];
					chartObject.chart.currentUrl = 0;
					
					var chartLinks = chartXMLNode.getElementsByTagName('link');
					
					for (var chartLinkCount = 0; chartLinkCount < chartLinks.length; chartLinkCount++) {
						chartObject.chart.urls.push(chartLinks[chartLinkCount].getAttribute('url'));
					}

				break;
			}

			chartObject.drawChart();

		}
	}
	
	this.getChartHandler = dax_chartViewer_getChartHandler;

	function dax_chartViewer_drawChart() {
	
		if ((this.chart.type == 'png') || (this.chart.type == 'urlimage')) {
			this.drawImage();
			return true;
		}

		if (this.chart.type == 'svg') {
			this.drawSVG();
			return true;
		}

		var chartNode = this.nodes.chartNode;
		var chart = this.chart.data;
	
		for (var labelCount = 0; labelCount < 5; labelCount++) {
			this.nodes.labelsY[labelCount].innerHTML = chart.yLabel[labelCount];
		}
		
		var labelCount = chart.x.length;
		switch (chart.x.length) {
			case 1:
				var labelXclassName = 'chart_x_one_label';
			break;
			case 2:
				var labelXclassName = 'chart_x_two_labels';
			break;
			case 3:
				var labelXclassName = 'chart_x_three_labels';
			break;
			case 4:
				var labelXclassName = 'chart_x_four_labels';
			break;
			default:
				var labelXclassName = 'chart_x_five_labels';
				var labelCount = 5;
			break;
		}

		this.nodes.labelX.className = labelXclassName + ' chart_x_label';

		for (var xLabelCount = 0; xLabelCount < labelCount; xLabelCount++) {
			getElementByClassName(this.nodes.labelX, 'chart_x_label_' + (+xLabelCount + 1), true).innerHTML = chart.x[xLabelCount];			
		}

				
		var canvas = this.nodes.canvas;
		var ctx = canvas.getContext('2d');

		ctx.clearRect(0,0, this.chart.scale.x, this.chart.scale.y);
		
		var verScale = this.chart.scale.y / (chart.maxY - chart.minY);

        ctx.strokeStyle = "rgb(128,128,128)";

        ctx.beginPath();
		ctx.lineWidth = this.chart.scale.thinLine;
		ctx.moveTo(0,(this.chart.scale.y / 4));
		ctx.lineTo(this.chart.scale.x, (this.chart.scale.y / 4));
		ctx.moveTo(0,(this.chart.scale.y * 3 / 4));
		ctx.lineTo(this.chart.scale.x, (this.chart.scale.y * 3 / 4));
		ctx.moveTo(0,(this.chart.scale.y / 2));
		ctx.lineTo(this.chart.scale.x, (this.chart.scale.y / 2));
		ctx.stroke();		

		
        ctx.strokeStyle = "rgb(255,255,255)";
       // ctx.fillStyle = "rgb(128,128,128)";

		
        ctx.beginPath();
		ctx.lineWidth = this.chart.scale.thickLine;
		ctx.moveTo(0,0);
		ctx.lineTo(0, this.chart.scale.y);
		ctx.lineTo(this.chart.scale.x, this.chart.scale.y);
		ctx.stroke();

		// create pattern
		//var ptrn = ctx.createPattern(this.nodes.graphPatternImg,'repeat');
		//ctx.fillStyle = ptrn;
		
		/*
		ctx.beginPath();
		ctx.moveTo(0, this.chart.scale.y);
		ctx.lineTo(0, this.chart.scale.y - (chart.y[0] - chart.minY) * verScale);
		for (var lineCount = 1; lineCount < chart.y.length; lineCount++) {
			var dotX = (this.chart.scale.x / chart.y.length) * (lineCount + 1);
			var dotY = (chart.y[lineCount] - chart.minY) * verScale;
			ctx.lineTo(dotX, this.chart.scale.y - dotY);
		}
		ctx.lineTo(this.chart.scale.x, this.chart.scale.y);
		ctx.fill();
		*/
		
		
		ctx.lineWidth = this.chart.scale.chartLine;
		ctx.lineJoin = 'round';
		ctx.lineCap = 'round';
        ctx.beginPath();
		ctx.moveTo(0, this.chart.scale.y - (chart.y[0] - chart.minY) * verScale);
		for (var lineCount = 1; lineCount < chart.y.length; lineCount++) {
			var dotX = (this.chart.scale.x / chart.y.length) * (lineCount + 1);
			var dotY = (chart.y[lineCount] - chart.minY) * verScale;
			ctx.lineTo(dotX, this.chart.scale.y - dotY);
		}
		ctx.stroke();
		
		// call next chart
		this.setRefreshTimer();
		
	}
	
	this.drawChart = dax_chartViewer_drawChart;
	
	function dax_chartViewer_redraw() {
	
		if (this.chart.type == 'canvas') {
	
			var canvasWidth = this.window.contentArea.offsetWidth - 50;
			var canvasHeight = this.window.contentArea.offsetHeight - 50;
		
			this.nodes.canvas.style.width = canvasWidth + 'px';
			this.nodes.canvas.style.height = canvasHeight+ 'px';

			this.nodes.labelX.style.width = canvasWidth + 'px';
			
			for (var labelCount = 0; labelCount < 5; labelCount++) {
				this.nodes.labelsY[labelCount].style.top = Math.round(-100 - (canvasHeight - 100) * labelCount / 4) + 'px';
			}
		} else if (this.chart.type == 'svg') {
		
			if (!this.nodes.svg)
				return false;
		
			this.nodes.svg.width = this.window.contentArea.clientWidth;
			this.nodes.svg.height = this.window.contentArea.clientHeight;
		}
	}
	
	this.redraw = dax_chartViewer_redraw;
	
	function dax_chartViewer_drawImage() {
	
		if (this.chart.urls[this.chart.currentUrl])
			this.nodes.image.src = this.chart.urls[this.chart.currentUrl] + '&random=' + Math.random();
	
		this.setRefreshTimer();
	
	}
	
	this.drawImage = dax_chartViewer_drawImage;

	function dax_chartViewer_drawSVG() {
	
		if (!this.chart.urls[this.chart.currentUrl])
			return false;
			
		var chartUrl = this.chart.urls[this.chart.currentUrl] + '&random=' + Math.random();	
			
		var svgNode	= document.createElement('object');
		svgNode.width = 500;
		svgNode.height = 500;
		svgNode.type = 'image/svg+xml';
		svgNode.data = chartUrl;
		
		this.nodes.svg = svgNode;
			
		removeChildren(this.window.contentArea);	
		this.window.contentArea.appendChild(svgNode);	

		this.setRefreshTimer();
		
	}
	
	this.drawSVG = dax_chartViewer_drawSVG;	
	
	function dax_reportchart_setRefreshTimer(){
		// set refresh timer
		if (this.refreshInterval > 0) {
		
			if (this.refreshTimeout)
				clearTimeout(this.refreshTimeout);
		
			var reportChartObject = this;
			var refreshInterval = this.refreshInterval;
		
			this.refreshTimeout = setTimeout(function() { reportChartObject.refresh(); }, refreshInterval);

		}
	}
	
	this.setRefreshTimer = dax_reportchart_setRefreshTimer;	
	
	function dax_reportChart_setRefreshInterval(refreshInterval) {
		// if refresh time is less than 5sec, set it to 5 sec.
		this.refreshInterval = (refreshInterval < 5) ? (1000 * 5) : (1000 * refreshInterval);
		this.refresh();	
	}
	
	this.setRefreshInterval = dax_reportChart_setRefreshInterval;
	
	function dax_reportchart_refreshBox_onmouseup(event) {
		var event = xpEvent(event, true);
		// WindowManager.hideWindow(this.parent);
		this.className = "window_refreshbox";
		this.parent.parent.refresh();
		return false;
	};
	
	this.window.refreshBox.onmouseup = dax_reportchart_refreshBox_onmouseup;

	function dax_reportchart_refresh() {
		this.getChart(this.chart.name);
	}
	
	this.refresh = dax_reportchart_refresh;
	
	this.getChart(this.chart.name);
	
}

// load canvas libary for IE only
if (browser.ie) {

	var e = document.createElement("script");
	e.src = "dax/external/excanvas-compressed.js";
	e.type="text/javascript";
	document.getElementsByTagName("head")[0].appendChild(e); 
}
