/**
 * Sidebar
 *
 * @author Rudolf Psenicnik
 *
 */

function Sidebar()
{

	// if sidebar doesn't exist, create one
	if ($('dax_sidebar0') != null) {
		return;
	}

	var sidebarNode = getTemplate('dax_sidebar');

	if (!sidebar)
		return false;

	this.dom = sidebarNode.cloneNode(true);
	this.viewport = new ViewPort ();
	
	// add sidebar to the document
	document.body.appendChild (this.dom);
	
	// set unique id and make it visible
	this.dom.id = 'dax_sidebar0';
	this.dom.parent = this;
	setStyle (this.dom, 'display', 'block');
	
	// get details about the object
	this.close = getElementByClassName (this.dom, 'close');
	this.contents = getElementByClassName (this.dom, 'contents');
	this.tile = getElementByClassName (this.dom, 'tile');
	this.l = getElementByClassName (this.dom, 'l');
	this.r = getElementByClassName (this.dom, 'r');
	this.t = getElementByClassName (this.dom, 't');
	this.b = getElementByClassName (this.dom, 'b');
	
	// setup viewport
	this.tile.appendChild (this.viewport.dom);
	this.viewport.parent = this.tile;
	this.viewport.setContent (this.contents);
	this.viewport.setOverflow (0,0);
	this.viewport.boundTo = this;
	this.viewport.resizeOnRefresh = false;
	this.viewport.owner = this;	
	
	// set properties
	this.state = 'hidden';
	this.animating = false;

	// buttons on the bottom of the sidebar
	this.buttons = new Array();
	
	// fix for ie6
	if (browser.ie6) {
		this.dom.style.position = 'absolute';
		//this.tile.style.paddingTop = '25px';
	}

	// set viewport scroll function
	this.viewport.onvscroll = function(aTop, aIsScrollDown)
	{
		// calculate height
		var calcHeight = (aTop * (-1) * this.vscrollbar.ratio);
		
		// reposition the contents
		this.boundTo.contents.style.top = calcHeight + "px";
		//setStyle(this.boundTo.contents.style, "top", calcHeight + "px");
	};
	
	this.redraw = function (ignoreHorPosition) {

		// set horizontal position
		if (!ignoreHorPosition) {
			if (this.state == 'hidden') {
				this.dom.style.left = '-' + this.dom.offsetWidth + 'px';
			} else {
				this.dom.style.left = '0px';
			}
		}
		
		// set vertial position
		if (!browser.ie6) {
			// all browsers but ie6
			var windowHeight = getWindowHeight();
			var taskbarHeight = 0;
			var top = '0px';
			var tileMargins = this.t.offsetHeight + this.b.offsetHeight;
			// if taskbar exists, position based on it
			if (taskbar) {
				var taskbarHeight = taskbar.dom.offsetHeight;
				if (taskbar.currentPosition == 'top') {
					// taskbar on top
					var top = taskbar.dom.offsetHeight + 'px';
				}
			}

			setStyle (this.dom, 'top', top);
			setStyle (this.dom, 'height', windowHeight - taskbarHeight + 'px');
			setStyle (this.tile, 'height', windowHeight - taskbarHeight - tileMargins + 'px');
			setStyle (this.r, 'height', windowHeight - taskbarHeight - tileMargins + 'px');
			setStyle (this.l, 'height', windowHeight - taskbarHeight - tileMargins + 'px');

			// cross-browser compatibility
			if (browser.ie) {
				// move contents down
				//this.contents.style.top = '50px';
				//this.contents.style.position = 'relative';
			}
		
		} else {
			// ie6 uses special positioning
			var windowHeight = getWindowHeight();
			var scrollPosition = document.body.scrollTop;
			var taskbarHeight = 0;
			if (taskbar) {
				var taskbarHeight = taskbar.dom.offsetHeight;
				setStyle (this.dom, 'height', getWindowHeight() - taskbarHeight + 'px');
				if (taskbar.currentPosition == 'top') {
					this.dom.style.top = scrollPosition + taskbarHeight + 'px';
				} else {
					this.dom.style.top = scrollPosition + 'px';
				}
			}

			// reposition the insides
			var tileMargins = this.t.offsetHeight + this.b.offsetHeight;
			var windowHeight = getWindowHeight();
			
			setStyle (this.tile, 'height', windowHeight - taskbarHeight - tileMargins + 6 + 'px');
			setStyle (this.r, 'height', windowHeight - taskbarHeight - tileMargins + 6 + 'px');
			setStyle (this.l, 'height', windowHeight - taskbarHeight - tileMargins + 6 + 'px');
			
		}
		
		// update viewport
		this.viewport.dom.style.width = this.tile.offsetWidth - 5 + 'px';
		this.viewport.dom.style.height = this.tile.offsetHeight +'px';
		
		// update viewport
		var overflow = this.contents.offsetHeight - this.viewport.dom.offsetHeight;

		// update viewport vertical scrollbar
		this.viewport.vscrollbar.isStatic = true;
		this.viewport.vscrollbar.areaHeight = this.contents.offsetHeight;
		
		this.viewport.setOverflow(overflow, 0);
		
		// redraw viewport
		this.viewport.refresh();
		//this.viewport.vscrollbar.dom.style.top = '0px';
		
		// if vscrollbar is not visible, make sure that contents snap to the top
		if (this.viewport.vscrollbar.dom.style.display == 'none')
			this.contents.style.top = '0px';
		
		// even more ie6 fixes
		if (browser.ie6) {
			this.viewport.vscrollbar.dom.style.height = this.viewport.dom.offsetHeight; 
		}
		
		// reposition close button
		var closeTop = (this.dom.offsetHeight / 2) - (this.close.offsetHeight / 2);
		this.close.style.top = closeTop + 'px';
	}

	// shows the sidebar
	this.show = function () {
		if ((this.state == 'visible') || (this.animating))
			return;
		this.animating = true;
		this.animate_show();
		this.state = 'visible';
		this.dom.state = this.state;
	}

	this.animate_show = function () {
		var sSidebar = $('dax_sidebar0');
		sSidebar.style.left = sSidebar.offsetLeft + 25 + 'px';
		if ((sSidebar.offsetLeft + 25) < 0) {
			window.setTimeout(sSidebar.animate_show, 50);
		} else {
			sSidebar.parent.animating = false;
			sSidebar.style.left = '0px';
		}
	}
	
	// hides the sidebar
	this.hide = function () {
		if ((this.state == 'hidden') || (this.animating))
			return;
		this.animating = true;
		this.animate_hide();
		this.state = 'hidden';
		this.dom.state = this.state;
	}

	this.animate_hide = function () {
		var sSidebar = $('dax_sidebar0');
		sSidebar.style.left = sSidebar.offsetLeft - 25 + 'px';
		if (sSidebar.offsetWidth > (0 - sSidebar.offsetLeft)) {
			window.setTimeout(sSidebar.animate_hide, 50);
		} else {
			sSidebar.parent.animating = false;
			sSidebar.style.left = '-' + sSidebar.offsetWidth + 'px';
		}
	}

	this.addButton = function (aTheme) {
	
		var aButton = getTemplate('dax_sidebar_button');
		
		if (!aButton)
			return null;
			
		bButton = aButton.cloneNode(true);

		this.b.appendChild (bButton);
		
		f = 1;
		while ($('dax_sidebar_button' + f) != null) {
			f++;
		}
		
		bButton.id = 'dax_sidebar_button' + f;
		bButton.className += ' ' + aTheme;
		bButton.defaultClass = aTheme;
		bButton.style.display = 'block';
		
		bButton.tooltip = null;
		
		// on mouse over and out events
		bButton.onmouseover = function (aEvent) {
			this.className = 'dax_sidebar_button ' + this.defaultClass + '_o';
			if (false) {
				aEvent = xpEvent (aEvent);
				mousePos = getMousePosition (aEvent);
				showTooltip(this.tooltip, mousePos.x, mousePos.y);
			}
		}

		bButton.onmousemove = function (aEvent) {
			//this.className = 'dax_sidebar_button ' + this.defaultClass + '_o';
			if (this.tooltip) {
				aEvent = xpEvent (aEvent);
				mousePos = getMousePosition (aEvent);
				showTooltip(this.tooltip, mousePos.x, mousePos.y);
			}
		}
		
		
		bButton.onmouseout = function () {
			this.className = 'dax_sidebar_button ' + this.defaultClass;
			if (this.tooltip)
				hideTooltip();
		}

		bButton.onmousedown = function () {
			this.className = 'dax_sidebar_button ' + this.defaultClass + '_d';
		}

		bButton.onmouseup = function () {
			this.className = 'dax_sidebar_button ' + this.defaultClass + '_o';
			if (this.tooltip)
				hideTooltip();			
		}
		
		return bButton;
		
	}
	
	// pass events to the sidebar dom
	this.dom.animate_hide = this.animate_hide;
	this.dom.animate_show = this.animate_show;
	
	// set hide button
	this.close.parent = this;
	this.close.onclick = function () {
		this.parent.hide();
	}
	
	this.redraw();
	
}