/**
 * Tabs
 *
 * @author Jonathan Le
 *
 */
function Tabs_destroy() {
	purgeparents(this);
	this.current = null;
	this.first = null;
	this.last = null;
	
	for (var i in this.tabs)
		this.tabs[i].destroy();
	
	delete this.tabs

}

function Tabs()
{
	var node = getTemplate("tabs");
	if (!node)
		return null;
		
	this.dom = node.cloneNode(true);
	//this.dom.id = "tabs" + getElementsByClassName("tabs").length;
	generateUniqueId(this.dom);	
	this.tabarea = getElementByClassName(this.dom, "tab_area");
	this.contents = getElementByClassName(this.dom, "contents");
	this.pagearea = getElementByClassName(this.contents, "tab_page_area");
	this.tabs = new Array();
	this.pages = new Array();
	this.closeButton = getElementByClassName (this.dom, "tab_close", true);
	this.firstDisplayTabIndex = 0;
	this.newDisplayTabIndex = 0;
	this.lastDisplayTabIndex = 0;
	this.isRefreshing = false;
	
	this.dom.parent = this;
	this.dom.style.display = "block";
	this.onswitch = function(index) { }; 
	this.destroy = Tabs_destroy;
	this.tab =
		function(index)
		{
			if (index >= 0 && index < this.tabs.length)
				return this.tabs[index];
			else
				return null;
		};
	
	this.insert =
		function(label, index, page, special)
		{
			var tab = new Tab(label, index);
			
			if (!special) {
				// insert the tab into the list
				if (index == null || index == undefined) {
					index = this.tabs.push(tab) - 1;
					this.last = this.tabs[index];
					this.pages.push(null);
				} else {
					this.tabs.splice(index, 0, tab);
					this.pages.splice(index, 0, null);
				}

				// set the current tab
				this.current = this.tabs[index];
				this.current.parent = this;
				this.tabs[index].index = index;

				// insert the HTML
				this.tabarea.insertBefore(this.current.dom,
					(this.tabs[index+1] ? this.tabs[index+1].dom : null));
				
			} else {
				// insert the HTML
				this.tabarea.insertBefore(tab.dom,
					(this.tabs[this.tabs.length] ? this.tabs[this.tabs.length].dom : null));			
			
			}
			

			// we have a page to link to the tab
			if (page) {
				this.current.page = page;
				this.pagearea.insertBefore(page,
					(this.pages[index+1] ? this.pages[index+1] : null));
				
				// as long as there is a page, make it visible
				this.contents.style.display = "block";
			}

			if (!special) {
				this.activate(this.current);
				return this.current;
			} else {
				return tab;
			}
		};
	
	this.remove =
		function(index, tab)
		{
			if (tab) {
				index = tab.index;
				this.tabarea.removeChild(tab.dom);
				delete tab;
				this.tabs.splice(index, 1);
			} else {
				if (index == null || index == undefined) {
					this.tabarea.removeChild(this.last.dom);
					
					// remove the last one
					var tab = this.tabs.pop();
					delete tab;
				} else {
					var tab = this.tabs[index];
					this.tabarea.removeChild(tab.dom);
					
					delete tab;
					this.tabs.splice(index, 1);
				}
			}
			
			this.updateIndex();
			
		};
	
	this.activateById =
		function(position)
		{
			if (position == undefined || position < 0 || position >= this.tabs.length)
				return;
			
			for (var i in this.tabs) {
				if (i == position) {
					
					// focus new tab
					this.current = this.tabs[i];
					this.current.focus();
				} else {
					this.tabs[i].blur();
				}
			}			
			
		};
	
	this.activate =
		function(tab)
		{
			if (!tab || tab == undefined)
				return;
				
			// go through the list of tabs
			for (var i in this.tabs) {
				if (this.tabs[i] == tab) {
					
					// focus new tab
					this.current = this.tabs[i];
					this.current.focus();
				} else {
					this.tabs[i].blur();
				}
			}
		};
	
	this.closeButton.parent = this;
	
	this.closeButton.onmouseout = function () {
		this.className = 'tab_close tab_close_off';
	}

	this.closeButton.onmouseover = function () {
		this.className = 'tab_close tab_close_on';
	}

	this.showCloseButton = function (showClose) {
		if (showClose)
			getElementByClassName (this.dom, 'tab_close', true).style.display = 'block';
		else
			getElementByClassName (this.dom, 'tab_close', true).style.display = 'none';
	}
	
	this.closeButton.onclick = function () {
		id = this.parent.current.dom.id;
		for (var i in this.parent.tabs) {
			if (this.parent.tabs[i].dom.id == id) {
				index = i;
			}
		}
		
		// make sure the tab that displays all records can not be closed
		if( index != 0 ){
			this.parent.remove(index);
			this.parent.activate(this.parent.tab(index - 1));
			this.parent.tab(index - 1).dom.onclick();
		}
	}

	// add scroll buttons
	this.scrollRightTab = null;
	
	this.scrollLeftTab = this.insert('', null, null, true);
	this.scrollLeftTab.dom.style.display = 'none';
	this.scrollLeftTab.dom.owner = this;
	this.scrollLeftTab.dom.className = 'tab tab_scroll_left';
	// take it out of tabs array
	// this.tabs.splice(0, 1);
	// update mouse events
	// this.scrollLeftTab.blur();
	this.scrollLeftTab.dom.onclick = function() { 
	
		if (this.owner.firstDisplayTabIndex > 0) {
			this.owner.newDisplayTabIndex--;
			this.owner.refresh();
		}
	
	};

	this.updateIndex = function () {
		for (var tabCount = 0; tabCount < this.tabs.length; tabCount++) {
			this.tabs[tabCount].index = tabCount;
		}
	}	
	
	// updates scroll left/right arrows if needed
	// Rudolf Psenicnik
	this.refresh = function() {
	
		if (this.firstDisplayTabIndex < 0)
			this.firstDisplayTabIndex = 0;

		if (this.newDisplayTabIndex < 0)
			this.newDisplayTabIndex = 0;
			
		// check if first display tab is in valid range, if not set it to one
		if (this.firstDisplayTabIndex > (this.tabs.length - 1)) {
			this.firstDisplayTabIndex = (this.tabs.length - 1);
			this.newDisplayTabIndex = (this.tabs.length - 1);
		}
	
		// remove right scroll button
		if (this.scrollRightTab) {
			this.tabarea.removeChild(this.scrollRightTab.dom)
			delete this.scrollRightTab;
		}

		// get offsettop index from very first tab
		this.scrollLeftTab.dom.style.display = 'block';
		var firstTabOffset = this.scrollLeftTab.dom.offsetTop;
			
		// check if we need to display left scroll button
		if (this.newDisplayTabIndex == 0)
			this.scrollLeftTab.dom.style.display = 'none';

		if (this.tabs.length == 0)
			return false;
			
		// flag
		var needScrollRightTab = false;
		var activeTabIndex = this;
		this.lastDisplayTabIndex = 0;
		
		this.firstDisplayTabIndex = this.newDisplayTabIndex;
	
		// hide or show tabs
		for (var tabCount = 0; tabCount < this.tabs.length; tabCount++) {
			if ((tabCount < this.firstDisplayTabIndex) || (needScrollRightTab)) {
				this.tabs[tabCount].dom.style.display = 'none';
			} else {
				this.tabs[tabCount].dom.style.display = 'block';

				// is this tab where it shouldn't be? make it invisible and add a flag for right arrow
				if ((this.tabs[tabCount].dom.offsetTop != firstTabOffset) && (this.tabs.length > 1)){
					needScrollRightTab = true;
					this.tabs[tabCount].dom.style.display = 'none';
					this.lastDisplayTabIndex = tabCount - 1;
				}
			}
		}
		
		// last tab index wasn't updated, happens if right scroll is not used, pick last one
		if (this.lastDisplayTabIndex == 0)
			this.lastDisplayTabIndex = this.tabs.length - 1;
		
		// add right arrow if needed
		if (needScrollRightTab) {
			this.scrollRightTab = this.insert('', null, null, true);
			this.scrollRightTab.dom.owner = this;
			this.scrollRightTab.dom.className = 'tab tab_scroll_right';
			setStyle (this.scrollRightTab.dom, 'float', 'right');
			
			// take it out of tabs array
			// this.tabs.splice(this.scrollRightTab.index, 1);
			// update mouse events
			// this.scrollRightTab.blur();
			this.scrollRightTab.dom.onclick = function() {
				this.owner.newDisplayTabIndex++;
				this.owner.refresh();
			};		
			
			// if right arrow is too low, take out tabs until it's properly visible
			while (this.scrollRightTab.dom.offsetTop != firstTabOffset) {
				this.tabs[this.lastDisplayTabIndex].dom.style.display = 'none';
				this.lastDisplayTabIndex--;
			}

		}
			
		// reselect tab if it is outside the range
		if (this.current.index > this.lastDisplayTabIndex)
			this.tabs[this.lastDisplayTabIndex].dom.onclick();
		else if (this.current.index < this.firstDisplayTabIndex)
			this.tabs[this.firstDisplayTabIndex].dom.onclick();
		
	}
	
	// initializer
	{
		this.first = null;
		this.last = null;
		this.current = null;
		
		setStyle(this.dom, "display", "block");
	}
	
	return this;
}

/**
 * Tab
 *
 * @author Jonathan Le
 *
 */
 
function Tab_destroy() {
	purgeparents(this);

}

function Tab(label, index)
{
	var node = getTemplate("tab");
	if (!node)
		return null;
	
	this.dom = node.cloneNode(true);
	//this.dom.id = "tab" + getElementsByClassName(document.body, "tab").length;
	generateUniqueId(this.dom);
	this.dom.parent = this;
	this.parent = null;
	this.label = "";
	this.focused = false;
	this.onclick = function() { };
	this.page = null;
	this.index = index;
	
	this.l = getElementByClassName(this.dom, "tab_l");
	this.m = getElementByClassName(this.dom, "tab_m");
	this.r = getElementByClassName(this.dom, "tab_r");
	
	this.destroy = Tab_destroy;
	
	this.focus =
		function()
		{
			this.l.className = "tab_l_a";
			this.m.className = "tab_m_a";
			this.r.className = "tab_r_a";
			this.focused = true;
			
			if (this.page)
				this.page.style.display = "block";
		};
		
	this.blur =
		function()
		{
			this.l.className = "tab_l";
			this.m.className = "tab_m";
			this.r.className = "tab_r";
			this.focused = false;
			
			if (this.page)
				this.page.style.display = "none";
				
		};
	
	this.hover =
		function()
		{
			if (!this.focused) {
				this.l.className = "tab_l_h";
				this.m.className = "tab_m_h";
				this.r.className = "tab_r_h";
			}
		};
	
	this.setLabel =
		function(label)
		{
			if (label != null && label != undefined) {
				this.m.innerHTML = label;
				this.label = label;
			}
		};
	
	this.getLabel =
		function(label)
		{
			return this.label;
		};
	
	this.setLabel(label);
	
	this.dom.onmouseover = 
		function()
		{
			this.parent.hover();
			this.parent.userMouseOver();
		};
	
	this.userMouseOver = function() { };
	this.userMouseOut = function() { };
	
	this.dom.onmouseout =
		function()
		{
			if (this.parent.focused) {
				this.parent.focus();
			} else {
				this.parent.blur();
			}
			
			this.parent.userMouseOut();
			
		};
		
	this.dom.onclick =
		function()
		{
			var tab = this.parent;
			tab.onclick();
			tab.parent.activate(tab);
			
			tab.onswitch();
			tab.parent.onswitch(tab.index);
		};
	
	// user-defined event when a tab is switched
	this.onswitch = function()
	{
//		on_tab_switched(this);
	};
	
	// initializer
	{
		setStyle(this.dom, "display", "block");
	}
	return this;
}