// test by Jonathan Le

function WindowManager_createNewWindow()
{
	var newWindow = null;
	try {
		newWindow = new Window();
   	} catch(e) {alert(e.description + e.message + e.number + e.name);}
   	return newWindow;
};

function WindowManager_insertWindow(aWindow)
{
	// check to see that the window isn't already added
	for (var i = 0; i < this.windows.length; i++) {
		if (this.windows[i] == aWindow) {
			return this.windows[i];
		}
	}
	
	if (!this.windowTarget)
		this.windowTarget = new WindowTarget();
		
	var target = this.windowTarget.area;
	target.insertBefore(aWindow.dom, null);
	
	return this.windows.push(aWindow);
};

function WindowManager_destroyWindow(aWindow)
{
	if (this.active == aWindow) {
		WindowManager.active = null;
	}
	
	var windowindex = -1;
	for (var i = 0; i < WindowManager.windows.length; i++)
		if (WindowManager.windows[i] == aWindow)
			windowindex = i;

	// close the window
	aWindow.close();
			
	// purge (aWindow.dom); 
	WindowManager.windowTarget.area.removeChild(aWindow.dom);
			
	if (windowindex > -1) {
		WindowManager.windows[windowindex] = null;
		WindowManager.windows.splice(windowindex, 1);			
	}
	//		delete aWindow;
};

function WindowManager_hideWindow(aWindow)
{
	if (this.active == aWindow) {
		WindowManager.active = null;
	}
	do_accordion(aWindow.dom, 'top')
	setStyle(aWindow.dom,'display','none');
};

function WindowManager_redraw()
{
	// redraw all the windows
	var winObj;
	for (var i = 0; i < this.windows.length; i++) {
		winObj = this.windows[i];
		winObj.size(winObj.dom.offsetWidth, winObj.dom.offsetHeight);
	}
};

// function WindowManager_

function WindowManager()
{
	this.active = null;
	this.maxZIndex = 0;
	this.maxZDashIndex = 9110;
//	this.windowTarget = {
//		area : document.body
//	};
	this.windowTarget = null;
	this.windows = new Array();
	this.scroll = null;
	
	// prototypes
	this.createNewWindow = WindowManager_createNewWindow;
	
	this.insertWindow = WindowManager_insertWindow;
	
	this.destroyWindow = WindowManager_destroyWindow;

	this.hideWindow = WindowManager_hideWindow;
	
/*
	
	this.fadeWindow = function(aWindow, fadeticker)
	{
		assert('testA',aWindow + ":" + fadeticker);
		if (this.active == aWindow) {
			WindowManager.active = null;
		}
		if (fadeticker > 0) {
			setStyle(aWindow.dom, 'opacity', '0.' + this.fadeticker);
			setStyle(aWindow.dom, '-moz-opacity', '0.' + this.fadeticker);
			setStyle(aWindow.dom, 'filter', 'alpha(opacity = ' + this.fadeticker + ')');
			fadeticker = fadeticker - 10;
			setTimeout (this.fadeWindow(aWindow,fadeticker), 1000); 
		} else {
			setStyle(aWindow.dom,'display','none');
		}
	};

*/
	
	this.redraw = WindowManager_redraw;
	
	return this;
}

var WindowManager = new WindowManager(); // singleton

// finds window by table id assigned to id
function findWindowByTableId(tableId) {

	// loop though window to find it
	for (ta = 0; ta < WindowManager.windows.length; ta++) {
		if (WindowManager.windows[ta].tableId == tableId)
			return WindowManager.windows[ta];
	}
	
	// not found
	return null;
}

function WindowTarget()
{
	this.area = document.body;
	return this;
}

function windowDragStart(event)
{	
	// activate the window
	this.parent.activate();
	this.parent.ondragstart();
	
	var x = 0;
	var y = 0;
	
	if (browser.ie) {
	    x = window.event.screenX;
	    y = window.event.screenY;
 
 	   document.onmousemove = windowDragging;
 	   document.onmouseup   = windowDragStop;
	} else {
		x = event.pageX;
		y = event.pageY;
 
		document.addEventListener("mousemove", windowDragging, true);
    	document.addEventListener("mouseup",   windowDragStop, true);
    	event.preventDefault();
	}
     
    WindowManager.active.offsetX = WindowManager.active.dom.offsetLeft - x;
    WindowManager.active.offsetY = WindowManager.active.dom.offsetTop - y;
    
    WindowManager.active.dragging = true;
}

function windowDragging(event)
{
	var x = 0;
	var y = 0;
	
	if (!WindowManager.active.dragging)
		return;
		
	if (browser.ie) {
		x = window.event.screenX;
		y = window.event.screenY;
		window.event.cancelBubble = true;
	    window.event.returnValue = false;
	} else {
		x = event.pageX;
		y = event.pageY;
		event.preventDefault();
	}
	
	x = (x+WindowManager.active.offsetX);
	y = (y+WindowManager.active.offsetY);
	
	if (x >= 0)
		WindowManager.active.dom.style.left = x +"px";
		
	if (y >= 0)
		WindowManager.active.dom.style.top = y +"px";
}

function windowDragStop(event)
{
	WindowManager.active.dragging = false;
	
	if (browser.ie) {
 		document.onmousemove = null;
 		document.onmouseup   = null;	
	} else {
		document.removeEventListener("mousemove", windowDragging, true);
	    document.removeEventListener("mouseup",   windowDragStop, true);
    }
	
	WindowManager.active.ondragstop();	
}

function Window_destroy() {
	purgeparents(this);
	this.contentArea.grid = null;

	if (this.currentTab) {	
		this.currentTab.owner = null;
		this.currentTab = null;
		delete this.currentTab;
	}
	
	this.grid.owner = null;
	this.grid.destroy();
	delete this.grid;
	
	this.search.owner = null;

	this.tabsbar.destroy();
	delete this.tabsbar;

	for (var i in this.mainToolbar.prv.mContent)
		this.mainToolbar.prv.mContent[i].parent = null;
	
	this.mainToolbar.prv.mDOM.parent = null;
	this.mainToolbar.prv.mDOM.contents = null;
	
	for (var i in this.toolbars)
		this.toolbars[i].parent = null;
		
	this.table = null;
	this.winObj = null;
};

function Window_toString()
{
	return "[object Window]";
};		

function Window_activate()
{
	// window object
	var win = this;
	
	// current active window was clicked again
	if (win == WindowManager.active)
		return;

	// unfocus the last window
	if (WindowManager.active) {
		WindowManager.active.deactivate();
	}
	
	// change the style
	with (this) {
		dom.className = "window";
		dom.t.className = "window_t_a";
		dom.tl.className = "window_tl_a";
		dom.tr.className = "window_tr_a";
		dom.l.className = "window_l_a";
		dom.r.className = "window_r_a";
		dom.b.className = "window_b_a";
		dom.bl.className = "window_bl_a";
		dom.br.className = "window_br_a";
		closeBox.className = "window_closebox";
		minimizeBox.className = "window_minimizebox";
		refreshBox.className = "window_refreshbox";
		savePrefsBox.className = "window_saveprefsbox";
		
		win.dom.style.zIndex = ++WindowManager.maxZIndex;
		//alert('Window Zindex1 = ' + win.dom.style.zIndex);
	}
	
	// set the current focus to this window
	WindowManager.active = win;
	
	setStyle(this.dom, 'display','block');
	
	// check if window has corresponding tab, if so select it
	if (this.tbitem) {
		this.tbitem.setSelected(true,true);
	}
};

function Window_deactivate()
{
	this.dom.className = "window window_inactive";
	this.dom.t.className = "window_t";
	this.dom.tl.className = "window_tl";
	this.dom.tr.className = "window_tr";
	this.dom.l.className = "window_l";
	this.dom.r.className = "window_r";
	this.dom.b.className = "window_b";
	this.dom.bl.className = "window_bl";
	this.dom.br.className = "window_br";
	this.closeBox.className = "window_closebox_o";
	this.minimizeBox.className = "window_minimizebox_o";
	this.refreshBox.className = "window_refreshbox_o";
	this.savePrefsBox.className = "window_saveprefsbox_o";
	
	WindowManager.active = null;
};

function Window_size(width, height, overrideMinimumSize)
{
	var win = this;
	var frame = win.dom;
	
	var closebox = win.closeBox;
	var minimizebox = win.minimizeBox;
	var refreshbox = win.refreshBox;
	var title = win.titleBar;
	var content = win.contentArea;
	var logo = win.logo;
	var savePrefsBox = win.savePrefsBox;
	
	// resize by width
	if (isDefined(width) && width > 0) { 

		// set minimum size if needed
		if ((!overrideMinimumSize) && (this.minimumWidth) && (width <= this.minimumWidth))
			width = this.minumumWidth;
	
		// resize the top portion width
		var dx = (frame.tr.offsetWidth ? frame.tr.offsetWidth : 0) + (frame.tl.offsetWidth ? frame.tl.offsetWidth : 0);
		
		// ie7 fix
		if (!width)
			return;
		
		frame.t.style.width = width - dx + "px";
		
		if (savePrefsBox.style.display != 'none') {
			title.style.width = width - (dx + savePrefsBox.offsetWidth + refreshbox.offsetWidth + minimizebox.offsetWidth + closebox.offsetWidth +
			logo.offsetWidth) + "px";
		} else { 
			title.style.width = width - (dx + refreshbox.offsetWidth + minimizebox.offsetWidth + closebox.offsetWidth +
				logo.offsetWidth) + "px";
		}
			
		// resize the bottom portion width
		dx = (frame.bl.offsetWidth ? frame.bl.offsetWidth : 0) + (frame.br.offsetWidth ? frame.br.offsetWidth : 0);
		frame.b.style.width = width - dx + "px";

		if (this.sidebar.enabled)
			this.sidebar.dom.style.width = this.sidebar.width + 'px';
		
		// resize the middle portion
		if (this.sidebar.enabled)
			sidebarWidth = this.sidebar.dom.offsetWidth;
		else
			sidebarWidth = 0;
			
		dx = (frame.r.offsetWidth ? frame.r.offsetWidth : 0) + (frame.l.offsetWidth ? frame.l.offsetWidth : 0);
		
		var contentWidth = width - dx - sidebarWidth;
		content.style.width = contentWidth + "px";
		 
		for (var i in this.toolbars) {
			var toolbar = this.toolbars[i];
			if (this.doResizeToolbar)
				toolbar.style.width = contentWidth + sidebarWidth + 'px';
			// resize tabs bar
			if ((toolbar.className == 'tabs_tb') || (toolbar.className == 'tabs'))
				toolbar.style.width = contentWidth + sidebarWidth + 'px';
			}
		
		win.status.style.width = contentWidth + sidebarWidth + 'px';
		frame.m.style.width = contentWidth + sidebarWidth + 'px';
		
		// required to properly resize the window
		frame.style.width = width + "px";
	}
	
	// resize by height
	if (isDefined(height) && height > 0) {

		// set minimum size if needed
		if ((!overrideMinimumSize) && (this.minimumHeight) && (height <= this.minimumHeight))
			height = this.minimumHeight;

		var dy = (frame.b.offsetHeight ? frame.b.offsetHeight : 0) + (frame.t.offsetHeight ? frame.t.offsetHeight : 0);
		var tbDy = 0;
		
		for (var i in this.toolbars) {
			tbDy += (this.toolbars[i].offsetHeight ? this.toolbars[i].offsetHeight : 0);
		}

		// ie7 fix
		if (!height)
			return;
		
		if (height - dy - tbDy - (win.status.offsetHeight ? win.status.offsetHeight : 0) > 0)
			content.style.height = height - dy - tbDy -	(win.status.offsetHeight ? win.status.offsetHeight : 0) + "px";
		frame.l.style.height = height - dy + "px" ;
		frame.r.style.height = height - dy + "px";
		
		frame.style.height = height + "px";
		
		if (this.sidebar.enabled) {
			this.sidebar.dom.style.height = content.style.height;
		
		}
	}
	
	this.onsize();
};

function Window_resize(dx, dy, overrideMinimumSize)
{
	var width = 0;
	var height = 0;
	
	if (dx == undefined || dx == null)
		dx = 0;
	
	if (dy == undefined || dy == null)
		dy = 0;
		
	// increase by dx, or dy
	width = this.dom.offsetWidth + dx;
	height = this.dom.offsetHeight + dy;
	
	this.size(width, height, overrideMinimumSize);
};

function Window_resizeToolbar(bool)
{
	if (bool == false)
		this.doResizeToolbar = false;
	else
		this.doResizeToolbar = true;
};

function Window_setTitle(newTitle)
{
	if (newTitle)
		this.titleBar.innerHTML = newTitle;
};

function Window_addToolbar(toolbar, obj)
{
	if (this.toolbars.length > 0) {
		var tb = this.toolbar.cloneNode(false);
		if (obj)
			this.dom.m.insertBefore(tb, obj);
		else
			this.dom.m.insertBefore(tb, getElementByClassName(this.dom, 'window_sidebarleft'));
	} else {
		var tb = this.toolbar;
	}
	
	tb.appendChild(toolbar);
	this.toolbars.push(toolbar);
};

function Window_removeToolbar(toolbar)
{
	for (var i=0; i < this.toolbars.length; i++) {
		if (toolbar == this.toolbars[i]) {
			this.toolbars.splice(i, 1);
			return this.dom.m.removeChild(toolbar.parentNode);
		}
	}
};

/*
	this.addContent = function(aDOM)
	{
		this.prv.mContent.push(aDOM);
		this.prv.mDOM.contents.insertBefore(aDOM, null);
	};

	this.removeContent = function(aDOM)
	{
		for (var i = 0; i < this.prv.mContent.length; i++) {
			if (aDOM == this.prv.mContent[i]) {
				this.prv.mContent.splice(i, 1);
				return this.prv.mDOM.contents.removeChild(aDOM);
			}
		}
	};
*/

function Window_close()
{
	// custom
	this.onclose();
};

function Window_open()
{
	// have the window manager insert ourselves
	WindowManager.insertWindow(this);
	
	// size and activate the window for the first time
	this.size(this.dom.offsetWidth, this.dom.offsetHeight);
   	this.activate();
	
	// custom
	this.onopen();
};

function Window_empty() { };

// function Window_

function Window_addSidebar(direction) {

	this.sidebar = {
		enabled		: true,
		direction	: (direction == 'left') ? 'left' : 'right',
		width		: 160,
		dom			: (direction == 'left') ? getElementByClassName(this.dom, 'window_sidebarleft') : getElementByClassName(this.dom, 'window_sidebarright')
	}

//	this.sidebar.node.style.width = this.sidebar.width + 'px';
//	this.sidebar.node.style.height = this.contentArea.clientHeight + 'px';
	this.sidebar.dom.style.display = 'block';
//	this.contentArea.style.width = this.dom.m.clientWidth - this.sidebar.width + 'px';
	
	
}

function Window_hideSidebar() {

	this.sidebar.enabled = false;
	this.sidebar.width = 0;
	
	this.sidebar.dom.style.display = 'none';

}

function Window()
{

	// custom events
	this.onclose = function () { };

	node = getTemplate("window");

	if (!node)
		return null;

	// override toString
	this.toString = Window_toString;		
		
	// prototypes
	this.destroy = Window_destroy;
	
	this.activate = Window_activate;
	
	this.deactivate = Window_deactivate;
	
	this.size = Window_size;
	
	this.resize = Window_resize;

	this.addSidebar = Window_addSidebar;
	this.hideSidebar = Window_hideSidebar;
	
	// added by Rudolf Psenicnik
	// disable toolbar resizing, useful when using overflow with toolbar
	this.resizeToolbar = Window_resizeToolbar;
	
	// set window title bar
	this.setTitle = Window_setTitle;
	
	// count
	var windowCount = WindowManager.windows.length;
	
	this.sidebar = {
		enabled		: false,
		direction	: null	
	}
	
	// create the nodes
	this.dom 	= node.cloneNode(true);
	this.dom.tl	= getElementByClassName(this.dom, "window_tl");
	this.dom.l	= getElementByClassName(this.dom, "window_l");
	this.dom.bl	= getElementByClassName(this.dom, "window_bl");
	this.dom.t	= getElementByClassName(this.dom, "window_t");
	this.dom.m	= getElementByClassName(this.dom, "window_m");
	this.dom.b	= getElementByClassName(this.dom, "window_b");
	this.dom.tr	= getElementByClassName(this.dom, "window_tr");
	this.dom.r	= getElementByClassName(this.dom, "window_r");
	this.dom.br	= getElementByClassName(this.dom, "window_br");
	this.dom.top	= getElementByClassName(this.dom, "window_top");
	this.dom.btm	= getElementByClassName(this.dom, "window_bottom");
	
	this.logo 			= getElementByClassName(this.dom, "window_logo");
	this.titleBar 		= getElementByClassName(this.dom, "window_title");
	this.toolbar 		= getElementByClassName(this.dom, "window_toolbar");
	this.contentArea 	= getElementByClassName(this.dom, "window_content");
	this.closeBox 		= getElementByClassName(this.dom, "window_closebox");
	this.minimizeBox	= getElementByClassName(this.dom, "window_minimizebox");
	this.refreshBox		= getElementByClassName(this.dom, "window_refreshbox");
	this.savePrefsBox	= getElementByClassName(this.dom, "window_saveprefsbox");
	this.status			= getElementByClassName(this.dom, "window_status");
	this.message		= getElementByClassName(this.dom, "window_status_message");
		
	// an array of toolbars
	this.toolbars = new Array();
	this.doResizeToolbar = true;
	
	// frame
	this.dom.id = this.dom.className+windowCount;
	this.dom.parent = this;
	this.dom.style.display = "block";
	if (WindowManager.active) {
		this.dom.style.top = WindowManager.active.dom.offsetTop + 25 + "px";
		this.dom.style.left = WindowManager.active.dom.offsetLeft + 25 + "px";	
		this.dom.style.width = WindowManager.active.dom.offsetWidth + "px";
		this.dom.style.height = WindowManager.active.dom.offsetHeight + "px";	
	}	
	else {
		this.dom.style.top = "40px";
		this.dom.style.left = "200px";	
	}
	this.dom.style.zIndex = ++WindowManager.maxZIndex;
	//alert('Toolbar zIndex = ' + this.dom.style.zIndex);

	// resizing events
	this.dom.tl.onmousedown = resizeDragStart;
	this.dom.t.onmousedown 	= resizeDragStart;
	this.dom.tr.onmousedown = resizeDragStart;
	this.dom.l.onmousedown 	= resizeDragStart;
	this.dom.r.onmousedown 	= resizeDragStart;
	this.dom.bl.onmousedown	= resizeDragStart;
	this.dom.b.onmousedown 	= resizeDragStart;
	this.dom.br.onmousedown	= resizeDragStart;

	// titleBar
	if (this.titleBar) {
		this.titleBar.id = this.titleBar.className+windowCount;
		this.titleBar.parent = this;

		// event handling
		this.titleBar.onmousedown = windowDragStart;
	}
	
	// content area
	if (this.contentArea) {
		this.contentArea.id = this.contentArea.className+windowCount;
		this.contentArea.parent = this;

		// event handling
		this.contentArea.onmousedown = function() { this.parent.activate(); };
	}
	
	//toolbar
	if (this.toolbar) {
		this.toolbar.parent = this;
		this.toolbar.id = this.toolbar.className+windowCount;
	}
		
	// close box
	if (this.closeBox) {
		this.closeBox.parent = this;
		this.closeBox.id = this.closeBox.className+windowCount;
		
		function Window_closeBox_onmousedown(event)
			{ 
				var event = xpEvent(event, true);
				this.className = "window_closebox_d";
				this.parent.activate();
			};
		
		function Window_closeBox_onmouseup(event)
		{
			if (event)
				var event = xpEvent(event, true);
			var aWindow = this.parent;
			aWindow.onclose();
			WindowManager.destroyWindow(this.parent);
			setTimeout(function() {aWindow = null;}, 1);
			return false;
		};
		
		function Window_closeBox_onmouseover() { this.className = "window_closebox_a"; };
		
		function Window_closeBox_onmouseout() 
			{
				if (WindowManager.active == this.parent)
					this.className = "window_closebox";
				else
					this.className = "window_closebox_o";
			};
			
		// function Window_closeBox_
		
		// Events
		this.closeBox.onmousedown = Window_closeBox_onmousedown;
		
		this.closeBox.onmouseup = Window_closeBox_onmouseup;
		
		this.closeBox.onmouseover = Window_closeBox_onmouseover;
		
		this.closeBox.onmouseout = Window_closeBox_onmouseout;
	}

	// minimize box
	if (this.minimizeBox) {
		this.minimizeBox.parent = this;
		this.minimizeBox.id = this.minimizeBox.className+windowCount;
		
		function Window_minimizeBox_onmouseover() { this.className = "window_minimizebox_a"; };

		function Window_minimizeBox_onmouseout() 
			{
				if (WindowManager.active == this.parent)
					this.className = "window_minimizebox";
				else
					this.className = "window_minimizebox_o";
			};		
		
		function Window_minimizeBox_onmousedown(event)
			{ 
				var event = xpEvent(event, true);
				this.className = "window_minimizebox_d";
				this.parent.activate();
			};
		
		function Window_minimizeBox_onmouseup(event)
		{
			var event = xpEvent(event, true);
			WindowManager.hideWindow(this.parent);
			return false;
		};
		
		// function Window_minimizeBox_
		
		this.minimizeBox.onmouseover = Window_minimizeBox_onmouseover;
		
		this.minimizeBox.onmouseout = Window_minimizeBox_onmouseout;		

		this.minimizeBox.onmousedown = Window_minimizeBox_onmousedown;
		
		this.minimizeBox.onmouseup = Window_minimizeBox_onmouseup;
			
	}

	// save prefs box
	if (this.savePrefsBox) {
		this.savePrefsBox.parent = this;
		this.savePrefsBox.id = this.savePrefsBox.className+windowCount;
		
		function Window_savePrefsBox_onmouseover(aEvent) 
			{
				this.className = "window_saveprefsbox_a";
				mousePos = getMousePosition (aEvent);
				showTooltip(STR['save_column_width'], mousePos.x, mousePos.y);				
				
			};

		function Window_savePrefsBox_onmouseout() 
			{
				hideTooltip();
				if (WindowManager.active == this.parent)
					this.className = "window_saveprefsbox";
				else
					this.className = "window_saveprefsbox_o";
			};		
		
		function Window_savePrefsBox_onmousedown(event)
			{ 
				var event = xpEvent(event, true);
				this.className = "window_saveprefsbox_d";
				this.parent.activate();
			};
		
		function Window_savePrefsBox_onmouseup(event)
		{
			var event = xpEvent(event, true);
			// WindowManager.hideWindow(this.parent);
			this.className = "window_saveprefsbox";
			var grid = this.parent.contentArea.grid;
			var url = dax_bridge.url + "AdminSetPreferences?sessionId=" + dax_bridge.sessionId + 
				"&dax_colwidth_" + this.parent.tableId + "=";
				
			// add column widths to save prefs
			for (var colCount = 0; colCount < grid.grid.prv.mColumns.length; colCount++) {
				url += grid.grid.prv.mColumns[colCount].fieldId + ':' +
					grid.grid.prv.mColumns[colCount].prv.mWidth + '_';			
			}
			
			makeCall (url, null, null, null);			
			//grid.reload();
			return false;
		};
		
		// function Window_refreshBox_
		
		this.savePrefsBox.onmouseover = Window_savePrefsBox_onmouseover;
		
		this.savePrefsBox.onmouseout = Window_savePrefsBox_onmouseout;		

		this.savePrefsBox.onmousedown = Window_savePrefsBox_onmousedown;
		
		this.savePrefsBox.onmouseup = Window_savePrefsBox_onmouseup;
			
	}	
	
	// refresh box
	if (this.refreshBox) {
		this.refreshBox.parent = this;
		this.refreshBox.id = this.refreshBox.className+windowCount;
		
		function Window_refreshBox_onmouseover() { this.className = "window_refreshbox_a"; };

		function Window_refreshBox_onmouseout() 
			{
				if (WindowManager.active == this.parent)
					this.className = "window_refreshbox";
				else
					this.className = "window_refreshbox_o";
			};		
		
		function Window_refreshBox_onmousedown(event)
			{ 
				var event = xpEvent(event, true);
				this.className = "window_refreshbox_d";
				this.parent.activate();
			};
		
		function Window_refreshBox_onmouseup(event)
		{
			var event = xpEvent(event, true);
			// WindowManager.hideWindow(this.parent);
			this.className = "window_refreshbox";
			var grid = (this.parent.contentArea.grid);
			if (grid)
				grid.reload();
			return false;
		};
		
		// function Window_refreshBox_
		
		this.refreshBox.onmouseover = Window_refreshBox_onmouseover;
		
		this.refreshBox.onmouseout = Window_refreshBox_onmouseout;		

		this.refreshBox.onmousedown = Window_refreshBox_onmousedown;
		
		this.refreshBox.onmouseup = Window_refreshBox_onmouseup;
			
	}

	this.status.parent = this;
	
	this.addToolbar = Window_addToolbar;
	this.removeToolbar = Window_removeToolbar;
	
	// close out the window
	this.close = Window_close;
	
	// open the window
	this.open = Window_open;
	
	// user-definable
	this.onsize = Window_empty;
	this.onsizestart = Window_empty;
	this.onsizestop = Window_empty;
	this.onclose = Window_empty;
	this.onopen = Window_empty;
	
	// added by Rudi
	// user-definable events for dragging
	this.ondragstart = Window_empty;
	this.ondragstop = Window_empty;
	
	// minimum window size
	this.minimumWidth = 150;
	this.minimumHeight = 125;
	
	return this;
}

function resizeDragStart(event)
{
	if (!WindowManager.active || WindowManager.active.dragging)
		return;
	
	WindowManager.active.resizing = true;
	
	var frame = getParentByClassName(this, "window");
	if (frame.parent.activate)
		frame.parent.activate();
	
	var x = 0;
	var y = 0;
		
	if (browser.ie) {
		document.onmousemove = resizeDragging;
		document.onmouseup   = resizeDragStop;
		
		if (document.documentElement && document.documentElement.scrollTop)	{
			addx = document.documentElement.scrollLeft;
			addy = document.documentElement.scrollTop;
		}
		else if (document.body) {
			addx = document.body.scrollLeft;
			addy = document.body.scrollTop;
		}
		
		x = window.event.clientX + addx;
		y = window.event.clientY + addy;
	} else {
		document.addEventListener("mousemove", resizeDragging, true);
    	document.addEventListener("mouseup",   resizeDragStop, true);
    	
 		x = event.pageX;
		y = event.pageY;
 	}

	var cname = this.className;
	var direction = cname.substring(cname.indexOf('_') +1, cname.lastIndexOf('_'));
	
 	// determine drag direction
   	WindowManager.active.north 	= direction.indexOf('t') != -1 ? true : false;
  	WindowManager.active.south 	= direction.indexOf('b') != -1 ? true : false;
 	WindowManager.active.east 	= direction.indexOf('r') != -1 ? true : false;
 	WindowManager.active.west 	= direction.indexOf('l') != -1 ? true : false;
 	
 	WindowManager.active.oldY = y;
 	WindowManager.active.oldCY = WindowManager.active.dom.offsetHeight;
 	WindowManager.active.oldX = x;
 	WindowManager.active.oldCX = WindowManager.active.dom.offsetWidth;
	
	WindowManager.active.onsizestart();
}

function resizeDragging(event)
{
	if (!WindowManager.active || !WindowManager.active.resizing)
		return;
	
	//assert("testA", WindowManager.active.north +","+WindowManager.active.south+","+WindowManager.active.east+","+WindowManager.active.west);
	
	var x = 0;
	var y = 0;
				
	if (browser.ie) {
	
		if (document.documentElement && document.documentElement.scrollTop)	{
			addx = document.documentElement.scrollLeft;
			addy = document.documentElement.scrollTop;
		}
		else if (document.body) {
			addx = document.body.scrollLeft;
			addy = document.body.scrollTop;
		}
		
		x = window.event.clientX + addx;
		y = window.event.clientY + addy;
		
		window.event.cancelBubble = true;
	    window.event.returnValue = false;
		
	} else {
		x = event.pageX;
		y = event.pageY;
		event.preventDefault();
	}
	
	var h = 0;
	var w = 0;
	
	if (WindowManager.active.north) {
		WindowManager.active.dom.style.top = y+"px";
		h = WindowManager.active.oldCY+WindowManager.active.oldY - y;		
	} else if (WindowManager.active.south) {
		h = y-WindowManager.active.dom.offsetTop;
	}
	
	if (WindowManager.active.east) {
		w = x-WindowManager.active.dom.offsetLeft;
	} else if (WindowManager.active.west) {
		WindowManager.active.dom.style.left = x+"px";
		w = WindowManager.active.oldCX+WindowManager.active.oldX - x;
	}
	
	WindowManager.active.size(w, h);
}

function resizeDragStop(event)
{
	if (!WindowManager.active || !WindowManager.active.resizing)
		return;

	WindowManager.active.onsizestop();
	WindowManager.active.resizing = false;

	if (browser.ie) {
		document.onmousemove = null;
 		document.onmouseup   = null;
 	} else {
		document.removeEventListener("mousemove", resizeDragging, true);
	    document.removeEventListener("mouseup",   resizeDragStop, true);
    }
}


/**
 * Shapable		an object that can be made to expand or contract
 *
 * @author	Jonathan Le
 */
function Shapable(aDOM)
{
	this.dom = aDOM;
	this.height = 0;
	this.width = 0;
	
	this.contract = function(aInterval, aDecrement, aMinimum)
	{
		if (browser.ie) {
			setStyle(this.dom, "display", "none");
			return;
		}
		
		if (!this.height) {
			this.height = xpGetSize(this.dom).height;
		}

		// no scrollbars
		setStyle(this.dom, "overflow", "hidden");
	
		var ref = this;
		var aDecrement = Math.min(aDecrement, this.dom.clientHeight);
		var aMinimum = isDefined(aMinimum) ? aMinimum : 0;
	
		if (this.dom.clientHeight > 0) {
			setStyle(this.dom, "height", (this.dom.clientHeight) - aDecrement + "px");
			setTimeout(function() { ref.contract(aInterval, aDecrement); }, aInterval);
		} else {
			setStyle(this.dom, "display", "none");
		}
	};
	
	this.expand = function(aInterval, aIncrement)
	{
		if (!isDefined(this.height))
			return;
		
		// because editor appears at the wrong location
		if (browser.ie) {
			setStyle(this.dom, "display", "block");
			setStyle(this.dom, "top", "29px");
			return;
		}
	
		// return style to block
		setStyle(this.dom, "display", "");
		setStyle(this.dom, "overflow", "hidden");
		
		var aInterval = isDefined(aInterval) ? aInterval : 1;
		var aIncrement = isDefined(aIncrement) ? aIncrement : Math.floor(this.height / 4);
			
		var ref = this;
		var aIncrement = Math.min(aIncrement, this.height - this.dom.clientHeight);
		
		if (this.dom.clientHeight < this.height) {
			setStyle(this.dom, "height", (this.dom.clientHeight) + aIncrement + "px");
			setTimeout(function() { ref.expand(aInterval, aIncrement); }, aInterval);
		} else {
			// must redraw when expansion is over
			this.prv.mDOM.owner.redraw();
			// restore overflow
			setStyle(this.dom, "overflow", "hidden");
		}
	};
};

/**
 * Tear-away window
 *
 * @author	Jonathan Le
 */
function TearAway()
{
	this.prv = {
		mDOM : getTemplate("tearaway").cloneNode(true)
	}; Function.decl.apply(this);
	
	this.prv.mDOM.parent = this;
	probe(this.prv.mDOM);

	Shapable.apply(this, [this.prv.mDOM]);
	this.getDOM().style.zIndex = ++WindowManager.maxZIndex;

	// assign unique id
	i = 1;
	while ($('tearaway' + i) != null) {
		i++;
	}
	
	this.prv.mDOM.id = 'tearaway' + i;	

	
	this.setContent = function(aContent)
	{
		// insert the content
		this.prv.mDOM.middle.m.innerHTML = "";
		this.prv.mDOM.middle.m.insertBefore(aContent, null);
	};
	
	this.size = function(aWidth, aHeight)
	{
		var dom = this.prv.mDOM;
		var style = dom.style;
		
		if (aWidth) {

			style.width = aWidth + "px";
			
			dom.middle.m.style.width = (aWidth - (xpGetSize(dom.middle.ml).width +
				xpGetSize(dom.middle.mr).width)) + "px";

			dom.bottom.b.style.width = dom.middle.m.style.width;
			dom.top.t.style.width = dom.middle.m.style.width;
		}
		
		if (aHeight) {

		style.height = aHeight + "px";
			dom.middle.ml.style.height = (aHeight - (xpGetSize(dom.top.tl).height +
				xpGetSize(dom.bottom.bl).height)) + "px";
			dom.middle.mr.style.height = (aHeight - (xpGetSize(dom.top.tr).height +
				xpGetSize(dom.bottom.br).height)) + "px";
		}
	};
	
	// generic draggable object
	this.prv.mDOM.onmousedown = startDragObject;
	
	this.prv.mDOM.ondragstart = function(aEvent)
	{
	};
			
	this.prv.mDOM.ondragging = function(aEvent)
	{
		this.className = "tearaway_drag";
	};
	
	this.prv.mDOM.ondragstop = function(aEvent)
	{
		this.className = "tearaway_torn";
	};
	
	// construct
	{
		setStyle(this.prv.mDOM, "display", "block");
	}
	
	return this;
}
TearAway.prototype = new Shapable();

// modal window
function modalWindow () {

	// create 'blackout' area to capture out-of-window clicks
	if (!($('modalwindowblackout0'))) {
		this.blackout = getTemplate('modalwindowblackout').cloneNode(true);
		this.blackout.id = 'modalwindowblackout0';
		document.body.appendChild (this.blackout);
	} else {
		// one already exists
		this.blackout = $('modalwindowblackout0');
	}

	// create tearaway window
	this.tearAway = new TearAway();
	// disable dragging
	this.tearAway.prv.mDOM.onmousedown = null;
	// link to tearAway dom
	this.dom = this.tearAway.dom;
	
	// set class name
	this.dom.className = "tearaway_torn";
	
	// get modal window template
	this.contents = getTemplate ('modalwindowcontents').cloneNode(true);
	// assign unique id
	i = 1;
	while ($('modalwindowcontents' + i) != null) {
		i++;
	}
	
	this.contents.id = 'modalwindowcontents' + i;
	this.contents.style.display = 'block';
	
	this.dom.middle.m.appendChild (this.contents);
	
	this.setContents = function (aContents) {
		this.tearAway.size(aContents.offsetWidth, aContents.offsetHeight);	
		this.contents.appendChild(aContents);
	}
	
	this.setSize = function (aWidth, aHeight) {
		// update contents size
		this.contents.style.width = aWidth + 'px';
		this.contents.style.height = aHeight + 'px';
		// update window size
		this.tearAway.size (this.contents.offsetWidth + this.tearAway.dom.middle.ml.offsetWidth + this.tearAway.dom.middle.mr.offsetWidth,
			this.contents.offsetHeight + this.tearAway.dom.top.t.offsetHeight + this.tearAway.dom.bottom.b.offsetHeight);
		
	}
	
	this.position = function (aLeft, aTop) {
		//this.tearAway.dom.style.position = 'absolute';
		this.tearAway.dom.style.left = aLeft + 'px';
		this.tearAway.dom.style.top = aTop + 'px';
	}
	
	this.show = function () {
	
		if (aSidebar)
			aSidebar.dom.style.display = 'none';
		
		if (taskbar)
			taskbar.dom.style.display = 'none';
			
		this.blackout.style.display = 'block';	
		this.dom.style.display = 'block';
		
		// set appropriate zIndex
		this.blackout.style.zIndex = ++WindowManager.maxZIndex + 23;
		this.dom.style.zIndex = ++WindowManager.maxZIndex + 24;
		
	}
	
	this.hide = function () {
	
		if (aSidebar)
			aSidebar.dom.style.display = 'block';
		
		if (taskbar)
			taskbar.dom.style.display = 'block';
			
		this.blackout.style.display = 'none';	
		
		this.dom.style.display = 'none';
	
	}

	
	
	// construct
	{
		
		// add window to the document		
		document.body.appendChild (this.dom);
		this.dom.style.position = 'absolute';
	
	}
	
	return this;
	
}

// toolbar.js by Jonathan Le

/**
 * Toolbar
 *
 * @author Jonathan Le
 */
function Toolbar(aParent)
{
	// declare private members
	this.prv = {
		mContent : new Array(),
		mDOM : getTemplate("toolbar").cloneNode(true)
	}; Function.decl.apply(this);
	
	// check the dom
	if (!this.prv.mDOM) {
		delete this.prv;
		return null;
	}
	
	// set the parent
	this.prv.mDOM.parent = this;
	probe(this.prv.mDOM);
	//this.prv.mDOM.id = "toolbar"+getElementsByClassName(document.body, "toolbar");
	generateUniqueId(this.prv.mDOM);
	
	
	this.addButton = function(aLabel)
	{
		var button = new ToolbarButton(aLabel);
		this.addContent(button.prv.mDOM);
		return button;
	};
	
	this.addContent = function(aDOM)
	{
		this.prv.mContent.push(aDOM);
		this.prv.mDOM.contents.insertBefore(aDOM, null);
	};
	
	this.removeContent = function(aDOM)
	{
		for (var i = 0; i < this.prv.mContent.length; i++) {
			if (aDOM == this.prv.mContent[i]) {
				this.prv.mContent.splice(i, 1);
				return this.prv.mDOM.contents.removeChild(aDOM);
			}
		}
	};
	
	// construct
	{
		this.prv.mDOM.style.display = "block";
		// add toolbar to specified div
		if (aParent)
			aParent.appendChild (this.prv.mDOM);
	}

	return this;
}

/**
 * ToolbarButton
 *
 * @author Jonathan Le
 */
function ToolbarButton(aLabel)
{
	this.prv = {
		mLabel : null,
		mDOM : getTemplate("toolbarbutton").cloneNode(true),
		mDisabled : false
	}; Function.decl.apply(this);
	
	// check the dom
	if (!this.prv.mDOM) {
		delete this.prv;
		return null;
	}

	// set the parent, probe
	this.prv.mDOM.parent = this;
	probe(this.prv.mDOM);
	//this.prv.mDOM.id = "toolbarbutton"+getElementsByClassName(document.body, "toolbarbutton");
	generateUniqueId(this.prv.mDOM);	
	
	this.enable = function()
	{
		this.prv.mDisabled = false;
		this.prv.mDOM.className = 'toolbarbutton';
	}

	this.disable = function()
	{
		this.prv.mDisabled = true;
		this.prv.mDOM.className = 'toolbarbutton toolbarbuttondisabled';
		
		this.prv.mDOM.l.className = "l";
		this.prv.mDOM.m.className = "m";
		this.prv.mDOM.r.className = "r";		
	}
	
	this.setLabel = function(aStr)
	{
		// set the internal label
		this.prv.mLabel = aStr;
		this.prv.mDOM.m.label.innerHTML = aStr;
	}
	
	this.prv.mDOM.onmouseover = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;
		
		var tbtn = this.parent;

		this.l.className = "l_over";
		this.m.className = "m_over";
		this.r.className = "r_over";
	};
	
	this.prv.mDOM.onmouseout = function(aEvent)
	{
		this.l.className = "l";
		this.m.className = "m";
		this.r.className = "r";
	};
	
	this.prv.mDOM.onmousedown = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;

		this.l.className = "l_down";
		this.m.className = "m_down";
		this.r.className = "r_down";
		
		return false;
	};
	
	this.prv.mDOM.onmouseup = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;	

		this.l.className = "l_over";
		this.m.className = "m_over";
		this.r.className = "r_over";

	};
	
	this.prv.mDOM.onclick = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;
	
		var tbtn = this.parent;
		tbtn.onclick(aEvent);
	};
	
	// user definable
	this.onclick = function() {};
	
	// construct here
	{
		this.setLabel(aLabel);
		this.prv.mDOM.style.display = "block";
	}
	
	return this;
}

function Viewport_destroy() {
	this.parent = null;
	this.owner = null;
	
	if (this.content)
		this.content.parent = null;
	
	this.content = null;
	
	this.hscrollbar.destroy();
	this.vscrollbar.destroy(); 
};

/**
 * Viewport
 *
 * @author Jonathan Le
 *
 */
 function ViewPort()
{
	var node = getTemplate("viewport");
	if (!node)
		return null;

	this.dom = node.cloneNode(true);
//	this.dom.id = "viewport" + getElementsByClassName(document.body, "viewport").length;
	generateUniqueId(this.dom);	

	// V-scrollbar
	this.vscrollbar = new VerticalScrollBar(this.dom);
	this.dom.insertBefore(this.vscrollbar.dom, null);
	this.vscrollbar.viewport = this;
	
	// H-scrollbar
	this.hscrollbar = new HorizontalScrollBar(this.dom);
	this.dom.insertBefore(this.hscrollbar.dom, null);
	this.hscrollbar.viewport = this;
	
	// bottom right corner
	var cornerNode = getTemplate('viewport_bottomrightcorner');
	this.bottomrightcorner = cornerNode.cloneNode(true);
	this.dom.insertBefore(this.bottomrightcorner, null);	
	cornerNode = null;

	i = 1;
	while ($('viewport_bottomrightcorner' + i) != null) {
		i++;
	}
	
	this.bottomrightcorner.id = 'viewport_bottomrightcorner' + i;
	
	this.parent 			= null;
	this.content 			= null;
	this.overflow 			= 0;
	this.hOverflow 			= 0;
	
	// resize viewport when refreshing
	this.resizeOnRefresh	= true;
	
	this.destroy = Viewport_destroy;
	
	this.setContent =
		function(content, overflow)
		{
			this.dom.insertBefore(content, this.hscrollbar.dom);

			this.content = content;
			
			if (isDefined(overflow)) {
				this.setOverflow(overflow);
			} else {
				this.setOverflow(this.content.offsetHeight - this.dom.offsetHeight,
					this.content.offsetWidth - this.dom.offsetWidth);
			}

			setStyle(this.content, "position", "relative");
			setStyle(this.content, "overflow", "hidden");
		};

	this.refresh =
		function()
		{
			// show or hide depending whether or not we have content
			if (!this.content) {
				this.vscrollbar.hide();
				this.hscrollbar.hide();
				// no need to continue because we have no content
				return;
			} else {
				// show scrollbars
				if (this.overflow > 0) {
					this.vscrollbar.show();
				}
				
				if (this.hOverflow > 0) {
					this.hscrollbar.show();
				}
			}
			
			// we've got to make sure the viewport matches the parent
			if ((this.parent) && (this.resizeOnRefresh)) {
				setStyle(this.dom, "width", this.parent.style.width);
				setStyle(this.dom, "height", this.parent.style.height);
			}
	
			// refresh the scrollbar
			this.vscrollbar.refresh();			
			this.hscrollbar.refresh();
		
			if(!browser.safari){
				setStyle(this.bottomrightcorner, "zIndex", ++WindowManager.maxZIndex);
				setStyle(this.vscrollbar.dom, "zIndex", ++WindowManager.maxZIndex);
				setStyle(this.hscrollbar.dom, "zIndex", WindowManager.maxZIndex);
			} else {
				//ACI0049049 fix
				var refreshOnSafari = true;
				if (this.owner) {
					if (this.owner.toString() == '[object GridObject]') {
						if (typeof(this.owner.owner) != 'undefined') {
							if (this.owner.owner.toString() == '[object DataGrid]') {
								if (typeof(this.owner.owner.editor) != 'undefined') {
									if (this.owner.owner.editor.prv.mTearAway.prv.mDOM.style.display != 'none') {
										refreshOnSafari = false;
									}
								}
							}
						}
					}
				}
				
				if (refreshOnSafari) {
					setStyle(this.bottomrightcorner, "zIndex", ++WindowManager.maxZIndex);
					setStyle(this.vscrollbar.dom, "zIndex", ++WindowManager.maxZIndex);
					setStyle(this.hscrollbar.dom, "zIndex", WindowManager.maxZIndex);
				}
			}

			// user defined event
			this.onrefresh();

			// reposition white space in bottom right corner
			if ((this.vscrollbar.dom.style.display != 'none') & (this.hscrollbar.dom.style.display != 'none')) {
				if (browser.ie) {
					var aLeftStyle = this.hscrollbar.dom.offsetWidth + 'px';
				} else {
					var aLeftStyle = this.hscrollbar.dom.offsetWidth + this.bottomrightcorner.offsetWidth - 4 + 'px';
				}
				this.bottomrightcorner.style.left = aLeftStyle;
				this.bottomrightcorner.style.top = parseInt(this.vscrollbar.dom.style.top) + this.vscrollbar.dom.offsetHeight + 'px';
				this.bottomrightcorner.style.display = 'block';
			} else {
				this.bottomrightcorner.style.display = 'none'
			}
		};
	
	this.setOverflow =
		function(overflow, hOverflow)
		{
			// allow the caller to specify an overflow
			// this determines how the scrollbar will be drawn
			this.overflow = overflow;
			if (isDefined(hOverflow)) {
				this.hOverflow = hOverflow;
			}
			
			if (this.overflow <= 0) {
				this.vscrollbar.hide();
			} else {
				this.vscrollbar.show();
			}
			
			if (this.hOverflow <= 0) {
				this.hscrollbar.hide();
			} else {
				this.hscrollbar.show();
			}
		};

	this.onrefresh = function() { };
	
	// user-defined, overrideable
	this.onvscrollstop = function(aTop) { };
	this.onvscrollstart = function(aTop) { };
	this.onvscroll = function(aTop, aIsScrollDown)
	{
		setStyle(this.content, "style", (aTop * (-1) * this.vscrollbar.ratio)+ "px");
	};
	
	this.onhscroll = function(aLeft, aIsScrollRight)
	{
		if (!(isNaN(aLeft * (-1) * this.hscrollbar.ratio))) {
			setStyle(this.content, "left", (aLeft * (-1) * this.hscrollbar.ratio)+ "px");
		}
		
		//RP 12/14/06
		
		//setStyle(this.dom, "width", xpGetSize(this.parent).width + aLeft + "px");
		
		//var newSize = xpGetSize(this.parent).width + aLeft + "px";
		
		//debugAlert ('width:' + newSize);
	};
}

function HorizontalScrollBar_destroy() {
	purgeparents(this);

	this.lArrow 	= null;
	this.rArrow 	= null;
	this.slideArea 	= null;
	this.slider 	= null;

	this.parent = null;
	
	this.viewport = null;
	this.prv.mParent = null;
		
};

/**
 * HorizontalScrollBar
 *
 * probably could've used inheritence...oh well.
 * 
 * @author Jonathan Le
 */
 function HorizontalScrollBar(aParentElement)
{
	var node = getTemplate("hscrollbar");
	if (!node)
		return null;

	// HTML elements
	this.dom = node.cloneNode(true); probe(this.dom);
//	this.dom.id = "hscrollbar" + getElementsByClassName(document.body, "hscrollbar").length;
	generateUniqueId(this.dom);
	
	this.lArrow 	= this.dom.track.arrowLeft;
	this.rArrow 	= this.dom.track.arrowRight;
	this.slideArea 	= this.dom.track.slideArea;
	this.slider 	= this.dom.track.slideArea.slider;
	
	for (var properties in this) {
		this[properties].parent = this;
	}

	// public properties
	this.viewport 	= null;
	this.ratio		= 1;
	this.last		= 0;
	
	this.pageIncrement	= 50;
	this.increment		= 10;
	
	// private properties
	
	this.destroy = HorizontalScrollBar_destroy;
	
	this.prv = {
		mParent : isDefined(aParentElement) ? aParentElement : null,
		mVisibility : true
	}; Function.decl.apply(this); // generate accessors
	
	this.show = function()
	{
		if (this.prv.mVisibility)
			return;
			
		setStyle(this.dom, "display", "block");
		this.prv.mVisibility = true;
	};
	
	this.hide = function()
	{
		if (!this.prv.mVisibility)
			return;
		
		setStyle(this.dom, "display", "none");
		this.prv.mVisibility = false;
	};
	
	// position the scroll bar on any point (x,y)
	this.position = function(aX, aY)
	{
		var v_height = xpGetSize(this.prv.mParent).height;
		var sb_height = xpGetSize(this.dom).height;
		
		if (isDefined(aX)) {
			this.dom.style.left = aX + "px";
		}
		
		if (!isDefined(aY)) {
			var aY = browser.ie ? v_height - sb_height + 1 : v_height - sb_height +
				this.prv.mParent.offsetTop;
		}
		
		// set
		setStyle(this.dom, "top", aY + "px");
	};
	
	// redraw the horizontal scrollbar
	this.refresh = function()
	{
		if (!this.prv.mParent || !this.prv.mVisibility || !this.viewport)
			return;

		this.position();
		
		// calculate the slider height
		var thumbW = 0;
		var parentWidth = xpGetSize(this.prv.mParent).width;
		
		// determine how big the scroll should be
		var scrollWidth = this.viewport.vscrollbar.visible ?
			(parentWidth - xpGetSize(this.viewport.vscrollbar.dom).width) :
			parentWidth;
		
		setStyle(this.dom, "width", scrollWidth + "px");

		var minWidth = xpGetSize(this.slider.sliderLeft).width +
			xpGetSize(this.slider.sliderRight).width;
		var trackSpace = scrollWidth - xpGetSize(this.lArrow).width -
			xpGetSize(this.rArrow).width;
		
		if (trackSpace > this.viewport.hOverflow) {
			// the slider height is the difference
			thumbW = trackSpace - this.viewport.hOverflow;
		} else {
			// the slider is the minimum
			this.ratio = (this.viewport.hOverflow > 0) && (trackSpace > 0) ?
				this.viewport.hOverflow / trackSpace : 1;
		}
		
		// size the slider
		thumbW = thumbW > minWidth ? thumbW : minWidth;

		setStyle(this.slider, "width", thumbW + "px");
		setStyle(this.slider.sliderMid, "width", thumbW -
			(xpGetSize(this.slider.sliderLeft).width +
			xpGetSize(this.slider.sliderRight).width) + "px");
		
		// save the trackSpace
		this.trackSpace = trackSpace;
		
		this.sliderWidth = this.slider.sliderLeft.offsetWidth + this.slider.sliderRight.offsetWidth
			+ this.slider.sliderMid.offsetWidth;
		
		// calculate increments
		this.pageIncrement = this.pageIncrement > this.trackSpace ? this.trackSpace :
			this.pageIncrement;
		this.increment = this.increment > this.trackSpace ? this.trackSpace :
			this.increment;
		
		// sliding area
		var sw = xpGetSize(this.dom.track).width -
			(xpGetSize(this.lArrow).width + xpGetSize(this.rArrow).width);
		setStyle(this.slideArea, "width", sw + "px");
		
		var swRight = this.slider.offsetLeft + thumbW;
		
		if (swRight > this.rArrow.offsetLeft) {
			this.scroll(this.last - (swRight - this.rArrow.offsetLeft));
		} else 
			this.scroll(this.last);
	};
	
	this.scroll = function(aPosition)
	{
		var sh = this.slider.sliderLeft.offsetWidth + this.slider.sliderRight.offsetWidth
			+ this.slider.sliderMid.offsetWidth;
		
		if (aPosition < 0 || (aPosition + sh) > this.slideArea.offsetWidth)
			return;
		
		// set slider position
		setStyle(this.slider, "left", aPosition + "px");
		
		// scrolling down
		var right = (this.last < aPosition);
		this.onscroll(aPosition, right);

		this.last = aPosition;
	};
	
	//
	// Event handlers
	//
	this.slider.onmousedown = function(aEvent)
	{
		var event = xpEvent(aEvent, true, true);
		var curs_pt = getMousePosition(event);
		
		WindowManager.scroll = this.parent;
		WindowManager.scroll.point = getObjectPosition(this.parent.slideArea);

		// calculate the mouse point from the top of the slider
		var point = getObjectPosition(this);
		WindowManager.scroll.offset = curs_pt.x - point.x;
		WindowManager.scroll.last = curs_pt.x - WindowManager.scroll.point.x - WindowManager.scroll.offset;
		
		// Gecko, Safari
		if (document.addEventListener) {
			document.addEventListener("mousemove", horizontalScrollDragging, true);
	   		document.addEventListener("mouseup",   horizontalScrollStop, true);
	    } else { //IE
	      	document.onmousemove = horizontalScrollDragging;
			document.onmouseup   = horizontalScrollStop;
	    }
	    
	    // user def event handler
	    this.parent.onscrollstart(WindowManager.scroll.last);
	    
	    return false;
	};
	
	this.lArrow.onmousedown = function(aEvent)
	{
		this.className = "arrow_left_down";
		var scroll = this.parent;
		var pos = ((scroll.last - scroll.increment) < 0) ? 0 : (scroll.last - scroll.increment);
		scroll.scroll(pos);
	};
	
	this.lArrow.onmouseup = function(aEvent)
	{
		this.className = "arrow_left";
	};
	
	this.rArrow.onmousedown = function(aEvent)
	{
		this.className = "arrow_right_down";
		var scroll = this.parent;
		var sh = scroll.slider.sliderLeft.offsetWidth +scroll.slider.sliderRight.offsetWidth
			+ scroll.slider.sliderMid.offsetWidth;
		var pos = scroll.last + scroll.increment;
		pos = (pos + sh) > scroll.slideArea.offsetWidth ?
			scroll.slideArea.offsetWidth - sh : pos;
		
		scroll.scroll(pos);
	};
	
	this.rArrow.onmouseup = function(aEvent)
	{
		this.className = "arrow_right";
	};

	this.slideArea.onmousedown = function(aEvent)
	{
		var event = xpEvent(aEvent);
		var scrollbar = this.parent;
		var sh = scrollbar.slider.sliderLeft.offsetWidth + scrollbar.slider.sliderRight.offsetWidth
			+ scrollbar.slider.sliderMid.offsetWidth;
		
		var pt = getMousePosition(event);
		
		var p = getObjectPosition(this);
		var x = pt.x - p.x;

		if (x < scrollbar.last) {
			scrollbar.scroll((scrollbar.last - scrollbar.pageIncrement) < 0 ? 0 : scrollbar.last - scrollbar.pageIncrement);
		} else if (x > scrollbar.last + sh) {
			scrollbar.scroll((scrollbar.last + scrollbar.pageIncrement + sh) > scrollbar.slideArea.offsetWidth ? scrollbar.slideArea.offsetWidth - sh : scrollbar.last + scrollbar.pageIncrement);
		}
		
		return false;
	};
	
	//
	// Abstract methods (overrideable)
	//
	this.onscrollstart = function(aLeft)
	{
		if (this.viewport && this.viewport.onhscrollstart)
			this.viewport.onhscrollstart(aLeft);
	};
	
	this.onscroll = function(aLeft, aIsScrollRight)
	{
		if (this.viewport && this.viewport.onhscroll)
			this.viewport.onhscroll(aLeft, aIsScrollRight);
	};
	
	this.onscrollstop = function(aLeft)
	{
		if (this.viewport && this.viewport.onhscrollstop)
			this.viewport.onhscrollstop(aLeft);
	};
	
	// init
	{
		setStyle(this.dom, "display", "block");
		setStyle(this.dom, "position", "absolute");
	}
	
	return this;
}

//
// Jonathan Le
//
// Handles the scroll thumb being dragged
//
function horizontalScrollDragging(aEvent)
{
	var event = xpEvent(aEvent, true);
	var point = getMousePosition(event);
	var scrollbar = WindowManager.scroll;
	
	// calculate the top position of the slider in relation to the cursor
	var left = point.x - scrollbar.point.x - scrollbar.offset;
	scrollbar.scroll(left);
	
	return false;
}

//
// Jonathan Le
//
// Handles the scroll thumb dragged to a stop
//
function horizontalScrollStop(aEvent)
{
	var event = xpEvent(aEvent, true);
	var point = getMousePosition(event);
	var scrollbar = WindowManager.scroll;
	
	// calculate the top position of the slider in relation to the cursor
	var left = point.x - scrollbar.point.x - scrollbar.offset;
	
	var scrollbar = WindowManager.scroll;
 	scrollbar.onscrollstop(left);
	
	WindowManager.scroll = null;

	// Gecko
	if (document.removeEventListener) {
		document.removeEventListener("mousemove", horizontalScrollDragging, true);
		document.removeEventListener("mouseup",   horizontalScrollStop, true);
	} else {
		document.onmousemove = null;
 		document.onmouseup   = null;
	}
	
	return false;
}

function VerticalScrollBar_destroy() {
	purgeparents(this);

	this.arrowTop 	= null;
	this.arrowBottom 	= null;
	this.slideArea 	= null;
	this.slider 	= null;
	
	this.parent = null;
	
	this.viewport = null;
};		

/**
 * VeritcalScrollBar
 *
 * @author Jonathan Le
 */
function VerticalScrollBar(aParentElement)
{
	var node = getTemplate("scrollbar");
	if (!node)
		return null;
	
	// HTML related properties
	this.dom = node.cloneNode(true); probe(this.dom);
	this.arrowTop 		= this.dom.track.arrowTop;
	this.arrowBottom 	= this.dom.track.arrowBottom;
	this.slideArea		= this.dom.track.slideArea;
	this.slider			= this.slideArea.slider;
	this.slider.top		= this.slider.sliderTop;
	this.slider.mid		= this.slider.sliderMid;
	this.slider.bottom	= this.slider.sliderBottom;

//	this.dom.id = "scrollbar" + getElementsByClassName(document.body, "scrollbar").length;
	generateUniqueId(this.dom);
	
	for (var properties in this) {
		this[properties].parent = this;
	}
	
	// properties
	this.parent 	= isDefined(aParentElement) ? aParentElement : null;
	this.visible 	= true;
	this.viewport 	= null;
	this.ratio 		= 1;
	this.last 		= 0;
	
	// arrow advancing
	this.increment 	= 10;
	this.pageIncrement = 50;
	
	// methods
	
	this.destroy = VerticalScrollBar_destroy;
	
	this.position = function(aX, aY)
	{
//		var pt = getObjectPosition(this.parent);
		var v_width = xpGetSize(this.parent).width;
		var sb_width = xpGetSize(this.dom).width;
		
		if (!isDefined(aX)) {
			var aX = (browser.ie) ? v_width - sb_width :
				v_width - sb_width + this.parent.offsetLeft;
		}
		
		if (!isDefined(aY)) {
			var aY = (browser.ie) ? 0 : this.parent.offsetTop;
		}
		
		setStyle(this.dom, "left", aX + "px");
		setStyle(this.dom, "top", aY + "px");
	};
	
	// before refresh, user customizable
	this.beforeRefresh = function () { };
	
	this.refresh = function()
	{
		if (!this.parent || !this.visible || !this.viewport)
			return;
		
		this.beforeRefresh();
		
		// calculate the slider height
		var sh = 0;
		this.position(); // reset position
		
		var parentHeight = (browser.ie) ? xpGetSize(this.parent).height + 3 :
			xpGetSize(this.parent).height;
		
		var minHeight = xpGetSize(this.slider.top).height +
			xpGetSize(this.slider.bottom).height;

		// determine scroll height
		var scrollHeight = this.viewport.hscrollbar.getVisibility() ?
			(parentHeight - xpGetSize(this.viewport.hscrollbar.dom).height) :
			parentHeight;
		
		setStyle(this.dom, "height", parseInt(scrollHeight) + "px");

		var trackSpace = scrollHeight - xpGetSize(this.arrowTop).height -
			xpGetSize(this.arrowBottom).height;
		/*
		if ((this.isStatic == undefined) || (!this.isStatic)) { // if (this.isStatic) {
			if (trackSpace < this.viewport.overflow) {
				// the slider height is the difference
				sh = trackSpace - this.viewport.overflow;
	
			} else {
				// the slider is the minimum
				this.ratio = (this.viewport.overflow > 0) && (trackSpace > 0) ? 
					this.viewport.overflow / trackSpace : 1;
			}
			
			// size the slider
			sh = sh > minHeight ? sh : minHeight;
		
		}
		*/
		if (!this.isStatic) {
			sh = (this.viewport.dom.offsetHeight - 2 * this.viewport.owner.grid.header.offsetHeight) * trackSpace / (this.viewport.dom.offsetHeight + this.viewport.overflow - 2 * this.viewport.owner.grid.header.offsetHeight);
			this.pageIncrement = sh;
			sh = sh > minHeight ? sh : minHeight;

			this.ratio = (this.viewport.overflow > 0) && ((trackSpace-sh) > 0) ? 
					this.viewport.overflow / (trackSpace-sh) : 1;
		}
		else {
			sh = (this.areaHeight - this.viewport.overflow) * trackSpace / this.areaHeight ;
			this.pageIncrement = sh;
			
			// size the slider
			sh = sh > minHeight ? sh : minHeight;
			this.ratio = (this.viewport.overflow > 0) && ((trackSpace-sh) > 0) ? 
					this.viewport.overflow / (trackSpace-sh) : 1;
//			this.ratio = (this.viewport.overflow > 0) && (trackSpace > 0) ?  
//				this.areaHeight / trackSpace : 1;
			
		}
		
		setStyle(this.slider, "height", sh + "px");
		setStyle(this.slider.mid, "height", sh - (xpGetSize(this.slider.top).height +
			xpGetSize(this.slider.bottom).height) + "px");
		//if ((this.slider.offsetHeight + this.slider.offsetTop) > (parentHeight - xpGetSize(this.arrowTop).height - xpGetSize(this.arrowBottom).height))
		//	setStyle(this.slider, "top", parentHeight - xpGetSize(this.arrowTop).height - xpGetSize(this.arrowBottom).height - sh + "px");
		
		// save the trackSpace
		this.trackSpace = trackSpace;

		this.sliderHeight = this.slider.top.offsetHeight + this.slider.bottom.offsetHeight
			+ this.slider.mid.offsetHeight;
		
		// calculate increments
		this.pageIncrement = this.pageIncrement > this.trackSpace ? this.trackSpace :
			this.pageIncrement;
		this.increment = this.increment > this.trackSpace ? this.trackSpace :
			this.increment;
		
		// calc sliding area
		var height = this.dom.track.offsetHeight -
			(xpGetSize(this.arrowTop).height + xpGetSize(this.arrowBottom).height);
			
		if (height < 0)
			return;
			
		this.slideArea.style.height = parseInt(height) + "px";

		if (height < this.slider.offsetTop) {
			setStyle(this.slider, "top",
				parseInt(height - this.arrowBottom.offsetHeight) + "px");
		}
	};
	
	this.show = function()
	{
		setStyle(this.dom, "display", "block");
		this.visible = true;
	};
	
	this.hide = function()
	{
		setStyle(this.dom, "display", "none");
		this.visible = false;
	};
	
	this.scroll = function(position, skipOnScroll)
	{
		var sh = this.slider.top.offsetHeight + this.slider.bottom.offsetHeight
			+ this.slider.mid.offsetHeight;
		
		if (position < 0 || (position + sh) > this.slideArea.offsetHeight)
			return;
		
		// set slider position
		setStyle(this.slider, "top", position + "px");
		//assert('testA', position);
		// scrolling down
		var down = (this.last < position);
		if (!skipOnScroll)
			this.onscroll(position, down);

		this.last = position;
	};
	
	//
	// Event Handlers
	//
	this.slider.onmousedown = function(aEvent)

	{
		// IE, W3C Compatability
		var event = xpEvent(aEvent, true);
		var pt = getMousePosition(event);
		
		WindowManager.scroll = this.parent;
		WindowManager.scroll.point = getObjectPosition(this.parent.slideArea);

		// calculate the mouse point from the top of the slider
		var point = getObjectPosition(this);
		WindowManager.scroll.offset = pt.y - point.y;
		WindowManager.scroll.last = pt.y - WindowManager.scroll.point.y - WindowManager.scroll.offset;
		
		// Gecko, Safari
		if (document.addEventListener) {
			document.addEventListener("mousemove", verticalScrollDragging, true);
	    	document.addEventListener("mouseup",   verticalScrollStop, true);
	    } else { // IE
	    	document.onmousemove = verticalScrollDragging;
			document.onmouseup   = verticalScrollStop;
	    }

		// user def event handler
	    this.parent.onscrollstart(WindowManager.scroll.last);
	    
	    return false;
	};
		
	this.arrowTop.onmousedown = function(aEvent)
	{
		var scrollbar = this.parent;
		var pos = scrollbar.last - scrollbar.increment < 0 ? 0 :
			scrollbar.last - scrollbar.increment;
		scrollbar.scroll(pos);
		
		this.className = "arrow_top_down";
	};
	
	this.arrowTop.onmouseup = function(aEvent)
	{
		this.className = "arrow_top";
		this.parent.onscrollstop(this.parent.last);
	};

	this.arrowBottom.onmousedown = function(aEvent)
	{
		var scrollbar = this.parent;
		var sh = scrollbar.slider.top.offsetHeight + scrollbar.slider.bottom.offsetHeight
			+ scrollbar.slider.mid.offsetHeight;
		var pos = scrollbar.last + scrollbar.increment;
		pos = (pos + sh) > scrollbar.slideArea.offsetHeight ?
			scrollbar.slideArea.offsetHeight - sh :	pos;
		
		scrollbar.scroll(pos);
		
		this.className = "arrow_bottom_down";
	};
	
	this.arrowBottom.onmouseup = function()
	{
		this.className = "arrow_bottom";
		this.parent.onscrollstop(this.parent.last);
	};
	
	this.slideArea.onmousedown = function(event)
	{
		var event = xpEvent(event);
		var scrollbar = this.parent;
		var sh = scrollbar.slider.top.offsetHeight + scrollbar.slider.bottom.offsetHeight
			+ scrollbar.slider.mid.offsetHeight;
		
		var pt = getMousePosition(event);
		
		var p = getObjectPosition(this);
		var y = pt.y - p.y;
		
		if (y < scrollbar.last) {
			scrollbar.scroll((scrollbar.last - scrollbar.pageIncrement) < 0 ? 0 : scrollbar.last - scrollbar.pageIncrement);
		} else if (y > scrollbar.last + sh) {
			scrollbar.scroll((scrollbar.last + scrollbar.pageIncrement + sh) > scrollbar.slideArea.offsetHeight ? scrollbar.slideArea.offsetHeight - sh : scrollbar.last + scrollbar.pageIncrement);
		}
		
		return false;
	};
	
	this.slideArea.onmouseup = function(aEvent)
	{
		var scrollbar = this.parent;
		scrollbar.arrowTop.onmouseup();
		scrollbar.arrowBottom.onmouseup();
	};
	
	//
	// custom handlers
	//
	this.onscrollstart = function(aTop)
	{	
		this.isScrolling = true;
		if (this.viewport)
			this.viewport.onvscrollstart(aTop);
	};
	
	this.onscroll = function(aTop, aIsScrollDown)
	{
		if (this.viewport)
			this.viewport.onvscroll(aTop, aIsScrollDown);
	};
	
	this.onscrollstop = function(aTop)
	{
		var sh = this.slider.top.offsetHeight + this.slider.bottom.offsetHeight
			+ this.slider.mid.offsetHeight;
			
		if ((aTop + sh) > this.slideArea.offsetHeight)
			var aTop = this.slideArea.offsetHeight - sh;
	
		if (this.viewport)
			this.viewport.onvscrollstop(aTop);

		this.isScrolling = false;
	};
	
	// initializer
	{
		setStyle(this.dom, "display", "block");
		setStyle(this.dom, "position", "absolute");
	}
	
	return this;
}

//
// Jonathan Le
//
// Handles the scroll thumb being dragged
//
function verticalScrollDragging(event)
{
	var event = xpEvent(event, true);
	var point = getMousePosition(event);
	var scrollbar = WindowManager.scroll;
	
	// calculate the top position of the slider in relation to the cursor
	var top = point.y - scrollbar.point.y - scrollbar.offset;
	scrollbar.scroll(top);
	
	return false;
}

//
// Jonathan Le
//
// Handles the scroll thumb dragged to a stop
//
function verticalScrollStop(event)
{
	var event = xpEvent(event, true);
	var point = getMousePosition(event);
	var scrollbar = WindowManager.scroll;
	
	// calculate the top position of the slider in relation to the cursor
	var top = point.y - scrollbar.point.y - scrollbar.offset;
 	scrollbar.onscrollstop(top);
	
	WindowManager.scroll = null;

	// Gecko
	if (document.removeEventListener) {
		document.removeEventListener("mousemove", verticalScrollDragging, true);
		document.removeEventListener("mouseup",   verticalScrollStop, true);
	} else {
		document.onmousemove = null;
 		document.onmouseup   = null;
	}
}

// controller for viewport, currently used for mouse scrolling
daxViewportController = {
	activeViewport : null
}

function dax_mouseWheel(event){
	var delta = 0;
	if (!event) event = window.event;
	if (event.wheelDelta) {
		delta = event.wheelDelta/120; 
		if (window.opera) delta = -delta;
	} else if (event.detail) {
		delta = -event.detail/3;
	}
	
	var useDaxScrollHandler = false;
	
	if (daxViewportController)
		if (daxViewportController.activeViewport)
			if (daxViewportController.activeViewport.boundTo.mouseWheelHandler)
				useDaxScrollHandler = true;
	
	if ((delta) && (useDaxScrollHandler)) {
		daxViewportController.activeViewport.boundTo.mouseWheelHandler(delta);
		if (event.preventDefault)
			event.preventDefault();
	    event.returnValue = false;
	}
}

/* Initialization code. */
if (window.addEventListener)
	window.addEventListener('DOMMouseScroll', dax_mouseWheel, false);
window.onmousewheel = document.onmousewheel = dax_mouseWheel;

// Rudolf Psenicnik
function dax_viewport(parentNode)
{

	this.nodes = {
		main		: getTemplate('dax_viewport').cloneNode(true),
		vscrollbar	: getTemplate('dax_vscrollbar').cloneNode(true),
		hscrollbar	: getTemplate('dax_hscrollbar').cloneNode(true)
	}
	
	generateUniqueId(this.nodes.main);
	generateUniqueId(this.nodes.vscrollbar);
	generateUniqueId(this.nodes.hscrollbar);
	
	this.nodes.main.style.display = 'block';
	this.nodes.vscrollbar.style.display = 'block';
	this.nodes.hscrollbar.style.display = 'block';
	
	parentNode.appendChild(this.nodes.main);
	this.nodes.main.appendChild(this.nodes.vscrollbar);
	this.nodes.main.appendChild(this.nodes.hscrollbar);
	
	this.dom = {
		main			: this.nodes.main,
		parent			: parentNode,
		container		: getElementByClassName(this.nodes.main, 'dax_viewport_container'),
		contents		: getElementByClassName(this.nodes.main, 'dax_viewport_contents'),
		vscrollbar		: {
			main			: this.nodes.vscrollbar,
			width			: 17,
			arrowup			: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_arrowup'),
			arrowdown		: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_arrowdown'),
			trackspace		: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_trackspace'),
			slider			: {
				main			: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_slider'),
				top				: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_slidertop'),
				middle			: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_slidermiddle'),
				bottom			: getElementByClassName(this.nodes.vscrollbar, 'dax_vscrollbar_sliderbottom')
			}
		},
		hscrollbar		: {
			main			: this.nodes.hscrollbar,
			arrowleft		: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_arrowleft'),
			height			: 17,
			arrowright		: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_arrowright'),
			trackspace		: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_trackspace'),
			slider			: {
				main			: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_slider'),
				right			: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_sliderright'),
				middle			: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_slidermiddle'),
				left			: getElementByClassName(this.nodes.hscrollbar, 'dax_hscrollbar_sliderleft')
			}
		}
	}
	
	this.dom.main.boundTo = this;
	
	this.scrollbars = {
		ver		: {
			show		: false,
			showUser	: true,		// allows user to override scrollbar showing
			tracking	: 0,		// size of trackspace in px
			scale		: 0,		// tracking px * scale = contents px; contents px / scale = tracking px; scale = contents px / tracking px
			position	: 0,
			height		: 0,		// contents height
			visibleHeight	: 0,	// visible section of the viewport
			autoHeight	: true,		// adjust height based on contents
			autoContent	: true,		// automatically move contents when scrolling
			arrowStep	: 20,		// click on arrows will make a shift for this many pixels 
			trackStep	: 100,		// click on slider area will make a shift for this many pixels 
			overflow	: 0,
			scrollableHeight : 0,	// px size of trackspace that's available for scrolling (trackspace - thumb slider)
			useContentPxSizeWhenScrolling	: false, // scrolling by default uses the pixel sizes of the actual scrollbar. To scroll for pixel values of the contents, set this to true. This refers to the arrowStep and trackStep attributes.
			customValues	: false,	// height and visibleHeight are set by user instead of being calculated based on actual content px size, 
			scrollInterval	: 50		// time in ms when next scroll event will fire, used when user holds down pointer on the arrows
		},
		hor		: {
			show		: false,
			showUser	: true,		// allows user to override scrollbar showing
			tracking	: 0,		// size of trackspace in px
			scale		: 0,		// tracking px * scale = contents px; contents px / scale = tracking px; scale = contents px / tracking px
			position	: 0,
			width		: 0,		
			visibleWidth	: 0,		
			autoWidth	: true,		// adjust width based on contents
			autoContent	: true,		// automatically move contents when scrolling
			arrowStep	: 20,		// click on arrows will make a shift for this many pixels 
			trackStep	: 100,		// click on slider area will make a shift for this many pixels 
			overflow	: 0,
			scrollableWidth : 0,	// px size of trackspace that's available for scrolling (trackspace - thumb slider)
			useContentPxSizeWhenScrolling	: false, // scrolling by default uses the pixel sizes of the actual scrollbar. To scroll for pixel values of the contents, set this to true. This refers to the arrowStep and trackStep attributes.
			customValues	: false,	// width and visibleWidth are set by user instead of being calculated based on actual content px size, 
			scrollInterval	: 50		// time in ms when next scroll event will fire, used when user holds down pointer on the arrows
		}		
	}
	
	// special mode where viewport is part of the super grid and scrollbars are external
	this.isSuperGridChild = false;
	
	// dev defined function
	
	function dax_viewport_blank() { };
	
	this.onvscroll = dax_viewport_blank;
	this.onhscroll = dax_viewport_blank;
	
	
	function dax_viewport_setContents(contentNode) {
	
		this.dom.contents = contentNode;
		this.dom.container.appendChild(contentNode);
		
		// reset position
		this.dom.contents.style.position = 'relative';
		this.dom.contents.style.top = '0px';
		this.dom.contents.style.left = '0px';
	}
	
	this.setContents = dax_viewport_setContents;
	
	function dax_viewport_redraw() {

		// if parent is provided, size viewport to fit parent
		if (this.dom.parent) {
			this.dom.main.style.width = this.dom.parent.clientWidth + 'px';
			this.dom.main.style.height = this.dom.parent.clientHeight + 'px';
		}

		// calculate if scrollbars should be shown
		var contentsWidth = (this.dom.contents) ? this.dom.contents.offsetWidth : 0;	
		var contentsHeight = (this.dom.contents) ? this.dom.contents.offsetHeight : 0;
		
		
		// check if scrollbars should be drawn
		// first, check if custom values are used, if so compare height/width and scrollable height/width
		// if custom values are not used, compare width/height of viewport visible area vs total area
		// showUser can override this to make scrollbars invisible
		if (!this.scrollbars.hor.customValues) {
			if ((contentsWidth > this.dom.main.offsetWidth) && (this.scrollbars.hor.showUser))
				this.scrollbars.hor.show = true;
			else
				this.scrollbars.hor.show = false;
		} else {
			if ((this.scrollbars.hor.visibleWidth < this.scrollbars.hor.width) && (this.scrollbars.hor.showUser))
				this.scrollbars.hor.show = true;
			else
				this.scrollbars.hor.show = false;
		}
		
		if (!this.scrollbars.ver.customValues) {
			if ((contentsHeight > this.dom.main.offsetHeight) && (this.scrollbars.ver.showUser))
				this.scrollbars.ver.show = true;
			else
				this.scrollbars.ver.show = false;
		} else {
			if ((this.scrollbars.ver.visibleHeight < this.scrollbars.ver.height) && (this.scrollbars.ver.showUser))
				this.scrollbars.ver.show = true;
			else
				this.scrollbars.ver.show = false;
		}

		// draw scrollbars
		if (!this.isSuperGridChild) {
			var vOffset = (this.scrollbars.hor.show ? this.dom.hscrollbar.height : 0);
			var hOffset = (this.scrollbars.ver.show ? this.dom.vscrollbar.width : 0);
		} else {
			var vOffset = 0;
			var hOffset = 0;
		}
		
		this.dom.container.style.height = this.dom.main.offsetHeight - vOffset + 'px';
		this.dom.container.style.width = this.dom.main.offsetWidth - hOffset + 'px';

		// show scrollbars?
		if (this.scrollbars.hor.show)
			var showHorScrollbar = true;
		else
			var showHorScrollbar = false;

		if (this.scrollbars.ver.show)
			var showVerScrollbar = true;
		else
			var showVerScrollbar = false;

		// set scrollbar heights and calculate ratio
		if (this.scrollbars.ver.autoHeight) {
			this.scrollbars.ver.height = contentsHeight;
			this.scrollbars.ver.visibleHeight = this.dom.container.offsetHeight;
		}
			
		// set scrollbar heights and calculate ratio
		if (this.scrollbars.hor.autoWidth) {
			this.scrollbars.hor.width = contentsWidth;
			this.scrollbars.hor.visibleWidth = this.dom.container.offsetWidth;
		}
			
		// display scrollbars
		if (showVerScrollbar) {
			this.dom.vscrollbar.main.style.display = 'block';
			this.dom.vscrollbar.main.style.height = this.dom.main.offsetHeight - vOffset + 'px';
			this.dom.vscrollbar.trackspace.style.height = this.dom.main.offsetHeight - vOffset - this.dom.vscrollbar.arrowup.offsetHeight - this.dom.vscrollbar.arrowdown.offsetHeight + 'px';

			// calculate scroll spacing
			this.scrollbars.ver.tracking = this.dom.vscrollbar.trackspace.offsetHeight;
			this.scrollbars.ver.scale = this.scrollbars.ver.height / this.scrollbars.ver.tracking;
				
			// resize scrollbar based on content size
			var totalSliderSize = Math.round(this.scrollbars.ver.visibleHeight / this.scrollbars.ver.scale);

			// IE7 fix
			if (isNaN(totalSliderSize))
				totalSliderSize = 0;
			
			// update slider sizes
			this.dom.vscrollbar.slider.main.style.height = totalSliderSize + 'px';
			var middleHeight = totalSliderSize - this.dom.vscrollbar.slider.top.offsetHeight - this.dom.vscrollbar.slider.bottom.offsetHeight;
			if (middleHeight < 0)
				middleHeight = 0;
			this.dom.vscrollbar.slider.middle.style.height = middleHeight + 'px';
			this.scrollbars.ver.scrollableHeight = this.scrollbars.ver.tracking - totalSliderSize;
			
			// position the slider thumb
			var sliderTop = Math.round(this.scrollbars.ver.position / this.scrollbars.ver.scale);
			if (isNaN(sliderTop))
				sliderTop = 0;
			this.dom.vscrollbar.slider.main.style.top = sliderTop + 'px';

		} else {
			this.dom.vscrollbar.main.style.display = 'none';
		}

		if (showHorScrollbar) {
			this.dom.hscrollbar.main.style.display = 'block';
			this.dom.hscrollbar.main.style.width = this.dom.main.offsetWidth - hOffset + 'px';
			this.dom.hscrollbar.trackspace.style.width = this.dom.main.offsetWidth - hOffset - this.dom.hscrollbar.arrowleft.offsetWidth - this.dom.hscrollbar.arrowright.offsetWidth + 'px';

			
			// calculate scroll spacing
			this.scrollbars.hor.tracking = this.dom.hscrollbar.trackspace.offsetWidth;
			this.scrollbars.hor.scale = this.scrollbars.hor.width / this.scrollbars.hor.tracking;
				
			// resize scrollbar based on content size
			var totalSliderSize = Math.round(this.scrollbars.hor.visibleWidth / this.scrollbars.hor.scale);

			// IE7 fix
			if (isNaN(totalSliderSize))
				totalSliderSize = 0;
			
			// update slider sizes
			this.dom.hscrollbar.slider.main.style.width = totalSliderSize + 'px';
			var middleWidth = totalSliderSize - this.dom.hscrollbar.slider.left.offsetWidth - this.dom.hscrollbar.slider.right.offsetWidth;
			if (middleWidth < 0)
				middleWidth = 0
			this.dom.hscrollbar.slider.middle.style.width = middleWidth + 'px';
			this.scrollbars.hor.scrollableWidth = this.scrollbars.hor.tracking - totalSliderSize;
		
			// position the slider thumb
			var sliderLeft = Math.round(this.scrollbars.hor.position / this.scrollbars.hor.scale);
			if (isNaN(sliderLeft))
				sliderLeft = 0;				
			this.dom.hscrollbar.slider.main.style.left = sliderLeft + 'px';

		} else {
			this.dom.hscrollbar.main.style.display = 'none';
		}

		// check if contents are below visible zone, if so shift back to visible section
		if (this.scrollbars.ver.autoHeight) {
			if ((this.scrollbars.ver.position > (contentsHeight - this.dom.main.offsetHeight - vOffset)) && (this.scrollbars.ver.position != 0)) {
				this.vscroll(contentsHeight - this.dom.main.offsetHeight - vOffset);
			}
		}

		// check if contents are left to visible zone, if so shift back to visible section
		if (this.scrollbars.hor.autoWidth) {
			if ((this.scrollbars.hor.position > (contentsWidth - this.dom.main.offsetWidth - hOffset)) && (this.scrollbars.hor.position != 0)) {
				this.hscroll(contentsWidth - this.dom.main.offsetWidth - hOffset);
			}
		}
		
		// if scrollbars are not visible, and auto width/height is turned on, shift the position to 0
		if ((this.scrollbars.ver.autoHeight) && (this.scrollbars.ver.position != 0) && (!showVerScrollbar)){
			this.vscroll(0);
		}

		if ((this.scrollbars.hor.autoWidth) && (this.scrollbars.hor.position != 0) && (!showHorScrollbar)){
			this.hscroll(0);
		}
	}

	this.redraw = dax_viewport_redraw;	
	
	function dax_viewport_vscroll(position, posInSliderPx) {
	
		if (!posInSliderPx) {
	
			// check if scrollbar is out of range, only for auto height
			if (this.scrollbars.ver.autoHeight) {
				if (position > (this.scrollbars.ver.height - this.dom.container.offsetHeight))
					position = this.scrollbars.ver.height - this.dom.container.offsetHeight;
			} else {
				if (position > (this.scrollbars.ver.height - this.scrollbars.ver.visibleHeight))
					position = this.scrollbars.ver.height - this.scrollbars.ver.visibleHeight;
			}
	
			this.scrollbars.ver.position = position;
			this.dom.vscrollbar.slider.main.style.top = Math.round(this.scrollbars.ver.position / this.scrollbars.ver.scale) + 'px';
			if (this.scrollbars.ver.autoContent)
				this.dom.contents.style.top = 0 - this.scrollbars.ver.position + 'px';
	
		} else {
			
			this.dom.vscrollbar.slider.main.style.top = position + 'px';
			
			var position = Math.round(this.scrollbars.ver.scale * position);
			
			this.scrollbars.ver.position = position;
			if (this.scrollbars.ver.autoContent)
				this.dom.contents.style.top = 0 - position + 'px';
			
		}
		
		this.onvscroll();
	}
	
	this.vscroll = dax_viewport_vscroll;
	
	function dax_viewport_setVheight(vHeight, vAuto) {
		if (vAuto) {
			this.scrollbars.ver.autoHeight = vAuto;
		} else {
			this.scrollbars.ver.height = vHeight;	
			this.scrollbars.ver.autoHeight = false;
		}
	}

	function dax_viewport_hscroll(position, posInSliderPx) {
	
		if (!posInSliderPx) {

			if (this.scrollbars.hor.autoWidth) {
				if (position > this.scrollbars.hor.width - this.dom.container.offsetWidth)
					position = this.scrollbars.hor.width - this.dom.container.offsetWidth;
			} else {
				if (position > this.scrollbars.hor.width - this.scrollbars.hor.visibleWidth)
					position = this.scrollbars.hor.width - this.scrollbars.hor.visibleWidth;
			}
	
			this.scrollbars.hor.position = position;
			this.dom.hscrollbar.slider.main.style.left = Math.round(this.scrollbars.hor.position / this.scrollbars.hor.scale) + 'px';
			if (this.scrollbars.hor.autoContent)
				this.dom.contents.style.left = 0 - this.scrollbars.hor.position + 'px';
	
		} else {
			
			this.dom.hscrollbar.slider.main.style.left = position + 'px';
			
			var position = Math.round(this.scrollbars.hor.scale * position);
			
			this.scrollbars.hor.position = position;
			if (this.scrollbars.hor.autoContent)
				this.dom.contents.style.left = 0 - position + 'px';
			
		}
		
		this.onhscroll();
	}
	
	this.hscroll = dax_viewport_hscroll;
	
	function dax_viewport_setHheight(hHeight, hAuto) {
		if (hAuto) {
			this.scrollbars.hor.autoWidth = hAuto;
		} else {
			this.scrollbars.hor.height = hHeight;	
			this.scrollbars.hor.autoWidth = false;
		}
	}

	
	function dax_viewport_verSlider_onMouseDown(aEvent)
	{

		// IE, W3C Compatability
		var event = xpEvent(aEvent, true);
		var pt = getMousePosition(event);
		
		WindowManager.scroll = this.boundTo;
		WindowManager.scroll.mouseposition = pt.y;
		WindowManager.scroll.sliderposition = parseInt(this.boundTo.dom.vscrollbar.slider.main.style.top, 10);
		
		// Gecko, Safari
		if (document.addEventListener) {
			document.addEventListener("mousemove", dax_newViewport_verticalScrollDragging, true);
	    	document.addEventListener("mouseup",   dax_newViewport_verticalScrollStop, true);
	    } else { // IE
	    	document.onmousemove = dax_newViewport_verticalScrollDragging;
			document.onmouseup   = dax_newViewport_verticalScrollStop;
	    }

	    return false;
	};

	this.dom.vscrollbar.slider.main.onmousedown = dax_viewport_verSlider_onMouseDown;
	this.dom.vscrollbar.slider.main.boundTo = this;
	
	function dax_viewport_horSlider_onMouseDown(aEvent)
	{

		// IE, W3C Compatability
		var event = xpEvent(aEvent, true);
		var pt = getMousePosition(event);
		
		WindowManager.scroll = this.boundTo;
		WindowManager.scroll.mouseposition = pt.x;
		WindowManager.scroll.sliderposition = parseInt(this.boundTo.dom.hscrollbar.slider.main.style.left, 10);
		
		// Gecko, Safari
		if (document.addEventListener) {
			document.addEventListener("mousemove", dax_newViewport_horizontalScrollDragging, true);
	    	document.addEventListener("mouseup",   dax_newViewport_horizontalScrollStop, true);
	    } else { // IE
	    	document.onmousemove = dax_newViewport_horizontalScrollDragging;
			document.onmouseup   = dax_newViewport_horizontalScrollStop;
	    }

	    return false;
	};

	this.dom.hscrollbar.slider.main.onmousedown = dax_viewport_horSlider_onMouseDown;
	this.dom.hscrollbar.slider.main.boundTo = this;	

	// set arrow events\
	function dax_viewport_stopArrowScrolling() {
		clearInterval(this.boundTo.scrollInterval);
	}
	
	function dax_viewport_startVerArrowUp() {
		var thisViewport = this.boundTo;
		var shiftAmount = thisViewport.scrollbars.ver.arrowStep;
		var scrollInterval = this.boundTo.scrollbars.hor.scrollInterval;
		dax_newViewport_verticalArrowScroll(thisViewport, 'up', shiftAmount);
		thisViewport.scrollInterval = window.setInterval(function() {dax_newViewport_verticalArrowScroll(thisViewport, 'up', shiftAmount); }, scrollInterval);
	}
	
	this.dom.vscrollbar.arrowup.onmousedown = dax_viewport_startVerArrowUp;
	this.dom.vscrollbar.arrowup.onmouseout = dax_viewport_stopArrowScrolling;
	this.dom.vscrollbar.arrowup.onmouseup = dax_viewport_stopArrowScrolling;
	this.dom.vscrollbar.arrowup.boundTo = this;

	
	function dax_viewport_startVerArrowDown() {
		var thisViewport = this.boundTo;
		var shiftAmount = thisViewport.scrollbars.ver.arrowStep;
		var scrollInterval = this.boundTo.scrollbars.hor.scrollInterval;
		dax_newViewport_verticalArrowScroll(thisViewport, 'down', shiftAmount);
		thisViewport.scrollInterval = window.setInterval(function() {dax_newViewport_verticalArrowScroll(thisViewport, 'down', shiftAmount); }, scrollInterval);
	}
	
	this.dom.vscrollbar.arrowdown.onmousedown = dax_viewport_startVerArrowDown;
	this.dom.vscrollbar.arrowdown.onmouseout = dax_viewport_stopArrowScrolling;
	this.dom.vscrollbar.arrowdown.onmouseup = dax_viewport_stopArrowScrolling;
	this.dom.vscrollbar.arrowdown.boundTo = this;

	function dax_viewport_startHorArrowLeft() {
		var thisViewport = this.boundTo;
		var shiftAmount = thisViewport.scrollbars.hor.arrowStep;
		var scrollInterval = this.boundTo.scrollbars.hor.scrollInterval;
		dax_newViewport_horizontalArrowScroll(thisViewport, 'left', shiftAmount);
		thisViewport.scrollInterval = window.setInterval(function() {dax_newViewport_horizontalArrowScroll(thisViewport, 'left', shiftAmount); }, scrollInterval);
	}
	
	this.dom.hscrollbar.arrowleft.onmousedown = dax_viewport_startHorArrowLeft;
	this.dom.hscrollbar.arrowleft.onmouseout = dax_viewport_stopArrowScrolling;
	this.dom.hscrollbar.arrowleft.onmouseup = dax_viewport_stopArrowScrolling;
	this.dom.hscrollbar.arrowleft.boundTo = this;

	
	function dax_viewport_startHorArrowRight() {
		var thisViewport = this.boundTo;
		var scrollInterval = this.boundTo.scrollbars.hor.scrollInterval;
		var shiftAmount = thisViewport.scrollbars.hor.arrowStep;
		dax_newViewport_horizontalArrowScroll(thisViewport, 'right', shiftAmount);
		thisViewport.scrollInterval = window.setInterval(function() {dax_newViewport_horizontalArrowScroll(thisViewport, 'right', shiftAmount); }, scrollInterval);
	}
	
	this.dom.hscrollbar.arrowright.onmousedown = dax_viewport_startHorArrowRight;
	this.dom.hscrollbar.arrowright.onmouseout = dax_viewport_stopArrowScrolling;
	this.dom.hscrollbar.arrowright.onmouseup = dax_viewport_stopArrowScrolling;
	this.dom.hscrollbar.arrowright.boundTo = this;

	function dax_viewport_verTrackspace_onMouseDown(event) {
		var event = xpEvent(event, true);
		var point = getMousePosition(event);
		var target = getObjectPosition(this.boundTo.dom.vscrollbar.slider.main);

		var shiftAmount = this.boundTo.scrollbars.ver.trackStep;
		
		if (point.y < target.y)
			dax_newViewport_verticalArrowScroll(this.boundTo, 'up', shiftAmount);
		else
			dax_newViewport_verticalArrowScroll(this.boundTo, 'down', shiftAmount);
		
	}
	
	this.dom.vscrollbar.trackspace.onmousedown = dax_viewport_verTrackspace_onMouseDown;
	this.dom.vscrollbar.trackspace.boundTo = this;

	function dax_viewport_horTrackspace_onMouseDown(event) {
		var event = xpEvent(event, true);
		var point = getMousePosition(event);
		var target = getObjectPosition(this.boundTo.dom.hscrollbar.slider.main);
		
		var shiftAmount = this.boundTo.scrollbars.hor.trackStep;
		
		if (point.x < target.x)
			dax_newViewport_horizontalArrowScroll(this.boundTo, 'left', shiftAmount);
		else
			dax_newViewport_horizontalArrowScroll(this.boundTo, 'right', shiftAmount);
		
	}
	
	this.dom.hscrollbar.trackspace.onmousedown = dax_viewport_horTrackspace_onMouseDown;
	this.dom.hscrollbar.trackspace.boundTo = this;
	
	// mouse wheel handler
	function dax_viewport_mouseWheelHandler(delta) {
		
		if (!this.scrollbars.ver.show)
			return false;
		
		if (delta > 0) {
			var shiftAmount = this.scrollbars.ver.arrowStep;
			dax_newViewport_verticalArrowScroll(this, 'up', shiftAmount)
		} else {
			var shiftAmount = this.scrollbars.ver.arrowStep;
			dax_newViewport_verticalArrowScroll(this, 'down', shiftAmount)
		}	
	}	
	
	this.mouseWheelHandler = dax_viewport_mouseWheelHandler;
	
	// let's window manager know if mouse is inside the object, used for mouse wheel
	function dax_viewport_mainDOMonMouseOver() {
		if (daxViewportController)
			daxViewportController.activeViewport = this;
	}

	function dax_viewport_mainDOMonMouseOut() {
		if (daxViewportController)
			daxViewportController.activeViewport = null;
	}

	this.dom.main.onmouseover = dax_viewport_mainDOMonMouseOver;
	this.dom.main.onmouseout = dax_viewport_mainDOMonMouseOut;
	
	// user API
	function dax_viewport_showVerScrollbar(showVer) {
		this.scrollbars.ver.showUser = showVer;
		this.redraw();
	}

	this.showVerScrollbar = dax_viewport_showVerScrollbar;
	
	function dax_viewport_showHorScrollbar(showHor) {
		this.scrollbars.hor.showUser = showHor;
		this.redraw();
	}

	this.showHorScrollbar = dax_viewport_showHorScrollbar;
	
	
}

//
// Handles the scroll thumb being dragged
//
function dax_newViewport_verticalScrollDragging(event)
{

	var event = xpEvent(event, true);
	var point = getMousePosition(event);

	var mousePos = point.y;
	var viewportObject = WindowManager.scroll;

	var sliderPosition = WindowManager.scroll.sliderposition - WindowManager.scroll.mouseposition + mousePos;

	
	if (sliderPosition < 0)
		sliderPosition = 0;
		
	if (sliderPosition > WindowManager.scroll.scrollbars.ver.scrollableHeight)
		sliderPosition = WindowManager.scroll.scrollbars.ver.scrollableHeight;

	debugAlert('sliderPos ' + sliderPosition);
		
	viewportObject.vscroll(sliderPosition, true);

	return false;
}

// Handles the scroll thumb dragged to a stop
//
function dax_newViewport_verticalScrollStop(event)
{

	// Gecko
	if (document.removeEventListener) {
		document.removeEventListener("mousemove", dax_newViewport_verticalScrollDragging, true);
		document.removeEventListener("mouseup",   dax_newViewport_verticalScrollStop, true);
	} else {
		document.onmousemove = null;
 		document.onmouseup   = null;
	}
}

//
// Handles the scroll thumb being dragged
//
function dax_newViewport_horizontalScrollDragging(event)
{

	var event = xpEvent(event, true);
	var point = getMousePosition(event);

	var mousePos = point.x;
	var viewportObject = WindowManager.scroll;

	var sliderPosition = WindowManager.scroll.sliderposition - WindowManager.scroll.mouseposition + mousePos;

	
	if (sliderPosition < 0)
		sliderPosition = 0;
		
	if (sliderPosition > WindowManager.scroll.scrollbars.hor.scrollableWidth)
		sliderPosition = WindowManager.scroll.scrollbars.hor.scrollableWidth;

	viewportObject.hscroll(sliderPosition, true);

	return false;
}

// Handles the scroll thumb dragged to a stop
//
function dax_newViewport_horizontalScrollStop(event)
{

	// Gecko
	if (document.removeEventListener) {
		document.removeEventListener("mousemove", dax_newViewport_horizontalScrollDragging, true);
		document.removeEventListener("mouseup",   dax_newViewport_horizontalScrollStop, true);
	} else {
		document.onmousemove = null;
 		document.onmouseup   = null;
	}
}

//
function dax_newViewport_verticalArrowScroll(viewport, direction, shiftInPx)
{
	if (direction == 'up') {
		var newPosition = viewport.scrollbars.ver.position - shiftInPx;
		if (newPosition < 0)
			newPosition = 0;
		viewport.vscroll(newPosition);
	} else {
	
		var newPosition = viewport.scrollbars.ver.position + shiftInPx;
		if (newPosition > viewport.scrollbars.ver.height)
			newPosition = viewport.scrollbars.ver.height;
		viewport.vscroll(newPosition);
	}

}

function dax_newViewport_horizontalArrowScroll(viewport, direction, shiftInPx)
{

	if (direction == 'left') {
		var newPosition = viewport.scrollbars.hor.position - shiftInPx;
		if (newPosition < 0)
			newPosition = 0;
		debugAlert ('left:' + newPosition);
		viewport.hscroll(newPosition);
	} else {
		var newPosition = viewport.scrollbars.hor.position + shiftInPx;
		if (newPosition > viewport.scrollbars.hor.width)
			newPosition = viewport.scrollbars.hor.width;
		debugAlert ('right:' + newPosition);
		viewport.hscroll(newPosition);
	}
}

function dax_tabs() {
	
	// dev defined functions
	this.onTabClick = function() { };
	
	function dax_tabs_addTab(label, disableRedraw, enableClose) {
		
		var newTab = {
			dom		: getTemplate('dax_tabs_tab').cloneNode(true),
			label	: label
		}
		
		generateUniqueId(newTab.dom);
		newTab.dom.style.display = 'block';
		
		newTab.dom.l = getElementByClassName(newTab.dom,'dax_tabs_tab_l');
		newTab.dom.m = getElementByClassName(newTab.dom,'dax_tabs_tab_m');
		newTab.dom.r = getElementByClassName(newTab.dom,'dax_tabs_tab_r');
		
		newTab.dom.properties = newTab;
		newTab.dom.boundTo = this;
		
		newTab.dom.onclick = this.tabOnClick;
		
		getElementByClassName(newTab.dom, 'dax_tabs_tab_label').innerHTML = label;
		
		this.tabs.elements.push(newTab);
		this.dom.holder.appendChild(newTab.dom);
		
		if (!disableRedraw)
			this.redraw();
		
		function dax_tabs_tab_onmouseover(event){
		
			// don't update if this tab is selected
			if (this.boundTo.tabs.current == this.properties)
				return false;
		
			this.className = 'dax_tabs_tab dax_tabs_tab_hover';
		
		}
		
		newTab.dom.onmouseover = dax_tabs_tab_onmouseover;
		
		function dax_tabs_tab_onmouseout(event) {

			// don't update if this tab is selected
			if (this.boundTo.tabs.current == this.properties)
				return false;
		
			this.className = 'dax_tabs_tab';
	
		}
		
		newTab.dom.onmouseout = dax_tabs_tab_onmouseout;
		
		function dax_tabs_tab_onclose(event) {
		
			xpEvent(event, true);
			
			this.tabsObject.tabOnClose(this.boundTo);
			//this.tabsObject.deleteTab(null, false, this.boundTo);
			return false;
		}
		
		var closeButton = getElementByClassName(newTab.dom, 'dax_tabs_tab_close');
		
		closeButton.onclick = dax_tabs_tab_onclose;
		closeButton.boundTo = newTab;
		closeButton.tabsObject = this;
		
		if (enableClose)
			closeButton.style.display = 'block';
		
		return newTab;
		
	}
	
	this.addTab = dax_tabs_addTab;
	
	function dax_tabs_deleteTab(tabPosition, disableRedraw, tabProperties) {
	
		if (tabPosition)
			var tabToRemove = this.tabs.elements[tabPosition];
		else
			var tabToRemove = tabProperties;
	
		// remove DOM element and events to prevent memory leak
		this.dom.boundTo = null;
		this.dom.properties = null;
		this.dom.holder.removeChild(tabToRemove.dom);
	
		// remove tab from array, find position if needed
		if (!tabPosition) {
			for (var tabCount = 0; tabCount < this.tabs.elements.length; tabCount++) {
				if (this.tabs.elements[tabCount] == tabProperties)
					tabPosition = tabCount;
			}		
		}
		
		
		this.tabs.elements.splice(tabPosition, 1);
		
		// redraw if allowed
		if (!disableRedraw)
			this.redraw();
	
	}
	
	this.deleteTab = dax_tabs_deleteTab;
	
	function dax_tabs_redraw() {
	
		var contentsWidth = 0;
		
		for (var tabCount = 0; tabCount < this.tabs.elements.length; tabCount++){
		
			contentsWidth += (+this.tabs.elements[tabCount].dom.l.clientWidth) +
				(+this.tabs.elements[tabCount].dom.m.clientWidth) +
				(+this.tabs.elements[tabCount].dom.r.clientWidth);
		
		}
		
		if (contentsWidth < this.dom.container.clientWidth) {
			// hide arrows
			this.dom.rightarrow.style.display = 'none';
			this.dom.leftarrow.style.display = 'none';
			var arrowWidth = 0;
			
		} else {
			// show arrows
			this.dom.rightarrow.style.display = 'block';
			this.dom.leftarrow.style.display = 'block';
			var arrowWidth = (+this.dom.rightarrow.clientWidth) + (+this.dom.leftarrow.clientWidth);
		}
		
		this.dom.contents.style.width = this.dom.container.clientWidth - arrowWidth + 'px';
		this.dom.holder.style.width = contentsWidth + 'px';		
		this.viewport.redraw();
		this.updateArrowState();
	}
	
	this.redraw = dax_tabs_redraw;
	
	this.dom = {
		container 	: getTemplate('dax_tabs_container').cloneNode(true),
		holder 		: getTemplate('dax_tabs_holder').cloneNode(true),
		contents	: null
	}
	
	this.dom.contents = getElementByClassName(this.dom.container, 'dax_tabs_contents');
	
	this.tabs = {
		elements	: [ ],
		current		: null	
	}

	function dax_tabs_tab_onClick(event){
		xpEvent(event, true);
		this.boundTo.selectTabByNode(this.properties);
		this.boundTo.onTabClick(this.properties);
		return false;
	}
	
	this.tabOnClick = dax_tabs_tab_onClick;
	
	function dax_tabs_selectTabByNode(tabRef) {
		
		if (this.tabs.current) {
			this.tabs.current.dom.className = 'dax_tabs_tab';
			this.tabs.current = null;
		}

		tabRef.dom.className = 'dax_tabs_tab dax_tabs_tab_selected';
		this.tabs.current = tabRef;
		
	}
	
	this.selectTabByNode = dax_tabs_selectTabByNode;
	
	function dax_tabs_selectTabByIndex(tabIndex) {
	
		var tabNode = this.tabs.elements[tabIndex];
		
		if (tabNode)
			this.selectTabByNode(tabNode);
	
	}
	
	this.selectTabByIndex = dax_tabs_selectTabByIndex;
	
	function dax_tabs_tabOnClose(tabReference) {
	
	}
	
	this.tabOnClose = dax_tabs_tabOnClose;
	
	generateUniqueId(this.dom.container);
	this.dom.container.style.display = 'block';

	generateUniqueId(this.dom.holder);
	this.dom.holder.style.display = 'block';
	
	this.viewport = new dax_viewport(this.dom.contents);
	this.viewport.boundTo = this;
	
	this.dom.leftarrow = getElementByClassName(this.dom.container, 'dax_tabs_arrowleft');
	this.dom.rightarrow = getElementByClassName(this.dom.container, 'dax_tabs_arrowright');
	
	var rightArrow = this.dom.rightarrow;
	
	rightArrow.boundTo = this.viewport;
	rightArrow.disabledState = true;
	
	rightArrow.onmouseover = function(event) {this.className = 'dax_tabs_arrowright dax_tabs_arrowright_hover';};
	rightArrow.onmousedown = function(event) {this.boundTo.dom.hscrollbar.arrowright.onmousedown(event)};
	rightArrow.ondblclick = function(event) {this.boundTo.hscroll(this.boundTo.scrollbars.hor.width); this.boundTo.boundTo.updateArrowState();};
	rightArrow.onmouseout = function(event) {this.className = this.disabledState ? 'dax_tabs_arrowright dax_tabs_arrowright_disabled' : 'dax_tabs_arrowright'; this.boundTo.dom.hscrollbar.arrowright.onmouseout(event)};
	rightArrow.onmouseup = function(event) {
		this.boundTo.dom.hscrollbar.arrowright.onmouseup(event);
		this.boundTo.boundTo.updateArrowState();
	};

	var leftArrow = this.dom.leftarrow;
	
	leftArrow.boundTo = this.viewport;
	leftArrow.disabledState = false;
	leftArrow.className = 'dax_tabs_arrowleft dax_tabs_arrowleft_disabled';
	
	leftArrow.onmouseover = function(event) {this.className = 'dax_tabs_arrowleft dax_tabs_arrowleft_hover';};
	leftArrow.ondblclick = function(event) {this.boundTo.hscroll(0); this.boundTo.boundTo.updateArrowState();};
	leftArrow.onmousedown = function(event) {this.boundTo.dom.hscrollbar.arrowleft.onmousedown(event)};
	leftArrow.onmouseout = function(event) {this.className = this.disabledState ? 'dax_tabs_arrowleft dax_tabs_arrowleft_disabled' : 'dax_tabs_arrowleft'; this.boundTo.dom.hscrollbar.arrowleft.onmouseout(event)};
	leftArrow.onmouseup = function(event) {
		this.boundTo.dom.hscrollbar.arrowleft.onmouseup(event);
		this.boundTo.boundTo.updateArrowState();
	};

	this.viewport.scrollbars.hor.arrowStep = 22.5;
	this.viewport.scrollbars.hor.scrollInterval = 50;
	
	// check if beginning or the end is reached, and if so dim out the arrows
	function dax_tabs_updateArrowState() {
	
		var horScrollbar = this.viewport.scrollbars.hor;
		if (horScrollbar.width == horScrollbar.position + horScrollbar.visibleWidth) {
			// viewport contents is all the way to the right, dim out right arrow
			this.dom.rightarrow.className = 'dax_tabs_arrowright dax_tabs_arrowright_disabled';
			this.dom.leftarrow.className = 'dax_tabs_arrowright';
			
			this.dom.rightarrow.disabledState = true;
			this.dom.leftarrow.disabledState = false;
			
		} else if (horScrollbar.position == 0) {
			// viewport contents is all the way to the left, dim out left arrow
			this.dom.leftarrow.className = 'dax_tabs_arrowleft dax_tabs_arrowleft_disabled';
			this.dom.rightarrow.className = 'dax_tabs_arrowright';
			
			this.dom.leftarrow.disabledState = true;
			this.dom.rightarrow.disabledState = false;
		
		} else {
			// viewport contents are in the middle, enable both arrows
			this.dom.leftarrow.className = 'dax_tabs_arrowleft';
			this.dom.rightarrow.className = 'dax_tabs_arrowright';
			
			this.dom.leftarrow.disabledState = false;
			this.dom.rightarrow.disabledState = false;
		
		}
	}
	
	this.updateArrowState = dax_tabs_updateArrowState;
	this.updateArrowState();
	
	this.viewport.showHorScrollbar(false);
	this.viewport.setContents(this.dom.holder);
	this.viewport.redraw();
	
	
	
}